/* biled_lib.c. Version 2.0
 * Library of routines to handle BILED (The Oxford timing module)
 * IM, 6 October 1998. First version - set up low level routines.
 * BJG/PWP 24.05.02 - additions for NIVXI with linux
 * JCH 24.06.02 - add support for SBCs via ATLAS RCC VME library
 */

/* includes and defines */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifdef NIVXI
#include <nivxi.h>
#endif

#ifdef OS9
#include <process.h>
#include <signal.h>
#include <types.h>
#endif

#ifdef RAID
#define CLOCKS_PER_SEC 100000
#endif

#ifdef OXVME
long *reg_add;
int fpagedat;
int fpagebase;
#define a24d16_base 0xF0000000
#define a24d16_wind 0x00FFFFFF
#define fpage_beg 0xFFF40000
#define fpage_wind 0x00000100
#include <math.h>
#include <errno.h>
#include <smem.h>
#endif

#ifdef QB_linux
#define WND_SIZE 0x1000
#include "../QB/InitVME.h"
#endif

#ifdef RCC
#include <rcc_error.h>
#include <vme_rcc.h>
#endif

#include "biled_local.h"
#include "biled_lib.h"


/* Global variables */
short biled_total = 0;           /*Total number of BILED boards*/
struct biled_info_structure *biled_info;      /*structure to hold BILED info*/

/*biled_vmemap
 * Map the BILED to the CPU address space. The input address is the true
 * VME address for UNIX and WNT implementations, mapped address for OS9 (this
 * is because the mapping in OS9 is CPU-specific and so it is not possible to
 * provide a generalised mapping within this routine). The mapping in UNIX/WNT
 * is NOT A32 - biled is an A24 module!!.
 */

int biled_vmemap(address, identifier)
unsigned long address;
short *identifier;
{
#if defined (NIVXI)
  short ret;
  UINT32 w,wstart,wend;
#elif defined(RCC)
  VME_ErrorCode_t status;
  VME_MasterMap_t master_map;
  u_int virtual_address;
  int handle;
#else
  int status;
#endif
  int am;
  char *addr;

  am = 0;
  *identifier = biled_total++;		/*Simplest approach? */

/* Use malloc/realloc to get space for the structure */
  if(biled_total == 1)
    biled_info = (struct biled_info_structure *) malloc(biled_info_size);
  else
    biled_info = (struct biled_info_structure *) realloc((void *)biled_info,biled_total*biled_info_size);

  biled_info[*identifier].address = address;
#ifdef NIVXI
  biled_info[*identifier].window = 0;
#endif

/* Now do the mapping - CPU-dependent - allow a 0x10000 window */

#ifdef OS9
  status = _os_permit((void *)address, 0x20000,2,0);
  if(status != 0) {
     printf("VME mapping failed, status = %d\n",status);
     return status;
  }
#endif

#ifdef RAID
  am = 0x0d;		/* A32 access if in second crate!! */
  status = RD13_VmeMap(address,am,0x1000,&addr,0);
  if(status != 0) {
     printf("VME mapping failed, status = %d\n",status);
     return status;
  }
  biled_info[*identifier].address = (unsigned long) addr;
#endif

#ifdef NIVXI
  if(biled_total == 1) InitVXIlibrary();		/*only call once*/
/*  addr = (char *)MapVXIAddress(2,address,5000,&w,&ret); */
  addr = (char *)MapVXIAddress(6,address,5000,&w,&ret);
  /* _MOST_ BiLED boards respond only to supervisory A24 access,
     hence VXI map code is 6... */
  if(ret != 0) {
     printf("VME mapping failed, status = %d\n",ret);
     return (int)ret;
  }
  printf("biled_vmemap: MapVXIAddress ret %d win %d addr %8x for vme %8x\n", 
		ret, w, addr, address);
  ret = GetWindowRange(w, &wstart, &wend);
  printf("biled_vmemap: WindowRange start %lx end %lx\n", wstart, wend);
  biled_info[*identifier].address = (unsigned long) addr;
  biled_info[*identifier].window = w;
#endif

#ifdef OXVME
/*
Enable A24D16 Default F0-Page on Motorolla VME 167
*/

fpagebase = (int)smem_create("fpage", (char*)fpage_beg,
                              fpage_wind, SM_READ|SM_WRITE);
   if (fpagebase == NULL)
        {
        printf("Failed to map VME, status = 1\n");
        return 1;
        }
reg_add = (long *) (fpagebase+0x2C);
fpagedat = *reg_add;
*reg_add = (fpagedat | 0x0FF00);
smem_create("fpage", (char*)fpagebase,fpage_wind,SM_DETACH);
smem_remove("fpage");

/*
Map vme address onto lynxOS address space
*/

 fpagebase = (int)smem_create("vmea24d16", (char*)a24d16_base,
                              a24d16_wind, SM_READ|SM_WRITE);

 if (fpagebase == NULL)
        {
        printf("Failed to map VME \n");
        return 1;
        }

  biled_info[*identifier].address = (unsigned long) fpagebase + address;
  biled_info[*identifier].virtbase = (int) fpagebase;

#endif

#ifdef QB_linux
  if(biled_total == 1) InitVME();		/*only call once*/
  Initqb(&address ,WND_SIZE,VME_A24);
  biled_info[*identifier].address = (unsigned long) address;
#endif

#ifdef RCC
  if(biled_total == 1) status = VME_Open();     /*Called once*/
/* Set up the structure for the mapping call */
  master_map.vmebus_address = address;
  master_map.window_size = 0x10000;
  master_map.address_modifier = VME_A24;
  master_map.options = 0;
  status = VME_MasterMap(&master_map,&handle);
  if(status != VME_SUCCESS) {
    VME_ErrorPrint(status);
    return (int)status;
  }
/* Now get the base address of the mapping*/ 
  status = VME_MasterMapVirtualAddress(handle, &virtual_address);
  printf("biled_vmemap: VME address %x mapped to %x, handle %d\n",
    address, virtual_address,handle);
  biled_info[*identifier].address = (unsigned long)virtual_address;
  biled_info[*identifier].window = handle;
#endif

  return 0;
}
/*end of biled_vmemap */

/******************************************************************/

/*biled_exit.
 *Exit routine for BILED library. Should be called once only to tidy things
 *up. Unmaps the modules and deletes the malloced memory.
 */

int biled_exit()
{
  int i;
#ifdef RCC
VME_ErrorCode_t status;
#else
int status;
#endif

#ifdef OXVME
  int virtbase;
#endif

  
/* loop over the modules */
  for(i=0;i<biled_total;i++) {
#ifdef RAID
    status = RD13_VmeUnmap(biled_info[i].address);
#endif
#ifdef NIVXI
    status = UnMapVXIAddress(biled_info[i].window);
#endif
#ifdef OXVME
 virtbase = biled_info[i].virtbase;
 smem_create("vmea24d16", (char*)virtbase,a24d16_wind,SM_DETACH);
 smem_remove("vmea24d16");
#endif
#ifdef QB_linux
    munmap((void *)(biled_info[i].address),WND_SIZE);
#endif
#ifdef RCC
  status = VME_MasterUnmap(biled_info[i].window);
#endif

  }  /*end of loop*/

#ifdef NIVXI
    CloseVXIlibrary();
#endif
#ifdef RCC
  status = VME_Close();
#endif

/* release memory */
  free((void *)biled_info);
  return 0;
}
/*end of biled_exit */

/**********************************************************/

/**************************************************************
*       Read / Write Routines                                 *
**************************************************************/

/* The VME_READ routine */

short biled_vme_read(int addr)
{

    volatile short *ip;

    ip = (short *)(addr);

    return (*ip);
}

/* The VME_WRITE routine */

void biled_vme_write(int addr, short data)
{

    volatile short *ip;


    ip = (short *)(addr);
    *ip = data;
}

/* set mode routine */
void biled_set_mode(int m, int identifier)
{
        short smod;
        unsigned long BASE;

        BASE = biled_info[identifier].address;

        m = (m << 4);
        smod= biled_vme_read(BASE);
        smod= smod & 0xff0f;
        smod= smod | m;
        biled_vme_write(BASE+ 0, smod);
}

/**********************************************************
    enables command output for selected channel
    the following values of channel are accepted:
    between 1 and 4: the selected channel is enabled
                  0: enable all channels
**********************************************************/       
int biled_channel_enable( int channel, int identifier)
{
  unsigned long BASE;
  short rdat;
  int enable;

  if ((channel < 0) || (channel > 4)) return 1;

  BASE = biled_info[identifier].address;   
    
  if (channel == 0 ) enable = 0xf; /* enable all  */
  else
     {   
        enable = 0x0;
        channel -= 1;
        enable = 0x1 << channel; /* enable the selected channel */
     }

  enable = enable << 8;
  rdat = biled_vme_read(BASE);
  rdat = rdat | enable;
  rdat = rdat | 0x8;
  biled_vme_write(BASE,rdat );

  return 0;

}

/**********************************************************
    disables command output for selected channel (clock is always
    transmitted ) the following values of channel are accepted:
    between 1 and 4: the selected channel is dissabled
                  0: dissable all channels
**********************************************************/       
int biled_channel_disable( int channel, int identifier)
{
  unsigned long BASE;
  short rdat;
  int disable;

  if ((channel < 0) || (channel > 4)) return 1;

  BASE = biled_info[identifier].address;   
    
  if (channel == 0)  disable = 0x0f00; /* disable all */
  else
     {   
        channel -= 1;
        disable = 0x1 << channel; /* enable the selected channel */
        disable = disable << 8;
     }
  rdat = biled_vme_read(BASE);
  rdat = rdat & ~disable;
  rdat = rdat | 0x8;
  biled_vme_write(BASE,rdat );

  return 0;

}

/******** initialization routine **********************/

int biled_init(identifier)
int identifier;
{
  unsigned long BASE;
  int i, q,status;

  BASE = biled_info[identifier].address;


  biled_info[identifier].A1D=0x013f;                     /* initialize BiLED Chip */
  biled_vme_write(BASE+0x10,biled_info[identifier].A1D);

  biled_info[identifier].A3D=0;
  biled_vme_write(BASE+0x14,biled_info[identifier].A3D);
        for(q=100000; q>=0; q--);
        biled_info[identifier].A3D=biled_info[identifier].A3D | 0x10;
  biled_vme_write(BASE+0x14,biled_info[identifier].A3D);
        for(q=100000; q>=0; q--);
        biled_info[identifier].A3D=biled_info[identifier].A3D | 0xf;
  biled_vme_write(BASE+0x14,biled_info[identifier].A3D);

  /* new 23.09. 99 
   * VcSEL drive currents set by DACs 9 through 11
   * - will be zero at switch on for safety
   */

  for(i=1;i<9;i++)  biled_info[identifier].DACS[i]=0xff; 
  for(i=9;i<13;i++) biled_info[identifier].DACS[i]=0x00;
  for(i=1;i<13;i++) biled_set_dac(identifier,i,biled_info[identifier].DACS[i]);
  
  /* enable all output channels */

  status = biled_channel_enable(0, identifier); 

  biled_set_mode(10, identifier);              /* go external */

  return 0;

}


/************************************************************
      biled_reset routine is used to reset the selected BiLED
      channel. It is used to get the 40 to 20 MHz clock in
      the right phase (for the Vernier board and overal timing
      - will not be needed for the new BiLED)
      input: channel number 1 - 4 or 0 for all
             board identifier
************************************************************/
int biled_reset(int channel, int identifier)
{
  unsigned long BASE;
  short qa,qo,i;
  static int rev_channel[4] = {4, 3, 2, 1};

  if (channel > 4 && channel < 0) return 1;


  BASE = biled_info[identifier].address;

  qa = 0;
  qo = 0;
  if (channel == 0) 
    { 
     qa = 0x10;
     qo = 0xf;
    }
  else
    {
     channel = rev_channel[channel-1]; 
     channel -= 1;
     qa = 1 << channel;
     qa = qa ^ 0x0f;
     qa = qa | 0x10;
     qo = 1 << channel;
     }

  
     biled_info[identifier].A3D = biled_info[identifier].A3D & qa;  /* hold reset bpm1-4 */
     biled_vme_write(BASE+0x14,biled_info[identifier].A3D);

     for(i=10000; i>=0; i--);

     biled_info[identifier].A3D = biled_info[identifier].A3D | qo;  /* clear reset bpm1-4 */
     biled_vme_write(BASE+0x14,biled_info[identifier].A3D);


     return 0;
}
/********************************************************************/
/*                  tmp  part for testing                           */


/*************************************************************
*            Routine to Clear a VT220 Screen                 *
*************************************************************/

void clear_scr(void)

{

    char clr_scr;

    clr_scr = 27;
    printf("%c[2J",clr_scr);
/*    printf("%c[50A",clr_scr);*/

    return;

}
/************************************************************
*                          MENU                             *
************************************************************/
void biled_menu (void)
  {
        char mnu;
        int channel, identifier, status;
        short value;
  do
   {
        clear_scr();
        printf("%c[1;1H",27);
        printf("-----------------------------------------------------------------\n");
        printf("|                    MENU                                       |\n");
        printf("|---------------------------------------------------------------|\n");
        printf("|                 E = Enable Channel                            |\n");
        printf("|                 D = Disable Channel                           |\n");
        printf("|                 R = Reset Channel                             |\n");
        printf("|                 S = Set DAC                                   |\n");
        printf("|                 Q = quit                                      |\n");
        printf("|                                                               |\n");
        printf("-----------------------------------------------------------------\n");

        printf("Command: ");

        printf("%c[10;10H",(char)27);

        scanf("%c",&mnu);

        switch (toupper(mnu))
	  {
           case 'E':
                clear_scr();
                printf("\n Please choose: \n");
                printf("\n 1 - 4 enable this channel number ");
                printf("\n     0 enable all 4 channels     ");
                
                printf("\n ? ");
                scanf("%d",&channel); 
                
                if (biled_total == 1) {
                 identifier = 0;
                }
                else { 
                 printf("\n Please select the board: ");
                 scanf("%d",&identifier);
                }                  
 
                status = biled_channel_enable(channel,identifier);

                break;

           case 'D':
                clear_scr();
                printf("\n Please choose: \n");
                printf("\n 1 - 4 disable this channel number ");
                printf("\n     0 disable all 4 channels     ");
                
                printf("\n ? ");
                scanf("%d",&channel); 
   
                if (biled_total == 1) {
                 identifier = 0;
                }
                else { 
                 printf("\n Please select the board: ");
                 scanf("%d",&identifier);
                }  
                
                status = biled_channel_disable(channel,identifier);

                break;

       	   case 'R':
                clear_scr();
                printf("\n Please choose: \n");
                printf("\n 1 - 4 reset this channel number ");
                printf("\n     0 reset all 4 channels     ");

                printf("\n ? ");
                scanf("%d",&channel); 
   
                if (biled_total == 1) {
                 identifier = 0;
                }
                else { 
                 printf("\n Please select the board: ");
                 scanf("%d",&identifier);
                }

                status = biled_reset(channel,identifier);

                break;                


       	   case 'S':
                clear_scr();
                
                if (biled_total == 1) {
                 identifier = 0;
                }
                else { 
                 printf("\n Please select the board: ");
                 scanf("%d",&identifier);
                }
                
                printf("\n DACS 9 to 12 set VCSEL drive current");
                printf("\n Select DAC number - 1 to 12 -");
                scanf("%d",&channel); 

                if (channel<1) channel =1;
		        else if (channel>12) channel = 12;

                printf("\n Present value is %d",biled_info[identifier].DACS[channel]);
                printf("\n Enter DAC value - 0 to 255 - ");
                scanf("%d",&value); 
   
                

		value = value &0xff;
        

		biled_info[identifier].DACS[channel]=value;
    biled_set_dac(identifier,channel,value);

                break;                


	  }
  }while (toupper(mnu) != 'Q');

  printf("Switching off all VCSELs for BiLED %d\n",identifier);
  for(channel=9;channel<13;channel++) biled_set_dac(identifier,channel,0);

  status = biled_exit();

  clear_scr();


}

/* code added for use of BiLED without Vernier - 23.09.99 */

/*************************************************************
*			Set DACs			     *
*************************************************************/

void biled_set_dac(short identifier, short dac,short dac_data)
{
  unsigned long BASE;
  short dataw;

  BASE = biled_info[identifier].address;

  switch(dac)
  {

    case 1:
      biled_info[identifier].A1D = biled_info[identifier].A1D & 0xfffd;		/* DAC1 low all others high*/
      dataw =  (0x9 << 8) | dac_data;	/* A0 high and shift       */
      break;

    case 2:
      biled_info[identifier].A1D = biled_info[identifier].A1D & 0xfffd;		/* DAC1 low all others high*/
      dataw =  (0xa << 8) | dac_data;	/* A1 high and shift       */
      break;

    case 3:
      biled_info[identifier].A1D = biled_info[identifier].A1D & 0xfffb;			
      dataw =  (0x9 << 8) | dac_data;	
      break;

    case 4:
      biled_info[identifier].A1D = biled_info[identifier].A1D & 0xfffb;
      dataw =  (0xa << 8) | dac_data;
      break;

    case 5:
      biled_info[identifier].A1D = biled_info[identifier].A1D & 0xfff7;
      dataw =  (0x9 << 8) | dac_data;
      break;

    case 6:
      biled_info[identifier].A1D = biled_info[identifier].A1D & 0xfff7;
      dataw =  (0xa << 8) | dac_data;
      break;

    case 7:
      biled_info[identifier].A1D = biled_info[identifier].A1D & 0xffef;
      dataw =  (0x9 << 8) | dac_data;
      break;

    case 8:
      biled_info[identifier].A1D = biled_info[identifier].A1D & 0xffef;
      dataw =  (0xa << 8) | dac_data;
      break;

    case 9:
      biled_info[identifier].A1D = biled_info[identifier].A1D & 0xffdf;
      dataw =  (0x9 << 8) | dac_data;
      break;

    case 10:
      biled_info[identifier].A1D = biled_info[identifier].A1D & 0xffdf;
      dataw =  (0xa << 8) | dac_data;
      break;

    case 11:
      biled_info[identifier].A1D = biled_info[identifier].A1D & 0xfeff;
      dataw =  (0x9 << 8) | dac_data;
      break;

    case 12:
      biled_info[identifier].A1D = biled_info[identifier].A1D & 0xfeff;
      dataw =  (0xa << 8) | dac_data;
      break;

    default:
      biled_info[identifier].A1D = biled_info[identifier].A1D & 0xffff;
      dataw =  (0x9 << 8) | dac_data;
      break;
  }
	
  biled_vme_write(BASE+0x10,biled_info[identifier].A1D); 		/* set CS		*/
  write_serial(identifier, dataw);			/* Send data via din	*/
  biled_info[identifier].A1D = biled_info[identifier].A1D | 0x013f;
  biled_vme_write(BASE+0x10,biled_info[identifier].A1D); 		/* load dac		*/

  printf("Biled_set_dac: board %d dac %d set to %d\n", identifier, dac, dac_data);
}

void write_serial(short identifier, short dataw)
{
  unsigned long BASE;
  int i;

  BASE = biled_info[identifier].address;

  for(i=15; i>=0; i--)  
  {
    if(get_bit(dataw,i)) {	
      biled_info[identifier].A1D=biled_info[identifier].A1D|0x400;
    } else {
      biled_info[identifier].A1D=biled_info[identifier].A1D&0xfbff;
    }

    biled_vme_write(BASE+0x10,biled_info[identifier].A1D);
    biled_info[identifier].A1D=biled_info[identifier].A1D|0x2000;
    biled_vme_write(BASE+0x10,biled_info[identifier].A1D);
    biled_info[identifier].A1D=biled_info[identifier].A1D&0xdfff;
    biled_vme_write(BASE+0x10,biled_info[identifier].A1D);
  }
}

/****************************************************************************/
/* get bit bitnum in word inword , return bitval                            */
/****************************************************************************/
int get_bit(int inword, int bitnum) 
{

  unsigned int bitval; 

  bitval = inword & (1 << bitnum);
  return (int) (bitval >> bitnum);
}

