/* Simple control program for Clock and Control Master from UCL
 * Original code by J.B.Lane.
 * Rejigged by JCH for RAID etc. at CERN. 8 Sep 1998
 */

#include <stdio.h>

/* Module base address */
#define CCBASE    0x00080000

/* Register offsets */
#define ENABLE    0x0
#define COMMAND   0x2
#define STRING    0x4
#define FREQUENCY 0x6
#define WINDOW    0x8
#define DELAY     0xA
#define STATE     0xC

/* Command strings loaded into "string" register */
#define TRIG      0x0003
#define SRST      0x0015
#define BCRST     0x0025
#define CAL1      0x1875
#define CAL2      0x00c0

/* Command register commands */
#define VSINGLETRIG   0x0001
#define VSRST         0x0002
#define VBCRST        0x0004
#define VCAL          0x0008
#define VOARST        0x0020
#define VBUSY         0x0080
#define LDTRIG        0x0100
#define LDSRST        0x0200
#define LDBCRST       0x0400
#define LDCAL1        0x0800
#define LDCAL2        0x1000

/* Function prototypes */
int cc_menu();
int cc_menu_enable(int v);
int cc_menu_disable(int v);
int cc_menu_command(int v);
int cc_menu_trfreq(int v);
int cc_menu_rsfreq(int v);
int cc_vmemap(unsigned long address, unsigned long *base);
int cc_load_enable(unsigned short value);
int cc_read_enable(unsigned short *value);
int cc_load_command(unsigned short value);
int cc_read_command(unsigned short *value);
int cc_load_string(unsigned short value);
int cc_load_frequency(unsigned short value);
int cc_read_frequency(unsigned short *value);
int cc_load_window(unsigned short value);
int cc_read_window(unsigned short *value);
int cc_load_delay(unsigned short value);
int cc_read_delay(unsigned short *value);
int cc_read_state(unsigned short *value);

/* Global variable */

unsigned long unix_address;

/* Main routine */

main()
{
    int status;
    unsigned short value;
    unsigned long vme_address;
    int n;

/* map to the module*/

    vme_address = CCBASE;
    status = cc_vmemap(vme_address,&unix_address);
    if(status != 0) exit(0);

/* Reset all registers */

    value = VOARST;  /* Over-all Reset */
    cc_load_command(value);
    value = 0x0;
    cc_load_command(value);
    
/* Enable internal/external clocks & triggers */

    value = 0x1;  /* Enable internal clock */
    cc_load_enable(value);

/*Zero other registers */
    value = 0x0;
    cc_load_window(value);
    value = 0x0;
    cc_load_delay(value);
    value = 0x0;
    cc_load_frequency(value);

/* Load command string(s). The command register has to be loaded with
   0 first, then 1 in the appropriate bit. If the command bit is >7, this
   loads the appropriate ABC command from the string register */

    value = 0x0;
    cc_load_command(value);

    value = TRIG;
    cc_load_string(value);
    value = LDTRIG;
    cc_load_command(value);
    value = 0x0; 
    cc_load_command(value);

    value = SRST;
    cc_load_string(value);
    value = LDSRST;
    cc_load_command(value);
    value = 0x0;
    cc_load_command(value);

    value = CAL1;
    cc_load_string(value);
    value = LDCAL1;
    cc_load_command(value);
    value = 0x0;
    cc_load_command(value);

    value = CAL2;
    cc_load_string(value);
    value = LDCAL2;
    cc_load_command(value);
    value = 0x0;
    cc_load_command(value);

    value = VSINGLETRIG;
    cc_load_command(value);
    value = 0x0;
    cc_load_command(value);


    cc_menu();
    
    return 0;
}


int cc_menu() {
  char c;
  int v;
  int run = 1;

  while(run) {
    printf("CLOAC: e(nable) d(isable) c(ommand) b(urst) t(rigfreq) r(esetfreq) x\n");
    scanf("%c %d\n", &c, &v);
    if (c == 'x') run = 0;
    else if (c == 'e') cc_menu_enable(v);
    else if (c == 'd') cc_menu_disable(v);
    else if (c == 'c') cc_menu_command(v);
    else if (c == 'b') cc_menu_burst(v);
    else if (c == 't') cc_menu_trfreq(v);
    else if (c == 'r') cc_menu_rsfreq(v);
    else {
      printf("CLOAC: ERROR: unknown command %c\n", c);
    }
  }
}

int cc_menu_enable(int v){
  unsigned short prev,next;

  if ((v<0) || (v>15)){
    printf("CLOAC: ERROR: enable argument %d outside valid range\n", v);
  } else {
    cc_read_enable(&prev);
    printf("CLOAC: enable reg was %4x\n", prev);
    next = prev | (unsigned short) (1<<v);
    cc_load_enable(next);
    cc_read_enable(&prev);
    printf("CLOAC: enable reg should be %x and seems to be %x\n",next,prev);
  }
  return 0;
}

int cc_menu_disable(int v){
  unsigned short prev,next;

  if ((v<0) || (v>15)){
    printf("CLOAC: ERROR: disable argument %d outside valid range\n", v);
  } else {
    cc_read_enable(&prev);
    printf("CLOAC: enable reg was %4x\n", prev);
    next = prev & ~((unsigned short) (1<<v));
    cc_load_enable(next);
    cc_read_enable(&prev);
    printf("CLOAC: enable reg should be %x and seems to be %x\n",next,prev);
  }
    return 0;
}

int cc_menu_command(int v) {
  if ((v<0) || (v>15)){
    printf("CLOAC: ERROR: command argument %d outside valid range\n", v);
  } else {
    if ((v==6)||(v==7)) cc_load_command(1<<v);   /* ### how to unset ? */
    else {
      cc_load_command(1<<v);
      cc_load_command(0);
    }
  }

  return 0;
}

int cc_menu_trfreq(int v){
  unsigned short previous,next;
  
  if ((v<0) || (v > 32)) return -3;

  cc_read_frequency(&previous);
  printf("CLOAC: freq reg was %x\n", previous);

  next = (previous & 0x1f00) | (unsigned short) v;
  cc_load_frequency(next);

  cc_read_frequency(&previous);
  printf("CLOAC: freq reg seems to be %x and should be %x\n", previous,next);
  return 0;
}

int cc_menu_rsfreq(int v){
  return 0;
}

int cc_menu_burst(int v){
  unsigned short previous, next;
  
  if ((v<0) || (v>0xff)) return -3;

  cc_read_command(&previous);
  cc_load_command(previous | 0x0040); /* enable bursts; stop triggers now */

  cc_load_string((unsigned short) v);  /* put value in string reg */

  cc_load_command(previous | 0x8040);  /* enable triggers */
  cc_load_command(previous | 0x0040);  /* reset loadburst bit */
  return 0;
}


/* cc_load_command.
 * Load the contents of the input argument into the command register.
 */

int cc_load_command(unsigned short value)
{
    unsigned short *ip;

    ip = (unsigned short *)(unix_address+COMMAND);

    *ip = value;
    return 0;
}


/* cc_load_delay.
 * Load the contents of the input argument into the delay register.
 */

int cc_load_delay(unsigned short value)
{
    unsigned short *ip;

    ip = (unsigned short *)(unix_address+DELAY);

    *ip = value;
    return 0;
}


/* cc_load_enable.
 * Load the contents of the input argument into the enable register.
 */

int cc_load_enable(unsigned short value)
{


    unsigned short *ip;

    ip = (unsigned short *)(unix_address+ENABLE);

    *ip = value;
    return 0;
}

/* cc_load_frequency.
 * Load the contents of the input argument into the frequency register.
 */

int cc_load_frequency(unsigned short value)
{
    unsigned short *ip;

    ip = (unsigned short *)(unix_address+FREQUENCY);

    *ip = value;
    return 0;
}


/* cc_load_string.
 * Load the contents of the input argument into the string register.
 */

int cc_load_string(unsigned short value)
{
    unsigned short *ip;

    ip = (unsigned short *)(unix_address+STRING);

    *ip = value;
    return 0;
}


/* cc_load_window.
 * Load the contents of the input argument into the window register.
 */

int cc_load_window(unsigned short value)
{
    unsigned short *ip;

    ip = (unsigned short *)(unix_address+WINDOW);

    *ip = value;
    return 0;
}


/* cc_read_command.
 * Read the contents of command register into the output argument.
 */

int cc_read_command(unsigned short *value)
{
    unsigned short *ip;

    ip = (unsigned short *)(unix_address+COMMAND);

    *value = *ip;
    return 0;
}


/* cc_read_delay.
 * Read the contents of delay register into the output argument.
 */

int cc_read_delay(unsigned short *value)
{
    unsigned short *ip;

    ip = (unsigned short *)(unix_address+DELAY);

    *value = *ip;
    return 0;
}


/* cc_read_enable.
 * Read the contents of enable register into the output argument.
 */

int cc_read_enable(unsigned short *value)
{
    unsigned short *ip;

    ip = (unsigned short *)(unix_address+ENABLE);

    *value = *ip;
    return 0;
}


/* cc_read_frequency.
 * Read the contents of frequency register into the output argument.
 */

int cc_read_frequency(unsigned short *value)
{
    unsigned short *ip;

    ip = (unsigned short *)(unix_address+FREQUENCY);

    *value = *ip;
    return 0;
}


/* cc_read_state.
 * Read the contents of "busy state" register into the output argument.
 */

int cc_read_state(unsigned short *value)
{
    unsigned short *ip;

    ip = (unsigned short *)(unix_address+STATE);

    *value = *ip;
    return 0;
}


/* cc_read_window.
 * Read the contents of window register into the output argument.
 */

int cc_read_window(unsigned short *value)
{
    unsigned short *ip;

    ip = (unsigned short *)(unix_address+WINDOW);

    *value = *ip;
    return 0;
}


/* cc_vmemap.
 * Map from VME address space to unix address space.
 */

int cc_vmemap(unsigned long address, unsigned long *base)
{
    int status,am;
    char *addr;

    am = 0x39;         /*A24 access*/
    status = RD13_VmeMap(address,am,0x100,&addr,0);
    if(status != 0) {
       printf("VME mapping failed, status = %d\n",status);
       return status;
    }
    *base = (unsigned long) addr;

    return 0;
}







