//
//  TKwikFit.cpp
//
//  after TModule.cpp
//
//  Version 0.1 PWP 26.10.00
//  Version 0.2 PWP/BJG 03.04.02 - Support for Likelihood fits
//                                  of ERFC and ERF data
//  Version 0.21 PWP 04.04.02 - add support for normalised scans
//  Version 0.22 PWP 05.04.02 - debugged version
//

#include <iostream.h>
#include <iomanip.h>

extern "C" {

#include "../stlib/stlib.h"

}

#include "TKwikFit.h"

// The next section seems messy, but it is necessary because the
// LikelihoodFit functions must be implemented as simple static functions

TF1* ff;                      // pointer to fit function
TGraph* ch_data;              // pointer Graph to be fitted
double n[ST_MAX_SCANPOINTS];  // n_trig data array

// End of horrible bit!

ClassImp(TKwikFit)

TKwikFit::TKwikFit(short new_id){

  id = new_id;

  burst =0;
  scan_fitted = 0;
  use_ch_maxima = 0;
  normalised = 0;

  sprintf(name,"  ");

  char htitle[80];
  sprintf(longname,"Module %d %s", id, name);

  // Scan Histograms

  sprintf(htitle, "%s Stream 0 Fit Code", longname);
  h_code0 = new TH1S("h_code0", htitle, 768,0.5,768.5);
  h_code0->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 1 Fit Code", longname);
  h_code1 = new TH1S("h_code1", htitle, 768,0.5,768.5);
  h_code1->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 0 Fitted Mean", longname);
  h_mean0 = new TH1F("h_mean0", htitle, 768,0.5,768.5);
  h_mean0->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 1 Fitted Mean", longname);
  h_mean1 = new TH1F("h_mean1", htitle, 768,0.5,768.5);
  h_mean1->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 0 Fitted Sigma", longname);
  h_sigma0 = new TH1F("h_sigma0", htitle, 768,0.5,768.5);
  h_sigma0->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 1 Fitted Sigma", longname);
  h_sigma1 = new TH1F("h_sigma1", htitle, 768,0.5,768.5);
  h_sigma1->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 0 Chisq/NDF", longname);
  h_chisq0 = new TH1F("h_chisq0", htitle, 768,0.5,768.5);
  h_chisq0->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 1 Chisq/NDF", longname);
  h_chisq1 = new TH1F("h_chisq1", htitle, 768,0.5,768.5);
  h_chisq1->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 0 Figure of Merit", longname);
  h_fom0 = new TH1F("h_fom0", htitle, 768,0.5,768.5);
  h_fom0->SetXTitle("Channel Number");

  sprintf(htitle, "%s Stream 1 Figure of Merit", longname);
  h_fom1 = new TH1F("h_fom1", htitle, 768,0.5,768.5);
  h_fom1->SetXTitle("Channel Number");

  ResetData();

  ff=0;
  ch_data=0;
  likelihood=0; // default: use chisq minimisation
  use_range=1;  // default: trim fitted range to include only one bin
                //          at 100% occupancy and one bin 0% occupancy
}

  
TKwikFit::~TKwikFit(){

}


// These functions for book-keeping of data and displays:

void TKwikFit::ResetData(){
  short nch, nsp, stream;

  for (nch=0; nch<768; nch++){
    for (nsp=0; nsp<ST_MAX_SCANPOINTS; nsp++){
      scan[nsp][0][nch] = 0;
      scan[nsp][1][nch] = 0;
    }
  }

  for (stream=0;stream<2;stream++){
    for(nch=0;nch<6;nch++){
      chip_mean[stream][nch]=0;
      chip_sigma[stream][nch]=0;
      chip_chisq[stream][nch]=0;
      chip_prob[stream][nch]=0;
      chip_badfits[stream][nch]=0;
      chip_goodfits[stream][nch]=0;
    }
    side_mean[stream]=0;
    side_sigma[stream]=0;
    side_chisq[stream]=0;
    side_prob[stream]=0;
    side_badfits[stream]=0;
    side_goodfits[stream]=0;
  }

  overall_mean=0;
  overall_sigma=0;
  overall_chisq=0;
  overall_prob=0;
  overall_badfits=0;
  overall_goodfits=0;

  h_code0->Reset();
  h_code1->Reset();
  h_mean0->Reset();
  h_mean1->Reset();
  h_sigma0->Reset();
  h_sigma1->Reset();
  h_chisq0->Reset();
  h_chisq1->Reset();
  h_fom0->Reset();
  h_fom1->Reset();

  burst=0;
  scan_hi=0;
  scan_lo=0;
  scan_start=0;
  scan_stop=0;
  scan_step=0;
  scan_max_points=0;
  scan_fitted=0;
}


void TKwikFit::SetModuleName(char* mname){
  char htitle[80];

  sprintf(name,"%s",mname);
  sprintf(longname,"Module %d %s", id, name);

  
  sprintf(htitle, "%s Stream 0 Fit Code", longname);
  h_code0->SetTitle(htitle);
  sprintf(htitle, "%s Stream 1 Fit Code", longname);
  h_code1->SetTitle(htitle);

  sprintf(htitle, "%s Stream 0 Fitted Mean", longname);
  h_mean0->SetTitle(htitle);
  sprintf(htitle, "%s Stream 1 Fitted Mean", longname);
  h_mean1->SetTitle(htitle);

  sprintf(htitle, "%s Stream 0 Fitted Sigma", longname);
  h_sigma0->SetTitle(htitle);
  sprintf(htitle, "%s Stream 1 Fitted Sigma", longname);
  h_sigma1->SetTitle(htitle);

  sprintf(htitle, "%s Stream 0 Chisq/NDF", longname);
  h_chisq0 ->SetTitle(htitle);
  sprintf(htitle, "%s Stream 1 Chisq/NDF", longname);
  h_chisq1 ->SetTitle(htitle);

  sprintf(htitle, "%s Stream 0 Figure of Merit", longname);
  h_fom0 ->SetTitle(htitle);
  sprintf(htitle, "%s Stream 1 Figure of Merit ", longname);
  h_fom1 ->SetTitle(htitle);
}




//
// Scan functions
//


void TKwikFit::LoadScan(TH2F* h_scan_in0, TH2F* h_scan_in1, TH1F* h_nevent){

  int i,j;
  int norm = 1;

  ResetData();

  h_scan0 = (TH2F*) h_scan_in0;
  h_scan1 = (TH2F*) h_scan_in1;
 
  // Copy n_event data

  if(h_nevent!=0){
    for(i=0;i<h_nevent->GetNbinsX();i++){
      scan_trigs[i]= (int)h_nevent->GetBinContent(i);
      if(scan_trigs[i]>0){
        burst++;
      }
    }
  } else {
    cout << "LoadScan ERROR: histogram n(events) is not valid!!" << endl;
    return;
  }


  // Copy scan point data

  TAxis* axis = 0;
  if(h_scan0!=0){
    axis = h_scan0->GetYaxis();
  } else if(h_scan1!=0){
    axis = h_scan1->GetYaxis();
  }

  if(axis!=0){
    scan_max_points = axis->GetNbins();
    scan_start = (float) axis->GetBinCenter(1);
    scan_stop  = (float) axis->GetBinCenter(scan_max_points);
    scan_step  = (float) axis->GetBinWidth(1);

    if (scan_stop > scan_start){ 
      scan_lo = scan_start;
      scan_hi = scan_stop;
    }else{
      scan_lo = scan_stop;
      scan_hi = scan_start;
    }
    
    for(i=0;i<scan_max_points; i++){
      scan_points[i] = (float) axis->GetBinCenter(i+1);
    }
  } else {
    cout << "LoadScan ERROR: neither scan histogram is valid!!" << endl;
    return;
  }

  // Copy scan occupancy data for link 0

  if(h_scan0!=0){
    for(i=0;i<768;i++){
      for(j=0;j<scan_max_points; j++){
        scan[j][0][i] = (short)h_scan0->GetBinContent((770*(j+1))+i);
        if(h_scan0->GetBinContent((770*(j+1))+i)>2){
          norm = 0;
        }
      }
    }
  } else {
    cout << "LoadScan ERROR: h_scan0 is invalid!!" << endl;
  }

  // Copy scan occupancy data for link 1

  if(h_scan1!=0){
    for(i=0;i<768;i++){
      for(j=0;j<scan_max_points; j++){
        scan[j][1][i] = (short)h_scan1->GetBinContent((770*(j+1))+i);
      }
    }
  } else {
    cout << "LoadScan ERROR: h_scan1 is invalid!!" << endl;
  }

  if(norm){
    normalised = 1;
  }
}
 

void TKwikFit::FitScan(short fit_type, short cal_mode){

  // PWP 18.08.00
  // revised PWP/BJG 03.04.02

  double ch_max, ch_min;
  double err2;
  double level_1, level_50, level_82, level_98, level_100;
  double p0_guess, p1_guess, p2_guess;
  double range_lo, range_hi;

  double x[ST_MAX_SCANPOINTS];
  double y[ST_MAX_SCANPOINTS];
  double xe[ST_MAX_SCANPOINTS];
  double ye[ST_MAX_SCANPOINTS];

  unsigned short ch_code;
  int i,j,stream,start,step,chip;

  // 1   Initialisation

  // 1-1 clear variables

  for (stream=0;stream<2;stream++){
    for(chip=0;chip<6;chip++){
      chip_mean[stream][chip]=0;
      chip_sigma[stream][chip]=0;
      chip_chisq[stream][chip]=0;
      chip_fom[stream][chip]=0;
      chip_prob[stream][chip]=0;
      chip_badfits[stream][chip]=0;
      chip_goodfits[stream][chip]=0;
    }
    side_mean[stream]=0;
    side_sigma[stream]=0;
    side_chisq[stream]=0;
    side_fom[stream]=0;
    side_prob[stream]=0;
    side_badfits[stream]=0;
    side_goodfits[stream]=0;
  }

  overall_mean=0;
  overall_sigma=0;
  overall_chisq=0;
  overall_fom=0;
  overall_prob=0;
  overall_badfits=0;
  overall_goodfits=0;

  h_mean0->Reset();
  h_mean1->Reset();
  h_sigma0->Reset();
  h_sigma1->Reset();
  h_code0->Reset();
  h_code1->Reset();
  h_chisq0->Reset();
  h_chisq1->Reset();
  h_fom0->Reset();
  h_fom1->Reset();

  // 1-3 Select fit function appropriate to scan

  switch (fit_type){
  case ST_ERFC:
    ff = new TF1("ff", erfc, 0, 1000, 3);
    ff->SetParNames("Constant","Mean","Sigma");
    break;
  case ST_ERF:
    ff = new TF1("ff", erf,  0, 1000, 3);
    ff->SetParNames("Constant","Mean","Sigma");
    break;
  case ST_GAUS:
    ff = new TF1("ff", gaus,  0, 1000, 3); // not "gaus", PWP bugfix 27.10.00
    ff->SetParNames("Constant","Mean","Sigma");
    break;
  default:
    return;
    break;
  }

  // 1-3 Initialise x, xe and n (n_trigs)  arrays 
  // values of xe are somewhat arbitrary for now (1% of bin centre)
  // simultanesouly, calculate the largest expected event count for this scan

  int target_constant = 0;

  if(scan_stop>scan_start){
    for(j=0;j<=burst;j++){
      n[j] = scan_trigs[j];
      x[j] = scan_points[j];
      xe[j]= 0.01*x[j];

      if(n[j]==0) n[j]=1;  // give protection against divide by zero
                           // THIS SHOULD NOT HAPPEN!
      if(scan_trigs[j]>target_constant) target_constant = scan_trigs[j];
    }
  } else {
    for(j=0;j<=burst;j++){
      n[j] = scan_trigs[burst-j];
      x[j] = scan_points[burst-j];
      xe[j]= 0.01*x[j];

      if(n[j]==0) n[j]=1;  // give protection against divide by zero
                           // THIS SHOULD NOT HAPPEN!
      if(scan_trigs[j]>target_constant) target_constant = scan_trigs[j];
    }
  }


  if(normalised){
    target_constant = 1;
  }

  // 1-4 Default is to use target_constant as basis for target levels

  level_1 =   0.01 * target_constant;
  level_50 =  0.50 * target_constant;
  level_82 =  0.82 * target_constant;
  level_98 =  0.98 * target_constant;
  level_100 = target_constant;
  p0_guess =  target_constant;

  // 1-5 default values of p1_guess, p2_guess
  // NOT NECESSARY!! PWP 03.04.02

  // 2  Loop over channels
 
  if ((cal_mode<0) || (cal_mode>3)){  // all cal lines
    start = 0;
    step = 1;
  } else {             // only the designated cal line
    start = cal_mode;
    step = 4;
  }

  for(stream=0;stream<2;stream++){
    for (i=start; i<768; i+=step){

      // 2-1 Calculation of which chip hosts the current channel of interest

      chip = i/128;

      // 2-2 Form y array needed for the graph 
      //     and find max and min for this channel

      if(normalised){

        // Here we use the normalised data from the scan histogram

        ch_max = 0.0;
        ch_min = 1.0;

        if(scan_stop>scan_start){
          for(j=0;j<=burst;j++){
            if(stream){
              y[j]=h_scan1->GetBinContent((770*(j+1))+i);
            }else{
              y[j]=h_scan0->GetBinContent((770*(j+1))+i);
            }
            if(y[j]>ch_max) ch_max=y[j];
            if(y[j]<ch_min) ch_min=y[j];
          }
        } else {
          for(j=0;j<=burst;j++){
            if(stream){
              y[j]=h_scan1->GetBinContent((770*(burst+1-j))+i);
            }else{
              y[j]=h_scan0->GetBinContent((770*(burst+1-j))+i);
            }
            if(y[j]>ch_max) ch_max=y[j];
            if(y[j]<ch_min) ch_min=y[j];
          }
        }

      } else {

        // Here we use directly the raw data

        ch_max = 0.0;
        ch_min = 1000000.0;

        if(scan_stop>scan_start){
          for(j=0;j<=burst;j++){
            y[j] = scan[j][stream][i];
            if(y[j]>ch_max) ch_max=y[j];
            if(y[j]<ch_min) ch_min=y[j];
          }
        } else {
          for(j=0;j<=burst;j++){
            y[j] = scan[burst-j][stream][i];
            if(y[j]>ch_max) ch_max=y[j];
            if(y[j]<ch_min) ch_min=y[j];
          }
        }

      }

      // 2-3 Test channels for good data - 
      //     overtly bad channels will be skipped

      ch_code  = 0;

      if     (ch_max<level_1)   ch_code = CH_DEAD;  // max < 1%
      else if(ch_min>level_98)  ch_code = CH_STUCK; // min > 98% 
      else if(ch_max>target_constant) ch_code = CH_OVER;  // max > 100%
      else if(ch_max<target_constant) ch_code = CH_UNDER; // max < 100%


      if(ch_code&(unsigned short)BAD_TEST){
          chip_badfits[stream][chip]++;  // tally bad channels

      } else { // do the fit!!

        // 2-4-1 Calculate errors appropriate to binomial statistics 

        if(normalised){

          // Here, y is a number between zero and 1. We multiply this by the 
          // number of triggers to calculate the binomial error on the count,
          // then we divide by the number of triggers to obtain the error in
          // terms of the normalised count.

          for(j=0;j<=burst;j++){
            err2 = ((n[j]*y[j])+1) * (1+n[j]-(n[j]*y[j])) / n[j];
            ye[j]= sqrt(err2); // this gives the error on the original count
            ye[j]/=n[j];       // this scales the error to the range 0 to 1 
          }

        } else {

          for(j=0;j<=burst;j++){
            err2 = (y[j]+1) * (1+n[j]-y[j]) / n[j];
            ye[j]= sqrt(err2);
          }
        }

        // 2-4-2 Use channel's own maxima as basis for target levels
        //       if option "use_ch_maxima" has been selected

        if(use_ch_maxima){
          level_50 = 0.50 * ch_max;
          level_82 = 0.82 * ch_max;
          level_100 = ch_max;
          p0_guess = ch_max;
        }

        // 2-4-3 Calculation of initial guess 
        //       and range limits to be used during fitting

        range_lo = x[0];
        range_hi = x[burst];

        switch (fit_type){
          case ST_ERFC: // fit back edge
                        // start high and work down
            j=burst;
            while((j>=0)&&(y[j]==0)){
              range_hi = x[j];
              j--;
            }
            while((j>=0)&&(y[j]<level_100)){
              if (y[j]<level_50) p1_guess = x[j];
              if (y[j]<level_82) p2_guess = x[j];
              j--;
            }
            if(j>0) range_lo = x[j];

	    p2_guess = p1_guess - p2_guess;
            break;
          case ST_ERF: // fit front edge
                       // start low and work up
            j=0;
            while((j<=burst)&&(y[j]==0)){
              range_lo = x[j];
              j++;
            }
            while((j<=burst)&&(y[j]<level_100)){
              if (y[j]<level_50) p1_guess = x[j];
              if (y[j]<level_82) p2_guess = x[j];
              j++;
            }
            if(j<burst) range_hi = x[j];

	    p2_guess = p2_guess - p1_guess;
            break;
          default:
            break;
        }


        // 2-4-4 Protection against failure of section 2-4-3!!

        if(p1_guess==0) p1_guess = scan_start+scan_stop/2;
        if(p2_guess==0) p2_guess = scan_step * 2;
        if(p2_guess<0) p2_guess*=-1;

        // 2-4-5 Creation of graph and minimisation
 
        ch_data = new TGraphErrors((burst+1),x,y,xe,ye);

        ff->SetParameters(p0_guess,p1_guess,p2_guess);
        ff->SetParLimits(0,p0_guess,p0_guess);
        if(use_range){
          ff->SetRange(range_lo, range_hi);
        }

        if(likelihood){ // log likelihood fit

          switch(fit_type){
            case ST_ERFC:
              if(normalised){
                TVirtualFitter::Fitter(ch_data)->SetFCN(NErfcLikelihoodFit);
              } else {
                TVirtualFitter::Fitter(ch_data)->SetFCN(ErfcLikelihoodFit);
              }
              ch_data->Fit("ff","UQR"); // quiet, use range, user supplied FCN
              break;
            case ST_ERF:
              if(normalised){
                TVirtualFitter::Fitter(ch_data)->SetFCN(NErfLikelihoodFit);
              } else {
                TVirtualFitter::Fitter(ch_data)->SetFCN(ErfLikelihoodFit);
              }
              ch_data->Fit("ff","UQR"); // quiet, use range, user supplied FCN
              break;
            default:
              // Sorry, no more likelihood fits! Use chisq
              ch_data->Fit("ff","QR"); // quiet mode, use range
              break;
          }

        }else{ // minimisation of chisquare
          ch_data->Fit("ff","QR"); // quiet mode, use range
        }

        // 2-4-6 Validity checking on fit data:
        //       mean value must be within scanned range
        //       sigma must be greater than zero

        // if(ff->GetParameter(1)<scan_lo) ch_code+= FIT_UNDER;
        // if(ff->GetParameter(1)>scan_hi) ch_code+= FIT_OVER;
        if(ff->GetParameter(2)<0)       ch_code+= SIG_UNDER;


        // 2-4-7 Fill arrays and histograms if fit is good

        chip = i/128;

        if(ch_code&BAD_TEST){
          chip_badfits[stream][chip]++;

          if(stream){
            h_mean1->SetBinError(i,0); 
            h_sigma1->SetBinError(i,0); 
          }else{
            h_mean0->SetBinError(i,0); 
            h_sigma0->SetBinError(i,0); 
          }
        } else {

          // Perform chisq and FOM goodness of fit test
          // PWP 05.04.02

          double y_fit;
          double residual;
          double divisor;
          double chisq = 0;
          double fom   = 0;
          int ndf = 0;

          if(normalised){
            for(j=0;j<burst;j++){
              if(ff->IsInside(&x[j])){ // restrict calculation of chisq
                                       // and FOM to the fitted range
                y_fit = ff->Eval(x[j])*n[j];  // estimated count
                residual = y_fit-(y[j]*n[j]); // residual count

                // calculate chisq
                // using binomial errors based on the fit
                if(n[j]>0){  
                  err2 = (y_fit+1) * (1+n[j]-y_fit) / n[j];
                  chisq += ( (residual*residual) / err2);
                  ndf++;
                }

                // calculate FOM
                divisor  = n[j] * y_fit;
                if(divisor>0){
                  fom += TMath::Abs((residual*scan_step)/divisor);
                }
              }
            }
          } else {
            for(j=0;j<burst;j++){
              if(ff->IsInside(&x[j])){ // restrict calculation of chisq
                                       // and FOM to the fitted range
                y_fit = ff->Eval(x[j]); // estimated count
                residual = y_fit-y[j];  // residual count

                // calculate chisq
                // using binomial errors based on the fit
                if(n[j]>0){  
                  err2 = (y_fit+1) * (1+n[j]-y_fit) / n[j];
                  chisq += ( (residual*residual) / err2);
                  ndf++;
                }

                // calculate FOM
                divisor  = n[j] * y_fit;
                if(divisor>0){
                  fom += TMath::Abs((residual*scan_step)/divisor);
                }
              }
            }
          }

          // Evaluate probability that, for the given number of
          // degrees of freedom and the calculated value of chisq,
          // the data is described by the fit
          chip_prob[stream][chip]  += (float)TMath::Prob(chisq,ndf);

          // Normalise chisq by number of degrees of freedom
          chisq /= ndf;

          chip_mean[stream][chip]  += (float)ff->GetParameter(1);
          chip_sigma[stream][chip] += (float)ff->GetParameter(2);
          chip_chisq[stream][chip] += (float)chisq;
          chip_fom[stream][chip]   += (float)fom;
          chip_goodfits[stream][chip]++;

          if(stream){
            h_mean1->Fill(i,ff->GetParameter(1)); 
            h_sigma1->Fill(i,ff->GetParameter(2)); 
            h_chisq1->Fill(i,chisq); 
            h_fom1->Fill(i,fom); 
            h_mean1->SetBinError(i,ff->GetParError(1)); 
            h_sigma1->SetBinError(i,ff->GetParError(2)); 
          }else{
            h_mean0->Fill(i,ff->GetParameter(1)); 
            h_sigma0->Fill(i,ff->GetParameter(2)); 
            h_chisq0->Fill(i,chisq); 
            h_fom0->Fill(i,fom); 
            h_mean0->SetBinError(i,ff->GetParError(1)); 
            h_sigma0->SetBinError(i,ff->GetParError(2)); 
          }
        }
        ch_data->Delete();
        ch_data=0;
      }

      // 2-5 Fill code histogram

      if (stream) h_code1->Fill(i,ch_code); 
      else        h_code0->Fill(i,ch_code); 
    }

    // 3 divide summed mean and sigma by number of good fits
    //   to give the mean values

    for(chip=0;chip<6;chip++){
      side_mean[stream]+= chip_mean[stream][chip];
      side_sigma[stream]+= chip_sigma[stream][chip];
      side_chisq[stream]+= chip_chisq[stream][chip];
      side_fom[stream]+= chip_fom[stream][chip];
      side_prob[stream]+= chip_prob[stream][chip];
      side_badfits[stream]+= chip_badfits[stream][chip];
      side_goodfits[stream]+= chip_goodfits[stream][chip];

      if(chip_goodfits[stream][chip]>0){
        chip_mean[stream][chip]/=chip_goodfits[stream][chip];
        chip_sigma[stream][chip]/=chip_goodfits[stream][chip];
        chip_chisq[stream][chip]/=chip_goodfits[stream][chip];
        chip_fom[stream][chip]/=chip_goodfits[stream][chip];
        chip_prob[stream][chip]/=chip_goodfits[stream][chip];
      }
    }
    overall_mean+= side_mean[stream];
    overall_sigma+= side_sigma[stream];
    overall_chisq+= side_chisq[stream];
    overall_fom+= side_fom[stream];
    overall_prob+= side_prob[stream];
    overall_badfits+= side_badfits[stream];
    overall_goodfits+= side_goodfits[stream];

    if(side_goodfits[stream]>0){
      side_mean[stream]/=side_goodfits[stream];
      side_sigma[stream]/=side_goodfits[stream];
      side_chisq[stream]/=side_goodfits[stream];
      side_fom[stream]/=side_goodfits[stream];
      side_prob[stream]/=side_goodfits[stream];
     }
  }

  if(overall_goodfits>0){
    overall_mean/=overall_goodfits;
    overall_sigma/=overall_goodfits;
    overall_chisq/=overall_goodfits;
    overall_fom/=overall_goodfits;
    overall_prob/=overall_goodfits;
  }

  scan_fitted =1;
  ff->Delete();
  ff=0;
}


void TKwikFit::PrintScan(){

  short chip, chip2, stream;

  cout << longname   << " ";
  cout << " scan from ";
  cout << scan_start << " to ";
  cout << scan_stop  << " in steps of ";
  cout << scan_step << endl;

  cout << "scan_max_points " << scan_max_points;
  cout << " n_bursts " << burst << endl;


  if(scan_fitted){
    if(likelihood){
      cout << "  Fit Results for likelihood fit" << endl;
    }else{
      cout << "  Fit Results for chisq fit" << endl;
    }
    cout << "\t\tMean\tSigma\tChisq\tFOM\tProb\tnGood\tnBad" << endl;


    for(stream=0;stream<2;stream++){

      for(chip=0;chip<6;chip++){
        chip2 = chip;
        if(stream>0) chip2+=6;
 
        cout << "  Chip " << chip2  << "\t";

        cout << setprecision(5) << chip_mean[stream][chip]   << "\t";
        cout << setprecision(4) << chip_sigma[stream][chip]   << "\t";
        cout << chip_chisq[stream][chip]   << "\t";
        cout << chip_fom[stream][chip]   << "\t";
        cout << chip_prob[stream][chip]   << "\t";
        cout << chip_goodfits[stream][chip] << "\t";
        cout << chip_badfits[stream][chip]  << endl;
      }


      cout << "  Link " << stream << "\t";
      cout << setprecision(5) << side_mean[stream]   << "\t";
      cout << setprecision(4) << side_sigma[stream]   << "\t";
      cout << side_chisq[stream]   << "\t";
      cout << side_fom[stream]   << "\t";
      cout << side_prob[stream]   << "\t";
      cout << side_goodfits[stream] << "\t";
      cout << side_badfits[stream]  << endl;
    }

    cout << "  Overall "  << "\t";
    cout << setprecision(5) << overall_mean   << "\t";
    cout << setprecision(4) << overall_sigma   << "\t";
    cout << overall_chisq   << "\t";
    cout << overall_fom   << "\t";
    cout << overall_prob   << "\t";
    cout << overall_goodfits << "\t";
    cout << overall_badfits  << endl;
  }
}


void ErfLikelihoodFit(Int_t &npar, Double_t *gin,
                      Double_t &f, Double_t *par, Int_t iflag) {

  // Minimization function for Erf data using a Likelihood method
  // We calculate the likelihood by determining the Binomial probability
  // that given a number of entries in a particular bin, the fit would 
  // predict its value.  This is then done for each bin and the sum of 
  // the logs is taken as the likelihood.

  // based on work by Simon Peeters and Sandra Muijs of NIKHEF
  // with advice from Bill Murray of RAL
  // Version 0.2  PWP/BJG 03.04.02

  // Version for scan data that is NOT NORMALISED

  // Parameters of the fit:
  // par[0] - Constant, fixed equal to the number of triggers (trials)
  // par[1] - Mean
  // par[2] - Sigma

  if(ch_data==0) return;

  Int_t np    = ch_data->GetN();
  Double_t *x = ch_data->GetX();
  Double_t *y = ch_data->GetY();

  Double_t P1;
  Double_t f1;
  Int_t bin;

  f = 0.0;

  // Evaluate the binomial probability that the function describes this data

  if(par[2]>0){ // protect against divide by zero errors

    for(bin=0;bin<np;bin++){

      if(ff->IsInside(&x[bin])){ // restrict fitting to range of function 

        if( (y[bin]>=0) && (y[bin]<=n[bin]) ){

          P1 = 0.5*(1.0+TMath::Erf((x[bin]-par[1])/(par[2]*1.414213562) ));
          f1 = TMath::Log(P1);

          if( (P1>10E-10) && (P1<(1-10E-10)) ){
            f += y[bin]*f1 + (n[bin]-y[bin])*TMath::Log(1.0-P1);
          }
        }
      }
    }
  }

  f *= -2.0; // Scale log likelihood to look like chisq in limit of large stats
}


void ErfcLikelihoodFit(Int_t &npar, Double_t *gin,
                       Double_t &f, Double_t *par, Int_t iflag) {

  // Minimization function for Erfc data using a Likelihood method
  // We calculate the likelihood by determining the Binomial probability
  // that given a number of entries in a particular bin, the fit would 
  // predict its value.  This is then done for each bin and the sum of 
  // the logs is taken as the likelihood.

  // based on work by Simon Peeters and Sandra Muijs of NIKHEF
  // with advice from Bill Murray of RAL
  // Version 0.2  PWP/BJG 03.04.02

  // Version for scan data that is NOT NORMALISED

  // Parameters of the fit:
  // par[0] - Constant, fixed equal to the number of triggers (trials)
  // par[1] - Mean
  // par[2] - Sigma

  if(ch_data==0) return;

  Int_t np    = ch_data->GetN();
  Double_t *x = ch_data->GetX();
  Double_t *y = ch_data->GetY();

  Double_t P1;
  Double_t f1;
  Int_t bin;

  f = 0.0;

  // Evaluate the binomial probability that the function describes this data

  if(par[2]>0){ // protect against divide by zero errors

    for(bin=0;bin<np;bin++){

      if(ff->IsInside(&x[bin])){ // restrict fitting to range of function 

        if( (y[bin]>=0) && (y[bin]<=n[bin]) ){

          P1 = 0.5*(1.0+TMath::Erf((par[1]-x[bin])/(par[2]*1.414213562) ));
          f1 = TMath::Log(P1);

          if( (P1>10E-10) && (P1<(1-10E-10)) ){
            f += y[bin]*f1 + (n[bin]-y[bin])*TMath::Log(1.0-P1);
          }
        }
      }
    }
  }

  f *= -2.0; // Scale log likelihood to look like chisq in limit of large stats
}


void NErfLikelihoodFit(Int_t &npar, Double_t *gin,
                      Double_t &f, Double_t *par, Int_t iflag) {

  // Minimization function for Erf data using a Likelihood method
  // We calculate the likelihood by determining the Binomial probability
  // that given a number of entries in a particular bin, the fit would 
  // predict its value.  This is then done for each bin and the sum of 
  // the logs is taken as the likelihood.

  // based on work by Simon Peeters and Sandra Muijs of NIKHEF
  // with advice from Bill Murray of RAL
  // Version 0.2  PWP/BJG 03.04.02

  // Version for scan data that IS NORMALISED

  // Parameters of the fit:
  // par[0] - Constant, fixed equal to the number of triggers (trials)
  // par[1] - Mean
  // par[2] - Sigma

  if(ch_data==0) return;

  Int_t np    = ch_data->GetN();
  Double_t *x = ch_data->GetX();
  Double_t *y = ch_data->GetY();

  Double_t ly;
  Double_t P1;
  Double_t f1;
  Int_t bin;

  f = 0.0;

  // Evaluate the binomial probability that the function describes this data

  if(par[2]>0){ // protect against divide by zero errors

    for(bin=0;bin<np;bin++){

      if(ff->IsInside(&x[bin])){ // restrict fitting to range of function 

        ly = y[bin]*n[bin];

        if( (ly>=0) && (ly<=n[bin]) ){

          P1 = 0.5*(1.0+TMath::Erf((x[bin]-par[1])/(par[2]*1.414213562) ));
          f1 = TMath::Log(P1);

          if( (P1>10E-10) && (P1<(1-10E-10)) ){
            f += (ly*f1 + (n[bin]-ly)*TMath::Log(1.0-P1))/n[bin];
          }
        }
      }
    }
  }

  f *= -2.0; // Scale log likelihood to look like chisq in limit of large stats
}


void NErfcLikelihoodFit(Int_t &npar, Double_t *gin,
                       Double_t &f, Double_t *par, Int_t iflag) {

  // Minimization function for Erfc data using a Likelihood method
  // We calculate the likelihood by determining the Binomial probability
  // that given a number of entries in a particular bin, the fit would 
  // predict its value.  This is then done for each bin and the sum of 
  // the logs is taken as the likelihood.

  // based on work by Simon Peeters and Sandra Muijs of NIKHEF
  // with advice from Bill Murray of RAL
  // Version 0.2  PWP/BJG 03.04.02

  // Version for scan data that IS NORMALISED

  // Parameters of the fit:
  // par[0] - Constant, fixed equal to the number of triggers (trials)
  // par[1] - Mean
  // par[2] - Sigma

  if(ch_data==0) return;

  Int_t np    = ch_data->GetN();
  Double_t *x = ch_data->GetX();
  Double_t *y = ch_data->GetY();

  Double_t ly;
  Double_t P1;
  Double_t f1;
  Int_t bin;

  f = 0.0;

  // Evaluate the binomial probability that the function describes this data

  if(par[2]>0){ // protect against divide by zero errors

    for(bin=0;bin<np;bin++){

      if(ff->IsInside(&x[bin])){ // restrict fitting to range of function 

        ly = y[bin]*n[bin]; // observed count

        if( (ly>=0) && (ly<=n[bin]) ){

          P1 = 0.5*(1.0+TMath::Erf((par[1]-x[bin])/(par[2]*1.414213562) ));
          f1 = TMath::Log(P1);

          if( (P1>10E-10) && (P1<(1-10E-10)) ){
            f += (ly*f1 + (n[bin]-ly)*TMath::Log(1.0-P1))/n[bin];
          }
        }
      }
    }
  }

  f *= -2.0; // Scale log likelihood to look like chisq in limit of large stats
}





