/* mus_user.c */

/* version 0.1 GFM Sep98
   version 0.2 PWP 030399  MSVC++ C compatibility 
   version 0.3 GFM May99*/

#include <stdio.h>
#include "mus_lib.h"
#include "mus_user.h"


/* These variables store the default initial & reset values; they are
   not intended for operational reference, i.e., do not indicate current
   settings. For this reason it is not proposed to duplicate records
   for each separate Mustard that may be in the system. */

short enable[12] = {0,0,0,0,0,0,0,0,0,0,0,0};
short delay[12] = {12,12,12,12,12,12,12,12,12,12,12,12};
short paf = 3;   /* PAF = Programmable Almost Full: 3=>8k */
short trlim = 3; /* TrLim = Trigger Limit:  3=> trigger limit does not occur */
short raw_mode = 0;
short sample_all = 1;
short mdaqen = 0;
short mrdoen = 0;
short normal_mode = 0;
short test_mode = 0;
short internal_clock = 0;   /* Internal test clock: not normally used */

int mus_enables(short id, short *val)
{
  int j,code;
  short i,k;

  for (i=0; i<=MAX_STREAM; i++) { 
    if (val[i] == 0) enable[i] = 0;
    else             enable[i] = 1;
  }
  for (i=0; i<=MAX_STREAM; i++) {
    j = mus_set_stream_enable(id,i,enable[i]);
    if (i==0) j = mus_set_stream_enable(id,i,enable[i]); 
    if (j != 0) code = j;
    j = mus_read_stream_enable(id,i,&k); 
    if (j != 0) code = j;
    j = mus_read_stream_enable(id,i,&k);
    j = mus_read_stream_enable(id,i,&k);
    j = mus_read_stream_enable(id,i,&k);
    if (k != enable[i]) printf(
        "mus_init: stream enable wrong,id=%d, stream=%d, set=%d, read=%d\n",
        id,i,enable[i],k); 
  }
  return code;
}


int mus_delays(short id, short *var)
{
  int j,code;
  short i;

  for(i=0;i<=MAX_STREAM;i++) {
    delay[i] = var[i];
  }
  for(i=0;i<=MAX_STREAM;i++) {
    j = mus_set_stream_delay(id,i,delay[i]);
    if(j != 0) code = j;
  }
  return code;
}


int mus_init(short id)
{
  int j,code;
  short i,k,l;

  code = 0;
  j = mus_reset(id);		/*reset the module*/
  if(j != 0) code = j;

  j = mus_set_clock_cntl(id, internal_clock);

  for(i=0;i<=MAX_STREAM;i++) {
      j = mus_set_stream_enable(id,i,enable[i]);
      if (i==0) j = mus_set_stream_enable(id,i,enable[i]); 
      if(j != 0) code = j;
      j = mus_read_stream_enable(id,i,&k); 
      if(j != 0) code = j;
      j = mus_read_stream_enable(id,i,&k); 
      j = mus_read_stream_enable(id,i,&k); 
      j = mus_read_stream_enable(id,i,&k); /* ask Maurice */
      if(k != enable[i]) printf(
        "mus_init: stream enable wrong,id=%d, stream=%d, set=%d, read=%d\n",
        id,i,enable[i],k); 
  }

  for(i=0;i<=MAX_STREAM;i+=4) {
      j = mus_set_stream_paf(id,i,paf);
      if(j != 0) code = j;
      j = mus_read_stream_paf(id,i,&k);
      if(j != 0) code = j;

      if(k != paf) printf(
        "mus_init: stream PAF wrong,id=%d, stream=%d, set=%d, read=%d\n",
        id,i,paf,k);
      j = mus_set_stream_trlim(id,i,trlim);
      if(j != 0) code = j;
      j = mus_read_stream_trlim(id,i,&k);
      if(j != 0) code = j;
      if(k != trlim) printf(
        "mus_init: stream TRLIM wrong,id=%d, stream=%d, set=%d, read=%d\n",
        id,i,trlim,k);
  }

  for(i=0;i<=MAX_STREAM;i++) {
      j = mus_set_stream_delay(id,i,delay[i]);
      if(j != 0) code = j;
  }

  j = mus_set_testmode(id,normal_mode);
  j = mus_read_testmode(id,&k);
  if(k != normal_mode) printf(
     "mus_init: testmode wrong,id=%d, set=%d, read=%d\n",
     id,normal_mode,k);
  if(j != 0) code = j;

  j = mus_set_readout_modes(id,raw_mode,sample_all);
  if(j != 0) code = j;
  j = mus_read_readout_modes(id,&k,&l);
  if(j != 0) code = j;
  if(k != raw_mode || l != sample_all) printf(
     "mus_init: readout mode register wrong,id=%d, set=%d %d, read=%d %d\n",
     id,raw_mode, sample_all,k,l);

  j = mus_set_readout_cntl(id,mdaqen,mrdoen);
  if(j != 0) code = j;
  j = mus_read_readout_cntl(id,&k,&l);
  if(j != 0) code = j;
  if(k != mdaqen || l != mrdoen) printf(
     "mus_init: readout cntl register wrong,id=%d, set=%d %d, read=%d %d\n",
     id,mdaqen,mrdoen,k,l);

  printf("mus_init: setting readout_cntl %x mdaqen %x %x  mrdoen %x %x\n", 
    j, mdaqen, k, mrdoen, l);

  j = mus_read_status(id,&k);
  if(j != 0) code = j;
  printf("mus_init: status register contains %x\n",k);

  return code;
}


int mus_print_inits(short id)
{
  int i;
 
  printf("\n");
  printf("mus_user: Current mustard default settings are:\n");
  printf("     enables:");
  for (i=0; i<12; i++) printf(" %d",enable[i]);
  printf("\n");
  printf("      delays:");
  for (i=0; i<12; i++) printf(" %d",delay[i]);
  printf("\n");
  printf("         paf: %d       trlim: %d\n",paf,trlim);
  printf("    raw_mode: %d  sample_all: %d\n",raw_mode,sample_all);
  printf("      mdaqen: %d      mrdoen: %d\n",mdaqen,mrdoen);
  printf(" normal_mode: %d   test_mode: %d\n",normal_mode,test_mode);
  printf("\n");

  return(0);
}


int mus_print_status(short st)
{
  short sadd; 
  short rdpage,wrpage;

  printf("mus_stat: %3x ", st);
  sadd = st & 0x00f;  printf(" SADD: %1x ", sadd);
  if (st & 0x010) printf(" MDAQEN ");
  if (st & 0x020) printf(" MRDOEN ");
  if (st & 0x080) printf(" SSTAT ");
  if (st & 0x100) printf(" SDAV ");
  if (st & 0x200) printf(" SREN ");
  if ((st & 0x400) == 0) printf(" TRH ");
  if (st & 0x800) printf(" BCO-OK ");
 
  rdpage = ((st & 0x040) >> 6);
  rdpage = rdpage | ((st & 0x1000) >> 11); printf(" RDPAGE: %x ", rdpage);
  wrpage = ((st >> 13) & 3);
  printf("WRPAGE: %d",wrpage);

  printf("\n");

  return 0;
}

int pr16bits(unsigned short v)
{
  int i;
  printf(" %x ",v);
  for (i=15;i>=0;i--) { 
    if ((v>>i)&0x0001) printf("1");
    else printf("0");
  }
  return 0;
}

void mus_dump_event_to_file(FILE *file, unsigned short *data){
  int i, strip;
  int word_count, error_count, event_count;

  if(file==0) return;

  word_count = data[0] & 0x3fff;
  error_count = data[1];
  event_count = data[2] + (data[3] << 8);

  fprintf(file,"%4x  WORD_COUNT  %d\n",  (unsigned short)data[0],word_count);
  fprintf(file,"%4x  ERROR_COUNT  %d\n", (unsigned short)data[1],error_count);
  fprintf(file,"%4x\n",                  (unsigned short)data[2]);
  fprintf(file,"%4x  EVENT_COUNT  %d\n", (unsigned short)data[3],event_count);

  for (i=4; i<word_count; i++){
    fprintf(file,"%4x  ", (unsigned short)data[i]);
      if (data[i] & 0x8000) {
        fprintf(file,     "  STR HDR  L1 %1x  BC %2x\n", (data[i]&0x0f00)>>8, data[i]&0x00ff);
        fprintf(file,"%4x    STR NO.\n", (unsigned short)data[i+1]);
        i++;
      } else if (data[i] & 0x2000) {
        strip = data[i]&0x07ff;
        fprintf(file,"    NEXT HIT  Ch %3x\n", strip);
      } else {
        fprintf(file,"      Ch %3x\n", strip);
        strip++;
      }
  }
}

