#include <parameters.h>
#define MAXNDAT 20

void InjectionStrip(Int_t *run, Int_t *scans, Char_t *exttit=0, Int_t plot=0){

  Int_t i,j,k,chip,goodchip, nread=1, nmod, chan,link;
  Float_t refns[MAX_MODULES][MAX_LINKS][MAX_CHIPS*128];
  Float_t injns[MAX_MODULES][MAX_LINKS][MAX_CHIPS*128];
  Float_t diff[144],maxnse=0.,tmp, delt=-999.,ntrigs;
  Char_t fname[128],mname[30][MAX_MODULES],text[128],line[1024];

  TGraph *grm;
  TPostScript *ps=0;

  FILE *output;
  TFile *strun[2];
  TH1F *h_bst[2][MAX_LINKS][MAX_MODULES], *diffnse[MAX_LINKS][MAX_MODULES];
  TLegend *leg=0;

  if(run[0]<=0 || run[1]<=0 || scans[0]<=0 || scans[1]<=0){
    printf("Invalid run/scan numbers: %d,%d  %d,%d\n",run[0],scans[0],run[1],scans[1]);
    return;
  }

  // read from data files
  for(i=0;i<2;i++){
    sprintf(fname,"%sstrun%d_%d.root",DATA_DIRECTORY,run[i],scans[i]);
    strun[i] = new TFile(fname);
    TObjString* time  = (TObjString*)strun[i]->Get("Time");
    TObjString* place = (TObjString*)strun[i]->Get("Place");
    for(j=0;j<MAX_MODULES;j++){
      sprintf(text,"ModuleName;%d",j+1);
      TObjString* name = (TObjString*)strun[i]->Get(text);
      if(name){
	if(strcmp(name->GetName(),"missing")){
	  sprintf(mname[j],"%s",name->GetName());
	}
	else
	  break;
      }
      else
	break;
      for(link=0;link<MAX_LINKS;link++){
	sprintf(text,"h_sigma%d;%d",link,j+1);
	h_bst[i][link][j]= (TH1F*)strun[i]->Get(text);
      }
    }
    if(i==0){
      nmod = j;
      printf("found %d modules\n",nmod);
    }
  }
  
  // difference and scale by gain
  for(j=0;j<nmod;j++){
    for(link=0;link<2;link++){
      diffnse[link][j] = (TH1F*) h_bst[1][link][j]->Clone();
      diffnse[link][j]->Add(h_bst[0][link][j],-1);
      diffnse[link][j]->Scale(6240/55.);
    }
  }
    

  // plot
  
  // open canvas
  TCanvas* cinj= new TCanvas ("cinj","Noise Injection Profile",1100,750);
  gStyle->SetOptStat(0);
  
  // open ps file
  if(plot){
    sprintf(fname,"%sInjectionStrip_%d_%d.ps",PS_DIRECTORY,run[0],scans[0]);
    ps = new TPostScript(fname,112); 
  }

  for(j=0;j<nmod;j++){
  //for(j=3;j<4;j++){
    if(ps) ps->NewPage();
    cinj->cd();
    cinj->Clear();
    pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00,0);
    pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90,0);
    pad_text->Draw();
    pad_plot->Draw();
    pad_plot->Divide(2,1,0,0);
    cinj->Update();

    pad_text->cd();
    TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
    t1->SetFillColor(10);
    t1->SetTextAlign(13);
    t1->SetTextSize(0.3);
    t1->SetTextColor(4);
    sprintf(line,"ATLAS SCT: noise injection");
    if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
    if(place) sprintf(line,"%s  -  %s",line,place->GetName());
    t1->AddText(line);
    sprintf(line,"Difference in noise between (run %d, scan %d) and (run %d, scan %d) -- Module %s",
	    run[1],scans[1],run[0],scans[0],mname[j]);
    t1->AddText(line);
    if(exttit) t1->AddText(exttit);
    t1->Draw();
    cinj->Update();
    
    for(link=0;link<2;link++){
      pad_plot->cd(link+1);
      gPad->SetLeftMargin(0.15);
      diffnse[link][j]->SetXTitle("Strip");
      diffnse[link][j]->SetYTitle("Excess noise (ENC)");
      //diffnse[link][j]->SetTitleOffset(2.,"X");
      diffnse[link][j]->SetTitleOffset(1.8,"Y");
      diffnse[link][j]->SetNdivisions(508,"X");
      if(link)
	diffnse[link][j]->SetTitle("Bottom side");
      else
	diffnse[link][j]->SetTitle("Top side");
      diffnse[link][j]->SetLineColor(1);
      diffnse[link][j]->SetMarkerColor(1);
      diffnse[link][j]->Draw("AP");
    }
    cinj->Update();
    printf("type anything to continue:  ");
    scanf("%s",text);
  }

  if(ps) ps->Close();

  for(i=0;i<2;i++)
    strun[i]->Close();

  return;
}

void InjectionStrip(Int_t run, Int_t scan, Char_t *exttit=0, Int_t plot=0){

  Int_t runs[2], scans[2];

  runs[0] = run;
  runs[1] = run;

  scans[0] = scan-1;
  scans[1] = scan;

  InjectionStrip(runs,scans,exttit,plot);

}
