#include <../macros/parameters.h>

void NOvsQ2(Int_t run, Int_t scan, Int_t id){
  Double_t LOW_SCALE = 10E-8;
  Int_t i, nbins, link, chip, ngood;
  Double_t x[200],y[200], content, qthr, qn[MAX_CHIPS*MAX_LINKS];
  Float_t mvthr;
  char h_name[32], line[1024],mname[32];
  TCanvas* c3 =0;
  c3= new TCanvas ("c3","NO",1100,750);

  gROOT->LoadMacro("stan_util.cpp");
  gROOT->LoadMacro("ToolBox.cpp");
  gROOT->LoadMacro("mV2fC.cpp");
  gROOT->LoadMacro("PosProjY.cpp");

  TFile* strun;
  sprintf(line,"D:\\sctvar\\data\\strun%d_%d.root",run,scan);
  strun = new TFile(line);
  
  sprintf(h_name,"h_scan0;%d",id+1);
  TH2F* hscan0 = (TH2F*)strun->Get(h_name);
  sprintf(h_name,"h_scan1;%d",id+1);
  TH2F* hscan1 = (TH2F*)strun->Get(h_name);

  sprintf(h_name,"ModuleName;%d",id+1);
  TObjString* time  = (TObjString*)strun->Get("Time");
  TObjString* place = (TObjString*)strun->Get("Place");
  TObjString* name = (TObjString*)strun->Get(h_name);
  if(name)
    sprintf(mname,"%s",name->GetName());
  else
    return;

  TF1* f1;
  sprintf(h_name,"f1");
  f1 = new TF1(h_name, "pol1",  0, 1000);
  //f1->SetParNames("P0","P1");
  
  c3->cd();
  c3->Clear();
  
  pad_t = new TPad("pad_t","Text",        0.00,0.90,1.00,1.00,33);
  pad_p = new TPad("pad_p","Projections",   0.00,0.00,1.00,0.90,33);
  pad_t->Draw();
  pad_p->Draw();
  pad_p->Divide(4,3,0,0);
  
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(line,"ATLAS SCT Noise Occupancy");
  sprintf(line,"%s  -  log n.o. vs Qthr2",line);
  if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
  if(place) sprintf(line,"%s  -  %s",line,place->GetName());
  t1->AddText(line);
  
  sprintf(line,"Run %d   Scan %d   Module %d (%s)", run, scan, id, mname);
  t1->AddText(line);
  pad_t->cd();
  t1->Draw();
  
  pad_p->cd();
  for(link=0;link<MAX_LINKS;link++){
    for(chip=0;chip<MAX_CHIPS;chip++){
      
      if(link==0)
	//TH1F* tmp = (TH1F*) hscan0->ProjectionY("tmp",chip*128,(chip+1)*128-1);
	TH1F* tmp = (TH1F*) PosProjY(hscan0,chip*128,(chip+1)*128-1);
      else
	//TH1F* tmp = (TH1F*) hscan1->ProjectionY("tmp",chip*128,(chip+1)*128-1);
	TH1F* tmp = (TH1F*) PosProjY(hscan1,chip*128,(chip+1)*128-1);
      //tmp->Scale(1.0/128);
      nbins = tmp->GetNbinsX();
      ngood=0;
      for(i=0;i<nbins;i++){
	content = (Double_t) tmp->GetBinContent(i+1);
	//mvthr = tmp->GetBinCenter(i+1);
	mvthr = tmp->GetBinLowEdge(i+1);
	qthr = (Double_t) mV2fC(mname,chip+6*link,mvthr);
	if(qthr>0.0){ 
	  if(content>2.5e-9){
	    x[ngood] = qthr*qthr;
	    y[ngood] = TMath::Log(content);
	    ngood++;
	  }
	  else
	    break;
	}
      }
      TGraph* gr = new TGraph(ngood,x,y);

      // Qthr>>Qnoise for approximation
      f1->SetRange(0.5,2.5);
      gr->Fit("f1","NR"); // no graphics, quiet mode
      qn[chip+link*6] = 1./TMath::Sqrt(-2.*f1->GetParameter(1))/1.6e-4;
      
      pad_p->cd((link*6)+chip+1);
      
      sprintf(h_name,"Noise Occupancy link %d chip %d", link, chip);
      gr->SetTitle(h_name);
      gr->SetMarkerStyle(20);
      gr->SetMarkerSize(1);
      gr->SetMarkerColor((Color_t) 1);
      gr->Draw("AP");
      gr->GetXaxis()->SetTitle("Threshold, mV");
      gr->GetYaxis()->SetTitle("Log(occupancy)");
      gr->Draw("AP");
      f1->Draw("same");
    }
  }
  
  c3->Update();

  strun->Close();

  for(chip=0;chip<MAX_CHIPS*MAX_LINKS;chip++)
    printf("ENC(chip %d) = %.0lf\n",chip,qn[chip]);

  return;
}
Float_t find_grlevel2(TGraph* h_in, Float_t fraction, Int_t last){

  Int_t bin,i;
  Float_t y = fraction, *xdata, *ydata;

  xdata = h_in->GetX();
  ydata = h_in->GetY();

  if(last){         // find last bin which is > y
    for(i=1; i<h_in->GetN(); i++){
      if(ydata[i]>y) bin = i;
    }
  }else{             // find first bin which is > y
    for(i=h_in->GetN(); i>0; i--){
      if(ydata[i]>y) bin = i;
    }
  }

  Float_t x1 = xdata[bin-1];
  Float_t y1 = ydata[bin-1];
  Float_t x2 = xdata[bin+1];
  Float_t y2 = ydata[bin+1];

  Float_t dy = y2-y1;
  if(dy==0) dy=1; // protection against divide by 0

  return x1 + ((y-y1)*(x2-x1)/(dy));
}

