void NoQPlot(int run, int scan, int log, int graphics, int postscript){

  //
  // Version 0.10 PWP 08.12.00
  // Version 0.11 PWP 22.01.01
  // Version 0.12 LE  14.01.01 - Works for data taken without SCTLV, correct 
  //                             output to text file and uses parameters.h
  // Version 0.13 PWP 23.05.01 - fix several bugs, many still to go!!

  // STILL UNDER DEVELOPMENT!!
  #include <parameters.h>

  Double_t LOW_SCALE = 10E-8;
  Double_t onefc =1.;
  Double_t onefc_bin =0;

  Double_t occ[3];
  Int_t module, link, chip, chan, i;
  char f_name[64],h_name[32], line[128];
  char mname[MAX_MODULES][32];

  Float_t chip_no_off[MAX_LINKS][MAX_CHIPS];
  Float_t chip_mean_occ[MAX_LINKS][MAX_CHIPS];
  Float_t chip_rms_occ[MAX_LINKS][MAX_CHIPS];
  Float_t chip_est_enc[MAX_LINKS][MAX_CHIPS];

  // Locality options now moved to stan_util.cpp

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");
  gROOT->LoadMacro("ToolBox.cpp");

  // Global style variations
  stan_styles();
  gStyle.SetOptStat(0);

  // Create function objects

  TF1* f1[MAX_LINKS][MAX_CHIPS];
  for(link=0;link<MAX_LINKS;link++){
    for(chip=0;chip<MAX_CHIPS;chip++){
      sprintf(h_name,"f1_%d_%d",link,chip);
      f1[link][chip] = new TF1(h_name, erfc,  0, 1000, 3);
      f1[link][chip]->SetParNames("Constant","Mean","Sigma");
    }
  }

  // 
  // 2. Load the Fitted Scan Data
  //

  // Find today's date - needed for file output

  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Int_t the_time = dt->GetTime();

  sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan);

  TFile *strun = new TFile(f_name);

  if(strun==0){
    printf("Failed to open file %s - EXITING!!\n",f_name);
    return();
  }

  //
  // Open the output canvas
  //

  Int_t dowait=0;
  Int_t ps_page=0;
  Int_t screen_page=0;

  TCanvas* c3 =0;
  TPostScript* ps =0;

  if(graphics!=0){
    dowait=1;
  }

  if((graphics!=0)||(postscript!=0)){
    graphics = 1;
    c3= new TCanvas ("c3","NO",1100,750);
  }

  // Find histograms and graphs

  TH2F* h_scan[MAX_LINKS];
  TH1F* h_proj[MAX_LINKS];
  TH1F* h_sum;
  TGraph* icc;
  TGraph* idd;
  TGraph* power;
  TAxis*  axis;
  TMarker* OccMark[3];

  TH1F *h_nevent= (TH1F*)strun->Get("h_scan_evcnt");

  for(module=0;module<MAX_MODULES;module++){

    for(link=0;link<MAX_LINKS;link++){
      h_scan[link]=0;
      h_proj[link]=0;
    }
    for(link=0;link<3;link++){
      occ[link]=0;
      OccMark[link]=0;
    }
    h_sum=0;
    icc=0;
    idd=0;
    power=0;

    sprintf(h_name,"ModuleName;%d",module+1);
    TObjString* time  = (TObjString*)strun->Get("Time");
    TObjString* place = (TObjString*)strun->Get("Place");
    TObjString* name = (TObjString*)strun->Get(h_name);

    if(name) sprintf(mname[module],"%s",name->GetName());


    for(link=0;link<MAX_LINKS;link++){
      sprintf(h_name,"h_scan%d;%d",link,module+1);
      h_scan[link]= (TH2F*)strun->Get(h_name);
    }


    // proceed only if there is data for either link

    if((h_scan[0]!=0)&&(h_scan[1]!=0)){
      if((h_scan[0]->GetEntries()>0)||(h_scan[1]->GetEntries()>0)){

        printf("Analysing data for module %d\n",module);

        axis = (TAxis*)h_scan[0]->GetYaxis();

        for(link=0;link<MAX_LINKS;link++){
          sprintf(h_name,"h_proj_%d",link);
          h_scan[link]->ProjectionY(h_name,0,767);
          h_proj[link]= (TH1F*)gDirectory->GetList()->FindObject(h_name);
          h_proj[link]->Scale(1.0/768);
          h_proj[link]->SetXTitle(axis->GetTitle());

          occ[link] = h_proj[link]->GetBinContent(h_proj[link]->GetXaxis()->FindBin(onefc));
        }

        // sum the two histograms
        sprintf(h_name,"h_sum");
        h_scan[0]->ProjectionY(h_name,0,767);
        h_sum = (TH1F*)gDirectory->GetList()->FindObject(h_name);
        h_sum->Scale(1.0/768);
        h_sum->Add(h_proj[1],1);
        h_sum->Scale(1.0/2);
        h_sum->SetXTitle(axis->GetTitle());

        onefc_bin = h_sum->GetXaxis()->FindBin(onefc);
        occ[2] = h_sum->GetBinContent(onefc_bin);


	//
	// Analyse the data chip by chip - LE 15/5 2001
        // Revised to include error bars PWP 25.05.01
	//

        Int_t x;
        Float_t err2,n,z;
	TH1F* hp_chip[MAX_LINKS][MAX_CHIPS];
	TH1F* h_logocc = new TH1F("h_logocc","LogOccupancy at 1fC", 300, -10, 0);
	
	for(link=0;link<MAX_LINKS;link++){
	  for(chip=0;chip<MAX_CHIPS;chip++){
	    
	    // Project one histogram per chip	
	    sprintf(h_name,"hp_chip_%d_%d",link,chip);
	    h_scan[link]->ProjectionY(h_name,(chip*128),((chip+1)*128)-1);
	    hp_chip[link][chip] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
	    
            if(hp_chip[link][chip]->GetMaximum()>0.0){ // data for this chip
              hp_chip[link][chip]->Scale(1.0/128);

              // Calculate errors appropriate to binomial statistics
              for(x=1;x<hp_chip[link][chip]->GetNbinsX()+1;x++){
                n = h_nevent->GetBinContent(x);
                if (n==0) n=1;
                z = hp_chip[link][chip]->GetBinContent(x);
                err2 = (1+(n*z))*(1+(n-(n*z))) / n;
                hp_chip[link][chip]->SetBinError(x,sqrt(err2)/n);
              }

              // Clear variables
              h_logocc->Reset();

	      // Fill histogram with LogOccupancy
	      for(chan=chip*128;chan<(chip+1)*128;chan++){
                Float_t chan_occ = h_scan[link]->GetBinContent(770*(onefc_bin)+chan);
                Float_t log_chan_occ = TMath::Log10(chan_occ);
                h_logocc->Fill(log_chan_occ);

                // printf("Chip %d, chan %d: Occ %1.1e\tLogOcc %5.2f\n",chip,chan, chan_occ, log_chan_occ);
              }

      
	      // Extract mean and RMS occupancy
              chip_mean_occ[link][chip] = TMath::Power(10,h_logocc->GetMean());
              chip_rms_occ[link][chip]  = TMath::Power(10,h_logocc->GetRMS());
	     	      
	      
              // Fit Erfc to the projection.
              Float_t centre = (find_bin2(hp_chip[link][chip],0.99,0,1)+find_bin2(hp_chip[link][chip],0.99,1,0))/2;
              Float_t p0_guess = hp_chip[link][chip]->GetMaximum();
              Float_t p1_guess = find_level2(hp_chip[link][chip],0.50,1);          // last bin > 50%
              Float_t p2_guess = find_level2(hp_chip[link][chip],0.82,1)-p1_guess;  
              Float_t range_lo = centre;
              Float_t range_hi = hp_chip[link][chip]->GetBinCenter(hp_chip[link][chip]->GetNbinsX());
              if(p2_guess<0) p2_guess = p2_guess*-1.0;
      
              f1[link][chip]->SetParameters(p0_guess,p1_guess,p2_guess);
              f1[link][chip]->SetParLimits(0,p0_guess,p0_guess);
//              f1[link][chip]->SetRange(range_lo, range_hi);
              f1[link][chip]->SetRange(-.5,1.5);  // fits data, but why?  trimming??


              sprintf(h_name,"f1_%d_%d",link,chip);
              hp_chip[link][chip]->Fit(h_name,"NR"); // no graphics, quiet mode
              chip_no_off[link][chip] = f1[link][chip]->GetParameter(1); 
	    }
	  }
	}
	

        sprintf(h_name,"Icc;%d",module+1);
        icc = (TGraph*)strun->Get(h_name);
        sprintf(h_name,"Idd;%d",module+1);
        idd = (TGraph*)strun->Get(h_name);

	if(icc&&idd){
	  Float_t* xdata;
	  Float_t* icc_data;
	  Float_t* idd_data;
	  Float_t  power_data[128];
	  
	  Float_t  icc_xmax = -1;
	  Float_t  icc_ymax=0;
	  
	  Float_t  idd_xmax = -1;
	  Float_t  idd_ymax=0;
	  
	  Float_t  power_xmax = -1;
	  Float_t  power_ymax=0;

	  xdata = icc->GetX();
	  icc_data = icc->GetY();
	  idd_data = idd->GetY();
	  for(i=0;i<icc->GetN();i++){
	    if(icc_data[i]>icc_ymax){
	      icc_xmax = xdata[i];
	      icc_ymax = icc_data[i];
	    }
	    if(idd_data[i]>idd_ymax){
	      idd_xmax = xdata[i];
	      idd_ymax = idd_data[i];
	    }
	    power_data[i]=(0.0035*icc_data[i])+(0.004*idd_data[i]);
	    if(power_data[i]>power_ymax){
	      power_xmax = xdata[i];
	      power_ymax = power_data[i];
	    }
	  }

	  Float_t icc_typ=icc_data[onefc_bin-1];
	  Float_t idd_typ=idd_data[onefc_bin-1];
	  Float_t power_typ=power_data[onefc_bin-1];
	  
	  printf("Icc max %3.0fmA at threshold %3.2f\n",icc_ymax,icc_xmax);
	  printf("Icc typ %3.0fmA at threshold %3.2f\n",icc_typ,onefc);
	  printf("Idd max %3.0fmA at threshold %3.2f\n",idd_ymax,idd_xmax);
	  printf("Idd typ %3.0fmA at threshold %3.2f\n",idd_typ,onefc);
	  printf("Power max %1.2fW at threshold %3.2f\n",power_ymax,power_xmax);
	  printf("Power typ %1.2fW at threshold %3.2f\n",power_typ,onefc);

	  power = new TGraph((i),xdata,power_data);
	}

        if(c3){
          if(postscript){
            ps_page=0;
	    sprintf(f_name,"%s%s_NoPlot_%d_%d.ps",PS_DIRECTORY,mname[module],date,the_time);

            ps = new TPostScript(f_name,112);
          }

          if((screen_page>0)&&(dowait)){
            printf("Enter a string to continue\n");
            scanf("%s",line);
          }

          printf("Generating Plots for module %d\n",module);

          ps_page++;
          screen_page++;
          if(ps){
            ps->NewPage();
          }

          c3->cd();
          c3->Clear();

          pad_t = new TPad("pad_t","Text",        0.00,0.90,1.00,1.00,33);
          pad_s = new TPad("pad_s","Scans",       0.00,0.00,0.60,0.90,33);
          pad_p = new TPad("pad_p","Projections", 0.60,0.00,1.00,0.90,33);

          pad_t->Draw();
          pad_s->Draw();
          pad_s->Divide(1,2,0,0);
          pad_p->Draw();
          pad_p->Divide(1,2,0,0);

          for(link=0;link<MAX_LINKS;link++){
            pad_s->cd(link+1);
            gPad->SetLogz(log);
            pad_p->cd(link+1);
            gPad->SetLogy(log);
          }

          TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
          t1->SetFillColor(10);
          t1->SetTextAlign(13);
          t1->SetTextSize(0.3);
          t1->SetTextColor(4);
          sprintf(line,"ATLAS SCT Noise Occupancy");
          if(log) sprintf(line,"%s  -  log scale",line);
          else    sprintf(line,"%s  -  linear scale",line);
          if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
          if(place) sprintf(line,"%s  -  %s",line,place->GetName());
          t1->AddText(line);

          sprintf(line,"Page %d   Run %d   Scan %d   Module %d (%s)", 
                  ps_page, run, scan, module, mname[module]);
          t1->AddText(line);
          pad_t->cd();
          t1->Draw();

          for(link=0;link<MAX_LINKS;link++){
            pad_s->cd(link+1);
            gPad->SetLeftMargin(0.10);
            gPad->SetBottomMargin(0.15);
            h_scan[link]->SetTitleOffset(1.7,"X");
            h_scan[link]->SetTitleOffset(1.0,"Y");
            if(log){
              h_scan[link]->SetMinimum(LOW_SCALE);
            }
            h_scan[link]->SetXTitle("Channel Number");
            h_scan[link]->Draw("COLZ");

            pad_p->cd(link+1);
            gPad->SetLeftMargin(0.15);
            gPad->SetBottomMargin(0.15);
            h_proj[link]->SetTitleOffset(1.7,"X");
            h_proj[link]->SetTitleOffset(1.7,"Y");
            if(log){
              h_proj[link]->SetYTitle("Log(occupancy)");
              h_proj[link]->SetMinimum(LOW_SCALE);
              OccMark[link] = new TMarker (onefc,log10(occ[link]),23);
            } else {
              h_proj[link]->SetYTitle("Occupancy");
              OccMark[link] = new TMarker (onefc,occ[link],23);
            }
            h_proj[link]->Draw();
            OccMark[link]->SetMarkerColor(kBlue);
            OccMark[link]->SetMarkerSize(1.0);
            OccMark[link]->SetMarkerStyle(20);
            OccMark[link]->Draw("same");

            t_occ = new TText();
            sprintf(h_name,"%1.2E",occ[link]);
            if(log){
              t_occ->DrawText(onefc+10,log10(occ[link]),h_name);
            }else{
              t_occ->DrawText(onefc+10,occ[link],h_name);
            }
          }



          c3->Update();

          ps_page++;
          screen_page++;
          if(ps){
            ps->NewPage();
          }

          if((screen_page>0)&&(dowait)){
            printf("Enter a string to continue\n");
            scanf("%s",line);
          }

          c3->cd();
          c3->Clear();

	  if(icc&&idd){
	    pad_t = new TPad("pad_t","Text",        0.00,0.90,1.00,1.00,33);
	    pad_o = new TPad("pad_o","Occupancy",   0.00,0.00,0.60,0.90,33);
	    pad_g = new TPad("pad_g","Graphs", 0.60,0.00,1.00,0.90,33);
	    pad_t->Draw();
	    pad_o->Draw();
	    pad_g->Draw();
	    pad_g->Divide(1,3,0,0);

	  }else{
	    pad_t = new TPad("pad_t","Text",        0.00,0.90,1.00,1.00,33);
	    pad_o = new TPad("pad_o","Occupancy",   0.00,0.00,1.00,0.90,33);
	    pad_t->Draw();
	    pad_o->Draw();
          }

          TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
          t1->SetFillColor(10);
          t1->SetTextAlign(13);
          t1->SetTextSize(0.3);
          t1->SetTextColor(4);
          sprintf(line,"ATLAS SCT Noise Occupancy");
          if(log) sprintf(line,"%s  -  log scale",line);
          else    sprintf(line,"%s  -  linear scale",line);
          if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
          if(place) sprintf(line,"%s  -  %s",line,place->GetName());
          t1->AddText(line);

          sprintf(line,"Page %d   Run %d   Scan %d   Module %d (%s)", 
                  ps_page, run, scan, module, mname[module]);
          t1->AddText(line);
          pad_t->cd();
          t1->Draw();

          if(log){
            pad_o->SetLogy();
          }

          pad_o->cd();
          gPad->SetLeftMargin(0.15);
          gPad->SetBottomMargin(0.15);
          h_sum->SetTitleOffset(1.7,"X");
          h_sum->SetTitleOffset(2.0,"Y");
          if(log){
            h_sum->SetYTitle("Log(occupancy)");
            h_sum->SetMinimum(LOW_SCALE);
          } else {
            h_sum->SetYTitle("Occupancy");
          }
          h_sum->Draw();

          if(log){
            OccMark[2]=new TMarker (onefc,log10(occ[2]),23);
          }else{
            OccMark[2]=new TMarker (onefc,occ[2],23);
          }
          OccMark[2]->SetMarkerColor(kBlue);
          OccMark[2]->SetMarkerSize(1.0);
          OccMark[2]->SetMarkerStyle(20);
          OccMark[2]->Draw("same");

          TText* t_occ2 = new TText();
          sprintf(h_name,"%1.2E",occ[2]);
          if(log){
            t_occ2->DrawText(onefc+10,log10(occ[2]),h_name);
          }else{
            t_occ2->DrawText(onefc+10,occ[2],h_name);
          }
	  if(idd&&idd){
	    pad_g->cd(1);
	    gPad->SetLeftMargin(0.12);
	    gPad->SetBottomMargin(0.15);
	    icc->SetLineColor(kRed);
	    icc->SetMarkerColor(kRed);
	    icc->Draw("ALP");
	    TH1F* h_icc=(TH1F*) icc->GetHistogram();
	    h_icc->SetXTitle(axis->GetTitle());
	    h_icc->SetYTitle("Icc(mA)");
	    h_icc->SetTitleOffset(1.7,"X");
	    h_icc->SetTitleOffset(1.3,"Y");
	    
	    TMarker *IccMark1=new TMarker (onefc,icc_typ,23); 
	    IccMark1->SetMarkerColor(kBlue);
	    IccMark1->SetMarkerSize(0.5);
	    IccMark1->SetMarkerStyle(20);
	    IccMark1->Draw("same");
	    
	    TMarker *IccMark2=new TMarker (icc_xmax,icc_ymax,23); 
	    IccMark2->SetMarkerColor(kBlue);
	    IccMark2->SetMarkerSize(0.5);
	    IccMark2->SetMarkerStyle(20);
	    IccMark2->Draw("same");
	    
	    TText* t_icc0 = new TText();
	    sprintf(h_name,"%3.0fmA Typical",icc_typ);
	    t_icc0->DrawText(onefc+5,icc_typ,h_name);
	    
	    TText* t_icc1 = new TText();
	    sprintf(h_name,"%3.0fmA Max",icc_ymax);
	    t_icc1->DrawText(icc_xmax+5,icc_ymax,h_name);


	    pad_g->cd(2);
	    gPad->SetLeftMargin(0.12);
	    gPad->SetBottomMargin(0.15);
	    idd->SetLineColor(kRed);
	    idd->SetMarkerColor(kRed);
	    idd->Draw("ALP");
	    TH1F* h_idd=(TH1F*) idd->GetHistogram();
	    h_idd->SetXTitle(axis->GetTitle());
	    h_idd->SetYTitle("Idd(mA)");
	    h_idd->SetTitleOffset(1.7,"X");
	    h_idd->SetTitleOffset(1.3,"Y");
	    
	    TMarker *IddMark1=new TMarker (onefc,idd_typ,23); 
	    IddMark1->SetMarkerColor(kBlue);
	    IddMark1->SetMarkerSize(0.5);
	    IddMark1->SetMarkerStyle(20);
	    IddMark1->Draw("same");
	    
	    TMarker *IddMark2=new TMarker (idd_xmax,idd_ymax,23); 
	    IddMark2->SetMarkerColor(kBlue);
	    IddMark2->SetMarkerSize(0.5);
	    IddMark2->SetMarkerStyle(20);
	    IddMark2->Draw("same");
	    
	    TText* t_idd0 = new TText();
	    sprintf(h_name,"%3.0fmA Typical",idd_typ);
	    t_idd0->DrawText(onefc+5,idd_typ,h_name);
	    
	    TText* t_idd1 = new TText();
	    sprintf(h_name,"%3.0fmA Max",idd_ymax);
	    t_idd1->DrawText(idd_xmax+5,idd_ymax,h_name);
	    
	    
	    pad_g->cd(3);
	    gPad->SetLeftMargin(0.12);
	    gPad->SetBottomMargin(0.15);
	    sprintf(h_name,"Module %d %s Power",module,mname[module]);
	    power->SetTitle(h_name);
	    power->SetLineColor(kRed);
	    power->SetMarkerColor(kRed);
	    power->Draw("ALP");
	    TH1F* h_power=(TH1F*) power->GetHistogram();
	    h_power->SetXTitle(axis->GetTitle());
	    h_power->SetYTitle("Power(W)");
	    h_power->SetTitleOffset(1.7,"X");
	    h_power->SetTitleOffset(1.3,"Y");
	    
	    TMarker *PowerMark1=new TMarker (onefc,power_typ,23); 
	    PowerMark1->SetMarkerColor(kBlue);
	    PowerMark1->SetMarkerSize(0.5);
	    PowerMark1->SetMarkerStyle(20);
	    PowerMark1->Draw("same");
	    
	    TMarker *PowerMark2=new TMarker (power_xmax,power_ymax,23); 
	    PowerMark2->SetMarkerColor(kBlue);
	    PowerMark2->SetMarkerSize(0.5);
	    PowerMark2->SetMarkerStyle(20);
	    PowerMark2->Draw("same");
	    
	    TText* t_power0 = new TText();
	    sprintf(h_name,"%1.2fW Typical",power_typ);
	    t_power0->DrawText(onefc+5,power_typ,h_name);
	    
	    TText* t_power1 = new TText();
	    sprintf(h_name,"%1.2fW Max",power_ymax);
	    t_power1->DrawText(power_xmax+5,power_ymax,h_name);
	  } // end if(icc&&idd)

          c3->Update();

          ps_page++;
          screen_page++;
          if(ps){
            ps->NewPage();
          }

          if((screen_page>0)&&(dowait)){
            printf("Enter a string to continue\n");
            scanf("%s",line);
          }

          c3->cd();
          c3->Clear();

          pad_t = new TPad("pad_t","Text",        0.00,0.90,1.00,1.00,33);
          pad_p = new TPad("pad_p","Projections",   0.00,0.00,1.00,0.90,33);
          pad_t->Draw();
          pad_p->Draw();
          pad_p->Divide(4,3,0,0);

          TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
          t1->SetFillColor(10);
          t1->SetTextAlign(13);
          t1->SetTextSize(0.3);
          t1->SetTextColor(4);
          sprintf(line,"ATLAS SCT Noise Occupancy");
          if(log) sprintf(line,"%s  -  log scale",line);
          else    sprintf(line,"%s  -  linear scale",line);
          if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
          if(place) sprintf(line,"%s  -  %s",line,place->GetName());
          t1->AddText(line);

          sprintf(line,"Page %d   Run %d   Scan %d   Module %d (%s)", 
                  ps_page, run, scan, module, mname[module]);
          t1->AddText(line);
          pad_t->cd();
          t1->Draw();

          pad_p->cd();
          for(link=0;link<MAX_LINKS;link++){
            for(chip=0;chip<MAX_CHIPS;chip++){

              pad_p->cd((link*6)+chip+1);
              if(log){
                gPad->SetLogy();
              }

              sprintf(h_name,"Noise Occupancy link %d chip %d", link, chip);
              hp_chip[link][chip]->SetTitle(h_name);
              hp_chip[link][chip]->SetMinimum(LOW_SCALE);
	      hp_chip[link][chip]->SetXTitle(axis->GetTitle());
              if(log){
                hp_chip[link][chip]->SetYTitle("Log(occupancy)");
              }else{
                hp_chip[link][chip]->SetYTitle("Occupancy");
              }
	      hp_chip[link][chip]->Draw();
              f1[link][chip]->Draw("same");
            }
          }

          c3->Update();

        } //end if(c3)

        if(ps){
          ps->Close();
        }

        // 
        // Output data to text summary file
        //

        // Open file and print data-base header
        FILE* outfile = OpenResults("NoiseOccupancy", strun, module, run, scan, 1); 
       
        if(outfile==NULL){
          return();
        }

        // Print test-specific results
        if(icc&&idd){
         fprintf(outfile,"IccMax: %3.0f\n",icc_ymax);
         fprintf(outfile,"IddMax: %3.0f\n",idd_ymax);
         fprintf(outfile,"IccTyp: %3.0f\n",icc_typ);
         fprintf(outfile,"IddTyp: %3.0f\n",idd_typ);
         fprintf(outfile,"#\n");
        }
       
        fprintf(outfile,"%%Summary\n");
        fprintf(outfile,"#\n");
       
        fprintf(outfile,"#\t\tOffset\tMeanOcc\tRMSOcc\tEstENC\n");
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
	    fprintf(outfile,"Chip %2d:\t%4.1f\t%5.1g\t%4.1g\t%4.0f\n",
              (6*link)+chip,
              chip_no_off[link][chip],
              chip_mean_occ[link][chip],
              chip_rms_occ[link][chip],
              0.0);
          }
        }
        fprintf(outfile,"#\n");
        fclose(outfile);
      } else {
        printf("No data for module %d\n",module);
      }
    } else {
      printf("No histograms for module %d\n",module);
    }
  }//end for(module)
}

void NoPlot(){

  printf("Possible Uses of NoPlot:\n\n");
  printf(".x NoPlot.cpp(run,scan,graphics,postscript)\n");

}

