void cn_strip_matrix(Int_t runno, Int_t scanno) {

  gROOT->Reset();
  
#include "../macros/systest/Evtree.h"
#include "../macros/parameters.h"

  Char_t line[1024],filename[70],module0[30],module1[30], filestore[50],mod_name[50],mname[MAX_MODULES][50];
  Int_t nmod, totchips, nevt, chip, chip2, nchips=MAX_CHIPS*MAX_LINKS, strip, mod, mod2;

  Bool_t hita, hitb;

  Float_t meansqs[MAX_MODULES*MAX_CHIPS*MAX_LINKS];
 
  Float_t mean[MAX_MODULES*MAX_CHIPS*MAX_LINKS],sd[MAX_MODULES*MAX_CHIPS*MAX_LINKS];
  Float_t meansqd[MAX_MODULES*MAX_CHIPS*MAX_LINKS];
  Float_t meanprod[MAX_MODULES*MAX_CHIPS*MAX_LINKS][MAX_MODULES*MAX_CHIPS*MAX_LINKS];
  Float_t chipcorrel, stripcorrel;

  Double_t sum[MAX_MODULES*MAX_CHIPS*MAX_LINKS][MAX_TARGETS], denom;
  Double_t prod[MAX_MODULES*MAX_CHIPS*MAX_LINKS][MAX_MODULES*MAX_CHIPS*MAX_LINKS][MAX_TARGETS];
 
  Evtree* m[MAX_MODULES];
  TTree* tree[MAX_MODULES];

  TDatime *dt = new TDatime;
  printf("starting program at %d\n",dt->GetTime());

  //open the file

  TFile* f;
  sprintf(filename,"%sstrun%d_%d.root",DATA_DIRECTORY,runno,scanno);
  f = new TFile(filename);
  
  //loop over modules (over the trees) and open everything

  nmod = 0;
  for(Int_t mod=0;mod<MAX_MODULES;mod++){
    //for(mod=0;mod<3;mod++){
 
    if(mod==0){
      TObjString* time  = (TObjString*)f->Get("Time");
      TObjString* place = (TObjString*)f->Get("Place");
    }

    sprintf(mod_name,"ModuleName;%d",mod+1);
    TObjString* name = (TObjString*)f->Get(mod_name);
    if(name){
      sprintf(mname[mod],"%s",name->GetName());
      if(!strcmp(mname[mod],"missing")) break;
      nmod++;
    }
    else
      break;

    sprintf(module0,"EvTree_m%d",mod);
    tree[mod] = (TTree*)gDirectory->Get(module0);
    m[mod] = new Evtree(tree[mod]);
   
    if(mod==0) 
      nevt = tree[mod]->GetEntries();
    else
      if(nevt!=tree[mod]->GetEntries()){
	printf("trees have different number of events -> stop\n");
	return;
      }

  }
  printf("found %d modules with trees with %d entries\n",nmod,nevt);
  //nevt=2500;

  // define histogram

  totchips = nmod*nchips;
  TH2F *histo = new TH2F("histo","correlated noise chip by chip",totchips,0,totchips,totchips,0,totchips);
  histo->SetStats(kFALSE);

  //now loop over the events and the chips

  for(strip=0;strip<MAX_TARGETS;strip++){
    for(chip=0;chip<totchips;chip++){
      sum[chip][strip]=0.0;
      for(chip2=0;chip2<totchips;chip2++){
	prod[chip][chip2][strip]=0.0;
      }
    }
  }

  //start the event looop

  for(Int_t ev=0;ev<nevt;ev++){
    
    //load arrays
    for(mod=0;mod<nmod;mod++)
      m[mod]->GetEntry(ev);
 
    //get hits per strip in this event

    for(strip=0;strip<MAX_TARGETS;strip++){
      
      for(chip=0;chip<totchips;chip++){
	
	mod = chip/nchips;
	
	if((chip-mod*nchips)<MAX_CHIPS)
	  hita = m[mod]->strm0[strip+((chip-mod*nchips)*MAX_TARGETS)];
	else
	  hita = m[mod]->strm1[strip+((chip-mod*nchips-MAX_CHIPS)*MAX_TARGETS)];
	
	if(hita){
	  sum[chip][strip]++;
	
	  for(chip2=0;chip2<totchips;chip2++){
	    
	    mod2 = chip2/nchips;
	    
	    if((chip2-mod2*nchips)<MAX_CHIPS)
	      hitb = m[mod2]->strm0[strip+((chip2-mod2*nchips)*MAX_TARGETS)];
	    else
	      hitb = m[mod2]->strm1[strip+((chip2-mod2*nchips-MAX_CHIPS)*MAX_TARGETS)];
	    
	    if(hitb)
	      prod[chip][chip2][strip]++;
	  }
	  
	}
      }
    }
    
    if((ev+1)%100==0) printf("finished event %d \n",ev+1);  
    
  }
  
  //now have arrays sum, and prod
  
  //do the maths on the arrays
  
  for(strip=0;strip<MAX_TARGETS;strip++){
    for(chip=0;chip<totchips;chip++){
  
      //find mean
      
      sum[chip][strip]/=(Double_t)nevt;
  
      for(chip2=0;chip2<totchips;chip2++){
	
	//find mean of products
	
	prod[chip][chip2][strip]/=(Double_t)nevt;
      }
    }
  }
  
  for(chip=0;chip<totchips;chip++){
    for(chip2=0;chip2<totchips;chip2++){

      chipcorrel=0.;
      for(strip=0;strip<MAX_TARGETS;strip++){
	denom = TMath::Sqrt(sum[chip][strip]-sum[chip][strip]*sum[chip][strip])*
	            TMath::Sqrt(sum[chip2][strip]-sum[chip2][strip]*sum[chip2][strip]);
	if(denom)
	  stripcorrel = (Float_t)((prod[chip][chip2][strip]-sum[chip][strip]*sum[chip2][strip])/denom);
	else{
	  //printf("warning: zero denominator in element %d %d %d\n",chip,chip2,strip);
	  stripcorrel = 0.;
	}
	chipcorrel += stripcorrel;
      }
      chipcorrel/=(Float_t)MAX_TARGETS;
      //histo->Fill(chip,chip2,chipcorrel);
      histo->SetCellContent(chip+1,chip2+1,chipcorrel);
    }
  }
  
  TDatime *dt2 = new TDatime;
  printf("finished analysis at %d\n",dt2->GetTime());

  sprintf(filename,"%scmn_%d_%d.root",RESULTS_DIRECTORY,runno,scanno);
  TFile out(filename,"recreate");
  histo->Write("cmn_histo");
  for(mod=0;mod<nmod;mod++){
    TObjString * mns = new TObjString(mname[mod]);
    mns->Write("ModuleName");
  }
  sprintf(module0,"%d",nevt);
  TObjString * nevo = new TObjString(module0);
  nevo->Write("Events");
  if(time)  time->Write("Time");
  if(place) place->Write("Place");
  out->Close();

  return;
}
