#include <parameters.h>
#define MAXNDAT 20

void nL1AOver(Int_t runa, Int_t scana, Int_t runb, Int_t scanb){

  Int_t i,j,k,chip,goodchip, nread=1, nmod, chan,link;
  Float_t refns[MAX_MODULES][MAX_LINKS][MAX_CHIPS*128];
  Float_t injns[MAX_MODULES][MAX_LINKS][MAX_CHIPS*128];
  Float_t diff[144],maxnse=0.,tmp, delt=-999.,ntrigs;
  Char_t fname[128],mname[30][MAX_MODULES],text[128],line[1024];
  Int_t run[2],scans[2];

  TGraph *grm;
  TPostScript *ps;

  FILE *output;
  TFile *strun[2];
  TH1F *h_bst[2][MAX_LINKS][MAX_MODULES];
  TLegend *leg=0;

  run[0] = runa;
  run[1] = runb;
  scans[0]=scana;
  scans[1]=scanb;
  
  // read from data files
  for(i=0;i<2;i++){
    sprintf(fname,"%sstrun%d_%d.root",DATA_DIRECTORY,run[i],scans[i]);
    strun[i] = new TFile(fname);
    TObjString* time  = (TObjString*)strun[i]->Get("Time");
    TObjString* place = (TObjString*)strun[i]->Get("Place");
    TObjString* notrig = (TObjString*)strun[i]->Get("Ntrig");
    sscanf(notrig->GetName(),"%f", &ntrigs);
    if(i){
      TObjString* dt = (TObjString*)strun[i]->Get("TrDel");
      sscanf(dt->GetName(),"%f",&delt);
      delt *= 0.025;
    }
    for(j=0;j<MAX_MODULES;j++){
      sprintf(text,"ModuleName;%d",j+1);
      TObjString* name = (TObjString*)strun[i]->Get(text);
      if(name){
	if(strcmp(name->GetName(),"missing")){
	  sprintf(mname[j],"%s",name->GetName());
	}
	else
	  break;
      }
      else
	break;
      for(link=0;link<MAX_LINKS;link++){
	sprintf(text,"h_accbst%d_%d",link,j);
	h_bst[i][link][j]= (TH1F*)strun[i]->Get(text);
      }
    }
    if(i==0){
      nmod = j;
      printf("found %d modules\n",nmod);
    }
  }
  
  // plot
  
  // open canvas
  TCanvas* cinj= new TCanvas ("cinj","nL1A Occupancy Profile",1100,750);
  gStyle->SetOptStat(0);
  
  // open ps file
  sprintf(fname,"%snL1AOver_%d_%d.ps",PS_DIRECTORY,run[0],scans[0]);
  ps = new TPostScript(fname,112); 

  for(j=0;j<nmod;j++){
    ps->NewPage();
    cinj->cd();
    cinj->Clear();
    pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00,0);
    pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90,0);
    pad_text->Draw();
    pad_plot->Draw();
    pad_plot->Divide(2,1,0,0);
    cinj->Update();
    
    pad_text->cd();
    TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
    t1->SetFillColor(10);
    t1->SetTextAlign(13);
    t1->SetTextSize(0.3);
    t1->SetTextColor(4);
    sprintf(line,"ATLAS SCT: occupancy with double L1A");
    if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
    if(place) sprintf(line,"%s  -  %s",line,place->GetName());
    t1->AddText(line);
    sprintf(line,"#Deltat_{L1A's}=%.2f#mus (run %d, scan %d) compared to single-L1A (run %d, scan %d) -- Module %s",
	    delt,run[1],scans[1],run[0],scans[0],mname[j]);
    t1->AddText(line);
    sprintf(line,"Number of hits from  %.0f triggers",ntrigs);
    t1->AddText(line);
    t1->Draw();
    cinj->Update();
    
    if(leg) leg->Delete();
    leg = new TLegend(.7,.925,1.,1.);
    leg->SetTextSize(0.03);
    for(link=0;link<2;link++){
      pad_plot->cd(link+1);
      gPad->SetLogy(1);
      for(i=1;i>=0;i--){
	h_bst[i][link][j]->SetXTitle("Strip");
	h_bst[i][link][j]->SetYTitle("Hits");
	//h_bst[i][link][j]->SetTitleOffset(2.,"X");
	h_bst[i][link][j]->SetTitleOffset(1.2,"Y");
	h_bst[i][link][j]->SetNdivisions(508,"X");
	if(link){
	  h_bst[i][link][j]->SetTitle("Bottom side");
	  if(i)
	    leg->AddEntry(h_bst[i][link][j],"double-L1A","l");
	  else
	    leg->AddEntry(h_bst[i][link][j],"single-L1A","l");
	}	  
	else
	  h_bst[i][link][j]->SetTitle("Top side");
	if(i){
	  h_bst[i][link][j]->SetLineColor(2);
	  h_bst[i][link][j]->Draw("AL");
	} else{
	  h_bst[i][link][j]->SetLineColor(1);
	  h_bst[i][link][j]->Draw("SAMEL");
	}
      }
    }
    leg->Draw();
    cinj->Update();
    printf("type anything to continue:  ");
    scanf("%s",text);
  }

  ps->Close();

  for(i=0;i<2;i++)
    strun[i]->Close();


  return;
}
