//
//  stan_ascii2root.cpp
//  PWP 08.09.99
//

//
// definitions of relevant elements within the ASCII ntuple data row
//

#define SNO  0;  // scan number
#define MID  1;  // module ID
#define CHP  2;  // chip ID
#define MOD  3;  // cal_mode
#define VAR4 4;  // value set to variable TYP4
#define VAR3 5;  // value set to variable TYP3 
#define VAR2 6;  // value set to variable TYP2 
#define VAR1 7;  // value set to variable TYP1 
#define NEVT 8;  // number of events
#define NERR 9;  // number of errors - not yet used
#define INDX 10; // index for data

#define SOB_PNT4 11; // scan point counters 
#define SOB_PNT3 12;
#define SOB_PNT2 13;
#define SOB_PNT1 14;
#define SOB_TYP4 15; // scan variable types
#define SOB_TYP3 16;
#define SOB_TYP2 17;
#define SOB_TYP1 18;


Int_t get_val(char* line, Int_t index, Double_t* val){

  // convert next element of data row "line" to type double
  // (CINT sscanf is limited to <43 elements, hence this approach)

  char substring[20];

  sscanf(line+=index,"%s ",substring);
  sscanf(substring,"%lf",val);
  return 1 + strlen(substring);
}


void stan_ascii2root(Int_t run){

// Routine to convert ASCII ntuples into (simplified) ROOT ntuples
//
// PWP 01.09.99

  char line[1024], stub[4][16];
  char in_fname[64], out_fname[64], nt_name[32], nt_title[32], nt_members[64];
  Double_t chan, data[43];
  FILE* in_file = NULL;
  Int_t i, index, var_type[4];
  Int_t old_sno =0;

  // Load other files needed by this one

  gROOT->LoadMacro("stan_util.cpp");

  // open input and output files - strange mix of c and c++!!

  sprintf(in_fname,"H:\\sctvar\\data\\strun%d.dat",run);
  in_file=fopen(in_fname,"r");
  if (in_file == NULL){ 
    printf("stan_ascii2root: Failed to open st data file %s\n",in_fname);
    return;
  }
  printf("stan_ascii2root: File %s opened for reading\n",in_fname);

  sprintf(out_fname,"H:\\sctvar\\data\\strun%d.root",run);
  TFile* out_file = new TFile(out_fname,"RECREATE");

  for(i=0;i<4;i++){
    var_type[i]=0;
    match_var(var_type[i],stub[i]);
  }

  while(fgets(line,1024,in_file) != NULL){

    // only process two types of data record:
    // sob (to become sos) and occupancy data

    // read SNO, MID, CHP and MOD
    index = 0;
    for (i=0; i<11; i++) index += get_val(line, index, &data[i]);

      if(data[MOD]>=0){  /* REAL DATA!!! */
        chan = (((int)data[CHP])*128)+data[INDX];

        // it may seem strange that I put the fill loop
        // inside the switch construct, but it is an order
        // of magnitude faster this way!!! PWP 08.09.99

        switch(n_dims){
          case 4: 
            for (i=11; i<43; i++){
              index += get_val(line, index, &data[i]);
              st_nt->Fill(data[SNO],  data[MID],
                          data[VAR4], data[VAR3], data[VAR2], data[VAR1],
                          data[MOD],  chan,       data[NEVT], data[i]); 
              chan += 4;
            }              
            break;
          case 3: 
            for (i=11; i<43; i++){
              index += get_val(line, index, &data[i]);
              st_nt->Fill(data[SNO],  data[MID],
                          data[VAR3], data[VAR2], data[VAR1],
                          data[MOD],  chan,       data[NEVT], data[i]); 
              chan += 4;
            }              
            break;
          case 2: 
            for (i=11; i<43; i++){
              index += get_val(line, index, &data[i]);
              st_nt->Fill(data[SNO],  data[MID],
                          data[VAR2], data[VAR1],
                          data[MOD],  chan,       data[NEVT], data[i]); 
              chan += 4;
            }              
            break;
          case 1: 
            for (i=11; i<43; i++){
              index += get_val(line, index, &data[i]);
              st_nt->Fill(data[SNO],  data[MID],
                          data[VAR1],
                          data[MOD],  chan,       data[NEVT], data[i]); 
              chan += 4;
            }              
            break;
        }
      }

      if (data[MOD]==-1){  /* sob record */

        // read rest of data row
        for (i=11; i<43; i++) index += get_val(line, index, &data[i]);

        //if var_types 1,2,3,4 not same as before,
        // we must create a new ntuple

        int new_nt = 0;
        for(i=0;i<4;i++){
          if((int)data[(SOB_TYP1-i)]!= var_type[i]){
            var_type[i] = (int)data[(SOB_TYP1-i)];
            new_nt = 1;
            match_var(var_type[i],stub[i]);
          }
        }

        if(new_nt){
          int n_dims = 0;
          for(i=0;i<4;i++) if (var_type[i]) n_dims++;
          sprintf(nt_name,"STNT%d",(int)data[SNO]);
          sprintf(nt_title,"Data for run %d start scan %d",run,(int)data[SNO]);
          switch(n_dims){
            case 1:
              sprintf(nt_members,"sno:mid:%s:calmode:ch:nevt:nhit",
                             stub[0]);
              break;
            case 2:
              sprintf(nt_members,"sno:mid:%s:%s:calmode:ch:nevt:nhit",
                             stub[1],stub[0]);
              break;
            case 3:
              sprintf(nt_members,"sno:mid:%s:%s:%s:calmode:ch:nevt:nhit",
                             stub[2],stub[1],stub[0]);
              break;
            case 4:
              sprintf(nt_members,"sno:mid:%s:%s:%s:%s:calmode:ch:nevt:nhit",
                             stub[3],stub[2],stub[1],stub[0]);
              break;
          }
          printf("Creating new ROOT ntuple %s:\n",nt_name);
          TNtuple *st_nt = new TNtuple(nt_name,nt_title,nt_members,32000);
        }

        if(old_sno!=(int)data[SNO]){
          old_sno=(int)data[SNO];
          printf("  sno= %d; %s = %3.2f; %s = %3.2f; %s = %3.2f; %s = %3.2f;\n",
            data[SNO], stub[3],data[VAR4], stub[2],data[VAR3], 
                       stub[1],data[VAR2], stub[0],data[VAR1]); 
        }
      }
      if (data[MOD]==-2){  /* chip configuration record */
        printf("chip record for module %d chip %d - ignored!!\n",
       (int)data[MID], (int)data[CHP]);
      }

      if (data[MOD]==-3){  /* trimdac record */
        printf("trimdac record for module %d chip %d - ignored!!\n",
       (int)data[MID], (int)data[CHP]);
      }

      if (data[MOD]==-4){  /* module DCS record */
        printf("DCS record for module %d - ignored!!\n", (int)data[MID]);
      }

      if (data[MOD]==-5){  /* module sysmap record */
        printf("sysmap record for module %d - ignored!!\n", (int)data[MID]);
      }

    }

    fclose(in_file);
    out_file->Write();

  return;
}
        
