//
// 08/08/2000 SJMP
//
// Macro to compare the results of two different threshold scan with each other
// after fitting. 
//
// TAKE CARE: log    takes fabs( case1 - case2 ) => NOT RECOMMENDED (fitting!)
//            linear does not ( just case1 - case 2 )!
//
// 09/08/2000 Bad channel hand to avoid spoiling the fits
//            by looking if the last character in the read in string is non-zero
//            Better ideas are welcome..
// 
// To do list:
// - HTML table of results

#define NCHAN      768
#define NCHIP      6

TF1     * poly_m [NCHIP];
TF1     * poly_s [NCHIP];
TF1     * const_m[NCHIP]; 
TF1     * const_s[NCHIP];

Double_t my_const(Double_t *x, Double_t *par)
{
  return par[0];
}

void stan_compare_fit(int run1,int scan1,int moi1,int loi1,
		      int run2,int scan2,int moi2,int loi2,
		      int log) {

  // variable definitions
  Char_t  f_name1[64],f_name2[64],outname[128],psname[64],htmlname[128];
  Char_t  column1[2][10],column2[2][10],column3[2][10],column4[2][10];
  Char_t  column5[2][10],column6[2][10],column7[2][10],column8[2][10],s[100];
  Int_t   chip, channel, n_good[NCHIP], bad_chan  [NCHIP][NCHAN/NCHIP],i,j;
  Int_t   chan1   [NCHIP][NCHAN/NCHIP], code1     [NCHIP][NCHAN/NCHIP];
  Int_t   chan2   [NCHIP][NCHAN/NCHIP], code2     [NCHIP][NCHAN/NCHIP];
  Float_t mean1   [NCHIP][NCHAN/NCHIP], e_mean1   [NCHIP][NCHAN/NCHIP]; 
  Float_t sigma1  [NCHIP][NCHAN/NCHIP], e_sigma1  [NCHIP][NCHAN/NCHIP];
  Float_t mean2   [NCHIP][NCHAN/NCHIP], e_mean2   [NCHIP][NCHAN/NCHIP]; 
  Float_t sigma2  [NCHIP][NCHAN/NCHIP], e_sigma2  [NCHIP][NCHAN/NCHIP];
  Float_t chi_sq1 [NCHIP][NCHAN/NCHIP], chi_sq2   [NCHIP][NCHAN/NCHIP];
  Float_t dmean   [NCHIP][NCHAN/NCHIP], e_dmean   [NCHIP][NCHAN/NCHIP];
  Float_t dsigma  [NCHIP][NCHAN/NCHIP], e_dsigma  [NCHIP][NCHAN/NCHIP];
  Float_t x_values[NCHIP][NCHAN/NCHIP], e_x_values[NCHIP][NCHAN/NCHIP];
  Float_t fit_x   [NCHIP][NCHAN/NCHIP], e_fit_x   [NCHIP][NCHAN/NCHIP];
  Float_t min_mean1 , max_mean1 , min_mean2 , max_mean2 ;
  Float_t min_sigma1, max_sigma1, min_sigma2, max_sigma2;
  Float_t min_dmean , max_dmean , min_dsigma ,max_dsigma;
  Float_t tm1, tm2;

  // (pre)set some arrays
  for ( i = 0 ; i < NCHAN ; ++i ) {
    chip    = (i-i%(NCHAN/NCHIP))/(NCHAN/NCHIP);
    channel = i%(NCHAN/NCHIP);
    bad_chan[chip][channel] = 0;
  }
  for ( i = 0 ; i < NCHIP ; ++i ) {
    n_good[chip] = 0;
  }

  // load general utilities
  gROOT->LoadMacro("stan_util.cpp");
  stan_styles();
  gStyle->SetOptStat(0);

  // open files
  if(RAL) {
    sprintf(f_name1,"M:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run1,scan1,moi1,loi1);
    sprintf(f_name2,"M:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run2,scan2,moi2,loi2);
    sprintf(psname,"compare_fit_%d_%d_%d_%d_vs_%d_%d_%d_%d.ps",
	    run1,scan1,moi1,loi1,run2,scan2,moi2,loi2);
    sprintf(outname,"M:\\sctvar\\data\\compare_fit_%d_%d_%d_%d_vs_%d_%d_%d_%d.ps",
	    run1,scan1,moi1,loi1,run2,scan2,moi2,loi2);
    sprintf(htmlname,"M:\\sctvar\\data\\compare_fit_%d_%d_%d_%d_vs_%d_%d_%d_%d.html",
	    run1,scan1,moi1,loi1,run2,scan2,moi2,loi2);
  }
  else {
    sprintf(f_name1,"D:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run1,scan1,moi1,loi1);
    sprintf(f_name2,"D:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run2,scan2,moi2,loi2);
    sprintf(psname,"compare_fit_%d_%d_%d_%d_vs_%d_%d_%d_%d.ps",
	    run1,scan1,moi1,loi1,run2,scan2,moi2,loi2);
    sprintf(outname,"D:\\sctvar\\data\\compare_fit_%d_%d_%d_%d_vs_%d_%d_%d_%d.ps",
	    run1,scan1,moi1,loi1,run2,scan2,moi2,loi2);
    sprintf(htmlname,"D:\\sctvar\\data\\compare_fit_%d_%d_%d_%d_vs_%d_%d_%d_%d.html",
	    run1,scan1,moi1,loi1,run2,scan2,moi2,loi2);
  }
  FILE * f1 = fopen(f_name1,"r");
  FILE * f2 = fopen(f_name2,"r");

  // read files
  if ( f1 && f2 ) {

    printf("Reading in data from files:\n%s and\n%s\n\n",f_name1,f_name2);
    // ignore first line of fit files : column names
    fscanf(f1,"%s %s %s %s %s %s %s\n",
	   &column1[0],&column2[0],&column3[0],&column4[0],&column5[0],&column6[0],&column7[0]);
    fscanf(f2,"%s %s %s %s %s %s %s\n",
	   &column1[1],&column2[1],&column3[1],&column4[1],&column5[1],&column6[1],&column7[1]);

    // read data
    for ( i = 0 ; i < NCHAN ; ++i ) {
      fscanf(f1,"%s %s %s %s %s %s %s\n",
	     &column1[0],&column2[0],&column3[0],&column4[0],&column5[0],&column6[0],&column7[0]);
      fscanf(f2,"%s %s %s %s %s %s %s\n",
	     &column1[1],&column2[1],&column3[1],&column4[1],&column5[1],&column6[1],&column7[1]);

      // calculate chip/channel nr's
      chip    = (i-i%(NCHAN/NCHIP))/(NCHAN/NCHIP);
      channel = i%(NCHAN/NCHIP);
      if ( channel == 0 ) printf(" chip : %d, channel : %d\n",chip,channel);
      // identify bad channel
      tm1=atof(column3[0]);
      tm2=atof(column3[1]);
      if ( column2[0][5] != '0' || column2[1][5] != '0' 
	   column2[0][4] != '0' || column2[1][4] != '0' 
	   column2[0][3] != '0' || column2[1][3] != '0' 
	   column2[0][2] != '0' || column2[1][2] != '0') {
	bad_chan[chip][channel] = 1;
      }
      else {
	x_values  [chip][n_good[chip]] = i;
	e_x_values[chip][n_good[chip]] = 0;
	chan1     [chip][n_good[chip]] = atoi(column1[0]);
	code1     [chip][n_good[chip]] = atoi(column2[0]); // doesn't work
	mean1     [chip][n_good[chip]] = atof(column3[0]);
	e_mean1   [chip][n_good[chip]] = atof(column4[0]);
	sigma1    [chip][n_good[chip]] = atof(column5[0]);
	e_sigma1  [chip][n_good[chip]] = atof(column6[0]);
	chi_sq1   [chip][n_good[chip]] = atof(column7[0]);
	chan2     [chip][n_good[chip]] = atoi(column1[1]);
	code2     [chip][n_good[chip]] = atoi(column2[1]); // doesn't work
	mean2     [chip][n_good[chip]] = atof(column3[1]);
	e_mean2   [chip][n_good[chip]] = atof(column4[1]);
	sigma2    [chip][n_good[chip]] = atof(column5[1]);
	e_sigma2  [chip][n_good[chip]] = atof(column6[1]);
	chi_sq2   [chip][n_good[chip]] = atof(column7[1]);
	++n_good  [chip];
      }
    }
    fclose(f1);
    fclose(f2);
  }
  else {
    if ( !f1) printf("Failed to open %s\n",f_name1);
    if ( !f2) printf("Failed to open %s\n",f_name2);
    printf("Exiting..\n");
    return;
  }

  // calculate difference and error for mean and sigma
  for ( i = 0 ; i < NCHIP ; ++i ) {
    for ( j = 0 ; j < n_good[i] ; ++j ) {

      if ( log ) {
	dmean [i][j] = fabs( mean1 [i][j] - mean2 [i][j] );
  	dsigma[i][j] = fabs( sigma1[i][j] - sigma2[i][j] );
      }
      else {
  	dmean [i][j] = mean1 [i][j] - mean2 [i][j];
	dsigma[i][j] = sigma1[i][j] - sigma2[i][j];
      }
      e_dmean [i][j] = e_mean1 [i][j] + e_mean2 [i][j];
      e_dsigma[i][j] = e_sigma1[i][j] + e_sigma2[i][j];

      // calculate min/max values for graph
      if ( i == 0 &&  j == 0 ) {
  	min_mean1  = mean1 [i][j] - e_mean1 [i][j];
  	max_mean1  = mean1 [i][j] + e_mean1 [i][j];
	min_mean2  = mean2 [i][j] - e_mean2 [i][j];
	max_mean2  = mean2 [i][j] + e_mean2 [i][j];
	min_sigma1 = sigma1[i][j] - e_sigma1[i][j];
	max_sigma1 = sigma1[i][j] + e_sigma1[i][j];
	min_sigma2 = sigma2[i][j] - e_sigma2[i][j];
	max_sigma2 = sigma2[i][j] + e_sigma2[i][j];
	min_dmean  = dmean [i][j] - e_dmean [i][j];
	max_dmean  = dmean [i][j] + e_dmean [i][j];
	min_dsigma = dsigma[i][j] - e_dsigma[i][j];
	max_dsigma = dsigma[i][j] + e_dsigma[i][j];
      }
      else {
	if ( min_mean1  > mean1 [i][j] - e_mean1 [i][j] ) min_mean1  = mean1 [i][j] - e_mean1 [i][j];
	if ( max_mean1  < mean1 [i][j] + e_mean1 [i][j] ) max_mean1  = mean1 [i][j] + e_mean1 [i][j];
	if ( min_mean2  > mean2 [i][j] - e_mean2 [i][j] ) min_mean2  = mean2 [i][j] - e_mean2 [i][j];
	if ( max_mean2  < mean2 [i][j] + e_mean2 [i][j] ) max_mean2  = mean2 [i][j] + e_mean2 [i][j];
	if ( min_sigma1 > sigma1[i][j] - e_sigma1[i][j] ) min_sigma1 = sigma1[i][j] - e_sigma1[i][j];
	if ( max_sigma1 < sigma1[i][j] + e_sigma1[i][j] ) max_sigma1 = sigma1[i][j] + e_sigma1[i][j];
	if ( min_sigma2 > sigma2[i][j] - e_sigma2[i][j] ) min_sigma2 = sigma2[i][j] - e_sigma2[i][j];
	if ( max_sigma2 < sigma2[i][j] + e_sigma2[i][j] ) max_sigma2 = sigma2[i][j] + e_sigma2[i][j];
	if ( min_dmean  > dmean [i][j] - e_dmean [i][j] ) min_dmean  = dmean [i][j] - e_dmean [i][j];
	if ( max_dmean  < dmean [i][j] + e_dmean [i][j] ) max_dmean  = dmean [i][j] + e_dmean [i][j];
	if ( min_dsigma > dsigma[i][j] - e_dsigma[i][j] ) min_dsigma = dsigma[i][j] - e_dsigma[i][j];
	if ( max_dsigma < dsigma[i][j] + e_dsigma[i][j] ) max_dsigma = dsigma[i][j] + e_dsigma[i][j];
      }      
    }
  }
 
  // correct for log plot
  if ( log ) {
    min_dsigma = 0.01;
    min_dmean  = 0.01;
  }
  // and now we have DATA! (-;
  TGraphErrors * gr_plot_m1[NCHIP];
  TGraphErrors * gr_plot_m2[NCHIP];
  TGraphErrors * gr_plot_s1[NCHIP];
  TGraphErrors * gr_plot_s2[NCHIP];
  TGraphErrors * gr_plot_dm[NCHIP];
  TGraphErrors * gr_plot_ds[NCHIP];
  for ( i = 0 ; i < NCHIP ; ++ i ) { 
    gr_plot_m1[i] = new TGraphErrors(n_good[i],x_values[i],mean1 [i],e_x_values[i],e_mean1 [i]);
    gr_plot_m2[i] = new TGraphErrors(n_good[i],x_values[i],mean2 [i],e_x_values[i],e_mean2 [i]);
    gr_plot_s1[i] = new TGraphErrors(n_good[i],x_values[i],sigma1[i],e_x_values[i],e_sigma1[i]);
    gr_plot_s2[i] = new TGraphErrors(n_good[i],x_values[i],sigma2[i],e_x_values[i],e_sigma2[i]);
    gr_plot_dm[i] = new TGraphErrors(n_good[i],x_values[i],dmean [i],e_x_values[i],e_dmean [i]);
    gr_plot_ds[i] = new TGraphErrors(n_good[i],x_values[i],dsigma[i],e_x_values[i],e_dsigma[i]);
  }
  
  // and now we have PLOTS! (-;
  // make canvas
  TCanvas * c1      = new TCanvas ("c1"   ,"stan_compare_fit",1100,750);
  TPad    * pad_t   = new TPad    ("pad_t","text"            ,0.01,0.91,0.99,0.99,10); 
  TPad    * pad_c_1 = new TPad    ("pad_c","plots"           ,0.01,0.61,0.50,0.90,10);
  TPad    * pad_c_2 = new TPad    ("pad_c","plots"           ,0.51,0.61,0.99,0.90,10);
  TPad    * pad_c_3 = new TPad    ("pad_c","plots"           ,0.01,0.31,0.50,0.60,10);
  TPad    * pad_c_4 = new TPad    ("pad_c","plots"           ,0.51,0.31,0.99,0.60,10);
  TPad    * pad_c_5 = new TPad    ("pad_c","plots"           ,0.01,0.01,0.50,0.30,10);
  TPad    * pad_c_6 = new TPad    ("pad_c","plots"           ,0.51,0.01,0.99,0.30,10);
  pad_t->Draw();
  pad_c_1->Draw();
  pad_c_2->Draw();
  pad_c_3->Draw();
  pad_c_4->Draw();
  pad_c_5->Draw();
  pad_c_6->Draw();
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.15);
  t1->SetTextColor(4);
  if (log) {
    t1->AddText("ATLAS FWD SCT Scan Comparison - log scale");
  }
  else {
    t1->AddText("ATLAS FWD SCT Scan Comparison - linear scale");
  }
  sprintf(s,"Run %d   Scan %d   Module %d   Stream %d           vs.           "
	  "Run %d   Scan %d   Module %d   Stream %d",
	  run1, scan1, moi1, loi1, run2, scan2, moi2, loi2);
  t1->AddText(s);
  t1->AddText("Name:                                                 Date:");
  t1->AddText("Description:");
  pad_t->cd();
  t1->Draw();

  // plot data
  TH2F * th[6];
  pad_c_1->cd();
  pad_c_1->SetTicks(2,2);
  pad_c_1->SetGrid(1,1);
  sprintf(s,"Response 1 : Run %d   Scan %d   Module %d   Stream %d",
	  run1, scan1, moi1, loi1);
  th[0] = new TH2F("frame1",s,2,0,768,2,min_mean1,max_mean1);
  th[0]->Draw();
  for ( i = 0 ; i < NCHIP ; ++i ) {
    gr_plot_m1[i]->SetLineColor(i);
    gr_plot_m1[i]->SetMarkerColor(((i+2)%2)*3+14);
    gr_plot_m1[i]->Draw("P");
  }
  th[0]->SetXTitle("channel nr.");
  th[0]->SetYTitle("response (mV)");  

  pad_c_2->cd();
  pad_c_2->SetTicks(2,2);
  pad_c_2->SetGrid(1,1);
  sprintf(s,"Noise 1 : Run %d   Scan %d   Module %d   Stream %d",
	  run1, scan1, moi1, loi1);
  th[1] = new TH2F("frame2",s,2,0,768,2,min_sigma1,max_sigma1);
  th[1]->Draw();
  for ( i = 0 ; i < NCHIP ; ++i )  {
    gr_plot_s1[i]->SetLineColor(((i+2)%2)*3+14);
    gr_plot_s1[i]->SetMarkerColor(((i+2)%2)*3+14);
    gr_plot_s1[i]->Draw("P");
  }
  th[1]->SetXTitle("channel nr.");
  th[1]->SetYTitle("noise (mV)");

  pad_c_3->cd();
  pad_c_3->SetTicks(2,2);
  pad_c_3->SetGrid(1,1);
  sprintf(s,"Response 2 : Run %d   Scan %d   Module %d   Stream %d",
	  run2, scan2, moi2, loi2);
  th[2] = new TH2F("frame3",s,2,0,768,2,min_mean2,max_mean2);
  th[2]->Draw();
  for ( i = 0 ; i < NCHIP ; ++i )  {
    gr_plot_m2[i]->SetLineColor(((i+2)%2)*3+14);
    gr_plot_m2[i]->SetMarkerColor(((i+2)%2)*3+14);
    gr_plot_m2[i]->Draw("P");
  }
  th[2]->SetXTitle("channel nr.");
  th[2]->SetYTitle("response (mV)");

  pad_c_4->cd();
  pad_c_4->SetTicks(2,2);
  pad_c_4->SetGrid(1,1);
  sprintf(s,"Noise 2 : Run %d   Scan %d   Module %d   Stream %d",
	  run2, scan2, moi2, loi2);
  th[3] = new TH2F("frame4",s,2,0,768,2,min_sigma2,max_sigma2);
  th[3]->Draw();
  for ( i = 0 ; i < NCHIP ; ++i )  {
    gr_plot_s2[i]->SetLineColor(((i+2)%2)*3+14);
    gr_plot_s2[i]->SetMarkerColor(((i+2)%2)*3+14);
    gr_plot_s2[i]->Draw("P");
  }
  th[3]->SetXTitle("channel nr.");
  th[3]->SetYTitle("noise (mV)");

  pad_c_5->cd();
  pad_c_5->SetTicks(2,2);
  pad_c_5->SetGrid(1,1);
  if ( log )  pad_c_5->SetLogy();
  sprintf(s,"Difference in response");
  th[4] = new TH2F("frame5",s,2,0,768,2,min_dmean,max_dmean);
  th[4]->Draw();
  for ( i = 0 ; i < NCHIP ; ++i )  {
    gr_plot_dm[i]->SetLineColor(((i+2)%2)*3+14);
    gr_plot_dm[i]->SetMarkerColor(((i+2)%2)*3+14);
    gr_plot_dm[i]->Draw("P");
  }
  th[4]->SetXTitle("channel nr.");
  th[4]->SetYTitle("response1 - response2 (mV)");

  pad_c_6->cd();
  pad_c_6->SetTicks(2,2);
  pad_c_6->SetGrid(1,1);
  if ( log )  pad_c_6->SetLogy();
  th[5] = new TH2F("frame6",s,2,0,768,2,min_dsigma,max_dsigma);
  th[5]->Draw();
  for ( i = 0 ; i < NCHIP ; ++i )  {
    gr_plot_ds[i]->SetLineColor(((i+2)%2)*3+14);
    gr_plot_ds[i]->SetMarkerColor(((i+2)%2)*3+14);
    gr_plot_ds[i]->Draw("P");
  }
  th[5]->SetXTitle("channel nr.");
  th[5]->SetYTitle("noise1 - noise2 (mV)");


  printf("\nStupid ROOT or stupid me? Anyhow, please ignore the following 10 lines of errors\n\n");
  // fit chips of differences with a 2D polynomial
  for ( i = 0 ; i < NCHIP ; ++i ) {
    sprintf(s,"poly_m_%d",i);
    poly_m[i] = new TF1(s,pol2,i*NCHAN/NCHIP,(i+1)*NCHAN/NCHIP-1,3);
    poly_m[i]->SetParNames("a0","a1","a2"); 
    gr_plot_dm[i]->Fit(s,"QR");
  }
  for ( i = 0 ; i < NCHIP ; ++i ) {
    sprintf(s,"poly_s_%d",i);
    poly_s[i] = new TF1(s,pol2,i*NCHAN/NCHIP,(i+1)*NCHAN/NCHIP-1,3);
    poly_s[i]->SetParNames("a0","a1","a2"); 
    gr_plot_ds[i]->Fit(s,"QR");
  }
  // root feature? first five tries don't work, second time it works : do means again
  for ( i = 0 ; i < NCHIP ; ++i ) {
    sprintf(s,"poly_m_%d",i);
    poly_m[i] = new TF1(s,pol2,i*NCHAN/NCHIP,(i+1)*NCHAN/NCHIP-1,3);
    poly_m[i]->SetParNames("a0","a1","a2"); 
    gr_plot_dm[i]->Fit(s,"QR");
  }
  // fit chips with constant
  for ( i = 0 ; i < NCHIP ; ++i ) {
    sprintf(s,"const_s_%d",i);
    const_s[i] = new TF1(s,my_const,i*NCHAN/NCHIP,(i+1)*NCHAN/NCHIP-1,1);
    const_s[i]->SetParNames("a0"); 
    gr_plot_ds[i]->Fit(s,"QR");
  }
  for ( i = 0 ; i < NCHIP ; ++i ) {
    sprintf(s,"const_m_%d",i);
    const_m[i] = new TF1(s,my_const,i*NCHAN/NCHIP,(i+1)*NCHAN/NCHIP-1,1);
    const_m[i]->SetParNames("a0"); 
    gr_plot_dm[i]->Fit(s,"QR");
  }
  // root feature? first five tries don't work, second time it works : do means again
  for ( i = 0 ; i < NCHIP ; ++i ) {
    sprintf(s,"const_s_%d",i);
    const_s[i] = new TF1(s,my_const,i*NCHAN/NCHIP,(i+1)*NCHAN/NCHIP-1,1);
    const_s[i]->SetParNames("a0"); 
    gr_plot_ds[i]->Fit(s,"QR");
  }

  // write/print summary
  FILE * f3 = fopen(htmlname,"w");
  if ( f3 ) {
    fprintf(f3,"<html><head><title>\n");
    fprintf(f3,"RUN %d SCAN %d MODULE %d LINK %d vs. RUN %d SCAN %d MODULE %d LINK %d\n",
	    run1,scan1,moi1,loi1,run2,scan2,moi2,loi2);
    fprintf(f3,"</title></head><body  bgcolor=\"#FFFFFF\" text=\"#000000\">\n");
    fprintf(f3,"<center><h3>Comparison per chip of the fitted results of:<br>\n");
    fprintf(f3,"RUN %d SCAN %d MODULE %d LINK %d vs. RUN %d SCAN %d MODULE %d LINK %d</h3>\n",
	    run1,scan1,moi1,loi1,run2,scan2,moi2,loi2);
    fprintf(f3,"<br><a href=\"%s\">postscript plot of results</a>",psname);
    for ( i = 0 ; i < NCHIP ; ++i ) {
      fprintf(f3,"<br><br>\n");
      fprintf(f3,"<h4>Chip %d</h4>\n",i);
      fprintf(f3,"%d bad channels (defined by fit):",NCHAN/NCHIP-n_good[i]);
      for ( j = 0 ; j < NCHAN/NCHIP ; ++j ) {
	if ( bad_chan[i][j] == 1 ) fprintf(f3," %d",j);
      }
      fprintf(f3,"<br><br>\n");
      fprintf(f3,"<table border=\"2\" cellpadding=\"2\"><tr><td></td><td>Mean</td><td>Sigma</td></tr>");
      fprintf(f3,"<tr><td>constant</td><td>%9.1e +/- %9.1e</td><td>%9.1e +/- %9.1e</td></tr>\n",
	      const_m[i]->GetParameter(0),const_m[i]->GetParError(0),
	      const_s[i]->GetParameter(0),const_s[i]->GetParError(0));
      fprintf(f3,"<tr><td>ChiSq/NDF</td><td>%5.2f</td><td>%5.2f</td></tr>\n",
	      const_m[i]->GetChisquare()/const_m[i]->GetNDF(),
	      const_s[i]->GetChisquare()/const_s[i]->GetNDF());      
      fprintf(f3,"<tr><td>poly: a0</td><td>%9.1e +/- %9.1e</td><td>%9.1e +/- %9.1e</td></tr>\n",
	      poly_m[i]->GetParameter(0),poly_m[i]->GetParError(0),
	      poly_s[i]->GetParameter(0),poly_s[i]->GetParError(0));
      fprintf(f3,"<tr><td>poly: a1</td><td>%9.1e +/- %9.1e</td><td>%9.1e +/- %9.1e</td></tr>\n",
	      poly_m[i]->GetParameter(1),poly_m[i]->GetParError(1),
	      poly_s[i]->GetParameter(1),poly_s[i]->GetParError(1));
      fprintf(f3,"<tr><td>poly: a2</td><td>%9.1e +/- %9.1e</td><td>%9.1e +/- %9.1e</td></tr>\n",
	      poly_m[i]->GetParameter(2),poly_m[i]->GetParError(2),
	      poly_s[i]->GetParameter(2),poly_s[i]->GetParError(2));
      fprintf(f3,"<tr><td>ChiSq/NDF</td><td>%5.2f</td><td>%5.2f</td></tr>\n",
	      poly_m[i]->GetChisquare()/poly_m[i]->GetNDF(),
	      poly_s[i]->GetChisquare()/poly_s[i]->GetNDF());      
      fprintf(f3,"</table>\n");
    }
    fprintf(f3,"</center></body></html>\n");
    fclose(f3);
    printf("Web summary file: %s has been created\n",htmlname);
  } 
  else {
    printf("Could not open web summary file: %s\n",htmlname);
  }
  
  // save canvas
  for ( i  = 0 ; i < NCHIP ; ++i ) {
    pad_c_5->cd(); 
    poly_m[i]->SetLineColor(10);
    poly_m[i]->SetLineWidth(10);
    poly_m [i]->Draw("same");
    const_m[i]->SetLineColor(1);
    const_m[i]->SetLineWidth(8);
    const_m [i]->Draw("same");
    pad_c_6->cd(); 
    poly_s[i]->SetLineColor(10);
    poly_s[i]->SetLineWidth(10);
    poly_s[i]->Draw("same");
    const_s[i]->SetLineColor(1);
    const_s[i]->SetLineWidth(8);
    const_s[i]->Draw("same");
  }
  c1->Update();
  c1->SaveAs(outname);

  // may be this will mess up some other ST software, but it 
  // avoids some not understandable 'array index out of range' errors..
  gROOT->Reset();
}

void stan_compare_fit(int run1,int scan1,int moi1,int loi1,
		      int run2,int scan2,int moi2,int loi2){
  // assume linear scale if called without additional arguments

  stan_compare_fit(run1, scan1, moi1, loi1, run2, scan2, moi2, loi2, 0);

}

void stan_compare_fit(){

  printf("Possible Uses of stan_compare_fit:\n\n");
  printf(".x stan_compare_fit.cpp(run1,scan1,moi1,loi1,run2,scan2,moi2,loi2)\n");
  printf(".x stan_compare_fit.cpp(run1,scan1,moi1,loi1,run2,scan2,moi2,loi2,log)\n");

}

