void stan_fitscan(int run, int sno, int moi, int loi, int ascii, int postscript){

  // Routine to fit individual st scan histograms
  //
  // Cuts have been implemented as follows:
  // o Channels with <1% occupancy in any bin are marked as "DEAD"
  // o Channels with >98% occupancy in every bin are marked "STUCK"
  // o Channels which do not reach 100% efficiency are marked "UNDER"
  // o Channels which reach > 100% efficiency are marked "OVER"
  //   (Such a condition would imply decoder errors, inviting the
  //    user to check the digital integrity of their system)
  // o include only fits for which the mean value (e.g. vt50)
  //   falls within the scanned range
  // o include only fits for which the fitted sigma
  //   is positive and less than twice the mean sigma

  // Version 0.32 P.W.Phillips, 06.01.00
  // two bugfixes and improvements suggsted by Martin Morrissey

  // Version 0.33 P.W.Phillips, 17.03.00
  // bugfix: case where ndf=0;

  // Version 0.34 BJG 30.04.02   Fix some memory leaks


  // The normalisation "factor of four" is now taken out in TST.cpp
  // hence it is no longer needed here.  WARNING!! This change is 
  // NOT BACKWARDS COMPATIBLE.  To analyse data taken with earlier
  // versions of ST change "base_factor" to be four

  float base_factor = 1.0;    // new data
  //float base_factor = 4.0;  // old data

  char  f_name[128], out_fname[128];            // up from 64, 14.12.99
  char  h_name[64], h_name2[64], scan_name[64]; // up from 64, 14.12.99
  char  h_title[64], m_title[64], s_title[64];  // up from 64, 14.12.99
  char  s[256], unit[32];                       // unit up from 12, 14.12.99
  int   bin,i,j,k,x,y;
  short scan_type;
  float data[3], err2, n, z;

  // Locality options now moved to stan_util.cpp

  // Options which change macro output
  int draw_fit_projections = 0;

  printf("stan_fitscan called for module %d link %d\n",moi,loi);

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");

  // Global style variations
  stan_styles();
  gStyle.SetOptStat(0);


  // Open file, or obtain pointer to histogram in memory
  
  if(run==-1){  

    if(loi) TH2F *h_scan = (TH2F*)e->m[moi]->h_scan1;
    else    TH2F *h_scan = (TH2F*)e->m[moi]->h_scan0;
    TH1F *h_nevent = (TH1F*)e->h_scan_evcnt;

    run = e->runnum;
    scan_type = e->scan_type;
    sprintf(scan_name,"%s",e->scan_name);
    sno = e->scan_count;

    if(sno==0){
      printf("  NO DATA in memory!!  exiting\n\n");
      return;
    }
    printf("  Using Data in memory\n");

    TDatime *dt = new TDatime;
    TObjString *time = new TObjString(dt->AsString());
    int place =0; 
    int mname =0;
   
  } else {

    if(RAL) sprintf(f_name,"M:\\sctvar\\data\\strun%d_%d.root",run,sno);
    else    sprintf(f_name,"D:\\sctvar\\data\\strun%d_%d.root",run,sno);

    if(loi) sprintf(h_name,"h_scan1;%d",moi+1);
    else    sprintf(h_name,"h_scan0;%d",moi+1);
    printf("  Using histogram %s from ROOT Datafile %s\n",h_name,f_name);

    TFile *strun = new TFile(f_name);
    TH2F *h_scan= (TH2F*)strun->Get(h_name);
    TH1F *h_nevent= (TH1F*)strun->Get("h_scan_evcnt");

    sprintf(h_name,"ModuleName;%d",moi+1);
    TObjString *time  = (TObjString*)strun->Get("Time");
    TObjString *place = (TObjString*)strun->Get("Place");
    TObjString *mname = (TObjString*)strun->Get(h_name);

    sprintf(scan_name,"Unknown");
    sprintf(unit,"");
    TString title = h_scan->GetTitle();
    title.ToUpper();
    if (title.Contains("THRESHOLD")){
      scan_type = 1;
      sprintf(scan_name,"THRESHOLD (mV)");
      sprintf(unit,"mV");
    }
    if (title.Contains("VCAL")){
      scan_type = 2;
      sprintf(scan_name,"VCAL (mV)");
      sprintf(unit,"mV");
    }
    if (title.Contains("STROBE")){
      scan_type = 3;
      sprintf(scan_name,"Strobe Delay");
      sprintf(unit,"");
    }
    if (title.Contains("PREAMP")){
      scan_type = 4;
      sprintf(scan_name,"Preamp Current (uA)");
      sprintf(unit,"uA");
    }
    if (title.Contains("SHAPER")){
      scan_type = 5;
      sprintf(scan_name,"Shaper Current (uA)");
      sprintf(unit,"uA");
    }
    if (title.Contains("QTHR")){
      scan_type = 42;
      sprintf(scan_name,"QTHR (fC)");
      sprintf(unit,"fC");
    }
    if (title.Contains("QCAL")){
      scan_type = 42;
      sprintf(scan_name,"QCAL (fC)");
      sprintf(unit,"fC");
    }
  }

// titles for graphs - might as well do this now.

  sprintf(m_title,"Fitted Mean(%s)",scan_name);
  sprintf(s_title,"Fitted Sigma(%s)",scan_name);

  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  
  sprintf(s,"ATLAS SCT Module Test");
  if(time)  sprintf(s,"%s  -  %s",s,time->GetName());
  if(place) sprintf(s,"%s  -  %s",s,place->GetName());
  if(mname) sprintf(s,"%s  -  Module %s",s,mname->GetName());
  t1->AddText(s);
  sprintf(s,"Run %d   Scan %d   Module %d   Stream %d   Scan type: %s", run, sno, moi, loi, scan_name);
  t1->AddText(s);

// Choose appropriate fit function

  switch (scan_type){
    case 1: // ST_VTHR
      printf("ST_VTHR\n");
      TF1 *f1 = new TF1 ("f1",erfc,  0, 1000, 3);
      f1->SetParNames("Constant","Mean","Sigma");
      break;
    case 2: // ST_VCAL
      printf("ST_VCAL\n");
      TF1 *f1 = new TF1 ("f1", erf,  0, 1000, 3);
      f1->SetParNames("Constant","Mean","Sigma");
      break;
    case 3: // ST_STROBE_DELAY
      printf("ST_STROBE_DELAY\n");
      // fit the back edge - > front edge of charge injection pulse
      TF1 *f1 = new TF1 ("f1", erfc,  0, 1000, 3);
      f1->SetParNames("Constant","Mean","Sigma");
      break;
    case 41: // ST_QTHR
      printf("ST_QTHR\n");
      TF1 *f1 = new TF1 ("f1",erfc,  0, 1000, 3);
      f1->SetParNames("Constant","Mean","Sigma");
      break;
    case 42: // ST_QCAL
      printf("ST_QCAL\n");
      TF1 *f1 = new TF1 ("f1", erf,  0, 1000, 3);
      f1->SetParNames("Constant","Mean","Sigma");
      break;
    default: // use gaussian for now 
      TF1 *f1 = new TF1 ("f1", gaus,  0, 1000, 3);
      f1->SetParNames("Constant","Mean","Sigma");
      break;
  }

// Calculate errors appropriate to binomial statistics 

  for(y=1;y<h_scan->GetNbinsY()+1;y++){
    n = h_nevent->GetBinContent(y);
    if (n ==0) n =1;
    for(x=0;x<h_scan->GetNbinsX();x++){
      z = h_scan->GetCellContent(x,y);
      err2 = (z+1) * (1+n-z) / n;
      h_scan->SetCellError(x,y,sqrt(err2));
    }
  }

// Before we draw the main canvas, open the postscript file
// if the relevant option has been selected 

  if(postscript){
    if(RAL) sprintf(out_fname,"M:\\sctvar\\data\\strun%d_%d_%d_%d.ps",run,sno,moi,loi);
    else    sprintf(out_fname,"D:\\sctvar\\data\\strun%d_%d_%d_%d.ps",run,sno,moi,loi);

    TPostScript *ps = new TPostScript(out_fname,112);
    ps->NewPage();
  }

// Open and draw the first canvas

  TCanvas* c1=new TCanvas ("c1","stan_fitscan",1100,750);
  pad_t = new TPad("pad_t","Text",         0.01,0.91,0.99,0.99,33);
  pad_m = new TPad("pad_m","Fitted Means", 0.01,0.61,0.60,0.90,33);
  pad_s = new TPad("pad_s","Fitted Sigmas",0.01,0.31,0.60,0.60,33);
  pad_c = new TPad("pad_c","Fitted Chisq", 0.01,0.01,0.60,0.30,33);
  pad_p = new TPad("pad_p","Projections",  0.61,0.01,0.99,0.90,33);

  pad_t->Draw();
  pad_m->Draw();
  pad_s->Draw();
  pad_c->Draw();
  pad_p->Draw();
  pad_p->Divide(2,3);

  pad_m->cd();
  h_scan->Draw("COLZ");
  c1->Update();

// projections to give the initial values for use with FitSlicesY


  TH1* h_dp[6];
  TH1* h_dpr[6];
  TH1* h_ptr;
  Float_t chip_p0[6];
  Float_t chip_p1[6];
  Float_t chip_p2[6];
  Float_t scale_factor = base_factor/(float)(128*h_nevent->GetMaximum());

// Fit the projections - unless the histograms are empty!!!

  for(i=0; i<6; i++){
    pad_p->cd(i+1);
    sprintf(h_name,"h_dp%d",i);
    sprintf(h_name2,"h_dpr%d",i);
    sprintf(h_title,"Efficiency, chip %d",i);

    // delete projection if it already exists
    if(h_ptr = (TH1F*) gDirectory->GetList()->FindObject(h_name)) delete h_ptr;

    h_scan->ProjectionY(h_name,(i*128),((i+1)*128)-1);
    h_dp[i] = (TH1F*) gDirectory->GetList()->FindObject(h_name);

    if(i==0){
      int n_bins = h_dp[0]->GetNbinsX();
      float scan_lo = h_dp[0]->GetBinCenter(1);
      float scan_hi = h_dp[0]->GetBinCenter(n_bins);
      float half_range = (scan_hi - scan_lo)/4;
    }

    // delete range histogram if it already exists
    if(h_ptr = (TH1F*) gDirectory->GetList()->FindObject(h_name2)) delete h_ptr;
    TH2F *h_range = new TH2F(h_name2,h_title,2,scan_lo,scan_hi,2,0,1);
    h_dpr[i] = (TH1F*) gDirectory->GetList()->FindObject(h_name2);

    h_dp[i]->Scale(scale_factor);
    h_dp[i]->SetTitle(h_title);
    h_dp[i]->SetXTitle(scan_name);
    h_dpr[i]->SetTitle(h_title);
    h_dpr[i]->SetXTitle(scan_name);
    h_dp[i]->SetLineColor(kRed);
    h_dp[i]->SetMarkerColor(kBlack);


    for(x=1;x<=h_dp[i]->GetNbinsX();x++){
      n = h_nevent->GetBinContent(x);
      if (n ==0) n =1;
      z = h_dp[i]->GetBinContent(x);
      err2 = sqrt((1+(n*z))*(1+(n-(n*z))) / n) / n;
      h_dp[i]->SetBinError(x,err2);
    }

    // Fit the projections - unless the histograms are empty!!!

    if(h_dp[i]->GetMaximum()>0.0){

      Float_t p0_guess = h_dp[i]->GetMaximum();

      if( (scan_type==1)||(scan_type==41)||(scan_type==3)){
        // VTHR, QTHR or STROBE DELAY
        Float_t p1_guess = find_level2(h_dp[i],0.50,1);          // last bin > 0%
        Float_t p2_guess = find_level2(h_dp[i],0.82,1)-p1_guess;  
        Float_t range_lo = find_bin2(h_dp[i],0.99,0,0);            // first bin > 99%
        Float_t range_hi = h_dp[i]->GetBinCenter(h_dp[i]->GetNbinsX());
        if(p2_guess<0) p2_guess = p2_guess*-1.0;
      } else if( (scan_type==2)||(scan_type==42)){
        // VCAL or QCAL 
        Float_t p1_guess = find_level2(h_dp[i],0.50,0);          // first bin < 50%
        Float_t p2_guess = find_level2(h_dp[i],0.82,0)-p1_guess;
        Float_t range_lo = h_dp[i]->GetBinCenter(1);
        Float_t range_hi = find_bin2(h_dp[i],0.99,1,0);            // last bin > 99%
        if(p2_guess<0) p2_guess = p2_guess*-1.0;
      } else {
        Float_t p1_guess = h_dp[i]->GetMean(); 
        Float_t p2_guess = h_dp[i]->GetRMS(); 
        Float_t range_lo = h_dp[i]->GetBinCenter(1);
        Float_t range_hi = h_dp[i]->GetBinCenter(h_dp[i]->GetNbinsX());
      }

      f1->SetParameters(p0_guess,p1_guess,p2_guess);
      f1->SetParLimits(0,p0_guess,p0_guess);
      f1->SetRange(range_lo, range_hi);
      h_dp[i]->Fit("f1","RQ"); // quiet mode

      chip_p0[i] = f1->GetParameter(0); 
      chip_p1[i] = f1->GetParameter(1); 
      chip_p2[i] = f1->GetParameter(2);
    } else {
      chip_p0[i] = -1;
      chip_p1[i] = -1;
      chip_p2[i] = -1;
    }
    printf("Chip %d guesses p0=%3.2f p1=%3.2f p2=%3.2f rlo=%3.2f rhi=%3.2f\n",i,p0_guess,p1_guess,p2_guess, range_lo, range_hi); 
    printf("Chip %d fit     p0=%3.2f p1=%3.2f p2=%3.2f\n",i,chip_p0[i],chip_p1[i],chip_p2[i]); 
    h_dpr[i]->Draw();
    h_dp[i]->Draw("same");

    TText * t = new TText();
    sprintf(h_title,"Mean = %03.2f %s",chip_p1[i],unit);
    t->DrawText(scan_lo,0.3,h_title);
    sprintf(h_title,"Sigma = %03.2f %s",chip_p2[i],unit);
    t->DrawText(scan_lo,0.2,h_title);
    sprintf(h_title,"Max Eff. = %03.2f",chip_p0[i]);
    t->DrawText(scan_lo,0.1,h_title);

    c1->Update();
  }


  Int_t i_code[768];
  Float_t i_mean[768];
  Float_t i_sigma[768];
  Float_t i_chisq[768];
  Float_t ie_mean[768];
  Float_t ie_sigma[768];

  // Fit channels one at a time, using "tweaked" start conditions where possible
  // This gives much more reliable results than "fitslices"

  n = (h_nevent->GetMaximum()) / base_factor;
  if ((p0_guess=n)==0) p0_guess = 1;         // protection against divide by zero
  for(i=0;i<768;i++){

    // delete projection if it already exists
    if(h_ptr = (TH1F*) gDirectory->GetList()->FindObject("h_tp")){
      delete h_ptr;
    }

    h_scan->ProjectionY("h_tp",i,i,"egoff"); // propogate errors - PWP 28.10.99
    h_tp = (TH1F*) gDirectory->Get("h_tp");

    // error conditions: flag CH_UNDER and CH_OVER but still fit
    //                   do not attempt to fit CH_DEAD or CH_STUCK 

    Int_t code=0;
//    Float_t p0_guess = h_tp->GetMaximum(); // CAUTION!! ADDED FOR TEST PURPOSES PWP 29.09.99 

    if(h_tp->GetMaximum()<1.0){
      printf("channel %d is DEAD\n",i);
      code = CH_DEAD;
    } else if(h_tp->GetMinimum()>n*0.98){
      printf("channel %d is STUCK\n",i);
      code = CH_STUCK;
    } else if(h_tp->GetMaximum()<n){
      printf("channel %d is UNDER THE TOP\n",i);
      code = CH_UNDER;
    } else if(h_tp->GetMaximum()>n){
      printf("channel %d is OVER THE TOP\n",i);
      code = CH_OVER;
    }

    if(!(code&BAD_TEST)){ // attempt to fit CH_UNDER
                          // but skip CH_OVER, CH_DEAD and CH_STUCK

      if( (scan_type==1)||(scan_type==41)||(scan_type==3)){
        // VTHR, QTHR or STROBE DELAY
        Float_t p1_guess = find_level2(h_tp,0.50,1);          // last bin > 0%
        Float_t p2_guess = find_level2(h_tp,0.82,1)-p1_guess;  
        Float_t range_lo = find_bin2(h_tp,0.99,0,0);            // first bin > 99%
        Float_t range_hi = h_tp->GetBinCenter(h_tp->GetNbinsX());
        if(p2_guess<0) p2_guess = p2_guess*-1.0;
      } else if( (scan_type==2)||(scan_type==42)){
        // VCAL or QCAL 
        Float_t p1_guess = find_level2(h_tp,0.50,0);          // first bin < 50%
        Float_t p2_guess = find_level2(h_tp,0.82,0)-p1_guess;
        Float_t range_lo = h_tp->GetBinCenter(1);
        Float_t range_hi = find_bin2(h_tp,0.99,1,0);            // last bin > 99%
        if(p2_guess<0) p2_guess = p2_guess*-1.0;
      } else {
        Float_t p1_guess = h_tp->GetMean(); 
        Float_t p2_guess = h_tp->GetRMS(); 
//        Float_t p1_guess = chip_p1[i/128];
//        Float_t p2_guess = chip_p2[i/128];
        Float_t range_lo = h_tp->GetBinCenter(1);
        Float_t range_hi = h_tp->GetBinCenter(h_tp->GetNbinsX());
      }
    
      if(scan_type==3) p0_guess = h_tp->GetMaximum(); 

      f1->SetParameters(p0_guess,p1_guess,p2_guess);
      f1->SetParLimits(0,p0_guess,p0_guess);
      f1->SetRange(range_lo, range_hi);
      h_tp->Fit("f1","NRQ"); // no graphics, use range, quiet mode

      // Validity checking on fit data:
      // mean value must be within scanned range
      // sigma must be greater than zero

      Float_t mean = f1->GetParameter(1);
      if(mean<scan_lo){
        // printf("fit for channel %d is UNDER THE BOTTOM\n",i);
        code += FIT_UNDER;
      } else if(mean>scan_hi){
        // printf("fit for channel %d is OVER THE TOP\n",i);
        code += FIT_OVER;
      }
      if(f1->GetParameter(2)<0) code+=SIG_UNDER;
    }

    if(!(code&BAD_TEST)){
      i_mean[i]=f1->GetParameter(1); 
      i_sigma[i]=f1->GetParameter(2); 
      if(f1->GetNDF()>0)i_chisq[i]=f1->GetChisquare()/f1->GetNDF(); 
      ie_mean[i]=f1->GetParError(1); 
      ie_sigma[i]=f1->GetParError(2); 
    } else {
      i_mean[i]=0; 
      i_sigma[i]=0; 
      i_chisq[i]=0; 
    }
    i_code[i]=code;
  }

  // projections of all mean values...

  printf("Projecting Mean values for all channels....\n");
  TH1F* h_mall  =  new TH1F("h_mall", "Mean, all channels", 50,0,100);
  for(i=0;i<768;i++){
    if(!(i_code[i]&BAD_TEST)) h_mall->Fill(i_mean[i]);
  } 
  h_mall->Fit("gaus","0");
  f2=h_mall->GetFunction("gaus");

  Float_t offset_m = f2->GetParameter(1);
  Float_t offset_s = f2->GetParameter(2);
  printf("Fitted mean of mean values is %3.2f\n", offset_m);


  // projections of all sigma values - to facilitate range setting...

  // PWP 06.01.99 finally make some effort to tune hist ranges according to scan type

  printf("Projecting Sigma values for all channels....\n");
  if(scan_type==2){ // ST_VCAL
    TH1F* h_spall  =  new TH1F("h_spall", "Sigma, all channels", 40,0.01,10.001);
  }else if(scan_type==42)){ // ST_QCAL
    TH1F* h_spall  =  new TH1F("h_spall", "Sigma, all channels", 40,0.001,1.001);
  }else{
    TH1F* h_spall  =  new TH1F("h_spall", "Sigma, all channels", 20,0,50);
  }

  for(i=0;i<768;i++){
    if(!(i_code[i]&BAD_TEST)) h_spall->Fill(i_sigma[i]);
  } 
  h_spall->Fit("gaus","0");
  f2=h_spall->GetFunction("gaus");

  Float_t sigma_m = f2->GetParameter(1);
  Float_t sigma_s = f2->GetParameter(2);
  if (sigma_m<0){  // fit failed - fall back to "sensible" defaults
    sigma_m = (scan_hi-scan_lo)/2;
    sigma_s = sigma_m/2;
    printf("Fit failed - reverting to cut at %3.2f\n", 2*sigma_m);
  } else {
    printf("Fitted mean of sigma values is %3.2f, therefore upper cut is %3.2f\n",
      sigma_m, 2*sigma_m);
  }
  Float_t sigma_lo = sigma_m - (6*sigma_s);
  Float_t sigma_hi = sigma_m + (6*sigma_s);

  // Create histograms to hold the fitted scan data  

  TH1S * h_code  = new TH1S("h_code",  "Fit Code",     768,0.5,768.5);
  TH1F * h_mean  = new TH1F("h_mean",  "Fitted Means", 768,0.5,768.5);
  TH1F * h_sigma = new TH1F("h_sigma", "Fitted Sigma", 768,0.5,768.5);
  TH1F * h_chisq = new TH1F("h_chisq", "Chisq/NDF",    768,0.5,768.5);

  // Second pass at sigma data to mask excessive sigma values
  // also fill the histograms during this loop

  for(i=0;i<768;i++){
    if(i_sigma[i]>(2*sigma_m)){
      i_code[i]=i_code[i]+SIG_OVER;
    }
    if(!(i_code[i]&BAD_TEST)){
      h_mean->Fill(i,i_mean[i]); 
      h_mean->SetBinError(i,(float)ie_mean[i]); 
      h_sigma->Fill(i,i_sigma[i]); 
      h_sigma->SetBinError(i,(float)ie_sigma[i]); 
      h_chisq->Fill(i,i_chisq[i]); 
    }
    h_code->Fill(i,i_code[i]); 
  } 

  // Second pass at projections, to exclude bad channels from fit
  // projections of all offset values...

  printf("Projecting Offset values for all channels....\n");
  TH1F* h_mall  =  new TH1F("h_mall", "Mean, all channels", 50,scan_lo,scan_hi);
  for(i=0;i<768;i++){
    if(!(i_code[i]&BAD_TEST)) h_mall->Fill(i_mean[i]);
  } 
  h_mall->Fit("gaus","0");
  f2=h_mall->GetFunction("gaus");

  Float_t offset_m = f2->GetParameter(1);
  Float_t offset_s = f2->GetParameter(2);
  printf("Fitted mean of Offset values is %3.2f\n", offset_m);


  // projections of all sigma values - to facilitate range setting...

  // PWP 06.01.99 finally make some effort to tune hist ranges according to scan type

  printf("Projecting Sigma values for all channels....\n");
  if(scan_type==2){ // ST_VCAL
    TH1F* h_spall  =  new TH1F("h_spall", "Sigma, all channels", 40,0.01,10.01);
  }else if(scan_type==42)){ // ST_QCAL
    TH1F* h_spall  =  new TH1F("h_spall", "Sigma, all channels", 40,0.001,1.001);
  }else{
    TH1F* h_spall  =  new TH1F("h_spall", "Sigma, all channels", 20,0,50);
  }

  for(i=0;i<768;i++){
    if(!(i_code[i]&BAD_TEST)) h_spall->Fill(i_sigma[i]);
  } 
  h_spall->Fit("gaus","0");
  f2=h_spall->GetFunction("gaus");

  Float_t sigma_m = f2->GetParameter(1);
  Float_t sigma_s = f2->GetParameter(2);
  if (sigma_m<0){  // fit failed - fall back to "sensible" defaults
    sigma_m = (scan_hi-scan_lo)/2;
    sigma_s = sigma_m/2;
    printf("Fit failed - reverting to cut at %3.2f\n", 2*sigma_m);
  } else {
    printf("Fitted mean of sigma values is %3.2f, therefore upper cut is %3.2f\n",
      sigma_m, 2*sigma_m);
  }
  Float_t sigma_lo = sigma_m - (6*sigma_s);
  Float_t sigma_hi = sigma_m + (6*sigma_s);

  // Draw the fit parameters using the pads of canvas 1

  pad_m->cd();
  h_mean->SetLineColor(kRed);
  h_mean->SetMarkerColor(kBlack);
  h_mean->Draw("samep");
  TText * t = new TText();
  sprintf(h_title,"Mean (mean) = %03.2f%s, Sigma (mean) = %03.2f%s, Nentries = %d",
          offset_m, unit, offset_s, unit, h_mean->GetEntries());
  t->DrawText(32,(scan_lo+0.9*(scan_hi-scan_lo)),h_title);
  
  pad_s->cd();
  h_sigma->SetXTitle("Channel Number");
  h_sigma->SetYTitle(s_title);
  h_sigma->SetMaximum(2*sigma_m);
  h_sigma->SetMinimum(0);
  h_sigma->SetLineColor(kRed);
  h_sigma->SetMarkerColor(kBlack);
  h_sigma->Draw("ep");
  TText * t = new TText();
  sprintf(h_title,"Mean (sigma) = %03.2f%s, Sigma (sigma) = %03.2f%s, Nentries = %d",
          sigma_m, unit, sigma_s, unit, h_sigma->GetEntries());
  t->DrawText(32,(1.8*sigma_m),h_title);

  pad_c->cd();
  if(h_chisq->GetMaximum()>0.0) pad_c->SetLogy();
  h_chisq->SetXTitle("Channel Number");
  h_chisq->SetYTitle("Chisq");
  h_chisq->SetLineColor(kRed);
  h_chisq->SetMarkerColor(kBlack);
  h_chisq->Draw("ep");

  pad_t->cd();
  t1->Draw();
  c1->Update();


//  Create 18 histograms for projections of Mean, Sigma and Chisq
//   0 to  5  - all channels
//   6 to 11  - odd channels
//  12 to 17  - even channels


  TH1F* h_mp[18];
  TH1F* h_sp[18];

  TH1F* h_mp[0]  =  new TH1F("h_mp0", "Mean, all channels, chip 0",  50,scan_lo,scan_hi);
  TH1F* h_mp[1]  =  new TH1F("h_mp1", "Mean, all channels, chip 1",  50,scan_lo,scan_hi);
  TH1F* h_mp[2]  =  new TH1F("h_mp2", "Mean, all channels, chip 2",  50,scan_lo,scan_hi);
  TH1F* h_mp[3]  =  new TH1F("h_mp3", "Mean, all channels, chip 3",  50,scan_lo,scan_hi);
  TH1F* h_mp[4]  =  new TH1F("h_mp4", "Mean, all channels, chip 4",  50,scan_lo,scan_hi);
  TH1F* h_mp[5]  =  new TH1F("h_mp5", "Mean, all channels, chip 5",  50,scan_lo,scan_hi);
  TH1F* h_mp[6]  =  new TH1F("h_mp6", "Mean, odd channels, chip 0",  50,scan_lo,scan_hi);
  TH1F* h_mp[7]  =  new TH1F("h_mp7", "Mean, odd channels, chip 1",  50,scan_lo,scan_hi);
  TH1F* h_mp[8]  =  new TH1F("h_mp8", "Mean, odd channels, chip 2",  50,scan_lo,scan_hi);
  TH1F* h_mp[9]  =  new TH1F("h_mp9", "Mean, odd channels, chip 3",  50,scan_lo,scan_hi);
  TH1F* h_mp[10] =  new TH1F("h_mp10","Mean, odd channels, chip 4",  50,scan_lo,scan_hi);
  TH1F* h_mp[11] =  new TH1F("h_mp11","Mean, odd channels, chip 5",  50,scan_lo,scan_hi);
  TH1F* h_mp[12] =  new TH1F("h_mp12","Mean, even channels, chip 0", 50,scan_lo,scan_hi);
  TH1F* h_mp[13] =  new TH1F("h_mp13","Mean, even channels, chip 1", 50,scan_lo,scan_hi);
  TH1F* h_mp[14] =  new TH1F("h_mp14","Mean, even channels, chip 2", 50,scan_lo,scan_hi);
  TH1F* h_mp[15] =  new TH1F("h_mp15","Mean, even channels, chip 3", 50,scan_lo,scan_hi);
  TH1F* h_mp[16] =  new TH1F("h_mp16","Mean, even channels, chip 4", 50,scan_lo,scan_hi);
  TH1F* h_mp[17] =  new TH1F("h_mp17","Mean, even channels, chip 5", 50,scan_lo,scan_hi);

  TH1F* h_sp[0]  =  new TH1F("h_sp0", "Sigma, all channels, chip 0",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[1]  =  new TH1F("h_sp1", "Sigma, all channels, chip 1",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[2]  =  new TH1F("h_sp2", "Sigma, all channels, chip 2",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[3]  =  new TH1F("h_sp3", "Sigma, all channels, chip 3",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[4]  =  new TH1F("h_sp4", "Sigma, all channels, chip 4",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[5]  =  new TH1F("h_sp5", "Sigma, all channels, chip 5",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[6]  =  new TH1F("h_sp6", "Sigma, odd channels, chip 0",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[7]  =  new TH1F("h_sp7", "Sigma, odd channels, chip 1",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[8]  =  new TH1F("h_sp8", "Sigma, odd channels, chip 2",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[9]  =  new TH1F("h_sp9", "Sigma, odd channels, chip 3",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[10] =  new TH1F("h_sp10","Sigma, odd channels, chip 4",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[11] =  new TH1F("h_sp11","Sigma, odd channels, chip 5",  50,sigma_lo,sigma_hi);
  TH1F* h_sp[12] =  new TH1F("h_sp12","Sigma, even channels, chip 0", 50,sigma_lo,sigma_hi);
  TH1F* h_sp[13] =  new TH1F("h_sp13","Sigma, even channels, chip 1", 50,sigma_lo,sigma_hi);
  TH1F* h_sp[14] =  new TH1F("h_sp14","Sigma, even channels, chip 2", 50,sigma_lo,sigma_hi);
  TH1F* h_sp[15] =  new TH1F("h_sp15","Sigma, even channels, chip 3", 50,sigma_lo,sigma_hi);
  TH1F* h_sp[16] =  new TH1F("h_sp16","Sigma, even channels, chip 4", 50,sigma_lo,sigma_hi);
  TH1F* h_sp[17] =  new TH1F("h_sp17","Sigma, even channels, chip 5", 50,sigma_lo,sigma_hi);


// Phew!!
// Create and fill these histograms...

  for (i=0;i<6;i++){
    for(j=0;j<128;j++){
      bin = j + (128*i);
      code  = (Int_t)h_code->GetBinContent(bin)&BAD_TEST;
      if(!code){ // fill only good channels
        h_mp[i]->Fill(i_mean[bin]);
        h_sp[i]->Fill(i_sigma[bin]);
        if(j%2){ // odd channel
          h_mp[i+6]->Fill(i_mean[bin]);
          h_sp[i+6]->Fill(i_sigma[bin]);
        }else{
          h_mp[i+12]->Fill(i_mean[bin]);
          h_sp[i+12]->Fill(i_sigma[bin]);
        }
      }
    }
  }

// ...and fit gaussians to them, storing the results in the arrays

  float chip_fit_par[3][18];
  float chip_fit_err[3][18];

  for(i=0;i<18;i++){
    if(h_mp[i]->GetMaximum()>0.0){
      h_mp[i]->Fit("gaus","0QL"); // quiet mode
      f2=h_mp[i]->GetFunction("gaus");
      chip_fit_par[0][i]=f2->GetParameter(1);
      chip_fit_err[0][i]=f2->GetParameter(2);
    } else {
      chip_fit_par[0][i]=-1;
      chip_fit_err[0][i]=-1;
    }
    if(h_sp[i]->GetMaximum()>0.0){
      h_sp[i]->Fit("gaus","0QL"); // quiet mode 
      f2=h_sp[i]->GetFunction("gaus");
      chip_fit_par[1][i]=f2->GetParameter(1);
      chip_fit_err[1][i]=f2->GetParameter(2);
    } else {
      chip_fit_par[1][i]=-1;
      chip_fit_err[1][i]=-1;
    }
  }

// Optionally, two canvases are created to show the projections
// of fit data on a chip by chip basis.  This may be prevented by
// setting draw_fit_projections = 0.

  if(draw_fit_projections){

  // Open and draw new canvas to show the Mean Projections

    if(postscript) ps->NewPage();
    TCanvas* c2=new TCanvas ("c2","stan_fitscan_mean",1100,750);
    pad_mpt = new TPad("pad_mpt", "Text",                        0.01,0.91,0.99,0.99,33);
    pad_mp0 = new TPad("pad_mp0","Projection of Mean values",   0.01,0.61,0.99,0.90,33);
    pad_mp1 = new TPad("pad_mp1","Projection of Mean values",   0.01,0.31,0.99,0.60,33);
    pad_mp2 = new TPad("pad_mp2","Projection of Mean values",   0.01,0.01,0.99,0.30,33);

    pad_mpt->Draw();
    pad_mp0->Draw();
    pad_mp1->Draw();
    pad_mp2->Draw();
    pad_mp0->Divide(6,1);
    pad_mp1->Divide(6,1);
    pad_mp2->Divide(6,1);
    c2->Update();

    for(i=0; i<6; i++){
      pad_mp0->cd(i+1);
      h_mp[i]->SetXTitle(m_title);
      h_mp[i]->SetYTitle("N(Mean)");
      h_mp[i]->SetLineColor(kRed);
      h_mp[i]->SetMarkerColor(kBlack);
      h_mp[i]->Draw("ep");
    }

    for(i=0; i<6; i++){
      pad_mp1->cd(i+1);
      h_mp[i+6]->SetXTitle(m_title);
      h_mp[i+6]->SetYTitle("N(Mean)");
      h_mp[i+6]->SetLineColor(kRed);
      h_mp[i+6]->SetMarkerColor(kBlack);
      h_mp[i+6]->Draw("ep");
    }

    for(i=0; i<6; i++){
      pad_mp2->cd(i+1);
      h_mp[i+12]->SetXTitle(m_title);
      h_mp[i+12]->SetYTitle("N(Mean)");
      h_mp[i+12]->SetLineColor(kRed);
      h_mp[i+12]->SetMarkerColor(kBlack);
      h_mp[i+12]->Draw("ep");
    }

    pad_mpt->cd();
    t1->Draw();
    c2->Update();

  // Open and draw new canvas to show the Sigma Projections

    if(postscript) ps->NewPage();
    TCanvas* c3=new TCanvas ("c3","stan_fitscan_sigma",1100,750);
    pad_spt = new TPad("pad_spt", "Text",                        0.01,0.91,0.99,0.99,33);
    pad_sp0 = new TPad("pad_sp0","Projection of Sigma values",   0.01,0.61,0.99,0.90,33);
    pad_sp1 = new TPad("pad_sp1","Projection of Sigma values",   0.01,0.31,0.99,0.60,33);
    pad_sp2 = new TPad("pad_sp2","Projection of Sigma values",   0.01,0.01,0.99,0.30,33);

    pad_spt->Draw();
    pad_sp0->Draw();
    pad_sp1->Draw();
    pad_sp2->Draw();
    pad_sp0->Divide(6,1);
    pad_sp1->Divide(6,1);
    pad_sp2->Divide(6,1);
    c3->Update();

    for(i=0; i<6; i++){
      pad_sp0->cd(i+1);
      h_sp[i]->SetXTitle(s_title);
      h_sp[i]->SetYTitle("N(Sigma)");
      h_sp[i]->SetLineColor(kRed);
      h_sp[i]->SetMarkerColor(kBlack);
      h_sp[i]->Draw("ep");
    }

    for(i=0; i<6; i++){
      pad_sp1->cd(i+1);
      h_sp[i+6]->SetXTitle(s_title);
      h_sp[i+6]->SetYTitle("N(Sigma)");
      h_sp[i+6]->SetLineColor(kRed);
      h_sp[i+6]->SetMarkerColor(kBlack);
      h_sp[i+6]->Draw("ep");
    }

    for(i=0; i<6; i++){
      pad_sp2->cd(i+1);
      h_sp[i+12]->SetXTitle(s_title);
      h_sp[i+12]->SetYTitle("N(Sigma)");
      h_sp[i+12]->SetLineColor(kRed);
      h_sp[i+12]->SetMarkerColor(kBlack);
      h_sp[i+12]->Draw("ep");
    }

    pad_spt->cd();
    t1->Draw();
    c3->Update();

  // Open and draw new canvas to show the chisq Projections

  }  // end if(draw_fit_projections)

  if(postscript) {
    ps->Close();
    delete ps;
  }


  // Tally total numbers of good and bad channels  

  short chip_bad_ch[7][10];
  short chip_good_ch[7];

  for(i=0;i<7;i++){
     chip_good_ch[i]=0;     // clear good channel count
    for(k=0;k<10;k++) {     // clear bad channel count
      chip_bad_ch[i][k]=0;
    } 
  }

  for(i=0;i<6;i++){
    for(j=0;j<128;j++){

      code=(Int_t)h_code->GetBinContent(j+(128*i));
      if(code){
        if (code&CH_DEAD)  chip_bad_ch[i][0]++;
        if (code&CH_STUCK) chip_bad_ch[i][1]++;
        if (code&CH_UNDER) chip_bad_ch[i][2]++;
        if (code&CH_OVER)  chip_bad_ch[i][3]++;
        if (code&FIT_UNDER) chip_bad_ch[i][4]++;
        if (code&FIT_OVER)  chip_bad_ch[i][5]++;
        if (code&SIG_UNDER) chip_bad_ch[i][6]++;
        if (code&SIG_OVER)  chip_bad_ch[i][7]++;

        if (code&DEAD_TEST) chip_bad_ch[i][8]++; // DEAD  channels
        if (code&DODGY_TEST) chip_bad_ch[i][9]++; // DODGY channels
      }
      if(!(code&BAD_TEST)) chip_good_ch[i]++;
    }
  }

  for(i=0;i<6;i++){
    chip_good_ch[6] += chip_good_ch[i];
    for(k=0;k<10;k++) chip_bad_ch[6][k] += chip_bad_ch[i][k];
  }

// Summary data to screen

  // 06.12.00 PWP add code to use chip numbers 6 to 11 in case where link=1.
  //              (in response to a request from MM)

  Int_t   chip_offset=0;
  if(loi) chip_offset=6;

  if(time)  sprintf(s,"%s",time->GetName());
  if(place) { sprintf(s,"%s  -  %s",s,place->GetName()); delete place; }
  if(mname) { sprintf(s,"%s  -  Module %s",s,mname->GetName()); delete mname; }

  Float_t noise_factor;
  if(scan_type==2) noise_factor = 625.0;
  else             noise_factor = 6250.0;

//  Print Summary Data (0) - added 06.12.00 PWP
//  Print overall summary data - mean of distribution of mean, sigma
//  Also show noise in electrons if appropriate

  if(time) printf("\n%s\n\n",s);
  printf("Overall Summary for link %d\n",loi);
  printf("\tMean(%s)  %3.2f  error %3.2f %s\n",
    scan_name, offset_m, offset_s, unit);
  printf("\tSigma(%s) %3.2f  error %3.2f %s\n",
    scan_name, sigma_m, sigma_s, unit);

  if((scan_type==2)||(scan_type==42)){ // ST_VCAL or ST_QCAL
    printf("\tNoise %3.2f ENC error %3.2f ENC\n", 
      sigma_m*noise_factor, sigma_s*noise_factor);
  }

//  Print Summary Data (1)
//  Fitted mean and associated error for all, odd and even channels

  printf("\nChip\tMean\t\t\terror\n");
  printf(  "\t(all)\t(odd)\t(even)\t(all)\t(odd)\t(even)\n");
  for(i=0;i<6;i++){
    printf("%d\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\n", i+chip_offset,
      chip_fit_par[0][i], chip_fit_par[0][i+6], chip_fit_par[0][i+12],
      chip_fit_err[0][i], chip_fit_err[0][i+6], chip_fit_err[0][i+12]);
  }
  printf("\t%s\t%s\t%s\t%s\t%s\t%s\n",
    unit,unit,unit,unit,unit,unit);

//  Print Summary Data (2)
//  Fitted sigma and associated error for all, odd and even channels

  printf("\nChip\tSigma\t\t\terror\n");
  printf(  "\t(all)\t(odd)\t(even)\t(all)\t(odd)\t(even)\n");
  for(i=0;i<6;i++){
    printf("%d\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\n", i+chip_offset,
      chip_fit_par[1][i], chip_fit_par[1][i+6], chip_fit_par[1][i+12],
      chip_fit_err[1][i], chip_fit_err[1][i+6], chip_fit_err[1][i+12]);
  }
  printf("\t%s\t%s\t%s\t%s\t%s\t%s\n",
    unit,unit,unit,unit,unit,unit);

//  Print Summary Data (2.1)
//  Noise in ENC for all, odd and even channels if appropriate

  if((scan_type==2)||(scan_type==42)){ // ST_VCAL or ST_QCAL
    printf("\nChip\tNoise\t\t\terror\n");
    printf(  "\t(all)\t(odd)\t(even)\t(all)\t(odd)\t(even)\n");
    for(i=0;i<6;i++){
      printf("%d\t%4.0f\t%4.0f\t%4.0f\t%4.0f\t%4.0f\t%4.0f\n", i+chip_offset,
        chip_fit_par[1][i]*noise_factor, chip_fit_par[1][i+6]*noise_factor, 
        chip_fit_par[1][i+12]*noise_factor, chip_fit_err[1][i]*noise_factor, 
        chip_fit_err[1][i+6]*noise_factor, chip_fit_err[1][i+12]*noise_factor);
    }
    printf("\tENC\tENC\tENC\tENC\tENC\tENC\n");
  }

//  Print Summary Data (3)
//  Is there a difference between odd and even channels??? 

    printf("\nChip\tO_mean-E_mean\tO_sig-E_sig\n");
    printf(      "\t-------------\t-----------\n");
    printf(      "\tO_mean+E_mean\tO_sig+E_sig\n");
  for(i=0;i<6;i++){
    printf("%d\t%03.2f\t\t%03.2f\n",i+chip_offset,
      (chip_fit_par[0][i+6]-chip_fit_par[0][i+12])/(chip_fit_par[0][i+6]+chip_fit_par[0][i+12]),
      (chip_fit_par[1][i+6]-chip_fit_par[1][i+12])/(chip_fit_par[1][i+6]+chip_fit_par[1][i+12]));
  }
  printf("\t%s\t\t%s\n",
    unit,unit);


//  Print Summary Data (3)
//  Bad Channel Summary

  printf("\nBad Channel Summary\n");
  printf("Chip\tDead\tStuck\tN_under\tN_over\tF_under\tF_over\tS_under\tS_over\n");
  for(i=0;i<6;i++){
    printf("%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n", i+chip_offset,
      chip_bad_ch[i][0], chip_bad_ch[i][1],
      chip_bad_ch[i][2], chip_bad_ch[i][3],
      chip_bad_ch[i][4], chip_bad_ch[i][5],
      chip_bad_ch[i][6], chip_bad_ch[i][7]);
  }
  printf("total\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",
    chip_bad_ch[6][0], chip_bad_ch[6][1],
    chip_bad_ch[6][2], chip_bad_ch[6][3],
    chip_bad_ch[6][4], chip_bad_ch[6][5],
    chip_bad_ch[6][6], chip_bad_ch[6][7]);

  printf("\nChannel Totals\n");
  printf("Chip\tDead\tDodgy\tUseful\n");
  for(i=0;i<6;i++){
    printf("%d\t%d\t%d\t%d\n", i+chip_offset, chip_bad_ch[i][8], 
      chip_bad_ch[i][9], chip_good_ch[i]);
  }
  printf("total\t%d\t%d\t%d\n", chip_bad_ch[6][8], 
    chip_bad_ch[6][9], chip_good_ch[6]);

// Ascii file output - if relevant option(s) selected

// Output fit results to text file

  if(ascii){
    if(RAL) sprintf(f_name,"M:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run,sno,moi,loi);
    else    sprintf(f_name,"D:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run,sno,moi,loi);

    FILE* fitfile = fopen(f_name,"w");
    if(fitfile==NULL){
      printf("Could not open file for fit data ascii output\n");
    } else {
      fprintf(fitfile,"ch   code  mean   err  sigma err  chisq/NDF\n");
      for(j=0;j<768;j++){
        fprintf(fitfile,"%3d 0x%04x %3.2f %3.2f %3.2f %3.2f %3.2f\n",
          j, h_code->GetBinContent(j),
          h_mean->GetBinContent(j), h_mean->GetBinError(j),
          h_sigma->GetBinContent(j), h_sigma->GetBinError(j),
          h_chisq->GetBinContent(j));
      }
      fclose(fitfile);
    }
  }

// Output summary and bad channel data to text file



  if(ascii){

    if(RAL) sprintf(f_name,"M:\\sctvar\\data\\strun%d_%d_%d_%d.sum",run,sno,moi,loi);
    else    sprintf(f_name,"D:\\sctvar\\data\\strun%d_%d_%d_%d.sum",run,sno,moi,loi);

    FILE* sumfile = fopen(f_name,"w");

  //  Print Summary Data (0) - added 06.12.00 PWP
  //  Print overall summary data - mean of distribution of mean, sigma
  //  Also show noise in electrons if appropriate

    if(time) { fprintf(sumfile,"%s\n\n",s); delete time; }
    fprintf(sumfile,"Overall Summary for link %d\n",loi);
    fprintf(sumfile,"\tMean  %3.2f %s  error %3.2f %s\n",
      offset_m, unit, offset_s, unit);
    fprintf(sumfile,"\tSigma %3.2f %s  error %3.2f %s\n",
      sigma_m, unit, sigma_s, unit);

    if((scan_type==2)||(scan_type==42)){ // ST_VCAL or ST_QCAL
      fprintf(sumfile,"\tNoise %3.2f ENC error %3.2f ENC\n", 
        sigma_m*noise_factor, sigma_s*noise_factor);
    }

  //  Print Summary Data (1)
  //  Fitted mean and associated error for all, odd and even channels
  
    fprintf(sumfile,"\nChip\tMean\t\t\terror\n");
    fprintf(sumfile,"\t(all)\t(odd)\t(even)\t(all)\t(odd)\t(even)\n");
    for(i=0;i<6;i++){
      fprintf(sumfile,"%d\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\n", i+chip_offset,
        chip_fit_par[0][i], chip_fit_par[0][i+6], chip_fit_par[0][i+12],
        chip_fit_err[0][i], chip_fit_err[0][i+6], chip_fit_err[0][i+12]);
    }
    fprintf(sumfile,"\t%s\t%s\t%s\t%s\t%s\t%s\n",
      unit,unit,unit,unit,unit,unit);

  //  Print Summary Data (2)
  //  Fitted sigma and associated error for all, odd and even channels

    fprintf(sumfile,"\nChip\tSigma\t\t\terror\n");
    fprintf(sumfile,  "\t(all)\t(odd)\t(even)\t(all)\t(odd)\t(even)\n");
    for(i=0;i<6;i++){
      fprintf(sumfile,"%d\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\t%3.2f\n", i+chip_offset,
        chip_fit_par[1][i], chip_fit_par[1][i+6], chip_fit_par[1][i+12],
        chip_fit_err[1][i], chip_fit_err[1][i+6], chip_fit_err[1][i+12]);
    }
    fprintf(sumfile,"\t%s\t%s\t%s\t%s\t%s\t%s\n",
      unit,unit,unit,unit,unit,unit);

  //  Print Summary Data (2.1)
  //  Noise in ENC for all, odd and even channels if appropriate

    if((scan_type==2)||(scan_type==42)){ // ST_VCAL or ST_QCAL
      fprintf(sumfile,"\nChip\tNoise\t\t\terror\n");
      fprintf(sumfile,"\t(all)\t(odd)\t(even)\t(all)\t(odd)\t(even)\n");
      for(i=0;i<6;i++){
        fprintf(sumfile,"%d\t%4.0f\t%4.0f\t%4.0f\t%4.0f\t%4.0f\t%4.0f\n", i+chip_offset,
          chip_fit_par[1][i]*noise_factor, chip_fit_par[1][i+6]*noise_factor, 
          chip_fit_par[1][i+12]*noise_factor, chip_fit_err[1][i]*noise_factor, 
          chip_fit_err[1][i+6]*noise_factor, chip_fit_err[1][i+12]*noise_factor);
      }
      fprintf(sumfile,"\tENC\tENC\tENC\tENC\tENC\tENC\n");
    }

  //  Print Summary Data (3)
  //  Is there a difference between odd and even channels??? 
  
    fprintf(sumfile,"\nChip\tO_mean-E_mean\tO_sig-E_sig\n");
    fprintf(sumfile,      "\t-------------\t-----------\n");
    fprintf(sumfile,      "\tO_mean+E_mean\tO_sig+E_sig\n");
    for(i=0;i<6;i++){
      fprintf(sumfile,"%d\t%03.2f\t\t%03.2f\n",i+chip_offset,
        (chip_fit_par[0][i+6]-chip_fit_par[0][i+12])/(chip_fit_par[0][i+6]+chip_fit_par[0][i+12]),
        (chip_fit_par[1][i+6]-chip_fit_par[1][i+12])/(chip_fit_par[1][i+6]+chip_fit_par[1][i+12]));
    }
    fprintf(sumfile,"\t%s\t\t%s\n",
      unit,unit);

  //  Print Summary Data (3)
  //  Bad Channel Summary

    fprintf(sumfile,"\nValues from fits and cuts (LE 27/1 2000):\n");
    fprintf(sumfile,"Fitted mean of mean values is %3.2f\n", offset_m);
    fprintf(sumfile,"Fitted mean of sigma values is %3.2f, therefore upper cut is %3.2f\n",
          sigma_m, 2*sigma_m);
    fprintf(sumfile,"Fitted mean of Offset values is %3.2f, with a spread of %3.2f\n", 
          offset_m,offset_s);

    fprintf(sumfile,"\nBad Channel Summary\n");
    fprintf(sumfile,"Chip\tDead\tStuck\tN_under\tN_over\tF_under\tF_over\tS_under\tS_over\n");
    for(i=0;i<6;i++){
      fprintf(sumfile,"%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n", i+chip_offset,
        chip_bad_ch[i][0], chip_bad_ch[i][1],
        chip_bad_ch[i][2], chip_bad_ch[i][3],
        chip_bad_ch[i][4], chip_bad_ch[i][5],
        chip_bad_ch[i][6], chip_bad_ch[i][7]);
    }
    fprintf(sumfile,"total\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\n",
      chip_bad_ch[6][0], chip_bad_ch[6][1],
      chip_bad_ch[6][2], chip_bad_ch[6][3],
      chip_bad_ch[6][4], chip_bad_ch[6][5],
      chip_bad_ch[6][6], chip_bad_ch[6][7]);

    fprintf(sumfile,"\nBad Channel Totals\n");
    fprintf(sumfile,"Chip\tDead\tDodgy\tUseful\n");
    for(i=0;i<6;i++){
      fprintf(sumfile,"%d\t%d\t%d\t%d\n",  i+chip_offset, chip_bad_ch[i][8], 
        chip_bad_ch[i][9],chip_good_ch[i]);
    }
    fprintf(sumfile,"total\t%d\t%d\t%d\n", chip_bad_ch[6][8], 
        chip_bad_ch[6][9],chip_good_ch[6]);

  // List anomalous channels
  
    fprintf(sumfile,"\nBad Channel Data\n");
    fprintf(sumfile,"ch   code  mean   err  sigma err  chisq/NDF\n");
    for(j=0;j<768;j++){
      code = h_code->GetBinContent(j);
      if(code){
        fprintf(sumfile,"%d 0x%04x %3.2f %3.2f %3.2f %3.2f %3.2f\n",j,code,
          h_mean->GetBinContent(j), h_mean->GetBinError(j),
          h_sigma->GetBinContent(j), h_sigma->GetBinError(j),
          h_chisq->GetBinContent(j));
      }
    }
  fclose(sumfile);
  }

}

// And now some wrappers to save those fingers...

void stan_fitscan(int run, int sno, int moi, int loi){

  stan_fitscan(run,sno,moi,loi,1,1);
}

void stan_fitscan(int moi, int loi){

  stan_fitscan(-1,-1,moi,loi,1,1);
}

void stan_fitscan(){

  printf("Possible Uses of stan_fitscan:\n\n");
  printf(".x stan_fitscan.cpp(module,link)\n");
  printf(".x stan_fitscan.cpp(run,scan,module,link)\n");
  printf(".x stan_fitscan.cpp(run,scan,module,link,ascii,postscript)\n\n");

}



