void stan_no(int run1,int scan1,int moi,int loi){

  // Macro to plot log(occupancy) vs (thr^2)
  // Warning: only LINEAR response curve fit is supported for the time being
  //
  // PWP 31.05.00
  // Version 0.11 BJG, 30.04.02   Fix some memory leaks

  char f_name[64], h_name1[32],h_name2[32],s[128],line[1024];
  int lo_ch, hi_ch;

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");

  // Global style variations
  stan_styles();
  gStyle.SetOptStat(0);

  if(RAL) sprintf(f_name,"M:\\sctvar\\data\\strun%d_%d.root",run1,scan1);
  else    sprintf(f_name,"D:\\sctvar\\data\\strun%d_%d.root",run1,scan1);

  if(loi)sprintf(h_name1,"h_scan1;%d",moi+1);
  else    sprintf(h_name1,"h_scan0;%d",moi+1);
  printf("  Histogram1: %s from ROOT Datafile %s\n",h_name1,f_name);

  TFile *strun = new TFile(f_name);
  TH2F  *h1= (TH2F*)strun->Get(h_name1);

  sprintf(h_name1,"ModuleName;%d",moi+1);
  TObjString *time  = (TObjString*)strun->Get("Time");
  TObjString *place = (TObjString*)strun->Get("Place");
  TObjString *mname = (TObjString*)strun->Get(h_name1);

  TCanvas* c1=new TCanvas ("c1","stan_no",1100,750);
  pad_t = new TPad("pad_t","Text",           0.00,0.91,0.99,0.99,33);
  pad_p1 = new TPad("pad_p1","Projection 1", 0.00,0.00,0.99,0.90,33);

  pad_t->Draw();
  pad_p1->Draw();
  pad_p1->Divide(3,2);

  // load Response Curve Data file

  Int_t chip, m,f;
  Int_t method[12], function[12];
  Float_t p0,p1,p2;
  Float_t par[12][3];

  if(RAL) sprintf(f_name,"M:\\sctvar\\config\\%s.rcdat",mname->GetName());
  else    sprintf(f_name,"D:\\sctvar\\config\\%s.rcdat",mname->GetName());

  FILE* rcfile = fopen(f_name,"r");

  if(rcfile==NULL){
    printf("Could not open Response Curve file %s\n",f_name);
    return;
  } else {
    printf("File %s opened for reading\n",f_name);

    fgets(line,1024,rcfile);
    while(fgets(line,1024,rcfile) != NULL){
      printf("%s",line);
      sscanf(line,"%d %d %d %f %f %f",&chip,&m,&f,&p0,&p1,&p2);
      method[chip]=m;
      function[chip]=f;
      par[chip][0]=p0;
      par[chip][1]=p1;
      par[chip][2]=p2;
       printf("chip %d %d %d p0 %3.2f p1 %3.2f p2 %3.2f\n",chip,m,f,p0,p1,p2);
    }
  }
  fclose(rcfile);

  // load Mask File

  Int_t chan;
  Int_t chan_per_chip[12]={128,128,128,128,128,128,128,128,128,128,128,128};

  if(RAL) sprintf(f_name,"M:\\sctvar\\config\\%s.mask",mname->GetName());
  else    sprintf(f_name,"D:\\sctvar\\config\\%s.mask",mname->GetName());

  FILE* maskfile = fopen(f_name,"r");

  if(maskfile==NULL){
    printf("Could not open mask file %s\n",f_name);
    return;
  } else {
    printf("File %s opened for reading\n",f_name);

    while(fgets(line,1024,maskfile) != NULL){
      sscanf(line,"%d",&chan);
      chan_per_chip[chan/128]--;
    }
  }
  fclose(maskfile);

  for(chip=0;chip<12;chip++){
    printf("chip %d has %d active channels\n",chip,chan_per_chip[chip]);
  }

  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(s,"ATLAS SCT Noise Occupancy Scan");
  if(time)  sprintf(s,"%s  -  %s",s,time->GetName());
  if(place) sprintf(s,"%s  -  %s",s,place->GetName());
  if(mname) sprintf(s,"%s  -  Module %s",s,mname->GetName());
  t1->AddText(s);

  sprintf(s,"Run %d   Scan %d   Module %d   Stream %d  log(occupancy) (vs thr^2)", run1, scan1, moi, loi);
  t1->AddText(s);
  pad_t->cd();
  t1->Draw();

  Int_t bin,chip2;
  TH1F* hnop[6];
  TH2F* hlnop[6];
  Int_t point[6];
  Double_t nse1[6];
  Double_t nse2[6];
  Float_t occ[6][100];
  Float_t thr[6][100];
  Double_t threshold,occupancy;
  Float_t occe[100];
  Float_t thre[100];

  for(chip=0;chip<100;chip++){
    occe[chip]=0.001;
    thre[chip]=0.001;
  }

  for(chip=0;chip<6;chip++){
    lo_ch = (chip*128);
    hi_ch = ((chip+1)*128)-1;

    if(loi)chip2=chip+6;
    else   chip2=chip;
    printf("chip %d, lo_ch %d, hi_ch %d\n",chip,lo_ch, hi_ch);

    pad_p1->cd(chip+1);

    h1->Print();

    sprintf(h_name1,"hnop%d",chip);
    h1->ProjectionY(h_name1,lo_ch,hi_ch);
    hnop[chip] = (TH1F*) gDirectory->GetList()->FindObject(h_name1);

    Double_t scale_factor;
    if(loi) scale_factor = 1.0/chan_per_chip[chip+6];
    else    scale_factor = 1.0/chan_per_chip[chip];
    hnop[chip]->Scale(scale_factor);
    // hnop[chip]->Draw();

    point[chip]=0;

    for(bin=1; bin<=hnop[chip]->GetNbinsX();bin++){

      threshold = (hnop[chip]->GetBinCenter(bin)-par[chip2][0])/par[chip2][1];
      occupancy = log(hnop[chip]->GetBinContent(bin));

      if((threshold>=0)&&(occupancy>-20)){
        occ[chip][point[chip]] = occupancy;
        thr[chip][point[chip]] = threshold**2;
        // printf("%f %f\n",occ[chip][point[chip]], thr[chip][point[chip]]);
        point[chip]++;
      }
    }

    sprintf(h_name1,"hlnop%d",chip);
    sprintf(h_name2,"Noise Occupancy chip %d",chip);
    TH2F *h1p = new TH2F(h_name1,h_name2,2,0,2,2,-20,0);
    h1p->SetXTitle("Q^2 (fC^2)");
    h1p->SetYTitle("ln(Occupancy)");
    h1p->Draw();

    gno= new TGraph(point[chip],thr[chip],occ[chip]);
    gno->SetMarkerStyle(6);
    gno->SetMarkerColor(kBlack);
    gno->Draw("ep");

    TF1 *f3 = new TF1("f3","pol1",0,2);
    gno->Fit("f3","RQ");
    f3->Draw("same");

    nse1[chip] = TMath::Sqrt(1/(-2*f3->GetParameter(1)));
    nse2[chip] = nse1[chip]*6250;

    printf("Alleged Noise chip %d %3.2f fC or %3.2f ENC\n",chip,nse1[chip],nse2[chip]);
  }

  c1->Update();

  if(time) delete time;
  if(place) delete place;
  if(mname) delete mname;
}

void stan_no(){

  printf("Possible Uses of stan_no:\n\n");
  printf(".x stan_no.cpp(run,scan,moi,loi)\n");

}

