Int_t get_val(char* line, Int_t index, double* val){

  // convert next element of data row "line" to type double
  // (CINT sscanf is limited to <43 elements, hence this approach)

  char substring[20];

  sscanf(line+=index,"%s ",substring);
  sscanf(substring,"%lf",val);
  return 1 + strlen(substring);
}

void stan_ntplot(int run, int sno, char* var2_name, char* var1_name, short dowait, short postscript){

  // version 0.1 PWP 27.09.00
  // version 0.2 PWP 28.09.00
  // version 0.3 PWP 29.01.01 automatic setting of x axis range
  // Version 0.31 BJG, 30.04.02   Fix some memory leaks


#define NUM_VARS     8
#define MAX_MODULES  6
#define MAX_CHIPS    12
#define MAX_POINTS2  32
#define MAX_POINTS1  8

  short module_active[MAX_MODULES];

  short np1[MAX_MODULES][MAX_CHIPS][MAX_POINTS2];  // eg. vcal
  short np2[MAX_MODULES][MAX_CHIPS][MAX_POINTS1];  // eg. strobe delay

  float x_var1[MAX_MODULES][MAX_CHIPS][MAX_POINTS2][MAX_POINTS1];
  float x_var2[MAX_MODULES][MAX_CHIPS][MAX_POINTS1][MAX_POINTS2];

  float y_rc[MAX_MODULES][MAX_CHIPS][MAX_POINTS2][MAX_POINTS1];
  float y_gain[MAX_MODULES][MAX_CHIPS][MAX_POINTS2];
  float y_offset[MAX_MODULES][MAX_CHIPS][MAX_POINTS2];

  float y_vt50[MAX_MODULES][MAX_CHIPS][MAX_POINTS1][MAX_POINTS2];
  float y_outnse[MAX_MODULES][MAX_CHIPS][MAX_POINTS1][MAX_POINTS2];
  float y_goodfits[MAX_MODULES][MAX_CHIPS][MAX_POINTS1][MAX_POINTS2];
  float y_innse[MAX_MODULES][MAX_CHIPS][MAX_POINTS1][MAX_POINTS2];

  char   f_name[64], ps_fname[64], line[1024];
  char   gr_title[64], h_name[64],h_title[64];
  double var;
  float  data_row[NUM_VARS];
  int    i,j,k,l, index, link;

  printf("stan_ntuple called for run %d start scan %d\n",run,sno);

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");

  // Global style variations
  stan_styles();
  gStyle.SetOptStat(0);
  gStyle.SetOptFit(10);

  // clear arrays

  for(i=0;i<MAX_MODULES;i++){
    module_active[i]=0;
    for(j=0;j<MAX_CHIPS;j++){
      for(k=0;k<MAX_POINTS2;k++){
        np1[i][j][k]=0;
      }
      for(k=0;k<MAX_POINTS1;k++){
        np2[i][j][k]=0;
        for(l=0;l<MAX_POINTS2;l++){
          y_vt50[i][j][k][l]=0;
          y_outnse[i][j][k][l]=0;
          y_goodfits[i][j][k][l]=0;
        }
      }
    }
  }


  if (RAL) sprintf(f_name,"M:\\sctvar\\data\\stnt_%d_%d.dat",run,sno);
  else sprintf(f_name,"D:\\sctvar\\data\\stnt_%d_%d.dat",run,sno);

  FILE* ascii_ntuple = fopen(f_name,"r");
  if(ascii_ntuple==NULL){
    printf("Could not open ascii ntuple of fit summary data %s\n",f_name);
    return;
  } else {
    printf("File %s opened for reading\n",f_name);

    // First line contains column definitions
    fgets(line,1024,ascii_ntuple);
    TNtuple* nt_data = new TNtuple("nt_data","Fit Results",line);

    while(fgets(line,1024,ascii_ntuple) != NULL){
      index = 0;
      for(i=0;i<NUM_VARS;i++){
        index+=get_val(line,index,&var);
        data_row[i] = var;
      }
      nt_data->Fill(data_row);
    }
    fclose(ascii_ntuple);
  }

  nt_data->Print();

  // load the second of the root files to obtain timestamp info

  if(RAL) sprintf(f_name,"M:\\sctvar\\data\\strun%d_%d.root",run,sno);
  else    sprintf(f_name,"D:\\sctvar\\data\\strun%d_%d.root",run,sno);
  
  printf("Using Time/Place/Module name information from %s\n",f_name); 
  TFile *strun = new TFile(f_name,"READ");


  printf("Forming vectors for graphs....\n");


  Float_t scan,module,chip,var2,var1,vt50,outnse,goodfits;
  
  //  set the branches
  
  nt_data.SetBranchAddress("scan",&scan);
  nt_data.SetBranchAddress("module",&module);
  nt_data.SetBranchAddress("chip",&chip);
  nt_data.SetBranchAddress(var2_name,&var2);
  nt_data.SetBranchAddress(var1_name,&var1);
  nt_data.SetBranchAddress("vt50",&vt50);
  nt_data.SetBranchAddress("outnse",&outnse);
  nt_data.SetBranchAddress("goodfits\n",&goodfits);

  nt_data.GetEntry(0);

  Int_t p2 = 0;
  Float_t lo_var2 =  999999;
  Float_t hi_var2 = -999999;
  Float_t start_var2 = var2;
  Float_t last_var2  = var2;
  printf("start_var2 = %f\n",start_var2);

  Int_t p1 = 0;
  Float_t start_var1 = var1;
  Float_t last_var1  = var1;
  printf("start_var1 = %f\n",start_var1);

  for(i=0;i<nt_data->GetEntries();i++){       // ...look at each event.. .

    nt_data.GetEntry(i);

    Int_t i_module = (int)module;
    Int_t i_chip   = (int)chip;

    if(var2!=last_var2){  // eg. strobe delay
      p2++;
    }
    if(var2==start_var2) {
      p2=0;
    }
    if(var2<lo_var2){
      lo_var2=var2;
    }
    if(var2>hi_var2){
      hi_var2=var2;
    }
    last_var2 = var2;

    if(var1!=last_var1){  // eg. vcal
      p1++;
    }
    if(var1==start_var1) {
      p1=0;
    }
    last_var1 = var1;

    // fill y arrays if indices are within bounds

    if((i_module<MAX_MODULES)&&(i_chip<MAX_CHIPS)&&(p2<MAX_POINTS2)&&(p1<MAX_POINTS1)){

      //printf("Fill %d %d %d %d\n",i_module,i_chip,p1,p2);


      // module chip (delay) (vcal)
      x_var1[i_module][i_chip][p2][p1]=var1;
      y_rc[i_module][i_chip][p2][p1]= vt50;

      // module chip (vcal) (delay)
      x_var2[i_module][i_chip][p1][p2]=var2;

      y_vt50[i_module][i_chip][p1][p2]= vt50;
      y_outnse[i_module][i_chip][p1][p2]= outnse;
      y_goodfits[i_module][i_chip][p1][p2]= goodfits;

      np1[i_module][i_chip][p2]++;  // number of points to each graphs  vs var1(vcal) for given var2(delay)
      np2[i_module][i_chip][p1]++;  // number of points to each graphs  vs var2(delay) for given var1(vcal)
      if(module_active[i_module]==0) module_active[i_module]=1;
    }
  }


  printf("Forming the graphs of vt50, output noise and n(good fits)...\n");


  TGraph* gr_vt50[MAX_MODULES][MAX_CHIPS][MAX_POINTS2];
  TGraph* gr_outnse[MAX_MODULES][MAX_CHIPS][MAX_POINTS2];
  TGraph* gr_goodfits[MAX_MODULES][MAX_CHIPS][MAX_POINTS2];
  TGraph* gr_innse[MAX_MODULES][MAX_CHIPS][MAX_POINTS2];

  TGraph* gr_rc[MAX_MODULES][MAX_CHIPS][MAX_POINTS1];
  TGraph* gr_gain[MAX_MODULES][MAX_CHIPS];
  TGraph* gr_offset[MAX_MODULES][MAX_CHIPS];

  for(i=0;i<MAX_MODULES;i++){      // loop over modules
    if(module_active[i]){
      for(j=0;j<MAX_CHIPS;j++){      // loop over chips
        for(k=0;k<MAX_POINTS1;k++){  // loop over point1 (typically charge)

          if(np2[i][j][k]>0){
            sprintf(gr_title,"Mod %d Chip %d vt50 vs %s",i,j,var2_name);
            gr_vt50[i][j][k] = new TGraph(np2[i][j][k],x_var2[i][j][k],y_vt50[i][j][k]);
            gr_vt50[i][j][k]->SetTitle(gr_title);
            gr_vt50[i][j][k]->SetLineColor(k+1);
            gr_vt50[i][j][k]->SetMarkerColor(k+1);

            sprintf(gr_title,"Mod %d Chip %d outnse vs %s",i,j,var2_name);
            gr_outnse[i][j][k] = new TGraph(np2[i][j][k],x_var2[i][j][k],y_outnse[i][j][k]);
            gr_outnse[i][j][k]->SetTitle(gr_title);
            gr_outnse[i][j][k]->SetLineColor(k+1);
            gr_outnse[i][j][k]->SetMarkerColor(k+1);

            sprintf(gr_title,"Mod %d Chip %d goodfits vs %s",i,j,var2_name);
            gr_goodfits[i][j][k] = new TGraph(np2[i][j][k],x_var2[i][j][k],y_goodfits[i][j][k]);
            gr_goodfits[i][j][k]->SetTitle(gr_title);
            gr_goodfits[i][j][k]->SetLineColor(k+1);
            gr_goodfits[i][j][k]->SetMarkerColor(k+1);

          }
        }

        // Now for each chip and each value of (delay) we form a response curve 
        // in order to evaluate the gain

        for(k=0;k<MAX_POINTS2;k++){  // loop over point2 (typically delay)
          if(np1[i][j][k]>0){
            sprintf(gr_title,"Mod %d Chip %d vt50 vs %s",i,j,var1_name);
            gr_rc[i][j][k] = new TGraph(np1[i][j][k],x_var1[i][j][k],y_rc[i][j][k]);
            gr_rc[i][j][k]->SetTitle(gr_title);
            gr_rc[i][j][k]->SetLineColor(k+1);
            gr_rc[i][j][k]->SetMarkerColor(k+1);

            TF1* rc = new TF1("rc","pol1",0,8);
            rc->SetParameter(0,70);
            rc->SetParameter(1,60);
            gr_rc[i][j][k]->Fit("rc","RQ");
            y_gain[i][j][k]=rc->GetParameter(1);
            y_offset[i][j][k]=rc->GetParameter(0);
          }
        }

        for(k=0;k<MAX_POINTS1;k++){  // loop over point1 (typically charge)
          if(np2[i][j][k]>0){
            for(l=0;l<MAX_POINTS2;l++){  // loop over point2 (typically delay)
              if(y_gain[i][j][l]>0){
                y_innse[i][j][k][l]= 6250.0 * y_outnse[i][j][k][l]/ y_gain[i][j][l]; 
              }
            }
            sprintf(gr_title,"Mod %d Chip %d Input Noise vs %s",i,j,var1_name);
            gr_innse[i][j][k] = new TGraph(np2[i][j][k],x_var2[i][j][k],y_innse[i][j][k]);
            gr_innse[i][j][k]->SetTitle(gr_title);
            gr_innse[i][j][k]->SetLineColor(k+1);
            gr_innse[i][j][k]->SetMarkerColor(k+1);
          }
        }
        

        // And now form the graphs of gain and output noise

        if(np2[i][j][0]>0){
          sprintf(gr_title,"Mod %d Chip %d Gain vs %s",i,j,var2_name);
          gr_gain[i][j] = new TGraph(np2[i][j][0],x_var2[i][j][0],y_gain[i][j]);
          gr_gain[i][j]->SetTitle(gr_title);
          gr_gain[i][j]->SetLineColor(kRed);
          gr_gain[i][j]->SetMarkerColor(kRed);

          sprintf(gr_title,"Mod %d Chip %d Offset vs %s",i,j,var2_name);
          gr_offset[i][j] = new TGraph(np2[i][j][0],x_var2[i][j][0],y_offset[i][j]);
          gr_offset[i][j]->SetTitle(gr_title);
          gr_offset[i][j]->SetLineColor(kRed);
          gr_offset[i][j]->SetMarkerColor(kRed);
        }
      }
    }
  }


  // Open the postscript file if requested

  if(postscript){
    if(RAL) sprintf(ps_fname,"M:\\sctvar\\data\\stnt_%d_%d.ps",run,sno);
    else    sprintf(ps_fname,"D:\\sctvar\\data\\stnt_%d_%d.ps",run,sno);

    TPostScript *ps = new TPostScript(ps_fname,112);
  }


 // Open and draw the one and only canvas

  TCanvas* c1= new TCanvas ("c2","stan_ntplot",1100,750);
  pad_text   = new TPad("pad_text",  "Text",   0.00,0.90,1.00,1.00,33);
  pad_plot   = new TPad("pad_plot",  "Plots",  0.00,0.00,1.00,0.90,33);

  pad_text->Draw();
  pad_plot->Draw();
  pad_plot->Divide(6,3,0,0);

  c2->Update();

  for(i=1;i<19;i++){
    pad_plot->cd(i);
    gPad->SetLeftMargin(0.15);
    gPad->SetBottomMargin(0.15);
  }

  TH2F* h_vt50[MAX_MODULES][MAX_CHIPS];
  TH2F* h_outnse[MAX_MODULES][MAX_CHIPS];
  TH2F* h_goodfits[MAX_MODULES][MAX_CHIPS];

  TH2F* h_gain[MAX_MODULES][MAX_CHIPS];
  TH2F* h_offset[MAX_MODULES][MAX_CHIPS];
  TH2F* h_innse[MAX_MODULES][MAX_CHIPS];

  TObjString *time;
  TObjString *place;
  TObjString *mname;

  Int_t page = 0;

  for(i=0;i<MAX_MODULES;i++){  // loop over modules
    if(module_active[i]){

      for(link=0;link<2;link++){   // loop over links

        if(postscript){
          ps->NewPage();
          page++;
        }

        pad_text->cd();
        pad_text->Clear();

        if(strun){
          sprintf(gr_title,"ModuleName;%d",i+1);
          time  = (TObjString*)strun->Get("Time");
          place = (TObjString*)strun->Get("Place");
          mname = (TObjString*)strun->Get(gr_title);
        }

        TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
        t1->SetFillColor(10);
        t1->SetTextAlign(13);
        t1->SetTextSize(0.3);
        t1->SetTextColor(4);
        sprintf(line,"ATLAS SCT Module Response vs %s",var2_name);
        if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
        if(place) sprintf(line,"%s  -  %s",line,place->GetName());
        if(mname) sprintf(line,"%s  -  Module %s",line,mname->GetName());
        t1->AddText(line);
        sprintf(line,"Page %d   Run %d   Start Scan %d   Module %d   Stream %d", 
                page,run, sno,i,link);
        t1->AddText(line);
        t1->Draw();



        for(j=(link*6);j<((link+1)*6);j++){ // loop over chips

          pad_plot->cd((j%6)+1);
          sprintf(h_name, "h_vt50_%d_%d",i,j,var2_name);
          sprintf(h_title,"Chip %d vt50 vs %s",j,var2_name);
          h_vt50[i][j] = new TH2F(h_name,h_title,2,lo_var2,hi_var2,2,0,640);
          h_vt50[i][j]->SetXTitle(var2_name);
          h_vt50[i][j]->SetYTitle("vt50 (mV)");
          h_vt50[i][j]->SetTitleOffset(1.5,"X");
          h_vt50[i][j]->SetTitleOffset(1.5,"Y");
          h_vt50[i][j]->Draw();

          for(k=0;k<MAX_POINTS1;k++){  // loop over point1
            if(np2[i][j][k]>0){
              gr_vt50[i][j][k]->Draw("LP");
            }
          }

          pad_plot->cd((j%6)+7);
          sprintf(h_name, "h_outnse_%d_%d",i,j,var2_name);
          sprintf(h_title,"Chip %d Output Noise vs %s",j,var2_name);
          h_outnse[i][j] = new TH2F(h_name,h_title,2,lo_var2,hi_var2,2,0,30);
          h_outnse[i][j]->SetXTitle(var2_name);
          h_outnse[i][j]->SetYTitle("Output Noise (mV)");
          h_outnse[i][j]->SetTitleOffset(1.5,"X");
          h_outnse[i][j]->SetTitleOffset(1.5,"Y");
          h_outnse[i][j]->Draw(); 

          for(k=0;k<MAX_POINTS1;k++){  // loop over point1
            if(np2[i][j][k]>0){
              gr_outnse[i][j][k]->Draw("LP");
            }
          }

          pad_plot->cd((j%6)+13);
          sprintf(h_name, "h_goodfits_%d_%d",i,j,var2_name);
          sprintf(h_title,"Chip %d n(Good Fits) vs %s",j,var2_name);
          h_goodfits[i][j] = new TH2F(h_name,h_title,2,lo_var2,hi_var2,2,0,130);
          h_goodfits[i][j]->SetXTitle(var2_name);
          h_goodfits[i][j]->SetYTitle("n(Good Fits)");
          h_goodfits[i][j]->SetTitleOffset(1.5,"X");
          h_goodfits[i][j]->SetTitleOffset(1.5,"Y");
          h_goodfits[i][j]->Draw(); 

          for(k=0;k<MAX_POINTS1;k++){  // loop over point1
            if(np2[i][j][k]>0){
              gr_goodfits[i][j][k]->Draw("LP");
            }
          }

        }

        c2->Update();

        if(dowait){
          printf("Enter a string to continue\n");
          scanf("%s",line);
        }

        if(postscript){
          ps->NewPage();
          page++;
        }

        pad_text->cd();
        pad_text->Clear();

        TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
        t1->SetFillColor(10);
        t1->SetTextAlign(13);
        t1->SetTextSize(0.3);
        t1->SetTextColor(4);
        sprintf(line,"ATLAS SCT Module Response vs %s",var2_name);
        if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
        if(place) sprintf(line,"%s  -  %s",line,place->GetName());
        if(mname) sprintf(line,"%s  -  Module %s",line,mname->GetName());
        t1->AddText(line);
        sprintf(line,"Page %d   Run %d   Start Scan %d   Module %d   Stream %d", 
                page,run, sno,i,link);
        t1->AddText(line);
        t1->Draw();

        for(j=(link*6);j<((link+1)*6);j++){ // loop over chips

          pad_plot->cd((j%6)+1);
          sprintf(h_name, "h_gain_%d_%d",i,j,var2_name);
          sprintf(h_title,"Chip %d Gain vs %s",j,var2_name);
          h_gain[i][j] = new TH2F(h_name,h_title,2,lo_var2,hi_var2,2,0,100);
          h_gain[i][j]->SetXTitle(var2_name);
          h_gain[i][j]->SetYTitle("Gain (mV/fC)");
          h_gain[i][j]->SetTitleOffset(1.5,"X");
          h_gain[i][j]->SetTitleOffset(1.5,"Y");
          h_gain[i][j]->Draw();
  
          if(np2[i][j][0]>0){
            gr_gain[i][j]->Draw("LP");
          }

          pad_plot->cd((j%6)+7);
          sprintf(h_name, "h_offset_%d_%d",i,j,var2_name);
          sprintf(h_title,"Chip %d Offset vs %s",j,var2_name);
          h_offset[i][j] = new TH2F(h_name,h_title,2,lo_var2,hi_var2,2,0,100);
          h_offset[i][j]->SetXTitle(var2_name);
          h_offset[i][j]->SetYTitle("Extrapolated Offset (mV)");
          h_offset[i][j]->SetTitleOffset(1.5,"X");
          h_offset[i][j]->SetTitleOffset(1.5,"Y");
          h_offset[i][j]->Draw(); 
 
          if(np2[i][j][0]>0){
            gr_offset[i][j]->Draw("LP");
          }
 
          pad_plot->cd((j%6)+13);
          sprintf(h_name, "h__%d_%d",i,j,var2_name);
          sprintf(h_title,"Chip %d Input Noise vs %s",j,var2_name);
          h_innse[i][j] = new TH2F(h_name,h_title,2,lo_var2,hi_var2,2,0,2500);
          h_innse[i][j]->SetXTitle(var2_name);
          h_innse[i][j]->SetYTitle("Input Noise (ENC)");
          h_innse[i][j]->SetTitleOffset(1.5,"X");
          h_innse[i][j]->SetTitleOffset(1.5,"Y");
          h_innse[i][j]->Draw(); 
  
          for(k=0;k<MAX_POINTS1;k++){  // loop over point1
            if(np2[i][j][k]>0){
              gr_innse[i][j][k]->Draw("LP");
            }
          }
        }

        c2->Update();

        if(dowait){
          printf("Enter a string to continue\n");
          scanf("%s",line);
        }

      } // end for(link)
    } // end if(module_active)
  }

  if(time) delete time;
  if(place) delete place;
  if(mname) delete mname;

  if(postscript) {
    ps->Close();
    delete ps;
  }
  if(strun) {
    strun->Close();
    delete strun;
  }

  return;
}

void stan_ntplot(int run, int sno, char* var2_name, char* var1_name, short dowait){

  stan_ntplot(run,sno,var2_name,var1_name,dowait,1);
}

void stan_ntplot(int run, int sno, char* var2_name, char* var1_name){

  stan_ntplot(run,sno,var2_name,var1_name,0,1);
}

void stan_ntplot(){

  printf("Possible Uses of stan_ntplot:\n\n");
  printf(".x stan_ntplot.cpp(run,start_scan,var2_name,var1_name)\n");
  printf(".x stan_ntplot.cpp(run,start_scan,var2_name,var1_name,dowait)\n");
  printf(".x stan_ntplot.cpp(run,start_scan,var2_name,var1_name,dowait,postscript)\n");
}
