Int_t get_val(char* line, Int_t index, double* val){

  // convert next element of data row "line" to type double
  // (CINT sscanf is limited to <43 elements, hence this approach)

  char substring[20];

  sscanf(line+=index,"%s ",substring);
  sscanf(substring,"%lf",val);
  return 1 + strlen(substring);
}

Int_t dotrim(Float_t vt50[1536][16], Short_t trim[1536], Float_t target, Short_t ntrim[13]){
  // Revised Version 0.11 BJG 30.04.02 - Fix some memory leaks

  Int_t ch,i;

  Short_t dac;
  Short_t no_trim  = 0x10;
  Short_t max_trim = 15;
  Short_t min_trim =  0;

  Float_t meantrim[13];
  Short_t sum[13];

  for(i=0;i<13;i++){
    ntrim[i]=0;
    sum[i]=0;
    meantrim[i]=0;
  }

  for(ch=0;ch<1536;ch++){

    // set initial value of trim to be 16
    // not a valid value, but clearly indicates channel untrimmed

    trim[ch]=0x10;

    dac=max_trim;
    while(dac>min_trim){
     if((vt50[ch][dac]>=target)&&(vt50[ch][dac-1]<target)
                               &&(vt50[ch][dac-1]>0)){
       if((vt50[ch][dac]-target)<(target-vt50[ch][dac-1])){
         trim[ch]=dac; 
       }else{
         trim[ch]=dac-1;
       }
       dac=0;
     } else dac--;  
    }

    // action if channel untrimmed
    // leave bit 4 set to denote untrimmed
    // but set bits 0-3 to best choice between max_trim and min_trim
    if (trim[ch]==no_trim) {
   
       if(abs(vt50[ch][max_trim]-target)<abs(target-vt50[ch][min_trim])){
         trim[ch]=no_trim+max_trim; 
       }else{
         trim[ch]=no_trim+min_trim;
       }
    
    } else {

      ntrim[ch/128]++;
      sum[ch/128]+=trim[ch];
    }
  }

  for(i=0;i<12;i++){
    if(ntrim[i])meantrim[i]= sum[i]/ntrim[i];
    ntrim[12]+=ntrim[i];
    sum[12]+=sum[i];
    meantrim[12]+=meantrim[i];
  }


  printf("Target %3.0f: Mean %2.1f: Fail %4d: ",target,meantrim[12]/12,1536-ntrim[12]);
  printf("%d %d %d %d %d %d %d %d %d %d %d %d\n",
        128-ntrim[0],128-ntrim[1],128-ntrim[2],128-ntrim[3],
        128-ntrim[4],128-ntrim[5],128-ntrim[6],128-ntrim[7],
        128-ntrim[8],128-ntrim[9],128-ntrim[10],128-ntrim[11]);
//  printf("%2.2f %2.2f %2.2f %2.2f %2.2f %2.2f %2.2f %2.2f %2.2f %2.2f %2.2f %2.2f\n",
//        meantrim[0],meantrim[1],meantrim[2],meantrim[3],
//        meantrim[4],meantrim[5],meantrim[6],meantrim[7],
//        meantrim[8],meantrim[9],meantrim[10],meantrim[11]);
  return ntrim[12];
}


void stan_trim2(Int_t run, Int_t start_scan, Int_t moi, Int_t loi_in, Float_t target){

  // Second go at a macro to set trimdac levels for ABCDT
  // Version 0.3, PWP, working 25.10.99
  // Version 0.4, LE/PWP, 08.03.00

  char f_name[256], out_fname[256] ,line[1024],string[32];
  double mean;
  Float_t vt50[1536][16];
  Short_t trim[1536];
  Int_t index,i,j,loi;
  Int_t ntrimmed[60];
  Short_t ntrim[61][13];

  // Display options: by LE 7/3-00
  Int_t postscript = 1;  //Save output to postscript
  Int_t showdisplay = 1;      //Show trimming performance in canvas.

  // Global style variations

  gStyle->SetPaperSize(20,26); // I find that the postscript output
                              // is mangled without this line - 
                              // although this size (A4) should in
                              // principle be the default...
 //Global style options
    gStyle->SetMarkerStyle(20);
    gStyle->SetMarkerSize(0.4);    
    gStyle->SetMarkerColor(1);    
    gStyle->SetOptStat(11);    

  // Locality options now moved to stan_util.cpp

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");

  //  initialise variables

  for(i=0;i<1536;i++){
    for(j=0;j<16;j++) vt50[i][j]=0;
  }


  // load data

  for(j=0;j<16;j++){
    for(loi=0;loi<2;loi++){
      
      if (RAL) sprintf(f_name,"M:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run,(start_scan+j),moi,loi);
      else     sprintf(f_name,"D:\\sctvar\\data\\strun%d_%d_%d_%d.fit",run,(start_scan+j),moi,loi);
    //else     sprintf(f_name,"/home/eklund/sctdaq/data/strun%d_%d_%d_%d.fit",run,(start_scan+j),moi,loi);
      FILE* fitfile = fopen(f_name,"r");

      if(fitfile==NULL){
        printf("Could not open fit file %s\n",f_name);
      }else{
        printf("File %s opened for reading\n",f_name);

        if(loi) i=768; else i=0;

        fgets(line,1024,fitfile);
        while(fgets(line,1024,fitfile) != NULL){
          index = 10;
          get_val(line,index,&mean);
          vt50[i][j]=(float)mean;
          i++;
        }
      fclose(fitfile);
      }
    }
  }

  // process data //Some minor changes here LE 7/3-00

  if(target==-1){ // no target value specified
    Int_t best = 0;
    Int_t show_trimmable = 1;
    for(i=0; i<60; i++){
      target = 5*i;
      ntrimmed[i]=dotrim(vt50, trim, target,ntrim[i]);
      if(ntrimmed[i]>best){
        best = ntrimmed[i];
        Float_t best_target = target;
      }
    }
    printf("Best target found to be %3.2f\n",best_target);
//    for(i=0;i<1536;i++){
//      trim[i]=15;
//    }
    dotrim(vt50, trim, best_target,ntrim[60]);

  }else{
    Int_t show_trimmable = 0;
    Int_t trimmable = dotrim(vt50, trim, target,ntrim[60]);
  }

  // open trim file

  if (RAL) sprintf(f_name,"M:\\sctvar\\data\\stmod_%d.trim",moi);
  else     sprintf(f_name,"D:\\sctvar\\data\\stmod_%d.trim",moi);
//else     sprintf(f_name,"/home/eklund/sctdaq/data/stmod_%d.trim",moi);
  FILE* trimfile = fopen(f_name,"w");
  if(trimfile==NULL){
      printf("Could not open trim file %s\n",f_name);
  } else {
    for(i=0;i<1536;i++){
      fprintf(trimfile,"%f\n",((float)(trim[i]&0x0f))/15);
    }
  fclose(trimfile);
  }

  // open channel mask file

  if (RAL) sprintf(f_name,"M:\\sctvar\\data\\stmod_%d.mask",moi);
  else     sprintf(f_name,"D:\\sctvar\\data\\stmod_%d.mask",moi);
//else     sprintf(f_name,"/home/eklund/sctdaq/data/stmod_%d.mask",moi);
  FILE* trimfile = fopen(f_name,"w");
  if(trimfile==NULL){
      printf("Could not open mask file %s\n",f_name);
  } else {
    for(i=0;i<1536;i++){
      if(trim[i]&0x10)fprintf(trimfile,"%d\n", i);
    }
  fclose(trimfile);
  }

//*************** Graphical output added by LE 6/3-99 **************
  if (showdisplay) {

    if(RAL) sprintf(f_name,"M:\\sctvar\\data\\strun%d_%d.root",run,start_scan);
    else    sprintf(f_name,"D:\\sctvar\\data\\strun%d_%d.root",run,start_scan);

    TFile *strun = new TFile(f_name);

    sprintf(f_name,"ModuleName;%d",moi+1);
    TObjString *time  = (TObjString*)strun->Get("Time");
    TObjString *place = (TObjString*)strun->Get("Place");
    TObjString *mname = (TObjString*)strun->Get(f_name);

// Variables added by LE
    Float_t trimdac[] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
    Float_t tempmean[16],targetarray[60],trimmedarray[60];
    char h_title[40], s[256];
    Int_t bin, ch_0;

// Before we draw the main canvas, open the postscript file
// if the relevant option has been selected 

    if(postscript){
      if(RAL) sprintf(out_fname,"M:\\sctvar\\data\\strun%d_%d_%d_trim.ps",run,moi,loi_in);
      else    sprintf(out_fname,"D:\\sctvar\\data\\strun%d_%d_%d_trim.ps",run,moi,loi_in);
  //  else    sprintf(out_fname,"/home/eklund/sctdaq/data/strun%d_%d_%d_trim.ps",run,moi,loi_in);
      
      TPostScript *ps = new TPostScript(out_fname,112);
      printf("Generating postscript file %s\n",out_fname);
      ps->NewPage();
    }
    
    // Open and draw the canvas
    TCanvas* c1=new TCanvas ("c1","stan_trims",1100,750);
    TPad* pad_t0 = new TPad("pad_t0","Title", 0.01,0.91,0.99,0.99,33);
    pad_t0->Draw();
    
    //Add title to canvas
    TPaveText *ttitle = new TPaveText(0.01,0.05,0.60,0.95);
    ttitle->SetFillColor(10);  
    ttitle->SetTextAlign(13);
    ttitle->SetTextSize(0.3);
    ttitle->SetTextColor(4);
    
    sprintf(s,"ATLAS SCT Module Test");
    if(time)  sprintf(s,"%s  -  %s",s,time->GetName());
    ttitle->AddText(s);
    if(place) sprintf(s,"%s",place->GetName());
    if(mname) sprintf(s,"%s  -  Module %s",s,mname->GetName());
    ttitle->AddText(s);
    sprintf(s,"Run %d Trim Scans Module %d Link %d",run,moi,loi_in);
    ttitle->AddText(s);
    pad_t0->cd();
    ttitle->Draw();
  
    //Display top or bottom side?
    if (loi_in) { 
      Int_t first = 6;
      Int_t last = 12;
    } else {
      Int_t first = 0;
      Int_t last = 6;
    }

    if (show_trimmable) { // ********* If no target is given
      //Add text with trim report
      pad_t0->cd();
      TPaveText *ttext = new TPaveText(0.61,0.05,0.99,0.95);
      ttext->SetFillColor(10);  
      ttext->SetTextAlign(13);
      ttext->SetTextSize(0.3);
      ttext->SetTextColor(4);
      ttext->Draw();
      sprintf(s,"Best target value found to be %3.0f mV",best_target);
      ttext->AddText(s);
      sprintf(s,"This gives %4.0f trimmable channels", best);
      ttext->AddText(s);
      ttext->Draw();
      
      //Create pads for output
      c1->cd();
      TPad* pad_tc =  new TPad("pad_tc","Trim Characteristics", 0.01,0.01,0.60,0.90,33);
      TPad* pad_no =  new TPad("pad_no","Trimmed Channels",0.61,0.60,0.99,0.90,33);
      TPad* pad_tr =  new TPad("pad_tr","Trim Settings", 0.61,0.30,0.99,0.60,33);      
      TPad* pad_ucd = new TPad("pad_ucd","Untrimmed channel distribution", 0.61,0.01,0.99,0.30,33);      

      pad_tc->Draw();
      pad_tc->Divide(2,3,0,0); 
      
      pad_no->Draw();
      pad_tr->Draw();
      pad_ucd->Draw();
      
      //Text with trim data
      //Plot number of trimmable channels
      for(i=0; i<60; i++){
        targetarray[i] = 5*i;
	trimmedarray[i] = 0;
        for (int ichip=first;ichip<last;ichip++) { 
	  trimmedarray[i] += ntrim[i][ichip]; //Sum trimmed channels on this side.
	}
      }
      pad_no->cd();
      TGraph* gr_no = new TGraph(60,targetarray,trimmedarray);
      gr_no->Draw("ALP");      
      sprintf(s,"Trimmable channels, chip %d to %d",first,(last-1));
      gr_no->SetTitle(s);
      gr_no->GetHistogram()->SetXTitle("Target value (mV)");
      gr_no->GetHistogram()->SetYTitle("# channels");
      
    } else { // ********* If a target is given

      pad_t0->cd();
      TPaveText *ttext = new TPaveText(0.51,0.05,0.99,0.95);
      ttext->SetFillColor(10);  
      ttext->SetTextAlign(13);
      ttext->SetTextSize(0.4);
      ttext->SetTextColor(4);
      ttext->Draw();
      sprintf(s,"Given target value %3.0f mV",target);
      ttext->AddText(s);
      sprintf(s,"Gives %4.0f trimmable channels", trimmable);
      ttext->AddText(s);
      ttext->Draw();
      
      //Create pads for output
      c1->cd();
      TPad* pad_tc =  new TPad("pad_tc","Trim Characteristics", 0.01,0.01,0.60,0.90,33);
      TPad* pad_tr =  new TPad("pad_tr","Trim Settings", 0.61,0.45,0.99,0.90,33);      
      TPad* pad_ucd = new TPad("pad_ucd","Untrimmed channel distribution", 0.61,0.01,0.99,0.45,33);      

      pad_tc->Draw();
      pad_tc->Divide(2,3,0,0); 

      pad_tr->Draw();
      pad_ucd->Draw();
    }

    //Plot untrimmable channel distribution
     TH1F* h_tr  = new TH1F("h_tr", "Trim Setting - Trimmable Channels",  768,0.5,768.5);
     TH1F* h_utr = new TH1F("h_utr","Trim Setting - Untrimmable Channels",768,0.5,768.5);
     if(loi_in){
       for (i=768;i<1536;i++) {
         if (trim[i]&0x10) h_utr->Fill(i-768,(float)(trim[i]&0x0f)+0.01);
         else  h_tr->Fill(i-768,(float)(trim[i]&0x0f)+0.01);
       }
     } else {
       for (i=0;i<768;i++) {
         if (trim[i]&0x10) h_utr->Fill(i,(float)(trim[i]&0x0f)+0.01);
         else h_tr->Fill(i,(float)(trim[i]&0x0f)+0.01);
       }
     }
     pad_tr->cd();
     h_tr->Draw("P");
     h_tr->SetMaximum(16.05);
     h_tr->SetMinimum(-0.05);
     h_tr->SetXTitle("Channel Number");
     h_tr->SetYTitle("TrimDAC value");
     
     pad_ucd->cd();
     h_utr->Draw("P");
     h_utr->SetMaximum(16.05);
     h_utr->SetMinimum(-0.05);
     h_utr->SetXTitle("Channel Number");
     h_utr->SetYTitle("untrimmed");
 
   

    //Plot trim characteristics
    TGraph* gr[768];

    for(Int_t ichip=first;ichip<last;ichip++) {

      pad_tc->cd((ichip%6)+1);
      Int_t gr_ch = (ichip%6)*128;
      Int_t vt_ch =  ichip   *128;

      //Plot first channel
      gr[gr_ch] = new TGraph(16,trimdac,vt50[vt_ch]);
      sprintf(h_title,"TrimDAC characteristics, chip %d",ichip);
      gr[gr_ch]->SetTitle(h_title);
      gr[gr_ch]->SetMaximum(350);
      gr[gr_ch]->SetMinimum(0);
      gr[gr_ch]->SetLineColor(0);
      gr[gr_ch]->SetMarkerColor(0);
      gr[gr_ch]->Draw("ALP");
      gr[gr_ch]->GetHistogram()->SetXTitle("TrimDAC value");
      gr[gr_ch]->GetHistogram()->SetYTitle("50%-points (mV)");
      c1->Update();
      
      for(i=1;i<128;i++) { //The rest of the channels
        gr_ch++;
        vt_ch++;
        gr[gr_ch] = new TGraph(16,trimdac,vt50[vt_ch]);
        gr[gr_ch]->SetLineColor(i);
        gr[gr_ch]->SetMarkerColor(i);
        gr[gr_ch]->Draw("LP");
      }
      c1->Update();
    }  // end chip 

    if(postscript) {
      ps->Close();
      delete ps;
    }
  }  // end trimdac plots

  if(time) delete time;
  if(place) delete place;
  if(mname) delete mname;
}  // end if showdisplay

void stan_trim2(Int_t run, Int_t start_scan, Int_t moi){
  stan_trim2(run,start_scan,moi,0,-1);
}

void stan_trim2(Int_t run, Int_t start_scan, Int_t moi, Int_t loi_in){
  stan_trim2(run,start_scan,moi,loi_in,-1);
}


