//
// stan_util.cpp
// PWP 11.11.99
// revised PWP 24.07.00
// definitions

#define RAL 0;

#define BAD_TEST     0xfffb  // all except CH_UNDER - still up for debate
#define DEAD_TEST    0x0003  // CH_DEAD and CH_STUCK
#define DODGY_TEST   0xfffc  // all except CH_DEAD and CH_STUCK

#define CH_DEAD      0x0001
#define CH_STUCK     0x0002
#define CH_UNDER     0x0004
#define CH_OVER      0x0008
#define FIT_UNDER    0x0010
#define FIT_OVER     0x0020
#define SIG_UNDER    0x0040
#define SIG_OVER     0x0080
#define GAIN_UNDER   0x0100
#define GAIN_OVER    0x0200
#define OFFSET_UNDER 0x0400
#define OFFSET_OVER  0x0800
#define NOISE_UNDER  0x1000
#define NOISE_OVER   0x2000
#define MEAN_ERROR   0x4000
#define SIG_ERROR    0x2000

//
// conversion from ascii to var_type and vice-versa
//

Int_t match_stub(char* stub){

  // ascii stub -> integer

  if (!strncmp(stub,"vthr",4)) return 1;
  if (!strncmp(stub,"vcal",4)) return 2;
  if (!strncmp(stub,"strb",4)) return 3;
  if (!strncmp(stub,"preamp",6)) return 4;
  if (!strncmp(stub,"shaper",6)) return 5;
  if (!strncmp(stub,"trim",  4)) return 6;

  if (!strncmp(stub,"strm_del", 8)) return 20;
  if (!strncmp(stub,"strm_del0",9)) return 21;
  if (!strncmp(stub,"strm_del1",9)) return 22;
  if (!strncmp(stub,"clk_duty", 8)) return 23;
  if (!strncmp(stub,"clk_phase",9)) return 24;
  if (!strncmp(stub,"trig_del", 8)) return 25;

  if (!strncmp(stub,"vdet",4)) return 30;
  if (!strncmp(stub,"vcc", 3)) return 31;
  if (!strncmp(stub,"vi1", 3)) return 32;
  if (!strncmp(stub,"vdd", 3)) return 33;

  if (!strncmp(stub,"vt50",4)) return 101;
  if (!strncmp(stub,"vc50",4)) return 102;

  return 0; // unknown
}

void match_var(Int_t type, char* stub){

  // integer - > ascii stub

  switch(type){
    case 1:
      sprintf(stub,"vthr");
      break;
    case 2:
      sprintf(stub,"vcal");
      break;
    case 3:
      sprintf(stub,"strb_del");
      break;
    case 4:
      sprintf(stub,"preamp");
      break;
    case 5:
      sprintf(stub,"shaper");
      break;
    case 6:
      sprintf(stub,"trim");
      break;
    case 20:
      sprintf(stub,"strm_del");
      break;
    case 21:
      sprintf(stub,"strm_del0");
      break;
    case 22:
      sprintf(stub,"strm_del1");
      break;
    case 23:
      sprintf(stub,"clk_duty");
      break;
    case 24:
      sprintf(stub,"clk_phase");
      break;
    case 25:
      sprintf(stub,"trig_del");
      break;
    case 30:
      sprintf(stub,"vdet");
      break;
    case 31:
      sprintf(stub,"vcc");
      break;
    case 32:
      sprintf(stub,"vi1");
      break;
    case 33:
      sprintf(stub,"vdd");
      break;
    case 101:
      sprintf(stub,"vt50");
      break;
    case 102:
      sprintf(stub,"vc50");
      break;
    default:
      sprintf(stub,"vartype%d",type);
      break;
   } 
}

//
// fit functions
//
// PWP 11.10.99 - added expo and "grillo" functions
// PWP 11.09.00 - protection against divide by zero errors
//                to erf, erfc and grillo functions
//

Double_t erf(Double_t *x, Double_t *par){
  Double_t xx, y;

  if(par[2] == 0){
    xx = 0;
  }else{
    xx = ( (par[1]-x[0]) / par[2]) / 1.414213562; 
  }

  y  = (par[0]/2) * ( 1 - TMath::Erf(xx) );

  return y;
}

Double_t erfc(Double_t *x, Double_t *par){
  Double_t xx, y;

  if(par[2] == 0){
    xx = 0;
  }else{
    xx = ( (par[1]-x[0]) / par[2]) / 1.414213562; 
  }

  y  = (par[0]/2) * ( 1 + TMath::Erf(xx) );

  return y;
}

Double_t st_exp(Double_t *x, Double_t *par){

  return par[2] + par[0]/(1 + TMath::Exp(-x[0]/par[1]));
}

Double_t gaus(Double_t *x, Double_t *par){

  return par[0] * TMath::Gaus(x[0],par[1],par[2]); 
}

Double_t grillo(Double_t *x, Double_t *par){

  Double_t xx;

  if((par[1]*x[0]) == 0){
    xx = 0;
  }else{
    xx = par[2]/(par[1]*x[0]);
  }

  return par[0] + par[2]/TMath::Sqrt(1+(xx*xx)); 
}

Double_t pol2(Double_t *x, Double_t *par){

  return par[0] + x[0]*(par[1]+(par[2]*x[0]));
}

Double_t tophat(Double_t *x, Double_t *par){

  if (x[0]< (par[1]-par[2])) return 0;
  if (x[0]> (par[1]+par[2])) return 0;
  return par[0];
}

//
// miscellaneous
//
  
void stan_styles(){

  gStyle.SetPaperSize(20,26); // I find that the postscript output
                              // is mangled without this line - 
                              // although this size (A4) should in
                              // principle be the default...
  gStyle.SetMarkerStyle(21);
  gStyle.SetMarkerSize(0.3);
  gStyle.SetOptFit(0);
  gStyle.SetOptStat(10);
  gStyle.SetTitleW(0.6);
  gStyle.SetHistFillColor(46);
}

Float_t find_level(TH1* h_in, Float_t fraction){

  Int_t bin,i;
  Float_t y = h_in->GetMaximum()*fraction;

  // NB. bins "0" and "n" are the overflow bins, hence the
  // initial test must use bins "1" and "n-1" PWP 07.09.99

  if(h_in->GetBinContent(1)>h_in->GetBinContent(h_in->GetNbinsX()-1)){
    for(i=1; i<h_in->GetNbinsX(); i++){
      if(h_in->GetBinContent(i)>y) bin = i;
    }
  }else{
    for(i=1; i<h_in->GetNbinsX(); i++){
      if(h_in->GetBinContent(i)<y) bin = i;
    }
  }

  Float_t x1 = h_in->GetBinCenter(bin-1);
  Float_t x2 = h_in->GetBinCenter(bin+1);
  Float_t y1 = h_in->GetBinContent(bin-1);
  Float_t y2 = h_in->GetBinContent(bin+1);

  Float_t dy = y2-y1;
  if(dy==0) dy=1; // protection against divide by 0

  return x1 + ((y-y1)*(x2-x1)/(dy));
}

Float_t find_level2(TH1* h_in, Float_t fraction, Int_t last){

  Int_t bin,i;
  Float_t y = h_in->GetMaximum()*fraction;

  if(last){         // find last bin which is > y
    for(i=1; i<h_in->GetNbinsX(); i++){
      if(h_in->GetBinContent(i)>y) bin = i;
    }
  }else{             // find first bin which is > y
    for(i=h_in->GetNbinsX(); i>0; i--){
      if(h_in->GetBinContent(i)>y) bin = i;
    }
  }

  Float_t x1 = h_in->GetBinCenter(bin-1);
  Float_t x2 = h_in->GetBinCenter(bin+1);
  Float_t y1 = h_in->GetBinContent(bin-1);
  Float_t y2 = h_in->GetBinContent(bin+1);

  Float_t dy = y2-y1;
  if(dy==0) dy=1; // protection against divide by 0

  return x1 + ((y-y1)*(x2-x1)/(dy));
}

Float_t find_bin2(TH1* h_in, Float_t fraction, Int_t last, Int_t offset){

  Int_t bin,i;
  Float_t y = h_in->GetMaximum()*fraction;

  if(last){         // find last bin which is > y
    bin = h_in->GetNbinsX();
    for(i=1; i<h_in->GetNbinsX(); i++){
      if(h_in->GetBinContent(i)>y) bin = i;
    }
  }else{             // find first bin which is > y
    bin = 1;
    for(i=h_in->GetNbinsX(); i>0; i--){
      if(h_in->GetBinContent(i)>y) bin = i;
    }
  }

  return h_in->GetBinCenter(bin+offset);
}

Float_t find_bin2(TH1* h_in, Float_t fraction, Int_t last){

  find_bin2(h_in, fraction, last, 0);
}
