void BypassAnalysis(Int_t run, Int_t scan, Int_t moi, Int_t n_configs){

  // analysis of BypassTest
  // version 0.1 PWP 14.11.00
  // version 0.2 LE  19.02.00 - including BYPASS 13 and 14
  //                          - stricter definition of a good chip
  // version 0.3 PWP 25.05.01 - Use parameter.h and Toolbox.cpp
  // Version 0.31 BJG 30.04.02 - Fix some memory leaks, syntax errors
  //
  // Analysis of data:
  // Every channel is classified as 
  //   - Good (98-102% occupancy)
  //   - Dead (0% occupancy)
  //   - Unknown (otherwise)
  // Chips with more than 120 good channels are classified as good, 
  // chips with more than 120 dead channels are classified as dead,
  // otherwise, chip is in unknown state.
  //
  // For a read-out configuration to be classified as good, the pattern
  // of good and dead chip has to match the configuration. No chips  
  // classified as unknown are allowed.

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t module, link, chip, chan, i;
  char f_name[128], h_name[64];
  char mname[MAX_MODULES][32];

  Short_t chip_dead[MAX_LINKS][MAX_CHIPS];
  Short_t chip_good[MAX_LINKS][MAX_CHIPS];
  Short_t chip_error[MAX_LINKS][MAX_CHIPS];
  Short_t chip_status[MAX_CONFIGS][MAX_LINKS][MAX_CHIPS];

  Short_t desired_result[MAX_CONFIGS][MAX_LINKS][MAX_CHIPS] =
     { 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0,
       1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0,
       1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0,
       1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0,
       1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
       1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 0,
       1, 1, 0, 1, 0, 0, 1, 1, 0, 1, 0, 0,
       1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0,
       1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1,
       1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1,
       0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
       1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
       1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
       1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 
       1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0 };


  Short_t module_status[MAX_CONFIGS];
  Short_t n_bad_chips, n_good_chips, n_missing_chips, overall_status;

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");
  gROOT->LoadMacro("ToolBox.cpp");

  // 
  // Ensure that the module name array is cleared to start
  //

  for (module=0; module<MAX_MODULES; module++){
    sprintf(mname[module]," ");
  }


  // sanity checks

  printf("BypassAnalysis called to test %d configurations\n",n_configs);
  if(n_configs<11){
    printf("*** Minimum for barrrel hybrid/module test is 11, using this instead");
    n_configs=11;
  }

  if(n_configs>15){
    printf("*** Maximum for forward hybrid/module test is 15, using this instead");
    n_configs=15;
  }

  // 
  // Load and process the data
  //

  sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan);
  TFile *strun = new TFile(f_name);

  if(strun==0){
    printf("Failed to open file %s - EXITING!!\n",f_name);
    return;
  }

  //
  //  loop over modules
  //

  Int_t start = 0;
  Int_t end   = 0;

  if(moi==-1){
    start = 0;
    end   = MAX_MODULES;

  } else {
    start = moi;
    end   = moi+1;
  }

  for (module=start; module<end; module++){

    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        chip_dead[link][chip]=0;
        chip_good[link][chip]=0;
        for(i=0;i<MAX_CONFIGS;i++){
          chip_status[i][link][chip]=-1;
        }
      }
    }

    for(link=0;link<MAX_LINKS;link++){

      // find histograms

      sprintf(h_name,"ModuleName;%d",module+1);
      TObjString* name = (TObjString*)strun->Get(h_name);
      if(name) {
        sprintf(mname[module],"%s",name->GetName());
        delete name;
      }

      sprintf(h_name,"h_scan%d;%d",link,module+1);
      TH2F *h_scan= (TH2F*)strun->Get(h_name);

      if(h_scan){
        if(h_scan->GetMean()>0){
          printf("Analysing data for module %d link %d\n",
          module, link);

          // Loop over each tested configurations
          for(i=0;i<n_configs;i++){

            for(chip=0;chip<MAX_CHIPS;chip++){
              chip_dead[link][chip]=0;
              chip_good[link][chip]=0;
            }

            // Categorise each channel
            for(chan=0;chan<MAX_CHANS;chan++){
              chip = chan/128;
	      
              Float_t occ = h_scan->GetBinContent((770*(i+1))+chan+1);
	      
              if( (occ>-3)&&(occ<3) ){
		// allow +/-2 hits failing per channel
                chip_dead[link][chip]++;
              }else if( (occ>N_TRIGS-3)&&(occ<N_TRIGS+3) ) {
		// allow +/-2 hits failing per channel
                chip_good[link][chip]++;
              }else{
		chip_error[link][chip]++;
	      }
            } // end for(chan)
	    
            // How many chips gave data?

            for(chip=0;chip<MAX_CHIPS;chip++){
              if(chip_good[link][chip]>=120){

                chip_status[i][link][chip]=1; // CHIP GAVE CORRECT DATA
		
              }else if(chip_dead[link][chip]>=120){

                chip_status[i][link][chip]=0; // CHIP DEAD
	      
	      }else{
		
		chip_status[i][link][chip]=2; // CHIP STATUS UNDEFINED
              }
            }
          } // end for(i)


        } // end if(mean>0) [histogram is not empty]
      } // end if(h_scan) [histogram exists]
    } // end for(link)

    /* Analyse data to determine which token/data links that work
     * 
     * The token/data links are enumerated 0-5 for each link.
     * The results are stored in arrays called 'direct_token'
     * and 'bypass_token' according to the following:
     * '0' token/data link doesn't work properly
     * '1' token/data link works properly.
     */



    // does each configuration work as expected?

    // Loop over configurations
    for(i=0;i<n_configs;i++){
      n_bad_chips=0;
      n_good_chips=0;
      n_missing_chips=0;
      for(link=0;link<MAX_LINKS;link++){
        for(chip=0;chip<MAX_CHIPS;chip++){
          if(chip_status[i][link][chip]==desired_result[i][link][chip]){
            n_good_chips++;
          }else if(chip_status[i][link][chip]==-1){
            n_missing_chips++;
          }else{
            n_bad_chips++;
          }
        }
      }

      if((n_bad_chips+n_good_chips)==0){
        module_status[i] = -1;  // NO DATA

      }else if(n_good_chips==12){
        module_status[i] =  1; // PASS

      }else{
        module_status[i] =  0; // FAIL
      }

    }

    // And finally, has the module passed?

    overall_status=0;
    for(i=0;i<n_configs;i++){
      if(module_status[i]==1){
        overall_status++;
      }
    } 

    if(module_status[0]>-1){ // Do not display or save results if no data!!


      //
      // Results to screen
      //

      printf("\n  Key to read table.\n");
      printf("'1' > 120 channels/chip returns 98 - 102%% occupancy\n");
      printf("'0' > 120 channels/chip returns -2 - 2%% occupancy\n");
      printf("'2' otherwise\n\n");

      if(overall_status==n_configs){
        printf("  Module %d (%s) has PASSED\n",module,mname[module]);
      }else{
        printf("  Module %d (%s) has FAILED\n",module,mname[module]);
      }

      printf("            pass  M0  S1  S2  S3  S4  E5  M8  S9 S10 S11 S12 E13\n");

      for(i=0;i<n_configs;i++){

        if(module_status[i]){
          printf("  config %2d PASS  ", i);
        }else{
          printf("  config %2d FAIL  ", i);
        }

        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            printf("%2d  ",chip_status[i][link][chip]);

          } // end for(chip)
        } // end for(link)
        printf("\n");
      } // end for(config)


      //
      // Results to file
      //

      Short_t pass = 0;
      if(overall_status==n_configs){
        pass = 1;
      }

      // Open file and print data-base header
      FILE* outfile = OpenResults("BypassTest", strun, module, run, scan, pass, 0); 
       
      if(outfile) {

        // fprint test-specific results

        fprintf(outfile,"%%NConfigs  %d\n",n_configs);

        fprintf(outfile,"#\n");
        fprintf(outfile,"#cfg  pass  M0  S1  S2  S3  S4  E5  M8  S9 S10 S11 S12 E13\n");

        for(i=0;i<n_configs;i++){
          fprintf(outfile,"%2d     %1d", i, module_status[i]);

          for(link=0;link<MAX_LINKS;link++){
            for(chip=0;chip<MAX_CHIPS;chip++){
              fprintf(outfile,"  %2d",chip_status[i][link][chip]);
            } // end for(chip)
          } // end for(link)
          fprintf(outfile,"\n");
        } // end for(config)
        fclose(outfile);

      } // end if(outfile)

    }
  } // end(module)

  strun->Close();
  delete strun;
}


void BypassAnalysis(Int_t run, Int_t scan, Int_t module){

  BypassAnalysis(run,scan,module,11);
}

void BypassAnalysis(Int_t run, Int_t scan){

  BypassAnalysis(run,scan,-1,11);
}


