void ConfirmationTest(Int_t standalone, Int_t doPipeline = 1, Int_t restart_dcs = 1){

  //
  // Perform all tests as listed in Confirmation sequence
  //
  // Version 0.1 PWP 15.11.00
  // Version 0.2 PWP 23.11.00
  // Version 0.3 PWP 10.12.00
  // Version 0.4 PWP 18.05.00
  // Version 0.5 PWP 19.09.01
  // Version 0.6 PWP 20.12.01
  // Version 0.7 BJG 30.04.02 - Fix some memory leaks
  // Version 1.0 PWP 14.11.02 - generate config files in XML format at end of test
  // Version 1.1 PWP 26.11.02 - kill/start background DCS monitor at start/end of test
  // Version 1.2 PWP 26.11.02 - Do Replay in batch mode
  // Version 1.3 PWP 28.11.02 - Increment scan count by 2 for pipeline test
  //                            when in replay mode
  //

  // Clear abort flag to start
  e->abort=0;

  // Kill background DCS monitor
  st_kill_idle();

  TStopwatch* patek_phillipe = new TStopwatch();

  // Detector IV curve:
  //   it is left up to the user to do this

  if((e->abort==0)&&(standalone==0)){
    gROOT->ProcessLine(".L HardReset.cpp");
    HardReset();
  }

  if(e->abort==0){
    e->Restart();
    gROOT->ProcessLine(".L FullBypassTest.cpp");
    FullBypassTest();
  }

  if(e->abort==0){
    gROOT->ProcessLine(".L RedundancyTest.cpp");
    RedundancyTest(0);
  }

  if(e->abort==0 && doPipeline==1){
    gROOT->ProcessLine(".L PipelineTest.cpp");
    PipelineTest(0);
  }

  if(e->abort==0){
    e->Restart();
    gROOT->ProcessLine(".L StrobeDelay.cpp");
    StrobeDelay(1);
  }

  if(e->abort==0){
    gROOT->ProcessLine(".L ThreePointGain.cpp");
    ThreePointGain();
  }

  if(e->abort==0){
    e->WriteXMLConfigs(LOCATION_NAME);
  }

  if(restart_dcs){
    // Restart background DCS monitor
    st_set_idle();
  }

  printf("ConfirmationTest completed\n");
  patek_phillipe->Print();
  delete patek_phillipe;
}

void ConfirmationTest(){
  ConfirmationTest(0);
}

void ConfirmationReplay(Int_t run, Int_t start_scan, Int_t doPipeline = 1){

  // Reanalyse the data for a full confirmation test
  // PWP 20.12.01 (untested)

  Int_t scan_number = start_scan;

  gROOT->SetBatch(1);

  TStopwatch* patek_phillipe = new TStopwatch();

  gROOT->ProcessLine(".L FullBypassAnalysis.cpp");
  FullBypassAnalysis(run,scan_number,-1);
  scan_number += 6;

  gROOT->ProcessLine(".L RedundancyAnalysis.cpp");
  RedundancyAnalysis(run,scan_number,-1);
  scan_number += 2;

  if(doPipeline==1) {
    gROOT->ProcessLine(".L PipelineAnalysis.cpp");
    PipelineAnalysis(run,scan_number,-1);
    scan_number += 2;
  }

  gROOT->ProcessLine(".L StrobeDelayPlot.cpp");
  StrobeDelayPlot(run,scan_number,-1);
  scan_number += 1;

  gROOT->ProcessLine(".L ResponseCurvePlot.cpp");
  ThreePointGainPlot(run,scan_number,4);

  printf("ConfirmationReplay completed\n");
  patek_phillipe->Print();
  delete patek_phillipe;

  gROOT->SetBatch(0);
}

