void CurrentScanPlot(Int_t run, Int_t scan,Int_t moi, Int_t loi, int postscript=1) {

  /* Macro calls ExtractGain (via LoopExtract) to extract gain, offset
   * and input noise from a set of threshold scans taken by CurrentScans,
   * defined in c:\sctdaq\macros\currents.cpp.
   * 
   * Parameters to set:
   * - The fraction of channels (fgood) for a chip to be above a certain
   *   gain for it to be considered as fully functional.
   * - Binning of histograms, in the function ExtractGain.  
   *
   * Includes contributions from S. Gadomski and Y. Unno.
   * Version 1.0 Lars Eklund 22/10 2000.
   */

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");

  // Debug & output flag
  int debug = 0;

  // Defining cuts for setting chip response to zero
  Float_t fgood = 0.95;
  Float_t min_gain = 30;

  // Extract gain offset and input noise for all currents
  LoopExtract(run,scan,moi,loi);

  // Define variables used.
  char PrShpr[15];
  //char Directory[] = "d:\\sctvar\\data\\";
  char file[80];
  char figfile[50];
  char s[200], line[100];
  char h_name[40];
  char htit[80];

  //Defining output data variables.
  Double_t gain[9][9][6];
  Double_t gainspread[9][9][6];
  Double_t offset[9][9][6];
  Double_t offsetspread[9][9][6];
  Double_t mean[9][9][6];
  Double_t meanspread[9][9][6];
  Double_t noise[9][9][6];
  Double_t noisespread[9][9][6];

  // Arrays with containing current matrix
  int nchg;
  int npre;
  int nshpr;
  float charge[9];
  float preamp[9];
  float shaper[9];

  // Open file withcurrent configuration data
  sprintf(file,"C:\\sctdaq\\macros\\currents.dat");

  FILE* conffile = fopen(file,"r");
  if(conffile==NULL){
    printf("Could not open current/charge info file %s\n",file);
    return;
  } else {
    printf("File %s opened for reading\n",file);
  }

  // Read input charges.
  fgets(line,200,conffile);
  fgets(line,200,conffile);
  sscanf(line,"%d %f %f %f %f %f %f %f %f %f",&nchg,&charge[0],&charge[1],&charge[2],
	 &charge[3],&charge[4],&charge[5],&charge[6],&charge[7],&charge[8]);
  
  // Read input preamp.
  fgets(line,200,conffile);
  fgets(line,200,conffile);
  sscanf(line,"%d %f %f %f %f %f %f %f %f %f",&npre,&preamp[0],&preamp[1],&preamp[2],
	 &preamp[3],&preamp[4],&preamp[5],&preamp[6],&preamp[7],&preamp[8]);
  
  // Read input shaper.
  fgets(line,200,conffile);
  fgets(line,200,conffile);
  sscanf(line,"%d %f %f %f %f %f %f %f %f %f",&nshpr,&shaper[0],&shaper[1],&shaper[2],
	 &shaper[3],&shaper[4],&shaper[5],&shaper[6],&shaper[7],&shaper[8]);

  fclose(conffile);

  //Parameters for histogramming
  Float_t pre_step = (preamp[(npre-1)] - preamp[0])/(npre-1);
  Float_t shpr_step = (shaper[(nshpr-1)] - shaper[0])/(nshpr-1);
  Float_t min_pre = preamp[0]-pre_step/2;
  Float_t max_pre = preamp[(npre-1)]+pre_step/2;
  Float_t min_shpr = shaper[0]-shpr_step/2;
  Float_t max_shpr = shaper[(nshpr-1)]+shpr_step/2;
  
 
  //Create histograms pointers for data from files
  TH1F* h_gain[6];
  TH1F* gain_dist[6];
  TH1F* offset_dist[6];
  TH1F* mean_dist[6];
  TH1F* noise_dist[6];
 
  //*********************************************************
  //Loop over currents
  for (int ipre=0;ipre<npre;ipre++) {
    for (int ishpr=0;ishpr<nshpr;ishpr++) {

      //Open file and check correct TDirectory.
      sprintf(file,"%sRun%dm%dl%d_Pr%dSh%d.root",DATA_DIRECTORY,run,moi,loi,(int)preamp[ipre],(int)shaper[ishpr]);
      TFile f(file);
      if(f->IsOpen()) {
	printf("File sucsessfully opened: %s\n",file);
      } else {
	printf("Could not open file: %s\n",file);
      }
      //Loop over chips:
      for (int ichip=0;ichip<6;ichip++) {

    
        //Get histogram
	sprintf(h_name,"h_gain%d",ichip);
	h_gain[ichip] = (TH1F*)f->Get(h_name);
        sprintf(h_name,"h_gain_dist%d",ichip);
	gain_dist[ichip] = (TH1F*)f->Get(h_name);
	sprintf(h_name,"h_offset_dist%d",ichip);
	offset_dist[ichip] = (TH1F*)f->Get(h_name);
	sprintf(h_name,"h_mean_dist%d",ichip);
	mean_dist[ichip] = (TH1F*)f->Get(h_name);
	sprintf(h_name,"h_noise_dist%d",ichip);
	noise_dist[ichip] = (TH1F*)f->Get(h_name);


	//Count # of good channels for chip
	int nchgood = 0;
	for (int ichan=0;ichan<128;ichan++) {
	  if(h_gain[ichip]->GetBinContent(ichan)>min_gain) { 
	    nchgood++;
	  }
	}

	if(nchgood > fgood*128) {
	  gain[ipre][ishpr][ichip]=gain_dist[ichip]->GetMean();
	  gainspread[ipre][ishpr][ichip]=gain_dist[ichip]->GetRMS();
	  
	  offset[ipre][ishpr][ichip]=offset_dist[ichip]->GetMean();
	  offsetspread[ipre][ishpr][ichip]=offset_dist[ichip]->GetRMS();
	  
	  mean[ipre][ishpr][ichip]=mean_dist[ichip]->GetMean();
	  meanspread[ipre][ishpr][ichip]=mean_dist[ichip]->GetRMS();

	  noise[ipre][ishpr][ichip]=noise_dist[ichip]->GetMean();
	  noisespread[ipre][ishpr][ichip]=noise_dist[ichip]->GetRMS();
	} else {
	  gain[ipre][ishpr][ichip]=0;
	  gainspread[ipre][ishpr][ichip]=0;
	  offset[ipre][ishpr][ichip]=0;
	  offsetspread[ipre][ishpr][ichip]=0;
	  mean[ipre][ishpr][ichip]=0;
	  meanspread[ipre][ishpr][ichip]=0;
	  noise[ipre][ishpr][ichip]=0;
	  noisespread[ipre][ishpr][ichip]=0;
	}
	if (debug) printf("Gain %3.1f, Mean spread %3.2f, Noise %3.0f\n\n",
	       gain[ipre][ishpr][ichip],meanspread[ipre][ishpr][ichip],noise[ipre][ishpr][ichip]);

	if((noise[ipre][ishpr][ichip]<0) || (noise[ipre][ishpr][ichip]>5000.)) {
	  gain[ipre][ishpr][ichip]=0;
	  gainspread[ipre][ishpr][ichip]=0;
	  offset[ipre][ishpr][ichip]=0;
	  offsetspread[ipre][ishpr][ichip]=0;
	  mean[ipre][ishpr][ichip]=0;
	  meanspread[ipre][ishpr][ichip]=0;
	  noise[ipre][ishpr][ichip]=0;
	  noisespread[ipre][ishpr][ichip]=0;
	}
	
	//Clear histogram between files
	gain_dist[ichip]->Scale(0);
	offset_dist[ichip]->Scale(0);
	mean_dist[ichip]->Scale(0);
	noise_dist[ichip]->Scale(0);
      } // End chip
    } // End shaper
  } // End preamp

  //****************** Open PostScript file ******************
  if(postscript){
    sprintf(file,"%sstcurr%d_%d_%d_%d.ps",PS_DIRECTORY,run,scan,moi,loi);
    
    TPostScript *ps = new TPostScript(file,112);
    ps->NewPage();
  }
  
  // load the second of the root files to obtain timestamp info
  sprintf(file,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan+1);
  
  printf("Using Time/Place/Module name information from %s\n",file); 
  TFile *strun = new TFile(file);

  sprintf(h_name,"ModuleName;%d",moi+1);
  TObjString *time  = (TObjString*)strun->Get("Time");
  TObjString *place = (TObjString*)strun->Get("Place");
  TObjString *mname = (TObjString*)strun->Get(h_name);


  //********************* Otput Canvases **********************
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(s,"ATLAS SCT Module Tests - Noise vs FE Currents: Run %d Scans %d - %d", 
	  run, scan, scan+npre*nshpr*nchg-1);
  if(time)  sprintf(s,"%s  -  %s",s,time->GetName());
  if(place) sprintf(s,"%s  -  %s",s,place->GetName());
  if(mname) sprintf(s,"%s  -  Module %s",s,mname->GetName());
  t1->AddText(s);

  sprintf(s,"Preamplifier %4.1f-%4.1f   Shaper %3.1f-%3.1f  Module %d   Stream %d", 
	  preamp[0], preamp[npre-1], shaper[0], shaper[nshpr-1], moi, loi);
  t1->AddText(s);

  
  // Open and draw the noise canvas

  TCanvas* c2 = new TCanvas ("c1", "Noise_vs_Current",           1100,750);
  pad_tt      = new TPad("pad_tt", "Text",                    0.01,0.91,0.99,0.99,33);
  pad_nc      = new TPad("pad_nc", "Current Characteristics", 0.01,0.01,0.99,0.90,33);
  
  pad_tt->Draw();
  pad_nc->Draw();
  pad_nc->Divide(3,2);

  pad_tt->cd();
  t1->Draw();

  c1->Update();

  //Loop over chips to plot data.
  TGraph *gr[6];      
  for (int ichip=0;ichip<6;ichip++){
    pad_nc->cd(ichip+1);
    
    //Histogram title & Graph for output
    sprintf(htit,"Noise chip%d vs Preamp & Shaper",ichip);
    TH2F *noise_histo = new TH2F("noise_histo",htit,npre,min_pre,max_pre,nshpr,min_shpr,max_shpr);
    
    //Fill array with data to plot in graph. 
    for (int ipre=0;ipre<npre;ipre++) {
      for (int ishpr=0;ishpr<nshpr;ishpr++) {
	noise_histo->SetCellContent(ipre+1,ishpr+1,noise[ipre][ishpr][ichip]);
      }
    }
    //draw
    noise_histo->SetMinimum(200);
    noise_histo->SetMaximum(1200);
    noise_histo->GetXaxis()->SetTitleOffset(2);
    noise_histo->GetYaxis()->SetTitleOffset(2);
    noise_histo->SetXTitle("Preamp current");
    noise_histo->SetYTitle("Shaper current");
    noise_histo->Draw("LEGO2");
  }
  c1->Update();
  
  if(postscript) ps->NewPage(); 

  //********************* Otput Canvases **********************
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(s,"ATLAS SCT Module Tests - Gain vs FE Currents: Run %d Scans %d - %d", 
	  run, scan, scan+npre*nshpr*nchg-1);
  if(time)  sprintf(s,"%s  -  %s",s,time->GetName());
  if(place) sprintf(s,"%s  -  %s",s,place->GetName());
  if(mname) sprintf(s,"%s  -  Module %s",s,mname->GetName());
  t1->AddText(s);

  sprintf(s,"Preamplifier %4.1f-%4.1f   Shaper %3.1f-%3.1f  Module %d   Stream %d", 
	  preamp[0], preamp[npre-1], shaper[0], shaper[nshpr-1], moi, loi);
  t1->AddText(s);

  
  // Open and draw the gain canvas

  TCanvas* c2=new TCanvas ("c2", "Gain_vs_Current",           1100,750);
  pad_tt   = new TPad("pad_tt",  "Text",                    0.01,0.91,0.99,0.99,33);
  pad_gc   = new TPad("pad_gc",  "Current Characteristics", 0.01,0.01,0.99,0.90,33);
  
  pad_tt->Draw();
  pad_gc->Draw();
  pad_gc->Divide(3,2);
  
  pad_tt->cd();
  t1->Draw();

  c1->Update();
  
  //Loop over chips to plot data.
  TGraph *gr[6];      
  for (int ichip=0;ichip<6;ichip++){
    pad_gc->cd(ichip+1);
    
    //histogram title & Graph for output
    sprintf(htit,"Gain chip%d vs Preamp & Shaper",ichip);
    TH2F *gain_histo = new TH2F("GainCurrent",htit,npre,min_pre,max_pre,nshpr,min_shpr,max_shpr);
    
    //Fill array with data to plot in graph. 
    for (int ipre=0;ipre<npre;ipre++) {
      for (int ishpr=0;ishpr<nshpr;ishpr++) {
	gain_histo->SetCellContent(ipre+1,ishpr+1,gain[ipre][ishpr][ichip]);
      }
    }
    //draw
    gain_histo->SetMinimum(20);
    gain_histo->SetMaximum(80);
    gain_histo->GetXaxis()->SetTitleOffset(2);
    gain_histo->GetYaxis()->SetTitleOffset(2);
    gain_histo->SetXTitle("Preamp current");
    gain_histo->SetYTitle("Shaper current");
    gain_histo->Draw("LEGO2");
  }

  if(postscript) {
    ps->Close();
    delete ps;
  }
     
 //Output the data to screen
  printf("\n**** Run %d Preamp and Shaper characteristics *****\n",run);
  for(int ichip=0;ichip<6;ichip++) {
    printf("\nNoise chip %d:\n",ichip);
    printf("Pre/Shpr\t");
    for(int ishpr=0;ishpr<nshpr;ishpr++) printf("%3.1f\t",shaper[ishpr]); 
    for(int ipre=0;ipre<npre;ipre++) {
      printf("\n%4.1f\t",preamp[ipre]);
      for(int ishpr=0;ishpr<nshpr;ishpr++) printf("\t%3.0f",noise[ipre][ishpr][ichip]);
    }
    printf("\nGain chip %d:\n",ichip);
    printf("Pre/Shpr\t");
    for(int ishpr=0;ishpr<nshpr;ishpr++) printf("%3.1f\t",shaper[ishpr]); 
    for(int ipre=0;ipre<npre;ipre++) {
      printf("\n%4.1f\t",preamp[ipre]);
      for(int ishpr=0;ishpr<nshpr;ishpr++) printf("\t%3.0f",gain[ipre][ishpr][ichip]);
    }
    printf("\n");
  }


  //Output the data to file  added YU 21.10.2000
  char  g_name[128];

  sprintf(g_name,"%sstcurr%d_%d_%d_%d.txt",RESULTS_DIRECTORY,run,scan,moi,loi);

  FILE* gfile = fopen(g_name,"w");

  fprintf(gfile,"\n**** Run %d Preamp and Shaper characteristics *****\n",run);
  for(int ichip=0;ichip<6;ichip++) {
    fprintf(gfile,"\nNoise chip %d:\n",ichip);
    fprintf(gfile,"Pre/Shpr\t");
    for(int ishpr=0;ishpr<nshpr;ishpr++) fprintf(gfile,"%3.1f\t",shaper[ishpr]); 
    for(int ipre=0;ipre<npre;ipre++) {
      fprintf(gfile,"\n%4.1f\t",preamp[ipre]);
      for(int ishpr=0;ishpr<nshpr;ishpr++) fprintf(gfile,"\t%3.0f",noise[ipre][ishpr][ichip]);
    }
    fprintf(gfile,"\nGain chip %d:\n",ichip);
    fprintf(gfile,"Pre/Shpr\t");
    for(int ishpr=0;ishpr<nshpr;ishpr++) fprintf(gfile,"%3.1f\t",shaper[ishpr]); 
    for(int ipre=0;ipre<npre;ipre++) {
      fprintf(gfile,"\n%4.1f\t",preamp[ipre]);
      for(int ishpr=0;ishpr<nshpr;ishpr++) fprintf(gfile,"\t%3.0f",gain[ipre][ishpr][ichip]);
    }
    fprintf(gfile,"\n");
  }
  fclose(gfile);
}

void CurrentScanPlot() {
  printf("CurrentScanPlot(run, first_scan, moi, loi, postscript)\n");
}


void LoopExtract(int run, int scan, int moi, int loi) {

  // Loops over a set of scans taken by CurrenScans.cpp to extract the gain
  // The data is stored in d:\sctvar\data. 
  // 'scan' is the first scan in the set of scans defined in currents.cpp

  //gROOT->LoadMacro("ExtractGain.cpp");

  char line[100],f_name[100];
  int nchg;
  int npre;
  int nshpr;
  float charge[9];
  float preamp[9];
  float shaper[9];

  // Open file withcurrent configuration data
  sprintf(f_name,"C:\\sctdaq\\macros\\currents.dat");
  FILE* conffile = fopen(f_name,"r");
  if(conffile==NULL){
    printf("Could not open current/charge info file %s\n",f_name);
    return;
  } else {
    printf("File %s opened for reading\n",f_name);
  }

  // Read input charges.
  fgets(line,200,conffile);
  fgets(line,200,conffile);
  sscanf(line,"%d %f %f %f %f %f %f %f %f %f",&nchg,&charge[0],&charge[1],&charge[2],
	 &charge[3],&charge[4],&charge[5],&charge[6],&charge[7],&charge[8]);
  
  // Read input preamp.
  fgets(line,200,conffile);
  fgets(line,200,conffile);
  sscanf(line,"%d %f %f %f %f %f %f %f %f %f",&npre,&preamp[0],&preamp[1],&preamp[2],
	 &preamp[3],&preamp[4],&preamp[5],&preamp[6],&preamp[7],&preamp[8]);
  
  // Read input shaper.
  fgets(line,200,conffile);
  fgets(line,200,conffile);
  sscanf(line,"%d %f %f %f %f %f %f %f %f %f",&nshpr,&shaper[0],&shaper[1],&shaper[2],
	 &shaper[3],&shaper[4],&shaper[5],&shaper[6],&shaper[7],&shaper[8]);
  
  Float_t pre;
  Float_t shpr;
  for(int ipre=0;ipre<npre;ipre++) {
    for(int ishpr=0;ishpr<nshpr;ishpr++) {
      pre  = preamp[ipre];
      shpr = shaper[ishpr];
      
      printf("Extract from run %d, module %d, link %d, preamp %4.1f, shaper %3.1f \n",run,moi,loi,pre,shpr);
      ExtractGain(run,scan,moi,loi,pre,shpr,0,1);
    }
  }
  fclose(conffile);
}

void ExtractGain(int rno, int scan, int moi, int loi, float pre, float shpr, int display, int wr2file) {
 
  /* Extracts gain, offset and input noise for a certain current setting. Stores 
   * the processed data as root histograms in d:/sctvar/data.
   *
   * Includes contributions from S. Gadomski and Y. Unno.
   * Version 1.0 Lars Eklund 22/10 - 2000
   */

  // Debug flag, controls debug output.
  // Determines at witch charge the noise
  // is extracted and the mean showed.
  int debug = 0;  
  int show_point = 1;

  // Define variables
  char h_title[100];
  char h_name[100];
  char  f_name[100];
  char filetitle[100];
  char line[100];
  //char directory[] = "d:\\sctvar\\data";
  char fit_opt[3];
  Float_t tempgain;
  Float_t tempmean;
  Float_t tempnoise;
  float chip_fit_mean[4][6];
  float chip_fit_spread[4][6];

  int nchg;
  int npre;
  int nshpr;
  float charge[9];
  float preamp[9];
  float shaper[9];

  // Open file withcurrent configuration data
  sprintf(f_name,"C:\\sctdaq\\macros\\currents.dat");
  FILE* conffile = fopen(f_name,"r");
  if(conffile==NULL){
    printf("Could not open current/charge info file %s\n",f_name);
    return;
  } else {
    printf("File %s opened for reading\n",f_name);
  }

    // Read input charges.
    fgets(line,200,conffile);
    fgets(line,200,conffile);
    sscanf(line,"%d %f %f %f %f %f %f %f %f %f",&nchg,&charge[0],&charge[1],&charge[2],
	   &charge[3],&charge[4],&charge[5],&charge[6],&charge[7],&charge[8]);

    // Read input preamp.
    fgets(line,200,conffile);
    fgets(line,200,conffile);
    sscanf(line,"%d %f %f %f %f %f %f %f %f %f",&npre,&preamp[0],&preamp[1],&preamp[2],
	   &preamp[3],&preamp[4],&preamp[5],&preamp[6],&preamp[7],&preamp[8]);

    // Read input shaper.
    fgets(line,200,conffile);
    fgets(line,200,conffile);
    sscanf(line,"%d %f %f %f %f %f %f %f %f %f",&nshpr,&shaper[0],&shaper[1],&shaper[2],
	   &shaper[3],&shaper[4],&shaper[5],&shaper[6],&shaper[7],&shaper[8]);

  fclose(conffile);

  //Define data arrays for data from scurves
  Short_t code[768][9];
  Float_t mean[768][9];
  Float_t sigma[768][9];
  
  //Clear the arrays
  for(int ichg;ichg<nchg;ichg++) {
    for(int ichan;ichan<768;ichan++) {
      code[ichan][ichg]=0;
      mean[ichan][ichg]=0;
      sigma[ichan][ichg]=0;
    }
  }

  int start_scan = 0;
  for (int ipre=0;ipre<npre;ipre++) {
   for (int ishpr=0;ishpr<nshpr;ishpr++) {
      for (int ichg=0;ichg<nchg;ichg++) {
	if(preamp[ipre]==pre && shaper[ishpr]==shpr) {
	  start_scan = scan + nchg * (ishpr + ipre*nshpr);
	}
      }
    }
  }

  
  // don't use largest Qin - ugly fix LE 8/5 2001 - to be REMOVED!!!!!!!!!
  //nchg--;

  for(int ichg=0;ichg<nchg;ichg++) {
    // Open file
    sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,(int)rno,start_scan+ichg);
      
    TFile *strun = new TFile(f_name);

    if(strun==0){
      printf("Failed to open file %s - EXITING!!\n",f_name);
      return;
    } else {
      printf("Opened file: %s\n",f_name);
    }
    
      TH1S *h_code;
      TH1F *h_vt50;
      TH1F *h_sigma;
    
    // Find histograms and fill arrays
    sprintf(h_name,"h_code%d;%d",loi,moi+1);
    TH1S *h_code = (TH1S*)strun->Get(h_name);
    sprintf(h_name,"h_mean%d;%d",loi,moi+1);
    TH1F *h_vt50 = (TH1F*)strun->Get(h_name);
    sprintf(h_name,"h_sigma%d;%d",loi,moi+1);
    TH1F *h_sigma = (TH1F*)strun->Get(h_name);
    
    if(h_vt50){ // fitted data exists
      if(h_vt50->GetMean()>0){
	printf("Filling arrays for charge %4.2f module %d link %d\n",
	       charge[ichg], moi, loi);
 
	for(int ichan=0;ichan<768;ichan++){
	  code[ichan][ichg] = h_code->GetBinContent(ichan);
  	  mean[ichan][ichg] = h_vt50->GetBinContent(ichan);
  	  sigma[ichan][ichg]= h_sigma->GetBinContent(ichan);
	} 
      }
    }
    else { // added to read .fit file  YU 21.10.2000

      sprintf(f_name,"%sstrun%d_%d_%d_%d.fit",DATA_DIRECTORY,(int)rno,start_scan+ichg,moi,loi);
  
      FILE* fitfile = fopen(f_name,"r");

      if(fitfile==NULL){
        printf("Could not open fit file %s\n",f_name);
        return;
      } else {
        printf("File %s opened for reading\n",f_name);

        Int_t i,index;
        double var;

        i=0;

        fgets(line,100,fitfile);
        while(fgets(line,100,fitfile) != NULL){
          index = 3;
          index+=get_val(line,index,&var);
          code[i][ichg] = (Short_t)var;
          index+=get_val(line,index,&var);
          mean[i][ichg]=(float)var;
          index+=get_val(line,index,&var);
          index+=get_val(line,index,&var);
          sigma[i][ichg]=(float)var;

	  //printf("%d %f %f\n",code[i][ichg],mean[i][ichg],sigma[i][ichg]);

          i++;
        }
      fclose(fitfile);
      }
    }
    strun->Close();
    delete strun;
  } 


  // ************************************************************
  // Open output file to store histograms
  if (wr2file == 1) {
    //Recreate file to store histograms
    sprintf(f_name,"%sRun%dm%dl%d_Pr%dSh%d.root",DATA_DIRECTORY,rno,moi,loi,(int)pre,(int)shpr);
    printf("Output f_name %s\n",f_name);
    sprintf(filetitle,"Histograms for run %d, Preamp %d Shaper %d",rno,(int)pre,(int)shpr);
    TFile *outfile = gROOT->FindObject(f_name); if (outfile) outfile->Close(); //PWP's closing
    
    // Open file
    TFile *outfile = new TFile(f_name,"RECREATE",filetitle);
    
    if (outfile->IsOpen()) {
      printf("File opened sucsessfully\n");
    } else {
      printf("Problems opening file\n");
    }
  }


  //*******************************************************************************************       
  if (debug) printf("Create histos\n");
  //Create histograms for plots by channel
  TH1F* h_gain[6];
  TH1F* h_gain[0]  =  new TH1F("h_gain0", "Gain, all channels, chip 0",  128,0,128);
  TH1F* h_gain[1]  =  new TH1F("h_gain1", "Gain, all channels, chip 1",  128,0,128);
  TH1F* h_gain[2]  =  new TH1F("h_gain2", "Gain, all channels, chip 2",  128,0,128);
  TH1F* h_gain[3]  =  new TH1F("h_gain3", "Gain, all channels, chip 3",  128,0,128);
  TH1F* h_gain[4]  =  new TH1F("h_gain4", "Gain, all channels, chip 4",  128,0,128);
  TH1F* h_gain[5]  =  new TH1F("h_gain5", "Gain, all channels, chip 5",  128,0,128);
  
  TH1F* h_offset[6];
  TH1F* h_offset[0]  =  new TH1F("h_offset0", "Offset, all channels, chip 0",  128,0,128);
  TH1F* h_offset[1]  =  new TH1F("h_offset1", "Offset, all channels, chip 1",  128,0,128);
  TH1F* h_offset[2]  =  new TH1F("h_offset2", "Offset, all channels, chip 2",  128,0,128);
  TH1F* h_offset[3]  =  new TH1F("h_offset3", "Offset, all channels, chip 3",  128,0,128);
  TH1F* h_offset[4]  =  new TH1F("h_offset4", "Offset, all channels, chip 4",  128,0,128);
  TH1F* h_offset[5]  =  new TH1F("h_offset5", "Offset, all channels, chip 5",  128,0,128);
  
  TH1F* h_noise[6];
  TH1F* h_noise[0]  =  new TH1F("h_noise0", "Noise, all channels, chip 0",  128,0,128);
  TH1F* h_noise[1]  =  new TH1F("h_noise1", "Noise, all channels, chip 1",  128,0,128);
  TH1F* h_noise[2]  =  new TH1F("h_noise2", "Noise, all channels, chip 2",  128,0,128);
  TH1F* h_noise[3]  =  new TH1F("h_noise3", "Noise, all channels, chip 3",  128,0,128);
  TH1F* h_noise[4]  =  new TH1F("h_noise4", "Noise, all channels, chip 4",  128,0,128);
  TH1F* h_noise[5]  =  new TH1F("h_noise5", "Noise, all channels, chip 5",  128,0,128);
  
  TH1F* h_mean[6];
  TH1F* h_mean[0]  =  new TH1F("h_mean0", "Mean, all channels, chip 0",  128,0,128);
  TH1F* h_mean[1]  =  new TH1F("h_mean1", "Mean, all channels, chip 1",  128,0,128);
  TH1F* h_mean[2]  =  new TH1F("h_mean2", "Mean, all channels, chip 2",  128,0,128);
  TH1F* h_mean[3]  =  new TH1F("h_mean3", "Mean, all channels, chip 3",  128,0,128);
  TH1F* h_mean[4]  =  new TH1F("h_mean4", "Mean, all channels, chip 4",  128,0,128);
  TH1F* h_mean[5]  =  new TH1F("h_mean5", "Mean, all channels, chip 5",  128,0,128);

  if (debug) printf("Define min and max for histos\n");
  //Define min and max and number of bins
  int ngain = 40;
  int mingain = 0;
  int maxgain = 80;

  int noffset = 40;
  int minoffset =-150;
  int maxoffset = 150;

  int nmean = 30;
  int minmean = 50;
  int maxmean = 400;

  int nnoise = 40;
  int minnoise = 0;
  int maxnoise = 2200;
 
  //Create histograms for distubutions plots
  TH1F* h_gain_dist[6];
  TH1F* h_gain_dist[0]  =  new TH1F("h_gain_dist0", "Gain distribution, chip 0", ngain,mingain,maxgain);
  TH1F* h_gain_dist[1]  =  new TH1F("h_gain_dist1", "Gain distribution, chip 1", ngain,mingain,maxgain);
  TH1F* h_gain_dist[2]  =  new TH1F("h_gain_dist2", "Gain distribution, chip 2", ngain,mingain,maxgain);
  TH1F* h_gain_dist[3]  =  new TH1F("h_gain_dist3", "Gain distribution, chip 3", ngain,mingain,maxgain);
  TH1F* h_gain_dist[4]  =  new TH1F("h_gain_dist4", "Gain distribution, chip 4", ngain,mingain,maxgain);
  TH1F* h_gain_dist[5]  =  new TH1F("h_gain_dist5", "Gain distribution, chip 5", ngain,mingain,maxgain);
  
  TH1F* h_offset_dist[6];
  TH1F* h_offset_dist[0]  =  new TH1F("h_offset_dist0", "Offset distribution, chip 0", noffset,minoffset,maxoffset);
  TH1F* h_offset_dist[1]  =  new TH1F("h_offset_dist1", "Offset distribution, chip 1", noffset,minoffset,maxoffset);
  TH1F* h_offset_dist[2]  =  new TH1F("h_offset_dist2", "Offset distribution, chip 2", noffset,minoffset,maxoffset);
  TH1F* h_offset_dist[3]  =  new TH1F("h_offset_dist3", "Offset distribution, chip 3", noffset,minoffset,maxoffset);
  TH1F* h_offset_dist[4]  =  new TH1F("h_offset_dist4", "Offset distribution, chip 4", noffset,minoffset,maxoffset);
  TH1F* h_offset_dist[5]  =  new TH1F("h_offset_dist5", "Offset distribution, chip 5", noffset,minoffset,maxoffset);
  
  TH1F* h_mean_dist[6];
  TH1F* h_mean_dist[0]  =  new TH1F("h_mean_dist0", "Mean distribution, chip 0", nmean,minmean,maxmean);
  TH1F* h_mean_dist[1]  =  new TH1F("h_mean_dist1", "Mean distribution, chip 1", nmean,minmean,maxmean);
  TH1F* h_mean_dist[2]  =  new TH1F("h_mean_dist2", "Mean distribution, chip 2", nmean,minmean,maxmean);
  TH1F* h_mean_dist[3]  =  new TH1F("h_mean_dist3", "Mean distribution, chip 3", nmean,minmean,maxmean);
  TH1F* h_mean_dist[4]  =  new TH1F("h_mean_dist4", "Mean distribution, chip 4", nmean,minmean,maxmean);
  TH1F* h_mean_dist[5]  =  new TH1F("h_mean_dist5", "Mean distribution, chip 5", nmean,minmean,maxmean);
  
  TH1F* h_noise_dist[6];
  TH1F* h_noise_dist[0]  =  new TH1F("h_noise_dist0", "Noise distribution, chip 0", nnoise,minnoise,maxnoise);
  TH1F* h_noise_dist[1]  =  new TH1F("h_noise_dist1", "Noise distribution, chip 1", nnoise,minnoise,maxnoise);
  TH1F* h_noise_dist[2]  =  new TH1F("h_noise_dist2", "Noise distribution, chip 2", nnoise,minnoise,maxnoise);
  TH1F* h_noise_dist[3]  =  new TH1F("h_noise_dist3", "Noise distribution, chip 3", nnoise,minnoise,maxnoise);
  TH1F* h_noise_dist[4]  =  new TH1F("h_noise_dist4", "Noise distribution, chip 4", nnoise,minnoise,maxnoise);
  TH1F* h_noise_dist[5]  =  new TH1F("h_noise_dist5", "Noise distribution, chip 5", nnoise,minnoise,maxnoise);
  

  // *********************************************************
  // Create temporary histogram to fit the gain

  if (debug) printf("Create temporary histo\n");
  Float_t binsize = (charge[nchg-1] - charge[0])/(float)(nchg-1);
  Float_t chl = charge[0] - binsize/2.0;
  Float_t chh = charge[nchg-1] + binsize/2.0;

  sprintf(h_title,"Linear fit to charges %2.1f-%2.1f fC",charge[0],charge[nchg-1]);
  TH1F* h_temp  =  new TH1F("h_temp",h_title, nchg,chl,chh);
  h_temp->SetXTitle("Charge (fC)");
  h_temp->SetYTitle("50% points (mV)");


  //Define fit functions
  if (debug) printf("Define fit functions\n");
  TF1 *f1 = new TF1("f1","pol1", 0,4.0);
  TF1 *f2 = new TF1("f2","gaus"); 
  // TF1 *f3 = new TF1("f3","pol1", 1.25,2.75);

  if (debug) {
    printf("Creating debug canvas\n");
    //Create canvas for debug output
    TCanvas *cfit = (TCanvas*)gROOT->FindObject("Fit");
    if ( cfit ) delete cfit;
    cfit = new TCanvas("cfit","Debug Fit",700,500);
    cfit->UseCurrentStyle();
    cfit->Clear();
    //Set style options
    gStyle->SetOptStat(0);
    gStyle->SetOptFit(1);
  }

  if (debug) printf("Entering loop on chips\n");

  //***************************************************************************
  //Loop over chips to fill histograms.
  for (int ichip=0;ichip<6;ichip++){

    printf("Extracting gain for chip %d\n",ichip);  
    
    //Fill Histograms
    int bin;
    for (int k=0;k<128;k++) {
      bin = ichip*128 + k;
      //Fill temp histogram
     
      Int_t tofit = 1;
      for(int ichg = 0; ichg < nchg; ichg++) {
	if(!(code[bin][ichg]&BAD_TEST)) { 
          h_temp->SetBinContent(ichg+1,mean[bin][ichg]);
	} else {
          printf("Ch %d: bad code 0x%04x, charge %d\n",bin,code[bin][ichg],ichg);
	  tofit = 0; break;
	}
      }
      
      if(debug) {
	sprintf(fit_opt,"R");
      } else {
	sprintf(fit_opt,"QNR");
      }

      if(tofit) {
        //Fit the line to estimate gain and offset
        h_temp->Fit("f1",fit_opt);
     
	//Calclulate the gain
        tempgain = f1->GetParameter(1);
      } else {
	tempgain = 0.0; 
      }

      if(debug) {
	//cfit->Update();
	printf("Do you want to continue 'debug' (0/1)?\n");
	scanf("%d",&debug);
	printf("\n");
      }

      //Fill histograms
      h_gain[ichip]->SetBinContent(k+1,tempgain);
      h_gain_dist[ichip]->Fill(tempgain);
      h_offset[ichip]->SetBinContent(k+1, f1->GetParameter(0));
      h_offset_dist[ichip]->Fill(f1->GetParameter(0));
      
      //Calculate the mean & noise.
      if (tempgain != 0) { 
	tempmean = mean[bin][show_point];
	tempnoise = sigma[bin][show_point]*6242/tempgain;
      } else {
	tempmean = 0;
	tempnoise = 0;
      }
      //Fill histograms
      h_mean[ichip]->SetBinContent(k+1,tempmean);
      h_mean_dist[ichip]->Fill(tempmean);
      h_noise[ichip]->SetBinContent(k+1,tempnoise);
      h_noise_dist[ichip]->Fill(tempnoise);
    }    
  }
   
//**************************************************************************
  //Fit gaussian to the distribution histogram

  if(display) {
    sprintf(fit_opt,"Q");
  } else {
    sprintf(fit_opt,"QN");
  }

  for(int ichip=0;ichip<6;ichip++){
    if(h_gain_dist[ichip]->GetMaximum()>0.0){
      h_gain_dist[ichip]->Fit("f2",fit_opt); 
      chip_fit_mean[0][ichip]=f2->GetParameter(1);
      chip_fit_spread[0][ichip]=f2->GetParameter(2);
    } else {
      chip_fit_mean[0][ichip]=-1;
      chip_fit_spread[0][ichip]=-1;
    }
    if(h_offset_dist[ichip]->GetMaximum()>0.0){
      h_offset_dist[ichip]->Fit("f2",fit_opt); 
      chip_fit_mean[1][ichip]=f2->GetParameter(1);
      chip_fit_spread[1][ichip]=f2->GetParameter(2);
    } else {
      chip_fit_mean[1][ichip]=-1;
      chip_fit_spread[1][ichip]=-1;
    }
    if(h_mean_dist[ichip]->GetMaximum()>0.0){
      h_mean_dist[ichip]->Fit("f2",fit_opt); 
      chip_fit_mean[2][ichip]=f2->GetParameter(1);
      chip_fit_spread[2][ichip]=f2->GetParameter(2);
    } else {
      chip_fit_mean[2][ichip]=-1;
      chip_fit_spread[2][ichip]=-1;
    }
    if(h_noise_dist[ichip]->GetMaximum()>0.0){
      h_noise_dist[ichip]->Fit("f2",fit_opt);
      chip_fit_mean[3][ichip]=f2->GetParameter(1);
      chip_fit_spread[3][ichip]=f2->GetParameter(2);
    } else {
      chip_fit_mean[3][ichip]=-1;
      chip_fit_spread[3][ichip]=-1;
    }
  }

  if (display) {
    //********************************************************************************
    //Create canvases for display and set style:
    gROOT->SetStyle("Plain");
    gStyle->SetOptStat(0);
    gStyle->SetOptFit(1);
    

    cgain = new TCanvas("cgain","Gain",700,500);
    PlotHistos(cgain,h_gain,0,80,"Gain (mV/fC)","Channel #","Gain"); 

    cnoise = new TCanvas("cnoise","Noise",700,500);
    PlotHistos(cnoise,h_noise,200,2000,"Noise (e)","Channel #","Noise"); 

    cmean = new TCanvas("cmean","Mean",700,500);
    PlotHistos(cmean,h_mean,0,400,"Mean (mV)","Channel #","Mean"); 

    coffset = new TCanvas("coffset","Offset",700,500);
    PlotHistos(coffset,h_offset,-150,150,"Offset (mV)","Channel #","Offset"); 

    cgaindist = new TCanvas("cgaindist","Gain Distribution",700,500);
    PlotHistos(cgaindist,h_gain_dist,0,0,"Gain distribution","Gain (mV/fC)","# of channels"); 

    cnoisedist = new TCanvas("cnoisedist","Noise Distribution",700,500);
    PlotHistos(cnoisedist,h_noise_dist,0,0,"Noise distribution","Noise (e)","# of channels"); 

    cmeandist = new TCanvas("cmeandist","Mean Distribution",700,500);
    PlotHistos(cmeandist,h_mean_dist,0,0,"Mean distribution","Mean (mV)","# of channels"); 

    coffsetdist = new TCanvas("coffsetdist","Offset Distribution",700,500);
    PlotHistos(coffsetdist,h_offset_dist,0,0,"Offset distribution","Offset (mV)","# of channels");
  }

  if (wr2file == 1) {
    outfile->Write();
    outfile->Close();
  }
}

void ExtractGain() {
  printf("ExtractGain(run, start_scan, moi, loi, Pre, Shpr, display, wr2file\n");
}

void PlotHistos(TCanvas *canvas, TH1F *histo[6],Float_t ymin,Float_t ymax,Char_t *title,Char_t *xtit,Char_t *ytit) { 
   
  char h_title[50];

  canvas->UseCurrentStyle();
  canvas->Clear();
  canvas->Divide(3,2);
  
  for(int ichip=0;ichip<6;ichip++) {
    canvas->cd(ichip+1);
    sprintf(h_title,"%s for Chip %d",title,ichip);
    histo[ichip]->SetTitle(h_title);
    histo[ichip]->SetXTitle(xtit);
    histo[ichip]->SetYTitle(ytit);
    if(ymax>ymin) {
      histo[ichip]->SetMaximum(ymax);
      histo[ichip]->SetMinimum(ymin);
    }
    
    histo[ichip]->Draw();
    canvas->Update();
  }
}


Int_t get_val(char* line, Int_t index, double* val){

  // convert next element of data row "line" to type double
  // (CINT sscanf is limited to <43 elements, hence this approach)

  char substring[20];

  sscanf(line+=index,"%s ",substring);
  sscanf(substring,"%lf",val);
  return 1 + strlen(substring);
}
