void FullBypassTest(Float_t startVdd, Float_t stopVdd, Float_t stepVdd, Int_t last) { 

  //
  // A enlaged set of possible bypass configurations, and the 
  // associated changes to the enabled command and data streams 
  // have been implemented as ConfigureVariable(ST_TOKEN,var).
  //
  // The function st_configure_token takes an arbitrary bitpattern
  // as an argument and configures the bypass scheme accordinly. 
  // Variable ST_TOKEN (19) defines a complete set of bypass schemes
  // to test all the token/data links between the modules.
  //
  // Hence these configurations may be tested simply by executing a 
  // scan of variable type 19, classifying the token/data links as
  // functional, failing or unknown at each voltage.
  //
  // version 0.1 PWP 02.11.00
  // version 0.2 PWP 14.11.00 - include analysis
  // version 0.3 LE  21.11.00 - resetting values after scan
  // version 0.4 PWP 23.11.00 - replace maxtrigs by ntrigs (oops!!)
  // version 0.5 LE  16.02.01 - conforming tests to Module QA document 3.0
  // version 0.6 LE  27.02.01 - using extended bypass scheme ST_TOKEN
  //                          - looping over Vdd.
  // version 0.7 PWP 09.05.01 - trivial bugfix
  // version 0.8 BJG 22.05.02 - pause to reset Vdd voltage (in manual)
 
  #define MAX_VDD     20
  #define MAX_CONFIGS 62
  #define N_TRIGS    100

  // Set voltages manually on power supply
  const Int_t manual=0;

  //Various variables
  Int_t scan_array[MAX_VDD];
  Int_t run_array[MAX_VDD];
  Float_t vdd_array[MAX_VDD];
  Int_t n_scans = 0;
  Char_t key[10];

  //Store settings before the scan
  e->SaveState();

  // Configure the scan
  e->StOutScanReset();
  e->ascii_nt=0;
  
  e->do_cal_loop=0;       // cal loop OFF
  e->do_fits=0;           // integrated fitting OFF
  
  e->burst_trtype=0;      // L1A
  e->burst_ntrigs=N_TRIGS;
  e->debug_level=5;
  e->do_preburst=1;

  e->ConfigureVariable(11,1);  // X1X compression 
  e->ConfigureVariable(13,0);  // EDGE DETECT OFF
  e->ConfigureVariable(7,3);   // No masked channels
  e->ConfigureVariable(14,1);  // Route Mask to Pipeline      

  
  e->ConfigureScan(19,0,last,1);  // 17 - ST_TOKEN scan

  if(startVdd<stopVdd) {
    for(Float_t Vdd=startVdd; Vdd<stopVdd; Vdd+=stepVdd) {
      if(e->abort!=0) break;
      e->ConfigureVariable(33,Vdd);  // Set Vdd
      // set voltages manually
      if (manual)
	{
	  printf("Set voltage to Vdd=%f and hit RETURN to continue\n",Vdd);
	  scanf("%s",key);
	}	  
      st_scan(11,-1);                // Scan token links
      scan_array[n_scans] = e->scannum;
      run_array[n_scans] = e->runnum;
      vdd_array[n_scans] = Vdd;
      n_scans++;
    }                
  } else {
    for(Float_t Vdd=startVdd; Vdd>stopVdd; Vdd-=stepVdd) {
      if(e->abort!=0) break;
      e->ConfigureVariable(33,Vdd);  // Set Vdd
      // set volages manually
      if (manual)
	{
	  printf("Set voltage to Vdd=%f and hit RETURN to continue\n",Vdd);
	  scanf("%s",key);
	}	  
      st_scan(11,-1);                // Scan token links
      scan_array[n_scans] = e->scannum;
      run_array[n_scans] = e->runnum;
      vdd_array[n_scans] = Vdd;
      n_scans++;
    }
  }


  // Back to default chip configuration before clearing do_preburst
  // - in this way suspended but working links will be reenabled!!
  // PWP bugfix 11.05.01

  // Back to 4.0 Vdd to increase probability of success
  e->ConfigureVariable(33,4.0);

  e->ConfigureVariable(17,4);  // 12 chips through 2 links
  e->ExecuteConfigs();
  e->ExecuteBurst(11);
  
  // set voltages manually
  if (manual) {
    printf("Set Vdd voltage to normal running value and hit RETURN to continue\n");
    scanf("%s",key);
  }	  

  // Reset values after scan
  e->RestoreState();

  // load the analysis macro 

  gROOT->ProcessLine(".L FullBypassAnalysis.cpp");
 
  //
  // Call the analysis macro.
  // Arguments are:
  // run_array, scan_array, mod, vdd_array, n_scans, n_configs
  //

  FullBypassAnalysis(run_array,scan_array,-1,vdd_array,n_scans,(last+1));
} 

void FullBypassTest(){
  // Default Vdd 4.0 -> 3.5 V,  36 configuations

  // 20 configurations: Read 6 chips per link, no wrap-around.
  // 36 configurations: Full Barrel and Forward K3 test.
  // 62 configurations: Full Forward K4 test.

  FullBypassTest(4.0, 3.5, 0.1, 36);
}






















