void ShowMXSetting(Short_t sid){
  // Show which multipler output should be selected, the multiplexer setting to be
  // used and hence the states of the multiplexer LEDS needed to observe stream sid
  // P.W.Phillips 17.07.02

  static char socket[4] = {'A', 'B', 'C', 'D'};

  printf("    Monitor Socket M%c, multiplex setting %d ", socket[sid/4], sid%4);

  if(((sid%4)/2)==1){    // MSB set
    printf("(LED M-ON ");
  }else{
    printf("(LED M-OFF");
  }

  if((sid%2)==1){        // LSB set
    printf(" L-ON )\n");
  }else{
    printf(" L-OFF)\n");
  }
}

void HardReset(){

  // Macro to guide the operator through the
  // Power on / Hard Reset test sequence
  // version 0.10 PWP 05.07.01
  // version 0.11 PWP 05.07.01 - tested working (with single module)
  // version 0.12 PWP 22.08.01 - database compliance
  // version 0.13 PWP 12.09.01 - further database compliance
  // version 0.20 PWP 15.11.01 - new DB format
  // version 0.21 PWP 16.01.02 - placeholder for TIME_POWERED field
  // version 0.22 BJG 30.04.02 - minor syntax changes
  // version 0.23 PWP 17.05.02 - display the required states of M, L LEDs

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  char devtype[32];

  Short_t mid, sid;
  Short_t test,module,link;
  Short_t yesno;

  Float_t icc_data[3][MAX_MODULES];
  Float_t idd_data[3][MAX_MODULES];

  Short_t status[3][MAX_MODULES][MAX_LINKS];

  // Load other files needed by this one
  gROOT->LoadMacro("ToolBox.cpp");

  // Clear Array Contents

  for(test=0;test<3;test++){
    for(module=0;module<MAX_MODULES;module++){
      for(link=0;link<MAX_LINKS;link++){
        status[test][module][link]=-1;
      }
      icc_data[test][module]=-1;
      idd_data[test][module]=-1;
    }
  }

  // Find today's date - needed for outfile output
  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Int_t time = dt->GetTime();


  // Toggle Power
  // Operator verifies presence of clk/2

  printf("Switching LV off...\n");
  e->LVOff();

  printf("Waiting 2000mS...\n");
  gSystem->Sleep(2000);

  printf("Switching LV on...\n");
  e->LVOn();

  printf("\n");
  printf("Using an oscilloscope, please verify the presence of\n");
  printf("the clk/2 signal at each of the following outputs:\n");
  printf("\n");

  for(mid=0;mid<(e->nMuSTARDs);mid++){
    for(sid=0;sid<12;sid++){
      module = e->FindModule(mid,sid);
      link = e->FindLink(mid,sid);

      if((module>=0)&&(link>=0)){
        printf("  Mustard %d Stream %2d (Module %d (%s) Link %d)\n",
                  mid,sid,module,e->m[module]->name,link);
        ShowMXSetting(sid);

        yesno=2;
        while((yesno!=0)&&(yesno!=1)){
          printf("    Do you see clk/2? (0/1)");
          scanf("%d",&yesno);
        }
        if(module<MAX_MODULES){
          status[0][module][link]=yesno;
        }
      }
    }
  }

  // Make Current measurement
  e->DCSQuery();

  for(module=0;module<MAX_MODULES;module++){
    if(module<e->GetMaxModules()){
      if(e->m[module]->present){
        icc_data[0][module]=e->m[module]->DCS->icc;
        idd_data[0][module]=e->m[module]->DCS->idd;
      }
    }
  }


  // Send Full Configuration
  // Operator verifies lack of clk/2

  printf("Sending full configuration...\n");
  e->ExecuteConfigs();

  printf("\n");
  printf("Using an oscilloscope, please verify the abscence of\n");
  printf("the clk/2 signal at each of the following outputs:\n");
  printf("\n");

  for(mid=0;mid<e->nMuSTARDs;mid++){
    for(sid=0;sid<12;sid++){
      module = e->FindModule(mid,sid);
      link = e->FindLink(mid,sid);

      if((module>=0)&&(link>=0)){
        printf("  Mustard %d Stream %2d (Module %d (%s) Link %d)\n",
                  mid,sid,module,e->m[module]->name,link);
        ShowMXSetting(sid);

        yesno=2;
        while((yesno!=0)&&(yesno!=1)){
          printf("    Has the clk/2 stopped? (0/1)");
          scanf("%d",&yesno);
        }
        if(module<MAX_MODULES){
          status[1][module][link]=yesno;
        }
      }
    }
  }


  // Send HARD RESET
  // Operator verifies presence of clk/2

  printf("Sending HARD RESET...\n");
  e->HardReset();

  printf("\n");
  printf("Using an oscilloscope, please verify the presence of\n");
  printf("the clk/2 signal at each of the following outputs:\n");
  printf("\n");

  for(mid=0;mid<e->nMuSTARDs;mid++){
    for(sid=0;sid<12;sid++){
      module = e->FindModule(mid,sid);
      link = e->FindLink(mid,sid);

      if((module>=0)&&(link>=0)){
        printf("  Mustard %d Stream %2d (Module %d (%s) Link %d)\n",
                  mid,sid,module,e->m[module]->name,link);
        ShowMXSetting(sid);

        yesno=2;
        while((yesno!=0)&&(yesno!=1)){
          printf("    Do you see clk/2? (0/1)");
          scanf("%d",&yesno);
        }
        if(module<MAX_MODULES){
          status[2][module][link]=yesno;
        }
      }
    }
  }


  // Stop the clock!!
  // Record currents

  printf("STOP THE CLOCK!!\n");
  e->ClockOff();

  printf("Waiting 2000mS...\n");
  gSystem->Sleep(2000);

  // Make Current measurement
  e->DCSQuery();

  for(module=0;module<MAX_MODULES;module++){
    if(module<e->GetMaxModules()){
      if(e->m[module]->present){
        icc_data[1][module]=e->m[module]->DCS->icc;
        idd_data[1][module]=e->m[module]->DCS->idd;
      }
    }
  }


  // Restart the clock
  // Toggle power
  // Send Full Configuration

  printf("START THE CLOCK!!\n");
  e->ClockOn();

  printf("Waiting 2000mS...\n");
  gSystem->Sleep(2000);

  printf("Sending full configuration...\n");
  e->ExecuteConfigs();

  printf("Waiting 2000mS...\n");
  gSystem->Sleep(2000);

  // Make Current measurement
  e->DCSQuery();


  // Save results to the database file
  // NB. Cannot use Toolbox since there is no scan file!!

  Char_t f_name[64];
  Char_t mname[32];

  FILE* outfile;


  for(module=0;module<MAX_MODULES;module++){
    if(module<e->GetMaxModules()){
      if(e->m[module]->present){

        // Open File

        e->GetModuleName(module,mname);
        sprintf(f_name,"%s%s_%d.txt",RESULTS_DIRECTORY,mname,date);
        outfile = fopen(f_name,"a+");
  
        if(outfile==NULL){
          printf("Could not open result outfile %s\n",f_name);
        } else {

          printf("File %s opened to append results\n",f_name);
          printf("     for HardReset test\n\n");

          // Write Header

          fprintf(outfile,"#\n");
          fprintf(outfile,"%%NewTest\n");
          fprintf(outfile,"#\n");

          fprintf(outfile,"SERIAL NUMBER  : %s\n",mname);
          fprintf(outfile,"TEST MADE BY   : %s\n",gSystem->Getenv("SCTDB_USER"));
          fprintf(outfile,"LOCATION NAME  : %s\n",LOCATION_NAME);
          fprintf(outfile,"Run number     : %d\n",e->runnum); // no scan number!!

          Int_t date2,mon,year,hour,min,sec;

          // ROOT date format is YYYYMMDD
          year = date/10000;
          mon  = (date/100)%100;
          date2 = date%100;

          // ROOT time format is HHMMSS
          hour = time/10000;
          min  = (time/100)%100;
          sec  = time%100;

          fprintf(outfile,"TEST_DATE      : %02d/%02d/%04d\n",date2,mon,year);

          fprintf(outfile,"PASSED         : ");
          if(  (status[0][module][0]==1)&&(status[0][module][1]==1)
             &&(status[1][module][0]==1)&&(status[1][module][1]==1)
             &&(status[2][module][0]==1)&&(status[2][module][1]==1)){
            printf("Module %s has PASSED!!\n",mname);
            fprintf(outfile,"YES\n");
          }else{
            printf("Module %s has FAILED!!\n",mname);
            fprintf(outfile,"NO\n");
          }

          fprintf(outfile,"PROBLEM        : ");
          fprintf(outfile,"NO\n");


          fprintf(outfile,"#\n");
          fprintf(outfile,"%%DAQ_INFO\n");
          fprintf(outfile,"#\n");

          fprintf(outfile,"#HOST\n");
          fprintf(outfile,"\"%s\"\n",gSystem->Getenv("COMPUTERNAME"));

          fprintf(outfile,"#VERSION\n");
          fprintf(outfile,"\"%s\"\n",ANALYSIS_VERSION);

          e->GetDeviceType(module,devtype);
          fprintf(outfile,"#DUT\n");
          fprintf(outfile,"\"%s\"\n",devtype);

          fprintf(outfile,"#TIME\n");
          fprintf(outfile,"\"%02d:%02d:%02d\"\n",hour,min,sec);


          fprintf(outfile,"#\n");
          fprintf(outfile,"%%DCS_INFO\n");
          fprintf(outfile,"#\n");

          fprintf(outfile,"#T0\tT1\n");
          fprintf(outfile,"%2.1f \t%2.1f\n",e->m[module]->DCS->T1,  e->m[module]->DCS->T2);

          fprintf(outfile,"#VDET\tIDET\n");
          fprintf(outfile,"%3.0f \t%1.3f\n",e->m[module]->DCS->vdet,e->m[module]->DCS->idet);

          fprintf(outfile,"#VCC\tICC\n");
          fprintf(outfile,"%1.2f \t%4.0f\n",e->m[module]->DCS->vcc, e->m[module]->DCS->icc);

          fprintf(outfile,"#VDD\tIDD\n");
          fprintf(outfile,"%1.2f \t%4.0f\n",e->m[module]->DCS->vdd, e->m[module]->DCS->idd);

          fprintf(outfile,"#TIME_POWERED\n");
          fprintf(outfile,".\n");

          // Record Currents
          fprintf(outfile,"#\n");
          fprintf(outfile,"%%HardReset\n");
          fprintf(outfile,"#\n");

          fprintf(outfile,"#NOCONFIG\tICC\tIDD\n");
          fprintf(outfile,"\t%4.0f \t%4.0f\n",icc_data[0][module],idd_data[0][module]);
          fprintf(outfile,"#NOCLOCK\tICC\tIDD\n");
          fprintf(outfile,"\t%4.0f \t%4.0f\n",icc_data[1][module],idd_data[1][module]);

          // Record Defects

          Int_t n_defects=0;
          for(link=0;link<MAX_LINKS;link++){
            if(status[0][module][link]==0){
              printf("Link %d:  NO_CLOCK\n",link);
              ListDefect(outfile,"HR_NOCLK",(link*768),((link+1)*768)-1);
              n_defects++;
            }
            if(status[1][module][link]==0){
              printf("Link %d:  NO_CONFIG\n",link);
              ListDefect(outfile,"HR_NOCON",(link*768),((link+1)*768)-1);
              n_defects++;
            }
            if(status[2][module][link]==0){
              printf("Link %d:  NO_RESET\n",link);
              ListDefect(outfile,"HR_NORST",(link*768),((link+1)*768)-1);
              n_defects++;
            }
          } // end for (link)

          if(n_defects==0){
            fprintf(outfile,"#No defects found!\n");
          }else{
            fprintf(outfile,"#%d defects found\n",n_defects);
          }

          fclose(outfile);

        } // end if(outfile)
      }
    }
  }

}
