void NoPlot(Int_t run, Int_t scan, Int_t log, Int_t graphics, Int_t postscript, Int_t online=0){

  //
  // Version 0.10 PWP 08.12.00
  // Version 0.11 PWP 22.01.01
  // Version 0.12 LE  14.01.01 - Works for data taken without SCTLV, correct 
  //                             output to text file and uses parameters.h
  // Version 0.13 PWP 23.05.01 - fix several bugs, many still to go!!
  // Version 0.14 PWP 20.06.01 - Plot log occ vs thr^2
  // Version 0.15 PWP 07.06.01 - Read trim target info from results file
  // Version 0.16 PWP 06.08.01 - Read RC data and do full ln(occ) vs thr^2 fit
  //                             (thanks to JGK and PJB for finding the main bug!!)
  // version 0.20 PWP 12.09.01 - Comment out NO_LO defect; further DB compliance
  // version 0.21 PWP 28.01.02 - trivial bugfix re. defects
  // version 0.22 PWP 01.02.02 - close canvas when done
  // version 0.23 PWP 21.03.02 - add web link to postscript plot if WEB_KEY defined
  //                             place leading zero in PS file timestamp
  // version 0.24 PWP 27.03.02 - extend f_name char array to 128 in length
  //                             to accomodate long web link filenames
  // version 0.25 PWP 11.04.02 - use each chip's own trim target to evaluate occupancy at 1fC.
  //                             Detect trim method (global or chip by chip) and show occupancy
  //                             marker and value at 1fC only if global method in use.
  // version 0.26 BJG 30.04.02 - Put DUT type on plot
  //                             Fix some memory leaks
  // version 0.27 BJG 23.09.02 - Draw Scurves
  // version 0.28 PWP 24.09.02 - Load NoScurves only once
  // version 0.30 PWP 28.11.02 - if running online, use batch mode. Plots will not appear, but
  //                             this avoids some memory leakage in TWIN32Canvas
  // version 0.31 PWP 28.11.02 - add call to gPad after TCanvas creation. 
  //                             This solves a crashing problem that occurs only at my desktop...

  // Note that this macro will only work correctly if the user has
  // valid response curve and trim data for each module stored in 
  // appropriate result summary files within the results directory.
  // The software will use the _MOST RECENT_ RC and trim data from
  // this directory: it is left to the user to ensure that such data
  // is still applicable.  PWP 06.08.01

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Double_t LOW_SCALE = 10E-8;
  Double_t occ[3];
  Double_t onefc[3];
  Double_t onefc_bin[3];
  Double_t onefc_bin2[MAX_LINKS][MAX_CHIPS];

  Int_t module, link, chip, chan, i;
  char f_name[128],h_name[32], line[128];
  char mname[MAX_MODULES][32];
  char devtype[MAX_MODULES][32];

  Int_t  TPDate;
  Int_t  RCDate;
  Short_t RCFunction[MAX_LINKS][MAX_CHIPS];
  Float_t RCData[12][3];
  Float_t TrimTarget[MAX_LINKS][MAX_CHIPS];

  Float_t chip_no_off[MAX_LINKS][MAX_CHIPS];
  Float_t chip_mean_occ[MAX_LINKS][MAX_CHIPS];
  Float_t chip_rms_occ[MAX_LINKS][MAX_CHIPS];
  Float_t chip_est_enc[MAX_LINKS][MAX_CHIPS];

  // Locality options now moved to stan_util.cpp

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");
  gROOT->LoadMacro("ToolBox.cpp");
  gROOT->LoadMacro("NoScurves.cpp");

  // Global style variations
  stan_styles();
  gStyle->SetOptStat(0);

  // Create function objects

  TF1* f1[MAX_LINKS][MAX_CHIPS];
  TF1* f2[MAX_LINKS][MAX_CHIPS];
  for(link=0;link<MAX_LINKS;link++){
    for(chip=0;chip<MAX_CHIPS;chip++){
      sprintf(h_name,"f1_%d_%d",link,chip);
      f1[link][chip] = new TF1(h_name, erfc,  0, 1000, 3);
      f1[link][chip]->SetParNames("Constant","Mean","Sigma");
      sprintf(h_name,"f2_%d_%d",link,chip);
      f2[link][chip] = new TF1(h_name, "pol1",  0, 10);
    }
  }


  // 
  // 2. Load the Fitted Scan Data
  //

  // Find today's date - needed for file output

  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Int_t the_time = dt->GetTime();
  delete dt;

  sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan);

  TFile *strun = new TFile(f_name);

  if(strun==0){
    printf("Failed to open file %s - EXITING!!\n",f_name);
    return;
  }

  //
  // Open the output canvas
  //

  Int_t dowait=0;
  Int_t ps_page=0;
  Int_t screen_page=0;

  TCanvas* c3 =0;
  TPostScript* ps =0;

  if(graphics!=0){
    dowait=1;
  }

  if(online){ 
    // Set Batch mode to avoid memory leaks in TWIN32Canvas
    // PWP 28.11.02
    printf("Running in BATCH mode - plots will not appear\n");
    gROOT->SetBatch(1);
  }

  if((graphics!=0)||(postscript!=0)){
    graphics = 1;
    c3= new TCanvas ("c3","NO",1100,750);
    gPad->Modified(); // improves reliability? PWP 28.11.02
  }

  // Find histograms and graphs

  TH1F* hp_chip[MAX_LINKS][MAX_CHIPS];
  TGraphAsymmErrors* gr_chip[MAX_LINKS][MAX_CHIPS];

  TH2F* h_scan[MAX_LINKS];
  TH1F* h_proj[MAX_LINKS];
  TH1F* h_sum;
  TGraph* icc;
  TGraph* idd;
  TGraph* power;

  TAxis*  axis;
  TMarker* OccMark[3];
  TMarker* IccMark[MAX_LINKS];
  TMarker* IddMark[MAX_LINKS];
  TMarker* PowerMark[MAX_LINKS];

  TText* t_enc[MAX_LINKS][MAX_CHIPS];
  TText* t_occ[MAX_LINKS];
  TText* t_icc[MAX_LINKS];
  TText* t_idd[MAX_LINKS];
  TText* t_power[MAX_LINKS];

  TH1F *h_nevent= (TH1F*)strun->Get("h_scan_evcnt");

  TObjString* time  = 0;
  TObjString* place = 0;
  TObjString* name = 0;
  TObjString* type = 0;

  for(module=0;module<MAX_MODULES;module++){

    // Initialise pointers to graphs and histograms

    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        hp_chip[link][chip]=0;
        gr_chip[link][chip]=0;
      }
      h_scan[link]=0;
      h_proj[link]=0;
    }
    h_sum=0;

    icc = 0;
    idd = 0;
    power = 0;

    // Markers and TText objects

    for(link=0;link<3;link++){
      occ[link]=0;
      onefc[link]=0;
      onefc_bin[link]=0;
      OccMark[link]=0;
    }

    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        t_enc[link][chip]=0;
        onefc_bin2[link][chip]=0;
      }
      IccMark[link]=0;
      IddMark[link]=0;
      PowerMark[link]=0;

      t_occ[link]=0;
      t_icc[link]=0;
      t_idd[link]=0;
      t_power[link]=0;
    }

    
    if(time) delete time;
    if(place) delete place;

    time  = (TObjString*)strun->Get("Time");
    place = (TObjString*)strun->Get("Place");
    sprintf(h_name,"ModuleName;%d",module+1);
    name = (TObjString*)strun->Get(h_name);
    if(name) {
      sprintf(mname[module],"%s",name->GetName());
      delete name;
    }

    sprintf(h_name,"DUT;%d",module+1);
    type = (TObjString*)strun->Get(h_name);
    if(type) {
      sprintf(devtype[module],"%s",type->GetName());
      delete type;
    } else {
      sprintf(devtype[module],"");
    }


    for(link=0;link<MAX_LINKS;link++){
      sprintf(h_name,"h_scan%d;%d",link,module+1);
      h_scan[link]= (TH2F*)strun->Get(h_name);
    }


    // proceed only if there is data for either link

    if((h_scan[0]!=0)&&(h_scan[1]!=0)){
      if((h_scan[0]->GetEntries()>0)||(h_scan[1]->GetEntries()>0)){

        // First we clear a few arrays
        printf("Preparing to analyse data for module %d\n",module);
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            RCFunction[link][chip]=0;
            TrimTarget[link][chip]=0;

            chip_no_off[link][chip]=0;
            chip_mean_occ[link][chip]=0;
            chip_rms_occ[link][chip]=0;
            chip_est_enc[link][chip]=0;
          }
        }

        // Find the TrimTarget information for this module 

        Int_t global_trim = 1;

        TPDate = FindTP("%TrimSummary",strun,module,&TrimTarget[0][0]);
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            onefc[link]+=TrimTarget[link][chip];
            if(TrimTarget[link][chip]!=TrimTarget[0][0]){
              global_trim=0;
            }
          }
          onefc[link]/=6;
        }
        onefc[2]=(onefc[0]+onefc[1])/2;

        if(global_trim){
          printf("Global trim detected\n");
        }else{
          printf("Chip by chip trim detected\n");
        }

        // Find the RC information for this module 

        RCDate = FindRCData("%ModuleResponseCurve",strun,module,&RCFunction[0][0],RCData);

        if(TPDate<0){
          printf("WARNING: Trim Data not found!!\n");
        }
        if(RCDate<0){
          printf("WARNING: Response Curve Data not found!!\n");
        }

        printf("Analysing data for module %d\n",module);

        axis = (TAxis*)h_scan[0]->GetYaxis();

        for(link=0;link<MAX_LINKS;link++){
          sprintf(h_name,"h_proj_%d",link);
          h_scan[link]->ProjectionY(h_name,0,767);
          h_proj[link]= (TH1F*)gDirectory->GetList()->FindObject(h_name);
          h_proj[link]->Scale(1.0/768);
          h_proj[link]->SetXTitle(axis->GetTitle());

          onefc_bin[link] = h_proj[link]->GetXaxis()->FindBin(onefc[link]);
          occ[link] = h_proj[link]->GetBinContent(onefc_bin[link]);
        }

        // Find 1fC bin for each chip PWP 11.04.02
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            onefc_bin2[link][chip] = h_proj[link]->GetXaxis()->FindBin(TrimTarget[link][chip]);
          }
        }


        // sum the two histograms
        sprintf(h_name,"h_sum");
        h_scan[0]->ProjectionY(h_name,0,767);
        h_sum = (TH1F*)gDirectory->GetList()->FindObject(h_name);
        h_sum->Scale(1.0/768);
        h_sum->Add(h_proj[1],1);
        h_sum->Scale(1.0/2);
        h_sum->SetXTitle(axis->GetTitle());

        onefc_bin[2] = h_sum->GetXaxis()->FindBin(onefc[2]);
        occ[2] = h_sum->GetBinContent(onefc_bin[2]);

        TAxis* axis = h_sum->GetXaxis();

        //
        // Analyse the data chip by chip - LE 15/5 2001
        // Revised to include error bars PWP 25.05.01
        //

        Float_t x[MAX_TARGETS];
        Float_t y[MAX_TARGETS];
        Float_t ex[MAX_TARGETS];
        Float_t eyl[MAX_TARGETS];
        Float_t eyh[MAX_TARGETS];

        Int_t bin;
        Float_t err2,n,z;
	
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
	    
	    // Project one histogram per chip	
            sprintf(h_name,"hp_chip_%d_%d",link,chip);
            h_scan[link]->ProjectionY(h_name,(chip*128),((chip+1)*128)-1);
            hp_chip[link][chip] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
	    
            if(hp_chip[link][chip]->GetMaximum()>0.0){ // data for this chip
              hp_chip[link][chip]->Scale(1.0/128);

              // Calculate errors appropriate to binomial statistics
              for(bin=1;bin<hp_chip[link][chip]->GetNbinsX()+1;bin++){
                n = h_nevent->GetBinContent(bin);
                if (n==0) n=1;
                z = hp_chip[link][chip]->GetBinContent(bin);
                err2 = (1+(n*z))*(1+(n-(n*z))) / n;
                hp_chip[link][chip]->SetBinError(bin,sqrt(err2)/n);
              }

              // Form graph of log occ vs thr^2 for this chip

              Int_t m=0;
              Float_t p0 = RCData[chip+(6*link)][0];
              Float_t p1 = RCData[chip+(6*link)][1];
              Float_t p2 = RCData[chip+(6*link)][2];

              for(bin=1;bin<hp_chip[link][chip]->GetNbinsX()+1;bin++){
                if((hp_chip[link][chip]->GetBinContent(bin)>0)
                  &&(hp_chip[link][chip]->GetBinContent(bin)<0.5)){

                  Float_t thr = hp_chip[link][chip]->GetBinCenter(bin);
                  
                  switch(RCFunction[link][chip]){
                    case 4:  // linear
                      if(p1!=0){
                        x[m] = (thr - p0) / p1;
                        x[m] *= x[m];
                      }
                      break;
                    case 3:  // exponential
                      if((thr-p2)!=0){
                        err2 =(p0/(thr-p2)-1);
                        if(err2>0){
                          // Conflict with log argument!!!
                          x[m] = p1*TMath::Log(err2);
                          x[m] *= x[m];
                        }
                      }
                      break;
                    case 2:  // grillo
                      break;
                    case 1:  // polynomial
                      if(p0!=0){
                        x[m] = (-p1 + sqrt((p1*p1)-(4*p0*p2))) / 2*p0;
                        x[m] *= x[m];
                      }
                      break;
                    default: // NO RESPONSE CURVE
                      x[m] = hp_chip[link][chip]->GetBinCenter(bin);
                      break;
                  } 

                  ex[m] = 0; // TODO something better!!
                  y[m]  = TMath::Log(hp_chip[link][chip]->GetBinContent(bin));
                  // data + error 
                  Float_t ahg1  = TMath::Log(hp_chip[link][chip]->GetBinContent(bin)
                                       + hp_chip[link][chip]->GetBinError(bin));
                  eyh[m] = ahg1 - y[m];

                  // data - error
                  if((hp_chip[link][chip]->GetBinContent(bin)-hp_chip[link][chip]->GetBinError(bin))>0){
                    Float_t ahg2  = TMath::Log(hp_chip[link][chip]->GetBinContent(bin)
                                         - hp_chip[link][chip]->GetBinError(bin));
                    eyl[m] = y[m] - ahg2;
                  }else{
                    eyl[m]=eyh[m];
                  }
                  //printf("m %d y %1.1e ahg1 %1.1e eyh %1.1e ahg2 %1.1e ehl %1.1e, err %1.1e\n",
                  //        m,y[m],ahg1,eyh[m],ahg2,eyl[m], hp_chip[link][chip]->GetBinError(bin));
                  m++;
                }
              }
              
              if(m>0){
                gr_chip[link][chip]= new TGraphAsymmErrors(m,x,y,ex,ex,eyl,eyh);

                // Fit straight line
                sprintf(h_name,"f2_%d_%d",link,chip);
                if(x[m-1]>=0.5){ // probably a module
                  f2[link][chip]->SetRange(0.3,2);
                }else{           // probably a hybrid
                  f2[link][chip]->SetRange(0.1,1);
                }
                gr_chip[link][chip]->Fit(h_name,"NRQ"); // no graphics, quiet mode
                if(f2[link][chip]->GetParameter(1)!=0){
                  err2 = -1/(2*f2[link][chip]->GetParameter(1));
                }else{
                  err2=0; // in case fit fails, return zero noise (???)
                }
                chip_est_enc[link][chip] = 6250* sqrt(err2);
              }else{
                printf("No valid NO data for link %d chip %d\n",link,chip);
              }

              // Calculate mean occupancy
              chip_mean_occ[link][chip]=0;
              for(chan=chip*128;chan<(chip+1)*128;chan++){
                chip_mean_occ[link][chip] += h_scan[link]->GetBinContent(770*(onefc_bin2[link][chip])+chan);
              }
              chip_mean_occ[link][chip]/=128;

              // Calculate RMS occupancy
              chip_rms_occ[link][chip]=0;
              for(chan=chip*128;chan<(chip+1)*128;chan++){
                chip_rms_occ[link][chip] += 
                  TMath::Power((chip_mean_occ[link][chip] - h_scan[link]->GetBinContent(770*(onefc_bin2[link][chip])+chan)),2);
              }
              chip_rms_occ[link][chip]/=128;
              chip_rms_occ[link][chip]=TMath::Power(chip_rms_occ[link][chip],0.5);
      
              // Fit Erfc to the projection.
              Float_t centre = (find_bin2(hp_chip[link][chip],0.99,0,1)+find_bin2(hp_chip[link][chip],0.99,1,0))/2;
              Float_t p0_guess = hp_chip[link][chip]->GetMaximum();
              Float_t p1_guess = find_level2(hp_chip[link][chip],0.50,1);          // last bin > 50%
              Float_t p2_guess = find_level2(hp_chip[link][chip],0.82,1)-p1_guess;  
              Float_t range_lo = centre;
              Float_t range_hi = hp_chip[link][chip]->GetBinCenter(hp_chip[link][chip]->GetNbinsX());
              if(p2_guess<0) p2_guess = p2_guess*-1.0;
      
              f1[link][chip]->SetParameters(p0_guess,p1_guess,p2_guess);
              f1[link][chip]->SetParLimits(0,p0_guess,p0_guess);
              f1[link][chip]->SetRange(TrimTarget[link][chip]-20,200);  // fits data, but why?  trimming??


              sprintf(h_name,"f1_%d_%d",link,chip);
              hp_chip[link][chip]->Fit(h_name,"NRQ"); // no graphics, quiet mode
              chip_no_off[link][chip] = f1[link][chip]->GetParameter(1); 
	    }else{
              // no data for this chip
              gr_chip[link][chip]=0;
            }
	  }
	}
	

        sprintf(h_name,"Icc;%d",module+1);
        icc = (TGraph*)strun->Get(h_name);
        sprintf(h_name,"Idd;%d",module+1);
        idd = (TGraph*)strun->Get(h_name);

	if(icc&&idd){

          // Since v2.25 TGraph used Double_t for internal storage
          // before it used Float_t.  Must choose appropriate types
          if(gROOT->GetVersionInt()<22500){
            Float_t* xdata;
            Float_t* icc_data;
            Float_t  power_data[160];  // was 128 PWP 05.12.01
            Float_t* idd_data;
 
            Float_t  icc_xmax = -1;
            Float_t  icc_ymax=0;
            Float_t  idd_xmax = -1;
            Float_t  idd_ymax=0;
            Float_t  power_xmax = -1;
            Float_t  power_ymax=0;

            Float_t  icc_typ;
            Float_t  idd_typ;
            Float_t  power_typ;

          }else{

            Double_t* xdata;
            Double_t* icc_data;
            Double_t* idd_data;
            Double_t  power_data[160];  // was 128 PWP 05.12.01

            Double_t  icc_xmax = -1;
            Double_t  icc_ymax=0;
            Double_t  idd_xmax = -1;
            Double_t  idd_ymax=0;
            Double_t  power_xmax = -1;
            Double_t  power_ymax=0;

            Double_t  icc_typ;
            Double_t  idd_typ;
            Double_t  power_typ;

	  }

	  xdata = icc->GetX();
	  icc_data = icc->GetY();
	  idd_data = idd->GetY();
	  for(i=0;i<icc->GetN();i++){
	    if(icc_data[i]>icc_ymax){
	      icc_xmax = xdata[i];
	      icc_ymax = icc_data[i];
	    }
	    if(idd_data[i]>idd_ymax){
	      idd_xmax = xdata[i];
	      idd_ymax = idd_data[i];
	    }
	    power_data[i]=(0.0035*icc_data[i])+(0.004*idd_data[i]);
	    if(power_data[i]>power_ymax){
	      power_xmax = xdata[i];
	      power_ymax = power_data[i];
	    }
	  }

          // bugfix for case where scan stops at threshold < 1fC
          // PWP 29.01.02

          Int_t use_bin = onefc_bin[2]-1;
          if(use_bin>(icc->GetN()-1)){
            use_bin =(icc->GetN()-1);
          }

	  icc_typ=icc_data[use_bin];
	  idd_typ=idd_data[use_bin];
	  power_typ=power_data[use_bin];
	  
	  printf("Icc max %3.0fmA at threshold %3.2f\n",icc_ymax,icc_xmax);
	  printf("Icc typ %3.0fmA at threshold %3.2f\n",icc_typ,onefc[2]);
	  printf("Idd max %3.0fmA at threshold %3.2f\n",idd_ymax,idd_xmax);
	  printf("Idd typ %3.0fmA at threshold %3.2f\n",idd_typ,onefc[2]);
	  printf("Power max %1.2fW at threshold %3.2f\n",power_ymax,power_xmax);
	  printf("Power typ %1.2fW at threshold %3.2f\n",power_typ,onefc[2]);

	  power = new TGraph((i),xdata,power_data);
	}

        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            printf("  link %d chip %d mn_occ %2.1e rms_occ %2.1e enc %4.0f\n",
              link,chip,chip_mean_occ[link][chip],chip_rms_occ[link][chip],chip_est_enc[link][chip]);
          }
        }

        if(c3){
          if(postscript){
            ps_page=0;
            sprintf(f_name,"%s%s_NoPlot_%d_%06d.ps",PS_DIRECTORY,mname[module],date,the_time);

            ps = new TPostScript(f_name,112);
          }

          if((screen_page>0)&&(dowait)){
            printf("Enter a string to continue\n");
            scanf("%s",line);
          }

          printf("Generating Plots for module %d\n",module);

          ps_page++;
          screen_page++;
          if(ps){
            ps->NewPage();
          }


          c3->cd();
          c3->Clear();

          TPad *pad_t = new TPad("pad_t","Text",        0.00,0.90,1.00,1.00,33);
          TPad *pad_s = new TPad("pad_s","Scans",       0.00,0.00,0.60,0.90,33);
          TPad *pad_p = new TPad("pad_p","Projections", 0.60,0.00,1.00,0.90,33);

          pad_t->Draw();
          pad_s->Draw();
          pad_s->Divide(1,2,0,0);
          pad_p->Draw();
          pad_p->Divide(1,2,0,0);

          for(link=0;link<MAX_LINKS;link++){
            pad_s->cd(link+1);
            gPad->SetLogz(log);
            pad_p->cd(link+1);
            gPad->SetLogy(log);
          }

          TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
          t1->SetBit(kCanDelete);

          t1->SetFillColor(10);
          t1->SetTextAlign(13);
          t1->SetTextSize(0.3);
          t1->SetTextColor(4);

          // ATLAS SCT Noise Occupancy  - log scale  -  time -- place
          sprintf(line,"ATLAS SCT Noise Occupancy");
          if(log) sprintf(line,"%s  -  log scale",line);
          else    sprintf(line,"%s  -  linear scale",line);
          if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
          if(place) sprintf(line,"%s  -  %s",line,place->GetName());
          t1->AddText(line);

          // Page %d   Run %d   Scan %d   Module %d (%s)
          sprintf(line,"Page %d   Run %d   Scan %d   Module %d (%s)", 
                  ps_page, run, scan, module, mname[module]);
          if(strlen(devtype[module]) > 0) 
            sprintf(line, "%s - Type %s", line, devtype[module]);
          t1->AddText(line);
          pad_t->cd();
          t1->Draw();

          for(link=0;link<MAX_LINKS;link++){
            pad_s->cd(link+1);
            gPad->SetLeftMargin(0.10);
            gPad->SetBottomMargin(0.15);
            h_scan[link]->SetTitleOffset(1.7,"X");
            h_scan[link]->SetTitleOffset(1.0,"Y");
            if(log){
              h_scan[link]->SetMinimum(LOW_SCALE);
            }
            h_scan[link]->SetXTitle("Channel Number");
            h_scan[link]->Draw("COLZ");

            pad_p->cd(link+1);
            gPad->SetLeftMargin(0.15);
            gPad->SetBottomMargin(0.15);
            h_proj[link]->SetTitleOffset(1.7,"X");
            h_proj[link]->SetTitleOffset(1.7,"Y");
            if(log){
              h_proj[link]->SetYTitle("Log(occupancy)");
              h_proj[link]->SetMinimum(LOW_SCALE);
            } else {
              h_proj[link]->SetYTitle("Occupancy");
            }
            h_proj[link]->Draw();
            // Show occupancy marker and value only if global trim in use.
            // Value is nonsensical for chip by chip trimming.  PWP 11.04.02
            if(global_trim){
              if(log){
                OccMark[link] = new TMarker (onefc[link],log10(occ[link]),23);
              } else {
                OccMark[link] = new TMarker (onefc[link],occ[link],23);
              }

              OccMark[link]->SetMarkerColor(kBlue);
              OccMark[link]->SetMarkerSize(1.0);
              OccMark[link]->SetMarkerStyle(20);
              OccMark[link]->Draw("same");

              t_occ[0] = new TText();
              sprintf(h_name,"%1.2E",occ[link]);
              if(log){
                t_occ[0]->DrawText(onefc[link]+10,log10(occ[link]),h_name);
              }else{
                t_occ[0]->DrawText(onefc[link]+10,occ[link],h_name);
              }
            }
          }


          c3->Update();

          ps_page++;
          screen_page++;
          if(ps){
            ps->NewPage();
          }


          if((screen_page>0)&&(dowait)){
            printf("Enter a string to continue\n");
            scanf("%s",line);
          }

          c3->cd();
          c3->Clear();

          TPad *pad_g = 0;
          TPad *pad_o = 0;

	  if(icc&&idd){
	    pad_t = new TPad("pad_t","Text",        0.00,0.90,1.00,1.00,33);
	    pad_o = new TPad("pad_o","Occupancy",   0.00,0.00,0.60,0.90,33);
	    pad_g = new TPad("pad_g","Graphs", 0.60,0.00,1.00,0.90,33);
	    pad_t->Draw();
	    pad_o->Draw();
	    pad_g->Draw();
	    pad_g->Divide(1,3,0,0);

	  }else{
	    pad_t = new TPad("pad_t","Text",        0.00,0.90,1.00,1.00,33);
	    pad_o = new TPad("pad_o","Occupancy",   0.00,0.00,1.00,0.90,33);
	    pad_t->Draw();
	    pad_o->Draw();
          }

          t1 = new TPaveText(0.0,0.0,1.0,1.0);
          t1->SetBit(kCanDelete);

          t1->SetFillColor(10);
          t1->SetTextAlign(13);
          t1->SetTextSize(0.3);
          t1->SetTextColor(4);
          sprintf(line,"ATLAS SCT Noise Occupancy");
          if(log) sprintf(line,"%s  -  log scale",line);
          else    sprintf(line,"%s  -  linear scale",line);
          if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
          if(place) sprintf(line,"%s  -  %s",line,place->GetName());
          t1->AddText(line);

          sprintf(line,"Page %d   Run %d   Scan %d   Module %d (%s)", 
                  ps_page, run, scan, module, mname[module]);
          if(strlen(devtype[module]) > 0) 
            sprintf(line, "%s - Type %s", line, devtype[module]);
          t1->AddText(line);
          pad_t->cd();
          t1->Draw();

          if(log){
            pad_o->SetLogy();
          }

          pad_o->cd();
          gPad->SetLeftMargin(0.15);
          gPad->SetBottomMargin(0.15);
          h_sum->SetTitleOffset(1.7,"X");
          h_sum->SetTitleOffset(2.0,"Y");
          if(log){
            h_sum->SetYTitle("Log(occupancy)");
            h_sum->SetMinimum(LOW_SCALE);
          } else {
            h_sum->SetYTitle("Occupancy");
          }
          h_sum->Draw();

          // Show occupancy marker and value only if global trim in use.
          // Value is nonsensical for chip by chip trimming.  PWP 11.04.02
          if(global_trim){
            if(log){
              OccMark[2]=new TMarker (onefc[2],log10(occ[2]),23);
            }else{
              OccMark[2]=new TMarker (onefc[2],occ[2],23);
            }
            OccMark[2]->SetMarkerColor(kBlue);
            OccMark[2]->SetMarkerSize(1.0);
            OccMark[2]->SetMarkerStyle(20);
            OccMark[2]->Draw("same");

            t_occ[1] = new TText();
            sprintf(h_name,"%1.2E",occ[2]);
            if(log){
              t_occ[1]->DrawText(onefc[2]+10,log10(occ[2]),h_name);
            }else{
              t_occ[1]->DrawText(onefc[2]+10,occ[2],h_name);
            }
          }

          if(idd&&idd){
            pad_g->cd(1);
            gPad->SetLeftMargin(0.12);
            gPad->SetBottomMargin(0.15);
            icc->SetLineColor(kRed);
            icc->SetMarkerColor(kRed);
            icc->Draw("ALP");
            TH1F* h_icc=(TH1F*) icc->GetHistogram();
            h_icc->SetXTitle(axis->GetTitle());
            h_icc->SetYTitle("Icc(mA)");
            h_icc->SetTitleOffset(1.7,"X");
            h_icc->SetTitleOffset(1.3,"Y");
    
            IccMark[0]=new TMarker (onefc[2],icc_typ,23); 
            IccMark[0]->SetMarkerColor(kBlue);
            IccMark[0]->SetMarkerSize(0.5);
            IccMark[0]->SetMarkerStyle(20);
            IccMark[0]->Draw("same");
	    
            IccMark[1]=new TMarker (icc_xmax,icc_ymax,23); 
            IccMark[1]->SetMarkerColor(kBlue);
            IccMark[1]->SetMarkerSize(0.5);
            IccMark[1]->SetMarkerStyle(20);
            IccMark[1]->Draw("same");
    
            t_icc[0] = new TText();
            sprintf(h_name,"%3.0fmA Typical",icc_typ);
            t_icc[0]->DrawText(onefc[2]+5,icc_typ,h_name);
	   
            t_icc[1] = new TText();
            sprintf(h_name,"%3.0fmA Max",icc_ymax);
            t_icc[1]->DrawText(icc_xmax+5,icc_ymax,h_name);


            pad_g->cd(2);
            gPad->SetLeftMargin(0.12);
            gPad->SetBottomMargin(0.15);
            idd->SetLineColor(kRed);
            idd->SetMarkerColor(kRed);
            idd->Draw("ALP");
            TH1F* h_idd=(TH1F*) idd->GetHistogram();
            h_idd->SetXTitle(axis->GetTitle());
            h_idd->SetYTitle("Idd(mA)");
            h_idd->SetTitleOffset(1.7,"X");
            h_idd->SetTitleOffset(1.3,"Y");
    
            IddMark[0]=new TMarker (onefc[2],idd_typ,23); 
            IddMark[0]->SetMarkerColor(kBlue);
            IddMark[0]->SetMarkerSize(0.5);
            IddMark[0]->SetMarkerStyle(20);
            IddMark[0]->Draw("same");
    
            IddMark[1]=new TMarker (idd_xmax,idd_ymax,23); 
            IddMark[1]->SetMarkerColor(kBlue);
            IddMark[1]->SetMarkerSize(0.5);
            IddMark[1]->SetMarkerStyle(20);
            IddMark[1]->Draw("same");
    
            t_idd[0] = new TText();
            sprintf(h_name,"%3.0fmA Typical",idd_typ);
            t_idd[0]->DrawText(onefc[2]+5,idd_typ,h_name);
   
            t_idd[1] = new TText();
            sprintf(h_name,"%3.0fmA Max",idd_ymax);
            t_idd[1]->DrawText(idd_xmax+5,idd_ymax,h_name);
    
    
            pad_g->cd(3);
            gPad->SetLeftMargin(0.12);
            gPad->SetBottomMargin(0.15);
            sprintf(h_name,"Module %d %s Power",module,mname[module]);
            power->SetTitle(h_name);
            power->SetLineColor(kRed);
            power->SetMarkerColor(kRed);
            power->Draw("ALP");
            TH1F* h_power=(TH1F*) power->GetHistogram();
            h_power->SetXTitle(axis->GetTitle());
            h_power->SetYTitle("Power(W)");
            h_power->SetTitleOffset(1.7,"X");
            h_power->SetTitleOffset(1.3,"Y");
   
            PowerMark[0]=new TMarker (onefc[2],power_typ,23); 
            PowerMark[0]->SetMarkerColor(kBlue);
            PowerMark[0]->SetMarkerSize(0.5);
            PowerMark[0]->SetMarkerStyle(20);
            PowerMark[0]->Draw("same");
    
            PowerMark[1]=new TMarker (power_xmax,power_ymax,23); 
            PowerMark[1]->SetMarkerColor(kBlue);
            PowerMark[1]->SetMarkerSize(0.5);
            PowerMark[1]->SetMarkerStyle(20);
            PowerMark[1]->Draw("same");
    
            t_power[0] = new TText();
            sprintf(h_name,"%1.2fW Typical",power_typ);
            t_power[0]->DrawText(onefc[2]+5,power_typ,h_name);
    
            t_power[1] = new TText();
            sprintf(h_name,"%1.2fW Max",power_ymax);
            t_power[1]->DrawText(power_xmax+5,power_ymax,h_name);
          } // end if(icc&&idd)

          c3->Update();

          ps_page++;
          screen_page++;
          if(ps){
            ps->NewPage();
          }


          if((screen_page>0)&&(dowait)){
            printf("Enter a string to continue\n");
            scanf("%s",line);
          }

          c3->cd();
          c3->Clear();

          pad_t = new TPad("pad_t","Text",        0.00,0.90,1.00,1.00,33);
          pad_p = new TPad("pad_p","Projections",   0.00,0.00,1.00,0.90,33);

          pad_t->Draw();
          pad_p->Draw();
          pad_p->Divide(4,3,0,0);

          t1 = new TPaveText(0.0,0.0,1.0,1.0);
          t1->SetBit(kCanDelete);

          t1->SetFillColor(10);
          t1->SetTextAlign(13);
          t1->SetTextSize(0.3);
          t1->SetTextColor(4);
          sprintf(line,"ATLAS SCT Noise Occupancy");
          if(log) sprintf(line,"%s  -  log scale",line);
          else    sprintf(line,"%s  -  linear scale",line);
          if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
          if(place) sprintf(line,"%s  -  %s",line,place->GetName());
          t1->AddText(line);

          sprintf(line,"Page %d   Run %d   Scan %d   Module %d (%s)", 
                  ps_page, run, scan, module, mname[module]);
          if(strlen(devtype[module]) > 0) 
            sprintf(line, "%s - Type %s", line, devtype[module]);
          t1->AddText(line);
          pad_t->cd();
          t1->Draw();

          pad_p->cd();
          for(link=0;link<MAX_LINKS;link++){
            for(chip=0;chip<MAX_CHIPS;chip++){

              if(gr_chip[link][chip]){
                pad_p->cd((link*6)+chip+1);

                gr_chip[link][chip]->SetMarkerColor(kBlue);
                gr_chip[link][chip]->Draw("AP");

                gr_chip[link][chip]->GetHistogram()->GetYaxis()->SetRange(-25,0);
                sprintf(h_name,"Noise Occupancy link %d chip %d", link, chip);
                gr_chip[link][chip]->GetHistogram()->SetTitle(h_name);

                if(RCDate>0){
                  gr_chip[link][chip]->GetHistogram()->SetXTitle("Threshold^2 (fC^2)");
                }else{
                  gr_chip[link][chip]->GetHistogram()->SetXTitle("Threshold (mV)");
                }
                gr_chip[link][chip]->GetHistogram()->SetYTitle("ln(occupancy)");
                if(f2[link][chip]){
                  f2[link][chip]->SetLineColor(kRed);
                  f2[link][chip]->Draw("same");
    
                  t_enc[link][chip] = new TText();
                  sprintf(h_name,"%4.0f ENC",chip_est_enc[link][chip]);
                  t_enc[link][chip]->DrawText(0,-20,h_name);
                }
              }
            }
          }

          c3->Update();

          if(ps){
            ps->Close();
            delete ps;
          }

        } //end if(c3)


        // 
        // Output data to text summary file
        //

        // Open file and print data-base header
        FILE* outfile = OpenResults("NoiseOccupancy", strun, module, run, scan, 1,0); 
       
        if(outfile==NULL){
          return;
        }

        fprintf(outfile,"#\n");
        fprintf(outfile,"%%NO\n");
        fprintf(outfile,"#\n");

        // Print test-specific results
        if(icc&&idd){
         fprintf(outfile,"#MAX\tICC\tIDD\n");
         fprintf(outfile,"\t%3.0f \t%3.0f\n",icc_ymax,idd_ymax);
         fprintf(outfile,"#TYP\tICC\tIDD\n");
         fprintf(outfile,"\t%3.0f \t%3.0f\n",icc_typ,idd_typ);
         fprintf(outfile,"#\n");
        }
       
        fprintf(outfile,"#\tOffset\tMeanOcc   \tRMSOcc    \tEstENC\n");
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
	    fprintf(outfile,"#%s\n",MseLabel(link,chip));
	    fprintf(outfile,"\t%3.1f \t%1.1e \t%1.1e \t%4.0f\n",
              chip_no_off[link][chip],
              chip_mean_occ[link][chip],
              chip_rms_occ[link][chip],
              chip_est_enc[link][chip]);
          }
        }
        fprintf(outfile,"#\n");


        Int_t first = -1;
        Int_t last = -1;
        Int_t n_defects=0;
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){

            first = -1;
            last  = -1;

            for(chan=(chip*128);chan<((chip+1)*128);chan++){
              if(h_scan[link]->GetCellContent(chan,onefc_bin2[link][chip])>5e-4){
                if(first==-1){
                  first = (link*MAX_CHANS)+chan;
                }
                last = (link*MAX_CHANS)+chan;
              }else if(first>-1){
                ListDefect(outfile,"NO_HI",first,last);
                n_defects++;
                first=-1;
                last=-1;
              }
            }

            if(first>-1){
              ListDefect(outfile,"NO_HI",first,last);
              n_defects++;
            }
          }
        }

        if(n_defects==0){
          fprintf(outfile,"#No defects found!\n");
        }else{
          fprintf(outfile,"#%d defects found\n",n_defects);
        }

        // Add a URL to the postscript file

  #ifdef WEB_KEY        
        if(postscript){
          sprintf(f_name,"%s%s_NoPlot_%d_%06d.ps",WEB_KEY,mname[module],date,the_time);
          ListWebLink(outfile,"Plots (postscript)",f_name);
          sprintf(f_name,"%s%s_NoScurve_%d_%06d.ps",WEB_KEY,mname[module],date,the_time);
          ListWebLink(outfile,"Scurves (postscript)",f_name);
        }
  #endif

        fclose(outfile);

        NoScurves(run, scan, module, mname[module], date, the_time);
//          NoScurves(run, scan, module, 1);

      } else {
        printf("No data for module %d\n",module);
      }
    } else {
      printf("No histograms for module %d\n",module);
    }

    // Delete graph and histogram objects

    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        if(hp_chip[link][chip]) hp_chip[link][chip]->Delete();
        if(gr_chip[link][chip]) gr_chip[link][chip]->Delete();
      }
      if(h_proj[link]) h_proj[link]->Delete();
    }
    if(h_sum) h_sum->Delete();
    if(icc)   icc->Delete();
    if(idd)   idd->Delete();
    if(power) power->Delete();

    // Delete remaining objects
    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        if(t_enc[link][chip]) delete t_enc[link][chip];
      }
      if(IccMark[link]) delete IccMark[link];
      if(IddMark[link]) delete IddMark[link];
      if(PowerMark[link]) delete PowerMark[link];

      if(t_occ[link]) delete t_occ[link];
      if(t_icc[link]) delete t_icc[link];
      if(t_idd[link]) delete t_idd[link];
      if(t_power[link]) delete t_power[link];
    }

  }//end for(module)


  // Delete remaining objects

  if(time) delete time;
  if(place) delete place;

  printf("Deleting Function Objects\n");

  for(link=0;link<MAX_LINKS;link++){
    for(chip=0;chip<MAX_CHIPS;chip++){
      if(f1[link][chip]) f1[link][chip]->Delete();
      if(f2[link][chip]) f2[link][chip]->Delete();
    }
  }
  if(c3) c3->Delete();

  if(online){ 
    // Clear Batch mode so that online burst and scan hists will still appear
    // Canvas PWP 28.11.02
    printf("Clearing BATCH mode - online plots reenabled\n");
    gROOT->SetBatch(0);
  }

  if(strun) {
    strun->Close();
    delete strun;
  }
}

void NoPlot(){
  printf("Possible Uses of NoPlot:\n\n");
  printf(".x NoPlot.cpp(run,scan,log,graphics,postscript,online=0)\n");
}

