void NoScurves(short run, short scan, short moi, char *name, int date, int anal_time){
  // Revised Version 0.2 PWP 11.03.00
  // Revised Version 0.3 PWP 08.08.01 - use parameters.h
  // Revised version 0.31 PWP 12.04.02 - now knows about command delay scans
  // Revised Version 0.32 BJG 30.04.02 - Fix some memory leaks
  // Revised Version 0.33 PWP 29.05.02 - use canvas->Print instead of TPostscript
  // Revised Version 0.34 BJG 23.09.02 - Modified from stan_scurves
  // Revised Version 0.34 BJG 24.09.02 - Use more descriptive chip names
  // Revised Version 0.40 PWP/BJG 28.11.02 - Tidy up; Close strun; Fix leaks; 

#ifdef DATA_DIRECTORY
  // do not reload parameters.h
#else
  // must load parameters.h
#include "../macros/parameters.h"
#endif

  char f_name[64], h_name[32];
  char o_name[64];
  char labelt[10],s[256],scan_name[128];
  char devtype[32];

  float scan_start, scan_stop, scan_step;
  int   link,chip,chan;
  int   i, j, n;
  short scan_max_points;

  float x[128];
  float y[128];

  TFile *strun;
  TGraph *gr[MAX_LINKS][MAX_CHANS];

  TH2F* h_scan[MAX_LINKS];
  TH1F* h_nevent =0;

  // Global style variations

  gStyle.SetPaperSize(20,26); // I find that the postscript output
  // is mangled without this line - 
  // although this size (A4) should in
  // principle be the default...
  gStyle.SetMarkerStyle(21);
  gStyle.SetMarkerSize(0.3);
  gStyle.SetOptFit(0);
  gStyle.SetOptStat(10);
  gStyle.SetTitleW(0.6);

  printf("NoScurves called for module %d\n",moi);

  // Open ROOT data file
  sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan);
  printf("  Using ROOT Datafile %s\n",f_name);
  strun = new TFile(f_name);

  if(strun){

    // obtain pointers to histograms

    sprintf(h_name,"h_scan0;%d",moi+1);
    h_scan[0]= (TH2F*)strun->Get(h_name);

    sprintf(h_name,"h_scan1;%d",moi+1);
    h_scan[1]= (TH2F*)strun->Get(h_name);

    h_nevent = (TH1F*)strun->Get("h_scan_evcnt");

    sprintf(h_name,"ModuleName;%d",moi+1);
    TObjString *time  = (TObjString*)strun->Get("Time");
    TObjString *place = (TObjString*)strun->Get("Place");
    TObjString *mname = (TObjString*)strun->Get(h_name);

    TObjString *type;

    sprintf(o_name,"DUT;%d",moi+1);
    type = (TObjString*)strun->Get(o_name);
    if(type) {
      sprintf(devtype,"%s",type->GetName());
      delete type;
    } else {
      sprintf(devtype,"");
    }

    n = (h_scan[0]->GetMaximum());
    if (n<1.0) n =1.0;


    sprintf(scan_name,"THRESHOLD (mV)");

    // titles for graphs - might as well do this now.

    TAxis * axis = h_scan[0]->GetYaxis();
    scan_max_points = axis->GetNbins();
    scan_start = axis->GetBinCenter(1);
    scan_stop  = axis->GetBinCenter(scan_max_points);
    scan_step  = axis->GetBinWidth(1);

    printf("scan_start %f, stop %f, npoints %f\n",scan_start, scan_stop, scan_max_points);


    // Create Frame histograms
    TH2F *hr[4];
    hr[0] = new TH2F("cal0","scurves cal0",2,scan_start,scan_stop,2,-(n*0.1),(n*1.1));
    hr[1] = new TH2F("cal1","scurves cal1",2,scan_start,scan_stop,2,-(n*0.1),(n*1.1));
    hr[2] = new TH2F("cal2","scurves cal2",2,scan_start,scan_stop,2,-(n*0.1),(n*1.1));
    hr[3] = new TH2F("cal3","scurves cal3",2,scan_start,scan_stop,2,-(n*0.1),(n*1.1));
    hr[0]->SetXTitle(scan_name);
    hr[1]->SetXTitle(scan_name);
    hr[2]->SetXTitle(scan_name);
    hr[3]->SetXTitle(scan_name);

    // Create Label Objects
    TPaveText *tlabel[MAX_LINKS][MAX_CHIPS];

    for(link=0;link<MAX_LINKS;link++) {
      for(chip=0;chip<MAX_CHIPS;chip++){
        tlabel[link][chip] = new TPaveText(0.25,0.05,0.75,0.95);
        tlabel[link][chip]->SetBit(kCanDelete);

        tlabel[link][chip]->SetFillColor(10);  
        tlabel[link][chip]->SetTextAlign(13);
        tlabel[link][chip]->SetTextSize(0.8);
        tlabel[link][chip]->SetTextColor(4);
    
        // Use canonical chip names
        sprintf(labelt,"%c%d",chip==0?'M':(chip==5?'E':'S'), chip+(link*8));
        tlabel[link][chip]->AddText(0.25,0.0,labelt);
      }
    }

    // Build TGraph Objects
    for(i=0;i<scan_max_points; i++){
      x[i] = axis->GetBinCenter(i+1);
    }

    for(link=0;link<MAX_LINKS;link++) {
      for(chan=0;chan<768;chan++){
        for(j=0;j<scan_max_points; j++){
          y[j] = h_scan[link]->GetBinContent((770*(j+1))+chan);
        }
        gr[link][chan] = new TGraph(scan_max_points,x,y);
        gr[link][chan]->SetMarkerStyle(6);
        gr[link][chan]->SetMarkerColor((chan/4)%32);
        gr[link][chan]->SetLineColor((chan/4)%32);
      }
    }


    // Set Batch mode - we do not need to see the plots on line
    // gROOT->SetBatch(1);  // in normal use this is done by TrimRangePlot

    // Open canvas
    TCanvas* csc=new TCanvas ("csc",s,1100,750);

    // Open PS file
    sprintf(f_name,"%s%s_NoScurve_%d_%06d.ps",PS_DIRECTORY, name, date, anal_time);
    TPostScript* ps = new TPostScript(f_name,112);


    for(link=0;link<MAX_LINKS;link++) {
      if(ps) {
        ps->NewPage();
      }

      csc->cd();
      csc->Clear();

      sprintf(s,"%s - scurves Module %d Link %d", scan_name, moi, link);

      TPad *pad_t0 = new TPad("pad_t0","title",        0.00,0.950,1.00,1.000,33);
      TPad *pad_c0 = new TPad("pad_c0","Scurves",      0.00,0.030,1.00,0.950,33);
      TPad *pad_t1 = new TPad("pad_t1","title",        0.00,0.000,1.00,0.030,33);
      pad_t0->SetBit(kCanDelete);
      pad_c0->SetBit(kCanDelete);
      pad_t1->SetBit(kCanDelete);

      pad_c0->Divide(6,4,0,0);
      pad_t1->Divide(6,1,0,0);

      pad_t0->Draw();
      pad_t1->Draw();
      pad_c0->Draw();

      TPaveText *ttitle = new TPaveText(0.01,0.05,0.99,0.95);
      ttitle->SetBit(kCanDelete);

      ttitle->SetFillColor(10);  
      ttitle->SetTextAlign(13);
      ttitle->SetTextSize(0.4);
      ttitle->SetTextColor(4);

      sprintf(s,"ATLAS SCT Noise Occupancy S-Curves");
      if(time)  sprintf(s,"%s  -  %s",s,time->GetName());
      if(place) sprintf(s,"%s  -  %s",s,place->GetName());
      //    if(mname) sprintf(s,"%s  -  Module %s",s,mname->GetName());
      ttitle->AddText(s);
      sprintf(s, "Page %d Run %d Scan %d Module %d (%s)", 
              link+1, run, scan, moi, name);
      if(strlen(devtype) > 0) 
        sprintf(s, "%s - Type %s", s, devtype);

      ttitle->AddText(s);
      pad_t0->cd();
      ttitle->Draw();

      // Draw frames and labels
      for(chip=0;chip<MAX_CHIPS;chip++){
        for(i=0;i<4;i++){
          pad_c0->cd((i*6)+chip+1);
          hr[i]->Draw();
        }
        pad_t1->cd(chip+1);
        tlabel[link][chip]->Draw();
      }

      csc->Update();

      // Draw SCurves
      for(chan=0;chan<768;chan++){
        pad_c0->cd((6*(chan%4))+(chan/128)+1);
        gr[link][chan]->Draw("LP");
      }

      csc->Update();
    }

    ps->Close();
    delete ps;

    for(link=0;link<MAX_LINKS;link++) {
      for(chan=0;chan<MAX_CHANS;chan++){
        gr[link][chan]->Delete();
      }
    }

    for(i=0;i<4;i++){
      hr[i]->Delete();
    }

    if(time) delete time;
    if(place) delete place;
    if(mname) delete mname;

    delete csc;

    // Clear Batch Mode so that Burst and Scan display will update
    //gROOT->SetBatch(0)  // in normal use this is done by TrimRangePlot;

    strun->Close();
    delete strun;

  } // end if(strun)

}

