void RedundancyAnalysis(Int_t run, Int_t scan, Int_t moi){

  // analysis of RedundancyTest
  // version 0.1 PWP 13.11.00
  // version 0.2 PWP 15.11.00  bugfix
  // version 0.3 PWP 25.05.01 - Use parameter.h and Toolbox.cpp
  // version 0.4 PWP 06.08.01 - bugfix after DGC, rename #define symbols
  //                            to avoid clashes with other macros
  // scan select line using same pipeline cell
  // version 0.41 PWP 21.08.01 - database compliance
  // version 0.42 PWP 12.09.01 - further database compliance
  // version 0.50 PWP 15.11.01 - new format database compliance
  // version 0.51 BJG 30.04.02 - Fix some memory leaks

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define R_STUCK         0  
  #define R_DEAD          1 
  #define ADDRESS_ERROR0  2
  #define ADDRESS_ERROR1  3
  #define COMMAND_ERROR   4
  #define COMMAND_ERROR0  5
  #define COMMAND_ERROR1  6
  #define R_MAX_CODES     7

  Int_t module, link, chip, chan, i;
  char f_name[128], h_name[64];
  char mname[MAX_MODULES][32];

  Short_t chip_good[MAX_LINKS][MAX_CHIPS];
  Short_t chip_bad[MAX_LINKS][MAX_CHIPS][R_MAX_CODES];
  Short_t chip_status[MAX_LINKS][MAX_CHIPS];
  Short_t module_status, n_bad_chips, n_good_chips;

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");
  gROOT->LoadMacro("ToolBox.cpp");

  // 
  // Ensure that the module name array is cleared to start
  //

  for (module=0; module<MAX_MODULES; module++){
    sprintf(mname[module]," ");
  }


  // 
  // Load and process the data
  //

  sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan);
  TFile *strun = new TFile(f_name);

  if(strun==0){
    printf("Failed to open file %s - EXITING!!\n",f_name);
    return;
  }

  //
  //  loop over modules
  //

  Int_t start = 0;
  Int_t end   = 0;

  if(moi==-1){
    start = 0;
    end   = MAX_MODULES;

  } else {
    start = moi;
    end   = moi+1;
  }

  for (module=start; module<end; module++){

    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        chip_good[link][chip]=0;
        chip_status[link][chip]=0;
        for(i=0;i<R_MAX_CODES;i++){
          chip_bad[link][chip][i]=0;
        }
      }
    }

    n_bad_chips  = 0; // no chips have yet failed
    n_good_chips = 0; // no chips have yet passed

    for(link=0;link<MAX_LINKS;link++){

      // find histograms

      sprintf(h_name,"ModuleName;%d",module+1);
      TObjString* name = (TObjString*)strun->Get(h_name);
      if(name) {
        sprintf(mname[module],"%s",name->GetName());
        delete name;
      }

      sprintf(h_name,"h_scan%d;%d",link,module+1);
      TH2F *h_scan= (TH2F*)strun->Get(h_name);

      if(h_scan){
        if(h_scan->GetMean()>0){
          printf("Analysing data for module %d link %d\n",
          module, link);

          // Categorise each channel
 
          for(chan=0;chan<MAX_CHANS;chan++){
            chip = chan/128;

            Float_t occ0 = h_scan->GetBinContent((770*(1))+chan);
            Float_t occ1 = h_scan->GetBinContent((770*(2))+chan);


            if((occ0==50)&&(occ1==50)){

              chip_good[link][chip]++; 

            }else if((occ0==100)&&(occ1==100)){
  
              chip_bad[link][chip][R_STUCK]++; 
  
            }else if((occ0==0)&&(occ1==0)){

              // command reception OK but channel stuck OFF
              // NB could be due to a dead pipeline cell!!
              //    hence we must make a later comparison with
              //    data from the pipeline scan to rule this out

              chip_bad[link][chip][R_DEAD]++; 

            }else if((occ0==50)&&((occ1==0)||(occ1==100))){

              // addressing error when SELECT=1
              chip_bad[link][chip][ADDRESS_ERROR1]++; 
  
            }else if((occ1==50)&&((occ0==0)||(occ0==100))){

              // addressing error when SELECT=0
              chip_bad[link][chip][ADDRESS_ERROR0]++; 

            }else if((occ0==50)&&((occ1>0)||(occ1<100))){

              // commands not reliable when SELECT=1
              chip_bad[link][chip][COMMAND_ERROR1]++; 

            }else if((occ1==50)&&((occ0>0)||(occ0<100))){

              // commands not reliable when SELECT=0
              chip_bad[link][chip][COMMAND_ERROR0]++; 

            }else{

              // two bursts give inconsistent results:
              // command reception is not reliable
              chip_bad[link][chip][COMMAND_ERROR]++; 

            }
          } // end for(chan)

          //
          // Second stage of analysis: how many chips have passed?
          //

          for(chip=0;chip<MAX_CHIPS;chip++){
            if( (chip_good[link][chip]>=120) &&
               ((chip_good[link][chip]+chip_bad[link][chip][R_STUCK]+chip_bad[link][chip][R_DEAD])==128)){

              chip_status[link][chip]=1; // PASS
              n_good_chips++;

            }else{
              chip_status[link][chip]=0; // FAIL
              n_bad_chips++;
            }
          }

        } // end if(mean>0) [histogram is not empty]
      } // end if(h_scan) [histogram exists]
    } // end for(link)

    //
    // Third stage of analysis: has the module passed?
    //

    if((n_bad_chips+n_good_chips)==0){
      module_status = -1;  // NO DATA

    }else if(n_good_chips==12){
      module_status =  1; // PASS

    }else{
      module_status =  0; // FAIL
    }


    if(module_status>-1){ // Do not display or save results if no data!!


      //
      // Results to screen
      //


      printf("          pass good stuck dead adr0 adr1 com0 com1 com\n");

      for(link=0;link<MAX_LINKS;link++){
        for(chip=0;chip<MAX_CHIPS;chip++){
          Int_t chip2 = (link*6)+chip;

          if(chip_status[link][chip]){
            printf("  chip %2d PASS  ", chip2);
          }else{
            printf("  chip %2d FAIL  ", chip2);
          }

          printf("%3d  %3d  %3d  %3d  %3d  %3d  %3d  %3d\n",
                  chip_good[link][chip],
                  chip_bad[link][chip][R_STUCK],
                  chip_bad[link][chip][R_DEAD],
                  chip_bad[link][chip][ADDRESS_ERROR0],
                  chip_bad[link][chip][ADDRESS_ERROR1],
                  chip_bad[link][chip][COMMAND_ERROR0],
                  chip_bad[link][chip][COMMAND_ERROR1],
                  chip_bad[link][chip][COMMAND_ERROR]);

        } // end for(chip)
      } // end for(link)


      //
      // Results to file
      //

      FILE* outfile = OpenResults("RedundancyTest", strun, module, run, scan, module_status,0); 

      if(outfile){

        fprintf(outfile,"%%RedundancyTest\n");
        fprintf(outfile,"#\n");

        fprintf(outfile,"#RedundancyTest Summary - not for the database\n");
        fprintf(outfile,"#chip pass good stuck dead adr0 adr1 com0 com1 com\n");

        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){

            Int_t chip2 = (link*6)+chip;

            fprintf(outfile,"#%2d  %1d  %3d  %3d  %3d  %3d  %3d  %3d  %3d  %3d\n",
                    chip2,
                    chip_status[link][chip],
                    chip_good[link][chip],
                    chip_bad[link][chip][R_STUCK],
                    chip_bad[link][chip][R_DEAD],
                    chip_bad[link][chip][ADDRESS_ERROR0],
                    chip_bad[link][chip][ADDRESS_ERROR1],
                    chip_bad[link][chip][COMMAND_ERROR0],
                    chip_bad[link][chip][COMMAND_ERROR1],
                    chip_bad[link][chip][COMMAND_ERROR]);

          } // end for(chip)
        } // end for(link)

        Int_t n_defects=0;
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            if(chip_bad[link][chip][ADDRESS_ERROR0]>120){
              ListDefect(outfile,"CLK_ADDR0", ((6*link)+chip)*128,(((6*link)+chip+1)*128)-1);
              n_defects++;
            }
            if(chip_bad[link][chip][ADDRESS_ERROR1]>120){
              ListDefect(outfile,"CLK_ADDR1", ((6*link)+chip)*128,(((6*link)+chip+1)*128)-1);
              n_defects++;
            }
            if(chip_bad[link][chip][COMMAND_ERROR0]>120){
              ListDefect(outfile,"CLK_COM0", ((6*link)+chip)*128,(((6*link)+chip+1)*128)-1);
              n_defects++;
            }
            if(chip_bad[link][chip][COMMAND_ERROR1]>120){
              ListDefect(outfile,"CLK_COM1", ((6*link)+chip)*128,(((6*link)+chip+1)*128)-1);
              n_defects++;
            }
            if(chip_bad[link][chip][COMMAND_ERROR]>120){
              ListDefect(outfile,"CLK_ERROR", ((6*link)+chip)*128,(((6*link)+chip+1)*128)-1);
              n_defects++;
            }
          }
        }
        if(n_defects==0){
          fprintf(outfile,"#No defects found!\n");
        }else{
          fprintf(outfile,"#%d defects found\n",n_defects);
        }

        fclose(outfile);

      } // end if(outfile)

    }
  } // end(module)

  strun->Close();
  delete strun;
}


void RedundancyAnalysis(Int_t run, Int_t scan){

  RedundancyAnalysis(run,scan,-1);
}

