void ResponseCurve(Short_t dowait, Short_t postscript){

  // Default is to take data for 10 charges
  // and to fit a linear curve to this function
  //
  // Version 0.2 PWP 15.11.00
  // Version 0.3 PWP 16.11.00
  // Version 0.4 PWP 21.11.00
  // Version 0.5 PWP 23.11.00
  // Version 0.6 PWP 10.12.00

  Int_t n_points = 10;
  Int_t special_point = 5; //2fC
  Int_t scans[10];
  Float_t charges[10] = {0.50,0.75,1.00,1.25,1.50,
                        2.00,3.00,4.00,6.00,8.00};

  Short_t fit_type = 3;   // not a linear fit

  // load the plot macro 
  gROOT->ProcessLine(".L ResponseCurvePlot.cpp");

  // Save settings
  e->SaveState();

  e->StOutScanReset();
  e->ascii_nt=0;
  e->abort=0;

  e->do_cal_loop=1;       // cal loop ON
  e->do_fits=1;           // integrated fitting ON

  e->burst_trtype = 3;   // back to calibration triggers
  e->burst_ntrigs = 1000;

  // S-curve fitting fails when channels do not reach 100%
  // efficiency - so for small charges we must still use 
  // LEVEL MODE

  e->ConfigureVariable(11,1);  // X1X compression 
  e->ConfigureVariable(13,0);  // EDGE DETECT OFF
  e->ConfigureVariable(14,0);  // Route Mask OFF

  Int_t i;
  for(i=0;i<n_points;i++){

    // How should we set the scan ranges?
    // Assume 1fC will be in the range 50 to 150mV threshold
    //  => 0 to 200mV range for 1fC

    if(e->abort==0){
      Float_t lo_thr =   0 + (30.0 * (charges[i]-1.0));
      if(lo_thr<0) lo_thr = 0;

      Float_t hi_thr = 200 + (70.0 * (charges[i]-1.0));
      if(hi_thr>640) hi_thr = 640;

      e->ConfigureVariable(42,charges[i]);  // 42 - QCAL (fC)
      e->ConfigureScan(1,lo_thr,hi_thr,5);  //  1 - VTHR (mV)

      st_scan(11,-1);

      scans[i]=e->scannum;
    }
  }

  // Restore Settings
  e->RestoreState();

  if(e->abort==0){
    ResponseCurvePlot(e->runnum,n_points,special_point,scans,charges,fit_type,dowait,postscript,1);
  }
}

void ResponseCurve(){

  // if called with no arguments, do not wait between pages of output
  // but still generate the postscript file

  ResponseCurve(0,1);
}


