
// ST.cpp    SCT Test Environment:  ROOT User Macro - display and control
// Version PWP 12.09.00
// Version PWP 28.11.00
// Version PWP 13.12.00
// Version PWP 08.03.01
// Version PWP 17.05.01
// Version PWP 18.05.01
// Version PWP 05.07.01  add buttons for IVCurve and HardReset macros
// Version PWP 30.07.01  write username and module temperatures to ROOT file
//                       in easily portable format (standard ROOT objects)
// Version PWP 06.08.01  fix re. output of fitted data objects and 
//                       non-consecutive module numbering
// Version PWP/JGK 08.08.01 merge system test version

// GLOBALS

// TControlBar* bar;
// TControlBar* abar;
// TControlBar* dbar;
// TControlBar* tbar;
// TControlBar* mbar;
// TControlBar* trbar;
// TControlBar* vbar;

TCanvas* pburst;
TPad* pburst_info;
TPad* pburst_hist;

TCanvas* pburst2;
TPad* pburst2_info;
TPad* pburst2_hist;

TCanvas* pscan;
TPad *pscan_info;
TPad *pscan_cnts;
TPad *pscan_hist;

TCanvas* dcs;
TPad* dcs_info;
TPad* dcs_tsent;
TPad* dcs_hist;

TCanvas* pcount;
TPad* pcount_info;
TPad* pcount_tsent;
TPad* pcount_hist;


void ST() {    // Needs to be same name as file for auto-start

  int ret;

  gROOT->Reset();

  printf("\n");
  printf("Attempting to load stdll.dll");
  gSystem.Load("stdll.dll");
  printf("...done\n");

  printf("\nCreating TST object 'e'\n");
  e = new TST;
  printf("...done\n");

  printf("Probing VME system to see if RESMAN needs to be run...\n");
  ret=e->VMETest();

  if(ret<0){
    printf("Please wait whilst the NIVXI Resource Manager is run");
    gSystem.Exec("resman.exe");
    printf("...done\n");
  }

  e->PrintStatus();
  e->UpdateRunInfo();

  e->ConfigureVariable(20,5.0);

  printf("\nStarting up SCT system\n");
  e->Startup();
  printf("...done\n");

  e->burst_type = 1;

  // Next line will inhibit use of the stopper
  e->stopper = 1;

  // Next line will turn off the automatic fitting of scans
  // e->do_fits = 0;

  // Next line will turn off the automatic plotting of DCS monitor info
  // e->do_action_DCS = 0;

  // Next line will alter the number of shown modules from the automatic setting
  e->num_showmods = 3;

  // Turn on temperature reading from SCTLV3.
//    for(int i=0;i<e->GetMaxModules();i++){
//      if(e->m[i]->present){
//        e->m[i]->DCS->t_source=1;
//      }
//    }
  
  st_menu();

  st_display_burst();

  // Change the prompt to "ST"

  ((TRint*)gROOT->GetApplication())->SetPrompt("ST [%d] ");

  printf("ST() ...done\n");
}


void st_menu(){

  bar = new TControlBar("vertical","ST");
  bar->SetNumberOfColumns(2);

  bar->AddButton("Startup","e->Startup()","RELOAD DEFAULT SYSTEM AND MODULE PARAMETERS");
  bar->AddButton("Shutdown","e->Shutdown()","OPTO TX off, SCTLV off");
  bar->AddButton("Restart","e->Restart()","RELOAD DEFAULT MODULE PARAMETERS");

  // Most CLOAC state changes moved to new CLOAC menu - PWP 07.08.00
  if(e->nCLOAC>0){
    bar->AddButton("CloacMenu","st_menu_cloac()","Present the CLOAC menu");
  }
  // Leave "Run" and "Stop" here because they're jolly useful - PWP 07.08.00
  bar->AddButton("Run","e->RunFree()","Free Running Triggers");
  bar->AddButton("Stop","e->RunStop()","Stop Running Triggers");

  // Simple interactive configs and bursts
  bar->AddButton("ChangeVariable","st_menu_change_variable()","Present the SetVariable panel");
  bar->AddButton("ChangeTrigger","st_menu_change_trigger()","Present the SetTrigger panel");
  bar->AddButton("ExecuteConfigs","e->ExecuteConfigs()","Configure modules");
  bar->AddButton("TriggerBurst","st_menu_trigger_burst()","tburst: read MuSTARD histogram");
  bar->AddButton("TriggerBurst2","st_menu_trigger_burst2()","tburst2: tries to pinpoint errors");
  bar->AddButton("RawBurst","st_menu_raw_burst()","rawburst: histogram raw data in software");
  bar->AddButton("DecodedBurst","st_menu_decoded_burst()","Decoded Trigger Burst");
  if(e->nCLOAC>0){
    bar->AddButton("VTriggerBurst","st_menu_vburst()","vburst with hardware histogramming");
    bar->AddButton("DecodedVBurst","st_menu_decoded_vburst()","vburst with software histogramming");
    bar->AddButton("DumpVBurst","st_menu_dump_vburst()","vburst with events dumped to file");
  }
  bar->AddButton("DumpBurst","st_menu_dump_burst()","Decoded Trigger Burst with events dumped to file");
  bar->AddButton("SendIDBurst","st_menu_sendid_test()","sendidburst: Subtract the expected result from the returned data");
 
  bar->AddButton("RepeatingBurst","st_menu_repeating_burst()","Repeat last used Trigger Burst until Stopped");
 
  bar->AddButton("ABCD Tests","st_menu_macros()","Hybrid/Module Tests with analysis");
  bar->AddButton("B186 Tests","st_menu_systest()","Tests for B186-ST with analysis");

  bar->AddButton("SimpleScans","st_menu_analogue_scans()","Simple Scans using tburst");
//  bar->AddButton("DigitalScans", "st_menu_digital_scans()", "Scans using sendidburst");
  bar->AddButton("KwikPlot","st_kwikplot()","Show results of integrated fitting for given module");
//  bar->AddButton("FitScan","st_fitscan()","Fit last scan for given module and link");
  bar->AddButton("ShowScurves","st_show_sc()","Display s curves for given module and link");
  bar->AddButton("ShowCounters","st_display_scan_counters()","Show MuSTARD counters for the last scan");

  // Change / Show configurations
  bar->AddButton("SysmapMenu","e->SysmapMenu()","Alter selected sysmap parameters");
  bar->AddButton("ShowSysmap","e->ShowSysmap()","Display current system configuration (sysmap)");
  bar->AddButton("ShowModuleConfig","st_dialog_show_module_config()","Display current settings for a given module");
  bar->AddButton("ShowModuleTrims","st_dialog_show_module_trims()","Display current trims for a given module");
  bar->AddButton("ShowModuleRC","st_dialog_show_module_RC()","Display current RC fits for a given module");

  if(e->nOPTIFs>0){
    bar->AddButton("OPTO TX on",  "e->OptoTXOn()",    "Turn on VcSELs");
    bar->AddButton("OPTO TX off", "e->OptoTXOff()",   "Turn off VcSELs");
    bar->AddButton("OPTIF Status","st_show_optifs()","Show status of OptIFs");
  }

  // Slow Controls
  // - show buttons only if the system contains appropriate VME modules

  if(e->nSCTHV>0){
    bar->AddButton("HV RampUp",  "e->HVRamp(0)",       "Ramp UP high voltage");
    bar->AddButton("HV RampDown","e->HVRamp(1)",       "Ramp DOWN high voltage");
    bar->AddButton("HV Status",  "e->HVStatus()",      "Show status of HV channels");
    bar->AddButton("HV Recovery","e->HVRecoverTrips()","Find and recover from SCTHV trips");
  }

  if(e->nSCTLV>0){
    bar->AddButton("HardReset",  "e->HardReset()",   "Hard reset to all modules");
    bar->AddButton("LV On",      "e->LVOn()",        "Turn low voltage on");
    bar->AddButton("LV Off",     "e->LVOff()",       "Turn low voltage off");
    bar->AddButton("LV Status",  "e->LVStatus()",    "Show status of LV channels");
    bar->AddButton("LV Recovery","e->RecoverTrips()","Find and recover from SCTLV trips");
  }
  bar->AddButton("DCSQuery","st_dcs_query()", "Query DCS for given module");
  bar->AddButton("DCS->Log","st_dcs_log()",   "DCS printout for log-file");

  bar->AddButton("TestPrograms","st_menu_test_programs()","Diagnostic programs");
  bar->AddButton("Status","e->PrintStatus()","System Status");
  bar->AddButton("Documentation","st_docs()","Browse Documentation");
  bar->AddButton("Exit","st_exit()","Power off and Exit ROOT");
  bar->Show();

  gROOT->SaveContext();

}

void st_exit(){
  char c1;
 
  printf("\nDo you really want to exit the CINT/ROOT/ST environment? (y/n)\n");
  scanf("%c",&c1);
  if(c1!='y') return;

  // First turn off SCTLV modules and VcSELs

  e->Shutdown();

  // The following method of exiting ST does so abruptly, without
  // the tedious infinite loop of error messages produced by .q

  gSystem->Exit(0,0);
}

void st_menu_analogue_scans(){

  abar = new TControlBar("vertical","ST scans");
  abar->SetNumberOfColumns(2);

  // Simple 1D scans using tburst.  Chips in data taking mode.
  // nb: scan loop must be in macro to permit dynamic histogram display updates
  abar->AddButton("Scan",           "st_scan(1,-1)","Repeat Last Scan");
  abar->AddButton("ScanElapsed",    "st_scan(1, 0)","Repeat Trigger Burst under the same conditions");
  abar->AddButton("ScanThreshold",  "st_scan(1, 1)","Scan Threshold in mV");
  abar->AddButton("ScanVCal",       "st_scan(1, 2)","Scan Calibration level in mV");
  //abar->AddButton("ScanQThreshold", "st_scan(1,41)","Scan Threshold in fC");
  abar->AddButton("ScanQCal",       "st_scan(1,42)","Scan Calibration level in fC");
  abar->AddButton("ScanStrobeDelay","st_scan(1, 3)","Scan On Chip Strobe Delay");
  abar->AddButton("ScanPreamp",     "st_scan(1, 4)","Scan Preamplifier Bias current in microA");
  abar->AddButton("ScanShaper",     "st_scan(1, 5)","Scan Shaper current in microA");
  abar->AddButton("ScanStreamDelay","st_scan(1,20)","Relative phase of CLK and DATA at MuSTARD");
  abar->AddButton("ScanClockDuty",  "st_scan(1,23)","Scan Clock Duty Cycle (NOT RECOMMENDED)");
  abar->AddButton("ScanComDelay",   "st_scan(1,25)","n BCOs between Reset and Command (SLOG only)");
  abar->AddButton("ScanPulseDelay", "st_scan(1,26)","n BCOs between Pulse Input and L1A");
  abar->AddButton("ScanTrigDelay",  "st_scan(1,27)","n BCOs between Cal Strobe and L1A ");
  abar->AddButton("ScanVcc",        "st_scan(1,31)","");
  abar->AddButton("ScanVdd",        "st_scan(1,33)","");
  abar->AddButton("ScanVLED",       "st_scan(1,35)","");

  if(e->nOPTIFs>0){
    abar->AddButton("ScanOPTO RX THR","st_scan(1,50)","");
    abar->AddButton("ScanOPTO TX DAC","st_scan(1,53)","");
  }

  abar->AddButton("ShowScan",       "e->ShowScan()",""); 
  abar->AddButton("ScanMenu",       "e->ScanMenu()","");

  abar->Show();
}

void st_menu_digital_scans(){

  dbar = new TControlBar("vertical","ST");

  // Simple 1D scans using rawburst.  Chips in send ID mode,
  // configs executed before each trigger.  Returned data is
  // compared against the expected result and errors totalled.
  dbar->AddButton("Scan",           "st_scan(6,-1)","Repeat Last Scan"); 
  dbar->AddButton("ScanStreamDelay","st_scan(6,20)","");
  dbar->AddButton("ScanClockDuty",  "st_scan(6,23)","");
  dbar->AddButton("ScanVdd",        "st_scan(6,33)","");
  dbar->AddButton("ScanVLED",       "st_scan(6,35)","");
  dbar->AddButton("ShowScan",       "e->ShowScan()",""); 
  dbar->AddButton("ScanMenu",       "e->ScanMenu()","");

  dbar->Show();
}

void st_menu_test_programs(){

  tbar = new TControlBar("vertical","ST test");

  tbar->AddButton("BiLED test",   "st_menu_btest()",""); 
  tbar->AddButton("CLOAC test",   "st_menu_ctest()","");
  tbar->AddButton("MuSTARD test", "st_menu_mtest()","");
  tbar->AddButton("OptIF test",   "st_menu_otest()","");
  tbar->AddButton("SCTHV test",   "st_menu_hvtest()","");
  tbar->AddButton("SCTLV test",   "st_menu_lvtest()","");
  tbar->AddButton("SLOG test",    "st_menu_stest()","");
  tbar->AddButton("TestBurst",    "st_menu_test_burst()","");
  tbar->AddButton("ScanSEUTest",  "st_seu","SEU Test Run");
  
  tbar->Show();
}

void st_menu_macros(){

  mbar = new TControlBar("vertical","ST Macros");

  if(e->nSCTHV>0){
    mbar->AddButton("IV Curve",         "st_iv()","Record the IV characteristic");
  }
  mbar->AddButton("Set Stream Delay", "st_tm()","Automatic Adjustment of Stream Delay");
  mbar->AddButton("Hard Reset Tests", "st_hr()","Verify Hard Reset Functionality");
  mbar->AddButton("Full Bypass Test", "st_fbt()","Verify Bypass Functionality");
  mbar->AddButton("Redundancy Test",  "st_rt()","Check Redundanct Clock and Command");
  mbar->AddButton("Pipeline Test",    "st_pt()","Find dead cells in the pipeline");
  mbar->AddButton("Set Strobe Delay", "st_sd()","Automatic Adjustment of Strobe Delay");
  mbar->AddButton("Three Point Gain", "st_tpg()","Fast measurement of gain and noise");
  mbar->AddButton("Trim Range",       "st_tr()","Trimming, all four ranges");
  mbar->AddButton("Trim Scan",        "st_ts()","Automatic Trimming");
  mbar->AddButton("Response Curve",   "st_rc()","");
  mbar->AddButton("Noise Occupancy",  "st_no()","");
  mbar->AddButton("Timewalk",         "st_tw()","");

  mbar->AddButton("Characterisation Sequence", "st_characterisation()","Full Test Sequence"); 
  mbar->AddButton("Confirmation Sequence",     "st_confirmation()","Short Test Sequence"); 
 
  mbar->Show();
}

void st_menu_systest(){

  stbar = new TControlBar("vertical","ST test");
  stbar->SetNumberOfColumns(1);

  stbar->AddButton("3pt-Gain vs Time","st_svt(0,0)","Three Point gain measurements at regular intervals");
  stbar->AddButton("3pt-Gain vs Time w/ Strobe","st_svt(0,1)","Three Point gain measurements at regular intervals");
  stbar->AddButton("Noise Occ. vs Time","st_novt(0)","Noise Occupancy measured at regular intervals");

  stbar->AddButton("Standalone 3pt-Gain","st_svt(0,1)","Three Point gain measurements at regular intervals");
  stbar->AddButton("Standalone N.O.","st_novt(1)","Noise Occupancy measured at regular intervals");

  stbar->AddButton("N.O. scan vs Qthr",   "st_no(0,1)","scans noise occupancy vs threshold charge");
  stbar->AddButton("Correlated Noise",    "st_cn()","Writes hit pattern to ntuple for correl. noise analysis");

  stbar->Show();

}

void st_menu_change_trigger(){

  trbar = new TControlBar("vertical","ST trigger");
  trbar->SetNumberOfColumns(1);

  trbar->AddButton("Set ntrigs",    "st_dialog_set_ntrigs()", "No. Triggers per burst"); 
  trbar->AddButton("L1A",           "st_menu_set_trigger(0)", "0 - using CLOAC"); 
  trbar->AddButton("CAL+L1A",       "st_menu_set_trigger(3)", "3 - using CLOAC");
  trbar->AddButton("PULSE+L1A",     "st_menu_set_trigger(20)","20 - using CLOAC");
  trbar->AddButton("Pretrigger+L1A","st_menu_set_trigger(30)","30 - PT, COM_DELAY, L1A"); 
  trbar->AddButton("BCR+L1A",       "st_menu_set_trigger(32)","32 - PT/BCR, COM_DELAY, L1A");
  trbar->AddButton("BCR+CAL+L1A",   "st_menu_set_trigger(34)","34 - PT/BCR, COM_DELAY, CAL, TRIG_DELAY, L1A");
  trbar->AddButton("BCR+PULSE+L1A", "st_menu_set_trigger(36)","36 - PT/BCR, COM_DELAY, CAL, PULSE_DELAY, L1A");
  trbar->AddButton("SR+L1A",        "st_menu_set_trigger(31)","31 - PT/SR, COM_DELAY, L1A");
  trbar->AddButton("SR+CAL+L1A",    "st_menu_set_trigger(33)","33 - PT/SR, COM_DELAY, CAL, TRIG_DELAY, L1A");
  trbar->AddButton("SR+PULSE+L1A",  "st_menu_set_trigger(35)","35 - PT/SR, COM_DELAY, CAL, PULSE_DELAY, L1A");

  trbar->AddButton("USER",          "st_menu_set_trigger(39)","39 - User sequence: file 'sctvar\\config\\slog.txt'");

  trbar->AddButton("8*L1A",         "st_menu_set_trigger(40)","40 - 8 Consecutive L1A");
  trbar->AddButton("SR+(8*L1A)",    "st_menu_set_trigger(41)","41 - SR, 8 Consecutive L1A");
  trbar->AddButton("SR+CAL+(8*L1A)","st_menu_set_trigger(43)","43 - SR, CAL,8 Consecutive L1A");
  trbar->AddButton("SR+PULSE+(8*L1A)", "st_menu_set_trigger(45)","45 - SR, PULSE,8 Consecutive L1A");

  trbar->AddButton("SR+(n*L1A)",    "st_menu_set_trigger(52)","52 - SR, n spaced L1A");
  trbar->AddButton("SR+CAL+(n*L1A)","st_menu_set_trigger(51)","51 - SR, CAL, n spaced L1A");
  trbar->AddButton("SR+PULSE+(n*L1A)", "st_menu_set_trigger(50)","50 - SR, PULSE,n spaced L1A");

  trbar->AddButton("do_cal_loop",   "st_menu_do_cal_loop(1)", "Loop over four cal lines");
  trbar->AddButton("skip_cal_loop", "st_menu_do_cal_loop(0)", "Single cal line only"); 
  trbar->Show();
}


void st_menu_change_variable(){
  int typ;
  float var;

  vbar = new TControlBar("vertical","ST variable");
  vbar->SetNumberOfColumns(2);

  vbar->AddButton("SetVThr","st_dialog_change_variable(1)",        "1 - Set Threshold in mV");
  vbar->AddButton("SetVCal","st_dialog_change_variable(2)",        "2 - Set Cal level in mV");
  vbar->AddButton("SetQThr","st_dialog_change_variable(41)",       "41 - Set Threshold in fC");
  vbar->AddButton("SetQCal","st_dialog_change_variable(42)",       "42 - Set Cal level in fC");
  vbar->AddButton("SetStrobeDelay","st_dialog_change_variable(3)", "3 - Set Strobe Delay");
  vbar->AddButton("SetPreamp","st_dialog_change_variable(4)",      "4 - Set Preamplifier Bias in microA");
  vbar->AddButton("SetShaper","st_dialog_change_variable(5)",      "5 - Set Shaper current in microA");
  vbar->AddButton("SetTrimDAC","st_dialog_change_variable(6)",     "6 - Set Trimdac in DAC bits");
  vbar->AddButton("SetCalMode","st_dialog_change_variable(10)",    "10 - Select Calibration line");
  vbar->AddButton("SetCompMode","st_dialog_change_variable(11)",   "11 - Select Data Compression Criteria");
  vbar->AddButton("SetTrimRange","st_dialog_change_variable(12)",  "12 - Select Trim Range");
  vbar->AddButton("SetEdgeDet","st_dialog_change_variable(13)",    "13 - Toggle Edge Detect Mode");
  vbar->AddButton("SetSendMask","st_dialog_change_variable(14)",   "14 - Toggle Send Mask");
  vbar->AddButton("SetAccumulate","st_dialog_change_variable(15)", "15 - Toggle Accumulate bit");
  vbar->AddButton("SetStrmDel","st_dialog_change_variable(20)",    "20 - Relative phase of CLK and DATA at MuSTARD");
  vbar->AddButton("SetClkDuty","st_dialog_change_variable(23)",    "23 - Clock Duty (NOT RECOMMENDED)");
//  vbar->AddButton("SetClkPhase","st_dialog_change_variable(24)",   "24 - Clock Phase (NOT RECOMMENDED)");
  vbar->AddButton("SetComDelay","st_dialog_change_variable(25)",   "25 - n BCOs between Reset and Cal or Pulse command - SLOG only, not CLOAC");
  vbar->AddButton("SetPulseDelay","st_dialog_change_variable(26)", "26 - n BCOs between Pulse Input and L1A");
  vbar->AddButton("SetTrigDelay","st_dialog_change_variable(27)",  "27 - n BCOs between Cal strobe and L1A");
//  vbar->AddButton("SetVDet","st_dialog_change_variable(30)",       "30");  // Not yet implemented
  vbar->AddButton("SetVcc","st_dialog_change_variable(31)",        "31");
  vbar->AddButton("SetVi1","st_dialog_change_variable(32)",        "32");
  vbar->AddButton("SetVdd","st_dialog_change_variable(33)",        "33");
  vbar->AddButton("SetVPin","st_dialog_change_variable(34)",       "34 - only for SCTLV version 5 or newer");
  vbar->AddButton("SetVLED","st_dialog_change_variable(35)",       "35");
  vbar->AddButton("SetSELECT","st_dialog_change_variable(38)",     "38");

  if(e->nOPTIFs>0){
    vbar->AddButton("SetOPTO RX THR","st_dialog_change_variable(50)","50 - OptIF RX THR");
    vbar->AddButton("SetOPTO TX DAC","st_dialog_change_variable(53)","58 - OptIF TX DAC");
  }
  vbar->Show();
}
 
void st_dialog_change_variable(short typ){
  float var;
  char nam[32];

  e->TranslateVariableName(typ,nam);
  printf("Change %s to  (float)  ?", nam);
  scanf("%f",&var);
  printf("\nChanging variable type %d %s to %f\n", typ,nam,var);
  e->ConfigureVariable(typ,var);
  e->ExecuteConfigs();
}

void st_dialog_show_module_config(){
  int imod;

  printf("Show config for module (int) ?");
  scanf("%d",&imod);
  e->ShowModule(imod);
}

void st_dialog_show_module_trims(){
  int imod;

  printf("Show trims for module (int) ?");
  scanf("%d",&imod);
  e->ShowTrims(imod);
}

void st_dialog_show_module_RC(){
  int imod;

  printf("Show Response Curve fits for module (int) ?");
  scanf("%d",&imod);
  e->ShowRC(imod);
}

void st_menu_btest(){
  char command[120];

  sprintf(command,"%sbin\\btest.exe",e->sct_path);
  gSystem->Exec(command);
}

void st_menu_ctest(){
  char command[120];

  sprintf(command,"%sbin\\ctest.exe",e->sct_path);
  gSystem->Exec(command);
}

void st_menu_mtest(){
  char command[120];

  sprintf(command,"%sbin\\mtest.exe",e->sct_path);
  gSystem->Exec(command);
}

void st_menu_otest(){
  char command[120];

  sprintf(command,"%sbin\\otest.exe",e->sct_path);
  gSystem->Exec(command);
}
void st_menu_hvtest(){
  char command[120];

  sprintf(command,"%sbin\\hvtest.exe",e->sct_path);
  gSystem->Exec(command);
}

void st_menu_lvtest(){
  char command[120];

  sprintf(command,"%sbin\\lv_test.exe",e->sct_path);
  gSystem->Exec(command);
}

void st_menu_stest(){
  char command[120];

  sprintf(command,"%sbin\\stest.exe",e->sct_path);
  gSystem->Exec(command);
}

void st_dialog_set_ntrigs(){
  int n;

  printf("Requested number of triggers per burst(int) ?");
  scanf("%d",&n);
  e->burst_ntrigs=n;
}

void st_menu_set_trigger(short trtype){
  short n;

  e->burst_trtype=trtype;
  printf("Trigger type %hd selected\n",trtype);
  
  if (trtype==39){
    printf("User sequence selected.  How many events per loop?(short) ?");
    scanf("%d",&n);
    e->L1A_per_loop = n;
  } else if ((trtype>39)&&(trtype<50)){
    e->L1A_per_loop = 8;
  } else if (trtype>49){
      printf("Multiple SLOG L1A selected.  How many L1A per loop?(short, 1-8) ?");
      scanf("%d",&n);
      e->L1A_per_loop = n;
      e->ConfigureVariable(45.0,n);
      printf("                             Spacing between L1A (short, 0-1024) ?");
      scanf("%d",&n);
      e->ConfigureVariable(46.0,n);
      printf("                             How many L1A do you wish to ignore (short, 0-%d) ?",(e->L1A_per_loop-1));
      scanf("%d",&n);
      e->throw_away = n;
  }
  else e->L1A_per_loop = 1;
}

void st_menu_trigger_burst(){
  int n;
  e->burst_type = 1;
  e->ExecuteBurst();
  st_update_burst_display();

  for (n=0; n<e->num_showmods; n++){
    if(e->m[n]->present) e->m[n]->PrintBurst();
  }
}

void st_menu_trigger_burst2(){
  int n;
  e->burst_type = 11;
  e->ExecuteBurst();
  st_update_burst_display();

  for (n=0; n<e->num_showmods; n++){
    if(e->m[n]->present) e->m[n]->PrintBurst();
  }
}

void st_menu_vburst(){
  int n;
  e->burst_type = 12;
  e->ExecuteBurst();
  st_update_burst_display();

  for (n=0; n<e->num_showmods; n++){
    if(e->m[n]->present) e->m[n]->PrintBurst();
  }
}

void st_menu_dump_burst(){
  int n;
  short old_burst = e->burst_type;
  short old_debug = e->debug_level;
  short old_loop  = e->do_cal_loop;

  e->burst_type=100;
  e->debug_level=999;
  e->do_cal_loop=0;

  e->ExecuteBurst();
  st_update_burst_display();

  e->burst_type=old_burst;
  e->debug_level=old_debug;
  e->do_cal_loop=old_loop;

  for (n=0; n<e->num_showmods; n++){
    if(e->m[n]->present) e->m[n]->PrintBurst();
  }
}

void st_menu_dump_vburst(){
  int n;
  short old_burst = e->burst_type;
  short old_debug = e->debug_level;
  short old_loop  = e->do_cal_loop;

  e->burst_type=102;
  e->debug_level=999;
  e->do_cal_loop=0;

  e->ExecuteBurst();
  st_update_burst_display();

  e->burst_type=old_burst;
  e->debug_level=old_debug;
  e->do_cal_loop=old_loop;

  for (n=0; n<e->num_showmods; n++){
    if(e->m[n]->present) e->m[n]->PrintBurst();
  }
}

void st_menu_decoded_burst(){
  int n;
  e->burst_type = 100;
  e->ExecuteBurst();
  st_update_burst_display();
  st_display_burst2();
  st_update_burst_display2();

  for (n=0; n<e->num_showmods; n++){
    if(e->m[n]->present) e->m[n]->PrintBurst();
  }
}

void st_menu_decoded_vburst(){
  int n;
  e->burst_type = 102;
  e->ExecuteBurst();
  st_update_burst_display();
  st_display_burst2();
  st_update_burst_display2();

  for (n=0; n<e->num_showmods; n++){
    if(e->m[n]->present) e->m[n]->PrintBurst();
  }
}

void st_menu_test_burst(){
  e->burst_type = 111;
  e->ExecuteBurst();
  e->num_showmods = 5;
  st_display_burst();
}

void st_menu_do_cal_loop(short val){
 
  e->do_cal_loop=val;
  printf("e->do_cal_loop=%d\n",e->do_cal_loop);
}

void st_menu_raw_burst(){

  e->burst_type = 5;
  e->ExecuteBurst();
  st_update_burst_display();

  for (n=0; n<e->num_showmods; n++){
    if(e->m[n]->present) e->m[n]->PrintBurst();
  }
}

void st_menu_sendid_test(){
  float setval;
  e->ConfigureVariable(16,0);  // configure sendID mode
  e->ExecuteConfigs();         // execute configs (which in sendID mode omits goto_datataking)
  e->burst_type = 6;;          // do type 6 burst
  e->ExecuteBurst();
  st_update_burst_display();   // display burst
  e->ConfigureVariable(16,1);  // configure sendDATA mode
  e->ExecuteConfigs();         // execute configs (now appending goto_datataking for normal running) 
}

void st_menu_repeating_burst(){
  int stop_loop = 0;

  st_start_stopper();
  while (!stop_loop){
    st_menu_trigger_burst();
    stop_loop=st_do_stop();
  }
  st_stop_stopper();
}

void st_scan(short burst_type, short scan_type){
  // The Basic Scan Routine!!
  // Revised PWP 09.05.01 to allow:
  //   Adjustment of burst_ntrigs during NO scans
  //   Autostop

  float setval;
  int stop_loop = 0;
  int n_to_continue=0;
  short n, fitf;

  st_display_burst();
  if((burst_type==100)||(burst_type==102)||(burst_type==112)){
    st_display_burst2();
  }

  st_start_stopper();  // start stop dialog
  e->ConfigureScanDefaults(scan_type);
  e->burst_type = burst_type;
  e->InitScan();

  stop_loop = 0;

  while(e->scan_running){ 
    e->ScanPoint();
    printf("ST::st_scan: finished ScanPoint()\n");
    st_update_burst_display();
    if((burst_type==100)||(burst_type==102) ||(burst_type==112)){
      st_update_burst_display2();
    }

    // Adjustment of burst_ntrigs for noise occupancy scans
    if(e->do_vary_ntrigs)&&(  (burst_type== 12)    // VBURST
                            ||(burst_type==100)    // DBURST
                            ||(burst_type==102))){ // DVBURST
      if(e->tsent>(4*e->burst_ntrigs)){
        e->burst_ntrigs*=2;
      }
    }

    // AutoStop facility
    if(e->do_autostop){
      n_to_continue=0;
      for(n=0;n<e->GetMaxModules();n++){
        if(e->m[n]->present){
          if(e->m[n]->OKtoStop(e->scan_type)){
            printf("  Module %d OK to stop\n",n);
          }else{
            printf("  Module %d needs to continue\n",n);
            n_to_continue++;
          }
        }
      }
      if(n_to_continue>0){
        printf("    Total of %d modules need to continue\n",n_to_continue);
      } else {
        printf("    STOPPING due to AUTOSTOP!!\n");
        e->scan_running=0;
      }
    }

    stop_loop=st_do_stop();
    if(stop_loop==1) e->scan_running=0; // abort if stopped
  }

  e->EndScan();
  st_display_scan();

  if(e->do_fits){
    switch(e->scan_type){
      case 1:   // ST_VTHR
      case 3:   // ST_QTHR
      case 41:  // ST_STROBE_DELAY
        fitf = 1;
        break;
      case 2:   // ST_VCAL
      case 42:  // ST_QCAL
        fitf = 2;
        break;
      default:
        fitf =-1;
        break;
    }
      
    for (n=0; n<e->GetMaxModules(); n++){
      if(e->m[n]->present){  // only fit scan for present modules
        printf("Fitting Scan for module %d\n",n);
        if((e->do_cal_loop)||(e->burst_trtype==0))e->m[n]->FitScan(fitf,-1);
        else e->m[n]->FitScan(fitf,0);
        e->m[n]->PrintScan();
      }
    }
    st_display_scanfits();
  }

  if(e->do_action_DCS){
    st_display_scan_dcs();
  }

  st_save_scan();
  st_stop_stopper();  // stop stop dialog
}

void st_seu(){
  
  int seu_count = 0;
  int stop_loop = 0;
  int seu_scan_header[14];

  st_start_stopper();  // start stop dialog
  e->scan_running =1;
  e->scannum++;


  while(e->scan_running){ 

    e->SEUBurst();
    printf("ST::st_seu: finished burst()\n");
    
    e->RecoverTrips(); // added PWP 11.09.00
 
    stop_loop=st_do_stop();
    if(stop_loop==1) e->scan_running=0; // abort if stopped
    seu_count += e->seu_test_count;
  }

  int date, time;
  TDatime dt;

  date = dt.GetDate();
  time = dt.GetTime();
  
  seu_scan_header[0] = e->runnum;
  seu_scan_header[1] = e->scannum;
  seu_scan_header[2] = e->burstnum;
  seu_scan_header[3] = seu_count;
  seu_scan_header[4] = date;
  seu_scan_header[5] = time;
  seu_scan_header[6] = e->run_tsent;
  seu_scan_header[7] = e->run_ercnt;
  seu_scan_header[8] = e->run_tocnt;
  seu_scan_header[9] = e->run_xecnt;
  seu_scan_header[10] = e->seu_good_count[2];
  seu_scan_header[11] = e->seu_good_count[3];
  seu_scan_header[12] = e->seu_bad_count[2];
  seu_scan_header[13] = e->seu_bad_count[3];

  e->SEUAppendSummary(14,seu_scan_header,1);  // grand total
}


void st_display_scan() {
  int col = 17;

  gStyle->SetHistFillColor(46);
//  gStyle->SetLabelSize(0.06,"x");
//  gStyle->SetLabelSize(0.06,"y");
  gStyle->SetTitleW(0.6);
  gStyle->SetTitleH(0.08);
  gStyle->SetOptStat(11);
  gStyle->SetStatH(0.3);
  gStyle->SetStatColor(0);

  char ctitle[128];
  sprintf(ctitle,"Scan Display  Run %d", e->runnum);

  TCanvas* pscan = new TCanvas("pscan",ctitle,1100,750);
  pscan->SetFillColor(18);

  TPad* pscan_info = new TPad("pscan_info","Scan Info Pad",0.0,0.9,0.5,1.0,col);
  pscan_info->Draw();
  TPad* pscan_cnts = new TPad("pscan_cnts","Scan Counter Pad",0.5,0.9,1.0,1.0,col);
  pscan_cnts->Draw();
  TPad* pscan_hist = new TPad("pscan_hist","Scan Histogram Pad", 0.0,0.0,1.0,0.9,col);
  pscan_hist->Draw();

  pscan_cnts->cd();
  pscan_cnts->Divide((e->nMuSTARDs+1),1,0,0);

  pscan_hist->cd();
  pscan_hist->Divide(2,e->num_showmods,0,0);

  st_update_scan_display();
}


void st_update_scan_display(){
  short i,n;
  char s[128];

  pscan->cd();
 
  pscan_info->cd();
  pscan_info->Clear();

  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->Draw();
  t1->SetTextAlign(13);
  t1->SetTextSize(0.15);
  t1->SetTextColor(4);
  t1->AddText("ATLAS SCT Module Test");
  sprintf(s,"Run %d  Scan %d  Burst %d  Run_tsent %u  Run_ercnt %u", 
     e->runnum, e->scannum, e->burstnum, e->run_tsent, e->run_ercnt);
  t1->AddText(s);
  sprintf(s,"%s", e->scan_description_string);
  t1->AddText(s);
  
  pscan_cnts->cd(1);
  e->h_scan_tsent->Draw();
  for (n=0; n<e->nMuSTARDs; n++){
    pscan_cnts->cd(2+n);
    e->h_scan_ercnt[n]->Draw();
  }

  pscan->Modified();    // Update display before long wait while histos fill
  pscan->Update();

  for (n=0; n<e->num_showmods; n++){
    pscan_hist->cd((n*2)+1);
    e->m[n]->h_scan0->SetYTitle(e->scan_name);
    e->m[n]->h_scan0->Draw("colz");
    pscan_hist->cd((n*2)+2);
    e->m[n]->h_scan1->SetYTitle(e->scan_name);
    e->m[n]->h_scan1->Draw("colz");
  }
  pscan->Modified();
  pscan->Update();
  // printf("finished upd scan disply\n");
}

void st_display_scanfits(){
  short i,n;
  char s[128];

  pscan->cd();
 
  for (n=0; n<e->num_showmods; n++){
    pscan_hist->cd((n*2)+1);
    e->m[n]->h_mean0->Draw("same");
    pscan_hist->cd((n*2)+2);
    e->m[n]->h_mean1->Draw("same");
  }
  pscan->Modified();
  pscan->Update();
}

void st_display_burst() {
  int col = 17;

//  gStyle->SetCanvasColor(33);
//  gStyle->SetPadColor(33);
//  gStyle->SetFrameFillColor(33);
  gStyle->SetHistFillColor(46);
  gStyle->SetLabelSize(0.06,"x");
  gStyle->SetLabelSize(0.06,"y");
  gStyle->SetTitleW(0.6);
  gStyle->SetTitleH(0.08);
  gStyle->SetOptStat(11);
  gStyle->SetStatH(0.3);
  gStyle->SetStatColor(0);

  char ctitle[128];
  sprintf(ctitle,"Burst Display  Run %d", e->runnum);

  TCanvas* pburst = new TCanvas("pburst",ctitle,1100,750);
  pburst->SetFillColor(18);

  TPad *pburst_info = new TPad("pburst_info","Burst Info Pad",0.0,0.88,1.0,1.0,col);
  pburst_info->Draw();
  TPad *pburst_hist = new TPad("pburst_hist","Burst Histogram Pad", 0.0,0.0,1.0,0.88,col);
  pburst_hist->Draw();

  pburst_info->cd();
  pburst_info->Divide(2,1,0,0);
  
  pburst_hist->cd();
  pburst_hist->Divide(2,e->num_showmods,0,0);

  st_update_burst_display();
}

void st_update_burst_display(){

  char s[128];
  int n;

  pburst->cd();

  pburst_info->cd(1);
  TPad *pburst_info_1 = (TPad*)gPad;
  pburst_info_1->Clear();
 
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->Draw();
  t1->SetTextAlign(13);
  t1->SetTextSize(0.15);
  t1->SetTextColor(4);

  TDatime *dt = new TDatime;
  sprintf(s,"ATLAS SCT Module Test  Run %d  Burst %d  Date %d  Time %d", e->runnum, e->burstnum, dt->GetDate(),dt->GetTime());
  t1->AddText(s);

  sprintf(s,"Burst Trigs_sent %u", e->tsent);
  t1->AddText(s);

  for(n=0;n<e->nMuSTARDs;n++){
    sprintf(s,"MuSTARD %d Events_seen %u  Stream_errors %u  Timeouts %u  Other_errors %u",
       n, e->evcnt[n], e->ercnt[n], e->tocnt[n], e->xecnt[n]);
    t1->AddText(s);
  }

  sprintf(s,"Run Trigs_sent %u  Events_seen %u  Stream_errors %u  Timeouts %u  Other_errors %u",
     e->run_tsent, e->run_evcnt, e->run_ercnt, e->run_tocnt, e->run_xecnt);
  t1->AddText(s); 
  
  if (e->scan_running) t1->AddText(e->scan_status_string);
  
  pburst_info->cd(2);
  e->h_merr->Draw();

  for (n=0; n<e->num_showmods; n++){
    pburst_hist->cd((n*2)+1);
    e->m[n]->h_burst0->SetYTitle("Hits");
    e->m[n]->h_burst0->Draw();
    pburst_hist->cd((n*2)+2);
    e->m[n]->h_burst1->SetYTitle("Hits");
    e->m[n]->h_burst1->Draw();
  }

  pburst->Modified();
  pburst->Update();
}


void st_display_burst2() {
  int col = 17;

//  gStyle->SetCanvasColor(33);
//  gStyle->SetPadColor(33);
//  gStyle->SetFrameFillColor(33);
  gStyle->SetHistFillColor(46);
  gStyle->SetLabelSize(0.06,"x");
  gStyle->SetLabelSize(0.06,"y");
  gStyle->SetTitleW(0.6);
  gStyle->SetTitleH(0.08);
  gStyle->SetOptStat(11);
  gStyle->SetStatH(0.3);
  gStyle->SetStatColor(0);

  char ctitle[128];
  sprintf(ctitle,"Burst2 Display  Run %d", e->runnum);

  TCanvas* pburst2 = new TCanvas("pburst2",ctitle,1100,750);
  pburst2->SetFillColor(18);

  TPad *pburst2_info = new TPad("pburst2_info","Burst Info Pad",0.0,0.88,1.0,1.0,col);
  pburst2_info->Draw();
  TPad *pburst2_hist = new TPad("pburst2_hist","Burst Histogram Pad", 0.0,0.0,1.0,0.88,col);
  pburst2_hist->Draw();

  pburst2_info->cd();
  pburst2_info->Divide(2,1,0,0);
  
  pburst2_hist->cd();
  pburst2_hist->Divide(5,e->num_showmods,0,0);

  st_update_burst_display2(); // was previously commented out "to avoid crashes with TimeEvBurst"
}

void st_update_burst_display2(){

  char s[128],h_name[64],mname[32];
  int n;

  pburst2->cd();

  pburst2_info->cd(1);
  TPad *pburst2_info_1 = (TPad*)gPad;
  pburst2_info_1->Clear();

  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->Draw();
  t1->SetTextAlign(13);
  t1->SetTextSize(0.17);
  t1->SetTextColor(4);

  TDatime *dt = new TDatime;
  sprintf(s,"ATLAS SCT Module Test  Run %d  Burst %d  Date %d  Time %d", e->runnum, e->burstnum, dt->GetDate(),dt->GetTime());
  t1->AddText(s);

  sprintf(s,"Burst Trigs_sent %u", e->tsent);
  t1->AddText(s);

  for(n=0;n<e->nMuSTARDs;n++){
    sprintf(s,"MuSTARD %d Events_seen %u  Stream_errors %u  Timeouts %u  Other_errors %u",
       n, e->evcnt[n], e->ercnt[n], e->tocnt[n], e->xecnt[n]);
    t1->AddText(s);
  }

  sprintf(s,"Run Trigs_sent %u  Events_seen %u  Stream_errors %u  Timeouts %u  Other_errors %u",
     e->run_tsent, e->run_evcnt, e->run_ercnt, e->run_tocnt, e->run_xecnt);
  t1->AddText(s); 
  
  if (e->scan_running) t1->AddText(e->scan_status_string);
  

  pburst2_info->cd(2);
  e->h_merr->Draw();

  for (n=0; n<e->num_showmods; n++){
    pburst2_hist->cd((n*5)+1);
    e->m[n]->h_cluster0->Draw();
    pburst2_hist->cd((n*5)+2);
    e->m[n]->h_cluster1->Draw();

    e->GetModuleName(n,mname);

    sprintf(h_name,"h_hit%d_0",n);
    sprintf(s,"Module %d %s  Stream 0 Hits",n,mname);
    e->m[n]->h_corr->ProjectionX(h_name,0,128);
    TH1F* h_hit0 = (TH1F*) gDirectory->GetList()->FindObject(h_name);
    pburst2_hist->cd((n*5)+3);
    h_hit0->SetTitle(s);
    h_hit0->SetXTitle("Hits Stream 0");
    h_hit0->SetYTitle("n(Events)");
    h_hit0->Draw();
  
    sprintf(h_name,"h_hit%d_1",n);
    sprintf(s,"Module %d %s  Stream 1 Hits",n,mname);
    e->m[n]->h_corr->ProjectionY(h_name,0,128);
    TH1F* h_hit1 = (TH1F*) gDirectory->GetList()->FindObject(h_name);
    pburst2_hist->cd((n*5)+4);
    h_hit1->SetTitle(s);
    h_hit1->SetXTitle("Hits Stream 1");
    h_hit1->SetYTitle("n(Events)");
    h_hit1->Draw();
  
    pburst2_hist->cd((n*5)+5);
    e->m[n]->h_corr->Draw();
  }

  pburst2->Modified();
  pburst2->Update();
}


void st_list_results() {

  char s[128];
  int n;
 
  p3->cd();

  TPaveText *t5 = new TPaveText(0.0,0.0,1.0,0.3);
  //t5->SetFillColor(col);
  t5->Draw();
  t5->SetTextAlign(13);
  t5->SetTextSize(0.05);
  // t5->SetTextFont(8);
  sprintf(s,"Number of bad channels: %d", e->v->number_bad_strips);
  t5->AddText(s);
  t5->AddText("Anomalous Strip : Pedestal: Noise : Flag");

  for (n=0; n<640; n++) {
    if (e->v->bad_strips[n] != 0) {
      sprintf(s,"%4d %4d %4d %4d", n, e->v->pedestals[n], e->v->noise[n], e->v->bad_strips[n]);
      t5->AddText(s);
    }
  }

  c1->Modified();
  c1->Update();
  c1->Draw();
}

void st_save_scan(TFile* out=0) {
  int n;
  char fname[128];
  char mname[32];
  TGraph* gr;

  int write_fits=0;

  // this function should be viewed as purely temporary... famous last words !

  sprintf(fname,"%sstrun%d_%d.root",e->sct_data,e->runnum,e->scannum);

  TDatime *dt = new TDatime;
  TObjString *dts = new TObjString(dt->AsString());
  TObjString *pls = new TObjString("UNKNOWN");
  TObjString *host= new TObjString(gSystem->Getenv("COMPUTERNAME"));
  TObjString *user= new TObjString(gSystem->Getenv("USERNAME"));

  if(out==0) out = new TFile(fname,"RECREATE");

  dts->Write("Time");
  pls->Write("Place");
  host->Write("Host");
  user->Write("User");

  for (n=0; n<e->GetMaxModules(); n++){
    if(e->m[n]->scan_fitted){
      write_fits++;
    }
  }

  for (n=0; n<e->GetMaxModules(); n++){
  //  if(e->m[n]->present){  // only write histograms for present modules

      e->GetModuleName(n,mname);
      TObjString * mns = new TObjString(mname);
      mns->Write("ModuleName");

      sprintf(fname,"%3.1f",e->m[n]->DCS->T1);
      TObjString * t1s = new TObjString(fname);
      t1s->Write("T0");

      sprintf(fname,"%3.1f",e->m[n]->DCS->T2);
      TObjString * t2s = new TObjString(fname);
      t2s->Write("T1");

      e->m[n]->DCS->Write();  //stream the DCS object

      e->m[n]->h_scan0->Write();
      e->m[n]->h_scan1->Write();

      if(e->nSCTLV>0){
        gr = e->m[n]->GetIddGraph();
        gr->Write("Idd");
        gr = e->m[n]->GetIccGraph();
        gr->Write("Icc");
      }
      if(e->nSCTHV>0){
        gr = e->m[n]->GetIdetGraph();
        gr->Write("Idet");
      }

      if(write_fits){
        e->m[n]->h_mean0->Write();
        e->m[n]->h_mean1->Write();
        e->m[n]->h_sigma0->Write();
        e->m[n]->h_sigma1->Write();
        e->m[n]->h_code0->Write();
        e->m[n]->h_code1->Write();
      }

      // TODO: Write the next three hists at each EOB
      // This code will write the hists to file only
      // for the LAST burst of each scan

      if((e->burst_type == 100)||(e->burst_type == 102)) { // decoded burst or decoded vburst
        e->m[n]->h_cluster0->Write();
        e->m[n]->h_cluster1->Write();
        e->m[n]->h_corr->Write();

        if(e->do_fill_evtree == 1){
          if(e->m[n]->evtree!=0){
            sprintf(fname,"EvTree_m%d",n);
            e->m[n]->evtree->Write(fname);
          }
        }
      }
  //  } 
  }

  e->h_scan_tsent->Write();
  for (n=0; n<e->nMuSTARDs; n++){
    e->h_scan_evcnt[n]->Write();
    e->h_scan_ercnt[n]->Write();
    e->h_scan_tocnt[n]->Write();
    e->h_scan_xecnt[n]->Write();
  }

  out.Close();


  sprintf(fname, "%sstlog%d.txt",e->sct_data,e->runnum);
  
  FILE *logfile;
  logfile = fopen(fname,"a+");

  fprintf(logfile,"runnum %u\n",e->runnum);
  fprintf(logfile,"scannum %u\n",e->scannum);
  fprintf(logfile,"date %u\n",dt->GetDate());
  fprintf(logfile,"time %u\n",dt->GetTime());
  fprintf(logfile,"scan_type %u\n",e->scan_type);
  fprintf(logfile,"scan_start %6.2f\n",e->scan_start);
  fprintf(logfile,"scan_stop %6.2f\n",e->scan_stop);
  fprintf(logfile,"scan_step %6.2f\n",e->scan_step);
  fprintf(logfile,"run_tsent %u\n",e->run_tsent);
  fprintf(logfile,"run_evcnt %u\n",e->run_evcnt);
  fprintf(logfile,"run_ercnt %u\n",e->run_ercnt);
  fprintf(logfile,"run_tocnt %u\n",e->run_tocnt);
 
  fclose(logfile);

}

void st_dcs_query() {
  int moi;

  printf("Show DCS for module (int) ?",moi);
  scanf("%d",&moi);

  if((moi>=0)&&(moi<e->GetMaxModules())){
    e->m[moi]->DCS->Print();
  } else {
   printf("Sorry, module %d out of range\n",moi);
  }
}

void st_dcs_log() {
  int moi;
  float tmp[4];
  
  printf("\nModule-specific DCS-Information:\n\n");
  printf("Module\tVcc\tIcc\tVdd\tIdd\tVdet\tIdet\tThyb1\tThyb2\n");

  for(moi=0;moi<e->num_showmods;moi++){
    if(e->m[moi]->present) {
      e->m[moi]->DCS->Query();
            printf("%s\t%.2f\t%.0f\t%.2f\t%.0f\t%.1f\t%.2f\t%.2f\t%.2f\n",
		   e->m[moi]->name,e->m[moi]->DCS->vcc,e->m[moi]->DCS->icc,
		   e->m[moi]->DCS->vdd,e->m[moi]->DCS->idd,
		   e->m[moi]->DCS->vdet,e->m[moi]->DCS->idet,
		   e->m[moi]->DCS->T1,e->m[moi]->DCS->T2);
    }
  }

  // OLD VERSION
  tmp[0] = e->m[0]->DCS->ET1;
  tmp[1] = e->m[0]->DCS->ET2;
  tmp[2] = e->m[0]->DCS->hum1;
  tmp[3] = e->m[0]->DCS->hum2;

  // SYSTEST VERSION
//  tmp[0] = e->DCSMessg("temperature 0");
//  tmp[1] = e->DCSMessg("temperature 1");
//  tmp[2] = e->DCSMessg("humidity 0");
//  tmp[3] = e->DCSMessg("dewpoint 0");

  // COMMON TO BOTH
  printf("\nEnvironmental DCS-Information:\n\n");
  printf("Tamb1\tTamb2\tHum\tDew\n");
  printf("%.2f\t%.2f\t%.1f\t%.2f\n\n",tmp[0],tmp[1],tmp[2],tmp[3]);

}

void st_show_sc() {
  int moi,loi;

  printf("Show s curves for module (int) ?",moi);
  scanf("%d",&moi);
  
  printf("link ?",loi);
  scanf("%d",&loi);
  
  gROOT->ProcessLine(".L stan_scurves.cpp");
  stan_scurves(moi,loi);
}

void st_kwikplot() {
  int moi;

  printf("Display last scan for module (int) ?",moi);
  scanf("%d",&moi);
  
  gROOT->ProcessLine(".L KwikPlot.cpp");
  KwikPlot(moi);
}

void st_fitscan() {
  int moi,loi;

  printf("Fit last scan for module (int) ?",moi);
  scanf("%d",&moi);
  
  printf("link ?",loi);
  scanf("%d",&loi);
  
  gROOT->ProcessLine(".L stan_fitscan.cpp");
  stan_fitscan(moi,loi);
}

int st_do_stop(){
  char string[64];

  // New version to handle STOP or ABORT
  // User macros must be written to read ABORT flag
  // PWP 11.12.00

  sprintf(string,"%sstop.txt",e->sct_etc);
  FILE* stop = fopen(string,"r");

  if(stop!=NULL){
    fgets(string,2,stop);

    if(string[0]=='A'){
      printf("ST_DO_STOP: ABORT\n");
      e->abort = 1;
    } else {
      printf("ST_DO_STOP: STOP\n");
    }

    fclose(stop);
    remove(string);
    return 1;

  } else {
    return 0;
  }
}

int st_docs(){
  char string[64];

  // use this line to browse the local documentation
  // sprintf(string,"START netscape.exe %ssctdaq.html",e->sct_path);

  // and this one to browse the documentation at CERN
  sprintf(string,"START netscape.exe http://atlas.web.cern.ch/Atlas/GROUPS/INNER_DETECTOR/SCT/testdaq/testdaq.html");
  gSystem->Exec(string);
}

int st_start_stopper(){
  char string[64];

  // Clear abort flag
  e->abort = 0;

  sprintf(string,"%sstop.txt",e->sct_etc);
  remove(string);

  if(e->stopper){
    sprintf(string,"START %sbin\\Stop.exe",e->sct_path);
    gSystem->Exec(string);

    return 1;
  }else{
    printf("st_start_stopper: [CURRENTLY DISABLED]\n");
  }
  return 0;
}

int st_stop_stopper(){
  char string[64];

  if(e->stopper){
    gSystem->Exec("c:\\sctdaq\\bin\\kill stop");

    sprintf(string,"%sstart.txt",e->sct_etc);
    remove(string);

    sprintf(string,"%sstop.txt",e->sct_etc);
    remove(string);

  }else{
    printf("st_stop_stopper: [CURRENTLY DISABLED]\n");
  }
  return 0;
}

void st_menu_cloac() {
 
  gROOT->ProcessLine(".L cc_menu.cpp");
  cc_menu();
}

void st_display_scan_counters() {
  int col = 17;

  gStyle->SetHistFillColor(46);
//  gStyle->SetLabelSize(0.06,"x");
//  gStyle->SetLabelSize(0.06,"y");
  gStyle->SetTitleW(0.6);
  gStyle->SetTitleH(0.08);
  gStyle->SetOptStat(11);
  gStyle->SetStatH(0.3);
  gStyle->SetStatColor(0);

  char ctitle[128];
  sprintf(ctitle,"Counter Display  Run %d", e->runnum);

  TCanvas* pcount = new TCanvas("pcount",ctitle,1100,750);
  pcount->SetFillColor(18);

  TPad *pcount_info = new TPad("pcount_info","Scan Info Pad",0.0,0.8,0.5,1.0,col);
  pcount_info->Draw();
  TPad *pcount_tsent = new TPad("pcount_tsent","Scan TSent Pad",0.5,0.8,1.0,1.0,col);
  pcount_tsent->Draw();
  TPad *pcount_hist = new TPad("pcount_hist","Scan MuSTARD Counters Pad", 0.0,0.0,1.0,0.8,col);
  pcount_hist->Draw();

  pcount_hist->cd();
  pcount_hist->Divide(4,e->nMuSTARDs,0,0);


  st_update_scan_counter_display();
}

void st_update_scan_counter_display(){
  short i,n;
  char s[128];

  pcount->cd();
 
  pcount_info->cd();
  pcount_info->Clear();

  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->Draw();
  t1->SetTextAlign(13);
  t1->SetTextSize(0.15);
  t1->SetTextColor(4);
  t1->AddText("ATLAS SCT Module Test");
  sprintf(s,"Run %d  Scan %d  Burst %d",
     e->runnum, e->scannum, e->burstnum);
  t1->AddText(s);
  sprintf(s,"Run_tsent %u  Run_ercnt %u", 
     e->run_tsent, e->run_ercnt);
  t1->AddText(s);
  sprintf(s,"%s", e->scan_description_string);
  t1->AddText(s);
  
  pcount_tsent->cd();
  e->h_scan_tsent->Draw();

  for (n=0; n<e->nMuSTARDs; n++){
    pcount_hist->cd(1+(4*n));
    e->h_scan_evcnt[n]->Draw();
    pcount_hist->cd(2+(4*n));
    e->h_scan_ercnt[n]->Draw();
    pcount_hist->cd(3+(4*n));
    e->h_scan_tocnt[n]->Draw();
    pcount_hist->cd(4+(4*n));
    e->h_scan_xecnt[n]->Draw();
  }

  pcount->Modified(); 
  pcount->Update();
}

void st_display_scan_dcs() {
  int col = 17;

  gStyle->SetHistFillColor(46);
//  gStyle->SetLabelSize(0.06,"x");
//  gStyle->SetLabelSize(0.06,"y");
  gStyle->SetTitleW(0.6);
  gStyle->SetTitleH(0.08);
  gStyle->SetOptStat(11);
  gStyle->SetStatH(0.3);
  gStyle->SetStatColor(0);

  char ctitle[128];
  sprintf(ctitle,"DCS Display  Run %d", e->runnum);

  TCanvas* dcs = new TCanvas("dcs",ctitle,1100,750);
  dcs->SetFillColor(18);

  TPad *dcs_info = new TPad("dcs_info","Scan Info Pad",0.0,0.8,0.5,1.0,col);
  dcs_info->Draw();
  TPad *dcs_tsent = new TPad("dcs_tsent","Scan TSent Pad",0.5,0.8,1.0,1.0,col);
  dcs_tsent->Draw();
  TPad *dcs_hist = new TPad("dcs_hist","Scan MuSTARD Counters Pad", 0.0,0.0,1.0,0.8,col);
  dcs_hist->Draw();

  dcs_hist->cd();
  dcs_hist->Divide(e->num_showmods,3,0,0);


  st_update_dcs_display();
}

void st_update_dcs_display(){
  short i,n;
  char s[128], h_name[32];

  dcs->cd();
 
  dcs_info->cd();
  dcs_info->Clear();

  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->Draw();
  t1->SetTextAlign(13);
  t1->SetTextSize(0.15);
  t1->SetTextColor(4);
  t1->AddText("ATLAS SCT Module Test");
  sprintf(s,"Run %d  Scan %d  Burst %d",
     e->runnum, e->scannum, e->burstnum);
  t1->AddText(s);
  sprintf(s,"Run_tsent %u  Run_ercnt %u", 
     e->run_tsent, e->run_ercnt);
  t1->AddText(s);
  sprintf(s,"%s", e->scan_description_string);
  t1->AddText(s);
  
  dcs_tsent->cd();
  e->h_scan_tsent->Draw();

  TGraph* gr_dcs[18][3];

  for (n=0; n<e->num_showmods; n++){

    if(e->m[n]->present){

      gr_dcs[n][0]= 0;
      gr_dcs[n][1]= 0;
      gr_dcs[n][2]= 0;
      gr_dcs[n][3]= 0;

      if(e->nSCTLV>0){
        gr_dcs[n][0]= e->m[n]->GetIccGraph();
        gr_dcs[n][1]= e->m[n]->GetIddGraph();
      }

      if(e->nSCTHV>0){
        gr_dcs[n][2]= e->m[n]->GetIdetGraph();
      }

      if(gr_dcs[n][0]){   
        dcs_hist->cd(n+1);
        gr_dcs[n][0]->SetMarkerStyle(21);
        gr_dcs[n][0]->Draw("AP");
        gr_dcs[n][0]->GetHistogram()->SetXTitle(e->scan_name);
        gr_dcs[n][0]->GetHistogram()->SetYTitle("Icc (mA)");
        gr_dcs[n][0]->GetHistogram()->Draw();
        gr_dcs[n][0]->Draw("P");
      }

      if(gr_dcs[n][1]){
        dcs_hist->cd((n+1)+e->num_showmods);
        gr_dcs[n][1]->SetMarkerStyle(21);
        gr_dcs[n][1]->Draw("AP");
        gr_dcs[n][1]->GetHistogram()->SetXTitle(e->scan_name);
        gr_dcs[n][1]->GetHistogram()->SetYTitle("Idd (mA)");
        gr_dcs[n][1]->GetHistogram()->Draw();
        gr_dcs[n][1]->Draw("P");
      }

      if(gr_dcs[n][2]){
        dcs_hist->cd((n+1)+(2*e->num_showmods));
        gr_dcs[n][2]->SetMarkerStyle(21);
        gr_dcs[n][2]->Draw("AP");
        gr_dcs[n][2]->GetHistogram()->SetXTitle(e->scan_name);
        gr_dcs[n][2]->GetHistogram()->SetYTitle("Idet (mA)");
        gr_dcs[n][2]->GetHistogram()->Draw();
        gr_dcs[n][2]->Draw("P");
      }

    }   
  }

  dcs->Modified(); 
  dcs->Update();
}

void st_show_optifs(){
  int i;

  for(i=0;i<e->nOPTIFs;i++){
    e->OptifStatus(i);
  }
}

// links to production test macros
// added PWP 16.11.00

void st_tm(){
  gROOT->ProcessLine(".L StreamDelay.cpp");
  StreamDelay(0);
}

void st_iv(){
  gROOT->ProcessLine(".L IVCurve.cpp");
  IVCurve();
}

void st_hr(){
  gROOT->ProcessLine(".L HardReset.cpp");
  HardReset();
}

void st_fbt(){
  float StartVdd,StopVdd,StepVdd;
  int n_configs;
  char ok;

  printf("FullBypassTest: Scan Vdd from ");
  scanf("%f",&StartVdd);
  printf("to ");
  scanf("%f",&StopVdd);
  printf("in steps of ");
  scanf("%f",&StepVdd);
  printf("\nNumber of configs (Barrel 36, Fwd:62): ");
  scanf("%d",&n_configs);

  printf("\nFullBypassTest Vdd = %3.1f-%3.1f step: %3.1f n_configs: %d\nOK? (y/n)",StartVdd,StopVdd,StepVdd,n_configs);
  scanf("%c",&ok);
  printf("%c",ok);

  if(ok!='n'){

    gROOT->ProcessLine(".L FullBypassTest.cpp");
    FullBypassTest(StartVdd,StopVdd,StepVdd,n_configs);
  }
}

void st_bt(){
  gROOT->ProcessLine(".L BypassTest.cpp");
  BypassTest();
}

void st_rt(){
  gROOT->ProcessLine(".L RedundancyTest.cpp");
  RedundancyTest();
}

void st_pt(){
  gROOT->ProcessLine(".L PipelineTest.cpp");
  PipelineTest();
}

void st_sd(){
  gROOT->ProcessLine(".L StrobeDelay.cpp");
  StrobeDelay(1);
}

void st_tpg(){
  gROOT->ProcessLine(".L ThreePointGain.cpp");
  ThreePointGain();
}


void st_tr(){
  gROOT->ProcessLine(".L TrimRange.cpp");
  TrimRange();
}

void st_ts(){
  Float_t charge;
  Int_t TrimRange;
  printf("TrimScan: Trim with charge: ");
  scanf("%f",&charge);
  printf("and TrimRange: ");
  scanf("%d",&TrimRange);

  gROOT->ProcessLine(".L TrimScan.cpp");
  TrimScan(charge,TrimRange,0,1);
}

void st_rc(){
  gROOT->ProcessLine(".L ResponseCurve.cpp");
  ResponseCurve();
}

void st_no(){
  gROOT->ProcessLine(".L No.cpp");
  No();
}

void st_tw(){
  gROOT->ProcessLine(".L Timewalk.cpp");
  Timewalk(1); // find 1fC point by threshold scan
}


void st_characterisation(){
  gROOT->ProcessLine(".L CharacterisationTest.cpp");
  CharacterisationTest();
}

void st_confirmation(){
  gROOT->ProcessLine(".L ConfirmationTest.cpp");
  ConfirmationTest();
}

void st_svt(Int_t runstrobe, Int_t standalone){
  gROOT->ProcessLine(".L systest/ScurveStability.cpp");
  ScurveStability(runstrobe, standalone);
}

void st_novt(Int_t standalone){
  gROOT->ProcessLine(".L systest/NOvsT.cpp");
  NOvsT(standalone);
}

void st_sasafe(){
  gROOT->ProcessLine(".L systest/safety.cpp");
  safety();
}

void st_cn(){
  Int_t evt;
  Float_t qthr;
  gROOT->ProcessLine(".L systest/CorrelNoise.cpp");
  printf("Enter number of events (int), please:");
  scanf("%d",&evt);
  printf("Enter threshold in fC (float) [-100 to leave unchanged]:");
  scanf("%f",&qthr);
  CorrelNoise(evt,qthr);
}


