void StrobeDelay(Int_t find_two_fc) {

  // version 0.1 LE 
  // version 0.2 PWP 30.10.00
  // version 0.3 PWP 01.11.00
  // version 0.4 PWP 15.11.00
  // version 0.5 LE/PWP 23.11.00
  // version 0.6 PWP 10.12.00

  Int_t module, chip;
  const int MAX_MODULES = 6;


  // Save settings before the scan
  e->SaveState();

  Float_t in_charge[MAX_MODULES][12];
  Int_t comp_mode[MAX_MODULES][12];
  Int_t edge[MAX_MODULES][12];
  Int_t route_mask[MAX_MODULES][12];

  for (module=0; module<MAX_MODULES; module++){
    if(e->m[module]->present){
      for(chip=0;chip<12;chip++){
	in_charge[module][chip] = e->RetrieveVariable(module,chip,2);
	comp_mode[module][chip] = e->RetrieveVariable(module,chip,11);
	edge[module][chip] = e->RetrieveVariable(module,chip,13);
	route_mask[module][chip] = e->RetrieveVariable(module,chip,14);
      }
    }
  }

  // Configure the scan
  e->StOutScanReset();
  e->ascii_nt=0;
  e->abort=0;

  e->do_cal_loop=1;        // cal loop ON 
  e->do_mask_test=0;       // alternating mask patterns OFF 

  e->burst_trtype=3;       // CAL + L1A
  e->burst_ntrigs=1000;

  e->ConfigureVariable(14,0);  // Route Mask OFF


  //
  // If the calling routine/user has not been able to set 2fC threshold,
  // we can find an approximate value now using the default timing
  //

  if(find_two_fc){
  e->do_fits=1;           // integrated fitting ON

    e->ConfigureVariable(11,0);   // Compression Mode 0.
    e->ConfigureVariable(13,0);   // Edge detect off.

    e->ConfigureVariable(2,20);   // 2.00fC input charge
    e->ConfigureScan(1,0,350,5);  // range tbc

    st_scan(11,-1);

    for (module=0; module<MAX_MODULES; module++){
      if(e->m[module]->present){
        for(chip=0;chip<12;chip++){
          e->ConfigureVariable(module,chip,1,e->m[module]->chip_mean[chip/6][chip%6]);
        } // end for(chip)  
      }
    } // end for(module)
  }



  //
  // Make a strobe delay scan with
  // Edge Detect ON, edge compression
  // and calculate the optimum strobe delay setting
  //

  // load the analysis macro 
  gROOT->ProcessLine(".L StrobeDelayPlot.cpp");

  if(e->abort==0){
    e->do_fits=0;           // integrated fitting OFF

    e->ConfigureVariable(11,2);   // Compression Mode 2.
    e->ConfigureVariable(13,1);   // Edge detect on.
  
    e->ConfigureVariable(2,40);   // 4.00fC input charge.
    e->ConfigureScan(3,0,63,1);

    st_scan(11,-1);
  }

  // Reset values after scan
  e->RestoreState();

  for (module=0; module<MAX_MODULES; module++){
    if(e->m[module]->present){
      for(chip=0;chip<12;chip++){
	e->ConfigureVariable(module,chip,2,in_charge[module][chip]);
	e->ConfigureVariable(module,chip,11,comp_mode[module][chip]);
	e->ConfigureVariable(module,chip,13,edge[module][chip]);
	e->ConfigureVariable(module,chip,14,route_mask[module][chip]);
      }
    }
  }

  //
  // Call the analysis macro.
  // Arguments are:
  //  run, scan, module, fraction,
  //  online(=1), graphics(=0), postscript(=0)
  //

  if(e->abort==0){
    StrobeDelayPlot(e->runnum, e->scannum,-1,0.25,1,0,0);
  }
}

void StrobeDelay() {

  StrobeDelay(0);
}


