void StrobeSpeedAnalysis(Int_t run, Int_t scan, Int_t module, Float_t Vdd, Int_t trigdelay, Int_t graphics, Int_t dowait) {

  //
  //
  // version 0.10 LE 
  // version 0.20 PWP 30.10.00
  // version 0.21 PWP/LE 01.11.00
  // version 0.22 LE/PWP 23.11.00
  // version 0.3  LE/HP  02.03.01 - To measure the chip speed via the strobe delay 
  // version 0.31 BJG    30.04.02 - Fix some memory leaks 


  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define MIN_RISE  0;  // NB these limits will change 
  #define MIN_FALL 28;  // system timing by 1 BCO....
  #define MAX_RISE 35;
  #define MAX_FALL 63;

  Char_t h_name[80], f_name[128], s[100];
  Int_t  chip, link;

  const Int_t debug = 0;

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");
  gROOT->LoadMacro("ToolBox.cpp");
  stan_styles();
  gStyle->SetOptStat(0);


  //
  // Sanity checks
  //

  //
  // Open the root data file and obtain pointers to the event histogram
  //


  sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan);

  TFile *strun = new TFile(f_name);

  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Int_t the_time = dt->GetTime();

  TObjString *time = 0;
  TObjString *place = 0;
  TObjString *mname = 0;

  //
  // Open the output canvas
  //

  Int_t screen_page=0;

  TCanvas* StrobeDelay =0;

  if(graphics!=0){
    graphics = 1;
    StrobeDelay=new TCanvas ("StrobeDelay","Strobe Delay Scan",1100,750);
  }


  //
  //  loop over modules
  //

  Int_t start = 0;
  Int_t end   = 0;

  if(module==-1){
    start = 0;
    end   = MAX_MODULES;
  } else {
    start = module;
    end   = module+1;
  }

  Int_t dowait=0;

  if((graphics!=0)&&(end-start>1)){
    dowait = 1;
  }

  TH1F* hp_chip[MAX_LINKS][MAX_CHIPS];
  TF1* f1[MAX_LINKS][MAX_CHIPS];
  TF1* f2[MAX_LINKS][MAX_CHIPS];

  for(link=0;link<MAX_LINKS;link++){
    for(chip=0;chip<MAX_CHIPS;chip++){
      hp_chip[link][chip]=0;
      
      sprintf(h_name,"f1_l%dc%d",link,chip);
      f1[link][chip]= new TF1 (h_name,erfc,  0, 1000, 3);
      f1[link][chip]->SetParNames("Constant","Mean","Sigma");
      
      sprintf(h_name,"f2_l%dc%d",link,chip);
      f2[link][chip]= new TF1 (h_name,erf,  0, 1000, 3);
      f2[link][chip]->SetParNames("Constant","Mean","Sigma");
    }
  }
    
  Float_t chip_fall[MAX_LINKS][MAX_CHIPS];
  Float_t chip_rise[MAX_LINKS][MAX_CHIPS];

  for(module=start;module<end;module++){
    
    if((screen_page>0)&&(dowait)){
      printf("Enter a string to continue\n");
      scanf("%s",s);
    }
    
    sprintf(h_name,"h_scan0;%d",module+1);
    TH2F *h_scan0= (TH2F*)strun->Get(h_name);
    sprintf(h_name,"h_scan1;%d",module+1);
    TH2F *h_scan1= (TH2F*)strun->Get(h_name);

    //
    // only attempt plotting or fitting if data exists for the module
    //

    Int_t chips_with_data=0;

    if((h_scan0!=0)&&(h_scan1!=0)){  // Histogram exists
      if((h_scan0->GetMean()>0)||(h_scan1->GetMean()>0)){  // And contains data

	sprintf(h_name,"ModuleName;%d",module+1);
    if(time) delete time;
    if(place) delete place;
    if(mname) delete mname;
    time  = (TObjString*)strun->Get("Time");
    place = (TObjString*)strun->Get("Place");
    mname = (TObjString*)strun->Get(h_name);

	//
	// Delete unnecessary objects and clear arrays
	//

	for(link=0;link<MAX_LINKS;link++){
	  for(chip=0;chip<MAX_CHIPS;chip++){
	    if(hp_chip[link][chip]) hp_chip[link][chip]->Delete();
	    chip_fall[link][chip]=-999;
	    chip_rise[link][chip]=-999;
	  }
	}
	
	printf("\nFitting data for module %d\n",module);

    TPad *pad_sd = 0;
	if(StrobeDelay){ // the canvas open, therefore the plots are to be drawn

	  StrobeDelay->cd();
	  StrobeDelay->Clear();
	  
	  TPad *pad_t  = new TPad("pad_t","Text",           0.01,0.91,0.99,0.99,33);
	  pad_sd = new TPad("pad_sd","Strobe Delays", 0.01,0.01,0.99,0.90,33);
	  
	  pad_t->Draw();
	  pad_sd->Draw();
	  pad_sd->Divide(3,4,0,0);
	  pad_t->cd();
	  
	  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
	  t1->SetFillColor(10);
	  t1->SetTextAlign(13);
	  t1->SetTextSize(0.3);
	  t1->SetTextColor(4);
	  
	  
	  sprintf(s,"ATLAS SCT Module Test");
	  if(time)  sprintf(s,"%s  -  %s",s,time->GetName());
	  if(place) sprintf(s,"%s  -  %s",s,place->GetName());
	  if(mname) sprintf(s,"%s  -  Module %s",s, mname->GetName());
	  t1->AddText(s);
	  sprintf(s,"Run %d   Scan %d   Module %d  -  Strobe Delay Optimisation", 
		  run, scan, module);
	  t1->AddText(s);
	  
	  t1->Draw();
	}
	

	//
	// Loop over the chips and do the calculations
	//
	
	for(link=0;link<MAX_LINKS;link++){
	  for(chip=0;chip<MAX_CHIPS;chip++) {

	    sprintf(h_name,"hp_chip_l%dc%d",link,chip);
	    if(link==0) h_scan0->ProjectionY(h_name,(chip*128),((chip+1)*128 - 1));
	    else h_scan1->ProjectionY(h_name,(chip*128),((chip+1)*128 - 1));
	    
	    hp_chip[link][chip] = (TH1F*) gDirectory->GetList()->FindObject(h_name);

	    if(hp_chip[link][chip]->GetMaximum()>0.0){ // data for this chip

	      chips_with_data++;
	      
	      sprintf(h_name,"%s chip %d", hp_chip[link][chip]->GetTitle(),chip);
	      hp_chip[link][chip]->SetTitle(h_name);
	      hp_chip[link][chip]->SetXTitle("Strobe Delay");
	      hp_chip[link][chip]->SetYTitle("Efficiency");
	      hp_chip[link][chip]->Scale(1.0/hp_chip[link][chip]->GetMaximum());

	      Float_t centre = (find_bin2(hp_chip[link][chip],0.99,0,1)+find_bin2(hp_chip[link][chip],0.99,1,0))/2;
	      
	    
	      // Falling edge
	      
	      Float_t p0_guess = hp_chip[link][chip]->GetMaximum();
	      Float_t p1_guess = find_level2(hp_chip[link][chip],0.50,1);          // last bin > 50%
	      Float_t p2_guess = find_level2(hp_chip[link][chip],0.82,1)-p1_guess;  
	      Float_t range_lo = centre;
	      Float_t range_hi = hp_chip[link][chip]->GetBinCenter(hp_chip[link][chip]->GetNbinsX());
	      if(p2_guess<0) p2_guess = p2_guess*-1.0;

	      f1[link][chip]->SetParameters(p0_guess,p1_guess,p2_guess);
	      f1[link][chip]->SetParLimits(0,p0_guess,p0_guess);
	      f1[link][chip]->SetRange(range_lo, range_hi);

	      sprintf(h_name,"f1_l%dc%d",link,chip);
	      hp_chip[link][chip]->Fit(h_name,"NRQ"); // no graphics, quiet mode
	      chip_fall[link][chip] = f1[link][chip]->GetParameter(1); 

	      // Rising edge

	      p0_guess = hp_chip[link][chip]->GetMaximum();
	      p1_guess = find_level2(hp_chip[link][chip],0.50,0);          // first bin < 50%
	      p2_guess = find_level2(hp_chip[link][chip],0.82,0)-p1_guess;
	      range_lo = hp_chip[link][chip]->GetBinCenter(1);
	      range_hi = centre;
	      if(p2_guess<0) p2_guess = p2_guess*-1.0;

	      f2[link][chip]->SetParameters(p0_guess,p1_guess,p2_guess);
	      f2[link][chip]->SetParLimits(0,p0_guess,p0_guess);
	      f2[link][chip]->SetRange(range_lo, range_hi);

	      sprintf(h_name,"f2_l%dc%d",link,chip);
	      hp_chip[link][chip]->Fit(h_name,"NRQ"); // no graphics, quiet mode
	      chip_rise[link][chip] = f2[link][chip]->GetParameter(1);
	      
	      if((chip_rise[link][chip]<-5)||(chip_rise[link][chip]>70)) chip_rise[link][chip] = -999;
	      if((chip_fall[link][chip]<-5)||(chip_fall[link][chip]>70)) chip_fall[link][chip] = -999;
	      
	      printf("  Chip %2d fall %2.1f  rise %2.1f diff %2.2f\n",
		     chip,chip_fall[link][chip],chip_rise[link][chip],chip_fall[link][chip]-chip_rise[link][chip]); 
	      
	      
	      if(StrobeDelay){ // the canvas open, therefore the plots are to be drawn

		pad_sd->cd(link*6+chip+1);
		hp_chip[link][chip]->Draw();
		
		f1[link][chip]->SetLineColor(kBlue);
		f1[link][chip]->Draw("same");
		f2[link][chip]->SetLineColor(kBlue);
		f2[link][chip]->Draw("same");
		
		TMarker *FallMark=new TMarker (chip_fall[link][chip],0.5,23); 
		FallMark->SetMarkerColor(kBlue);
		FallMark->SetMarkerSize(1);
		FallMark->SetMarkerStyle(23);
		FallMark->Draw("same");
		
		TMarker *RiseMark=new TMarker (chip_rise[link][chip],0.5,23); 
		RiseMark->SetMarkerColor(kBlue);
		RiseMark->SetMarkerSize(1);
		RiseMark->SetMarkerStyle(23);
		RiseMark->Draw("same");
		
		TText * t = new TText();
		sprintf(h_name,"%2.1f",chip_fall[link][chip]);
		t->DrawText(chip_fall[link][chip]-1,0.35,h_name);
		sprintf(h_name,"%2.1f",chip_rise[link][chip]);
		t->DrawText(chip_rise[link][chip]-1,0.35,h_name);
	      }
	      
	      
	    } else {
	      printf("  Link %2d Chip %2d NO DATA!!!\n",link,chip);
	      
	    } // end if (data for this chip)
	    
	  } // end for(chip)
	} //end for(link)
      
	if(StrobeDelay){
	  StrobeDelay->Update();
	}

	
	if(chips_with_data>0){

	  //
	  // Set the values
	  //
	  
	  Float_t mean_delay = 0.0;
	  Int_t n_good_chips = 0;
	  
	  if(n_good_chips>0){
	    mean_delay /= n_good_chips;
	  }
	  
	  //
	  // Open today's output file
	  // Output from a series of tests will be added to the results file
	  //

          FILE* outfile = OpenResults("StrobeSpeedAnalysis", strun, module, run, scan, 1, 0); 
	  
	  if(outfile==NULL){
	    printf("\nCould not open result file %s\n",f_name);
	    return;
	  } else {
 	    
	    fprintf(outfile,"# Run %d Scan %d Module %d\n",run,scan,module);
	    fprintf(outfile,"#\n");
	    fprintf(outfile,"%% TriggerDelay %d\n",trigdelay);
	    fprintf(outfile,"%% Vdd = %3.1f\n",Vdd);
	    fprintf(outfile,"#\n");

	    fprintf(outfile,"%%Chip\tRise\tFall\tDiff\n");
	    for(link=0;link<MAX_LINKS;link++){
	      for(chip=0;chip<MAX_CHIPS;chip++){
		fprintf(outfile,"%d\t%4.2f\t%4.2f\t%4.2f\n",(6*link+chip),
			chip_rise[link][chip],chip_fall[link][chip],chip_fall[link][chip]-chip_rise[link][chip]);
	      }
	    }
	    fprintf(outfile,"%%\n");
	    fclose(outfile);
	  }
	
	  //
	  // In addition, output data in condensed format to fit Heinz's chip speed
	  // analysis macros. 
	  //
	  
	  sprintf(f_name,"%sspeed_%s_%d.txt",RESULTS_DIRECTORY,mname->GetName(),date);
	  outfile = fopen(f_name,"a+");
	  
	  if(outfile==NULL){
	    printf("\nCould not open result file %s\n",f_name);
	    return;
	  } else {
	    printf("\nFile %s opened to append results \n",f_name);
	    
	    fprintf(outfile,"%%StrobeSpeedAnalysis\n");
	    fprintf(outfile,"%%\n");
	    
	    fprintf(outfile,"%3.1f %s %d %d %d\n",Vdd,mname->GetName(),run,scan,trigdelay);
	    for(link=0;link<MAX_LINKS;link++){
	      for(chip=0;chip<MAX_CHIPS;chip++){
		fprintf(outfile,"%d %4.2f %4.2f\n",(6*link+chip),chip_rise[link][chip],chip_fall[link][chip]);
	      }
	    }
	    fprintf(outfile,"%%\n");
	    fclose(outfile);
	  }


	  //
	  // Output results to screen
	  //

	  printf("\n  Number of strobe delay counts per 25 ns pulse\n");
	  for(link=0;link<MAX_LINKS;link++){
	    for(chip=0;chip<MAX_CHIPS;chip++){
	      printf("  Chip %d\t%3.1f\n",(link*6+chip),
		     chip_fall[link][chip]-chip_rise[link][chip]); 
	    }
	  }

	}else {
	  if(debug) printf("No data for module %d\n",module);
	}
      }else{
	if(debug) printf("Histograms empty for module %d\n",module);
      }
    }else{
      if(debug) printf("No histograms for module %d\n",module);
    }
  }  // end for (module)
  
  if(StrobeDelay==0){

    // Delete objects and close the ROOT file.
    // We do not do this if graphical display was 
    // selected since the plots will disappear!

    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
	if(hp_chip[link][chip]) hp_chip[link][chip]->Delete();
	if(f1[link][chip]) f1[link][chip]->Delete();
	if(f2[link][chip]) f2[link][chip]->Delete();
      }
    }
    strun->Close();
    delete strun;
  }    

  if(time) delete time;
  if(place) delete place;
  if(mname) delete mname;
}


//
// wrappers
//


void StrobeSpeedAnalysis(Char_t *file){

  Int_t trigdelay, run, scan, mod;
  Float_t Vdd;

  FILE *infile = fopen(file,"r");
 
  while(fscanf(infile,"%f %d %d %d %d",&Vdd, &trigdelay, &run, &scan, &mod)!=EOF) {
    StrobeSpeedAnalysis(run,scan,-1,Vdd,trigdelay,1,0);
  }
}
 
