void T7RadInSituTests(Char_t *logname, Short_t interactive) {

  // reduced set of module characterisation during irradition in T7
  // HP 3.4.01
  // mods PWP 11.04.01
  // mods LE  15.04.01
  // Further mods and merges. LE and other irradiatiors. From T7 to CVS 06.08.01 

  Int_t n_mods = 2;

  Bool_t DoByPass      = kTRUE;
  Bool_t DoSpeed       = kTRUE;
  Bool_t DoStrobeDelay = kTRUE;
  Bool_t DoTrim        = kTRUE;
  Bool_t DoThreePG     = kTRUE;
  Bool_t DoCurrent     = kTRUE;
  Bool_t DoPulse       = kTRUE;
  Bool_t DoPower       = kTRUE;
  Bool_t DoIccVsFE     = kTRUE;

  Float_t StartVdd,StopVdd,StepVdd;
  Int_t n_configs;


  // Try using the StopWatch
  TStopwatch* FryMe_speed = new TStopwatch();


  // no need to load macros which will not be used 
  // - will be more stable this way...

  if(DoByPass)     { gROOT->ProcessLine(".L FullBypassTest.cpp");  }
  if(DoSpeed)      { gROOT->ProcessLine(".L StrobeSpeed.cpp");     }
  if(DoStrobeDelay){ gROOT->ProcessLine(".L StrobeDelay.cpp");     }
  if(DoTrim)       { gROOT->ProcessLine(".L TrimRange.cpp");       }
  if(DoThreePG)    { gROOT->ProcessLine(".L ThreePointGain.cpp");  }
  if(DoCurrent)    { gROOT->ProcessLine(".L CurrentScan.cpp");     }
  if(DoPulse)      { gROOT->ProcessLine(".L ResponseVsDelay.cpp"); }
  if(DoPower)      { gROOT->ProcessLine(".L T7VoltageScan.cpp");   }
  if(DoIccVsFE)    { gROOT->ProcessLine(".L T7IccVsFEScan.cpp");   }

  // Turn off stop button.
  e->stopper = 0;

  FILE *logfile = fopen(logname,"a");
  
  TDatime *dt;
  dt = new TDatime;
  fprintf(logfile,"====T7-Module-characterization====\n");
  fprintf(logfile,"started %d at %d\n",dt->GetDate(),dt->GetTime());
	  

  if (DoByPass)
    {
      if(interactive){
	printf("\n **** Remember to give all values with a leading zero,    ****\n");
	printf("**** since the first character might be ignored by root. ****\n");
	printf("****              e.g. Vdd = 05.0                        ****\n\n");
	printf("FullBypassTest: Scan Vdd from ");
	scanf("%f",&StartVdd);
	printf(" to ");
	scanf("%f",&StopVdd);
	printf(" in steps of ");
	scanf("%f",&StepVdd);
	printf(".\nNumber of configs (Barrel 36, FWD:62): ");
	scanf("%d",&n_configs);
	printf("\nFullBypassTest Vdd = %3.1f-%3.1f step: %3.1f n_configs; %d\n",
	       StartVdd,StopVdd,StepVdd,n_configs);
      }else{
	StartVdd = 4.0;
	StopVdd  = 3.3;
	StepVdd  = 0.1;
	n_configs = 36;
      }
	
      e->RecoverTrips();
      dt->Set(); // get current system time without creation of new object
	
      fprintf(logfile,"BYPASS %d %d %d %d\n",dt->GetDate(),dt->GetTime(),e->runnum,e->scannum);
      FullBypassTest(StartVdd,StopVdd,StepVdd,n_configs);
      e->Restart();
    }


  if (DoSpeed)
    {
      if(interactive){
	printf("\n **** Remember to give all values with a leading zero,    ****\n");
	printf("**** since the first character might be ignored by root. ****\n");
	printf("****              e.g. Vdd = 05.0                        ****\n\n");
	printf("StrobeSpeed: Scan Vdd from ");
	scanf("%f",&StartVdd);
	printf(" to ");
	scanf("%f",&StopVdd);
	printf(" in steps of ");
	scanf("%f",&StepVdd);
	printf("\nStrobeSpeed Vdd = %3.1f-%3.1f step: %3.1f\n",StartVdd,StopVdd,StepVdd);
      }else{
	StartVdd = 4.6;
	StopVdd  = 3.4;
	StepVdd  = 0.2;
      }

	e->RecoverTrips();
	dt->Set();
	fprintf(logfile,"SPEED %d %d %d %d\n",dt->GetDate(),dt->GetTime(),e->runnum,e->scannum);
	StrobeSpeed(-1, StartVdd,StopVdd,StepVdd);
    }


  //
  // Set Vdd for the to be used during the following scans.
  //

  if(interactive){
    Float_t Vdd;
    for(int module=0;module<n_mods;module++)
      {
	if(e->m[module]->present){
	  printf("\n **** Remember to give all values with a leading zero,    ****\n");
	  printf("**** since the first character might be ignored by root. ****\n");
	  printf("****              e.g. Vdd = 05.0                        ****\n\n");
	  printf("\nPlease give Vdd for module %d: ",module);
	  scanf("%f",&Vdd);
	  printf("Setting Vdd = %3.1f for module %d",Vdd,module);
	  e->ConfigureVariable(module,33,Vdd);
	}
      }
  }else{
    e->ConfigureVariable(33,4.0);
  }

  if (DoStrobeDelay)
    {
      e->RecoverTrips();
      dt->Set();
      fprintf(logfile,"STROBE %d %d %d %d\n",dt->GetDate(),dt->GetTime(),e->runnum,e->scannum);
      StrobeDelay(1);
    }

  if (DoTrim) 
    {
      e->RecoverTrips();
      dt->Set();
      fprintf(logfile,"TRIM %d %d %d %d\n",dt->GetDate(),dt->GetTime(),e->runnum,e->scannum);
      fclose(logfile);
      TrimRange();
      FILE *logfile = fopen(logname,"a");
    }

  if (DoThreePG) 
    {
      e->RecoverTrips();
      dt->Set();
      fprintf(logfile,"THREE POINT GAIN %d %d %d %d\n",dt->GetDate(),dt->GetTime(),e->runnum,e->scannum);
      fclose(logfile);
      ThreePointGain();
      FILE *logfile = fopen(logname,"a");
    }


  if (DoCurrent) 
    {
      e->Restart();
      e->RecoverTrips();
      dt->Set();
      fprintf(logfile,"CURRENT %d %d %d %d\n",dt->GetDate(),dt->GetTime(),e->runnum,e->scannum);
      CurrentScan();
    }


  if (DoPulse)
    {
      e->RecoverTrips();
      dt->Set();
      fprintf(logfile,"PULSE %d %d %d %d\n",dt->GetDate(),dt->GetTime(),e->runnum,e->scannum);
      ResponseVsDelay(2.0,3.2,1.0);
    }

  if (DoPower)
    {
      dt->Set();
      fprintf(logfile,"POWER %d %d %d %d\n",dt->GetDate(),dt->GetTime(),e->runnum,e->scannum+1);
      T7VoltageScan(1);
    }

  if (DoIccVsFE)
    {
      e->RecoverTrips();
      dt->Set();
      fprintf(logfile,"IccVsFE %d %d %d %d\n",dt->GetDate(),dt->GetTime(),e->runnum,e->scannum+1);
      T7IccVsFEScan();
    }

  dt->Set();
  fprintf(logfile,"ended %d at %d %d %d\n",dt->GetDate(),dt->GetTime(),e->runnum,e->scannum);
  fclose(logfile);

  // Turn on stop button.
  e->stopper = 1;

  printf("InSitu Test completed\n");
  FryMe_speed->Print();

}









