void T7VoltageScan(Int_t find_two_fc)
{
  // version 0.1 HP
  // mods PWP 11.04.01
  // and again 06.06.01 for parameters.h

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t module, chip;

  // Store settings before the scan
  Int_t trtype    = e->burst_trtype;
  Int_t ntrigs    = e->burst_ntrigs;
  Int_t cal_loop  = e->do_cal_loop;
  Int_t do_fits   = e->do_fits;
  Char_t f_name[100];
  Int_t n;
  Int_t start_scan = e->scannum+1;
  Bool_t RAL=kFALSE;

  Float_t in_vdd[MAX_MODULES];
  Float_t in_charge[MAX_MODULES][12];
  Int_t comp_mode[MAX_MODULES][12];
  Int_t edge[MAX_MODULES][12];
  Int_t route_mask[MAX_MODULES][12];

  for (module=0; module<MAX_MODULES; module++){
    if(e->m[module]->present){
      for(chip=0;chip<12;chip++){
	in_vdd[module] =           e->RetrieveVariable(module,33);
	in_charge[module][chip] =  e->RetrieveVariable(module,chip,2);
	comp_mode[module][chip] =  e->RetrieveVariable(module,chip,11);
	edge[module][chip] =       e->RetrieveVariable(module,chip,13);
	route_mask[module][chip] = e->RetrieveVariable(module,chip,14);
      }
    }
  }


  sprintf(f_name,"%sstT7currents_%d_%d.root",DATA_DIRECTORY,e->runnum,start_scan);

  //outputfile

  TFile *outfile = new TFile(f_name,"recreate");
  TModuleDCS* dcs;

  TTree* DCSTree = new TTree("DCSTree","DCSTree");
  DCSTree->Branch("DCSBranch","TModuleDCS",&dcs,8000,1);


  // Configure the scan
  e->StOutScanReset();
  e->ascii_nt=0;
  e->abort=0;

  e->do_cal_loop=1;        // cal loop ON 
  e->do_mask_test=0;       // alternating mask patterns OFF 

  e->burst_trtype=3;       // CAL + L1A
  e->burst_ntrigs=1000;

  e->ConfigureVariable(14,0);  // Route Mask OFF


  /* I would suggest that we use Vth = 500, normally the standard for 
     Idd  vs Vdd measurements. In addition, saves time and risk of confusion.
     
     LE 16/6-01.

  //
  // If the calling routine/user has not been able to set 2fC threshold,
  // we can find an approximate value now using the default timing
  //

  if(find_two_fc){
    e->do_fits=1;                 // integrated fitting ON

    e->ConfigureVariable(11,0);   // Compression Mode 0.
    e->ConfigureVariable(13,0);   // Edge detect off.

    e->ConfigureVariable(2,20);   // 2.00fC input charge
    e->ConfigureScan(1,0,350,5);  // range tbc

    st_scan(11,-1);

    for (module=0; module<e->GetMaxModules(); module++){
      if(e->m[module]->present){

        for(chip=0;chip<12;chip++){
          e->ConfigureVariable(module,chip,1,e->m[module]->chip_mean[chip/6][chip%6]);
        } // end for(chip)  
      }
    } // end for(module)
  }
  */

  e->ConfigureVariable(1,500);  // Vth = infinity

  // get old module values for Vcc and Vdd
  Float_t vdd_set[10];
  Float_t vcc_set[10];

  e->DCSAction(); // DCSAction will attempt trip recovery

  for(n=0;n<e->GetMaxModules();n++){
    if(e->m[n]->present){
      vdd_set[n] = e->m[n]->DCS->vdd_set;
      vcc_set[n] = e->m[n]->DCS->vcc_set;
      printf("T7==> Module %s was Vdd=%2.2f and Vcc=%2.2f\n",e->m[n]->name,vdd_set[n],vcc_set[n]);
    }
  }


  //
  // Scan Vdd, making current measurements on the way
  //

  Float_t v;
  Float_t Vddstart=3.3;
  Float_t Vddstop=5.5;
  Float_t Vddstep=0.1;

  // First set Vdd to minimum value, and recover trips
  e->ConfigureVariable(33,Vddstart);
  gSystem->Sleep(5000); // Sleep 5000 mS
  e->DCSAction();       // DCSAction will attempt trip recovery

  printf("===>T7 Voltage scan on Vdd: \n");
  for(v=Vddstart;v<Vddstop;v+=Vddstep) {

    e->ConfigureVariable(33,v);
    gSystem->Sleep(5000); // Sleep 5000 mS
    e->DCSAction();       // DCSAction will attempt trip recovery

    for(n=0;n<e->GetMaxModules();n++){
      if(e->m[n]->present){
        DCSTree->SetBranchAddress("DCSBranch",&e->m[n]->DCS);
        DCSTree->Fill();
      }
    }
  }

  printf("T7==> Restoring Vdd Settings\n");
  for(n=0;n<e->GetMaxModules();n++){
    if(e->m[n]->present) e->ConfigureVariable(n,33,vdd_set[n]);
  }


  //
  // Scan Vcc, making current measurements on the way
  //

  Float_t Vccstart=3.0;
  Float_t Vccstop=4.5;
  Float_t Vccstep=0.1;

  // First set Vcc to minimum value, and recover trips
  e->ConfigureVariable(31,Vccstart);
  gSystem->Sleep(5000); // Sleep 5000 mS
  e->DCSAction();       // DCSAction will attempt trip recovery

  printf("===>T7 Voltage scan on Vcc: \n");
  for(v=Vccstart;v<Vccstop;v+=Vccstep) {

    e->ConfigureVariable(31,v);
    gSystem->Sleep(5000); // Sleep 5000 mS
    e->DCSAction();       // DCSAction will attempt trip recovery

    for(n=0;n<e->GetMaxModules();n++){
      if(e->m[n]->present){
        DCSTree->SetBranchAddress("DCSBranch",&e->m[n]->DCS);
        DCSTree->Fill();
      }
    }
  }

  printf("T7==> Restoring Vcc Settings\n");
  for(n=0;n<e->GetMaxModules();n++){
    if(e->m[n]->present) e->ConfigureVariable(n,31,vcc_set[n]);
  }

  
  gSystem->Sleep(5000); // Sleep 5000 mS
  e->DCSAction();       // DCSAction will attempt trip recovery

  DCSTree->Write();
  outfile->Close();
  e->LVStatus();

  // Reset values after scan
  e->burst_trtype = trtype;   
  e->burst_ntrigs = ntrigs;
  e->do_cal_loop  = cal_loop;
  e->do_fits      = do_fits;

  for (module=0; module<MAX_MODULES; module++){
    if(e->m[module]->present){
      for(chip=0;chip<12;chip++){
	e->ConfigureVariable(module,33,in_vdd[module]);
	e->ConfigureVariable(module,chip,2,in_charge[module][chip]);
	e->ConfigureVariable(module,chip,11,comp_mode[module][chip]);
	e->ConfigureVariable(module,chip,13,edge[module][chip]);
	e->ConfigureVariable(module,chip,14,route_mask[module][chip]);
      }
    }
  }
}

void T7Voltagescan() {

  T7VoltageScan(1);
}




