//
// TLTT.h   Class header file for Long Term
//          Tests of Hybrids or Modules
//
// Version 0.1  DGC  17/10/01
// Version 0.2  PWP  20/12/01
// Version 0.21 BJG  27/03/02
// Version 0.22 BJG  30/04/02 - Update function names + typo
// Version 0.23 PWP  08/07/02 - add dcs_only 
// Version 0.24 DGC  12/07/02 - Add cold test
// Version 0.25 BJG  23/09/02 - Add Alarm function
//

#define MAX_READINGS 10000

class TLTT {

public:
  TLTT(Int_t small = 300, Int_t big = 7200, Int_t tot = 360000) ;

  void HybridLTT();
  void HybridColdTest();
  void IVLTT();
  void ModuleLTT();

private:

  void DoTest();
  void InitialiseTiming() ;
  void OpenLogFiles() ;
  void MakeMonitoringMeasurements() ;
  void DoPeriodicTests() ;
  void DoFinalTests() ;
  void OpenDisplay() ;
  void UpdateDisplay() ;
//  void CloseDisplay() ;
  void WriteSummaryFile() ;
  void ActOnAlarm() ;

  Int_t testing_modules;
  Int_t cold_test;
  Int_t dcs_only;

  Int_t start_run ;
  Int_t start_scan ;

  Int_t small_interval ;
  Int_t big_interval ;
  Int_t total_length ;
  Int_t number_of_big_loops ;

  Float_t t1_val[MAX_MODULES][MAX_READINGS] ;
  Float_t t2_val[MAX_MODULES][MAX_READINGS] ;
  Float_t idet_val[MAX_MODULES][MAX_READINGS] ;
  Float_t icc_val[MAX_MODULES][MAX_READINGS] ;
  Float_t idd_val[MAX_MODULES][MAX_READINGS] ;
  Float_t no_val[MAX_MODULES][MAX_READINGS] ;
  Float_t time[MAX_MODULES][MAX_READINGS] ;
  Char_t mname[MAX_MODULES][32];

  TDatime *start_dt ;
  TStopwatch* timer ;
  Double_t start_time ;

  Char_t start_str[48] ;
  Char_t end_str[48] ;
  Char_t update_str[48] ;

  Char_t txt_fname[MAX_MODULES][128] ;
  Char_t psfilename[132] ;
  Char_t dcs_fname[64];
  TFile* dcs_outfile ;
  TModuleDCS* dcs;
  TTree* DCSTree ;

  Char_t failComment[128];
  
  Int_t iloop_readings ;
  Int_t alarm ;

  TCanvas* LTTCanvas ;
  TPad* LTTInfo ;
  TPaveText *tpaveinfo ;
  TPaveText *tpaveinfo2 ;
  Float_t tmax_actual ;
  Float_t tmin_actual ;
  Float_t imax_actual ;
  Float_t imin_actual ;
  Float_t idmax_actual ;
  Float_t idmin_actual ;
  Float_t nomax_actual ;
  Float_t nomin_actual ;

  TPad* LTTPadTGraph;
  TPad* LTTPadTVal;
  TPaveText *tpavet;

  TPad* LTTPadIGraph;
  TPad* LTTPadIVal;
  TPaveText *tpavei;

  TPad* LTTPadIdetGraph;
  TPad* LTTPadIdetVal;
  TPaveText *tpaveid;

  TPad* LTTPadNOGraph;
  TPad* LTTPadNOVal;
  TPaveText *tpaveno;

  TGraph *graph0 ;
  TMarker *markert1 ;
  TMarker *markert2 ;
  TText *keyt1 ;
  TText *keyt2 ;

  TGraph *graphi0 ;
  TMarker *markeri1 ;
  TMarker *markeri2 ;
  TText *keyi1 ;
  TText *keyi2 ;

  TGraph *graphid0 ;
  TMarker *markerid ;
  TText *keyid ;

  TGraph *graphno0 ;
  TMarker *markerno ;
  TText *keyno ;

  TGraph *grapht1[MAX_MODULES] ;
  TGraph *grapht2[MAX_MODULES] ;
  TGraph *graphi1[MAX_MODULES] ;
  TGraph *graphi2[MAX_MODULES] ;
  TGraph *graphid1[MAX_MODULES] ;
  TGraph *graphno1[MAX_MODULES] ;

  Float_t trange[2] ;
};

