void TestFitData(int run, int scan, int moi, int likelihood, int use_range){

  //
  //  A routine to fit st scan histograms - FAST!!!
  //  The fit routine used is based upon the integrated fitting within
  //  stdll, but compiled into a standalone DLL (KwikFit.dll)
  //
  // The fitted data is saved back into the root file to be compatible
  // with the on line fitting.  If online fit data is present in the
  // file, the exisiting fit data is used instead!!
  //
  //  A reduced set of cuts have been implemented as follows:
  //  o Channels with <1% occupancy in any bin are marked as "DEAD"
  //  o Channels with >98% occupancy in every bin are marked "STUCK"
  //  o Channels which do not reach 100% efficiency are marked "UNDER"
  //  o Channels which reach > 100% efficiency are marked "OVER"
  //    (Such a condition would imply decoder errors, inviting the
  //     user to check the digital integrity of their system)
  //
  //  Version 0.1, PWP 26.10.00
  //  Version 0.2, LE  23.05.01
  //  Version 0.21 BJG 30.04.02 - Fix some memory leaks
  //


  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  char  f_name[128];
  char  h_name[64], h_title[64];
  char  scan_name[64];
  char  s[256], unit[32];
  short scan_type, fit_type;


  printf("KwikFit called for run %d scan %d module %d\n",run,scan,moi);

  //
  // Load other files needed by this one
  //

  gSystem->Load("KwikFit.dll");
  gROOT->LoadMacro("stan_util.cpp");

  // Global style variations
  stan_styles();
  gStyle->SetOptStat(0);

  TObjString *time = 0;
  TObjString *place = 0;
  TObjString *mname = 0;

  //
  // Open the root data file and obtain pointers to the event histogram
  //

  sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan);

  TFile *strun = new TFile(f_name,"UPDATE");
  TH1F *h_nevent= (TH1F*)strun->Get("h_scan_evcnt");

  //
  // Open the output canvas
  //

  TCanvas* c1 =0;

  c1=new TCanvas ("c1","KwikFit",1100,750);


//
//  loop over modules here
//

  Int_t start = 0;
  Int_t end   = 0;

  if(moi==-1){
    start = 0;
    end   = MAX_MODULES;

  } else {
    start = moi;
    end   = moi+1;
  }


  Int_t last_good = -1;

  for(moi=start;moi<end;moi++){

    sprintf(h_name,"ModuleName;%d",moi+1);
    if(time) delete time;
    if(place) delete place;
    if(mname) delete mname;
    time  = (TObjString*)strun->Get("Time");
    place = (TObjString*)strun->Get("Place");
    mname = (TObjString*)strun->Get(h_name);

    sprintf(h_name,"h_scan0;%d",moi+1);
    TH2F *h_scan0= (TH2F*)strun->Get(h_name);
    sprintf(h_name,"h_scan1;%d",moi+1);
    TH2F *h_scan1= (TH2F*)strun->Get(h_name);

    sprintf(h_name,"h_mean0;%d",moi+1);
    TH2F *h_mean0= (TH2F*)strun->Get(h_name);
    sprintf(h_name,"h_mean1;%d",moi+1);
    TH2F *h_mean1= (TH2F*)strun->Get(h_name);

    sprintf(h_name,"h_sigma0;%d",moi+1);
    TH2F *h_sigma0= (TH2F*)strun->Get(h_name);
    sprintf(h_name,"h_sigma1;%d",moi+1);
    TH2F *h_sigma1= (TH2F*)strun->Get(h_name);

    sprintf(h_name,"h_code0;%d",moi+1);
    TH2F *h_code0= (TH2F*)strun->Get(h_name);
    sprintf(h_name,"h_code1;%d",moi+1);
    TH2F *h_code1= (TH2F*)strun->Get(h_name);

    TPad *pad_p = 0;
    //
    // only attempt plotting or fitting if data exists for the module
    //

    if((h_scan0!=0)||(h_scan1!=0)){

      last_good = moi;

      sprintf(scan_name,"Unknown");
      sprintf(unit,"");
      TString title = h_scan0->GetTitle();
      title.ToUpper();
      if (title.Contains("THRESHOLD")){
        fit_type = 1;
        scan_type = 1;
        sprintf(scan_name,"THRESHOLD (mV)");
        sprintf(unit,"mV");
      }
      if (title.Contains("VCAL")){
        fit_type = 2;
        scan_type = 2;
        sprintf(scan_name,"VCAL (mV)");
        sprintf(unit,"mV");
      }
      if (title.Contains("STROBE")){
        fit_type = 1;
        scan_type = 3;
        sprintf(scan_name,"Strobe Delay");
        sprintf(unit,"");
      }
      if (title.Contains("QTHR")){
        fit_type = 1;
        scan_type = 41;
        sprintf(scan_name,"QTHR (fC)");
        sprintf(unit,"fC");
      }
      if (title.Contains("QCAL")){
        fit_type = 2;
        scan_type = 42;
        sprintf(scan_name,"QCAL (fC)");
        sprintf(unit,"fC");
      }


      if(c1){
        c1->cd();
        c1->Clear();
        TPad *pad_t = new TPad("pad_t","Text",   0.00,0.90,1.00,1.00,33);
        pad_p = new TPad("pad_p","Plots",  0.00,0.00,1.00,0.90,33);

        pad_t->Draw();
        pad_p->Draw();
        pad_p->Divide(2,4,0,0);

        pad_t->cd();

        TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
        t1->SetFillColor(10);
        t1->SetTextAlign(13);
        t1->SetTextSize(0.3);
        t1->SetTextColor(4);
  
        sprintf(s,"ATLAS SCT Module Test");
        if(time)  sprintf(s,"%s  -  %s",s,time->GetName());
        if(place) sprintf(s,"%s  -  %s",s,place->GetName());
        if(mname) sprintf(s,"%s  -  Module %s",s,mname->GetName());
        t1->AddText(s);
        sprintf(s,"Run %d   Scan %d   Module %d   Scan type: %s", run, scan, moi, scan_name);
        t1->AddText(s);

        t1->Draw();

        pad_p->cd(1);
        h_scan0->Draw("COLZ");
        h_scan0->SetXTitle("Channel Number");
        pad_p->cd(2);
        h_scan1->Draw("COLZ");
        h_scan1->SetXTitle("Channel Number");

        c1->Update();
      }

      //
      // Fit the data again with KwikFit
      //

      TKwikFit *kf = new TKwikFit(moi);
      kf->likelihood=likelihood;
      kf->use_range=use_range;
      if(mname) kf->SetModuleName(mname->GetName());

      kf->LoadScan(h_scan0,h_scan1,h_nevent);
      kf->PrintScan();

      printf("\nFitting Data for module %d...\n",moi);

      TStopwatch* patek_phillipe = new TStopwatch();

      kf->FitScan(fit_type,-1);

      patek_phillipe->Print();

      kf->PrintScan();

      sprintf(h_title,"Sigma (%s)",scan_name);
      kf->h_sigma0->SetMarkerColor(kBlue);
      kf->h_sigma0->SetXTitle("Channel Number");
      kf->h_sigma0->SetYTitle(h_title);
      kf->h_sigma1->SetMarkerColor(kBlue);
      kf->h_sigma1->SetXTitle("Channel Number");
      kf->h_sigma1->SetYTitle(h_title);

      sprintf(h_title,"Fit Code");
      kf->h_code0->SetMarkerColor(kBlue);
      kf->h_code0->SetXTitle("Channel Number");
      kf->h_code0->SetYTitle(h_title);
      kf->h_code1->SetMarkerColor(kBlue);
      kf->h_code1->SetXTitle("Channel Number");
      kf->h_code1->SetYTitle(h_title);

      sprintf(h_title,"Chisq/NDF");
      kf->h_chisq0->SetMarkerColor(kBlue);
      kf->h_chisq0->SetXTitle("Channel Number");
      kf->h_chisq0->SetYTitle(h_title);
      kf->h_chisq1->SetMarkerColor(kBlue);
      kf->h_chisq1->SetXTitle("Channel Number");
      kf->h_chisq1->SetYTitle(h_title);


      // Plot new fit results

      pad_p->cd(1);
      kf->h_mean0->Draw("same");
      pad_p->cd(2);
      kf->h_mean1->Draw("same");

      pad_p->cd(3);
      kf->h_sigma0->Draw("P");
      pad_p->cd(4);
      kf->h_sigma1->Draw("P");

      pad_p->cd(5);
      kf->h_chisq0->Draw("P");
      pad_p->cd(6);
      kf->h_chisq1->Draw("P");

      pad_p->cd(7);
      kf->h_fom0->Draw("P");
      pad_p->cd(8);
      kf->h_fom1->Draw("P");

      c1->Update();

//

      float n[128];

      float x[128];
      float xe[128];
      float y[32][128];
      float ye[32][128];

      float scan_start, scan_stop, scan_step;
      short scan_max_points;

      Int_t start_ch=0;

      Int_t i,j;

      for(i=0;i<h_nevent->GetNbinsX();i++){
        n[i]= h_nevent->GetBinContent(i);
        if(n[i]==0) n[i]=1;  // maybe this causes some problems?
      }

      TAxis* axis = h_scan0->GetYaxis();
      scan_max_points = axis->GetNbins();
      scan_start = axis->GetBinCenter(1);
      scan_stop  = axis->GetBinCenter(scan_max_points);
      scan_step  = axis->GetBinWidth(1);
      for(i=0;i<scan_max_points; i++){
        x[i] = axis->GetBinCenter(i+1);
        xe[i] = 1;
      }

      Int_t nf;
      Double_t err2;

      nf = (h_scan0->GetMaximum());
      if (nf<1.0) nf =1.0;

      for(i=0;i<32;i++){
        for(j=0;j<scan_max_points; j++){
          y[i][j] = h_scan0->GetBinContent((770*(j+1))+i);    //PWP bugfix  10.03.00
          if(nf==1){
            err2 = ((n[j]*y[i][j])+1) * (1+n[j]-(n[j]*y[i][j])) / n[j];
            ye[i][j]= sqrt(err2); // this gives the error on the original count
            ye[i][j]/=n[j];       // this scales the error to the range 0 to 1 
          }else{
            err2 = (y[i][j]+1) * (1+n[j]-y[i][j]) / n[j];
            ye[i][j]= sqrt(err2);
          }
        }
      }


      TF1*    fr[32];
      TGraph* gr[32];
      TH2F*   hr[32];

      TCanvas* csc=new TCanvas ("csc",s,1100,750);
      TPad* pad_c0 = new TPad("pad_c0","Scurves", 0.00,0.00,1.00,1.00,33);

      pad_c0->Divide(4,2,0,0,10);

      pad_c0->Draw();

      for(i=0;i<8;i++){

        pad_c0->cd(i+1);

        sprintf(h_name,"h_sc%d",i);
        sprintf(h_title,"Channel %d",i+start_ch);
        hr[i] = new TH2F(h_name,h_title,2,scan_start,scan_stop,2,-(nf*0.1),(nf*1.1));
        hr[i]->Draw();  
        hr[i]->SetXTitle(scan_name);  

        sprintf(h_title,"fr_%d",i);
        fr[i] = new TF1 (h_title,erfc,  0, 1000, 3);
        fr[i]->SetParNames("Constant","Mean","Sigma");
        fr[i]->SetParameters(nf,kf->h_mean0->GetBinContent(i),kf->h_sigma0->GetBinContent(i));
        fr[i]->Draw("SAME");

        sprintf(h_title,"Channel %d",i+start_ch);
        gr[i] = new TGraphErrors(scan_max_points,x,y[i+start_ch],xe,ye[i]);
        gr[i]->SetMarkerColor(kBlue);
        gr[i]->SetTitle(h_title);
        gr[i]->Draw("EP");

        printf("Ch %d\tmn %3.2f\tsig %3.2f\tchisq %f\tfom %f\n", i,
          kf->h_mean0->GetBinContent(i),
          kf->h_sigma0->GetBinContent(i),
          kf->h_chisq0->GetBinContent(i),
          kf->h_fom0->GetBinContent(i));
      }

    } else {
      printf("\nNo data for module %d\n",moi);
    }
  }

  if(time) delete time;
  if(place) delete place;
  if(mname) delete mname;

  //
  // Note that the action of closing the root file causes all plots to disappear!!
  //

//  strun->Close();
//  delete strun;
}

