void TrimRange(Float_t charge, Int_t full, Int_t method, Int_t graphics, Int_t postscript){

  //
  // version 0.2 PWP 17.05.01
  // version 0.3 LE  07.06.01 - Use different number of points for different ranges.
  // version 0.31 PWP 12.06.01 - revised to use switch construct and add extra case.
  
  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t n_ranges = 4;
  Float_t ranges[5] = {0,1,2,3,-1};
  Int_t scans[64];
  
  switch(full){

    case 1:
      printf("Starting FULL TrimDAC/TrimRange characterisation\n");

      Int_t n_points[4] = {16, 16, 16, 16};
      Float_t trims[64] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
                           0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
                           0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
                           0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
      break;
 
    case 2:
      printf("Starting CUSTOM TrimDAC/TrimRange characterisation\n");

      Int_t n_points[4] = {16, 4, 4, 4};
      Float_t trims[64] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
                           3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0};
      break;

    case 3:
      printf("Starting FAST TrimDAC/TrimRange characterisation\n");

      Int_t n_points[4] = {2, 2, 2, 2};
      Float_t trims[64] = {3,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
      break;

    default:
      printf("Starting STANDARD TrimDAC/TrimRange characterisation\n");

      Int_t n_points[4] = {4, 4, 4, 4};
      Float_t trims[64] = {3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0};
      break;
  }

  // Save settings
  e->SaveState();

  e->StOutScanReset();
  e->ascii_nt=0;
  e->abort=0;

  e->do_cal_loop=1;        // cal loop ON
  e->do_autostop=1;        // autostop ON
  e->do_fits=1;            // integrated fitting ON
  e->do_mask_test=0;       // alternating mask patterns OFF 
  e->config_frequency = 0; // do NOT send full configuration before each event

  e->burst_trtype = 3;     // cal + L1A
  e->burst_ntrigs = 100;   // 100 triggers per burst per cal line

  e->ConfigureVariable(11,1);  // X1X compression 
  e->ConfigureVariable(13,0);  // EDGE DETECT OFF
  e->ConfigureVariable(14,0);  // Route Mask OFF
  e->ConfigureVariable(42,charge);  // 42 - QCAL (fC)


  Int_t i=0;
  Int_t range;
  Int_t point;

  for(range=0;range<n_ranges;range++){
    for(point=0;point<n_points[range];point++){

      if(e->abort==0){
        e->ConfigureVariable(12,ranges[range]);
        e->ConfigureVariable(6,trims[(range*MAX_POINTS)+point]);

        e->ConfigureScan(1,0,400,5);
        st_scan(11,-1);

        scans[(range*MAX_POINTS)+point] = e->scannum;
        i++;
      }
    }
  }

  // Restore settings
  e->RestoreState();

  // load the plot macro 
  gROOT->ProcessLine(".L TrimRangePlot.cpp");

  if(e->abort==0){
    TrimRangePlot(e->runnum,n_ranges,n_points,method,charge,
      scans,ranges,trims,1,graphics,postscript);
  }
}

void TrimRange(){

  TrimRange(1.0,2,0,0,1); // charge now in fC
}

void TrimRange(Float_t charge){

  TrimRange(charge,2,0,0,1);
}

void TrimRange(Float_t charge, Int_t full){

  TrimRange(charge,full,0,0,1);
}
