static void SetMask(Short_t mid,Short_t cid){
  e->ConfigureMask(mid,cid);
}

static void SetTrim(Short_t mid,Short_t cid, Short_t var){
  e->ConfigureTrim(mid,cid,var);
}

static void SetTrimRange(Short_t mid,Short_t cid, Float_t var){
  e->ConfigureVariable(mid,cid,12,var);
}

void TrimRangePlot(Int_t run, Int_t n_ranges, Int_t* n_points, Int_t method, Float_t charge, 
                   Int_t* scan, Float_t* TrimRange, Float_t* TrimDAC, 
                   Int_t online, Int_t graphics, Int_t postscript) {

  // version 0.21 PWP 01.11.00
  // version 0.22 PWP 14.11.00 - changes to plotted ranges
  // version 0.23 LE  19.11.00 - automatic compiled fitting if not already done.
  // version 0.24 LE  19.11.00 - chip by chip optimisation.
  // version 0.25 PWP 21.11.00 - various fixes
  // version 0.26 PWP 24.11.00 - add plots for TrimDAC slope and offset
  // version 0.27 PWP 28.11.00 - fixes; hists instead of graphs for par vs channel
  // version 0.28 PWP 22.01.01 - also mask DEAD or STUCK channels
  // version 0.29 LE  16.04.01 - output results to file, give trimrange and charge as arguments.
  // version 0.30 PWP 17.04.01 - consider all four trim ranges
  // version 0.31 LE  07.05.01 - Works for non-consecutive TrimRanges, correcting output, using parameters.h
  // version 0.32 PWP 17.05.01 - Write trimranges to trim and db files; misc fixes
  // version 0.32 PWP 17.05.01 - mask channels with noise  >125% chip mean noise (crude calculation!!)
  // version 0.33 LE  08.06.01 - accept arbitrary number of TrimDAC points for each range
  // version 0.34 PWP 12.06.01 - revisions to wrappers; minor fix re. opening PS file
  // version 0.35 PWP 01.08.01 - minor mods: attempt to trim channel only if slope of trim graph if +ve
  // version 0.36 PWP 06.08.01 - bugfix re. use of histogram bins
  // version 0.37 PWP 09.08.01 - bugfixes: initialisation of misc. variables and objects
  // version 0.38 PWP 23.08.01 - database compliance
  // version 0.39 PWP 24.08.01 - output TrimDAC step, not slope; new defect types
  // version 0.40 PWP 12.09.01 - further database compliance
  // version 0.41 PWP 04.10.01 - bugfixes
  // version 0.42 PWP 16.01.02 - add TIME_POWERED field; rewrite DCS_INFO;
  // version 0.43 PWP 06.02.02 - fix DUT bug; increase offset and step cuts to 4 sigma;
  // version 0.44 PWP 12.02.02 - fix bug affecting identification of DEAD or STUCK channels;
  //                             add extra dimension [link] to TrimGraph pointer array; 
  //                             correct several deficiencies wrt. deletion of objects
  // version 0.45 BJG 28.02.02 - bugfix line 1775 (was 1774)
  // version 0.46 PWP 21.03.02 - add web link to postscript plot if WEB_KEY defined
  //                             place leading zero in PS file timestamp
  // version 0.47 PWP 19.04.02 - bugfix re. TIME_POWERED field
  // version 0.48 BJG 30.04.02 - Rework to use Toolbox
  //                             Put DUT type on plots
  //                             Variable type fixes
  //                             Memory leak fixes
  // version 0.49 PWP 30.05.02 - list untrimmable channels as defects
  // version 0.50 PWP 31.10.02 - bugfix re. generation of final trim settings
  //                             from the fit data: integer representation of
  //                             trim target (with associated rounding errors)
  //                             replaced by float type
  // version 0.51 PWP 26.11.02 - Implement new cuts selected by new_cuts=1:
  //                             NEW: 
  //                               TR_RANGE  0:            1.5 < step <  5.0
  //                                         1: (1.60 * step0) < step < (2.40 * step0)
  //                                         2: (2.60 * step0) < step < (3.40 * step0)
  //                                         3: (3.60 * step0) < step < (4.40 * step0)
  //                               TR_STEP:   (0.60 * mean_step) <  step  < (1.40 * mean_step)
  //                               TR_OFFSET: (mean_offset-4*rms_offset) < offset < (mean_offset+4*rms_offset)
  //                             OLD: 
  //                               TR_RANGE  0:            1.5 < step <  5.0
  //                                         1:            5.0 < step <  8.5
  //                                         2:            8.5 < step < 12.0
  //                                         3:           12.0 < step < 15.5
  //                               TR_STEP:   (mean_step-4*rms_step) <  step  < (mean_step+4*rms_step)
  //                               TR_OFFSET: (mean_offset-4*rms_offset) < offset < (mean_offset+4*rms_offset)
  // version 0.52 PWP 28.11.02 - if running online, use batch mode. Plots will not appear, but
  //                             this avoids some memory leakage in TWIN32Canvas

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t module, range, link, chip, chan, i, point;
  char f_name[128], h_name[64], h_title[64], line[1024];
  char mdut[MAX_MODULES][32];
  char mname[MAX_MODULES][32];
  char rc_name[32];
  char s_method[20];

  Short_t code[MAX_MODULES][MAX_RANGES][MAX_LINKS][MAX_CHANS][MAX_POINTS];
  Float_t mean[MAX_MODULES][MAX_RANGES][MAX_LINKS][MAX_CHANS][MAX_POINTS];
  Float_t sigma[MAX_MODULES][MAX_RANGES][MAX_LINKS][MAX_CHANS][MAX_POINTS];
  Float_t mean_noise[MAX_MODULES][MAX_LINKS][MAX_CHANS];
  Float_t noise_cut[MAX_MODULES][MAX_LINKS][MAX_CHIPS];

  Short_t module_status[MAX_MODULES][MAX_RANGES][MAX_LINKS];

  Float_t target[MAX_TARGETS];
  Float_t ntrim[MAX_RANGES][MAX_TARGETS];
  Float_t trim_rms[MAX_LINKS][MAX_RANGES][MAX_CHIPS][MAX_TARGETS];

  Short_t bad_range[MAX_LINKS][MAX_RANGES][MAX_CHIPS];
  Short_t lo_target[MAX_LINKS][MAX_RANGES][MAX_CHIPS];
  Short_t hi_target[MAX_LINKS][MAX_RANGES][MAX_CHIPS];
  Short_t best_target[MAX_LINKS][MAX_RANGES+1][MAX_CHIPS];
  Short_t best_ntrim[MAX_LINKS][MAX_RANGES+1][MAX_CHIPS];
  Short_t best_range[MAX_LINKS][MAX_CHIPS];
  Short_t pass[MAX_RANGES+1];
  Short_t problem[MAX_RANGES+1];

  Float_t trimmed_mean[MAX_LINKS][MAX_RANGES+1][MAX_CHIPS];
  Float_t trimmed_rms[MAX_LINKS][MAX_RANGES+1][MAX_CHIPS];
  Float_t mean_offset[MAX_LINKS][MAX_RANGES][MAX_CHIPS];
  Float_t mean_step[MAX_LINKS][MAX_RANGES][MAX_CHIPS];
  Float_t rms_offset[MAX_LINKS][MAX_RANGES][MAX_CHIPS];
  Float_t rms_step[MAX_LINKS][MAX_RANGES][MAX_CHIPS];

  Short_t trim_setting[MAX_LINKS][MAX_RANGES+1][MAX_CHANS];
  Float_t vt50_setting[MAX_LINKS][MAX_CHANS];

  Short_t xchan[MAX_CHANS];

  Float_t p0[MAX_LINKS][MAX_RANGES][MAX_CHANS];
  Float_t p1[MAX_LINKS][MAX_RANGES][MAX_CHANS];

  Char_t date_string[MAX_RANGES][12];
  Char_t time_string[MAX_RANGES][12];

  Float_t step_cut[5] = {1.50, 5.00, 8.50, 12.00, 15.50};

  // Use new cuts? PWP 26.11.02
  Int_t new_cuts=1;

  TObjString* time  = 0;
  TObjString* place = 0;
  TObjString* host  = 0;
  TObjString* user  = 0;

  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");
  gROOT->LoadMacro("KwikFit.cpp");
  gROOT->LoadMacro("ToolBox.cpp");
  stan_styles();
  gStyle->SetOptFit(10);

  // 
  // 1. Ensure that the main storage arrays are cleared to start
  //

  printf("Clearing Arrays\n");

  Short_t*  code_ptr = code[0][0][0][0];
  Float_t*  mean_ptr = mean[0][0][0][0];
  Float_t*  sigma_ptr = sigma[0][0][0][0];

  for(i=0;i<(MAX_MODULES*MAX_RANGES*MAX_LINKS*MAX_CHANS*MAX_POINTS);i++){
    *code_ptr++ = 0;
    *mean_ptr++ = 0;
    *sigma_ptr++ = 0;
  }

  sigma_ptr = mean_noise[0][0];
  for(i=0;i<(MAX_MODULES*MAX_LINKS*MAX_CHANS);i++){
    *sigma_ptr++ = 0;
  }

  sigma_ptr = noise_cut[0][0];
  for(i=0;i<(MAX_MODULES*MAX_LINKS*MAX_CHIPS);i++){
    *sigma_ptr++ = 0;
  }

  Short_t*  mstatus_ptr = module_status[0][0];

  for(i=0;i<(MAX_MODULES*MAX_RANGES*MAX_LINKS);i++){
    *mstatus_ptr++ = 0;
  }

  for(module=0;module<MAX_MODULES;module++){
    sprintf(mdut[module],"");
    sprintf(mname[module]," ");
  }

  for(range=0;range<MAX_RANGES;range++){
    for(i=0;i<MAX_TARGETS;i++){
      ntrim[range][i] = 0;
    }
    sprintf(date_string[range]," ");
    sprintf(time_string[range]," ");
  }

  printf("Completed\n");

  // Initialise target array
 
  Float_t start = 2.5;
  Float_t stop = 300;  // was 200, but that is not good enough for
                       // all trim ranges PWP 08.12.00 
  Float_t step = 2.5;

  Float_t f_val;
  Int_t n_targets=0;
  for(f_val=start;f_val<=stop;f_val+=step){

    target[n_targets]=f_val;
    n_targets++;
  }

  for(i=0;i<MAX_CHANS;i++){
    xchan[i]=i;
  }

  // 
  // 2. Load the Fitted Scan Data
  //

  // 2.1 Find today's date - needed for file output

  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Int_t the_time = dt->GetTime();
  delete dt;

  // 2.2 Some simple validity tests and helpful (!) messages

  if(n_ranges<0){
    printf("You requested that scans be processed for %d TrimRange settings.\n",n_ranges);
    printf("You must be having a laugh!!\n");
    printf("Now calm down and try again!!!\n");
  }

  if(n_ranges>MAX_RANGES){
    printf("WARNING: requested number of TrimRange values (%d) exceeds maximum\n",n_ranges);
    printf("         only %d scans will be processed\n",MAX_RANGES);
    n_ranges = MAX_RANGES;
  }

  if(n_points[0]<0){
    printf("You requested that scans be processed for %d TrimDAC settings.\n",n_points[0]);
    printf("You must be having a laugh!!\n");
    printf("Now calm down and try again!!!\n");
  }

  if(n_points[0]>MAX_POINTS){
    printf("WARNING: requested number of TrimDAC values (%d) exceeds maximum\n",n_points[0]);
    printf("         only %d scans will be processed\n",MAX_POINTS);
    n_points[0] = MAX_POINTS;
  }


  // 2.3 Loop to load files

  for(range=0;range<n_ranges;range++){
    for(i=0;i<n_points[range];i++){

      // 2.3.1 Open file

      sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan[(range*MAX_POINTS)+i]);
      TFile *strun = new TFile(f_name);

      if(strun==0){
        printf("Failed to open file %s - EXITING!!\n",f_name);
        return;
      }

      printf("Opened file %s\n",f_name);

      // 2.3.2 Find histograms and fill arrays

      for (module=0; module<MAX_MODULES; module++){

        if(i==0){ 
          if(range==0){
            if(module==0){
              // These items need only be read once

              place = (TObjString*)strun->Get("Place");
              host  = (TObjString*)strun->Get("Host");
              user  = (TObjString*)strun->Get("User");
            }

            // These items need be read once per module

            TObjString* dut   = 0;
            TObjString* name  = 0;

            sprintf(h_title,"DUT;%d",module+1);
            dut  = (TObjString*)strun->Get(h_title);
            if(dut) {
              sprintf(mdut[module],"%s",dut->GetName());
              delete dut;
            }

            sprintf(h_title,"ModuleName;%d",module+1);
            name = (TObjString*)strun->Get(h_title);
            if(name) {
              sprintf(mname[module],"%s",name->GetName());
              delete name;
            }
          }


          if(module==0){
            // These items are read once per range only

            Char_t s_day[4];
            Char_t s_mon[4];
            Int_t date2,hour,min,sec,year;

            if(time) delete time;

            time  = (TObjString*)strun->Get("Time");
            if(time){
              sscanf(time->GetName(),"%s %s %d %d:%d:%d %d",s_day,s_mon,&date2,&hour,&min,&sec,&year);

              Int_t mon=0;
              if(strcmp(s_mon,"Jan")==0) mon=1;
              if(strcmp(s_mon,"Feb")==0) mon=2;
              if(strcmp(s_mon,"Mar")==0) mon=3;
              if(strcmp(s_mon,"Apr")==0) mon=4;
              if(strcmp(s_mon,"May")==0) mon=5;
              if(strcmp(s_mon,"Jun")==0) mon=6;
              if(strcmp(s_mon,"Jul")==0) mon=7;
              if(strcmp(s_mon,"Aug")==0) mon=8;
              if(strcmp(s_mon,"Sep")==0) mon=9;
              if(strcmp(s_mon,"Oct")==0) mon=10;
              if(strcmp(s_mon,"Nov")==0) mon=11;
              if(strcmp(s_mon,"Dec")==0) mon=12;

              sprintf(date_string[range],"%02d/%02d/%04d",date2,mon,year);  
              sprintf(time_string[range],"%02d:%02d:%02d",hour,min,sec);  
            }
          }
        }

        for(link=0;link<MAX_LINKS;link++){

          sprintf(h_name,"h_code%d;%d",link,module+1);
          TH1S *h_code= (TH1S*)strun->Get(h_name);
          sprintf(h_name,"h_mean%d;%d",link,module+1);
          TH1F *h_mean= (TH1F*)strun->Get(h_name);
          sprintf(h_name,"h_sigma%d;%d",link,module+1);
          TH1F *h_sigma= (TH1F*)strun->Get(h_name);

          if(h_mean==0){ // fitted data does not exist

            sprintf(h_name,"h_scan%d;%d",link,module+1);
            TH2F *h_scan= (TH2F*)strun->Get(h_name);

            if(h_scan->GetMean()>0){ // raw data exists: call KwikFit to fit the data
              strun->Close();
              delete strun;

              KwikFit(run,(int)scan[(range*MAX_POINTS)+i],module,1,0,0);
	  
              sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan[(range*MAX_POINTS)+i]);
              TFile *strun = new TFile(f_name);

              sprintf(h_name,"h_code%d;%d",link,module+1);
              TH1S *h_code= (TH1S*)strun->Get(h_name);
              sprintf(h_name,"h_mean%d;%d",link,module+1);
              TH1F *h_mean= (TH1F*)strun->Get(h_name);
              sprintf(h_name,"h_sigma%d;%d",link,module+1);
              TH1F *h_sigma= (TH1F*)strun->Get(h_name);
            }
          }
	  
          if(h_mean){ // fitted data now exists
            if(h_mean->GetSum()>0){
              printf("Filling arrays for scan %d TrimRange %d TrimDAC %f module %d link %d\n",
                      scan[(range*MAX_POINTS)+i], TrimRange[range], TrimDAC[(range*MAX_POINTS)+i], module, link);
              module_status[module][range][link] = 1;

              for(chan=0;chan<768;chan++){
                code[module][range][link][chan][i]=h_code->GetBinContent(chan);
                mean[module][range][link][chan][i]=h_mean->GetBinContent(chan);
                sigma[module][range][link][chan][i]=h_sigma->GetBinContent(chan);

                mean_noise[module][link][chan]+=h_sigma->GetBinContent(chan);
                noise_cut[module][link][chan/128]+=h_sigma->GetBinContent(chan);
              }
            }else{
              module_status[module][range][link] = 0;
            } 
          }else{ // no data for this module
            module_status[module][range][link] = 0;
          }
        } // end for(link)  
      } // end for(module)
    
      strun->Close();

      delete strun;
    } // end for(point)
  } // end for(range)

  // Noise cuts added PWP 17.05.01

  int n_scans = 0;
  for(range=0;range<n_ranges;range++){
    for(point=0;point<n_points[range];point++){
      n_scans++;
    }
  }

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      for(chan=0;chan<768;chan++){
        mean_noise[module][link][chan]/=n_scans;
      }
      for(chip=0;chip<MAX_CHIPS;chip++){
        noise_cut[module][link][chip]*=1.25;
        noise_cut[module][link][chip]/=(128*n_scans);
      }
    }
  }     

  //
  // 3. Creation of various objects
  //

  TF1* f3[MAX_LINKS][MAX_CHANS];

  TH2F* h_trim[MAX_LINKS][MAX_CHIPS];
  TH1S* h_ntrim[MAX_LINKS][MAX_RANGES][MAX_CHIPS];
  TH2F* h_trms[MAX_LINKS][MAX_CHIPS];
  TH2F* h_slope[MAX_LINKS][MAX_CHIPS];
  TH2F* h_offset[MAX_LINKS][MAX_CHIPS];

  TH1F* h_tp[MAX_LINKS][MAX_CHIPS];
  TH1F* h_ts[MAX_LINKS][MAX_CHIPS];

  TH1F* h_ts_vch[MAX_LINKS];
  TH1F* h_tp_vch[MAX_LINKS];

  printf("Creating function objects\n");
  for(link=0;link<MAX_LINKS;link++){
    for(chan=0;chan<MAX_CHANS;chan++){
      sprintf(h_name,"f3_%d_%d",link,chan);
      f3[link][chan]= new TF1 (h_name,"pol1",start,stop);
      f3[link][chan]->SetLineColor(chan%32);
      f3[link][chan]->SetMarkerColor(chan%32);
    }
  }

  printf("Creating histogram objects\n");
  for(link=0;link<MAX_LINKS;link++){
    for(range=0;range<MAX_RANGES;range++){
      for(chip=0;chip<MAX_CHIPS;chip++){

        sprintf(h_name, "h_ntrim_%d_%d_%d",link,range,chip);
        sprintf(h_title,"n(trimmable) chip %d",chip);
        h_ntrim[link][range][chip] = new TH1S(h_name,h_title,n_targets,start,stop);
        h_ntrim[link][range][chip]->SetXTitle("Target Threshold (mV)");
        h_ntrim[link][range][chip]->SetYTitle("n(trimmable)");
        h_ntrim[link][range][chip]->SetFillColor(range+1);
        h_ntrim[link][range][chip]->SetLineColor(range+1);
        h_ntrim[link][range][chip]->SetMarkerColor(range+1);
      }
    }
  }


  for(link=0;link<MAX_LINKS;link++){
    for(chip=0;chip<MAX_CHIPS;chip++){

      sprintf(h_name, "h_trms_%d_%d",link,chip);
      sprintf(h_title,"RMS(trimmed vt50) chip %d",chip);
      h_trms[link][chip] = new TH2F(h_name,h_title,2,start,stop,2,0,10);
      h_trms[link][chip]->SetXTitle("Target Threshold (mV)");
      h_trms[link][chip]->SetYTitle("RMS(vt50)");

      sprintf(h_name, "h_slope_%d_%d",link,chip);
      sprintf(h_title,"Slope chip %d",chip);
      h_slope[link][chip] = new TH2F(h_name,h_title,2,(chip*128),((chip+1)*128),2,0,16);
      h_slope[link][chip]->SetXTitle("Channel Number");
      h_slope[link][chip]->SetYTitle("Slope");

      sprintf(h_name, "h_offset_%d_%d",link,chip);
      sprintf(h_title,"Offset chip %d",chip);
      h_offset[link][chip] = new TH2F(h_name,h_title,2,(chip*128),((chip+1)*128),2,-32,32);
      h_offset[link][chip]->SetXTitle("Channel Number");
      h_offset[link][chip]->SetYTitle("Offset");

      sprintf(h_name, "h_trim_%d_%d",link,chip);
      sprintf(h_title,"TrimDAC Curve chip %d",chip);
      h_trim[link][chip] = new TH2F(h_name,h_title,2,start,stop,2,-0.5,15.5);
      h_trim[link][chip]->SetXTitle("Threshold (mV)");
      h_trim[link][chip]->SetYTitle("TrimDAC setting (bits)");

      sprintf(h_name, "h_tp_%d_%d",link,chip);
      sprintf(h_title,"Threshold Distribution chip %d",chip);
      h_tp[link][chip] = new TH1F(h_name,h_title,10*((stop-start)/step),(start-10),(stop-10));
      h_tp[link][chip]->SetXTitle("Threshold (mV)");
      h_tp[link][chip]->SetYTitle("n(channels)");
      h_tp[link][chip]->SetLineColor(kRed);
      h_tp[link][chip]->SetMarkerColor(kRed);

      sprintf(h_name, "h_ts_%d_%d",link,chip);
      sprintf(h_title,"Trim Distribution chip %d",chip);
      h_ts[link][chip] = new TH1F(h_name,h_title,16,-0.50,15.5);
      h_ts[link][chip]->SetXTitle("Trim Setting");
      h_ts[link][chip]->SetYTitle("n(channels)");
      h_ts[link][chip]->SetLineColor(kRed);
      h_ts[link][chip]->SetMarkerColor(kRed);

    } // end for(chip)

    sprintf(h_name, "h_ts_vch_%d",link);
    sprintf(h_title,"Trim Setting link %d",link);
    h_ts_vch[link] = new TH1F(h_name,h_title,768,-0.50,767.5);
    h_ts_vch[link]->SetXTitle("Channel Number");
    h_ts_vch[link]->SetYTitle("Trim Setting");
    h_ts_vch[link]->SetLineColor(kRed);
    h_ts_vch[link]->SetMarkerColor(kRed);

    sprintf(h_name, "h_tp_vch_%d",link);
    sprintf(h_title,"Trimmed Threshold link %d",link);
    h_tp_vch[link] = new TH1F(h_name,h_title,768,-0.50,767.5);
    h_tp_vch[link]->SetXTitle("Channel Number");
    h_tp_vch[link]->SetYTitle("Trimmed Threshold (mV)");
    h_tp_vch[link]->SetLineColor(kRed);
    h_tp_vch[link]->SetMarkerColor(kRed);

  } // end for(link)

  //
  // Open the output canvas
  //

  Int_t dowait=0;
  Int_t ps_page=0;
  Int_t screen_page=0;

  TCanvas* c3 =0;
  TPostScript* ps =0;

  if(graphics!=0){
    dowait=1;
  }

  if(online){ 
    // Set Batch mode to avoid memory leaks in TWIN32Canvas
    // PWP 28.11.02
    printf("Running in BATCH mode - plots will not appear\n");
    gROOT->SetBatch(1);
  }

  if((graphics!=0)||(postscript!=0)){
    graphics = 1;
    c3= new TCanvas ("c3","TC",1100,750);
  }


  // Sort ranges in increasing order for plotting. LE 5/5 2001
  // Only needs doing once
  Short_t order;
  Short_t range_order[4];
  for(range=0;range<n_ranges;range++){
    for(order=0;order<n_ranges;order++){
      if(TrimRange[range]==order) range_order[order] = range;
    }
  }

  //
  // 4. Loop over modules
  //


  Float_t lx[MAX_TARGETS];
  Float_t ly[MAX_TARGETS];

  TGraph* TrimGraph[MAX_LINKS][MAX_RANGES][MAX_CHANS];
  TGraph* RMSGraph[MAX_LINKS][MAX_RANGES][MAX_CHIPS];
  TGraph* OffsetGraph[MAX_LINKS][MAX_RANGES][MAX_CHIPS];
  TGraph* SlopeGraph[MAX_LINKS][MAX_RANGES][MAX_CHIPS];

  for(module=0;module<MAX_MODULES;module++){

    // Clear pointers to graph objects

    for(link=0;link<MAX_LINKS;link++){
      for(range=0;range<MAX_RANGES;range++){
        for(chan=0;chan<MAX_CHANS;chan++){
          TrimGraph[link][range][chan] = (TGraph*) 0;
        }
      }
    }

    for(link=0;link<MAX_LINKS;link++){
      for(range=0;range<MAX_RANGES;range++){
        for(chip=0;chip<MAX_CHIPS;chip++){
          RMSGraph[link][range][chip] = (TGraph*) 0;
          OffsetGraph[link][range][chip] = (TGraph*) 0;
          SlopeGraph[link][range][chip] = (TGraph*) 0;
        }
      }
    }


    // Set pass status back to fail
    // and problem status back to none
    for(range=0;range<=MAX_RANGES;range++){ //BJG bugfix 05.06.02
      pass[range]=0;
      problem[range]=0;
    }

    // Open the postscript file if canvas is open
    // and valid data exists for the module

    if((module_status[module][0][0]>0)||(module_status[module][0][0]>0)){
      if(c3){
        if(postscript){
          ps_page=0;
          
	  sprintf(f_name,"%s%s_TrimRangePlot_%d_%06d.ps",PS_DIRECTORY,mname[module],date,the_time);
          ps = new TPostScript(f_name,112);

          if(ps){
	    printf("File %s opened for PS output\n",f_name);
            ps->Off();
          } else {
	    printf("ERROR: failed to open PS file %s\n",f_name);
          }
        }
      }
    }

    for(link=0;link<MAX_LINKS;link++){
      for(range=0;range<n_ranges;range++){
        if(module_status[module][range][link]>0){

          // Reset the arrays and objects used for storage

          for(chip=0;chip<MAX_CHIPS;chip++){
            h_ntrim[link][range][chip]->Reset();
            h_tp[link][chip]->Reset();
          }

          printf("Calculating TrimDAC Response Curves for module %d link %d TrimRange %d\n",module,link,TrimRange[range]);

          Int_t lp = 0;
          for(chan=0;chan<MAX_CHANS;chan++){

            // Exclude bad vt50 points (fit errors) from the graph 
	  

            lp = 0;
            for(i=0;i<n_points[range];i++){
              if((code[module][range][link][chan][i]&BAD_TEST)==0){
                lx[lp]=mean[module][range][link][chan][i];
                ly[lp]=TrimDAC[(range*MAX_POINTS)+i];
                lp++;
              }
            }

            if(lp>1){
              sprintf(h_name,"f3_%d_%d",link,chan);
              TrimGraph[link][range][chan] = new TGraph(lp,lx,ly);
              TrimGraph[link][range][chan]->Fit(h_name,"NRQ");
              TrimGraph[link][range][chan]->SetLineColor(chan%32);
              TrimGraph[link][range][chan]->SetMarkerColor(chan%32);

              p0[link][range][chan]=f3[link][chan]->GetParameter(0);
              p1[link][range][chan]=f3[link][chan]->GetParameter(1);
            } else {
              TrimGraph[link][range][chan]=0;

              p0[link][range][chan]=0;
              p1[link][range][chan]=0;
            }
          } // end for(chan)


          //
          // Solve the function for each target value:
          // can the channel can be trimmed to that value?
          //

          printf("Studying targets for module %d link %d TrimRange %d\n",module,link,TrimRange[range]);

          for(i=0;i<n_targets;i++){

            for(chip=0;chip<MAX_CHIPS;chip++){
              h_tp[link][chip]->Reset();
              h_tp[link][chip]->SetBins(40,target[i]-(10*step),target[i]+(10*step));
            }

            for(chan=0;chan<MAX_CHANS;chan++){
              chip=chan/128;

              Int_t i_val;
              f_val = 0;
  
              // If trim data is valid, slope will be +ve
              if(p1[link][range][chan]>0){

                f_val = (p1[link][range][chan]*target[i]) + p0[link][range][chan];
                i_val =(int)((f_val+0.5)/1.0);

                if((i_val>=0)&&(i_val<=15)){ // trimmable

                  // calculate the best trim from the fit
  
                  f_val = ((float)i_val-p0[link][range][chan]) / p1[link][range][chan];

                  h_tp[link][chip]->Fill(f_val);
                  h_ntrim[link][range][chip]->Fill(target[i]);
                }
              }

            } // end for(chan)

            //
            // Evaluate the RMS of the distribution of trimmed points for this target
            //

            for(chip=0;chip<MAX_CHIPS;chip++){

              trim_rms[link][range][chip][i] = h_tp[link][chip]->GetRMS();
            } // end for(chip)
          } // end for(target)



	  //
	  // Create the RMS, Offset and Slope Graphs
	  //

          for(chip=0;chip<MAX_CHIPS;chip++){

            mean_offset[link][range][chip]=0;
            rms_offset[link][range][chip]=0;
            mean_step[link][range][chip]=0;
            rms_step[link][range][chip]=0;

            lp = 0;
            for(i=0;i<n_targets;i++){
              if(trim_rms[link][range][chip][i]>0){
                lx[lp]=target[i];
                ly[lp]=trim_rms[link][range][chip][i];
                lp++;
              }
            }

            if(lp>0){
              RMSGraph[link][range][chip] = new TGraph(lp,lx,ly);
              RMSGraph[link][range][chip]->SetLineColor(range+1);
              RMSGraph[link][range][chip]->SetMarkerColor(range+1);
            }

            lp = 0;
            for(chan=(chip*128);chan<((chip+1)*128);chan++){
 //             if(p0[link][range][chan]!=0){  // revised PWP 24.08.01
              if(p1[link][range][chan]!=0){
                lx[lp]=chan;
                ly[lp]=p0[link][range][chan];
                mean_offset[link][range][chip]+=ly[lp];
                lp++;
              }
            }

            if(lp>0){
              mean_offset[link][range][chip]/=lp;
              OffsetGraph[link][range][chip] = new TGraph(lp,lx,ly);
              OffsetGraph[link][range][chip]->SetLineColor(range+1);
              OffsetGraph[link][range][chip]->SetMarkerColor(range+1);

              // calculate rms of offset - PWP 24.08.01
              lp = 0;
              for(chan=(chip*128);chan<((chip+1)*128);chan++){
                if(p1[link][range][chan]!=0){
                  rms_offset[link][range][chip]+=(float)TMath::Power((ly[lp]-mean_offset[link][range][chip]),2);
                  lp++;
                }
              }
              rms_offset[link][range][chip]=(float)TMath::Sqrt(rms_offset[link][range][chip]/lp);
            }

            lp = 0;
            for(chan=(chip*128);chan<((chip+1)*128);chan++){
              if(p1[link][range][chan]!=0){
                lx[lp]=chan;
                ly[lp]=1/p1[link][range][chan];
                mean_step[link][range][chip]+=ly[lp];
                lp++;
              }
            }

            if(lp>0){
              mean_step[link][range][chip]/=lp;
              SlopeGraph[link][range][chip] = new TGraph(lp,lx,ly);
              SlopeGraph[link][range][chip]->SetLineColor(range+1);
              SlopeGraph[link][range][chip]->SetMarkerColor(range+1);

              // calculate rms of step size - PWP 24.08.01
              lp = 0;
              for(chan=(chip*128);chan<((chip+1)*128);chan++){
                if(p1[link][range][chan]!=0){
                  rms_step[link][range][chip]+=(float)TMath::Power((ly[lp]-mean_step[link][range][chip]),2);
                  lp++;
                }
              }
              rms_step[link][range][chip]=(float)TMath::Sqrt(rms_step[link][range][chip]/lp);
            }

          } // end for(chip)
        } // end if(module_status)
      } // end for(range)



      // 
      // Plot the Trim, RMS, Offset and Slope Graphs
      //

      if(module_status[module][0][link]>0){

        if(c3){

          if((screen_page>0)&&(dowait)){
            printf("Enter a string to continue\n");
            scanf("%s",line);
          }

          printf("Generating TrimDAC Characteristic Plots for module %d link %d\n",module,link);

          ps_page++;
          screen_page++;
          if(ps){
	    ps->On();
            ps->NewPage();
          }

          c3->cd();
          c3->Clear();

          TPad *pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00,33);
          TPad *pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90,33);

          pad_text->SetBit(kCanDelete);
          pad_plot->SetBit(kCanDelete);

          pad_text->Draw();
          pad_plot->Draw();
          pad_plot->Divide(6,4,0,0);

          c3->Update();
          pad_text->cd();

          TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
          t1->SetBit(kCanDelete);

          t1->SetFillColor(10);
          t1->SetTextAlign(13);
          t1->SetTextSize(0.3);
          t1->SetTextColor(4);

          // ATLAS SCT Module Test:  TrimDAC Characteristics  -  time  -  place
          sprintf(line,"ATLAS SCT Module Test:  TrimDAC Characteristics");
          if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
          if(place) sprintf(line,"%s  -  %s",line,place->GetName());
          t1->AddText(line);

          // Page %d   Run %d   Start Scan %d  Stream %d  Module %d  (%s) - Type %s
          sprintf(line,"Page %d   Run %d   Start Scan %d  Stream %d  Module %d (%s) ", 
                  ps_page,run, scan[0],link,module, mname[module]);
          if(strlen(mdut[module]) > 0) 
            sprintf(line, "%s - Type %s", line, mdut[module]);
          t1->AddText(line);
          t1->Draw();

          for(range=0;range<n_ranges;range++){
            for(chip=0;chip<6;chip++){
              pad_plot->cd((6*range)+chip+1);
              h_trim[link][chip]->Draw();
              for(chan=(chip*128);chan<((chip+1)*128);chan++){
		if(TrimGraph[link][range_order[range]][chan]!=0){
                  TrimGraph[link][range_order[range]][chan]->Draw("LP");
                }
              }
            }
          }

          c3->Update();

  
          if((screen_page>0)&&(dowait)){
            printf("Enter a string to continue\n");
            scanf("%s",line);
          }
          ps_page++;
          screen_page++;
          if(ps){
            ps->NewPage();
          }

          c3->cd();
          c3->Clear();

          pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00,33);
          pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90,33);

          pad_text->SetBit(kCanDelete);
          pad_plot->SetBit(kCanDelete);

          pad_text->Draw();
          pad_plot->Draw();
          pad_plot->Divide(6,4,0,0);

          c3->Update();
          pad_text->cd();

          t1 = new TPaveText(0.0,0.0,1.0,1.0);
          t1->SetBit(kCanDelete);

          t1->SetFillColor(10);
          t1->SetTextAlign(13);
          t1->SetTextSize(0.3);
          t1->SetTextColor(4);

          // ATLAS SCT Module Test:  TrimDAC Characteristics  -  time  -  place
          sprintf(line,"ATLAS SCT Module Test:  TrimDAC Characteristics");
          if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
          if(place) sprintf(line,"%s  -  %s",line,place->GetName());
          t1->AddText(line);

          // Page %d   Run %d   Start Scan %d  Stream %d  Module %d  (%s) - Type %s
          sprintf(line,"Page %d   Run %d   Start Scan %d  Stream %d  Module %d (%s) ", 
                  ps_page,run, scan[0],link,module, mname[module]);
          if(strlen(mdut[module]) > 0) 
            sprintf(line, "%s - Type %s", line, mdut[module]);
          t1->AddText(line);
          t1->Draw();

          for(chip=0;chip<6;chip++){
            pad_plot->cd(1+chip);
            h_ntrim[link][range_order[n_ranges-1]][chip]->SetMaximum(140);
            h_ntrim[link][range_order[n_ranges-1]][chip]->Draw();
            for(range=n_ranges-2;range>=0;range--){
              h_ntrim[link][range_order[range]][chip]->Draw("same");
            }
          }

          for(chip=0;chip<6;chip++){
            pad_plot->cd(7+chip);
            h_trms[link][chip]->Draw();
            for(range=0;range<n_ranges;range++){
              if(RMSGraph[link][range_order[range]][chip]) RMSGraph[link][range][chip]->Draw("LP");
            }
          }

          for(chip=0;chip<6;chip++){
            pad_plot->cd(13+chip);
            h_offset[link][chip]->Draw();
            for(range=0;range<n_ranges;range++){
              if(OffsetGraph[link][range_order[range]][chip]) OffsetGraph[link][range][chip]->Draw("LP");
            }
          }

          for(chip=0;chip<6;chip++){
            pad_plot->cd(19+chip);
            h_slope[link][chip]->Draw();
            for(range=0;range<n_ranges;range++){
              if(SlopeGraph[link][range_order[range]][chip]) SlopeGraph[link][range][chip]->Draw("LP");
            }
	    pad_plot->Update();
          }

          c3->Update();

        }// end if(c3)
      } //end if(module_status)
    } // end for(link)

    if(ps) ps->Off();

    //
    // 5. Pick the best target for each chip on each range
    //

    if((module_status[module][0][0]>0)||(module_status[module][0][1]>0)){

      // Initialisation

      for(link=0;link<MAX_LINKS;link++){
        for(range=0;range<=n_ranges;range++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            best_ntrim[link][range][chip]=0;
            best_target[link][range][chip]=0;
          }
        }
        for(range=0;range<n_ranges;range++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            lo_target[link][range][chip]=0;
            hi_target[link][range][chip]=0;
          } // end for(chip)
        } // end for(range)
        for(chip=0;chip<MAX_CHIPS;chip++){
          best_range[link][chip]=0;
        } // end for(chip)
      } // end for(link)


      // Determination of best trim range for each chip

      Float_t bin;
      Float_t content;
      for(link=0;link<MAX_LINKS;link++){
        for(range=0;range<n_ranges;range++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            for(i=0;i<n_targets;i++){
              bin=h_ntrim[link][range][chip]->FindBin(target[i]);
              content=h_ntrim[link][range][chip]->GetBinContent(bin);

              if(content>best_ntrim[link][range][chip]){
                best_ntrim[link][range][chip]=content;
                lo_target[link][range][chip]=i;
              }
              if(content==best_ntrim[link][range][chip]){
                hi_target[link][range][chip]=i;
              }
            } // end for(target)

            best_target[link][range][chip] = ( lo_target[link][range][chip]
                                             + hi_target[link][range][chip])/2;
            printf("Md%d Lk%d TR%d Cp%d lo_tar %3.2f hi_tar %3.2f -> best_tar %3.2f (%3d ch)\n",
                    module,link,range,chip,
                    target[lo_target[link][range][chip]],
                    target[hi_target[link][range][chip]], 
                    target[best_target[link][range][chip]],
                    best_ntrim[link][range][chip]);
          } // end for(chip)
        } // end for(range)
      } // end for(link)
	  

      // Select the best TrimRange, choosing the lowest one.

      for(link=0;link<MAX_LINKS;link++){
        for(range=0;range<n_ranges;range++){
          for(chip=0;chip<MAX_CHIPS;chip++){
	    if(best_ntrim[link][range_order[range]][chip]>best_ntrim[link][n_ranges][chip]){
              best_ntrim[link][n_ranges][chip] = best_ntrim[link][range_order[range]][chip];
              best_target[link][n_ranges][chip] = best_target[link][range_order[range]][chip];
              best_range[link][chip]=range_order[range];
            }
          } // end for(chip)
        } // end for(range)
      } // end for(link)

      for(link=0;link<MAX_LINKS;link++){
        for(chip=0;chip<MAX_CHIPS;chip++){
          printf("Md%d Lk%d Cp%d best TrimRange %d best target %3.2f (%3d ch)\n",
                  module,link,chip,
                  best_range[link][chip],
                  target[best_target[link][(best_range[link][chip])][chip]],
                  best_ntrim[link][(best_range[link][chip])][chip]);
        } // end for(chip)
      } // end for(link)


      if(method==0){ // all chips must have the same target

        // Consider the case where all chips are fixed to the same range

        Int_t low;
        Int_t high;
        Int_t best_yet;
        Int_t new_sum;   
        Int_t best_this_chip;

        for(range=0;range<n_ranges;range++){
          low=0;
          high=0;
          best_yet=0;

          for(i=0;i<n_targets;i++){

            new_sum=0;   
            for(link=0;link<MAX_LINKS;link++){
              for(chip=0;chip<MAX_CHIPS;chip++){
                bin=h_ntrim[link][range][chip]->FindBin(target[i]);
                new_sum+=h_ntrim[link][range][chip]->GetBinContent(bin);
              }
            }

            if(new_sum>best_yet){
              best_yet=new_sum;
              low = i;
            }
            if(new_sum==best_yet){
              high = i;
            }
          } // end for(target)

          i = (low+high)/2;
          printf("Md%d TR%d lo_tar %3.2f hi_tar %3.2f -> best_tar %3.2f (%3d ch)\n",
                  module,range,target[low],target[high],target[i],best_yet);

          for(link=0;link<MAX_LINKS;link++){
            for(chip=0;chip<MAX_CHIPS;chip++){
              bin=h_ntrim[link][range][chip]->FindBin(target[i]);
              best_ntrim[link][range][chip]=h_ntrim[link][range][chip]->GetBinContent(bin);
              best_target[link][range][chip]=i;
            } // end for(chip)
          } // end for(link)
        } // end for(range)

        // Now allow the range to vary from chip to chip

        low=0;
        high=0;
        best_yet=0;
        Float_t old_mean_range = 100;
        for(i=0;i<n_targets;i++){

          new_sum=0;
          Float_t mean_range=0;
          for(link=0;link<MAX_LINKS;link++){
            for(chip=0;chip<MAX_CHIPS;chip++){

              best_this_chip=0;
              best_range[link][chip]=0;
              for(range=0;range<n_ranges;range++){
                bin=h_ntrim[link][range_order[range]][chip]->FindBin(target[i]);
                content=h_ntrim[link][range_order[range]][chip]->GetBinContent(bin);
                if(content>best_this_chip){
                  best_this_chip = content;
                  best_range[link][chip]=range_order[range];
                }
              }
              new_sum+=best_this_chip;
              mean_range+= (float) (best_range[link][chip] * best_range[link][chip]);
            }
          } // end for(link)

          if(new_sum>best_yet){
            old_mean_range=mean_range;
            best_yet=new_sum;
            low = i;
          }
          if(new_sum==best_yet){
            if(mean_range==old_mean_range){
              high = i;
            }
            if(mean_range<old_mean_range){
              low = i;
              high = i;
              old_mean_range=mean_range;
            }
          }

       //   printf("target%d low%d high%d new_sum %d best yet %d mean_range%3.2f old_mean %3.2f\n",
       //             i,low,high,new_sum,best_yet,mean_range,old_mean_range);
        } // end for(target)

        i = (low+high)/2;
        printf("Md%d TR%d lo_tar %3.2f hi_tar %3.2f -> best_tar %3.2f (%3d ch)\n",
                module,-1,target[low],target[high],target[i],best_yet);

        // recalculate best ranges

        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            best_this_chip=0;
            for(range=0;range<n_ranges;range++){
              bin=h_ntrim[link][range_order[range]][chip]->FindBin(target[i]);
              content=h_ntrim[link][range_order[range]][chip]->GetBinContent(bin);
              if(content>best_this_chip){
                best_this_chip = content;
                best_range[link][chip]=range_order[range];
              }
            } // end for(range)
          printf("link %d chip%d best range %d\n",link,chip,best_range[link][chip]);
          } // end for(chip)
        } // end for(link)

        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            bin=h_ntrim[link][(best_range[link][chip])][chip]->FindBin(target[i]);
            best_ntrim[link][n_ranges][chip]=h_ntrim[link][(best_range[link][chip])][chip]->GetBinContent(bin);
            best_target[link][n_ranges][chip]=i;
          }
        } // end for(link)
      } // end if(method)


      // Trim once for each range, then once with optimised ranges
      // (if not the same as one of the existing trims....)

      for(range=0;range<=n_ranges;range++){

        // More initialisations...
        
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            h_tp[link][chip]->Reset();
            h_tp[link][chip]->SetBins(40,target[(best_target[link][range][chip])]-(10*step),
                                         target[(best_target[link][range][chip])]+(10*step));
            h_ts[link][chip]->Reset();
          }
          h_tp_vch[link]->Reset();
          h_ts_vch[link]->Reset();
        }
	  
        // Actually do the trimming

        for(link=0;link<MAX_LINKS;link++){
          for(chan=0;chan<MAX_CHANS;chan++){
            chip=chan/128;
            Int_t i_val = -1;
            f_val = -1;
            Float_t the_target = target[(best_target[link][range][chip])]; // was integer - PWP bugfix 31.10.02

            Int_t range2;
            if(range==n_ranges){
              range2 = best_range[link][chip];
            }else{
              range2 = range;
            }
 
            // If trim data is valid, slope will be +ve
            if(p1[link][range2][chan]>0){

              f_val = (p1[link][range2][chan]*the_target) + p0[link][range2][chan];
              i_val =(int)((f_val+0.5)/1.0);
	      
              if((i_val>=0)&&(i_val<=15)){ // trimmable

              // if vt50 data is available for all 16 trim scans, 
              // use the data - otherwise calculate from the fit

                if(n_points[range]==16){
                  f_val = mean[module][range2][link][chan][i_val];
                }else{
                  f_val = ((float)i_val-p0[link][range2][chan]) / p1[link][range2][chan];
                }
              }else{
                f_val = -1;
                i_val = -1;
              }
            }


            trim_setting[link][range][chan] = i_val;
            vt50_setting[link][chan] = f_val;
	    
            if((i_val>=0)&&(i_val<=15)){ // fill trim and vt50 histograms
                                       // only if channel is trimmable
              h_ts[link][chip]->Fill(i_val);
              h_tp[link][chip]->Fill(f_val);
              h_ts_vch[link]->Fill(chan,i_val);
              h_tp_vch[link]->Fill(chan,f_val);
            }

          } // end for(chan)
        } //end for(link)

        // Fit gaussians to the distributions of trim values and trimmed vt50 

        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            h_ts[link][chip]->Fit("gaus","Q");
            h_tp[link][chip]->Fit("gaus","Q");
            trimmed_mean[link][range][chip]=h_tp[link][chip]->GetMean();
            trimmed_rms[link][range][chip]=h_tp[link][chip]->GetRMS();
          }
        }


        // Plot the Results if the canvas has been opened

        if(c3){

          if((screen_page>0)&&(dowait)){
            printf("Enter a string to continue\n");
            scanf("%s",line);
          }

          printf("Generating Summary Plots for module %d TrimRange %d\n",module,range);

          ps_page++;
          screen_page++;
          if(ps){
	    ps->On();
            ps->NewPage();
          }

          c3->cd();
          c3->Clear();

          TPad *pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00,33);
          TPad *pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,0.60,0.90,33);
          TPad *pad_proj   = new TPad("pad_proj", "Projections", 0.60,0.00,1.00,0.90,33);

          pad_text->Draw();
          pad_plot->Draw();
          pad_plot->Divide(1,4,0,0);
          pad_proj->Draw();
          pad_proj->Divide(3,8,0,0);
 
          c3->Update();
          pad_text->cd();

          TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
          t1->SetBit(kCanDelete);
          t1->SetFillColor(10);
          t1->SetTextAlign(13);
          t1->SetTextSize(0.3);
          t1->SetTextColor(4);

          // ATLAS SCT Module Test:  TrimDAC Response TrimRange %d  -  time  -  place
          sprintf(line,"ATLAS SCT Module Test:  TrimDac Response TrimRange %d",TrimRange[range]);
          if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
          if(place) sprintf(line,"%s  -  %s",line,place->GetName());
          t1->AddText(line);

          // Page %d   Run %d   Start Scan %d  Module %d  (%s) - Type %s
          sprintf(line,"Page %d   Run %d   Start Scan %d  Module %d (%s) ", 
                  ps_page,run, scan[0],module, mname[module]);
          if(strlen(mdut[module]) > 0) 
            sprintf(line, "%s - Type %s", line, mdut[module]);
          t1->AddText(line);
          t1->Draw();

          pad_plot->cd(1);
          sprintf(h_name,"Trim Settings module %d link %d TrimRange %d",module,0,TrimRange[range]);
          h_ts_vch[0]->SetTitle(h_name);
          h_ts_vch[0]->Draw("P");

          pad_plot->cd(2);
          sprintf(h_name,"Trim Settings module %d link %d TrimRange %d",module,1,TrimRange[range]);
          h_ts_vch[1]->SetTitle(h_name);
          h_ts_vch[1]->Draw("P");

          pad_plot->cd(3);
          sprintf(h_name,"Trimmed Threshold module %d link %d TrimRange %d",module,0,TrimRange[range]);
          h_tp_vch[0]->SetTitle(h_name);
          h_tp_vch[0]->Draw("P");

          pad_plot->cd(4);
          sprintf(h_name,"Trimmed Threshold module %d link %d TrimRange %d",module,1,TrimRange[range]);
          h_tp_vch[1]->SetTitle(h_name);
          h_tp_vch[1]->Draw("P");

          for(chip=0;chip<MAX_CHIPS;chip++){
            pad_proj->cd(1+chip);
            h_ts[0][chip]->Draw();
 
            pad_proj->cd(7+chip);
            h_ts[1][chip]->Draw();

            pad_proj->cd(13+chip);
            h_tp[0][chip]->Draw();

            pad_proj->cd(19+chip);
            h_tp[1][chip]->Draw();
          }

          c3->Update();


          if(ps) ps->Off();

        } // end if(c3)

	
        //
        // 7. Generate trim and mask files and decide PASS/FAIL status
        //

        sprintf(f_name,"%s%s_tr%d_%d.trim",RESULTS_DIRECTORY,mname[module],TrimRange[range],date);
        FILE* trimfile = fopen(f_name,"w");

        if(trimfile==NULL){
          printf("Could not open trim file %s\n",f_name);
          return;
        } else {
          printf("File %s opened \n",f_name);
        }

        sprintf(f_name,"%s%s_tr%d_%d.mask",RESULTS_DIRECTORY,mname[module],TrimRange[range],date);
        FILE* maskfile = fopen(f_name,"w");

        if(maskfile==NULL){
          printf("Could not open mask file %s\n",f_name);
          return;
        } else {
          printf("File %s opened \n",f_name);
        }

        // write TrimRange settings into the trim file

        for(link=0; link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            if(range<n_ranges){
              fprintf(trimfile,"Chip %d %d\n",(link*6)+chip,TrimRange[range]);
            }else{
              fprintf(trimfile,"Chip %d %d\n",(link*6)+chip,best_range[link][chip]);
            }
          }
        }

        // Count untrimmable channels (NOT dead, stuck or noisy channels)
        // as defects.  If n_defects >0 then FAIL that portion of the TrimRangeScan

        Int_t n_defects=0;
        for(link=0; link<MAX_LINKS;link++){
          for(chan=0;chan<MAX_CHANS;chan++){

            fprintf(trimfile,"%f\n",(trim_setting[link][range][chan]/15.0));

            // Mask channels which cannot be trimmed including
            // channels which are DEAD or STUCK PWP 22.01.01

            if(range<n_ranges){
              if(TrimGraph[link][range][chan]==0){
                fprintf(maskfile,"%d  #DEAD or STUCK\n",(link*768)+chan);
              }
              else if(mean_noise[module][link][chan]>noise_cut[module][link][chan/128]){
                fprintf(maskfile,"%d  #NOISY\n",(link*768)+chan);
              }
              else if((trim_setting[link][range][chan]<0.0)||(trim_setting[link][range][chan]>15.0)){
                fprintf(maskfile,"%d  #UNTRIMMABLE\n",(link*768)+chan);
                n_defects++;
              }
            }else{
              if(TrimGraph[link][best_range[link][chan/128]][chan]==0){
                fprintf(maskfile,"%d  #DEAD or STUCK\n",(link*768)+chan);
              }
              else if(mean_noise[module][link][chan]>noise_cut[module][link][chan/128]){
                fprintf(maskfile,"%d  #NOISY\n",(link*768)+chan);
              }
              else if((trim_setting[link][range][chan]<0.0)||(trim_setting[link][range][chan]>15.0)){
                fprintf(maskfile,"%d  #UNTRIMMABLE\n",(link*768)+chan);
                n_defects++;
              }
            }
          }
        }

        fclose(maskfile);
        fclose(trimfile);

        // Section revised for new cuts PWP 26.11.02
        Float_t lo_cut, hi_cut;
        Int_t the_range;

        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){

            if(range<n_ranges){  // TrimRange 0 to 3
              bad_range[link][range][chip]=0;
              the_range = TrimRange[range];

              if((new_cuts)&&(the_range>0)){  // use new cut based on step size for TrimRange 0
                lo_cut = (0.60+(float)the_range)*mean_step[link][range_order[0]][chip];
                hi_cut = (1.40+(float)the_range)*mean_step[link][range_order[0]][chip];
              }else{                          // use old cut
                lo_cut = step_cut[the_range];
                hi_cut = step_cut[the_range+1];
              }

              if((mean_step[link][range][chip]<lo_cut)||
                 (mean_step[link][range][chip]>hi_cut)){
                bad_range[link][range][chip]=1;
                n_defects++;
              }

            }else{               // TrimRange -1
              if(bad_range[link][best_range[link][chip]][chip]){
                 n_defects++;
              }
            }

          } // end for(chip)
        } // end for(link)

        // end PWP revisions 26.11.02

        if(n_defects==0){
          pass[range]=1;
        }

      } // end for(range)

      if(ps){
	ps->On();	
	ps->Close();
        delete ps;
      }


      //
      // 8.1 Write data to new style (database) results file 
      //

        for(range=0;range<=n_ranges;range++){

          if(range<n_ranges){
            the_range = range;
          }else{
            the_range = 0;
          }

          sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan[(the_range*MAX_POINTS)]);
          TFile *strun = new TFile(f_name);

          if(strun == NULL) {
            printf("File %s failed for reading data\n",f_name);
            printf("     for TrimRange test\n\n");
          }


          printf("OpenResults(%s, %p, %d, %d, %d, %d, %d)\n", "Trim", 
                 strun, module, run, scan[(the_range*MAX_POINTS)], pass[range], problem[range]); 
          FILE* outfile = OpenResults("Trim", 
                 strun, module, run, scan[(the_range*MAX_POINTS)], pass[range], problem[range]); 

          strun->Close();
          delete strun;

          if(outfile==NULL){
            printf("Could not open result file\n");
            return;
          }

          // List points used for trim on individual range
          if(range<n_ranges){
            ListScanInfo(outfile,"TrimDAC (bits)",n_points[range],&TrimDAC[(range*MAX_POINTS)]);
          }else{
            fprintf(outfile,"#\n");
          }

          // test specific information

          fprintf(outfile,"%%Trim\n");
          fprintf(outfile,"#\n");

          fprintf(outfile,"#TRIM\tCHARGE\tTYPE\tALGORITHM\n");
          fprintf(outfile,"\t%3.1f \t%d \t%d\n",charge,TrimRange[range],method);
          if(method==1){
            fprintf(outfile,"#(trim chip by chip)\n");
          }else{
            fprintf(outfile,"#(trim whole module)\n");
          }

          fprintf(outfile,"#\n");
          fprintf(outfile,"#\trange\ttarget\tntrim\tvt50\tvt50rms\ttr_off\toff_rms\ttr_step\tstep_rms\n");
          for(link=0;link<MAX_LINKS;link++){
            for(chip=0;chip<MAX_CHIPS;chip++){
              fprintf(outfile,"#%s\n",MseLabel(link,chip));
              if(range<n_ranges){
                fprintf(outfile,"\t%d\t%3.1f\t%d\t%5.1f\t%5.2f\t%4.1f\t%4.1f\t%5.2f\t%5.2f\n",
			TrimRange[range],
			target[(best_target[link][range][chip])],
			best_ntrim[link][range][chip],
			trimmed_mean[link][range][chip],
			trimmed_rms[link][range][chip],
			mean_offset[link][range][chip],
			rms_offset[link][range][chip],
			mean_step[link][range][chip],
			rms_step[link][range][chip]);

              }else{
                fprintf(outfile,"\t%d\t%3.1f\t%d\t%5.1f\t%5.2f\t%4.1f\t%4.1f\t%5.2f\t%5.2f\n",
			TrimRange[best_range[link][chip]],
			target[(best_target[link][range][chip])],
			best_ntrim[link][range][chip],
			trimmed_mean[link][range][chip],
			trimmed_rms[link][range][chip],
			mean_offset[link][(best_range[link][chip])][chip],
			rms_offset[link][(best_range[link][chip])][chip],
			mean_step[link][(best_range[link][chip])][chip],
			rms_step[link][(best_range[link][chip])][chip]);

	      }
            } // end for(chip)
          } // end for(link)

          fprintf(outfile,"#\n");

          Float_t the_step;
          Float_t the_offset;

          Int_t n_defects=0;
          for(link=0;link<MAX_LINKS;link++){
            for(chip=0;chip<MAX_CHIPS;chip++){
              if(range<n_ranges){
                the_range = TrimRange[range];
              }else{
                the_range = TrimRange[best_range[link][chip]];
              }

              if(bad_range[link][the_range][chip]){
                ListDefect(outfile,"TR_RANGE",(((link*6)+chip)*128),(((link*6)+chip+1)*128)-1);
                n_defects++;
              }
            }
          }

          Int_t first, last;
          Int_t old_defect, new_defect;
          Float_t s_cut_lo, s_cut_hi;
          Float_t o_cut_lo, o_cut_hi;

          for(link=0;link<MAX_LINKS;link++){
            first=-1;
            last=-1;
            old_defect=0;

            for(chip=0;chip<MAX_CHIPS;chip++){
              if(range<n_ranges){
                the_range = TrimRange[range];
              }else{
                the_range = TrimRange[best_range[link][chip]];
              }

              // Section revised to permit use of new cuts PWP 26.11.02

              if(new_cuts){
                s_cut_lo = 0.60 * mean_step[link][the_range][chip];
                s_cut_hi = 1.40 * mean_step[link][the_range][chip];
                o_cut_lo = mean_offset[link][the_range][chip] - (4*rms_offset[link][the_range][chip]); 
                o_cut_hi = mean_offset[link][the_range][chip] + (4*rms_offset[link][the_range][chip]); 
              }else{
                s_cut_lo = mean_step[link][the_range][chip] - (4*rms_step[link][the_range][chip]); 
                s_cut_hi = mean_step[link][the_range][chip] + (4*rms_step[link][the_range][chip]);
                o_cut_lo = mean_offset[link][the_range][chip] - (4*rms_offset[link][the_range][chip]); 
                o_cut_hi = mean_offset[link][the_range][chip] + (4*rms_offset[link][the_range][chip]); 
              }

              for(chan=(chip*128);chan<((chip+1)*128);chan++){
                new_defect = 0;

                if(p1[link][the_range][chan]!=0){
                  the_step   =1/p1[link][the_range][chan];
                  the_offset =  p0[link][the_range][chan];

                  // Is there a defect on this strip?

                  if((trim_setting[link][range][chan]<0.0)||
                     (trim_setting[link][range][chan]>15.0)){
                    new_defect = 1; // TR_NOTRIM
                  }else if((the_step<s_cut_lo)||(the_step>s_cut_hi)){
                    new_defect = 2; // TR_STEP
                  }else if((the_offset<o_cut_lo)||(the_offset>o_cut_hi)){
                    new_defect = 3; // TR_OFFSET
                  }
                }

                // Is this a continued defect?
                if((old_defect)&&(new_defect==old_defect)){
                  last = ((link*MAX_CHANS)+chan);
                }

                // Is this a new defect?
                if((new_defect)&&(old_defect==0)){
                  old_defect = new_defect;
                  first = ((link*MAX_CHANS)+chan);
                  last = first;
                }

                // If a defect is associated with the previous strip and a defect
                // of a different type, or no defect at all, is associated with 
                // this strip then we must record the previous strip's defect

                if((old_defect)&&(new_defect!=old_defect)){
                  switch(old_defect){
                    case 1:
                      ListDefect(outfile,"TR_NOTRIM",first,last);
                      break;
                    case 2:
                      ListDefect(outfile,"TR_STEP",first,last);
                      break;
                    case 3:
                      ListDefect(outfile,"TR_OFFSET",first,last);
                      break;
                    default:
                      break;
                  }
                  n_defects++;
                  if(new_defect==0){ // no defect on this strip
                    old_defect = 0;
                    first = -1;
                    last = -1;
                  }else{ // this strip has a defect of a different type
                    old_defect = new_defect;
                    first = ((link*MAX_CHANS)+chan);
                    last = first;
                  }
                }

              } // end chan
            } // end chip
          } // end link

          if(n_defects==0){
            fprintf(outfile,"#No defects found!\n");
          }else{
            fprintf(outfile,"#%d defects found\n",n_defects);
          }

        // And finally we add the link to the raw data file

          fprintf(outfile,"#\n");
          sprintf(line,"%s%s_tr%d_%d.trim",RESULTS_DIRECTORY,mname[module],TrimRange[range],date);
          ListRawData(outfile,line);

        // And a URL to the postscript file

  #ifdef WEB_KEY        
        if(postscript){
	  sprintf(f_name,"%s%s_TrimRangePlot_%d_%06d.ps",WEB_KEY,mname[module],date,the_time);
          ListWebLink(outfile,"Plots (postscript)",f_name);
        }
  #endif
        fclose(outfile);

        } // end for(range)
 
      //
      // 8.2 Set the trim values and masked channels if running online
      //

      if(online){
        printf("Setting the trim values and masked channels for module %d\n",module);
	
        for(link=0;link<MAX_LINKS;link++){
          for(chan=0;chan<MAX_CHANS;chan++){
            SetTrim(module,(link*768)+chan,trim_setting[link][n_ranges][chan]);
	    
            if((trim_setting[link][n_ranges][chan]<0.0)||(trim_setting[link][n_ranges][chan]>15.0)||(TrimGraph[link][best_range[link][chan/128]][chan]==0)
	                                     ||(mean_noise[module][link][chan]>noise_cut[module][link][chan/128])){
              SetMask(module,(link*768)+chan);
            }
          }
        }
        printf("  and setting the trim ranges...\n");
	
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            SetTrimRange(module,((link*6)+chip),best_range[link][chip]);
          }
        }
      }

      //
      // 9. Delete graph objects
      //

      printf("Deleting Graph Objects\n");

      for(link=0;link<MAX_LINKS;link++){
        for(range=0;range<MAX_RANGES;range++){
          for(chan=0;chan<MAX_CHANS;chan++){
            if(TrimGraph[link][range][chan])   TrimGraph[link][range][chan]->Delete();
          }
        }
      }

      for(link=0;link<MAX_LINKS;link++){
        for(range=0;range<MAX_RANGES;range++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            if(RMSGraph[link][range][chip])    RMSGraph[link][range][chip]->Delete();
            if(OffsetGraph[link][range][chip]) OffsetGraph[link][range][chip]->Delete();
            if(SlopeGraph[link][range][chip])  SlopeGraph[link][range][chip]->Delete();
          }
        }
      }

    }else{
      printf("No data for module %d\n",module);
    } // end if(module_status)

  } // end for(module)

        
  //
  // 10. Delete remaining objects
  //

  if(place) delete place;
  if(host) delete host;
  if(user) delete user;
  if(time) delete time;

  printf("Deleting Function Objects\n");

  for(link=0;link<MAX_LINKS;link++){
    for(chan=0;chan<MAX_CHANS;chan++){
      if(f3[link][chan]!=0) f3[link][chan]->Delete();
    }
  }

  printf("Deleting Histogram Objects\n");

  for(link=0;link<MAX_LINKS;link++){
    for(range=0;range<MAX_RANGES;range++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        h_ntrim[link][range][chip]->Delete();
      }
    }
    for(chip=0;chip<MAX_CHIPS;chip++){
      h_trim[link][chip]->Delete();
      h_trms[link][chip]->Delete();
      h_slope[link][chip]->Delete();
      h_offset[link][chip]->Delete();

      h_tp[link][chip]->Delete();
      h_ts[link][chip]->Delete();
    }
    h_tp_vch[link]->Delete();
    h_ts_vch[link]->Delete();
  }

  if(c3) c3->Delete();

  if(online){ 
    // Clear Batch mode so that online burst and scan hists will still appear
    // Canvas PWP 28.11.02
    printf("Clearing BATCH mode - online plots reenabled\n");
    gROOT->SetBatch(0);
  }

//  printf("Anne Robinson says, \"You are the weakest link.  Goodbye!!\"\n");
  printf("Laurence Boccolini dit \"Vous etez la maillon faible. Au revoir!!\"\n");
}


void TrimRangePlot(Int_t run, Int_t start_scan, Int_t full, Int_t n_ranges, Float_t charge) {

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t range, point;
  Int_t method = 0;   // whole module

  Float_t ranges[5] = {0,1,2,3,-1};
  Int_t scans[64];

  switch(full){

    case 1:
      printf("Plotting FULL TrimDAC/TrimRange characterisation\n");

      Int_t n_points[4] = {16, 16, 16, 16};
      Float_t trims[64] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
                           0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
                           0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
                           0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};
      break;
 
    case 2:
      printf("Plotting CUSTOM TrimDAC/TrimRange characterisation\n");

      Int_t n_points[4] = {16, 4, 4, 4};
      Float_t trims[64] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
                           3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0};
      break;

    case 3:
      printf("Plotting FAST TrimDAC/TrimRange characterisation\n");

      Int_t n_points[4] = {2, 2, 2, 2};
      Float_t trims[64] = {3,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
      break;

    default:
      printf("Plotting STANDARD TrimDAC/TrimRange characterisation\n");

      Int_t n_points[4] = {4, 4, 4, 4};
      Float_t trims[64] = {3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0,
                           3,7,11,15,0,0,0,0,0,0,0,0,0,0,0,0};
      break;
  }

  for(range=0;range<4;range++){
    for(point=0;point<n_points[range];point++){
      scans[(range*MAX_POINTS)+point]=start_scan++;
    }
  }

  TrimRangePlot(run,n_ranges,n_points,method,charge, 
                scans,ranges,trims,0,0,1);
}

// Wrappers to match the various configurations of the test 

void TrimRangePlot(Int_t run, Int_t start_scan, Int_t n_ranges, Float_t charge) {
  TrimRangePlot(run,start_scan,0,n_ranges,charge);
} 

void FullTrimRangePlot(Int_t run, Int_t start_scan, Int_t n_ranges, Float_t charge) {
  TrimRangePlot(run,start_scan,1,n_ranges,charge);
} 

void CustomTrimRangePlot(Int_t run, Int_t start_scan, Int_t n_ranges, Float_t charge) {
  TrimRangePlot(run,start_scan,2,n_ranges,charge);
} 

void FastTrimRangePlot(Int_t run, Int_t start_scan, Int_t n_ranges, Float_t charge) {
  TrimRangePlot(run,start_scan,3,n_ranges,charge);
}
 
