// Macro for beta source tests - DAQ
// Created by Pavel Reznicek (reznicek@ipnp.troja.mff.cuni.cz)
// Last update 29/07/2002
// DUT must be module 0
// WARNING: Graphical analyses of many scans can lead to 'low virtual memory' windows-error and one must restart sctdaq to
//          recover it. So if you need to run more scans and can't stay whole the time of scans near the beta-source setup
//          (for example: night measurements), switch off all possible graphical analyses by variable 'b_run_show' - see
//          below B_Scan. This shloud minimize the possibility of appearing that error.
// Search for string "EXAMPLES:" in this file to see important (most usefull) examples of use
// Necessary files:  BT.cpp
//                   BT_Anal.cpp
//                   BT_Plot.cpp
//                   utils.cpp
//                   c:/sctdaq/stlib/sct_scans.h
//                   c:/sctdaq/stlib/sct_hardware.h
//                     - if using another sctdaq directory, retype folowing lines in macro BT() (see below):
//                       gROOT->ProcessLine(".L c:/sctdaq/stlib/sct_scans.h");
//                       gROOT->ProcessLine(".L c:/sctdaq/stlib/sct_hardware.h");
//                     - all other paths are relative: using definitions in sct_hardware.h
//                   modulename.btc in your sctvar-config directory (for exmaple: d:\\sctvar\\config\\k4_211.btc)
//                     - content of this file is written as c++ code:
//                   .--------------------------------------------------------------------------------------------------.
//                   | // configuration:                                                                                |
//                   | #define BTC_TRIG_DELAY            118    // trigger delay [clock-cycles]                         |
//                   | #define BTC_WIN_LENGTH            5      // external trigger window length [nanoseconds]         |
//                   | #define BTC_WIN_DELAY             6      // external trigger window delay [nanoseconds]          |
//                   | #define BTC_QTHR                  2      // initial threshold [fC] value                         |
//                   | #define BTC_QTHR_CORR             1.07   // correction to charge injectors                       |
//                   | #define BTC_WAIT_MINUTES_COOLING  20     // check temperature .. minutes after start of BT.cpp   |
//                   | #define BTC_STOP_TEMP             70     // maximal allowed temperature ['C]                     |
//                   | #define BTC_STOP_IDET             100    // maximal allowed leakage current [mikroA]             |
//                   |                                                                                                  |
//                   | // intro                                                                                         |
//                   | {                                                                                                |
//                   |  printf("Module %s beta-test configuration loaded.\n",e->m[0]->name);                            |
//                   |                                                                                                  |
//                   |  // type any code here to prepare module 0 for beta-source tests                                 |
//                   |  e->HVRamp(0);  // ramp HV up                                                                    |
//                   |                                                                                                  |
//                   |  printf("End of module %s beta-test configuration sequence.\n",e->m[0]->name);                   |
//                   | }                                                                                                |
//                   '--------------------------------------------------------------------------------------------------'
//

#define WIN32               // defines locality - see sctdaq/stlib/sct_hardware.h
#define B_BURST_TYPE    21  // type of burst - see sctdaq/stdll/TST.cpp TST::ExecuteBurst
#define B_BURST_TRTYPE  1   // type of trigger - see sctdaq/stlib/stlib.c st_single

Int_t b_runnum;            // run number - changes when BT.cpp starts - see below B_NextRunnum
Int_t b_scannum;           // scan number - number of burst - see below B_Run
Short_t b_error;           // indicates problems (high temperature or detector current)
                           //  - see BTC_STOP_TEMP, BTC_STOP_IDET, BTC_WAIT_MINUTES_COOLING in sctvar/config/modulename.btc
TStopwatch *b_runtime;     // timer
Char_t b_logname[500]="";  // name of log-file - see below B_NextRunnum
Char_t b_resname[500]="";  // name of results-file - see below B_NextRunnum
Short_t b_online=(gROOT->FindObject("e")!=NULL || gROOT->GetGlobal("e")!=NULL);
                           // is running sctdaq or simple root for offline analyses

#define B_MAX_VARTYPE   200  // maximal number of sctdaq variable - see sctdaq/stlib/sct_scans.h
#define B_MAX_VARINDEX  250  // maximal number of beta-source-tests variable - see below #define BT_.. near B_STConfigs

TArrayF b_mean_configs(B_MAX_VARINDEX);  // array of sctdaq and beta-source-tests variables
Short_t b_masks[2*768];                  // array of masks
Float_t b_trims[2*768];                  // array of trimmings values
Float_t *b_mc;                           // pointer to array b_mean_configs - see above
Float_t b_new_qthr_corr=-1;              // if -1 ... follows b_mc[BT_QTHR_CORR]
                                         // otherwise this value is used in analyses instead of b_mc[BT_QTHR_CORR]
Char_t b_mcname[30]="";     // name of analysed module - see below
Char_t b_filename[500]="";  // name of loaded data file - see sctvar/data/btrun?_?.root

void BT(){
// starts beta source tests and/or analyses
  Char_t name[500];
  gROOT->ProcessLine(".L utils.cpp");
  gROOT->ProcessLine(".L c:/sctdaq/stlib/sct_scans.h");     // adapt this path to your locality (WIN32/RAID/VXILINUX)
  gROOT->ProcessLine(".L c:/sctdaq/stlib/sct_hardware.h");  // adapt this path to your locality (WIN32/RAID/VXILINUX)
  gROOT->ProcessLine(".L BT_Anal.cpp");
  gROOT->ProcessLine(".L BT_Plot.cpp");
  b_runtime=new TStopwatch();
  b_runtime->Start();
  if(b_online){
    if(e->cc==NULL) printf("BT: CLOAC module not present !!!\n");
    if(!e->m[0]->present) printf("BT: Module 0 not present !!!\n");
    else{
      sprintf(name,".x %s%s.btc",ST_CONFIG_PATH,e->m[0]->name);
      gROOT->ProcessLine(name);
      B_Default();
      e->m[0]->DCS->Query();
    }
  }
  B_NextRunnum();
}

TNtuple *b_configs=NULL,*b_trimasks=NULL;  // ntuple of configuration, trimmings and masking read from data file
TTree *b_events=NULL;                      // tree of events from data file - see sctvar/data/btrun?_?.root
                                           //  - see below B_DataFile

Int_t bev_mult0,bev_mult1,bev_counts0,bev_counts1,bev_clust0[768/2],bev_clust1[768/2],bev_hits0[7],bev_hits1[7];
Float_t bev_pos0[768/2],bev_pos1[768/2];
Bool_t bev_strm0[768],bev_strm1[768];
  // variables for branches' addresses of tree b_events

void B_SetEventBranchAddress(){
// sets branches' addresses of tree b_events - see below B_DataFile
  if(b_events==NULL){
    printf("B_SetEventBranchAddress: Events tree (TTree*)b_events is NULL !\n");
    return;
  }
  b_events->SetBranchAddress("strm0",&bev_strm0);
  b_events->SetBranchAddress("strm1",&bev_strm1);
  b_events->SetBranchAddress("hits0",&bev_hits0);
  b_events->SetBranchAddress("hits1",&bev_hits1);
  b_events->SetBranchAddress("mult0",&bev_mult0);
  b_events->SetBranchAddress("mult1",&bev_mult1);
  b_events->SetBranchAddress("counts0",&bev_counts0);
  b_events->SetBranchAddress("counts1",&bev_counts1);
  b_events->SetBranchAddress("pos0",&bev_pos0);
  b_events->SetBranchAddress("pos1",&bev_pos1);
  b_events->SetBranchAddress("clust0",&bev_clust0);
  b_events->SetBranchAddress("clust1",&bev_clust1);
}

#define B_CLK      25                 // clock cycle [ns]
#define B_WIN_MAX  16                 // maximal positive delay of external trigger window
#define B_WIN_MIN  B_WIN_MAX-B_CLK+1  // minimal negative delay of external trigger window

struct B_windel{
  Short_t orig,real;
};

B_windel B_WinDel(Short_t delay=-100,Short_t change=1){
// transforms hardware (original) value of external trigger window delay (0..24) to real value (-8..16)
// if change=1 then sets the value of external trigger window delay
  B_windel ret;
  if(!b_online && (change || delay==-100)){
    printf("B_WinDel: Object (TST*)e does'nt exist !\n");
    ret.orig=ret.real=0;
    return ret;
  }
  if(delay!=-100){
    if(delay<0){
      if(delay<B_WIN_MIN) ret.real=B_WIN_MIN;
      else ret.real=delay;
      ret.orig=ret.real+B_CLK;
      if(change) e->cc->SetWindowDelay(ret.orig);
    }else{
      if(delay>=B_CLK) ret.orig=B_CLK-1;
      else ret.orig=delay;
      if(change) e->cc->SetWindowDelay(ret.orig);
      if(ret.orig>B_WIN_MAX) ret.real=ret.orig-B_CLK;
      else ret.real=ret.orig;
    }
  }else{
    ret.orig=e->cc->GetWindowDelay();
    if(ret.orig>B_WIN_MAX) ret.real=ret.orig-B_CLK;
    else ret.real=ret.orig;
  }
  return ret;
}

Short_t B_WinLength(Short_t length){
// sets external trigger window length
// if length is zero then disables external trigger window
  if(!b_online){
    printf("B_WinLength: Object (TST*)e does'nt exist !\n");
    return -1;
  }
  if(length){
    e->cc->EnableTriggerWindow();
    e->cc->SetWindowLength(length);
    return 1;  // external trigger window is enabled
  }else{
    e->cc->DisableTriggerWindow();
    e->cc->SetWindowLength(0);
    return 0;  // external trigger window is disabled
  }
}

void B_Default(){
// loads default configuration - see sctvar/config/modulename.btc
  if(!b_online){
    printf("B_Default: Object (TST*)e does'nt exist !\n");
    return;
  }
  e->burst_trtype=B_BURST_TRTYPE;
  e->burst_type=B_BURST_TYPE;
  e->ConfigureVariable(ST_QTHR,BTC_QTHR);
  e->ConfigureVariable(ST_TRIG_DELAY,BTC_TRIG_DELAY);
  B_WinDel(BTC_WIN_DELAY);
  if(!B_WinLength(BTC_WIN_LENGTH)) printf("B_Default: No CLOAC window !!!\n");;
  e->ExecuteConfigs();
}

void B_NextRunnum(){
// changes run number
// removes sctvar/etc/BT_ERROR.txt file - see below B_run
  FILE* flast;
  Char_t name[500];
  TDatime *dt=new TDatime();
  sprintf(b_resname,"%sresults/bt_results_%d.txt",ST_VAR,dt->GetDate());
  delete dt;
  sprintf(name,"%sLAST_BT_RUN.txt",ST_ETC);
  if((flast=fopen(name,"rt"))==NULL) b_runnum=1;
  else{
    fscanf(flast,"%d",&b_runnum);
    fclose(flast);
  }
  if((flast=fopen(name,"wt"))==NULL)
    printf("B_NextRunnum: File %s can not be opened !\n",name);
  else{
    fprintf(flast,"%d",b_runnum+1);
    fclose(flast);
  }
  sprintf(b_logname,"%sbtlog%d.txt",ST_DATA_PATH,b_runnum);
  b_scannum=b_error=0;
  sprintf(name,"%sBT_ERROR.txt",ST_ETC);
  remove(name);
}

#define BT_MAX_VARTYPE           B_MAX_VARINDEX-1
#define BT_NREQUESTED            B_MAX_VARTYPE+0
#define BT_NSENT                 B_MAX_VARTYPE+1
#define BT_RUNNUM                B_MAX_VARTYPE+2
#define BT_SCANNUM               B_MAX_VARTYPE+3
#define BT_WIN_ORIG_DEL          B_MAX_VARTYPE+4
#define BT_WIN_REAL_DEL          B_MAX_VARTYPE+5
#define BT_WIN_LENGTH            B_MAX_VARTYPE+6
#define BT_RUN_TIME              B_MAX_VARTYPE+7
#define BT_ERROR                 B_MAX_VARTYPE+8
#define BT_BURST_TRTYPE          B_MAX_VARTYPE+9
#define BT_BURST_TYPE            B_MAX_VARTYPE+10
#define BT_DATE                  B_MAX_VARTYPE+11
#define BT_TIME                  B_MAX_VARTYPE+12
#define BT_QTHR_CORR             B_MAX_VARTYPE+13
#define BT_DELAY                 B_MAX_VARTYPE+14
#define BT_STOP_TEMP             B_MAX_VARTYPE+15
#define BT_STOP_IDET             B_MAX_VARTYPE+16
#define BT_WAIT_MINUTES_COOLING  B_MAX_VARTYPE+17
  // beta-source-tests variables

void B_STConfigs(TNtuple *nconfigs=NULL,TNtuple *ntrimasks=NULL){
// reads current sctdaq variables and trimmings and masking
  if(!b_online){
    printf("B_STConfigs: Object (TST*)e does'nt exist !\n");
    return;
  }

  Int_t type,chip,chn;
  Float_t vals[B_MAX_VARTYPE][12];
  Float_t mean;
  TDatime *dt=new TDatime();

  // sctdaq variables
  for(type=0;type<B_MAX_VARTYPE;type++){
    mean=0;
    for(chip=0;chip<12;chip++){
      vals[type][chip]=0;
      if(type!=ST_QTHR){
        for(chn=0;chn<128;chn++) vals[type][chip]+=e->RetrieveVariable(0,chip,chn,type);
        vals[type][chip]/=128;
      }else 
        vals[type][chip]=b_mc[ST_QTHR];  // qthr[fC] can't be obtained by TST::RetrieveVariable in most cases
                                         //  - besides linear dependence of qthr[fC] on vthr[mV]
      mean+=vals[type][chip];
    }
    if(nconfigs!=NULL) nconfigs->Fill(vals[type]);
    b_mc[type]=mean/12;
  }

  // some beta-source-tests variables
  dt->Set();
  b_mc[BT_DATE]=dt->GetDate()%1000000;
  b_mc[BT_TIME]=dt->GetTime();
  b_mc[BT_QTHR_CORR]=BTC_QTHR_CORR;
  b_mc[BT_MAX_VARTYPE]=B_MAX_VARTYPE;
  b_mc[BT_DELAY]=b_mc[ST_TRIG_DELAY]*B_CLK-b_mc[BT_WIN_REAL_DEL];
  sprintf(b_mcname,"%s",e->m[0]->name);
  if(nconfigs!=NULL) nconfigs->SetTitle(b_mcname);
  if(ntrimasks!=NULL) ntrimasks->SetTitle(b_mcname);

  for(type=B_MAX_VARTYPE;type<B_MAX_VARINDEX;type++)
    if(nconfigs!=NULL) nconfigs->Fill(b_mc[type],b_mc[type],b_mc[type],b_mc[type],
                                      b_mc[type],b_mc[type],b_mc[type],b_mc[type],
                                      b_mc[type],b_mc[type],b_mc[type],b_mc[type]);
  // trimmings and masking
  for(chn=0;chn<2*768;chn++){
    b_trims[chn]=e->RetrieveVariable(0,chn/128,chn%128,ST_TRIM);
    b_masks[chn]=((1<<(chn%16)) & (Int_t)(e->RetrieveVariable(0,chn/128,7-((chn%128)/16),ST_MASK)))>>(chn%16);
    if(ntrimasks!=NULL) ntrimasks->Fill(b_trims[chn],b_masks[chn]);
  }
  delete dt;
}

Short_t B_IsOpenedDataFile(Int_t runnum=-1,Int_t scannum=-1,Char_t *filename=NULL){
// compares name of data file (determined by 'runnum','scannum' or 'filename') with name of currently opened data file
  Char_t name[500];
  if(runnum==-1 || scannum==-1) sprintf(name,"%sbtrun%d_%d.root",ST_DATA_PATH,b_runnum,b_scannum);
  else sprintf(name,"%sbtrun%d_%d.root",ST_DATA_PATH,runnum,scannum);
  if(filename!=NULL) strcpy(name,filename);
  return !strcmp(name,b_filename);
}

#define B_NO_DATA  1      // loads only configuration from data file - see below B_DataFile - parameter 'no_data'
Short_t b_clust_max=769;  // maximal alowed width of cluster - see below last loop in B_DataFile

Short_t B_DataFile(Int_t runnum=-1,Int_t scannum=-1,Char_t *filename=NULL,Short_t no_data=0){
// loads data, sctdaq and beta-source-tests variables, trimmings and masking from data file
  Char_t name[500];
  Float_t *data;
  Int_t type,chn,i;
  
  // find name of the requested data file
  if(runnum==-1 || scannum==-1) sprintf(name,"%sbtrun%d_%d.root",ST_DATA_PATH,b_runnum,b_scannum);
  else sprintf(name,"%sbtrun%d_%d.root",ST_DATA_PATH,runnum,scannum);
  if(filename!=NULL) strcpy(name,filename);
  TFile *fileconf=new TFile(name,"READ");
  if(!fileconf->IsOpen()){
    printf("B_DataFile: File %s can not be opened !\n",name);
    return 0;
  }
  strcpy(b_filename,name);

  // load sctdaq and beta-source-tests variables
  if((fileconf->Get("configs"))==NULL){
    fileconf->Close();
    //delete fileconf;
    printf("B_DataFile: File %s does not contain (TNtuple*)configs !\n");
    sprintf(b_filename,"");
    return 0;
  }
  Int_t tmp=configs->GetEntries();;
  const Int_t nentries=tmp;
  b_mean_configs.Set(nentries);
  b_mean_configs.Reset();
  b_mc=b_mean_configs.GetArray();
  sprintf(b_mcname,"%s",configs->GetTitle());
  Float_t conf[nentries][12];
  for(type=0;type<nentries;type++){
    configs->GetEntry(type);
    data=configs->GetArgs();
    for(chn=0;chn<12;chn++){
      b_mc[type]+=data[chn];
      conf[type][chn]=data[chn];
    }
    b_mc[type]/=12;
  }
  printf("Configuration loaded\n");

  // load trimmings nad masking
  if((fileconf->Get("trimasks"))==NULL){
    fileconf->Close();
    //delete fileconf;
    printf("B_DataFile: File %s does not contain (TNtuple*)trimasks !\n");
    sprintf(b_filename,"");
    return 0;
  }
  for(type=0;type<2*768;type++){
    trimasks->GetEntry(type);
    data=trimasks->GetArgs();
    b_trims[type]=data[0];
    b_masks[type]=data[1];
  }
  printf("Trimming and masking loaded\n");

  // load tree of events - see sctdaq/stdll/TModule.cpp TModule::CreateTree
  if((fileconf->Get("EvTree"))==NULL){
    fileconf->Close();
    //delete fileconf;
    printf("B_DataFile: File %s does not contain (TTree*)EvTree !\n");
    sprintf(b_filename,"");
    return 0;
  }
  tmp=EvTree->GetEntries();
  const Int_t nevents=tmp;
  Bool_t evtr_strm0[768],evtr_strm1[768];
  Int_t hits_strm0[7],hits_strm1[7];
  Bool_t evtr_strm0_new[nevents][768],evtr_strm1_new[nevents][768];
  Int_t hits_strm0_new[nevents][7],hits_strm1_new[nevents][7];
  if(!no_data){  
    EvTree->SetBranchAddress("strm0",&evtr_strm0);
    EvTree->SetBranchAddress("strm1",&evtr_strm1);
    EvTree->SetBranchAddress("hits0",&hits_strm0);
    EvTree->SetBranchAddress("hits1",&hits_strm1);
    for(i=0;i<nevents;i++){
      EvTree->GetEntry(i);
      for(chn=0;chn<7;chn++){
        hits_strm0_new[i][chn]=hits_strm0[chn];
        hits_strm1_new[i][chn]=hits_strm1[chn];
      }
      for(chn=0;chn<768;chn++){
        evtr_strm0_new[i][chn]=evtr_strm0[chn];
        evtr_strm1_new[i][chn]=evtr_strm1[chn];
      }
      if(!(i%10)) printf("%cReading data: %d of %d",13,i+1,nevents);
    }
    printf("%c%d events read\t\t\t\t\t\t\n",13,nevents);
  }

  fileconf->Close();
  //delete fileconf;

  // create ntuple of sctdaq and beta-source-tests variables
  if(b_configs!=NULL) delete b_configs;
  b_configs=new TNtuple("b_configs",b_mcname,"chip0:chip1:chip2:chip3:chip4:chip5:chip6:chip7:chip8:chip9:chip10:chip11");
  for(type=0;type<nentries;type++) b_configs->Fill(conf[type]);

  // create ntruple of trimmings nad masking
  if(b_trimasks!=NULL) delete b_trimasks;
  b_trimasks=new TNtuple("b_trimasks",b_mcname,"trim:mask");
  for(type=0;type<2*768;type++) if(b_trimasks!=NULL) b_trimasks->Fill(b_trims[type],b_masks[type]);

  if(no_data){  // return if data tree b_evtree shloudn't be created
    strcat(b_filename," config");
    return 2;
  }

  // create tree of events
  if(b_events!=NULL) delete b_events;
  b_events=new TTree("b_events",b_mcname);
  b_events->Branch("strm0",&bev_strm0,"strm0[768]/B");     // hit map on link 0
  b_events->Branch("strm1",&bev_strm1,"strm1[768]/B");     // hit map on link 1
  b_events->Branch("hits0",&bev_hits0,"hits0[7]/I");       // number of active channels on every chip of link 0
  b_events->Branch("hits1",&bev_hits1,"hits1[7]/I");       // number of active channels on every chip of link 1
  b_events->Branch("mult0",&bev_mult0,"mult0/I");          // multiplicity (number of active channels) on link 0
  b_events->Branch("mult1",&bev_mult1,"mult1/I");          // multiplicity (number of active channels) on link 1
  b_events->Branch("counts0",&bev_counts0,"counts0/I");    // number of hits (clusters) on link 0
  b_events->Branch("counts1",&bev_counts1,"counts1/I");    // number of hits (clusters) on link 1
  b_events->Branch("pos0",&bev_pos0,"pos0[384]/F");        // mean positions of hits (clusters) on link 0
  b_events->Branch("pos1",&bev_pos1,"pos1[384]/F");        // mean positions of hits (clusters) on link 1
  b_events->Branch("clust0",&bev_clust0,"clust0[384]/I");  // cluster sizes of hits on link 0
  b_events->Branch("clust1",&bev_clust1,"clust1[384]/I");  // cluster sizes of hits on link 1

  for(i=0;i<nevents;i++){  // loop over all events of tree EvTree - see sctdaq/stdll/TModule.cpp TModule::CreateTree
    bev_counts0=bev_counts1=bev_mult0=bev_mult1=0;
    for(chn=0;chn<768/2;chn++) bev_pos0[chn]=bev_pos1[chn]=bev_clust0[chn]=bev_clust1[chn]=0;
    for(chn=0;chn<7;chn++){
      bev_hits0[chn]=hits_strm0_new[i][chn];
      bev_hits1[chn]=hits_strm1_new[i][chn];
      bev_mult0+=bev_hits0[chn];
      bev_mult1+=bev_hits1[chn];
    }
    for(chn=0;chn<768;chn++){
      bev_strm0[chn]=evtr_strm0_new[i][chn];
      if(bev_strm0[chn]){
        if(!bev_clust0[bev_counts0]) bev_pos0[bev_counts0]=chn;
        bev_clust0[bev_counts0]++;
      }
      if(bev_clust0[bev_counts0]>=b_clust_max || (!bev_strm0[chn]) && bev_clust0[bev_counts0]>0){
        bev_pos0[bev_counts0]+=0.5*((Float_t)bev_clust0[bev_counts0]-1.);
        bev_counts0++;
        bev_clust0[bev_counts0]=0;
      }
      bev_strm1[chn]=evtr_strm1_new[i][chn];
      if(bev_strm1[chn]){
        if(!bev_clust1[bev_counts1]) bev_pos1[bev_counts1]=chn;
        bev_clust1[bev_counts1]++;
      }
      if(bev_clust1[bev_counts1]>=b_clust_max || !bev_strm1[chn] && bev_clust1[bev_counts1]){
        bev_pos1[bev_counts1]+=0.5*((Float_t)bev_clust1[bev_counts1]-1.);
        bev_counts1++;
        bev_clust1[bev_counts1]=0;
      }
    }
    b_events->Fill();
    if(!(i%10)) printf("%cFormating data: %d of %d\t\t",13,i+1,nevents);
  }
  printf("%c%d events created\t\t\t\t\t\t\n",13,nevents);
  
  return 1;
}

#define B_MAX_VARNAME_LENGTH  35   // maximal length of name of sctdaq variables
                                   //  - see sctdaq/stdll/TST.cpp TST::TranslateVariableName
#define B_MAX_NSTEPS          200  // maximal number of scan-points - see below B_Scan
#define B_SHOW_SCAN_ANALYSIS  4    // after scan is finished, show it's analysis - see below B_Scan
#define B_SHOW_ONLINE         2    // show correlations and burst histogram during burst - see below B_Run
#define B_SHOW_ANALYSIS       1    // after burst is finihed, show it's analysis - see below B_Run
#define B_CALC_ANALYSIS       8    // after burst is finihed, calculate but not show it's analysis - see below B_Run
Short_t b_run_show=B_SHOW_ONLINE+B_SHOW_ANALYSIS+B_SHOW_SCAN_ANALYSIS;
                                   // change this variable to enable/disable analyses etc. - see above
                                   //  - for example when running scan, analysis of every burst is not needed =>
                                   //    b_run_show=B_SHOW_ONLINE+B_SHOW_SCAN_ANALYSIS;

Int_t B_Scan(Int_t vartype=-1,Float_t start=-1,Float_t stop=-1,Float_t step=-1,Float_t *list=NULL,Int_t nlist=0){
// performs scan of arbitrary sctdaq variable
// number of triggers is determined by 'e->burst_ntrigs' variable
/* EXAMPLES:
   B_Scan(); - just follow instructions in the root window
   B_Scan(41); - scans threshold [fC] (because 41=ST_QTHR - see sctdaq/stlib/sct_scans.h)
               - follow instructions in the root window
   B_Scan(41,1,5,1); - scans threshold [fC] from 1fC to 5fC by step 1fC
   Float_t l[5]={0.8,1.0,1.5,2.0,2.5};B_Scan(41,-1,-1,-1,list,5); - scans threshold [fC] defined by field 'l'
*/
  if(!b_online){
    printf("B_Scan: Object (TST*)e does'nt exist !\n");
    return 0;
  }

  Short_t i,j,odd=0,s_vartype,s_nlist=0,charge;
  Float_t s_start,s_stop,s_step,remember,thr;
  Char_t varname[B_MAX_VARNAME_LENGTH],scan[4];
  Float_t s_list[B_MAX_NSTEPS];
  FILE *flog;

  if(vartype==-1){
    printf("Named variables (not 'UKNOWN'):\n");

    for(i=0;i<B_MAX_VARTYPE;i++){  // show all named sctdaq variables
      e->TranslateVariableName(i,varname);
      if(strcmp(varname,"UKNOWN")){
        odd=1-odd;
        printf(" %3d - %s",i,varname);
        for(j=strlen(varname);j<B_MAX_VARNAME_LENGTH;j++) printf(" ");
        if(odd) printf("\t");
        else    printf("\n");
      }
    }
    if(odd) printf("\n");

    printf("Select No. of VARIABLE to be scanned: ");
    scanf("%d",&s_vartype);
    e->TranslateVariableName(s_vartype,varname);

  }else{  // scanned variable has been already selected
    e->TranslateVariableName(vartype,varname);
    printf("Selected No. of VARIABLE to be scanned: %d - %s\n",vartype,varname);
    s_vartype=vartype;
  }

  if(!nlist){
    if(start==-1){  // manually type scan-values of scanned variable
      printf("Scan %s (%d) from Start to Stop by Step or from List Of Parameters ? [sss/lop]: ",varname,s_vartype);
      scanf("%s",scan);
      if(!strcmp(scan,"sss")){  // use start-stop-step sequence
        printf("Start Stop Step: ");
        scanf("%f %f %f",&s_start,&s_stop,&s_step);
      }else{                    // type every scan-value of scanned variable
        printf("Select number of values: ");
        scanf("%d",&s_nlist);
        if(s_nlist>B_MAX_NSTEPS){
          printf("B_Scan: Too many scan-points - more then B_MAX_NSTEPS = %d\n",B_MAX_NSTEPS);
          s_nlist=B_MAX_NSTEPS);
        }
        for(i=0;i<s_nlist;i++){
          printf("%3d.: ",i);
          scanf("%f",&s_list[i]);
        }
        goto B_SCAN_CONT;
      }
    }else{  // list of scan-values of scanned variables has been already selected by start-stop-step sequence
      s_start=start;
      s_stop=stop;
      s_step=step;
    }
    remember=s_start;
    for(i=0;i<B_MAX_NSTEPS;i++){
      if(remember>s_stop && s_step>=0 || remember<s_stop && s_step<0) break;
      else s_list[i]=remember;
      remember+=s_step;
    }
    s_nlist=i;
    if(!(remember>s_stop && s_step>=0 || remember<s_stop && s_step<0))
      printf("B_scan: Too many scan-points - more then B_MAX_NSTEPS = %d\n",B_MAX_NSTEPS);

  }else{  // list of scan-values of scanned variables has been already created
    s_nlist=nlist;
    if(s_nlist>B_MAX_NSTEPS){
      printf("B_scan: Too many scan-points - more then B_MAX_NSTEPS = %d\n",B_MAX_NSTEPS);
      s_nlist=B_MAX_NSTEPS);
    }
    for(i=0;i<s_nlist;i++) s_list[i]=list[i];
  }

  B_SCAN_CONT:
  printf("Saving variable %s (%d) of module 0 = %f\n",varname,s_vartype,remember=e->RetrieveVariable(0,vartype));
  e->ExecuteConfigs();

  // set the threshold during scan of some other variable than threshold
  if(s_vartype!=ST_QTHR && s_vartype!=ST_VTHR){
    printf("Scan with charge(1) or threshold(0) [1/0 , threshold-value]: ");
    scanf("%d %f",&charge,&thr);
  }

  if(s_vartype==ST_VTHR) charge=0;
  if(s_vartype==ST_QTHR) charge=1;

  if((flog=fopen(b_logname,"at"))==NULL)
    printf("B_Scan: %s can not be opened !\n",b_logname);
  else{  // open log-file and write record about this scan
    fprintf(flog,"Module %s\n",e->m[0]->name);
    fprintf(flog,"Start scan %s (type = %d)",varname,s_vartype);
    if(s_vartype!=ST_VTHR && s_vartype!=ST_QTHR) fprintf(flog,"  Threshold %.2f %c%c",thr*(1+charge*(B_QTHR_CORR-1)),'m'+charge*('f'-'m'),'V'+charge*('C'-'V'));
    fprintf(flog,"\n%d values:",s_nlist);
    for(i=0;i<s_nlist;i++) fprintf(flog," %f",s_list[i]);
    fprintf(flog,"\n");
    fclose(flog);
  }

  for(i=0;i<s_nlist;i++){  // loop over all scan-values
    if(s_vartype==ST_VTHR || s_vartype==ST_QTHR) B_Run(s_list[i],charge);
    else{
      e->ConfigureVariable(s_vartype,s_list[i]);
      B_Run(thr,charge);
    }
    if(e->abort || b_error) {i++;break;}  // check for errors or abort
  }

  if((flog=fopen(b_logname,"at"))==NULL)
    printf("B_Scan: %s can not be opened !\n",b_logname);
  else{  // close log-record
    fprintf(flog,"End of scan\n\n");
    fclose(flog);
  }
  
  if(b_run_show&B_SHOW_SCAN_ANALYSIS) B_ScanPlot(s_vartype,b_scannum-i+1);  // show analysis of the scan
  // restore original value of scanned variable
  printf("\nSetting original variable (%d) %s = %f\n",s_vartype,varname,remember);
  e->ConfigureVariable(s_vartype,remember);
  e->ExecuteConfigs();
  return i;
}

void B_QthrScan(Int_t ntrigs=-1,
                Float_t thr00=-1,Float_t thr01=-1,Float_t thr02=-1,Float_t thr03=-1,Float_t thr04=-1,Float_t thr05=-1,Float_t thr06=-1,
                Float_t thr07=-1,Float_t thr08=-1,Float_t thr09=-1,Float_t thr10=-1,Float_t thr11=-1,Float_t thr12=-1,Float_t thr13=-1,
                Float_t thr14=-1,Float_t thr15=-1,Float_t thr16=-1,Float_t thr17=-1,Float_t thr18=-1,Float_t thr19=-1,Float_t thr20=-1){
// threshold [fC] scan (max. 21 values of qthr)
/* EXAMPLES:
   B_QthrScan(1000,0.8,1.3,2.8,3.5);
     - scans threshold [fC] with 1000 triggers during burst and with scan-values: 0.8,1.3,2.8,3.5 fC
*/
  if(!b_online){
    printf("B_QthrScan: Object (TST*)e does'nt exist !\n");
    return;
  }
  Float_t thr[21]={thr00,thr01,thr02,thr03,thr04,thr05,thr06,thr07,thr08,thr09,thr10,thr11,thr12,thr13,thr14,thr15,thr16,thr17,thr18,thr19,thr20};
  Int_t n;
  Int_t burst_ntrigs=e->burst_ntrigs;
  if(ntrigs!=-1) e->burst_ntrigs=ntrigs;
  for(n=0;n<21 && thr[n]!=-1;n++);
  B_Scan(ST_QTHR,-1,-1,-1,thr,n);
  e->burst_ntrigs=burst_ntrigs;
}

void B_VthrScan(Int_t ntrigs=-1,
                Float_t thr00=-1,Float_t thr01=-1,Float_t thr02=-1,Float_t thr03=-1,Float_t thr04=-1,Float_t thr05=-1,Float_t thr06=-1,
                Float_t thr07=-1,Float_t thr08=-1,Float_t thr09=-1,Float_t thr10=-1,Float_t thr11=-1,Float_t thr12=-1,Float_t thr13=-1,
                Float_t thr14=-1,Float_t thr15=-1,Float_t thr16=-1,Float_t thr17=-1,Float_t thr18=-1,Float_t thr19=-1,Float_t thr20=-1){
// threshold [mV] scan (max. 21 values of vthr)
/* EXAMPLES:
   B_VthrScan(1000,50,80,100,150,200);
     - scans threshold [fC] with 1000 triggers during burst and with scan-values: 50,80,100,150,200 mV
*/
  if(!b_online){
    printf("B_VthrScan: Object (TST*)e does'nt exist !\n");
    return;
  }
  Float_t thr[21]={thr00,thr01,thr02,thr03,thr04,thr05,thr06,thr07,thr08,thr09,thr10,thr11,thr12,thr13,thr14,thr15,thr16,thr17,thr18,thr19,thr20};
  Int_t n;
  Int_t burst_ntrigs=e->burst_ntrigs;
  if(ntrigs!=-1) e->burst_ntrigs=ntrigs;
  for(n=0;n<21 && thr[n]!=-1;n++);
  B_Scan(ST_VTHR,-1,-1,-1,thr,n);
  e->burst_ntrigs=burst_ntrigs;
}

#define B_ERROR_TEMP  1  // data acquisition stopped due to high temperature
#define B_ERROR_IDET  2  // data acquisition stopped due to high leakage current
Int_t b_run_info=20;     // update burst information, burst histograms and correlations graph after 'b_run_info' triggers
                         //  - every burst of 'e->burst_ntrigs' triggers (see below B_Run) is performed as a sequence of
                         //    bursts of 'b_run_info' triggers
struct B_mult{
  Float_t mult[2],sigma[2];
  Int_t n[2],nz[2];
};

struct B_clust{
  Float_t clust[2],sigma[2];
  Int_t n[2];
};

struct B_eff{
  Int_t n0,n1,n01;
  Float_t eff01,eff10,eff;
  Float_t err01,err10,err;
};

struct B_pos2{
  Float_t mean[2],sigma[2],width[2];
  Int_t n[2],nmax[2];
};

struct B_pos{
  Float_t mean,sigma,width;
  Int_t n,nmax;
};

struct B_analysis{
  Float_t mc[B_MAX_VARINDEX];
  Char_t name[30];
  B_eff eff;
  B_pos2 cross;
  B_pos along;
  B_clust clust;
  B_mult mult;
};

B_analysis B_Run(Float_t qvthr=1,Int_t q=1){
// burst of 'e->burst_ntrigs' external triggers
// if e->burst_ntrigs=0 then loops untill stopped
/* EXAMPLES:
   e->burst_ntrigs=1000;B_Run(1.3); - burst of 1000 triggers and threshold 1.3fC
   e->burst_ntrigs=5000;B_Run(80,0); - burst of 5000 triggers and threshold 80mV
   e->burst_ntrigs=0;B_Run(2); - burst of non-defined (untill stop is pressed) number of triggers and threshold 2fC
*/
  B_analysis ret;
  Int_t i,j;

  if(!b_online){
    printf("B_Run: Object (TST*)e does'nt exist !\n");
    for(i=0;i<B_MAX_VARINDEX;i++) ret.mc[i]=0;
    sprintf(ret.name,"ERR: not online");
    return ret;
  }

  TStopwatch time;
  Int_t cur_time;
  Int_t nevent=0,oneside=0,oneone=0,bothsides=0,overflow=0,zero=0,last_event_decoded=0,hits0,hits1,chn0,chn1;
  Int_t stop_loop=0;
  Char_t name[500];
  Int_t L1A_per_loop=e->L1A_per_loop,burst_ntrigs=e->burst_ntrigs,do_fill_evtree=e->do_fill_evtree;

  // set threshold (q=1 => [fC], q=0 => [mV])
  if(q) e->ConfigureVariable(ST_QTHR,qvthr);
  else  e->ConfigureVariable(ST_VTHR,qvthr);
  e->ExecuteConfigs();  

  b_scannum++;
  e->do_fill_evtree=1+2;  // fill event tree EvTree - see sctdaq/stdll/TST.cpp TST::DecodeEvent
  e->m[0]->DeleteTree();
  e->m[0]->CreateTree();  // create event tree EvTree - see sctdaq/stdll/TModule.cpp TModule::CreateTree
  e->burst_ntrigs=b_run_info;
  e->L1A_per_loop=1;

  TCanvas *c_runinfo;
  TPaveText *t_runinfo;
  TGraph *g_runcorr;
  TGraph *g_runlast;
  TH1F *hburst0,*hburst1;
  Int_t nlast=0;
  Int_t last_decoded_event=-1;
  // create canvas of online information, burst histograms and correlations graph
  if(b_run_show&B_SHOW_ONLINE){
    sprintf(name,"BT Run %d Scan %d Threshold %.2f %c%c",b_runnum,b_scannum,qvthr*(1+q*(BTC_QTHR_CORR-1)),'m'+q*('f'-'m'),'V'+q*('C'-'V'));
    c_runinfo=new TCanvas("Bc_runinfo",name,700,600);
    c_runinfo->Divide(2,2);
    c_runinfo->cd(1);
    t_runinfo=new TPaveText(0.,0.,1.,1.);
    t_runinfo->SetTextSize(0.1);
    t_runinfo->SetTextColor(kBlue);
    t_runinfo->SetTextAlign(1*10+2);
    t_runinfo->Draw();
    c_runinfo->cd(2);
    g_runcorr=new TGraph(1,0,0);
    g_runcorr->SetTitle("Correlations");
    g_runcorr->Draw("AP");
    g_runcorr->GetXaxis()->SetTitle("Channel number LINK 0");
    g_runcorr->GetYaxis()->SetTitle("Channel number LINK 1");
    g_runcorr->Draw("AP");
    g_runlast=new TGraph(1,0,0);
    g_runlast->SetMarkerColor(kRed);
    g_runlast->SetMarkerStyle(3);
    g_runlast->SetMarkerSize(.5);
    g_runlast->Draw("P");
    c_runinfo->cd(3);
    hburst0=new TH1F(*(e->m[0]->h_burst0));
    hburst0->Reset("ICE");
    hburst0->Draw();
    c_runinfo->cd(4);
    hburst1=new TH1F(*(e->m[0]->h_burst1));
    hburst1->Reset("ICE");
    hburst1->Draw();
    c_runinfo->Modified();
    c_runinfo->Update();
  }

  time.Start();
  st_start_stopper();

  // main loop - stop if temperature or leakage current is too high - see above description of 'b_error'
  while(!stop_loop && (nevent<burst_ntrigs || burst_ntrigs==0) &&
        e->m[0]->DCS->idet<=BTC_STOP_IDET &&
        (b_runtime->RealTime()<=BTC_WAIT_MINUTES_COOLING || e->m[0]->DCS->T1<=BTC_STOP_TEMP)){

    if(nevent+b_run_info>burst_ntrigs) e->burst_ntrigs=burst_ntrigs-nevent;
      // adapt number of triggers of last burst so that requested overall number of triggers is achieved
    e->ExecuteBurst();

    if(b_run_show&B_SHOW_ONLINE){  // show online information, burst histograms and correlations graph
      e->m[0]->PrintBurst();
      nevent=e->m[0]->evtree->GetEntries();
      for(i=last_decoded_event+1;i<nevent;i++){  // read hit maps of last triggers
        e->m[0]->evtree->GetEntry(i);
        hits0=e->m[0]->hits_strm0[0]+e->m[0]->hits_strm0[1]+e->m[0]->hits_strm0[2]+e->m[0]->hits_strm0[3]+
              e->m[0]->hits_strm0[4]+e->m[0]->hits_strm0[5]+e->m[0]->hits_strm0[6];
        hits1=e->m[0]->hits_strm1[0]+e->m[0]->hits_strm1[1]+e->m[0]->hits_strm1[2]+e->m[0]->hits_strm1[3]+
              e->m[0]->hits_strm1[4]+e->m[0]->hits_strm1[5]+e->m[0]->hits_strm1[6];
        if(hits0+hits1==0) zero++;
        hits0=hits1=0;
        for(j=0;j<768;j++){
          if(e->m[0]->evtr_strm0[j] && hits0==2) hits0=3;
          if(e->m[0]->evtr_strm0[j] && !hits0) {hits0=1;chn0=j;}
          if(!e->m[0]->evtr_strm0[j] && hits0==1) hits0=2;
          if(e->m[0]->evtr_strm1[j] && hits1==2) hits1=3;
          if(e->m[0]->evtr_strm1[j] && !hits1) {hits1=1;chn1=j;}
          if(!e->m[0]->evtr_strm1[j] && hits1==1) hits1=2;
        }
        if(hits0!=0 && hits0<3 && hits1==0 || hits1!=0 && hits1<3 && hits0==0) oneside++;
        if(hits0!=0 && hits0<3 && hits1!=0 && hits1<3) oneone++;
        if(hits0 && hits1){  // fill correlations graph
          g_runcorr->SetPoint(bothsides,chn0,chn1);
          g_runlast->SetPoint(nlast,chn0,chn1);
          nlast++;
          bothsides++;
        }
        if(!(i%10) || i==nevent-1) printf("%cAnalysing %d events: %d done",13,i+1,nevent);
      }
      printf("\n");
      overflow+=e->burst_ntrigs-(nevent-last_decoded_event-1);
      last_decoded_event=nevent-1;

      c_runinfo->Clear("D");
      c_runinfo->cd(1);
      t_runinfo->Clear();
      sprintf(name,"Sent:  %d");  // number of sent triggers
      if(burst_ntrigs) sprintf(name,"%s  of  %d",name,burst_ntrigs);  // number of requsted triggers
      t_runinfo->AddText(name);
      sprintf(name,"Zero:  %d",zero);  // number of zero-multiplicity events
      t_runinfo->AddText(name);
      sprintf(name,"Single:  %d",oneside);  // number of 1-hit on 1 link events
      t_runinfo->AddText(name);
      sprintf(name,"Double:  %d",oneone);  // number of 1-hit on both links events
      t_runinfo->AddText(name);
      sprintf(name,"Missed:  %d",overflow);  // number of missed (not decoded) events
      t_runinfo->AddText(name);
      sprintf(name,"Temp:  %d 'C",e->m[0]->DCS->T1);  // hybrid temperature
      t_runinfo->AddText(name);
      sprintf(name,"Idet:  %.2f mikroA",e->m[0]->DCS->idet);  // leakage current
      t_runinfo->AddText(name);
      cur_time=(Int_t)ceil(time.RealTime());
      time.Continue();
      sprintf(name,"Time:  %2d:%2d:%2d",cur_time/60/60,(cur_time/60)%60,cur_time%60);  // time of measurement
      t_runinfo->AddText(name);
      t_runinfo->AddText("  ");
      t_runinfo->Draw();
      c_runinfo->cd(2);
      g_runcorr->Draw("AP");
      if(nlast) g_runlast->Draw("P");
      c_runinfo->Modified();
      c_runinfo->Update();
      g_runlast->Set(1);
      nlast=0;

      // update burst histograms
      hburst0->Add(e->m[0]->h_burst0);
      hburst1->Add(e->m[0]->h_burst1);
      c_runinfo->cd(3);
      hburst0->Draw();
      c_runinfo->cd(4);
      hburst1->Draw();
      c_runinfo->Modified();
      c_runinfo->Update();
      st_display_burst();
    }
    stop_loop=st_do_stop();

  }
  st_stop_stopper();
  delete c_runinfo;
  printf("\nTime of run: %.1lf sec.\n",time.RealTime());

  // create configuration, trimmings and masking ntuples
  TNtuple *configs=new TNtuple("configs","name","chip0:chip1:chip2:chip3:chip4:chip5:chip6:chip7:chip8:chip9:chip10:chip11");
  TNtuple *trimasks=new TNtuple("trimasks","name","trim:mask");

  // fill beta-source-tests variables
  b_mean_configs.Set(B_MAX_VARINDEX);
  b_mean_configs.Reset();
  b_mc=b_mean_configs.GetArray();
  if(q) b_mc[ST_QTHR]=qvthr;
  else b_mc[ST_QTHR]=-1;
  b_mc[BT_NREQUESTED]=burst_ntrigs;
  b_mc[BT_NSENT]=nevent;
  b_mc[BT_RUNNUM]=b_runnum;
  b_mc[BT_SCANNUM]=b_scannum;
  B_windel wnd;
  wnd=B_WinDel(e->cc->GetWindowDelay(),0);
  b_mc[BT_WIN_ORIG_DEL]=wnd.orig;
  b_mc[BT_WIN_REAL_DEL]=wnd.real;
  b_mc[BT_WIN_LENGTH]=e->cc->GetWindowLength();
  b_mc[BT_RUN_TIME]=time.RealTime();
  b_mc[BT_BURST_TRTYPE]=e->burst_trtype;
  b_mc[BT_BURST_TYPE]=e->burst_type;  
  b_mc[BT_WAIT_MINUTES_COOLING]=BTC_WAIT_MINUTES_COOLING;
  b_mc[BT_STOP_TEMP]=BTC_STOP_TEMP;
  b_mc[BT_STOP_IDET]=BTC_STOP_IDET;
  if(b_runtime->RealTime()>BTC_WAIT_MINUTES_COOLING && e->m[0]->DCS->T1>BTC_STOP_TEMP) b_error|=B_ERROR_TEMP;
  if(e->m[0]->DCS->idet>BTC_STOP_IDET) b_error|=B_ERROR_IDET;
  b_mc[BT_ERROR]=b_error;

  // fill sctdaq variables and trimming and masking - see above B_STConfigs
  B_STConfigs(configs,trimasks);

  // write configuration, trimmings, masking nad events tree to data file
  sprintf(name,"%sbtrun%d_%d.root",ST_DATA_PATH,b_runnum,b_scannum);
  TFile *filevent=new TFile(name,"NEW");
  if(filevent->IsOpen()){
    e->m[0]->evtree->Write();
    configs->Write();
    trimasks->Write();
    filevent->Flush();
    filevent->Close();
    delete filevent;
  }else 
    printf("B_Run: File %s can not be opened !\n",name);

  delete configs;
  delete trimasks;

  e->burst_ntrigs=burst_ntrigs;
  e->do_fill_evtree=do_fill_evtree;
  e->L1A_per_loop=L1A_per_loop;

  FILE *flog;
  if((flog=fopen(b_logname,"at"))==NULL)
    printf("B_Run: %s can not be opened !\n",b_logname);
  else{  // open log-file and write record about this burst
    fprintf(flog,"scannum %d \t threshold %.2f %c%c \t no. trigs. sent %d",b_scannum,qvthr,'m'+q*('f'-'m'),'V'+q*('C'-'V'),nevent);
    if(b_error) if(e->m[0]->DCS->idet>B_STOP_IDET) fprintf(flog," \t ERROR Idet %.2f mikroA > %.2f mikroA\n",e->m[0]->DCS->idet,B_STOP_IDET);
                else fprintf(flog," \t ERROR Temp %d 'C > %d 'C\n",e->m[0]->DCS->T1,B_STOP_TEMP);
    else fprintf(flog,"\n");
    fclose(flog);
  }

  FILE *ferr;
  if(b_error){  // check for errors
    e->LVOff();    // switch off LV
    e->HVRamp(1);  // switch off HV
    if(b_error&B_ERROR_TEMP) printf("\nBreaking run due to high temperature T1 = %.0f 'C\n",e->m[0]->DCS->T1);
    if(b_error&B_ERROR_IDET) printf("\nBreaking run due to high leakage current Idet = %.2f mikroA\n",e->m[0]->DCS->idet);
    printf("Set 'b_error=0' when having fixed the problem\n");
    sprintf(name,"%sBT_ERROR.txt",ST_ETC);
    if((ferr=fopen(name,"wt"))==NULL)
      printf("B_Run: File %s can not be opened !\n",name);
    else{  // create file sctar/etc/BT_ERROR.txt as indication of error for remote users on the net
      fprintf(ferr,"Module: %s\nRun: \t %d\nScan: \t %d\nTemperature: \t %.0f 'C (limit %.0f 'C)\nIdet: \t %.2f mikroA (limit %.2f mikroA)\nThreshold: \t %.2f %c%c \t",
                   e->m[0]->name,b_runnum,b_scannum,e->m[0]->DCS->T1,BTC_STOP_TEMP,e->m[0]->DCS->idet,BTC_STOP_IDET,qvthr,'m'+q*('f'-'m'),'V'+q*('C'-'V'));
      fclose(ferr);
    }
  }

  if(b_run_show&B_SHOW_ANALYSIS) ret=B_RunPlot();                                     // show analysis of the burst
  else if(b_run_show&B_CALC_ANALYSIS) ret=B_RunPlot(-1,-1,NULL,NULL,NULL,B_NO_PLOT);  // calculate analysis of the burst
       else{                                                                          // analysis is missed
         for(i=0;i<B_MAX_VARINDEX;i++) ret.mc[i]=b_mc[i];
         sprintf(ret.name,"ERR: not analysed");
       }

  return ret;
}

Float_t B_TriggerBurst(Short_t counts=1){
// simple trigger burst(s) of external triggers with trigger rate measurement
/* EXAMPLES:
   e->burst_ntrigs=1000;B_TriggerBurst(); - 1 burst of 1000 triggers
   e->burst_ntrigs=500;B_TriggerBurst(0); - non-defined (untill stop is pressed) number of bursts of 500 triggers
   e->burst_ntrigs=100;B_TriggerBurst(5); - 5 bursts of 100 triggers
*/
  if(!b_online){
    printf("B_TriggerBurst: Object (TST*)e does'nt exist !\n");
    return 0;
  }

  TStopwatch time;
  Int_t realcount=0;
  Int_t stop_loop=0;
  Float_t freq;

  time.Start();
  if(counts!=1) st_start_stopper();
  while(!stop_loop && (realcount<counts || counts==0)){
    printf("Burst no. %d: waiting for %d external triggers\n",realcount+1,e->burst_ntrigs);
    e->ExecuteBurst();
    st_display_burst();
    realcount++;
    if(counts!=1) stop_loop=st_do_stop();
  }
  if(counts!=1) st_stop_stopper();
  freq=ceil(ceil((Float_t)e->burst_ntrigs/(Float_t)e->L1A_per_loop))*(Float_t)e->L1A_per_loop/time.RealTime()*(Float_t)realcount;
  printf("Mean frequency of triggers (not real for few burts_ntrigs): %.1f Hz\n",freq);
  return freq;
}

#define B_MAX_TRIG_DELAY  141  // maximal possible trigger delay

struct B_delay{
  Float_t trig;
  B_windel win;
};

B_delay B_TriggerWinDelay(Float_t qvthr=1,const Int_t trigstart=100,const Int_t trigstop=140,const Int_t winstep=1,Short_t eff=1,Short_t q=1){
// scans over selected interval of trigger delay and finds trigger delay with maximal number of hits
// than scans window delay over previous, this and following trigger delay (window delay is scanned with step 'winstep')
// if eff=1 then number of efficient both-links hits is considered, otherwise overall number of active channels is counted
// finds the ideal external window position and trigger delay (with the highest efficiency)
// number of triggers is determined by 'e->burst_ntrigs' variable
/* EXAMPLES:
   B_TriggerWinDelay(2.5,110,130,3); - scans trigger delay from 110 to 130, threshold is 2.5 fC, window delay is scanned by
                                       step 3, every burst is analysed and number of efficient hits is calculated
   B_TriggerWinDelay(2,100,135,5,0); - scans trigger delay from 100 to 135, threshold is 2 fC, window delay is scanned by
                                       step 5, only number of active channels is being compared
   B_TriggerWinDelay(200,100,135,5,0,0); - scans trigger delay from 100 to 135, threshold is 200 mV, window delay is scanned
                                           by step 5, only number of active channels is being compared
   B_TriggerWinDelay(200,100,135,5,1,0); - scans trigger delay from 100 to 135, threshold is 200 mV, window delay is scanned
                                           by step 5, every burst is analysed and number of efficient hits is calculated
*/
  B_delay ret;

  if(!b_online){
    printf("B_TriggerWinDelay: Object (TST*)e does'nt exist !\n");
    ret.trig=ret.win.real=ret.win.orig=0;
    return ret;
  }

  Float_t max_counts=0,counts;
  Int_t i,j,max=trigstart;
  B_pos posC,posA;
  B_eff effM;
  Int_t stop_loop=0;
  B_analysis anal;
  
  // set threshold (q=1 => [fC], q=0 => [mV])
  if(q) e->ConfigureVariable(ST_QTHR,qvthr);
  else  e->ConfigureVariable(ST_VTHR,qvthr);
  e->ExecuteConfigs();

  B_WinLength(0);  // switch of external trigger window - see above B_WinLength

  FILE *flog;
  if((flog=fopen(b_logname,"at"))==NULL)
    printf("B_TriggerWinDelay: %s can not be opened !\n",b_logname);
  else{  // open log-file and write record about this scan
    if(eff){
      fprintf(flog,"Start scan Trigger-Window delay (eff. hits)  Threshold %.2f %c%c\n",qvthr,'m'+q*('f'-'m'),'V'+q*('C'-'V'));
      fprintf(flog,"Trigger Delay: %d .. %d , Window delay step: %d\n",trigstart,trigstop,winstep);
    }else
      fprintf(flog,"Start scan %d Trigger-Window delay (all hits)  Threshold %.2f %c%c\n",b_scannum+1,qvthr,'m'+q*('f'-'m'),'V'+q*('C'-'V'));
    fclose(flog);
  }

  TFile *fileburst;
  TNtuple *configs,*trimasks;
  Char_t name[500];
  if(!eff){
    b_scannum++;
    sprintf(name,"%sbtrun%d_%d.root",ST_DATA_PATH,b_runnum,b_scannum);
    fileburst=new TFile(name,"NEW");
    if(!fileburst->IsOpen()) printf("B_TriggerWinDelay: File %s can not be opened !\n",name);
    configs=new TNtuple("configs","name","chip0:chip1:chip2:chip3:chip4:chip5:chip6:chip7:chip8:chip9:chip10:chip11");
    trimasks=new TNtuple("trimasks","name","trim:mask");
    b_mean_configs.Set(B_MAX_VARINDEX);
    b_mean_configs.Reset();
    b_mc=b_mean_configs.GetArray();
    if(q) b_mc[ST_QTHR]=qvthr;
    else b_mc[ST_QTHR]=-1;
    b_mc[BT_NSENT]=e->burst_ntrigs;
    b_mc[BT_RUNNUM]=b_runnum;
    b_mc[BT_SCANNUM]=b_scannum;
    b_mc[BT_BURST_TRTYPE]=e->burst_trtype;
    b_mc[BT_BURST_TYPE]=e->burst_type;
    st_start_stopper();
  }else
    b_run_show|=B_SHOW_ANALYSIS;
  
  for(i=trigstart;i<=trigstop;i++){  // loop over trigger delays
    if(i>B_MAX_TRIG_DELAY || i<0) continue;
    e->ConfigureVariable(ST_TRIG_DELAY,(Float_t)i);
    e->ExecuteConfigs();
    if(eff){
      if(e->abort || b_error) break;
      anal=B_Run(qvthr,q);
      if(max_counts<anal.eff.n01){
        max=i;
        max_counts=anal.eff.n01;
      }
    }else{
      if((stop_loop=st_do_stop())!=0) break;
      B_TriggerBurst();
      counts=e->m[0]->h_burst0->Integral()+e->m[0]->h_burst1->Integral();
      if(fileburst->IsOpen()){
        e->m[0]->h_burst0->Write();            // write to data file burst histograms
        e->m[0]->h_burst1->Write();
        B_STConfigs(configs,trimasks);         // read sctdaq variables (configuration)
        configs->Write();                      // write sctdaq variables (configuration)
        if(trimasks!=NULL) trimasks->Write();  // write trimmings and masking
        fileburst->Flush();
        delete configs;
        if(trimasks!=NULL) delete trimasks;
        configs=new TNtuple("configs","name","chip0:chip1:chip2:chip3:chip4:chip5:chip6:chip7:chip8:chip9:chip10:chip11");
      }
      printf("***  Trigger delay:  %d , Number of hits:  %.0f  ***\n",i,counts);
      if(max_counts<counts){
        max=i;
        max_counts=counts;
      }
    }
  }
  printf("***  Max. number of (efficient) hits found at trigger delay = %d  ***\n",max);
  if(!eff) st_stop_stopper();
  else{
    if((flog=fopen(b_logname,"at"))==NULL)
      printf("B_TriggerWinDelay: %s can not be opened !\n",b_logname);
    else{  // write scan record to log-file
      fprintf(flog,"End of trigger delay scan\n");
      fclose(flog);
    }
  }
  
  B_WinLength(BTC_WIN_LENGTH);  // set requested external trigger window length
  B_windel wnd;
  Int_t c=0;

  if(!eff) st_start_stopper();
  for(i=max-1;i<=max+1;i++) for(j=B_WIN_MAX-1-BTC_WIN_LENGTH/2;j>=B_WIN_MIN+1+BTC_WIN_LENGTH/2;j-=winstep){
    // loop over 3 trigger delays and over external window delays
    e->ConfigureVariable(ST_TRIG_DELAY,(Float_t)i);
    wnd=B_WinDel(j);
    e->ExecuteConfigs();
    if(eff){
      if(e->abort || b_error) break;
      c++;
      anal=B_Run(qvthr,q);
    }else{
      if((stop_loop=st_do_stop())!=0) break;
      B_TriggerBurst();
      counts=e->m[0]->h_burst0->Integral()+e->m[0]->h_burst1->Integral();
      if(fileburst->IsOpen()){
        e->m[0]->h_burst0->Write();  // write to data file burst histograms
        e->m[0]->h_burst1->Write();
        b_mc[BT_WIN_ORIG_DEL]=wnd.orig;
        b_mc[BT_WIN_REAL_DEL]=wnd.real;
        b_mc[BT_WIN_LENGTH]=e->cc->GetWindowLength();
        B_STConfigs(configs);        // read sctdaq variables (configuration)
        configs->Write();            // write sctdaq variables (configuration)
        fileburst->Flush();
        delete configs;
        configs=new TNtuple("configs","name","chip0:chip1:chip2:chip3:chip4:chip5:chip6:chip7:chip8:chip9:chip10:chip11");
      }
      printf("***  Trigger delay:  %d , Window delay:  %d , Number of hits:  %.0f  ***\n",i,j,counts);
    }
  }

  // analyse this delay scan - see BT_Plot.cpp B_TriggerWinDelayPlot
  if(!eff){
    st_stop_stopper();
    fileburst->Flush();
    fileburst->Close();
    delete fileburst;
    ret=B_TriggerWinDelayPlot(b_scannum);
  }else{
    ret=B_TriggerWinDelayPlot(b_scannum-c+1);
    if((flog=fopen(b_logname,"at"))==NULL)
      printf("B_TriggerWinDelay: %s can not be opened !\n",b_logname);
    else{  // close log-record
      fprintf(flog,"End of window delay scan\n");
      fclose(flog);
    }
  }
  
  printf("Setting new values of TRIGGER DELAY and EXTERNAL TRIGGER WINDOW DELAY\n");
  e->ConfigureVariable(ST_TRIG_DELAY,ret.trig);
  B_WinDel(ret.win.real);
  return ret;
}

void B_TriggerRate(Int_t start_ntrigs=0,Int_t stop_ntrigs=0,Int_t n_ntrigs=0,Int_t *ntrigs=NULL){
// shows dependece of trigger rate on number of triggers of the burst
// that means that this shows, when the trigger rate is influenced by e->ExecuteBurst() sowtware header
//  - variable b_run_info (see above B_Run) should be so high so that there is nealy no such flunce
/* EXAMPLES:
   B_TriggerRate(10,80,8); - sequence of bursts of 10,20,30,..,80 triggers follows
   Float_t n[3]={5,50,200};B_TriggerRate(0,0,3,n); - sequence of bursts of 5,50,200 triggers follows
*/
  const Int_t nlist=n_ntrigs;
  Float_t freqlist[nlist],triglist[nlist];
  Int_t i,burst_ntrigs=e->burst_ntrigs;
  if(n_ntrigs<2) return;

  for(i=0;i<n_ntrigs;i++){
    if(ntrigs==NULL) triglist[i]=e->burst_ntrigs=start_ntrigs+i*(stop_ntrigs-start_ntrigs)/(n_ntrigs-1);
    else triglist[i]=e->burst_ntrigs=ntrigs[i];
    freqlist[i]=B_TriggerBurst();
  }
  
  B_TriggerRatePlot(triglist,freqlist,nlist);
  e->burst_ntrigs=burst_ntrigs;
}