// Macro for beta source tests - Plots
// Created by Pavel Reznicek (reznicek@ipnp.troja.mff.cuni.cz)
// Last update 29/07/2002
// Search for string "EXAMPLES:" in this file to see important (most usefull) examples of use
// Search for string "WARNING:" in this file to see how to fix possible problems with elder sctdaq software versions

Int_t b_bond_max_dist=6;  // bonding histogram containts correlations of active channels distant
                          // maximally 'b_bond_max_dist' chnannels
Float_t b_strip_pitch=0.07;   // strip pitch in [mm]
Float_t b_strip_diverg=0.04;  // strip divergence [rad]
Short_t b_eff_chan_average=6,b_clust_chan_average=6;  // efficiency and cluster size vs. channel number graphs average
                                                      // eff. and clust. size over certain number of channels determined
                                                      // by this two variables
Float_t b_min_along_width=10;  // minimal along strips width of beam
Int_t b_clust_min_stat=1;      // minimal statistics needed for painting points of cluster size vs. channel number graphs
Int_t b_eff_min_stat=1;        // minimal statistics needed for painting points of efficiency vs. channel number graphs
#define B_CENTURY  2000        // present century
#define B_NO_PLOT  1           // analyze burst but do not show plots - see below B_RunPlot - parameter 'no_plot'

B_analysis B_RunPlot(Int_t runnum=-1,Int_t scannum=-1,Char_t *filename=NULL,B_pos2 *posC=NULL,B_pos *posA=NULL,Short_t no_plot=0){
// performs analysis of simple burst - see B_Run macro in BT.cpp
/* EXAMPLES:
   B_RunPlot(); - analyze and show plots of last measured burst - determined by 'b_runnum' and 'b_scannum' variables
   B_RunPlot(4,754); - analyze and show plots of burst from data file: sctvar/data/btrun754_4.root
   B_RunPlot(-1,-1,"d:\\btrun43_7.root"); - analyze and show plots of burst from data file: d:\btrun43_7.root
   B_RunPlot(-1,-1,NULL,pC,pA); - analyze and show plots of last measured burst, but for analyses use beam region defined
                                  by 'pC' and 'pA' - see B_RunCrossPosition and B_RunAlongPosition macros in BT_Anal.cpp
   B_RunPlot(2,9,NULL,NULL,NULL,B_NO_PLOT); - analyze but not show plots of burst from data file: sctvar/data/btrun9_2.root
*/
  Char_t name[500];
  B_analysis anal;
  Int_t i,j,k,l;
  
  if(!B_IsOpenedDataFile(runnum,scannum,filename))  // open requested data file
    if(!B_DataFile(runnum,scannum,filename)){
      printf("B_RunPlot: File can not be opened !\n");
      sprintf(anal.name,"ERR: bad filename");
      return anal;
    }
  
  printf("Analysing: Run %.0f Scan %.0f\n",b_mc[BT_RUNNUM],b_mc[BT_SCANNUM]);
  for(i=0;i<B_MAX_VARINDEX;i++) anal.mc[i]=b_mc[i];
  Int_t tmp=b_events->GetEntries();
  const Int_t nevents=tmp;
  if(posC==NULL) anal.cross=B_RunCrossPosition(runnum,scannum,filename);
  else anal.cross=*posC;
  if(posA==NULL) anal.along=B_RunAlongPosition(runnum,scannum,filename);
  else anal.along=*posA;
  anal.mult=B_RunMultiplicity(runnum,scannum,filename);
  anal.clust=B_RunClusterSize(anal.cross,anal.along,runnum,scannum,filename);
  anal.eff=B_RunEfficiency(anal.cross,anal.along,runnum,scannum,filename);
  sprintf(anal.name,"%s",b_mcname);
  if(no_plot) return anal;

  Float_t along_width=anal.along.width;
  if(along_width<b_min_along_width) along_width=b_min_along_width;  // set proper beam width in direction along strips
  
  Int_t mult0,mult1,counts0,counts1,clust0[768/2],clust1[768/2];
  Float_t pos0[768/2],pos1[768/2];
  Bool_t strm0[768],strm1[768];
  b_events->SetBranchAddress("strm0",&strm0);
  b_events->SetBranchAddress("strm1",&strm1);
  b_events->SetBranchAddress("mult0",&mult0);
  b_events->SetBranchAddress("mult1",&mult1);
  b_events->SetBranchAddress("counts0",&counts0);
  b_events->SetBranchAddress("counts1",&counts1);
  b_events->SetBranchAddress("pos0",&pos0);
  b_events->SetBranchAddress("pos1",&pos1);
  b_events->SetBranchAddress("clust0",&clust0);
  b_events->SetBranchAddress("clust1",&clust1);

  TCanvas *crun=new TCanvas("B_crun","BT Run Display",1200,850);
  crun->cd();
  TPad *pinfocorrbeam=new TPad("B_pinfocorrbeam","Run Info-Correlations-BeamProfile Pad",0.0,0.75,1.0,1.0);
  pinfocorrbeam->Divide(3,1,0,0);
  pinfocorrbeam->Draw();
  TPad *pcountmultclust=new TPad("B_pcountmultclust","Run Counts-Multiplicity-ClusterSize Pad",0.0,0.5,1.0,0.75);
  pcountmultclust->Divide(4,1,0,0);
  pcountmultclust->Draw();
  TPad *peffclust=new TPad("B_peffclust","Run Efficiency-ClusterSize position Pad",0.0,0.25,1.0,0.5);
  peffclust->Divide(4,1,0,0);
  peffclust->Draw();
  TPad *pbondhist=new TPad("B_pbondhist","Run Bonding-BurstHistogram Pad",0.0,0.0,1.0,0.25);
  pbondhist->Divide(4,1,0,0);
  pbondhist->Draw();

  pinfocorrbeam->cd(1);
  TPaveText *text0=new TPaveText(0.0,0.0,0.5,1.0);  // text information about this burst
  text0->Draw();
  text0->SetTextSize(0.08);
  text0->SetTextColor(kBlue);
  text0->SetTextAlign(1*10+2);
  sprintf(name,"Module: %s",b_mcname);
  text0->AddText(name);
  sprintf(name,"Run Scan: %.0f %.0f",b_mc[BT_RUNNUM],b_mc[BT_SCANNUM]);
  text0->AddText(name);
  sprintf(name,"Runtime %.1f",b_mc[BT_RUN_TIME]);
  text0->AddText(name);
  sprintf(name,"Date: %02d/%02d/%04d",(Int_t)b_mc[BT_DATE]%100,((Int_t)b_mc[BT_DATE]/100)%100,(Int_t)b_mc[BT_DATE]/100/100+B_CENTURY);
  text0->AddText(name);
  sprintf(name,"Time: %02d:%02d:%02d",(Int_t)b_mc[BT_TIME]/100/100,((Int_t)b_mc[BT_TIME]/100)%100,(Int_t)b_mc[BT_TIME]%100);
  text0->AddText(name);
  sprintf(name,"Vdet: %.2f V",b_mc[ST_MON_VDET]);
  text0->AddText(name);
  sprintf(name,"Idet: %.3f mikroA",b_mc[ST_MON_IDET]);
  text0->AddText(name);
  text0->AddText("  ");
  text0->AddText("LINK 0:");  // text information about link 0 of this burst
  sprintf(name,"Efficiency: %.2f %% (%.2f %%)",anal.eff.eff10,anal.eff.err10);
  text0->AddText(name);
  sprintf(name,"Cluster Size: %.4f (%.4f)",anal.clust.clust[0],anal.clust.sigma[0]);
  text0->AddText(name);
  sprintf(name,"Multiplicity: %.4f (%.4f)",anal.mult.mult[0],anal.mult.sigma[0]);
  text0->AddText(name);
  sprintf(name,"Non-Zero Ev.: %d",(Int_t)b_mc[BT_NSENT]-anal.mult.nz[0]);
  text0->AddText(name);
  sprintf(name,"In Beam Region: %d",anal.eff.n0);
  text0->AddText(name);
  text0->AddText("  ");

  TPaveText *text1=new TPaveText(0.5,0.0,1.0,1.0);  // text information about this burst
  text1->Draw();
  text1->SetTextSize(0.08);
  text1->SetTextColor(kBlue);
  text1->SetTextAlign(1*10+2);
  if(b_new_qthr_corr==-1) sprintf(name,"Threshold: %.2f %c%c",b_mc[ST_QTHR]*b_mc[BT_QTHR_CORR]*(b_mc[ST_QTHR]!=-1)+(b_mc[ST_QTHR]==-1)*b_mc[ST_VTHR],'m'+(b_mc[ST_QTHR]!=-1)*('f'-'m'),'V'+(b_mc[ST_QTHR]!=-1)*('C'-'V'));
  else sprintf(name,"Threshold: %.2f %c%c",b_mc[ST_QTHR]*b_new_qthr_corr*(b_mc[ST_QTHR]!=-1)+(b_mc[ST_QTHR]==-1)*b_mc[ST_VTHR],'m'+(b_mc[ST_QTHR]!=-1)*('f'-'m'),'V'+(b_mc[ST_QTHR]!=-1)*('C'-'V'));
  text1->AddText(name);
  sprintf(name,"Trig.Delay: %.0f",b_mc[ST_TRIG_DELAY]);
  text1->AddText(name);
  sprintf(name,"Ext.Win.Delay: %.0f(%.0f)",b_mc[BT_WIN_REAL_DEL],b_mc[BT_WIN_ORIG_DEL]);
  text1->AddText(name);
  sprintf(name,"Ext.Win.Length: %.0f",b_mc[BT_WIN_LENGTH]);
  text1->AddText(name);
  sprintf(name,"No.Ext.Trigs: %.0f",b_mc[BT_NSENT]);
  text1->AddText(name);
  sprintf(name,"Error: %.0f",b_mc[BT_ERROR]);
  text1->AddText(name);
  sprintf(name,"No.Eff.Events: %d",anal.eff.n01);
  text1->AddText(name);
  text1->AddText("  ");
  text1->AddText("LINK 1:");  // text information about link 1 of this burst
  sprintf(name,"Efficiency: %.2f %% (%.2f %%)",anal.eff.eff01,anal.eff.err01);
  text1->AddText(name);
  sprintf(name,"Cluster Size: %.4f (%.4f)",anal.clust.clust[1],anal.clust.sigma[1]);
  text1->AddText(name);
  sprintf(name,"Multiplicity: %.4f (%.4f)",anal.mult.mult[1],anal.mult.sigma[1]);
  text1->AddText(name);
  sprintf(name,"Non-Zero Ev.: %d",(Int_t)b_mc[BT_NSENT]-anal.mult.nz[1]);
  text1->AddText(name);
  sprintf(name,"In Beam Region: %d",anal.eff.n1);
  text1->AddText(name);
  text1->AddText("  ");

  pinfocorrbeam->Modified();
  pinfocorrbeam->Update();
  printf("Text panel created\n");

  TH2F* hcorr=new TH2F("B_hcorr","Correlations",768,-0.5,767.5,768,-0.5,767.5);
  for(i=0;i<nevents;i++){
    b_events->GetEntry(i);
    for(j=0;j<counts0 && counts1>0;j++) for(k=0;k<counts1;k++) hcorr->Fill(pos0[j],pos1[k]);
    if(!(i%10)) printf("%cReading events: %d of %d",13,i+1,nevents);
  }
  pinfocorrbeam->cd(2);
  hcorr->GetXaxis()->SetTitle("Channels on link 0");
  hcorr->GetYaxis()->SetTitle("Channels on link 1");
  hcorr->SetStats(kFALSE);
  hcorr->DrawCopy("colz");
  pinfocorrbeam->Modified();
  pinfocorrbeam->Update();
  delete hcorr;
  printf("%cCorrelations graph created\t\t\t\t\t\t\n",13);

  TH2F* hbeam=new TH2F("B_hbeam","Beam Profile",
                       (Int_t)(anal.cross.width[0]+anal.cross.width[1])*2,
                       (anal.cross.mean[0]-anal.cross.mean[1]-anal.cross.width[0]-anal.cross.width[1])*b_strip_pitch-0.5,
                       (anal.cross.mean[0]-anal.cross.mean[1]+anal.cross.width[0]+anal.cross.width[1])*b_strip_pitch-0.5,
                       (Int_t)along_width*2,
                       (anal.along.mean-along_width)*b_strip_pitch/b_strip_diverg-0.5,
                       (anal.along.mean+along_width)*b_strip_pitch/b_strip_diverg-0.5);
  for(i=0;i<nevents;i++){
    b_events->GetEntry(i);
    for(j=0;j<counts0 && counts1>0;j++) for(k=0;k<counts1;k++)
      hbeam->Fill((pos0[j]-pos1[k])*b_strip_pitch,(pos0[j]+pos1[k]-767)*b_strip_pitch/b_strip_diverg);
    if(!(i%10)) printf("%cReading events: %d of %d",13,i+1,nevents);
  }
  pinfocorrbeam->cd(3);
  hbeam->GetXaxis()->SetTitle("Cross strips position [mm]");
  hbeam->GetYaxis()->SetTitle("Along strips position [mm]");
  hbeam->SetStats(kFALSE);
  hbeam->DrawCopy("colz");
  pinfocorrbeam->Modified();
  pinfocorrbeam->Update();
  delete hbeam;
  printf("%cBeam-profile graph created\t\t\t\t\t\t\n",13);

  TH1F *hcount0=new TH1F("B_hcounts0","Not Used",768,-0.5,767.5);
  TH1F *hcount1=new TH1F("B_hcounts1","Not Used",768,-0.5,767.5);
  hcount0->SetFillColor(kBlue);
  hcount0->SetBarWidth(.2);
  hcount0->SetBarOffset(.4);
  hcount1->SetFillColor(kBlue);
  hcount1->SetBarWidth(.2);
  hcount1->SetBarOffset(.4);
  TH1F *hmult0=new TH1F("B_hmult0","Multiplicity(RED)/No.Hits(BLUE) on link 0",768,-0.5,767.5);
  TH1F *hmult1=new TH1F("B_hmult1","Multiplicity(RED)/No.Hits(BLUE) on link 1",768,-0.5,767.5);
  hmult0->SetFillColor(kRed);
  hmult1->SetFillColor(kRed);
  for(i=0;i<nevents;i++){
    b_events->GetEntry(i);
    hcount0->Fill(counts0);
    hcount1->Fill(counts1);
    hmult0->Fill(mult0);
    hmult1->Fill(mult1);
    if(!(i%10)) printf("%cReading events: %d of %d",13,i+1,nevents);
  }
  hmult0->GetXaxis()->SetTitle("Multiplicity/No.Hits");
  hmult0->GetYaxis()->SetTitle("Number of events");
  hmult0->GetXaxis()->SetRange(MinNonZeroBinTH1(hmult0),MaxNonZeroBinTH1(hmult0)+1);
  if(hcount0->GetMaximum()>hmult0->GetMaximum()) hmult0->GetYaxis()->SetRange(0,1.1*hcount0->GetMaximum()+1);
  pcountmultclust->cd(1);
  hmult0->DrawCopy();
  hcount0->DrawCopy("B SAME");
  hmult1->GetXaxis()->SetTitle("Multiplicity/No.Hits");
  hmult1->GetYaxis()->SetTitle("Number of events");
  hmult1->GetXaxis()->SetRange(MinNonZeroBinTH1(hmult1),MaxNonZeroBinTH1(hmult1)+1);
  if(hcount1->GetMaximum()>hmult1->GetMaximum()) hmult1->GetYaxis()->SetRange(0,1.1*hcount1->GetMaximum()+1);
  pcountmultclust->cd(3);
  hmult1->DrawCopy();
  hcount1->DrawCopy("B SAME");
  pcountmultclust->Modified();
  pcountmultclust->Update();
  delete hmult0;
  delete hmult1;
  delete hcount0;
  delete hcount1;
  printf("%cCounts histogram created\t\t\t\t\t\t\n",13);
  printf("Multiplicity histogram created\n");

  TH1F *hclust0=new TH1F("B_hclust0","Cluster size on link 0",768,-0.5,767.5);
  TH1F *hclust1=new TH1F("B_hclust1","Cluster size on link 1",768,-0.5,767.5);
  hclust0->SetFillColor(kRed);
  hclust1->SetFillColor(kRed);
  for(i=0;i<nevents;i++){
    b_events->GetEntry(i);
    for(j=0;j<counts0;j++) hclust0->Fill(clust0[j]);
    for(j=0;j<counts1;j++) hclust1->Fill(clust1[j]);
    if(!(i%10)) printf("%cReading events: %d of %d",13,i+1,nevents);
  }
  pcountmultclust->cd(2);
  hclust0->GetXaxis()->SetTitle("Cluster size");
  hclust0->GetYaxis()->SetTitle("Number of hits");
  hclust0->GetXaxis()->SetRange(MinNonZeroBinTH1(hclust0),MaxNonZeroBinTH1(hclust0)+1);
  hclust0->DrawCopy();
  pcountmultclust->cd(4);
  hclust1->GetXaxis()->SetTitle("Cluster size");
  hclust1->GetYaxis()->SetTitle("Number of hits");
  hclust1->GetXaxis()->SetRange(MinNonZeroBinTH1(hclust1),MaxNonZeroBinTH1(hclust1)+1);
  hclust1->DrawCopy();
  pcountmultclust->Modified();
  pcountmultclust->Update();
  delete hclust0;
  delete hclust1;
  printf("%cCluster-size histogram created\t\t\t\t\t\n",13);  

  TH1F *hhit0=new TH1F("B_hhit0","Hit channels on link 0",768,-0.5,767.5);
  TH1F *hhit1=new TH1F("B_hhit1","Hit channels on link 1",768,-0.5,767.5);
  hhit0->SetFillColor(kRed);
  hhit1->SetFillColor(kRed);
  TH1F *hbond0=new TH1F("B_hbond0","Bonding on link 0",768,0.5,768.5);
  TH1F *hbond1=new TH1F("B_hbond1","Bonding on link 1",768,0.5,768.5);
  hbond0->SetFillColor(kRed);
  hbond1->SetFillColor(kRed);
  Int_t bond;
  for(i=0;i<nevents;i++){
    b_events->GetEntry(i);
    bond=-1;
    for(j=0;j<768;j++) if(strm0[j]){
      hhit0->Fill(j);
      if(bond!=-1) hbond0->Fill(j-bond);
      bond=j;
    }
    bond=-1;
    for(j=0;j<768;j++) if(strm1[j]){
      hhit1->Fill(j);
      if(bond!=-1) hbond1->Fill(j-bond);
      bond=j;
    }
    if(!(i%10)) printf("%cReading events: %d of %d",13,i+1,nevents);
  }
  pbondhist->cd(2);
  hhit0->GetXaxis()->SetTitle("Channel no.");
  hhit0->GetYaxis()->SetTitle("Number of hits");
  hhit0->GetXaxis()->SetRange(MinNonZeroBinTH1(hhit0),MaxNonZeroBinTH1(hhit0)+1);
  hhit0->DrawCopy();
  pbondhist->cd(4);
  hhit1->GetXaxis()->SetTitle("Channel no.");
  hhit1->GetYaxis()->SetTitle("Number of hits");
  hhit1->GetXaxis()->SetRange(MinNonZeroBinTH1(hhit1),MaxNonZeroBinTH1(hhit1)+1);
  hhit1->DrawCopy();
  printf("%cBurst histogram created\t\t\t\t\t\t\n",13);
  pbondhist->cd(1);
  hbond0->GetXaxis()->SetTitle("Channel distance");
  hbond0->GetYaxis()->SetTitle("Number of found");
  hbond0->GetXaxis()->SetRange(1,b_bond_max_dist);
  hbond0->DrawCopy();
  pbondhist->cd(3);
  hbond1->GetXaxis()->SetTitle("Channel distance");
  hbond1->GetYaxis()->SetTitle("Number of found");
  hbond1->GetXaxis()->SetRange(1,b_bond_max_dist);
  hbond1->DrawCopy();
  pbondhist->Modified();
  pbondhist->Update();
  delete hhit0;
  delete hhit1;
  delete hbond0;
  delete hbond1;
  printf("Bonding-checking histogram created\n");

  Float_t n01[768],n0[768],n10[768],n1[768];
  for(i=0;i<768;i++) n01[i]=n10[i]=n0[i]=n1[i]=0;
  for(i=0;i<nevents;i++){
    b_events->GetEntry(i);
    for(j=0;j<counts0;j++){
      for(k=0;k<counts1;k++)
        if(pos0[j]-(clust0[j]-1)/2 <= 767+anal.along.mean-pos1[k]+along_width+(clust1[k]-1)/2 &&
           pos0[j]+(clust0[j]-1)/2 >= 767+anal.along.mean-pos1[k]-along_width-(clust1[k]-1)/2){
          n01[(Int_t)(pos0[j]/b_eff_chan_average)]++;
          break;
        }
      n0[(Int_t)(pos0[j]/b_eff_chan_average)]++;
    }
    for(j=0;j<counts1;j++){
      for(k=0;k<counts0;k++)
        if(pos0[k]-(clust0[k]-1)/2 <= 767+anal.along.mean-pos1[j]+along_width+(clust1[j]-1)/2 &&
           pos0[k]+(clust0[k]-1)/2 >= 767+anal.along.mean-pos1[j]-along_width-(clust1[j]-1)/2){
          n10[(Int_t)(pos1[j]/b_eff_chan_average)]++;
          break;
        }
      n1[(Int_t)(pos1[j]/b_eff_chan_average)]++;
    }
    if(!(i%10)) printf("%cReading events: %d of %d",13,i+1,nevents);
  }
  Float_t chn0[768],chn1[768];
  Float_t eff0[768],eff1[768];
  Float_t err0[768],err1[768];
  Int_t np0=0,np1=0;
  for(i=0;i<768;i++){
    if(n0[i]>=b_eff_min_stat){
      chn0[np0]=i*b_eff_chan_average+(b_eff_chan_average-1)*0.5;
      eff0[np0]=100.*(Float_t)n01[i]/(Float_t)n0[i];
      err0[np0]=sqrt(n01[i]+n0[i]);
      np0++;
    }
    if(n1[i]>=b_eff_min_stat){
      chn1[np1]=i*b_eff_chan_average+(b_eff_chan_average-1)*0.5;
      eff1[np1]=100.*(Float_t)n10[i]/(Float_t)n1[i];
      err1[np1]=sqrt(n10[i]+n1[i]);
      np1++;
    }
  }
  peffclust->cd(1);
  TGraphErrors *geff0=new TGraphErrors(np0,chn0,eff0,0,err0);
  geff0->SetName("B_geff0");
  sprintf(name,"Efficiency over %d channels link 0",b_eff_chan_average);
  geff0->SetTitle(name);
  geff0->SetMarkerStyle(5);
  geff0->SetMarkerColor(kRed);
  geff0->SetLineColor(kBlack);
  geff0->Draw("PA");
  if(np0>0){
    geff0->GetXaxis()->SetTitle("Channel no.");
    geff0->GetYaxis()->SetTitle("Efficiency [%]");
    geff0->Draw("PA");
  }
  peffclust->cd(3);
  TGraphErrors *geff1=new TGraphErrors(np1,chn1,eff1,0,err1);
  geff1->SetName("B_geff1");
  sprintf(name,"Efficiency over %d channels link 1",b_eff_chan_average);
  geff1->SetTitle(name);
  geff1->SetMarkerStyle(5);
  geff1->SetMarkerColor(kRed);
  geff1->SetLineColor(kBlack);
  geff1->Draw("PA");
  if(np1>0){
    geff1->GetXaxis()->SetTitle("Channel no.");
    geff1->GetYaxis()->SetTitle("Efficiency [%]");
    geff1->Draw("PA");
  }
  peffclust->Modified();
  peffclust->Update();
  printf("%cEfficiency-vs.-channel-number graph created\t\t\t\t\n",13);

  Float_t clustsz0[768],clustsz1[768],errsz0[768],errsz1[768];
  Int_t nsz0[768],nsz1[768];
  for(i=0;i<768;i++) clustsz0[i]=clustsz1[i]=errsz0[i]=errsz1[i]=nsz0[i]=nsz1[i]=0;
  for(i=0;i<nevents;i++){
    b_events->GetEntry(i);
    for(j=0;j<counts0 && counts1>0;j++) for(k=0;k<counts1;k++){
      clustsz0[(Int_t)(pos0[j]/b_eff_chan_average)]+=clust0[j];
      clustsz1[(Int_t)(pos1[k]/b_eff_chan_average)]+=clust1[k];
      nsz0[(Int_t)(pos0[j]/b_eff_chan_average)]++;
      nsz1[(Int_t)(pos1[k]/b_eff_chan_average)]++;
      errsz0[(Int_t)(pos0[j]/b_eff_chan_average)]+=clust0[j]*clust0[j];
      errsz1[(Int_t)(pos1[k]/b_eff_chan_average)]+=clust1[k]*clust1[k];
    }
    if(!(i%10)) printf("%cReading events: %d of %d",13,i+1,nevents);
  }
  Float_t chan0[768],chan1[768];
  Float_t csz0[768],csz1[768];
  Float_t esz0[768],esz1[768];
  Int_t nc0=0,nc1=0;
  for(i=0;i<768;i++){
    if(nsz0[i]>=b_clust_min_stat){
      chan0[nc0]=i*b_clust_chan_average+(b_clust_chan_average-1)*0.5;
      csz0[nc0]=clustsz0[i]/nsz0[i];
      esz0[nc0]=sqrt((errsz0[i]/nsz0[i]-csz0[nc0]*csz0[nc0])/nsz0[i]);
      nc0++;
    }
    if(nsz1[i]>=b_clust_min_stat){
      chan1[nc1]=i*b_clust_chan_average+(b_clust_chan_average-1)*0.5;
      csz1[nc1]=clustsz1[i]/nsz1[i];
      esz1[nc1]=sqrt((errsz1[i]/nsz1[i]-csz1[nc1]*csz1[nc1])/nsz1[i]);
      nc1++;
    }
  }
  peffclust->cd(2);
  TGraphErrors *gclustsz0=new TGraphErrors(nc0,chan0,csz0,0,esz0);
  gclustsz0->SetName("B_gclustsz0");
  sprintf(name,"Cluster size over %d channels link 0",b_clust_chan_average);
  gclustsz0->SetTitle(name);
  gclustsz0->SetMarkerStyle(5);
  gclustsz0->SetMarkerColor(kRed);
  gclustsz0->SetLineColor(kBlack);
  gclustsz0->Draw("PA");
  if(nc0>0){
    gclustsz0->GetXaxis()->SetTitle("Channel no.");
    gclustsz0->GetYaxis()->SetTitle("Cluster size");
    gclustsz0->Draw("PA");
  }
  peffclust->cd(4);
  TGraphErrors *gclustsz1=new TGraphErrors(nc1,chan1,csz1,0,esz1);
  gclustsz1->SetName("B_gclustsz1");
  sprintf(name,"Cluster size over %d channels link 1",b_clust_chan_average);
  gclustsz1->SetTitle(name);
  gclustsz1->SetMarkerStyle(5);
  gclustsz1->SetMarkerColor(kRed);
  gclustsz1->SetLineColor(kBlack);
  gclustsz1->Draw("PA");
  if(nc1>0){
    gclustsz1->GetXaxis()->SetTitle("Channel no.");
    gclustsz1->GetYaxis()->SetTitle("Cluster size");
    gclustsz1->Draw("PA");
  }
  peffclust->Modified();
  peffclust->Update();
  printf("%cCluster-size-vs.-channel-number graph created\t\t\t\t\n\n",13);

  crun->Modified();
  crun->Update();
  B_SetEventBranchAddress();  // recover addresses of 'b_events' branches
  return anal;
}

B_delay B_TriggerWinDelayPlot(Int_t start_scannum=1,Int_t stop_scannum=-1,Int_t runnum=-1){
// plots efficiency (number of hits) vs. external trigger window position - see B_TriggerWinDelay in BT.cpp
/* EXAMPLES:
   B_TriggerWinDelayPlot(1,20,12); - show plots for bursts measured by B_TriggerWinDelay macro in BT.cpp - bursts are stored
                                     in data files: sctvar/data/btrun12_1.root, .. , sctvar/data/btrun12_20.root
                                   - because start_scannum!=stop_scannum, B_TriggerWinDelay was run with parameter eff=1
   B_TriggerWinDelayPlot(1,1,12); - show plots for bursts measured by B_TriggerWinDelay macro - bursts are stored in data
                                    file: sctvar/data/btrun12_1.root
                                  - because start_scannum=stop_scannum, B_TriggerWinDelay was run with parameter eff=0
   B_TriggerWinDelayPlot(9); - show plots for burst measured by B_TriggerWinDelay macro- bursts are stored in data files:
                               sctvar/data/btrun'b_runnum'_9.root, .. , sctvar/data/btrun'b_runnum'_'b_scannum'.root
                             - if start_scannum=b_scannum then B_TriggerWinDelay was run with parameter eff=0,
                               else B_TriggerWinDelay was run with parameter eff=1
*/
  Int_t i=1,j,c=0,k;
  const Int_t start=start_scannum;
  if(stop_scannum==-1) const Int_t stop=b_scannum;
  else const Int_t stop=stop_scannum;
  if(runnum==-1) const Int_t run=b_runnum;
  else const Int_t run=runnum;

  Float_t counts[3*B_CLK+B_MAX_TRIG_DELAY+1];
  Float_t effs[3*B_CLK+B_MAX_TRIG_DELAY+1],erreffs[3*B_CLK+B_MAX_TRIG_DELAY+1];
  Float_t delays[3*B_CLK+B_MAX_TRIG_DELAY+1];
  
  B_pos2 posC;
  B_pos posA;
  B_eff effM;
  B_delay ret;
  Float_t max=0;

  Int_t start_scan=start,stop_scan;
  Short_t q;
  Float_t thr;
  Int_t ntrigs;

  TFile *fileburst;
  Char_t name[500];
  TH1F *burst0,*burst1;
  TNtuple *config;
  Float_t *data;
  Int_t nentries;
  if(start==stop){  // B_TriggerWinDelay was run with parameter eff=0
    sprintf(name,"%sbtrun%d_%d.root",ST_DATA_PATH,run,start);
    fileburst=new TFile(name,"READ");  // open requested data file
    if(!fileburst->IsOpen()){
      printf("B_TriggerWinDelayPlot: File %s can not be opened !\n",name);
      ret.trig=ret.win=0;
      return ret;
    }
    do{
      /* WARNING: elder versions of sctdaq software use for burst histrogram of link 0 name "h_burst0;%d" instead
                  of "h_burst0_m0;%d" - retype it here if necessary (look into your data file to see if change is needed)
      */
      sprintf(name,"h_burst0_m0;%d",i);  // read burst histogram of link 0 from data file
      burst0=(TH1F*)(fileburst->Get(name));
      /* WARNING: elder versions of sctdaq software use for burst histrogram of link 1 name "h_burst1;%d" instead
                  of "h_burst1_m0;%d" - retype it here if necessary (look into your data file to see if change is needed)
      */
      sprintf(name,"h_burst1_m0;%d",i);  // read burst histogram of link 1 from data file
      burst1=(TH1F*)(fileburst->Get(name));
      sprintf(name,"configs;%d",i);      // read configuration from data file
      config=(TNtuple*)(fileburst->Get(name));
      if(config==NULL) break;            // there isn't any next burst
      nentries=config->GetEntries();
      b_mean_configs.Set(nentries);
      b_mean_configs.Reset();
      b_mc=b_mean_configs.GetArray();
      for(j=0;j<nentries;j++){
        config->GetEntry(j);
        data=config->GetArgs();
        for(k=0;k<12;k++) b_mc[j]+=data[k];
        b_mc[j]/=12;
      }
      if(b_mc[BT_WIN_LENGTH]!=0){  // external trigger window was enabled => read number of hits
        sprintf(b_mcname,"%s",config->GetTitle());
        counts[c]=burst0->Integral()+burst1->Integral();
        printf("Trigger delay:  %.0f ,\tWindow delay:  %.0f ,\tNumber of hits:  %.0f\n",
               b_mc[ST_TRIG_DELAY],b_mc[BT_WIN_REAL_DEL],counts[c]);
        delays[c]=b_mc[ST_TRIG_DELAY]*B_CLK-b_mc[BT_WIN_REAL_DEL];
        if(counts[c]>max){  // find trigger delay and external window delay with the highest number of hits
          ret.trig=b_mc[ST_TRIG_DELAY];
          ret.win.real=(Short_t)b_mc[BT_WIN_REAL_DEL];
          ret.win.orig=(Short_t)b_mc[BT_WIN_ORIG_DEL];
          max=counts[c];
        }
        ntrigs=(Int_t)b_mc[BT_NSENT];
        if(b_mc[ST_QTHR]!=-1){
          thr=b_mc[ST_QTHR];
          q=1;
        }else{
          thr=b_mc[ST_VTHR];
          q=0;
        }
        c++;
      }
      i++;
    }while(config!=NULL);  // there isn't any next burst
    fileburst->Close();
    delete fileburst;

  }else{  // B_TriggerWinDelay was run with parameter eff=1
    for(i=start;i<=stop;i++){
      if(!B_DataFile(run,i)) continue;  // data file is corrupted or doesn't exist
      if(b_mc[BT_WIN_LENGTH]!=0){  // external trigger window was enabled => read efficiency (number of hits)
        if(!c) start_scan=i;
        stop_scan=i;
        ntrigs=(Int_t)b_mc[BT_NSENT];
        if(b_mc[ST_QTHR]!=-1){
          thr=b_mc[ST_QTHR];
          q=1;
        }else{
          thr=b_mc[ST_VTHR];
          q=0;
        }
        posC=B_RunCrossPosition(run,i);
        posA=B_RunAlongPosition(run,i);
        effM=B_RunEfficiency(posC,posA,run,i);
        counts[c]=effM.n01;
        effs[c]=effM.eff;
        erreffs[c]=effM.err;
        delays[c]=b_mc[ST_TRIG_DELAY]*B_CLK-b_mc[BT_WIN_REAL_DEL];
        if(counts[c]>max){  // find trigger delay and external window delay with the highest efficiency (number of hits)
          ret.trig=b_mc[ST_TRIG_DELAY];
          ret.win.real=(Short_t)b_mc[BT_WIN_REAL_DEL];
          ret.win.orig=(Short_t)b_mc[BT_WIN_ORIG_DEL];
          max=counts[c];
        }
        c++;
      }
    }
  }

  TCanvas *ctime=new TCanvas("B_ctime","BT Timing properties fit",1000,750);
  ctime->cd();
  TPad *ptime=new TPad("B_ptime","Timing properties graphs",0.,0.,1.,.85);
  ptime->Draw();
  TPad *ptimeinfo=new TPad("B_ptimeinfo","Timing properties information",0.,.85,1.,1.);
  ptimeinfo->Draw();
  ptimeinfo->cd();

  TPaveText *text=new TPaveText(0.0,0.0,1.0,1.0);  // text information about this scan
  text->Draw();
  text->SetTextSize(0.17);
  text->SetTextColor(kBlue);
  text->SetTextAlign(1*10+2);
  if(start==stop) sprintf(name,"Module %s    Run: %d    Scan: %d    ",b_mcname,run,start_scan);
  else sprintf(name,"Module %s    Run: %d    Scans: %d ... %d    ",b_mcname,run,start_scan,stop_scan);
  if(b_new_qthr_corr==-1) sprintf(name,"%sThreshold: %.2f %c%c (correction %.3f)",name,thr*b_mc[BT_QTHR_CORR],'m'+q*('f'-'m'),'V'+q*('C'-'V'),b_mc[BT_QTHR_CORR]);
  else sprintf(name,"%sThreshold: %.2f %c%c (correction %.3f)",name,thr*b_new_qthr_corr,'m'+q*('f'-'m'),'V'+q*('C'-'V'),b_new_qthr_corr);
  text->AddText(name);
  sprintf(name,"No. Of External Triggers: %d    Max. No. Of Hits: %d",ntrigs,max);
  text->AddText(name);
  sprintf(name,"Scan Range (TRIGGER DELAY,WINDOW DELAY):  %d , %d  ...  %d , %d",
               floor((delays[0]+B_WIN_MAX)/B_CLK),B_CLK*floor((delays[0]+B_WIN_MAX)/B_CLK)-delays[0],
               floor((delays[c-1]+B_WIN_MAX)/B_CLK),B_CLK*floor((delays[c-1]+B_WIN_MAX)/B_CLK)-delays[c-1]);
  text->AddText(name);
  sprintf(name,"Max. No. Of Hits Found At Trigger Delay = %d , Window Delay = %d (= %d)",ret.trig,ret.win.real,ret.win.orig);
  text->AddText(name);
  text->AddText("  ");

  TGraphErrors *gtimeeff;
  if(start!=stop){  // draw graph of efficiency vs. delay if B_TriggerWinDelay was run with parameter eff=1
    ptime->Divide(1,2);
    ptime->Draw();
    ptime->cd(2);
    gtimeeff=new TGraphErrors(c,delays,effs,0,erreffs);
    gtimeeff->SetName("B_gtimeeff");
    gtimeeff->SetTitle("Efficiency");
    gtimeeff->SetMarkerStyle(5);
    gtimeeff->SetMarkerColor(kRed);
    gtimeeff->SetLineColor(kBlack);
    gtimeeff->Draw("PA");
    gtimeeff->GetXaxis()->SetTitle("Delay [ns]");
    gtimeeff->GetYaxis()->SetTitle("Efficiency [%]");
    gtimeeff->Draw("PA");
    ptime->cd(1);
  }else
    ptime->cd();

  TGraph *ggauss=new TGraph(c,delays,counts);  // draw graph of number of hits vs. delay
  ggauss->SetName("B_ggauss");
  if(start!=stop) ggauss->SetTitle("Number of efficient hits");
  else ggauss->SetTitle("Number of hits");
  ggauss->SetMarkerStyle(5);
  ggauss->SetMarkerColor(kRed);
  ggauss->Draw("ALP");
  ggauss->GetXaxis()->SetTitle("Delay [ns]");
  ggauss->GetYaxis()->SetTitle("No. of hits");
  ggauss->Draw("LP");

  if(start!=stop) B_ScanPlot(BT_DELAY,start_scan,stop_scan,run);  // draw scan analysis - see below B_ScanPlot

  printf("\nMax. no. of (efficient) hits found at TRIG. DELAY %.0f and WIN. DELAY %.0f (~ %.0f)\n",ret.trig,ret.win.real,ret.win.orig);
  return ret;  // return trigger delay and external window delay with the highest number of hits (efficiency)
}

#define B_SAVE_RUNS_PS_RESULTS    1   // save canvas as postscript of analysis of every burst
#define B_SAVE_RUNS_CXX_RESULTS   2   // save canvas as macro of canvas of analysis of every burst
#define B_SAVE_PS_RESULTS         4   // save canvas as postscript of analysis of the scan
#define B_SAVE_CXX_RESULTS        8   // save canvas as macro of analysis of the scan
#define B_SAVE_TXT_RESULTS        16  // save results of the scan into a text file
Short_t b_common_effhits_window=1;    // if 1 then uses common beam region for all bursts as intersection of beam regions
                                      // calculated for every burst
                                      // if 0 then uses for analysis of every burst it's own calculated beam region
Short_t b_scan_save=B_SAVE_PS_RESULTS+B_SAVE_TXT_RESULTS;  // change this variable to enable/disable postcripts, macros and
                                                           // text files creation - see above

void B_ScanPlot(Int_t vartype,Int_t start_scannum=1,Int_t stop_scannum=-1,Int_t runnum=-1,Int_t *scan_list=NULL,Int_t *run_list=NULL,Int_t n_list=0){
// performs analysis of scan - see for example B_Scan macro in BT.cpp
/* EXAMPLES:
   B_ScanPlot(41,1,10,54); - show scan analysis where scanned variable was threshold [fC] (41 - see sct_scans.h file), data
                             files are: sctvar/data/btrun54_1.root, .. , sctvar/data/btrun54_10.root
   B_ScanPlot(41,1); - show scan analysis where scanned variable was threshold [fC] (41 - see sct_scans.h file), data files
                       are: sctvar/data/btrun'b_runnum'_1.root, .. , sctvar/data/btrun'b_runnum'_'b_scannum'.root
   Int_t s[3]={2,3,1};Int_t r[3]={65,65,66};B_ScanPlot(41,-1,-1,-1,s,r,3);
                - show scan analysis where scanned variable was threshold [fC] (41 - see sct_scans.h file), data files are:
                  sctvar/data/btrun65_2.root, sctvar/data/btrun65_3.root, sctvar/data/btrun66_1.root
*/
  const Int_t start=start_scannum;
  if(stop_scannum==-1) const Int_t stop=b_scannum;
  else const Int_t stop=stop_scannum;
  if(runnum==-1) const Int_t run=b_runnum;
  else const Int_t run=runnum;
  if(n_list) const Int_t nlist=n_list;
  else const Int_t nlist=stop-start+1;
  
  B_analysis anal;
  Float_t var[nlist],allhits[2][nlist],beamhits[2][nlist],effhits[2][nlist],nonzero[2][nlist],ntrigs[nlist];
  Float_t eff[2][nlist],mult[2][nlist],clust[2][nlist],clust_scaled[2][nlist],effabs[2][nlist];
  Float_t eeff[2][nlist],emult[2][nlist],eclust[2][nlist],eclust_scaled[2][nlist],eeffabs[2][nlist];
  Char_t name[500],varname[500];
  TGraphErrors *geffsabs[2],*geffs[2],*gmult[2],*gclust[2],*gbeamn[2],*geffn[2],*galln[2],*gnonzeron[2],*gntrigs[2];
  TH2F *hscancorr=NULL,*hscanbeam=NULL,*hfound2;
  TH2F *hscanhit[2]={NULL,NULL};
  TH1F *hfound1;
  Int_t i,j,start_date,stop_date,start_time,stop_time;
  TDatime *dt=new TDatime();

  B_pos2 posC,tmpC;
  B_pos posA,tmpA;
  Float_t left,right;
  if(b_common_effhits_window){  // should be used common beam region ? - see above variable 'b_common_effhits_window'
    for(i=0;i<nlist;i++){
      if(n_list){
        B_DataFile(run_list[i],scan_list[i]);
        tmpC=B_RunCrossPosition(run_list[i],scan_list[i]);
        tmpA=B_RunAlongPosition(run_list[i],scan_list[i]);
      }else{
        B_DataFile(run,start+i);
        tmpC=B_RunCrossPosition(run,start+i);
        tmpA=B_RunAlongPosition(run,start+i);
      }
      if(!i){
        posC=tmpC;
        posA=tmpA;
      }else{  // calculate intersection of beam regions of all bursts
        left=tmpA.mean-tmpA.width;
        if(posA.mean-posA.width>left) left=posA.mean-posA.width;
        right=tmpA.mean+tmpA.width;
        if(posA.mean+posA.width<right) right=posA.mean+posA.width;
        posA.mean=(right+left)/2.;
        posA.width=(right-left)/2.;
        left=tmpC.mean[0]-tmpC.width[0];
        if(posC.mean[0]-posC.width[0]>left) left=posC.mean[0]-posC.width[0];
        right=tmpC.mean[0]+tmpC.width[0];
        if(posC.mean[0]+posC.width[0]<right) right=posC.mean[0]+posC.width[0];
        posC.mean[0]=(right+left)/2.;
        posC.width[0]=(right-left)/2.;
        left=tmpC.mean[1]-tmpC.width[1];
        if(posC.mean[1]-posC.width[1]>left) left=posC.mean[1]-posC.width[1];
        right=tmpC.mean[1]+tmpC.width[1];
        if(posC.mean[1]+posC.width[1]<right) right=posC.mean[1]+posC.width[1];
        posC.mean[1]=(right+left)/2.;
        posC.width[1]=(right-left)/2.;
      }
      printf("Current beam cross strip position link0: channel %.1f , width %.1f\n",posC.mean[0],posC.width[0]);
      printf("Current beam cross strip position link1: channel %.1f , width %.1f\n",posC.mean[1],posC.width[1]);
      printf("Current beam along strip position: channel %.1f , width %.1f\n",posA.mean,posA.width);
      if(tmpC.width[0]!=0) posC.sigma[0]=tmpC.sigma[0]/tmpC.width[0]*tmpC.width[0];
      else posC.sigma[0]=0;
      if(tmpC.width[1]!=0) posC.sigma[1]=tmpC.sigma[1]/tmpC.width[1]*tmpC.width[1];
      else posC.sigma[1]=0;
      if(tmpA.width!=0) posA.sigma=tmpA.sigma/tmpA.width*tmpA.width;
      else posA.sigma=0;
    }
  }

  hscanhit[0]=new TH2F("B_hscanhit0","Burst scan link 0",768,-0.5,767.5,nlist,-0.5,nlist-0.5);
  hscanhit[1]=new TH2F("B_hscanhit1","Burst scan link 1",768,-0.5,767.5,nlist,-0.5,nlist-0.5);
  for(i=0;i<nlist;i++){
    if(b_common_effhits_window){
      if(n_list) anal=B_RunPlot(run_list[i],scan_list[i],NULL,&posC,&posA);
      else anal=B_RunPlot(run,start+i,NULL,&posC,&posA);
    }else{
      if(n_list) anal=B_RunPlot(run_list[i],scan_list[i]);
      else anal=B_RunPlot(run,start+i);
    }
    if(!i){
      start_date=(Int_t)b_mc[BT_DATE];
      start_time=(Int_t)b_mc[BT_TIME];
    }
    if(i==nlist-1){
      stop_date=(Int_t)b_mc[BT_DATE];
      stop_time=(Int_t)b_mc[BT_TIME];
    }
    eff[0][i]=anal.eff.eff10;   // efficiency of link 0 (link 1 is reference - see B_RunEfficiency in BT_Anal.cpp)
    eeff[0][i]=anal.eff.err10;  // error on efficiency of link 0
    eff[1][i]=anal.eff.eff01;   // efficiency of link 1 (link 0 is reference - see B_RunEfficiency in BT_Anal.cpp)
    eeff[1][i]=anal.eff.err01;  // error on efficiency of link 1
    effabs[0][i]=anal.eff.n0/b_mc[BT_NSENT];         // number of in-beam-region hits / number of external triggers - link 0
    eeffabs[0][i]=sqrt(anal.eff.n0+b_mc[BT_NSENT]);  // error on this type of efficency - link 0
    effabs[1][i]=anal.eff.n1/b_mc[BT_NSENT];         // number of in-beam-region hits / number of external triggers - link 1
    eeffabs[1][i]=sqrt(anal.eff.n1+b_mc[BT_NSENT]);  // error on this type of efficency - link 1
    beamhits[0][i]=anal.eff.n0;  // number of events containing hit in beam region - link 0
    beamhits[1][i]=anal.eff.n1;  // number of events containing hit in beam region - link 1
    effhits[0][i]=anal.eff.n01;  // number of events containing neighbouring hits in beam region on both links - link 0
    effhits[1][i]=anal.eff.n01;  // number of events containing neighbouring hits in beam region on both links - link 1
    mult[0][i]=anal.mult.mult[0];    // mean multpilicity on link 0
    emult[0][i]=anal.mult.sigma[0];  // error on mean multiplicity on link 0
    mult[1][i]=anal.mult.mult[1];    // mean multpilicity on link 1
    emult[1][i]=anal.mult.sigma[1];  // error on mean multiplicity on link 1
    clust[0][i]=anal.clust.clust[0];   // mean cluster size of in-beam-region hits on link 0
    eclust[0][i]=anal.clust.sigma[0];  // error on mean cluster size of in-beam-region hits on link 0
    clust[1][i]=anal.clust.clust[1];   // mean cluster size of in-beam-region hits on link 1
    eclust[1][i]=anal.clust.sigma[1];  // error on mean cluster of in-beam-region hits size on link 1
    allhits[0][i]=anal.mult.n[0];  // number of all hits (clusters) on link 0
    allhits[1][i]=anal.mult.n[1];  // number of all hits (clusters) on link 1
    nonzero[0][i]=b_mc[BT_NSENT]-anal.mult.nz[0];  // number of nonzero events on link 0
    nonzero[1][i]=b_mc[BT_NSENT]-anal.mult.nz[1];  // number of nonzero events on link 1
    ntrigs[i]=b_mc[BT_NSENT];  // number of external triggers
    var[i]=b_mc[vartype];  // value of scanned variable
    if(vartype==ST_QTHR){  // do correction to charge injectors is scanned variable is threshold [fC] (qthr)
      if(b_new_qthr_corr==-1) var[i]*=b_mc[BT_QTHR_CORR];  // use correction from module-config. file during measurement
      else var[i]*=b_new_qthr_corr;                        // use correction by 'b_new_qthr_corr' variable - see BT.cpp
    }
    hfound1=(TH1F*)UnderCanvas("B_hhit0",B_crun);  // connect burst histogram of link 0 - see above B_RunPlot
    for(j=0;j<768;j++) hscanhit[0]->SetBinContent(j+1,i+1,hfound1->GetBinContent(j+1));
    hfound1=(TH1F*)UnderCanvas("B_hhit1",B_crun);  // connect burst histogram of link 1 - see above B_RunPlot
    for(j=0;j<768;j++) hscanhit[1]->SetBinContent(j+1,i+1,hfound1->GetBinContent(j+1));
    hfound2=(TH2F*)UnderCanvas("B_hcorr",B_crun);  // connect correlations histogram - see above B_RunPlot
    if(hscancorr!=NULL) hscancorr->Add(hfound2);
    else hscancorr=new TH2F(*hfound2);
    hfound2=(TH2F*)UnderCanvas("B_hbeam",B_crun);  // connect beam profile histogram - see above B_RunPlot
    if(hscanbeam!=NULL) hscanbeam->Add(hfound2);
    else hscanbeam=new TH2F(*hfound2);
    if(b_scan_save&B_SAVE_RUNS_PS_RESULTS){  // save postscript of canvas of burst analysis
      if(n_list) sprintf(name,"%sps/btrun%d_%d_datime%d_%d.ps",ST_VAR,run_list[i],scan_list[i],dt->GetDate(),dt->GetTime());
      else sprintf(name,"%sps/btrun%d_%d_datime%d_%d.ps",ST_VAR,run,start+i,dt->GetDate(),dt->GetTime());
      B_crun->SaveAs(name);
    }
    if(b_scan_save&B_SAVE_RUNS_CXX_RESULTS){  // save macro of canvas of burst analysis
      if(n_list) sprintf(name,"%smacros/btrun%d_%d_datime%d_%d.cxx",ST_VAR,run_list[i],scan_list[i],dt->GetDate(),dt->GetTime());
      else sprintf(name,"%smacros/btrun%d_%d_datime%d_%d.cxx",ST_VAR,run,start+i,dt->GetDate(),dt->GetTime());
      B_crun->SaveAs(name);
    }
  }
  if(b_online) e->TranslateVariableName(vartype,varname);  // find name of scanned variable ('b_online' variable must be 1)
  else sprintf(varname,"? (OFFLINE)");

  TCanvas *cscan=new TCanvas("B_cscan","BT Scan Display",1100,850);
  cscan->cd();
  TPad *pscan=new TPad("B_pscan","Scan graphs",0.,0.,1.,.75);
  pscan->Divide(2,3);
  pscan->Draw();
  TPad *pscaninfo=new TPad("B_pscaninfo","Scan Information",0.,.75,1.,1.);
  pscaninfo->Divide(5,1);
  pscaninfo->Draw();
  pscaninfo->cd(1);

  TPaveText *itext=new TPaveText(0.0,0.0,1.0,1.0);
  itext->Draw();
  itext->SetTextSize(0.08);
  itext->SetTextColor(kBlue);
  itext->SetTextAlign(1*10+2);
  sprintf(name,"Module: %s",b_mcname);
  itext->AddText(name);
  sprintf(name,"Scanned: %d = %s",vartype,varname);
  itext->AddText(name);
  sprintf(name,"No. Of Points: %d",nlist);
  itext->AddText(name);
  sprintf(name,"No. Of Ext. Trigs.: %.0f",b_mc[BT_NSENT]);
  itext->AddText(name);
  sprintf(name,"Start Date: %02d/%02d/%02d",start_date%100,(start_date/100)%100,start_date/100/100+B_CENTURY);
  itext->AddText(name);
  sprintf(name,"Start Time: %02d:%02d:%02d",start_time/100/100,(start_time/100)%100,start_time%100);
  itext->AddText(name);
  sprintf(name,"Stop Date: %02d/%02d/%02d",stop_date%100,(stop_date/100)%100,stop_date/100/100+B_CENTURY);
  itext->AddText(name);
  sprintf(name,"Stop Time: %02d:%02d:%02d",stop_time/100/100,(stop_time/100)%100,stop_time%100);
  itext->AddText(name);
  itext->AddText("  ");

  pscaninfo->cd(2);
  hscancorr->SetTitle("Correlations");
  hscancorr->SetName("B_hscancorr");
  hscancorr->SetStats(kFALSE);
  hscancorr->GetXaxis()->SetTitle("Channel no. link 0");
  hscancorr->GetYaxis()->SetTitle("Channel no. link 1");
  hscancorr->Draw("colz");

  pscaninfo->cd(3);
  hscanbeam->SetTitle("Beam profile");
  hscanbeam->SetName("B_hscanbeam");
  hscanbeam->SetStats(kFALSE);
  hscanbeam->GetXaxis()->SetTitle("Cross strips position [mm]");
  hscanbeam->GetYaxis()->SetTitle("Along strips position [mm]");
  hscanbeam->Draw("colz");

  pscaninfo->Modified();
  pscaninfo->Update();

  Float_t multmax=0,clustmax=0;  // for secondary y-axis of multiplicity/cluster-size graphs
  TGaxis *aclust[2];

  for(i=0;i<2;i++){

    pscan->cd(1+i);
    geffs[i]=new TGraphErrors(nlist,var,eff[i],0,eeff[i]);
    sprintf(name,"B_geffs%d",i);
    geffs[i]->SetName(name);
    sprintf(name,"Efficiency (RED) In beam region hits (BLUE) link %d",i);
    geffs[i]->SetTitle(name);
    geffs[i]->SetMarkerStyle(5);
    geffs[i]->SetMarkerColor(kRed);
    geffs[i]->SetLineColor(kBlack);
    geffs[i]->Draw("AP");
    sprintf(name,"%s: %d",varname,vartype);
    geffs[i]->GetXaxis()->SetTitle(name);
    geffs[i]->GetYaxis()->SetTitle("Efficiency [%]");
    geffs[i]->Draw("AP");

    if(vartype==ST_QTHR || vartype==ST_VTHR){
      // TODO: fit landau => median charge, sigma
    }
    if(vartype==BT_DELAY){
      // TODO: fit gauss (pulse shape), mean (trig.delay,win.delay), sigma
    }

    geffsabs[i]=new TGraphErrors(nlist,var,effabs[i],0,eeffabs[i]);
    sprintf(name,"B_geffsabs%d",i);
    geffsabs[i]->SetName(name);
    geffsabs[i]->SetMarkerStyle(2);
    geffsabs[i]->SetMarkerColor(kBlue);
    geffsabs[i]->SetLineColor(kBlack);
    geffsabs[i]->Draw("P");

    pscan->cd(3+i);

    for(j=0;j<nlist;j++){
      if(mult[i][multmax]<mult[i][j]) multmax=j;
      if(clust[i][multmax]<clust[i][j]) clustmax=j;
    }
    gmult[i]=new TGraphErrors(nlist,var,mult[i],0,emult[i]);
    sprintf(name,"B_gmult%d",i);
    gmult[i]->SetName(name);
    sprintf(name,"Mean Multiplicity (RED) > Cluster size (BLUE) link %d",i);
    gmult[i]->SetTitle(name);
    gmult[i]->SetMarkerStyle(5);
    gmult[i]->SetMarkerColor(kRed);
    gmult[i]->SetLineColor(kBlack);
    gmult[i]->Draw("AP");
    sprintf(name,"%s: %d",varname,vartype);
    gmult[i]->GetXaxis()->SetTitle(name);
    gmult[i]->GetYaxis()->SetTitle("Multiplicity");
    gmult[i]->GetYaxis()->SetRange(0,1.1*mult[i][multmax]+1);
    gmult[i]->Draw("AP");

    if(clust[i][clustmax]!=0){
     for(j=0;j<nlist;j++){  // draw secondary y-axis for cluster size graphs
        if(mult[i][multmax]!=0) clust_scaled[i][j]=clust[i][j]*mult[i][multmax]/clust[i][clustmax];
        else clust_scaled[i][j]=clust[i][j]/clust[i][clustmax];
        if(mult[i][multmax]!=0) eclust_scaled[i][j]=eclust[i][j]*mult[i][multmax]/clust[i][clustmax];
        else eclust_scaled[i][j]=eclust[i][j]/clust[i][clustmax];
      }
      gclust[i]=new TGraphErrors(nlist,var,clust_scaled[i],0,eclust_scaled[i]);
      if(mult[multmax]!=0) aclust[i]=new TGaxis(gPad->GetUxmax(),gPad->GetUymin(),gPad->GetUxmax(),gPad->GetUymax(),
                                                0,(1.1*mult[i][multmax]+1)*clust[i][clustmax]/mult[i][multmax],510,"+L");
      else aclust[i]=new TGaxis(gPad->GetUxmax(),gPad->GetUymin(),gPad->GetUxmax(),gPad->GetUymax(),0,clust[i][clustmax],510,"+L");
      aclust[i]->SetTitle("Cluster size");
      aclust[i]->SetLineColor(kBlue);
      aclust[i]->SetTextColor(kBlue);
      aclust[i]->Draw();
    }else gclust[i]=new TGraphErrors(nlist,var,clust[i],0,eclust[i]);
    sprintf(name,"B_gclust%d",i);
    gclust[i]->SetName(name);
    gclust[i]->SetMarkerStyle(5);
    gclust[i]->SetMarkerColor(kBlue);
    gclust[i]->SetLineColor(kBlack);
    gclust[i]->Draw("P");

    pscan->cd(5+i);

    galln[i]=new TGraphErrors(nlist,var,allhits[i]);
    sprintf(name,"B_galln%d",i);
    galln[i]->SetName(name);
    sprintf(name,"Link %d: No.Hits(RED) No.Trigs(YELLOW) Non-Zero(CYAN) In-Beam(GREEN) Efficient(BLUE)",i);
    galln[i]->SetTitle(name);
    galln[i]->SetMarkerStyle(5);
    galln[i]->SetMarkerColor(kRed);
    galln[i]->SetLineColor(kRed);
    galln[i]->Draw("ALP");
    sprintf(name,"%s: %d",varname,vartype);
    galln[i]->GetXaxis()->SetTitle(name);
    galln[i]->GetYaxis()->SetTitle("Number of hits");
    galln[i]->Draw("ALP");

    gntrigs[i]=new TGraphErrors(nlist,var,ntrigs);
    sprintf(name,"B_gntrigs%d",i);
    gntrigs[i]->SetName(name);
    gntrigs[i]->SetMarkerStyle(25);
    gntrigs[i]->SetMarkerColor(kYellow);
    gntrigs[i]->SetLineColor(kYellow);
    gntrigs[i]->Draw("LP");

    gnonzeron[i]=new TGraphErrors(nlist,var,nonzero[i]);
    sprintf(name,"B_nonzeron%d",i);
    gnonzeron[i]->SetName(name);
    gnonzeron[i]->SetMarkerStyle(2);
    gnonzeron[i]->SetMarkerColor(kCyan);
    gnonzeron[i]->SetLineColor(kCyan);
    gnonzeron[i]->Draw("LP");

    geffn[i]=new TGraphErrors(nlist,var,effhits[i]);
    sprintf(name,"B_effn%d",i);
    geffn[i]->SetName(name);
    geffn[i]->SetMarkerStyle(3);
    geffn[i]->SetMarkerColor(kBlue);
    geffn[i]->SetLineColor(kBlue);
    geffn[i]->Draw("LP");

    gbeamn[i]=new TGraphErrors(nlist,var,beamhits[i]);
    sprintf(name,"B_beamn%d",i);
    gbeamn[i]->SetName(name);
    gbeamn[i]->SetMarkerStyle(4);
    gbeamn[i]->SetMarkerColor(kGreen);
    gbeamn[i]->SetLineColor(kGreen);
    gbeamn[i]->Draw("LP");

    pscan->Modified();
    pscan->Update();

    pscaninfo->cd(4+i);
    hscanhit[i]->Draw("colz");  // analogy of scan display in sctdaq software
    pscaninfo->Modified();
    pscaninfo->Update();
  }

  cscan->Modified();
  cscan->Update();

  if(b_scan_save&B_SAVE_PS_RESULTS){  // save postscript of canvas of scan analysis
    if(n_list) sprintf(name,"%sps/btscan%d_datime%d_%d.ps",ST_VAR,nlist,dt->GetDate(),dt->GetTime());
    else sprintf(name,"%sps/btrun%d_datime%d_%d.ps",ST_VAR,nlist,dt->GetDate(),dt->GetTime());
    cscan->SaveAs(name);
  }
  if(b_scan_save&B_SAVE_CXX_RESULTS){  // save macro of canvas of burst analysis
    if(n_list) sprintf(name,"%smacros/btscan%d_datime%d_%d.cxx",ST_VAR,nlist,dt->GetDate(),dt->GetTime());
    else sprintf(name,"%smacros/btrun%d_datime%d_%d.cxx",ST_VAR,nlist,dt->GetDate(),dt->GetTime());
    cscan->SaveAs(name);
  }
  FILE *txtout;
  if(b_scan_save&B_SAVE_TXT_RESULTS){  // save text output of scan analysis
    if((txtout=fopen(b_resname,"at"))==NULL){
      printf("B_ScanPlot: File %s can not be opened !\n",b_resname);
    }else{
      fprintf(txtout,"# Scan Plot: %d points\n",nlist);
      fprintf(txtout,"# Module %s (0) , scanned variable %s (%d) , Number of external triggers %.0f\n",b_mcname,varname,vartype,b_mc[BT_NSENT]);
      if(b_common_effhits_window)
        fprintf(txtout,"# Beam position: link0: %d ... %d , link1: %d ... %d , link1+link0-767: %d .. %d , used mean + - width\n",
                       posC.mean[0]-posC.width[0],posC.mean[0]+posC.width[0],posC.mean[1]-posC.width[1],posC.mean[1]+posC.width[1],
                       posA.mean-posA.width,posA.mean+posA.width,b_cut);
      fprintf(txtout,"# Notes: Index [0],[1] => link 0,1 , Format => value(error)\n");
      fprintf(txtout,"# Run \t Scan \t %s (%d) \t Efficiency [0] \t Efficiency [1] \t Cluster size [0] \t Cluster size [1] \t Multiplicity [0] \t Multiplicity [1]",varname,vartype);
      fprintf(txtout," \t No. of triggers \t No. of hits [0] \t No. of hits [1] \t No. of non-zero events [0] \t No. of non-zero events [1]");
      fprintf(txtout," \t No. of efficient events [0] \t No. of efficient events [1] \t No. of hits in beam region [0] \t No. of hits in beam region [1]\n");
      for(i=0;i<nlist;i++){
        if(n_list) fprintf(txtout,"%d \t %d \t ",run_list[i],scan_list[i]);
        else fprintf(txtout,"%d \t %d \t ",run,start+i);
        fprintf(txtout,"%f \t %.4f (%.4f) \t %.4f (%.4f) \t %.6f (%.6f) \t %.6f (%.6f) \t %.6f (%.6f) \t %.6f (%.6f) \t ",
                       var[i],eff[0][i],eeff[0][i],eff[1][i],eeff[1][i],clust[0][i],eclust[0][i],clust[1][i],eclust[1][i],mult[0][i],emult[0][i],mult[1][i],emult[1][i]);
        fprintf(txtout,"%d \t %d \t %d \t %d \t %d \t %d \t %d\n",
                       ntrigs[i],allhits[0][i],allhits[1][i],nonzero[0][i],nonzero[1][i],
                       effhits[0][i],effhits[1][i],beamhits[0][i],beamhits[1][i]);
      }
      fprintf(txtout,"# End of Scan Plot\n");
      fclose(txtout);
    }
  }
}

void B_TriggerRatePlot(Float_t *trigs,Float_t *freqs,Int_t nlist){
// simple plot of trigger rate vs. number of triggers during burst - see B_TriggerRate in BT.cpp
  TCanvas *cfreq=new TCanvas("B_cfreq","Frequency of triggers during burst",700,600);
  TGraph *gfreq=new TGraph(nlist,trigs,freqs);
  gfreq->SetTitle("Frequency of triggers during burst");
  gfreq->SetName("B_gfreq");
  gfreq->SetMarkerStyle(5);
  gfreq->SetMarkerColor(kBlack);
  gfreq->SetLineColor(kRed);
  gfreq->Draw("ALP");
  gfreq->GetXaxis()->SetTitle("Number of triggers");
  gfreq->GetYaxis()->SetTitle("Frequncy of triggers");
  gfreq->Draw("ALP");
}