Int_t AccumBursts(Int_t Nbursts){

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t i,j,n,ntrig,ntottrig=0;
  char s[128],fname[128],mname[128];
  Bool_t nogood;
  
  TGraph* gr;
  TPaveText *t1,*t2;

  // some settings for the records

  e->scan_running =1;
  e->scannum++;

  e->scan_type=999;
  e->scan_start=0;
  e->scan_stop=1;
  e->scan_step=1;

  if(Nbursts<=0){
    printf("AccumBursts:Nbursts = %d, but must be 1 at least!\n",Nbursts);
    return;
  }

  // run requested number of bursts
  for(i=0;i<Nbursts && st_do_stop()==0;i++){
    e->ExecuteBurst();
    st_display_burst();
    printf("\nFinished burst %d out of %d\n\n",i+1,Nbursts);

    // get individual burst histos accumulated
    for (n=0; n<e->num_showmods; n++){
      if(e->m[n]->present){
	if(i==0){
	  e->m[n]->h_accbst0->Reset();
	  e->m[n]->h_accbst1->Reset();
	}
	e->m[n]->h_accbst0->Add(e->m[n]->h_burst0);
	e->m[n]->h_accbst1->Add(e->m[n]->h_burst1);
      }
    }
    ntrig=0;
    for(n=0;n<e->nMuSTARDs;n++){
      if(e->evcnt[n]>ntrig) 
	ntrig=e->evcnt[n];
    }
    if(  e->burst_type == 100){
      ntrig = (ntrig*(e->L1A_per_loop-e->throw_away))/e->L1A_per_loop;
      if(e->L1A_per_loop>1) printf("Actually read triggers: %d\n",ntrig);
    }
    ntottrig+=ntrig;
  }

  e->scan_running =0;
  if(e->abort) return;

  // save data

  st_save_scan(0,1);
  sprintf(fname,"%sstrun%d_%d.root",DATA_DIRECTORY,e->runnum,e->scannum);
  TFile *out = new TFile(fname,"UPDATE");
  sprintf(s,"%d",ntottrig);
  TObjString *nts = new TObjString(s);
  TStopwatch clock;
  sprintf(s,"%f",clock.GetRealTime()/3600.);
  TObjString *tis = new TObjString(s);
  nts->Write("Ntrig");
  tis->Write("Clock");
  out->Close();

  // show accumulated results
  char ctitle[128],t[128];
  Int_t col=17;
  Int_t n,npan,ipan,nmodppan;
  TCanvas *pacbst[4];
  TPad *pacbst_hist[4], *pacbst_info[4], *pacbst_text[4];

  nmodppan=e->num_panelmods;
  npan = (e->num_showmods-1)/nmodppan+1;
  if(npan>4) npan = 4;

  for(ipan=0;ipan<npan;ipan++){
    sprintf(t,"pacbst%d",ipan);
    pacbst[ipan] = (TCanvas*) gROOT->FindObject(t);
    if(pacbst[ipan]==0){
      st_styles();
      if(npan==1)
        sprintf(ctitle,"Accum. Bursts Display  Run %d", e->runnum);
      else 
        sprintf(ctitle,"Accum. Bursts Display  Run %d Modules %d-%d", 
                       e->runnum,ipan*nmodppan,(ipan+1)*nmodppan-1);
      pacbst[ipan] = new TCanvas(t,ctitle,1100,1000);
      pacbst[ipan]->SetFillColor(18);
    }
    pacbst[ipan]->cd();
    
    sprintf(t,"pacbst_info%d",ipan);
    pacbst_info[ipan] = (TPad*) gROOT->FindObject(t);
    if(pacbst_info[ipan]==0){
      pacbst[ipan]->cd();
      pacbst_info[ipan] = new TPad(t,"Acc. Bursts Info Pad",0.0,0.9,1.0,1.0,col);
      pacbst_info[ipan]->Draw();
      pacbst_info[ipan]->cd();
      pacbst_info[ipan]->Divide(2,1,0,0);
    }

    sprintf(t,"pacbst_histo%d",ipan);
    pacbst_hist[ipan] = (TPad*) gROOT->FindObject(t);
    if(pacbst_hist[ipan]==0){
      pacbst[ipan]->cd();
      pacbst_hist[ipan] = new TPad(t,"Acc. Bursts Histogram Pad", 0.0,0.0,0.9,0.9,col);
      pacbst_hist[ipan]->Draw();
      pacbst_hist[ipan]->cd();
      pacbst_hist[ipan]->Divide(2,nmodppan,0,0);
    }

    sprintf(t,"pacbst_text%d",ipan);
    pacbst_text[ipan] = (TPad*) gROOT->FindObject(t);
    if(pacbst_text[ipan]==0){
      pacbst[ipan]->cd();
      pacbst_text[ipan] = new TPad(t,"Acc. Bursts Text Pad", 0.9,0.0,1.0,0.9,col);
      pacbst_text[ipan]->Draw();
      pacbst_text[ipan]->cd();
      pacbst_text[ipan]->Divide(1,nmodppan,0,0);
    }

    pacbst[ipan]->Update();
    
    pacbst_info[ipan]->cd(1);
    gPad->Clear();
    
    t1 = new TPaveText(0.0,0.0,1.0,1.0);
    t1->SetFillColor(10);
    t1->Draw();
    t1->SetTextAlign(13);
    t1->SetTextSize(0.15);
    t1->SetTextColor(4);
    
    TDatime *dt = new TDatime;
    sprintf(s,"ATLAS SCT Module Test  Run %d  Bursts %d-%d Date %d  Time %d", 
              e->runnum, e->burstnum-Nbursts, e->burstnum, dt->GetDate(),dt->GetTime());
    t1->AddText(s);
    sprintf(s,"Total Burst Trigs_sent %u from %d bursts, Scan %d", ntottrig, Nbursts,e->scannum);
    t1->AddText(s);
    sprintf(s,"Run Trigs_sent %u  Events_seen %u  Stream_errors %u  Timeouts %u  Other_errors %u",
	    e->run_tsent, e->run_evcnt, e->run_ercnt, e->run_tocnt, e->run_xecnt);
    t1->AddText(s); 
    t1->Draw();
    
    pacbst_info[ipan]->cd(2);
    gPad->Clear();
    t2 = new TPaveText(0.0,0.0,1.0,1.0);
    t2->SetFillColor(10);
    t2->Draw();
    t2->SetTextAlign(13);
    t2->SetTextSize(0.15);
    t2->SetTextColor(4);
    t2->Clear();
    sprintf(s," ");
    t2->AddText(s);
    sprintf(s,"Your advert could have been here!");
    t2->AddText(s);


    for (n=ipan*nmodppan; n<e->num_showmods && n<(ipan+1)*nmodppan; n++){
      pacbst_hist[ipan]->cd(((n-ipan*nmodppan)*2)+1);
      gPad->SetLogy(1);
      e->m[n]->h_accbst0->SetYTitle("Hits");
      e->m[n]->h_accbst0->Draw();
      pacbst_hist[ipan]->cd(((n-ipan*nmodppan)*2)+2);
      gPad->SetLogy(1);
      e->m[n]->h_accbst1->SetYTitle("Hits");
      e->m[n]->h_accbst1->Draw();
      pacbst_text[ipan]->cd(n-ipan*nmodppan+1);
      gPad->Clear();
      e->m[n]->GetDCSScanText()->Draw();
    }
    
    pacbst[ipan]->Modified();
    pacbst[ipan]->Update();
  }
  
  return ntottrig;
}
