// Macro to do correlated noise analysis on an event by event basis
// version 0.10 13/07/2001 JGK,PB  (partially based on a version by SJMP)
// version 0.11 18/07/2001 PWP
// version 0.12 06/08/2001 JGK - fixed problem with saving trees to file
// version 1,10 25/02/2002 JGK - added #hits/chip branches to tree
// version 1,11 27/03/2002 JGK/PJB - new plotting/analysis routine
//
// Arguments:
// n_events  : number of events to be taken
// Qthr      : threshold in fC
// standalone: check DCS status in regular intervals
// ENCref    : run a 3-pt gain scan for ENC reference values
//

void Gamma(Int_t n_events, Float_t Qthr, Int_t standalone=1, Int_t ENCref=0) {

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  Int_t burst, moi, stop_loop;
  Int_t n_bursts, bst_rem;
  char fname[128];

  Int_t MAX_NTRIGS=2500;
  if(Qthr>0.8) MAX_NTRIGS = 25000;

  // load DCS communication macro
  gROOT->ProcessLine(".L DCSSafety.cpp");
  gROOT->ProcessLine(".L tpg.cpp");

  // Save settings
  e->SaveState();

  // run tpg to have ENC reference
  if(ENCref) tpg(0);

  // now the actual gamma stuff
  e->abort=0;              // perhaps unnecessary?

  e->do_cal_loop=0;        // cal loop OFF
  e->do_autostop=0;        // autostop OFF
  e->do_fits=0;            // integrated fitting OFF
  e->do_mask_test=0;       // alternating mask patterns OFF 
  e->do_fill_evtree=2;     // build ntuple of number of hits in each chip
  e->config_frequency = 0; // do NOT send full configuration before each event

  e->burst_type   = 100;   // 100 - ST_DBURST
  e->burst_trtype = 0;     // 0 - L1A
 
  e->ConfigureScan(0,0,1,1);      // 0 - SCAN_ELAPSED
  if(Qthr>-99) e->ConfigureVariable(41,Qthr);  // 41 - QTHR (fC)
  e->ExecuteConfigs();

  e->scannum++;

  // open output file, needed here to have a buffer for the ntuples
  sprintf(fname,"%sstrun%d_%d.root",e->sct_data,e->runnum,e->scannum);
  TFile* out = new TFile(fname,"RECREATE");

  // clear all histogram, ntuple and data storage
  for(moi=0;moi<e->GetMaxModules();moi++){
    e->m[moi]->ResetData();
    e->m[moi]->CreateTree();
  }

  // trigger settings - split into several rounds to allow for an abort
  if(n_events>MAX_NTRIGS){
    e->burst_ntrigs = MAX_NTRIGS;
    n_bursts = n_events/MAX_NTRIGS;
    bst_rem = n_events-n_bursts*MAX_NTRIGS;
  }
  else{
    e->burst_ntrigs = n_events;
    n_bursts = 1;
    bst_rem = 0;
  }

  st_start_stopper();

  // execute burst

  burst=0;
  e->burst_count = 0;
  stop_loop=0;

  while((stop_loop==0)&&(burst<n_bursts)){

    if(standalone)
      if(DCSSafety(kTRUE)) return;

    printf("requesting burst %d\n",burst);
    e->ExecuteBurst();

    st_display_burst();
    st_display_burst2();
    for (moi=0; moi<e->num_showmods; moi++){
      if(e->m[moi]->present){
        e->m[moi]->PrintBurst();
      }
    }
    stop_loop = st_do_stop();
    burst++;

  }

  if((stop_loop==0) && bst_rem){
    printf("requesting burst %d\n",burst);
    e->burst_ntrigs = bst_rem;
    e->ExecuteBurst();

    st_display_burst();
    st_display_burst2();
    for (moi=0; moi<e->num_showmods; moi++){
      if(e->m[moi]->present){
        e->m[moi]->PrintBurst();
      }
    }
    stop_loop = st_do_stop();
    burst++;
  }

  st_stop_stopper();

  if(!e->abort){
    // print ntuple(s) for present modules
    for(moi=0;moi<MAX_MODULES;moi++){
      if(e->m[moi]->present){
        e->m[moi]->evtree->Print();
      }
    }

    st_save_scan(out);
  }

  // Restore settings
  e->RestoreState();

  if(standalone)
    DCSSafety(kFALSE);

  // do the analysis
  gROOT->ProcessLine(".L GammaAna.cpp");
  if(ENCref)
    GammaAna(e->runnum,e->scannum,Qthr,kFALSE,kFALSE,-1);
  else
    GammaAna(e->runnum,e->scannum,Qthr);

  return;
}
