void MultiInj(Int_t nrep, Char_t *genlabel, Int_t refrun=-1, Int_t refscan = -1){

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define MAXITER 30

  Int_t runs[MAXITER], scans[MAXITER],i;
  Char_t label[1024],text[50];

  gROOT->ProcessLine(".L DCSSafety.cpp");
  gROOT->ProcessLine(".L FastInjPlot.cpp");
  gROOT->ProcessLine(".L sleep.cpp");

  // check #iterations
  if(nrep>=MAXITER) {
    printf("Can only do up to %d iteration, requested %d\n",MAXITER-1,nrep);
    return;
  }

  // save state
  e->SaveState();

  // Configure the scan
  e->StOutScanReset();
  e->ascii_nt=0;
  e->abort=0;

  e->do_cal_loop=1;        // cal loop ON
  e->do_fits=1;            // integrated fitting ON

  e->burst_trtype = 3;     // CAL + L1A
  e->burst_ntrigs = 500;

  e->ConfigureVariable(11,1);  // X1X compression 
  e->ConfigureVariable(13,0);  // EDGE DETECT OFF
  e->ConfigureVariable(14,0);  // Route Mask OFF

  // take zero-injection measurement if requested

  sprintf(label,"refer");

  if(refrun>0 && refscan>0){
    scans[0]=refscan;
    runs[0]=refrun;
  } else{
    
    if(DCSSafety(kTRUE)) return;
    printf("Set signal generator output to zero output for reference measurement and type OK\n");
    scanf("%s",text);
    
    e->ConfigureVariable(42,2.0);
    e->ConfigureScan(1,50,300,5);
    st_scan(11,-1);
    scans[0]=e->scannum;
    runs[0]=e->runnum;
    
    if(e->abort) {
      DCSSafety(kFALSE);
      e->abort=0;
      return;
    }
  }

  // repeat threshold-scan to collect various injection currents

  if(strncmp(gSystem->GetName(),"Unix",4)) 
      gSystem.Exec("C:\\WNT\\System32\\rundll32.exe C:\\WNT\\System32\\amovie.ocx,RunDll /play /close C:\\Wnt\\Media\\ringout.wav");
  printf("Set signal generator output to a new value and type OK\n");
  scanf("%s",&text);
  
  for(i=1;i<=nrep;i++){

    if(DCSSafety(kTRUE)) return;
      
    if(e->abort) 
      break;

    sprintf(label,"%s:%s (take %d)",label,genlabel,i);

    e->ConfigureVariable(42,2.0);
    e->ConfigureScan(1,50,300,5);
    st_scan(11,-1);
    scans[i]=e->scannum;
    runs[i]=e->runnum;
    // run analysis
    FastInjPlot(i+1,runs,scans,label,0,0);
  }

  // Restore settings
  e->RestoreState();

  DCSSafety(kFALSE);

  if(!e->abort) FastInjPlot(nrep+1,runs,scans,label,0,1);

  return;
}
