void NOvsT(Int_t n_exec, Int_t tottrg, Int_t delt, Float_t thr_chg, Int_t standalone, Bool_t LVoff) { 

  // repeatedly run noise occupancy at required
  // Qthr and record average occ. per chip
  //
  // Version 0.1 JGK     15.05.01
  // Version 1.1 JGK,PB  20.07.01: improved analysis macro, can cut noisy
  //                               strips via median over a chip now
  //                               (now default analysis mode)
  // Version 1.2 JGK     31.07.01: added some interplay with DCS for 
  //                               stand-alone running
  //

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define MAX_EXECS  1000
  #define TRG_BST   25000

  Int_t i,j,round,towait,n_bursts,scan_f,scan_l,ntrig, response, t_HVupdate;
  char line[1024],h_name[64],h_title[64],f_name[128],s[60];
  TDatime* dt = new TDatime();

  // init for accumulated bursts
  n_bursts = (Int_t) tottrg/TRG_BST;
  gROOT->ProcessLine(".L systest/AccumBursts.cpp");
  gROOT->ProcessLine(".L systest/NOvsTPlot.cpp");
  gROOT->ProcessLine(".L systest/DCSSafety.cpp");
  gROOT->ProcessLine(".L systest/sleep.cpp");

  //Store settings before the scan
  e->SaveState();

  // Configure the scan
  e->StOutScanReset();
  e->ascii_nt=0;

  e->do_cal_loop=0;       // cal loop OFF
  e->do_fits=0;           // integrated fitting OFF
  e->do_vary_ntrigs=0;    // don't increase number of triggers

  e->burst_trtype=0;      // single L1A
  e->burst_type = 1;      // simple trigger burst
  e->burst_ntrigs=TRG_BST;

  e->ConfigureVariable(11,1);       // X1X compression 
  e->ConfigureVariable(13,0);       // EDGE DETECT OFF
  e->ConfigureVariable(41,thr_chg); // set Qthr

  e->cc->periodic_sr=0;
  e->cc->periodic_bcr=0;
  e->cc->randomiser=0;
  e->cc->SetPeriodicFreq((double)75.0); // trigger frequency in kHz

  e->ExecuteConfigs();

  if(n_exec<1 || n_exec > MAX_EXECS){
    printf("too many/few iterations: %d\n",n_exec);
    return;
  }
  printf("You asked for %d minutes wait.\n",delt);
  sleep(10);

  // HV timer
  dt->Set();
  t_HVupdate = dt->GetTime();
  // reset abort flag
  e->abort = 0;
  // start stop/abort panel
  st_start_stopper();

  for(round=0;round<n_exec && e->abort==0;round++){
    printf("Starting round %d out of %d\n",round+1,n_exec);
    
    // query DCS when in stand-alone mode
    if(standalone)
      if(DCSSafety(kTRUE)) return;

    // send ramp command every 10 min to keep SCTHV's busy
    dt->Set();
    if( ((dt->GetTime()-t_HVupdate)>600)
        ||((dt->GetTime()-t_HVupdate)<0)  ){ // past midnight!!
      t_HVupdate = dt->GetTime();
      e->HVRamp(0);
    }

    // Accumulated Bursts
    st_do_stop();
    if(e->abort==0) {
      ntrig = AccumBursts(n_bursts);
      sprintf(s,"%sstrun%d_%d.root",DATA_DIRECTORY,e->runnum,e->scannum);
      TFile *out = new TFile(s,"UPDATE");
      sprintf(s,"%f",thr_chg);
      TObjString *qts = new TObjString(s);
      qts->Write("Qthrq");
      out->Close();
    }
    else
      break;
    if(round==0)        scan_f=e->scannum;
    if(round==n_exec-1) scan_l=e->scannum;

    // wait
    if(round<n_exec-1){
      for(j=0;j<delt;j++){
	towait = delt-j;
	if(towait%3==0){
	  printf("waiting for %d minutes\n",towait);
	  // query DCS when in stand-alone mode
	  if(standalone)
	    if(DCSSafety(kTRUE)) return;
	}
	st_do_stop();
	if(e->abort==0) 
	  sleep(60);
	else
	  break;
      }
    }

  }

  // remove stop/abort panel
  st_stop_stopper();

  // switch LV off when in stand-alone mode
  j=0;
  i=60;
  if(LVoff && standalone){
    st_start_stopper();
    if(strncmp(gSystem->GetName(),"Unix",4)) 
      gSystem.Exec("C:\\WNT\\System32\\rundll32.exe C:\\WNT\\System32\\amovie.ocx,RunDll /play /close C:\\Wnt\\Media\\ringin.wav");
    printf("Will switch off LV+HV in 10 minutes - press ABORT button to leave them on\n");
    for(j=0;j<i;j++){
      towait = i-j;
      if(towait%10==0) printf("waiting %.2f minutes\n",((Float_t)towait)/6.);
      if(st_do_stop()) break;
      else sleep(10);
    }
    if(j==i) {
      printf("\nwill switch off LV+HV now\n\n");
      e->LVOff();
      e->HVRamp(1);
      DCSSafety(kFALSE);
    }
    st_stop_stopper();
  }

  // Reset values after scan
  e->RestoreState();

  // instructions to run analysis macro
  NOvsTPlot(e->runnum,scan_f,scan_l,50);
  printf("\nFinished...............\n\n");
  printf("You can plot DCS info by calling \".x DCS_NOvsT.cpp(%d,%d,%d)\"\n\n",
	 e->runnum,scan_f,scan_l);

  return;
}
void NOvsT(Int_t standalone) { 
  NOvsT(1.0,standalone);
  return;
}
void NOvsT() { 
  NOvsT(1.0,0);
  return;
}
