void NOvsTPlot(Int_t run, Int_t firstscan, Int_t lastscan, Int_t setcut){


  //Paul Bell 7/6/2001
  //Plots noise occupancy as a function of time by chip

  // JGK 08.06.2001
  // fixed problem with errors,
  // modified graphics, added to NOvsT macro.

  // JGK 10.10.2001  -  added average and peak plots

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define MAX_SCANS   1000

  //declarations
    
  Int_t n,nplotpt;
  Float_t occcut, qth;
  Char_t cutstr[128];
  Float_t omax=5.e-3, omin=1.e-8;
  TFile *f=0;
  TH1F *histo=0, *histo2=0;
  TObjString *time=0, *place=0, *clock=0, *name=0, *notrig=0, *qthrq=0;

  // check we don't exceed max scans before proceeding

  n=lastscan-firstscan+1;
  if(n>MAX_SCANS){
    printf("Error: too many scans!\n");
      return;
  }

  Char_t filename[40],module[MAX_MODULES][20],gtitle[60],histogram[20],histogram2[20],modname[20];
  Char_t fname[40],line[1024];

  Int_t first,last,l,k,nmod ;
  Int_t Nevt,i,j,t,s,link,chip,firstchan,lastchan,h,run,firstscan,lastscan;
  Int_t cutstrip[2*MAX_CHANS],cut[12];;

  Float_t ntrigs,meanpass[12],pass[MAX_SCANS][12];
  Float_t x,occ,sum[MAX_SCANS][12],minmeanmap,maxmeanmap;
  Float_t truehours[MAX_SCANS];
  Float_t mean1d[MAX_SCANS], err_up1d[MAX_SCANS], err_dn1d[MAX_SCANS], cut1d[MAX_SCANS];
  Float_t maxdif[MAX_LINKS*MAX_CHIPS][MAX_MODULES],avg[MAX_LINKS*MAX_CHIPS][MAX_MODULES];
  Float_t chipid[MAX_LINKS*MAX_CHIPS*MAX_MODULES],plotpk[MAX_LINKS*MAX_CHIPS*MAX_MODULES];
  Float_t plotavg[MAX_LINKS*MAX_CHIPS*MAX_MODULES],archavg[MAX_LINKS*MAX_CHIPS*MAX_MODULES];
  Float_t dif1, dif2, dif3;

  Float_t err_dn[MAX_SCANS][12],  err_up[MAX_SCANS][12];
  Float_t   mean[MAX_SCANS][12];
  Float_t meanmap[MAX_SCANS][2*MAX_CHANS], median[2*MAX_CHANS];
  Float_t xerrup[MAX_SCANS],      xerrdn[MAX_SCANS];

  Double_t hours,start,hrs[MAX_SCANS];

  FILE* txtout,*archdat;

  TLegend *leg=0;
  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();

  // arrays with binomial upper and lower errors

  Float_t clim2[101]={1.841,3.299,4.637,5.918,7.162,8.382,9.583,10.770,
		       11.944,13.110,14.27,15.41,16.56,17.70,18.83,19.96,
		       21.06,22.20,23.32,24.43,25.54,26.65,27.76,28.86,29.97,
		       31.07,32.16,33.26,34.35,35.45,36.54,37.63,38.72,39.80,
		       40.89,41.97,43.05,44.14,45.22,46.30,47.38,48.45,49.52,
		       50.61,51.68,52.76,53.83,54.90,55.97,57.05,58.11,59.19,
		       60.26,61.32,62.39,63.46,64.54,65.59,66.66,67.72,68.79,
		       69.85,70.91,71.98,73.04,74.10,75.16,76.22,77.29,78.35,
		       79.40,80.46,81.65,82.58,83.64,84.70,85.75,86.82,87.87,
		       88.92,89.98,91.04,92.09,93.14,94.20,95.25,96.31,97.36,
		       98.41,99.47,100.52,101.57,102.62,103.68,104.73,105.78,
		       106.85,107.88,108.93,109.98,111.03};
  Float_t clim1[101]={0.,0.1727,0.7083,1.368,2.086,2.842,3.623,4.421,5.234,
		       6.058,6.90,7.73,8.59,9.44,10.30,11.17,12.04,12.91,
		       13.80,14.68,15.57,16.46,17.35,18.24,19.14,20.03,20.93,
		       21.84,22.74,23.65,24.55,25.46,26.37,27.29,28.20,29.11,
		       30.03,30.95,31.86,32.78,33.70,34.62,35.55,36.47,37.39,
		       38.32,39.24,40.17,41.10,42.02,42.95,43.88,44.81,45.74,
		       46.67,47.61,48.54,49.48,50.41,51.34,52.28,53.21,54.15,
		       55.08,56.02,56.96,57.90,58.84,59.78,60.72,61.65,62.60,
		       63.54,64.58,65.42,66.36,67.30,68.24,69.19,70.13,71.08,
		       72.02,72.96,73.91,74.85,75.80,76.75,77.69,78.64,79.59,
		       80.54,81.48,82.43,83.38,84.32,85.27,86.22,87.18,88.12,
		       89.07,90.02};

  // to make arrays easier to follow set first scan = 0 and last scan = last - first
  first=0;
  last=lastscan-firstscan;
 
  // which cut to be used?
  if(setcut<1){
    sprintf(cutstr,"no upper cut");
    occcut = 1.1;
  }
  else if(setcut==1){
    sprintf(cutstr,"upper cut at 1e-3 occ.");
    occcut = 0.001;
  }
  else{
    sprintf(cutstr,"upper cut at %d strip-median",setcut);
 
  }

  // open new canvas
  TCanvas* can;
  can = new TCanvas("novst","novst",1100,750);
  gStyle->SetOptStat(0);
  gStyle->SetOptFit(0);      

  // open PS output file
  sprintf(fname,"%sNOvsT_%d_%d-%d.ps",PS_DIRECTORY,run,firstscan,lastscan);
  TPostScript* ps = new TPostScript(fname,112);

  // loop over all modules

  for(h=0;h<MAX_MODULES;h++){

    for (t=0;t<MAX_LINKS*MAX_CHIPS;t++){
      maxdif[t][h]=0.;
      avg[t][h]=0.;
      for (s=first;s<=last;s++){
	sum[s][t]=0.0;
	pass[s][t]=0.0;
      }
    }
            
    //loop over all scans (time)
  
    for (i=first;i<=last;i++){
      
      //write the file to open to char string and open the file
      
      sprintf(filename,"%sstrun%d_%d.root",DATA_DIRECTORY,run,firstscan+i);
      f = new TFile(filename);
            
      sprintf(histogram,"h_accbst0_%d;1",h);
      sprintf(histogram2,"h_accbst1_%d;1",h);
 
      histo  = (TH1F*) f->Get(histogram);
      histo2 = (TH1F*) f->Get(histogram2);
   
      //get the time stamp and name

      sprintf(modname,"ModuleName;%d",h+1);
      //      printf("%s",modname);
      time  = (TObjString*) f->Get("Time");
      place = (TObjString*) f->Get("Place");
      clock = (TObjString*) f->Get("Clock");     
      qthrq = (TObjString*) f->Get("Qthrq");
      notrig  = (TObjString*) f->Get("Ntrig");
      name  = (TObjString*) f->Get(modname);
      if(name){
	if(strcmp(name->GetName(),"missing")){
	  sscanf(name->GetName(),"%s", module[h]);
	}
	else{
	  nmod=h;
	  f->Close();
	  goto nomod;
	}
      }
      else{
	nmod=h;
	f->Close();
        goto nomod;
      }
      
      //write the time stamp to variable hours and no. trigs to ntrigs
      
      sscanf(clock->GetName(),"%lf", &hours);
      sscanf(notrig->GetName(),"%f", &ntrigs);
      if(qthrq)
	sscanf(qthrq->GetName(),"%f", &qth);
      else 
	qth = -999.;
      
      //fill array hrs with the times
      
      hrs[i]=hours;
      
      //find the mean number of hits over each chip
      
      for (chip=0;chip<MAX_LINKS*MAX_CHIPS;chip++){
	
	firstchan=(chip*128);
	lastchan=(chip*128)+128;
	for (t=firstchan;t<lastchan;t++) {
	  if(chip<6)	//chips 0 to 5
	    x = (Float_t) histo->GetBinContent(t+1);
	  else          // chips 6 to 11
	    x = (Float_t) histo2->GetBinContent(t+1-MAX_CHANS);
	  meanmap[i][t] = x/(Float_t)ntrigs;
	}
	if(i==first){ // set strips-cut mask first time round
	  if(setcut>1){  // find median if requested
	    for (l=firstchan;l<lastchan;l++){  // put meanmap in order
	      for (k=firstchan;k<lastchan-1;k++){
		if(meanmap[i][k]<meanmap[i][k+1]){
		  minmeanmap=meanmap[i][k];
		  maxmeanmap=meanmap[i][k+1];
		}else{
		  minmeanmap=meanmap[i][k+1];
		  maxmeanmap=meanmap[i][k];
		}
		median[k]=minmeanmap;
		median[k+1]=maxmeanmap;
	      }
	    } // end meanmap ordering
	    occcut=((Float_t)setcut)*median[(firstchan+lastchan)/2];  // median
	    if(occcut<=0.){   // if zero, find next non-zero element
	      for(l=(firstchan+lastchan)/2;l<lastchan && occcut<=0.;l++)
		occcut=((Float_t)setcut)*median[l];
	    }
	    if(occcut<=0.)  // if still zero assum some tiny limit
	      occcut = 1.e-5;
	  } // end if setcut>1
	  for (l=firstchan;l<lastchan;l++){  //apply occcut and derive cut-map
	    if(meanmap[i][l]<occcut)
	      cutstrip[l] = 1;
	    else
	      cutstrip[l] = 0;
	  }
	}
	// average and number of strips passed
	for (l=firstchan;l<lastchan;l++){  
	  if(cutstrip[l]){
	    pass[i][chip] += 1.0;
	    sum[i][chip]  += meanmap[i][l];
	  }
	}
	
	if(pass[i][chip]){
	  mean[i][chip]=sum[i][chip]/pass[i][chip];
	  Nevt=(Int_t)((Float_t)ntrigs*mean[i][chip]);
	  if(Nevt>100){
	    err_dn[i][chip] = (Float_t) TMath::Sqrt((Double_t)Nevt);
	    err_up[i][chip] = err_dn[i][chip];
	  }
	  else if(Nevt>0){
	    err_dn[i][chip] = (Float_t)Nevt-clim1[Nevt];
	    err_up[i][chip] = clim2[Nevt]-(Float_t)Nevt;
	  }
	  else if(Nevt==0){
	    err_dn[i][chip] = mean[i][chip]*(Float_t)ntrigs;
	    err_up[i][chip] = clim2[Nevt]-(Float_t)Nevt;
	  }
	  else{
	    err_dn[i][chip] = 0.;
	    err_up[i][chip] = 0.;
	  }
	  err_dn[i][chip]=err_dn[i][chip]/(Float_t)ntrigs/(Float_t)TMath::Sqrt((Double_t)pass[i][chip]);
	  err_up[i][chip]=err_up[i][chip]/(Float_t)ntrigs/(Float_t)TMath::Sqrt((Double_t)pass[i][chip]);
	  // process average occ. and peak detection
	  avg[chip][h]+=mean[i][chip];
	  if(i>5+first){
	    dif1 = mean[i-1][chip]-mean[i][chip];
	    dif2 = mean[i-1][chip]-mean[i-2][chip];
	    dif3 = mean[i-2][chip]-mean[i-3][chip];
	    if(dif1>maxdif[chip][h] && (dif2/dif1>0.5 || dif3/dif1>0.5)) 
	    //if(dif1>maxdif[chip][h] && dif2/dif1>0.5) 
	      maxdif[chip][h]=dif1;
	  }
	}
	else{
	  printf("No strip passed cut in module %d chip %d: %f\n",h,chip);
	  mean[i][chip]=0.;
	  err_dn[i][chip] = 0.;
	  err_up[i][chip] = 0.;	  
	}
	if(i==first) cut[chip]=128-(Int_t)pass[i][chip];	
      }
      f->Close();
    }

    //find time of start
    
    start=hrs[0];
    
    for (i=0;i<n;i++) {
      truehours[i]=(Float_t)(hrs[i]-start);
    }

    // open TXT output file
    sprintf(fname,"%s%s_NOvsT_%d_%d-%d.txt",RESULTS_DIRECTORY,module[h],run,firstscan,lastscan);
    txtout = fopen(fname,"w");
    if(!txtout){
      printf("ERROR opening output file %s!\n",fname);
      return;
    }
    fprintf(txtout,"time\tchip0\t1\t2\t3\t4\t5\t6\t7\t8\t9\t10\t11\n");

    // write results to file

    for (s=0;s<n;s++) {
      fprintf(txtout,"%f\t",hrs[s]);
      for (i=0;i<MAX_LINKS*MAX_CHIPS;i++) {
	fprintf(txtout,"%e",mean[s][i]);
	if(i<11) 
	  fprintf(txtout,"\t");
	else
	  fprintf(txtout,"\n");
      }
    }

    //draw the graphs
    
    for (i=0;i<n;i++){
      xerrup[i]=0.0;
      xerrdn[i]=0.0;
    }
    
    TGraphAsymmErrors* gr;
    //TGraph* gr;

    for(link=0;link<2;link++){
      ps->NewPage();
      can->cd();
      can->Clear();
      pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00);
      pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90);
      pad_text->Draw();
      pad_plot->Draw();
      pad_plot->Divide(3,2,0,0);
      
      pad_text->cd();
      TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
      t1->SetFillColor(10);
      t1->SetTextAlign(13);
      t1->SetTextSize(0.3);
      t1->SetTextColor(4);
      sprintf(line,"ATLAS SCT: Noise Occ. vs Time");
      if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
      if(place) sprintf(line,"%s  -  %s",line,place->GetName());
      sprintf(line,"%s  -  Module %s",line,module[h]);
      t1->AddText(line);
      sprintf(line,"Run %d   Scans %d-%d   Module %d   Stream %d", run,firstscan,lastscan,h,link);
      t1->AddText(line);
      if(qth!=-999.)
	sprintf(line,"Noise Occupancy at Qthr=%.1ffC vs Time - Projection per Chip, %s",qth,cutstr);
      else
	sprintf(line,"Noise Occupancy (unknown threshold) vs Time - Projection per Chip, %s",cutstr);
      t1->AddText(line);
      t1->Draw();
      can->Update();
      
      for (i=0;i<6;i++) {
	for (s=0;s<n;s++) {
	  mean1d[s]=mean[s][i+6*link];
	  err_dn1d[s]=err_dn[s][i+6*link];
	  err_up1d[s]=err_up[s][i+6*link];
	}
	gr = new TGraphAsymmErrors(n,truehours,mean1d,xerrup,xerrdn,err_dn1d,err_up1d);
	//gr = new TGraph(n,truehours,mean1d);
	sprintf(gtitle,"Chip %d - %d strips cut",i+6*link,cut[i+6*link]);
	pad_plot->cd(i+1);
	gPad->SetLeftMargin(0.14);
	gPad->SetTopMargin(0.12);
	gr->SetTitle(gtitle);
	gr->Draw("AP");
	gr->GetXaxis()->SetTitle("Time[h]");
	gr->GetYaxis()->SetTitle("Occupancy");
	gr->GetYaxis()->SetTitleOffset(1.6);
	gr->Draw("AP");
      }
      can->Modify();
      can->Update();
    }
     
    // close TXT output file
    fclose(txtout);
    
  }
  
  nmod=h;
  f->Close();
  
 nomod:
  
  ps->Close();
  ps->Delete();
  
  // open TXT output file
  sprintf(fname,"%sNOpeakavg_%d_%d-%d.txt",RESULTS_DIRECTORY,run,firstscan,lastscan);
  txtout = fopen(fname,"w");
  if(!txtout){
    printf("ERROR opening output file %s!\n",fname);
    return;
  }
  fprintf(txtout,"module\tpeak\taverage\n");

  // process average and peak arrays

  dif1=0;
  nplotpt=0;
  for(h=0;h<nmod;h++){
    //sprintf(fname,"%s_occ.dat",module[h]);
    //sprintf(fname,"J:\\www\\barrel_web\\ind_mod\\%s\\%s",module[h],fname);
    //archdat = fopen(fname,"r");
    //if(!archdat)
    //  printf("can't open file %s\n",fname);
    for(i=0;i<MAX_LINKS*MAX_CHIPS;i++){
      //  if(archdat)
      //fscanf(archdat,"%e",&occ);
      //  else
      occ = 1.;
      avg[i][h]/=(Float_t)(last-first);
      if(avg[i][h]>1.e-9){
	plotavg[nplotpt]= avg[i][h];
	archavg[nplotpt]= occ;
	maxdif[i][h]*=TMath::Sqrt(1e6/avg[i][h])/7;
	if(maxdif[i][h]>dif1) dif1 = maxdif[i][h];
	chipid[nplotpt] = 0.5+(Float_t)(i+MAX_LINKS*MAX_CHIPS*h);
	plotpk[nplotpt] = maxdif[i][h];
	nplotpt++;
	fprintf(txtout,"%s\t%f\t%e\n",module[h],maxdif[i][h],avg[i][h]);
      }
      else
	fprintf(txtout,"%s\t999\t999\n");
    }
    //if(archdat) fclose(archdat);
  }
  dif1*=1.1;
  dif1=(Float_t)(1+(Int_t)dif1);
  fclose(txtout);

  // open new PS output file
  sprintf(fname,"%sNOavg_%d_%d-%d.ps",PS_DIRECTORY,run,firstscan,lastscan);
  ps = new TPostScript(fname,112);

  // plot average

  ps->NewPage();
  can->cd();
  can->Clear();
  pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00);
  pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90);
  pad_text->Draw();
  pad_plot->Draw();
  
  pad_text->cd();
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(line,"ATLAS SCT: Noise Occ.");
  if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
  if(place) sprintf(line,"%s  -  %s",line,place->GetName());
  t1->AddText(line);
  sprintf(line,"Run %d   Scans %d-%d", run,firstscan,lastscan);
  t1->AddText(line);
  sprintf(line,"Average Noise Occupancy at Qthr=%.1ffC - Projection per Chip",qth);
  t1->AddText(line);
  t1->Draw();
  can->Update();

  pad_plot->cd();
  gPad->SetLogy(1);
  pad_plot->Range(-1.-(Float_t)nmod,-8.7,12*nmod+1+.5*(Float_t)nmod,log10(7.e-3));
  TGaxis *xax = new TGaxis(0,log10(1.e-8),12*nmod,log10(1.e-8),0,12,12,"U");
  xax->SetName("xaxis");
  xax->SetTitle("Module");
  xax->Draw();
  TGaxis *yax = new TGaxis(0,log10(1.e-8),0,log10(5.e-3),1.e-8,5.e-3,612,"G");
  yax->SetName("yaxis");
  yax->SetTitle("Average Occupancy");
  yax->Draw();
  ln = new TLine(0,log10(5.e-3),12*nmod,log10(5.e-3));
  ln->Draw();
  ln = new TLine(12*nmod,log10(1.e-8),12*nmod,log10(5.e-3));
  ln->Draw();

  ln = new TLine(0.,log10(5.e-4),12.*nmod,log10(5.e-4));
  ln->SetLineColor((Color_t) 2);
  ln->SetLineWidth(2);
  ln->Draw();

  for(i=0;i<nmod;i++){
    ln = new TLine(12*i,log10(1.e-8),12*i,log10(5.e-3));
    ln->Draw();
    TText tx;
    tx.SetTextSize(0.04);
    tx.DrawText((Double_t)(i*12+1),log10(6.e-9),module[i]);
  }
  TGraph* gr2;
  gr2 = new TGraph(nplotpt,chipid,plotavg);
  gr2->SetTitle("peaks");
  gr2->SetMarkerStyle(20);
  gr2->SetMarkerSize(1);
  gr2->SetMarkerColor((Color_t) 1);
  gr2->SetLineColor((Color_t) 1);
  // draw graph
  gr2->Draw("PL");
  //gr3 = new TGraph(nplotpt,chipid,archavg);
  //gr3->SetTitle("avg");
  //gr3->SetMarkerStyle(20);
  //gr3->SetMarkerSize(1);
  //gr3->SetMarkerColor((Color_t) 2);
  //gr3->SetLineColor((Color_t) 2);
  // draw graph
  //gr3->Draw("PL");
  // legend
  leg = new TLegend(.8,.9,1.,1.);
  leg->SetTextSize(0.03);
  leg->AddEntry(gr2,"Multimodule","p");
  //leg->AddEntry(gr3,"Individual","p");
  //leg->Draw();

  can->Update();
  
  ps->Close();
  ps->Delete();
  
  // open new PS output file
  sprintf(fname,"%sNOpeak_%d_%d-%d.ps",PS_DIRECTORY,run,firstscan,lastscan);
  ps = new TPostScript(fname,112);

  // plot peaks

  can->cd();
  can->Clear();
  pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00);
  pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90);
  pad_text->Draw();
  pad_plot->Draw();
  can->Update();

  pad_text->cd();
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(line,"ATLAS SCT: Peaks in occupancy");
  if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
  if(place) sprintf(line,"%s  -  %s",line,place->GetName());
  t1->AddText(line);
  sprintf(line,"Run %d   Scans %d-%d", run,firstscan,lastscan);
  t1->AddText(line);
  t1->Draw();
  can->Update();
  
  pad_plot->cd();
  pad_plot->Range(-15,-.15*dif1,12*nmod+10,dif1*1.1);
  TGaxis *xax = new TGaxis(0,0,12*nmod,0,0,nmod,600+nmod,"U");
  xax->SetName("xaxis");
  xax->SetTitle("Module");
  xax->Draw();
  TGaxis *yax = new TGaxis(0,0,0,dif1,0,dif1,808,"");
  yax->SetName("yaxis");
  yax->SetTitle("Amplitude of peaks (arb. units)");
  yax->Draw();
  for(i=0;i<=nmod;i++){
    ln = new TLine(i*12,0,i*12,dif1);
    ln->Draw();
    TText tx;
    tx.SetTextSize(0.035);
    tx.DrawText((Double_t)(i*12+1),-.05*dif1,module[i]);
  }
  ln = new TLine(0,dif1,12*nmod,dif1);
  ln->Draw();

  gr2 = new TGraph(nplotpt,chipid,plotpk);
  gr2->SetTitle("peaks");
  gr2->SetMarkerStyle(20);
  gr2->SetMarkerSize(1);
  gr2->SetMarkerColor((Color_t) 4);
  gr2->SetLineColor((Color_t) 4);
  // draw graph
  gr2->Draw("PL");

  can->Update();

  // done!

  ps->Close();  
  ps->Delete();

  return;
}
