void NoiseinjPlot(Int_t NDAT, Int_t *run, Int_t *scans, Float_t *curr){

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define MAXNDAT 20

  Int_t i,j,chip,goodchip, nread=1, nmod, refer=-1;
  Float_t noise[MAXNDAT][12][12],diff[144],maxnse=0.;
  Char_t fname[128],mname[30][MAX_MODULES],text[128],line[1024];
  Float_t allnse[MAX_MODULES*MAX_LINKS*MAX_CHIPS], id[MAX_MODULES*MAX_LINKS*MAX_CHIPS];
  TGraph *grm;

  FILE *input, *output;

  gROOT->ProcessLine(".L ENCsum.cpp");

  if(NDAT>MAXNDAT){
    printf("Too many data points, maximum is %d\n",MAXNDAT);
    return;
  }

  // try to find zero-current reference measurement
  for(i=0;i<NDAT;i++){
    if(curr[i] == 0.0){
      refer = i;
      break;
    }
  }
  if(refer<0){
    printf("Couldn't find any measurement with current=0, no reference -> QUIT\n");
    return;
  }

  // open canvas
  TCanvas* cinj= new TCanvas ("cinj","Noise Injection",1100,750);

  for(i=0;i<NDAT;i++){
    sprintf(fname,"%sENCsum_%d_%d-%d.txt",
	    RESULTS_DIRECTORY,run[i],scans[i],scans[i]+2);
    input = fopen(fname,"r");
    if(!input){
      ENCsum(run[i],scans[i]);
      sprintf(fname,"%sENCsum_%d_%d-%d.txt",
	      RESULTS_DIRECTORY,run[i],scans[i],scans[i]+2);
      input = fopen(fname,"r");
    }
    if(!input){
      printf("can't open file %s\n",fname);
      return;
    }
    chip=0;
    nread=1;

    for(j=0;j<7 && nread==1;j++)
      nread = fscanf(input,"%s",text);

    if(nread!=1){
      printf("no data in file %s\n",fname);
      return;
    }
    
    for(j=0;j<MAX_MODULES && nread==1;j++){
      for(chip=0;chip<MAX_LINKS*MAX_CHIPS && nread==1;chip++){
	if(chip)
	  nread = fscanf(input,"%s",text);
	else
	  nread = fscanf(input,"%s",mname[j]);
	nread = fscanf(input,"%s",text);
	nread = fscanf(input,"%f",&(noise[i][j][chip]));
      }
    }
    fclose(input);
    if(i==0){
      if(nread==1) nmod = j;
      else         nmod = j-1;
    }
    else if((j-1 != nmod && nread!=1) || (j != nmod && nread==1)){
      printf("Measurement %d has different number of modules than reference: %d %d\n",i,j-1,nmod);
      return;
    }
  }
  
  printf("found %d modules\n",nmod);
  
  // find maximum noise difference
  for(i=0;i<NDAT;i++){
    for(chip=0;chip<MAX_LINKS*MAX_CHIPS;chip++){
      for(j=0;j<nmod;j++){
	  if((noise[i][j][chip]-noise[refer][j][chip])>maxnse)
	    maxnse = noise[i][j][chip]-noise[refer][j][chip];
      }
    }
  }
  maxnse*=1.1;

  // get time/place info

  sprintf(fname,"%sstrun%d_%d.root",DATA_DIRECTORY,run[refer],scans[refer]);
  TFile strun(fname);
  TObjString* time  = (TObjString*)strun.Get("Time");
  TObjString* place = (TObjString*)strun.Get("Place");
  strun.Close();

  // plot

  sprintf(fname,"%sENCninj_%d_%d-%d.ps",PS_DIRECTORY,run[refer],scans[refer],scans[refer]+2);
  ps = new TPostScript(fname,112);

  ps->NewPage();
  cinj->cd();
  cinj->Clear();
  pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00);
  pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90);
  pad_text->Draw();
  pad_plot->Draw();
  cinj->Update();

  pad_text->cd();
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(line,"ATLAS SCT: ENC from 3pt gain with noise injection");
  if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
  if(place) sprintf(line,"%s  -  %s",line,place->GetName());
  t1->AddText(line);
  sprintf(line,"Input-noise at 2fC from 3point gain scans - difference to 0V injected (run %d, scans %d-%d)",
	  run[refer],scans[refer],scans[refer]+2);
  t1->AddText(line);
  t1->Draw();
  cinj->Update();
  
  pad_plot->cd();

  // draw drawing area
  pad_plot->Range(-15,-100-.13*maxnse,nmod*12+10,1.12*maxnse);
  TLegend *leg = new TLegend(.7,1.-.04*(Float_t)(NDAT-1),1.,1.);
  leg->SetTextFont(62);
  leg->SetTextSize(0.03);
  TGaxis *xax = new TGaxis(0,-100,nmod*12,-100,0,nmod,6*100+nmod,"U");
  xax->SetName("xaxis");
  xax->SetTitle("Module");
  xax->Draw();
  TGaxis *yax = new TGaxis(0,-100,0,maxnse,-100,maxnse,808,"");
  yax->SetName("yaxis");
  yax->SetTitle("Difference in input-noise (ENC)");
  yax->Draw();
  for(i=0;i<=nmod;i++){
    l = new TLine(i*12,-100,i*12,maxnse);
    l->Draw();
  }
  for(i=0;i<nmod;i++){
    TText t;
    t.SetTextSize(0.04);
    t.DrawText((Double_t)(i*12+1),-100-.05*maxnse,mname[i]);
  }
  l = new TLine(0,maxnse,nmod*12,maxnse);
  l->Draw();
  l = new TLine(0,0,nmod*12,0);
  l->Draw();

  // draw graphs
  for(j=1;j<NDAT;j++){
    if(j!=refer){
      goodchip=0;
      for(i=0;i<nmod;i++){
	for(chip=0;chip<12;chip++){
	  if(noise[j][i][chip]>1000){
	    id[goodchip]=(Float_t)chip + 12*i;
	    allnse[goodchip] = noise[j][i][chip]-noise[refer][i][chip];
	    goodchip++;
	  }
	}
      }
      grm = new TGraph(goodchip,id,allnse);
      grm->SetTitle("");
      grm->SetMarkerStyle(20);
      grm->SetMarkerSize(1);
      grm->SetMarkerColor((Color_t) j);
      grm->SetLineColor((Color_t) j);
      sprintf(text,"%.1fV injected (%d, %d-%d)",curr[j],run[j],scans[j],scans[j]+2);
      leg->AddEntry(grm,text,"p");
      grm->Draw("PL");
    }
  }

  leg->Draw();
  cinj->Update();
  ps->Close();

  sprintf(fname,"%sENCninj_%d_%d-%d.txt",RESULTS_DIRECTORY,run[refer],scans[refer],scans[refer]+2);
  output = fopen(fname,"w");
    
  for(i=0;i<nmod;i++){
    for(chip=0;chip<12;chip++){
      fprintf(output,"%s\t",mname[i]);
      for(j=1;j<NDAT;j++)
	if(j!=refer) fprintf(output,"%.1f\t",noise[j][i][chip]-noise[refer][i][chip]);
      fprintf(output,"\n");
      //fprintf(output,"%.1f\n",noise[refer][i][chip]);
    }
  }
  fclose(output);

  return;
}

