  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define MAX_SCANS   10

void ScurveFit(Int_t run, Int_t n_points, Int_t special_point,
			Int_t* scan, Float_t* charge, Short_t fit_type,
			Float_t out_gain[MAX_MODULES][MAX_LINKS][MAX_CHIPS],
			Float_t out_offs[MAX_MODULES][MAX_LINKS][MAX_CHIPS],
			Float_t out_inns[MAX_MODULES][MAX_LINKS][MAX_CHIPS]){

  // version 0.5 PWP 12.10.00
  // version 0.6 LE  18.11.00 Plotting with TGraphErrors, calculated as RMS.
  // version 0.7 PWP 21.11.00 changes to suit 10 point scan
  // version 0.71 PWP 21.11.00 bugfixes 
  // version 0.72 PWP 24.11.00 raise upper noise cut
  // version 0.8 PWP 08.12.00 add code to upload RC fit to ST
  //
  // STILL UNDER DEVELOPMENT!!!


  #define CH_LOST        0x1003  // dead, stuck or unbonded
  #define CH_DODGY       0x6004  // inefficient, low noise, or high noise

  #define CH_DEAD        0x0001
  #define CH_STUCK       0x0002
  #define CH_INEFFICIENT 0x0004
  #define CH_UNBONDED    0x1000
  #define CH_LO_GAIN     0x0100
  #define CH_HI_GAIN     0x0200
  #define CH_LO_OFFSET   0x0400
  #define CH_HI_OFFSET   0x0800
  #define CH_LO_NOISE    0x2000
  #define CH_HI_NOISE    0x4000

  #define LOST         0
  #define DODGY        1
  #define DEAD         2
  #define STUCK        3
  #define INEFFICIENT  4
  #define UNBONDED     5
  #define LO_GAIN      6
  #define HI_GAIN      7
  #define LO_OFFSET    8
  #define HI_OFFSET    9
  #define LO_NOISE    10
  #define HI_NOISE    11
  #define MAX_CODES   12

  #define MIN_GAIN         0
  #define MAX_GAIN       100
//#define MIN_OFFSET    -100  // doesn't work
  Float_t MIN_OFFSET = -100;
  #define MAX_OFFSET     120
  #define BONDED_NOISE   750
  #define MIN_INNSE     1100
//  #define MAX_INNSE     1600
  #define MAX_INNSE     1800

  Int_t module, link, chip, chan, i;
  char f_name[128], h_name[64], h_title[64], line[1024];
  char mname[MAX_MODULES][32];
  char rc_name[32];

  Short_t code[MAX_MODULES][MAX_LINKS][MAX_CHANS][MAX_SCANS];
  Float_t mean[MAX_MODULES][MAX_LINKS][MAX_CHANS][MAX_SCANS];
  Float_t sigma[MAX_MODULES][MAX_LINKS][MAX_CHANS][MAX_SCANS];

  Short_t ngood[MAX_MODULES][MAX_LINKS][MAX_CHANS];
  Short_t module_status[MAX_MODULES][MAX_LINKS];

  Float_t ch_mean[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_SCANS];
  Float_t ch_sigma[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_SCANS];
  Float_t ch_mean_err[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_SCANS];
  Float_t ch_sigma_err[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_SCANS];
  Float_t ch_ngood[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_SCANS];

  Float_t ch_gain[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_SCANS];
  Float_t ch_offset[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  Float_t ch_innse[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_SCANS];
  Float_t ch_innse_err[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_SCANS];
  Double_t ch_rcfit[MAX_MODULES][MAX_LINKS][MAX_CHIPS][3];

  Short_t ch_bad[MAX_MODULES][MAX_LINKS][MAX_CHIPS][MAX_CODES];
  Short_t lk_bad[MAX_MODULES][MAX_LINKS+1][MAX_CODES];

  Float_t charge_err[MAX_SCANS];

  TH1S *h_code;
  TH1F *h_mean, *h_sigma;

  for(int i=0;i<MAX_SCANS;i++) charge_err[i] = 0.2; 


  // Load other files needed by this one
  gROOT->LoadMacro("stan_util.cpp");
  stan_styles();
  gStyle.SetOptFit(10);

  // 
  // 1. Ensure that the arrays are cleared to start
  //

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      for(chan=0;chan<MAX_CHANS;chan++){
        for(i=0;i<MAX_SCANS;i++){
          code[module][link][chan][i]=0;
          mean[module][link][chan][i]=0;
          sigma[module][link][chan][i]=0;
        }
        ngood[module][link][chan]=0;
      }
      for(chip=0;chip<MAX_CHIPS;chip++){
        for(i=0;i<MAX_SCANS;i++){
          ch_mean[module][link][chip][i]=0;
          ch_sigma[module][link][chip][i]=0;
          ch_mean_err[module][link][chip][i]=0;
          ch_sigma_err[module][link][chip][i]=0;
          ch_ngood[module][link][chip][i]=0;

          ch_gain[module][link][chip][i]=0;
          ch_innse[module][link][chip][i]=0;
          ch_innse_err[module][link][chip][i]=0;
        }
        for(i=0;i<MAX_CODES;i++){
          ch_bad[module][link][chip][i]=0;
        }
        ch_rcfit[module][link][chip][0]=0;
        ch_rcfit[module][link][chip][1]=0;
        ch_rcfit[module][link][chip][2]=0;
        ch_offset[module][link][chip]=0;
      }
      module_status[module][link] = 0;
    }
    for(link=0;link<MAX_LINKS+1;link++){
      for(i=0;i<MAX_CODES;i++){
        lk_bad[module][link][i]=0;
      }
    }
    sprintf(mname[module]," ");
  }
  

  // 
  // 2. Load the Fitted Scan Data
  //

  // 2.1 Find today's date - needed for file output

  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();
  Int_t the_time = dt->GetTime();

  // 2.2 Some simple validity tests and helpful (!) messages

  if(n_points<0){
    printf("You requested that %d scans be processed.\n");
    printf("You must be having a laugh!!\n");
    printf("Now calm down and try again!!!\n");
  }

  if(n_points>MAX_SCANS){
    printf("WARNING: requested number of scans (%d) exceeds maximum\n",n_points,MAX_SCANS);
    printf("         only %d scans will be processed\n");
    n_points = MAX_SCANS;
  }

  if((special_point>n_points)||(special_point<0)){
    printf("WARNING: requested special scan (%d) out of range\n",special_point,n_points);
    printf("         scan %d will be used instead\n",(int)(n_points/2));
    special_point = (int)(n_points/2);
  }


  // 2.3 Loop to load files

  for(i=0;i<n_points;i++){

    // 2.3.1 Open file

    sprintf(f_name,"%sstrun%d_%d.root",DATA_DIRECTORY,run,scan[i]);
    TFile *strun = new TFile(f_name);

    if(strun==0){
      printf("Failed to open file %s - EXITING!!\n",f_name);
      return();
    }

    // 2.3.2 Find histograms and fill arrays

    for (module=0; module<MAX_MODULES; module++){
      if(i==0){
        sprintf(h_title,"ModuleName;%d",module+1);
        TObjString* time  = (TObjString*)strun->Get("Time");
        TObjString* place = (TObjString*)strun->Get("Place");
        TObjString* name = (TObjString*)strun->Get(h_title);

        if(name) sprintf(mname[module],"%s",name->GetName());
      }

      for(link=0;link<MAX_LINKS;link++){

        sprintf(h_name,"h_code%d;%d",link,module+1);
        h_code= (TH1S*)strun->Get(h_name);
        sprintf(h_name,"h_mean%d;%d",link,module+1);
        h_mean= (TH1F*)strun->Get(h_name);
        sprintf(h_name,"h_sigma%d;%d",link,module+1);
        h_sigma= (TH1F*)strun->Get(h_name);

        if(h_mean){ // fitted data exists
          if(h_mean->GetMean()>0){
            printf("Filling arrays for scan %d charge %f module %d link %d\n",
              scan[i], charge[i], module, link);
            module_status[module][link] = 1;
 
            for(chan=0;chan<768;chan++){
              code[module][link][chan][i] = h_code->GetBinContent(chan);
              mean[module][link][chan][i] = h_mean->GetBinContent(chan);
              sigma[module][link][chan][i]= h_sigma->GetBinContent(chan);
            } 
          }
        }
      } // end for(link)  
    }

    strun->Close();

  } // end for(module)


  //
  // 3. Calculate the arithmetic mean vt50 and outnse values for each chip
  //    and form the response curve graphs
  //

  TGraphErrors* gr_rc2[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  TGraph* gr_gain[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  TGraphErrors* gr_outnse[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  TGraphErrors* gr_innse[MAX_MODULES][MAX_LINKS][MAX_CHIPS];

  printf("Calculating Chip Averages....");

  for(module=0;module<MAX_MODULES;module++){
    if(module_status>0){
      for(link=0;link<MAX_LINKS;link++){
        for(chip=0;chip<MAX_CHIPS;chip++){

          // 3.1 Calculation of averages

          for(i=0;i<n_points;i++){

	    // Sum to calculate mean
            for(chan=(chip*128);chan<((chip+1)*128);chan++){

              if((code[module][link][chan][i]&BAD_TEST)==0){ // if (fit is good)
                ngood[module][link][chan]++;

                ch_ngood[module][link][chip][i]++;
                ch_mean[module][link][chip][i]+=mean[module][link][chan][i];
                ch_sigma[module][link][chip][i]+=sigma[module][link][chan][i];
              }
            }
	    // Calculate average
            if(ch_ngood[module][link][chip][i]>0){
              ch_mean[module][link][chip][i]/=ch_ngood[module][link][chip][i];
              ch_sigma[module][link][chip][i]/=ch_ngood[module][link][chip][i];
            }

	    // Sum to calculate RMS
            for(chan=(chip*128);chan<((chip+1)*128);chan++){

              if((code[module][link][chan][i]&BAD_TEST)==0){ // if (fit is good)

                ch_mean_err[module][link][chip][i]+=
		  (float)TMath::Power((ch_mean[module][link][chip][i]-mean[module][link][chan][i]),2);

                ch_sigma_err[module][link][chip][i]+=
		  (float)TMath::Power((ch_sigma[module][link][chip][i]-sigma[module][link][chan][i]),2);
              }
            }
	    // Calculate RMS
            if(ch_ngood[module][link][chip][i]>0){
              ch_mean_err[module][link][chip][i]=
		(float)TMath::Sqrt(ch_mean_err[module][link][chip][i]/ch_ngood[module][link][chip][i]);
	      ch_sigma_err[module][link][chip][i]=
		(float)TMath::Sqrt(ch_sigma_err[module][link][chip][i]/ch_ngood[module][link][chip][i]);
            }
          } // end for(i<n_points)

          // 3.2 Formation of response curve graphs
        
          gr_rc2[module][link][chip]= new TGraphErrors(n_points,charge,ch_mean[module][link][chip],
						       charge_err,ch_mean_err[module][link][chip]);
          gr_rc2[module][link][chip]->SetMarkerColor(kBlue);
          gr_outnse[module][link][chip]= new TGraphErrors(n_points,charge,ch_sigma[module][link][chip],
							  charge_err,ch_sigma_err[module][link][chip]);
          gr_outnse[module][link][chip]->SetMarkerColor(kBlue);
        }
      }
    }
  }
  printf(" done!\n");


  //
  // 4. Fit a response curve to the set of vt50 data for each channel
  //

  TF1* rc;

  TH1F* h_vt50[MAX_MODULES][MAX_LINKS];
  TH1F* h_gain[MAX_MODULES][MAX_LINKS];
  TH1F* h_offset[MAX_MODULES][MAX_LINKS];
  TH1F* h_innse[MAX_MODULES][MAX_LINKS];

  TH1F* hp_vt50[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  TH1F* hp_gain[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  TH1F* hp_offset[MAX_MODULES][MAX_LINKS][MAX_CHIPS];
  TH1F* hp_innse[MAX_MODULES][MAX_LINKS][MAX_CHIPS];

  // 4.1 Select the fit function

  switch(fit_type){
    case 4:   // linear 
      rc = new TF1("rc","pol1",0,8);
      sprintf(rc_name,"linear fit");
      break;
    case 3:   // exponential
      rc = new TF1("rc",st_exp,0,8,3);
      sprintf(rc_name,"exponential fit");
      break;
    case 2:   // grillo
      rc = new TF1("rc",grillo,0,8,3);
      sprintf(rc_name,"grillo fit");
      break;
    default:  // polynomial
      rc = new TF1("rc","pol2",0,8);
      sprintf(rc_name,"Second order polynomial fit");
      break;
  }

  // 4.2 Fit a response curve to each channel

  for (module=0; module<MAX_MODULES; module++){
    for(link=0;link<MAX_LINKS;link++){

      // 4.2.1 Create histograms to hold gain, offset and input noise

      sprintf(h_name, "h_vt50_%d_%d",module,link);
      h_vt50[module][link] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
      if(h_vt50[module][link]) h_vt50[module][link]->Delete();  //avoid memory leaks
      sprintf(h_title,"Vt50 at %2.2ffC",charge[special_point]);
      h_vt50[module][link] = new TH1F(h_name,h_title,768,0,768);
      h_vt50[module][link]->SetXTitle("Channel Number");
      h_vt50[module][link]->SetYTitle("Vt50 (mV");

      sprintf(h_name, "h_gain_%d_%d",module,link);
      h_gain[module][link] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
      if(h_gain[module][link]) h_gain[module][link]->Delete();  //avoid memory leaks
      sprintf(h_title,"Gain at %2.2ffC",charge[special_point]);
      h_gain[module][link] = new TH1F(h_name,h_title,768,0,768);
      h_gain[module][link]->SetXTitle("Channel Number");
      h_gain[module][link]->SetYTitle("Gain (mV/fC)");

      sprintf(h_name, "h_offset_%d_%d",module,link);
      h_offset[module][link] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
      if(h_offset[module][link]) h_offset[module][link]->Delete();  //avoid memory leaks
      sprintf(h_title,"Extrapolated Offset");
      h_offset[module][link] = new TH1F(h_name,h_title,768,0,768);
      h_offset[module][link]->SetXTitle("Channel Number");
      h_offset[module][link]->SetYTitle("Extrapolated Offset (mV)");

      sprintf(h_name, "h_innse_%d_%d",module,link);
      h_innse[module][link] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
      if(h_innse[module][link]) h_innse[module][link]->Delete();  //avoid memory leaks
      sprintf(h_title,"Input Noise at %2.2ffC",charge[special_point]);
      h_innse[module][link] = new TH1F(h_name,h_title,768,0,768);
      h_innse[module][link]->SetXTitle("Channel Number");
      h_innse[module][link]->SetYTitle("Input Noise (ENC)");

      for(chip=0;chip<6;chip++){
        Float_t xmin = ch_mean[module][link][chip][special_point]-50;
        Float_t xmax = ch_mean[module][link][chip][special_point]+50;

        sprintf(h_name, "h_vt50_%d_%d_%d",module,link,chip);
	hp_vt50[module][link][chip] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
	if(hp_vt50[module][link][chip]) hp_vt50[module][link][chip]->Delete();  //avoid memory leaks
        sprintf(h_title,"Chip %d Vt50 at %2.2ffC",chip,charge[special_point]);
        hp_vt50[module][link][chip] = new TH1F(h_name,h_title,200,xmin,xmax);
        hp_vt50[module][link][chip]->SetXTitle("Vt50 (mV)");
        hp_vt50[module][link][chip]->SetYTitle("n(Channels)");
        hp_vt50[module][link][chip]->SetLineColor(kBlue);
        hp_vt50[module][link][chip]->SetMarkerColor(kBlue);

        sprintf(h_name, "h_gain_%d_%d_%d",module,link,chip);
	hp_gain[module][link][chip] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
	if(hp_gain[module][link][chip]) hp_gain[module][link][chip]->Delete();  //avoid memory leaks
        sprintf(h_title,"Chip %d Gain at %2.2ffC",chip,charge[special_point]);
        hp_gain[module][link][chip] = new TH1F(h_name,h_title,200,0,100);
        hp_gain[module][link][chip]->SetXTitle("Gain (mV/fC)");
        hp_gain[module][link][chip]->SetYTitle("n(Channels)");
        hp_gain[module][link][chip]->SetLineColor(kBlue);
        hp_gain[module][link][chip]->SetMarkerColor(kBlue);

        sprintf(h_name, "h_offset_%d_%d_%d",module,link,chip);
	hp_offset[module][link][chip] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
	if(hp_offset[module][link][chip]) hp_offset[module][link][chip]->Delete();  //avoid memory leaks
        sprintf(h_title,"Chip %d Extrapolated Offset",chip);
        hp_offset[module][link][chip] = new TH1F(h_name,h_title,100,0,100);
        hp_offset[module][link][chip]->SetXTitle("Extrapolated Offset (mV)");
        hp_offset[module][link][chip]->SetYTitle("n(Channels)");
        hp_offset[module][link][chip]->SetLineColor(kBlue);
        hp_offset[module][link][chip]->SetMarkerColor(kBlue);

        sprintf(h_name, "h_innse_%d_%d_%d",module,link,chip);
	hp_innse[module][link][chip] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
	if(hp_innse[module][link][chip]) hp_innse[module][link][chip]->Delete();  //avoid memory leaks
        sprintf(h_title,"Chip %d Input Noise at %2.2ffC",chip,charge[special_point]);
        hp_innse[module][link][chip] = new TH1F(h_name,h_title,75,0,2500);
        hp_innse[module][link][chip]->SetXTitle("Input Noise (ENC)");
        hp_innse[module][link][chip]->SetYTitle("n(Channels)");
        hp_innse[module][link][chip]->SetLineColor(kBlue);
        hp_innse[module][link][chip]->SetMarkerColor(kBlue);
      }


      if(module_status[module][link]>0){
        printf("  module %d link %d\n",module,link);

        // 4.2.2 Loop over channels of each module for which we have data

        for(chan=0;chan<MAX_CHANS;chan++){
          chip=chan/128;

          // Revised PWP 12.10.00
          // Exclude bad vt50 points (fit errors) from the RC

          Float_t lx[MAX_SCANS];
          Float_t ly[MAX_SCANS];

          Int_t lp = 0;
          for(i=0;i<n_points;i++){
            if((code[module][link][chan][i]&BAD_TEST)==0){
              lx[lp]=charge[i];
              ly[lp]=mean[module][link][chan][i];
              lp++;
            }
          }

          // If enough vt50 points are good, fit the response curve

          if(lp>1){ // at least 2 good points

            // Set initial guesses

            switch(fit_type){
              case 3:   // exponential
                rc->SetParameters(900,6,-400);
                break;
              case 2:   // grillo
                rc->SetParameters(70,40,600);
                break;
              default:  // polynomial
                rc->SetParameters(70,50,0);
                break;
            }

            TGraph* gr_rc = new TGraph(lp,lx,ly);
            gr_rc->Fit("rc","RQ");

            Double_t the_rc[3];
            the_rc[0]=(double)rc->GetParameter(0);
            the_rc[1]=(double)rc->GetParameter(1);
            the_rc[2]=(double)rc->GetParameter(2); 

            // Calculate gain and input noise as f(charge)

            Float_t the_gain = 0;
            Float_t the_innse = 0;
            Float_t the_offset = rc->Eval(0);
            Short_t the_code = code[module][link][chan][special_point];

            switch(fit_type){
              case 3:   // exponential
                if(the_rc[1]>0){
                  the_gain = (the_rc[0]*exp(-charge[special_point]/the_rc[1]))/
                    (the_rc[1] * pow( (1.0+(exp(-charge[special_point]/the_rc[1]))),2));
                }
                break;
              case 2:   // grillo
                Float_t d1 = 1.0 +  pow( (the_rc[2]/(the_rc[1]*charge[special_point])),2);
                Float_t d2 = pow(the_rc[1],2) * pow(charge[special_point],3) * pow(d1,1.5);
                if(d2>0) the_gain= pow(the_rc[2],3)/d2;
                break;
              default:  // polynomial
                the_gain =  the_rc[1]+((double)charge[special_point]*(double)2.0*(double)the_rc[2]);
                break;
            }

            if(the_gain>0){
              the_innse = 6250 * sigma[module][link][chan][special_point]/the_gain;
            } else {
              the_innse = 0;
            }

            h_vt50[module][link]->Fill(chan,mean[module][link][chan][special_point]);
            hp_vt50[module][link][chip]->Fill(mean[module][link][chan][special_point]);
            h_gain[module][link]->Fill(chan,the_gain);
            hp_gain[module][link][chip]->Fill(the_gain);
            h_offset[module][link]->Fill(chan,the_offset);
            hp_offset[module][link][chip]->Fill(the_offset);
            h_innse[module][link]->Fill(chan,the_innse);
            hp_innse[module][link][chip]->Fill(the_innse);

            // Identify bad channels

            if(the_gain<MIN_GAIN)        the_code+=CH_LO_GAIN;
            else if(the_gain>MAX_GAIN)   the_code+=CH_HI_GAIN;

            if(the_offset<MIN_OFFSET)      the_code+=CH_LO_OFFSET;
            else if(the_offset>MAX_OFFSET) the_code+=CH_HI_OFFSET;

            if(the_innse<BONDED_NOISE)   the_code+=CH_UNBONDED;
            else if(the_innse<MIN_INNSE) the_code+=CH_LO_NOISE;
            else if(the_innse>MAX_INNSE) the_code+=CH_HI_NOISE;

            code[module][link][chan][special_point] = the_code;

            gr_rc->Delete();
          } // end for(channel)
        }
      } // end for(link)  
    }
  } // end for(module)


  //
  // 5. Bad channel totals
  //

  for(module=0;module<MAX_MODULES;module++){
    for(link=0;link<MAX_LINKS;link++){
      for(chan=0;chan<MAX_CHANS;chan++){

        chip = chan/128;
        the_code=code[module][link][chan][special_point];
        if(the_code>0){
          if (the_code&CH_LOST)        ch_bad[module][link][chip][LOST]++;
          if (the_code&CH_DODGY)       ch_bad[module][link][chip][DODGY]++;

          if (the_code&CH_DEAD)        ch_bad[module][link][chip][DEAD]++;
          if (the_code&CH_STUCK)       ch_bad[module][link][chip][STUCK]++;
          if (the_code&CH_INEFFICIENT) ch_bad[module][link][chip][INEFFICIENT]++;
          if (the_code&CH_UNBONDED)    ch_bad[module][link][chip][UNBONDED]++;
          if (the_code&CH_LO_GAIN)     ch_bad[module][link][chip][LO_GAIN]++;
          if (the_code&CH_HI_GAIN)     ch_bad[module][link][chip][HI_GAIN]++;
          if (the_code&CH_LO_OFFSET)   ch_bad[module][link][chip][LO_OFFSET]++;
          if (the_code&CH_HI_OFFSET)   ch_bad[module][link][chip][HI_OFFSET]++;
          if (the_code&CH_LO_NOISE)    ch_bad[module][link][chip][LO_NOISE]++;
          if (the_code&CH_HI_NOISE)    ch_bad[module][link][chip][HI_NOISE]++;
        }
      }
    }

    for(link=0;link<MAX_LINKS;link++){
      for(chip=0;chip<MAX_CHIPS;chip++){
        for(i=0;i<MAX_CODES;i++){
          lk_bad[module][link][i]+= ch_bad[module][link][chip][i];
          lk_bad[module][2][i]+= ch_bad[module][link][chip][i];
        }
      }
    }
  }


  //
  // 6. Process the Results
  //

  for(module=0;module<MAX_MODULES;module++){  // loop over modules
    if(module_status[module][0]>0){

      for(link=0;link<MAX_LINKS;link++){   // loop over links

        for(chip=0;chip<MAX_CHIPS;chip++){  // loop over chips

          hp_vt50[module][link][chip]->Fit("gaus","LQ0");
          hp_gain[module][link][chip]->Fit("gaus","LQ0");
          hp_offset[module][link][chip]->Fit("gaus","LQ0");
          hp_innse[module][link][chip]->Fit("gaus","LQ0");

          // Set initial guesses

          switch(fit_type){
            case 3:   // exponential
              rc->SetParameters(900,6,-400);
              break;
            case 2:   // grillo
              rc->SetParameters(70,40,600);
              break;
            default:  // polynomial
              rc->SetParameters(70,50,0);
              break;
          }

          gr_rc2[module][link][chip]->Fit("rc","RQ0");
          Double_t* the_rc2 = ch_rcfit[module][link][chip];
          the_rc2[0]=rc->GetParameter(0);
          the_rc2[1]=rc->GetParameter(1);
          the_rc2[2]=rc->GetParameter(2); 
        
          ch_offset[module][link][chip] = rc->Eval(0);

          // Calculate gain and input noise as f(charge)

          for(i=0;i<n_points;i++){
            switch(fit_type){
              case 3:   // exponential
                if(the_rc2[1]>0){
                  ch_gain[module][link][chip][i]= (the_rc2[0]*exp(-charge[i]/the_rc2[1]))/
                    (the_rc2[1] * pow( (1.0+(exp(-charge[i]/the_rc2[1]))),2));
                }
                break;
              case 2:   // grillo
                Float_t d1 = 1.0 +  pow( (the_rc[2]/(the_rc2[1]*charge[i])),2);
                Float_t d2 = pow(the_rc2[1],2) * pow(charge[i],3) * pow(d1,1.5);
                if(d2>0) ch_gain[module][link][chip][i]= pow(the_rc2[2],3)/d2;
                break;
              default:  // polynomial
                ch_gain[module][link][chip][i]=  the_rc2[1]+(charge[i]*2*the_rc2[2]);
                break;
            }
            if(ch_gain[module][link][chip][i]>0){
              ch_innse[module][link][chip][i]=6250.0 * ch_sigma[module][link][chip][i]
                                                     / ch_gain[module][link][chip][i];
              ch_innse_err[module][link][chip][i]=6250.0 * ch_sigma_err[module][link][chip][i]
		                                     / ch_gain[module][link][chip][i];
            }
          }

        }// end for(chip) 
      } // end for(link)
    } // end if(module_status)
  } // end for(module)

  rc->Delete();
  //c2->Delete();


  //
  // 7. Write the results to an ascii file, one per module, suited to the database
  //



  for(module=0;module<MAX_MODULES;module++){
    if(module_status[module][0]>0){

      // Open today's output file
      // Output from a series of tests will be added to this file

      sprintf(f_name,"%s%s_stfit.txt",RESULTS_DIRECTORY,mname[module]);
      FILE* outfile = fopen(f_name,"a+");

      if(outfile==NULL){
        printf("Could not open result file %s\n",f_name);
        return();
      } else {
        printf("File %s opened to append results \n",f_name);

        fprintf(outfile,"%%ModuleResults\n");
        fprintf(outfile,"#\n");
        fprintf(outfile,"# %s\n",time->GetName());
        fprintf(outfile,"# Fit type %d, %s\n",
           fit_type, rc_name);

        fprintf(outfile,"#\n");
        fprintf(outfile,"# run scan charge\n");
        for(i=0;i<n_points;i++){
          fprintf(outfile,"# %d  %d %1.2f\n",
            run, scan[i],charge[i]); 
        }


        fprintf(outfile,"#\n");
        fprintf(outfile,"%%ModuleGainOffsetNoise\n");
        fprintf(outfile,"#  at %2.2ffC\n",charge[special_point]);
        fprintf(outfile,"#          vt50 outnse ngood gain offset innse\n");
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
            fprintf(outfile,"Chip %2d:   %3.1f %2.2f %3d %2.1f %2.1f %4.0f\n",
              (6*link)+chip,
              ch_mean[module][link][chip][special_point],
              ch_sigma[module][link][chip][special_point],
              ch_ngood[module][link][chip][special_point],
              ch_gain[module][link][chip][special_point],
              ch_offset[module][link][chip],
              ch_innse[module][link][chip][special_point],
              ch_innse[module][link][chip][special_point]);
	    out_gain[module][link][chip] = ch_gain[module][link][chip][special_point];
	    out_offs[module][link][chip] = ch_offset[module][link][chip];
	    out_inns[module][link][chip] = ch_innse[module][link][chip][special_point];
           }
         }

        fprintf(outfile,"#\n");
        fprintf(outfile,"%%ModuleBadChannelSummary\n");
        fprintf(outfile,"#  at %2.2ffC\n",charge[special_point]);
        fprintf(outfile,"#          lost dodgy dead stuck inefficient unbonded lo_gn hi_gn lo_off hi_off lo_nse hi_nse\n");
  
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
             fprintf(outfile,"Chip %2d:   %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d\n",
               (6*link)+chip,
               ch_bad[module][link][chip][LOST],
               ch_bad[module][link][chip][DODGY],
               ch_bad[module][link][chip][DEAD],
               ch_bad[module][link][chip][STUCK],
               ch_bad[module][link][chip][INEFFICIENT],
               ch_bad[module][link][chip][UNBONDED],
               ch_bad[module][link][chip][LO_GAIN],
               ch_bad[module][link][chip][HI_GAIN],
               ch_bad[module][link][chip][LO_OFFSET],
               ch_bad[module][link][chip][HI_OFFSET],
               ch_bad[module][link][chip][LO_NOISE],
               ch_bad[module][link][chip][HI_NOISE]);
          }
        }

        for(link=0;link<MAX_LINKS+1;link++){
           fprintf(outfile,"Link %2d:   %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d %3d\n",
             link,
             lk_bad[module][link][LOST],
             lk_bad[module][link][DODGY],
             lk_bad[module][link][DEAD],
             lk_bad[module][link][STUCK],
             lk_bad[module][link][INEFFICIENT],
             lk_bad[module][link][UNBONDED],
             lk_bad[module][link][LO_GAIN],
             lk_bad[module][link][HI_GAIN],
             lk_bad[module][link][LO_OFFSET],
             lk_bad[module][link][HI_OFFSET],
             lk_bad[module][link][LO_NOISE],
             lk_bad[module][link][HI_NOISE]);
        }

        char comment[16];
        fprintf(outfile,"#\n");
        fprintf(outfile,"%%ModuleBadChannelList\n");
    //    fprintf(outfile,"%%ModuleChannelList\n");
        fprintf(outfile,"#  at %2.2ffC\n",charge[special_point]);
        fprintf(outfile,"#channel code gain offset noise comment\n");
        for(link=0;link<MAX_LINKS;link++){
          for(chan=0;chan<MAX_CHANS;chan++){

            the_code=code[module][link][chan][special_point];
            if(the_code&BAD_TEST){ // if (fit is bad)
    //        if(1){ // if (show all fits)
              sprintf(comment,"unknown");

              // I have made some attempt to prioritise these comments.
              // If a channel is intrinsically sick (dead,stuck, -ve gain etc.)
              // then noise anomalies would seem to be less important.
              // This area should be revisited.

              if (the_code&CH_DEAD)             sprintf(comment,"dead");
              else if (the_code&CH_STUCK)       sprintf(comment,"stuck"); 
              else if (the_code&CH_LO_GAIN)     sprintf(comment,"negative gain!!!");
              else if (the_code&CH_HI_GAIN)     sprintf(comment,"high gain");
              else if (the_code&CH_LO_OFFSET)   sprintf(comment,"low offset");
              else if (the_code&CH_HI_OFFSET)   sprintf(comment,"high offset");
              else if (the_code&CH_UNBONDED)    sprintf(comment,"unbonded");
              else if (the_code&CH_LO_NOISE)    sprintf(comment,"low noise");
              else if (the_code&CH_HI_NOISE)    sprintf(comment,"high noise");
              else if (the_code&CH_INEFFICIENT) sprintf(comment,"inefficient");

              fprintf(outfile,"%4d  %4hx  %2.1f  %2.1f  %4.0f  %s\n",
                (768*link)+chan,
                code[module][link][chan][special_point],
                h_gain[module][link]->GetBinContent(chan+1),
                h_offset[module][link]->GetBinContent(chan+1),
                h_innse[module][link]->GetBinContent(chan+1),
                comment);
            }
          }
        }

        fprintf(outfile,"#\n");
        fprintf(outfile,"%%ModuleResponseCurvePlot\n");
        fprintf(outfile,"#          function p0 p1 p2\n");
        for(link=0;link<MAX_LINKS;link++){
          for(chip=0;chip<MAX_CHIPS;chip++){
             fprintf(outfile,"Chip %2d:   %d %3.2f %3.2f %3.2f\n",
               (6*link)+chip, fit_type,
               ch_rcfit[module][link][chip][0],
               ch_rcfit[module][link][chip][1],
               ch_rcfit[module][link][chip][2]);
          }
        }

        fclose(outfile);

      }
    } // end if(module_status)
  }

  // clean up
  for (module=0; module<MAX_MODULES; module++){
    for(link=0;link<MAX_LINKS;link++){
      sprintf(h_name, "h_vt50_%d_%d",module,link);
      h_vt50[module][link] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
      if(h_vt50[module][link]) h_vt50[module][link]->Delete();  //avoid memory leaks
      sprintf(h_name, "h_gain_%d_%d",module,link);
      h_gain[module][link] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
      if(h_gain[module][link]) h_gain[module][link]->Delete();  //avoid memory leaks
      sprintf(h_name, "h_offset_%d_%d",module,link);
      h_offset[module][link] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
      if(h_offset[module][link]) h_offset[module][link]->Delete();  //avoid memory leaks
      sprintf(h_name, "h_innse_%d_%d",module,link);
      h_innse[module][link] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
      if(h_innse[module][link]) h_innse[module][link]->Delete();  //avoid memory leaks
      for(chip=0;chip<6;chip++){
        sprintf(h_name, "h_vt50_%d_%d_%d",module,link,chip);
	hp_vt50[module][link][chip] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
	if(hp_vt50[module][link][chip]) hp_vt50[module][link][chip]->Delete();  //avoid memory leaks
        sprintf(h_name, "h_gain_%d_%d_%d",module,link,chip);
	hp_gain[module][link][chip] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
	if(hp_gain[module][link][chip]) hp_gain[module][link][chip]->Delete();  //avoid memory leaks
        sprintf(h_name, "h_offset_%d_%d_%d",module,link,chip);
	hp_offset[module][link][chip] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
	if(hp_offset[module][link][chip]) hp_offset[module][link][chip]->Delete();  //avoid memory leaks
        sprintf(h_name, "h_innse_%d_%d_%d",module,link,chip);
	hp_innse[module][link][chip] = (TH1F*) gDirectory->GetList()->FindObject(h_name);
	if(hp_innse[module][link][chip]) hp_innse[module][link][chip]->Delete();  //avoid memory leaks
      }
    }
  }

  return;
}


