void nL1APlot(Int_t *runs, Int_t *scans, Int_t n, Int_t setcut, Int_t output=1){

  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define MAX_SCANS   100

  //declarations
    
  Int_t nmod;
  Float_t occcut, qth;
  Char_t cutstr[128];
  Float_t omax=5.e-3, omin=1.e-8;
  TFile *f=0;
  TH1F *histo=0, *histo2=0;
  TObjString *time=0, *place=0, *name=0, *notrig=0, *qthrq=0, *trdel=0;
  TGraph *gr=0;
  TGraphErrors *vst[MAX_MODULES];

  // check we don't exceed max scans before proceeding

  if(n>MAX_SCANS){
    printf("Error: too many scans!\n");
      return;
  }

  Char_t filename[40],module[MAX_MODULES][20],gtitle[60],histogram[20],histogram2[20],modname[20];
  Char_t fname[40],line[1024],label[MAX_SCANS][60];

  Int_t l,k;
  Int_t Nevt,i,j,t,s,link,chip,firstchan,lastchan,h;
  Int_t cutstrip[MAX_LINKS*MAX_CHANS],cut[12];;

  Float_t ntrigs,meanpass[12],pass[MAX_SCANS][12],tdel[MAX_SCANS];
  Float_t x,occ,sum[MAX_SCANS][12],minmeanmap,maxmeanmap;
  Float_t ratio[MAX_SCANS][MAX_LINKS*MAX_CHIPS*MAX_MODULES];
  Float_t maxdif[MAX_LINKS*MAX_CHIPS][MAX_MODULES],avg[MAX_LINKS*MAX_CHIPS][MAX_MODULES];
  Float_t chipid[MAX_LINKS*MAX_CHIPS*MAX_MODULES],plotpk[MAX_LINKS*MAX_CHIPS*MAX_MODULES];
  Float_t plotavg[MAX_LINKS*MAX_CHIPS*MAX_MODULES],archavg[MAX_LINKS*MAX_CHIPS*MAX_MODULES];
  Float_t diff=0;

  Float_t err_dn[MAX_SCANS],  err_up[MAX_SCANS];
  Float_t   mean[MAX_SCANS][12];
  Float_t meanmap[MAX_SCANS][MAX_LINKS*MAX_CHANS], median[MAX_LINKS*MAX_CHANS];
  Float_t errup[MAX_SCANS], errdn[MAX_SCANS], ratio1d[MAX_SCANS],refratio;

  FILE* txtout;

  TLegend *leg=0;
  TDatime *dt = new TDatime;
  Int_t date = dt->GetDate();

  // which cut to be used?
  if(setcut<1){
    sprintf(cutstr,"no upper cut");
    occcut = 1.1;
  }
  else if(setcut==1){
    sprintf(cutstr,"upper cut at 1e-3 occ.");
    occcut = 0.001;
  }
  else{
    sprintf(cutstr,"upper cut at %d strip-median",setcut);
 
  }

  // open TXT output file
  if(output){
    sprintf(fname,"%snL1A_%d_%d.txt",RESULTS_DIRECTORY,runs[0],scans[0]);
    txtout = fopen(fname,"w");
    if(!txtout){
      printf("ERROR opening output file %s!\n",fname);
      return;
    }
    fprintf(txtout,"module\tchip\tncut");
    for (i=0;i<n;i++)
      fprintf(txtout,"\tscan %d",scans[i]);
    fprintf(txtout,"\n");
  }

  // loop over all modules

  for(h=0;h<MAX_MODULES;h++){

    for (t=0;t<MAX_LINKS*MAX_CHIPS;t++){
      maxdif[t][h]=0.;
      for (s=0;s<n;s++){
	sum[s][t]=0.0;
	pass[s][t]=0.0;
      }
    }
            
    //loop over all scans (time)
  
    for (i=0;i<n;i++){
      
      //write the file to open to char string and open the file
      
      sprintf(filename,"%sstrun%d_%d.root",DATA_DIRECTORY,runs[i],scans[i]);
      f = new TFile(filename);
            
      sprintf(histogram,"h_accbst0_%d;1",h);
      sprintf(histogram2,"h_accbst1_%d;1",h);
 
      histo  = (TH1F*) f->Get(histogram);
      histo2 = (TH1F*) f->Get(histogram2);
   
      //get the time stamp and name

      sprintf(modname,"ModuleName;%d",h+1);
      //      printf("%s",modname);
      time   = (TObjString*) f->Get("Time");
      place  = (TObjString*) f->Get("Place");
      qthrq  = (TObjString*) f->Get("Qthrq");
      notrig = (TObjString*) f->Get("Ntrig");
      trdel  = (TObjString*) f->Get("TrDel");
      if(trdel && i>0){
	sscanf(trdel->GetName(),"%f",&(tdel[i-1]));
	tdel[i-1] *= 0.025;
	sprintf(label[i],"run %d scan %d, #Deltat=%.2f#mus",runs[i],scans[i],tdel[i-1]);
      }
      else
	sprintf(label[i],"run %d scan %d",runs[i],scans[i]);
      name  = (TObjString*) f->Get(modname);
      if(name){
	if(strcmp(name->GetName(),"missing")){
	  sscanf(name->GetName(),"%s", module[h]);
	}
	else{
	  nmod=h;
	  f->Close();
	  goto nomod;
	}
      }
      else{
	nmod=h;
	f->Close();
        goto nomod;
      }
      
      //write no. trigs to ntrigs
      
      sscanf(notrig->GetName(),"%f", &ntrigs);
      if(qthrq)
	sscanf(qthrq->GetName(),"%f", &qth);
      else{
	printf("Please enter Qthr (float) used for these measurements:");
	scanf("%f",qth);
      }
      
      //find the mean number of hits over each chip
      
      for (chip=0;chip<MAX_LINKS*MAX_CHIPS;chip++){
	
	firstchan=(chip*128);
	lastchan=(chip*128)+128;
	for (t=firstchan;t<lastchan;t++) {
	  if(chip<6)	//chips 0 to 5
	    x = (Float_t) histo->GetBinContent(t+1);
	  else          // chips 6 to 11
	    x = (Float_t) histo2->GetBinContent(t+1-MAX_CHANS);
	  meanmap[i][t] = x/(Float_t)ntrigs;
	}
	if(i==0){ // set strips-cut mask first time round
	  if(setcut>1){  // find median if requested
	    for (l=firstchan;l<lastchan;l++){  // put meanmap in order
	      for (k=firstchan;k<lastchan-1;k++){
		if(meanmap[i][k]<meanmap[i][k+1]){
		  minmeanmap=meanmap[i][k];
		  maxmeanmap=meanmap[i][k+1];
		}else{
		  minmeanmap=meanmap[i][k+1];
		  maxmeanmap=meanmap[i][k];
		}
		median[k]=minmeanmap;
		median[k+1]=maxmeanmap;
	      }
	    } // end meanmap ordering
	    occcut=((Float_t)setcut)*median[(firstchan+lastchan)/2];  // median
	    if(occcut<=0.){   // if zero, find next non-zero element
	      for(l=(firstchan+lastchan)/2;l<lastchan && occcut<=0.;l++)
		occcut=((Float_t)setcut)*median[l];
	    }
	    if(occcut<=0.)  // if still zero assum some tiny limit
	      occcut = 1.e-5;
	  } // end if setcut>1
	  for (l=firstchan;l<lastchan;l++){  //apply occcut and derive cut-map
	    if(meanmap[i][l]<occcut)
	      cutstrip[l] = 1;
	    else
	      cutstrip[l] = 0;
	    // temp fix for two bad strips on 0018
	    if(h==4 && l>(MAX_CHANS+511) && l<(MAX_CHANS+514))
	      cutstrip[l] = 0;
	  }
	}
	// average and number of strips passed
	for (l=firstchan;l<lastchan;l++){  
	  if(cutstrip[l]){
	    pass[i][chip] += 1.0;
	    sum[i][chip]  += meanmap[i][l];
	  }
	}
	
	if(pass[i][chip])
	  mean[i][chip]=sum[i][chip]/pass[i][chip];
	else{
	  printf("No strip passed cut in module %d chip %d: %f\n",h,chip);
	  mean[i][chip]=0.;
	}
	if(i==0) cut[chip]=128-(Int_t)pass[i][chip];	
      }
      f->Close();
    }

    // write results to file

    for (chip=0;chip<MAX_LINKS*MAX_CHIPS;chip++) {
      if(output) fprintf(txtout,"%s\t%d\t%d",module[h],chip,cut[chip]);
      for (s=0;s<n;s++) {
	if(s==0)
	  chipid[chip+h*MAX_LINKS*MAX_CHIPS] = (Float_t) chip+h*MAX_LINKS*MAX_CHIPS;
	else{
	  if(mean[0][chip]>0) 
	    ratio[s][chip+h*MAX_LINKS*MAX_CHIPS] = mean[s][chip]/mean[0][chip];
	  else
	    ratio[s][chip+h*MAX_LINKS*MAX_CHIPS] = 1.;
	  if(ratio[s][chip+h*MAX_LINKS*MAX_CHIPS]>diff) diff = ratio[s][chip+h*MAX_LINKS*MAX_CHIPS];	  
	}
	if(output) fprintf(txtout,"\t%e",mean[s][chip]);
      }
      if(output) fprintf(txtout,"\n");
    }
    for (s=1;s<n;s++) {
      ratio1d[s-1] = 0.;
      refratio=0.;
      for (chip=0;chip<MAX_LINKS*MAX_CHIPS;chip++) {
	ratio1d[s-1] += mean[s][chip]*ntrigs;
	refratio     += mean[0][chip]*ntrigs;
      }
      if(refratio>0){
	err_up[s-1] = sqrt(1./ratio1d[s-1]+1./refratio);
	ratio1d[s-1] /= refratio;
	err_up[s-1] *= ratio1d[s-1];
	errup[s-1] = 0.;
      } else{
	err_up[s-1] = 0.;
	ratio1d[s-1] =0.;
	err_up[s-1] = 0.;
	errup[s-1] = 0.;
      }
    }
    vst[h] = new TGraphErrors(n-1,tdel,ratio1d,errup,err_up);
    sprintf(gtitle,"Module %s",module[h]);
    vst[h]->SetTitle(gtitle);
    vst[h]->SetMarkerStyle(20);
    vst[h]->SetMarkerSize(1);
    vst[h]->SetMarkerColor((Color_t) 4);
    vst[h]->SetLineColor((Color_t) 4);
  }

 nomod:
  if(output) fclose(txtout);
  nmod = h;
  diff *=1.1;
  if(diff<10) diff=10;

  // open new canvas
  TCanvas* can;
  can = new TCanvas("avgocc","avgocc",1100,750);
  gStyle->SetOptStat(0);
  gStyle->SetOptFit(0);      

  // open ps-file
  if(output){
    sprintf(fname,"%snL1A_%d_%d.ps",PS_DIRECTORY,runs[0],scans[0]);
    TPostScript* ps = new TPostScript(fname,112);
    ps->NewPage();
  }

  can->cd();
  can->Clear();
  pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00);
  pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90);
  pad_text->Draw();
  pad_plot->Draw();
  
  pad_text->cd();
  TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
  t1->SetFillColor(10);
  t1->SetTextAlign(13);
  t1->SetTextSize(0.3);
  t1->SetTextColor(4);
  sprintf(line,"ATLAS SCT: Successive L1As");
  if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
  if(place) sprintf(line,"%s  -  %s",line,place->GetName());
  t1->AddText(line);
  sprintf(line,"Reference (single L1A): run %d scan %d; other scans double L1A apart by #Deltat, only reading data from 2nd L1A", 
	  runs[0],scans[0]);
  t1->AddText(line);
  sprintf(line,"Ratio in Noise Occupancy at Qthr=%.1ffC - Projection per Chip, %s",qth,cutstr);
  t1->AddText(line);
  t1->Draw();
  can->Update();

  pad_plot->cd();
  gPad->SetLogy(1);
  pad_plot->Range(-2*nmod,-1.5,18*nmod,log10(diff*1.1));
  TGaxis *xax = new TGaxis(0,-1,12*nmod,-1,0,nmod,600+nmod,"U");
  xax->SetName("xaxis");
  xax->SetTitle("Module");
  xax->Draw();
  TGaxis *yax = new TGaxis(0,-1,0,log10(diff),.1,diff,808,"G");
  yax->SetName("yaxis");
  yax->SetTitle("Occupancy, ratio to single L1A");
  yax->Draw();
  for(i=0;i<nmod;i++){
    ln = new TLine(i*12,-1,i*12,log10(diff));
    ln->Draw();
    TText tx;
    tx.SetTextSize(0.035);
    tx.DrawText((Double_t)(i*12+1),-1.3,module[i]);
  }
  ln = new TLine(nmod*12,-1,nmod*12,log10(diff));
  ln->Draw();
  ln = new TLine(0,log10(diff),12*nmod,log10(diff));
  ln->Draw();

  TLegend *leg = new TLegend(.7,1.-.03*(Float_t)n,1.,1.);
  leg->SetTextSize(0.03);
  for(s=1;s<n;s++){
    gr = new TGraph(nmod*MAX_LINKS*MAX_CHIPS,chipid,ratio[s]);
    gr->SetTitle("peaks");
    gr->SetMarkerSize(1);
    if(s<8){
      gr->SetMarkerStyle(20);
      gr->SetMarkerColor((Color_t) s);
      gr->SetLineColor((Color_t) s);
    } else if(s<15){
      gr->SetMarkerStyle(24);
      gr->SetMarkerColor((Color_t) s-7);
      gr->SetLineColor((Color_t) s-7);
    } else if(s<22){
      gr->SetMarkerStyle(29);
      gr->SetMarkerColor((Color_t) s-14);
      gr->SetLineColor((Color_t) s-14);
    } else {
      gr->SetMarkerStyle(28);
      gr->SetMarkerColor((Color_t) s-21);
      gr->SetLineColor((Color_t) s-21);
    }
    // draw graph
    gr->Draw("PL");
    leg->AddEntry(gr,label[s],"p");
  }
  leg->Draw();

  can->Update();

  if(output){
    ps->NewPage();

    can->cd();
    can->Clear();
    pad_text   = new TPad("pad_text", "Text",        0.00,0.90,1.00,1.00);
    pad_plot   = new TPad("pad_plot", "Plots",       0.00,0.00,1.00,0.90);
    pad_text->Draw();
    pad_plot->Draw();
    pad_plot->Divide(nmod/2,2,0,0);
    
    pad_text->cd();
    TPaveText *t1 = new TPaveText(0.0,0.0,1.0,1.0);
    t1->SetFillColor(10);
    t1->SetTextAlign(13);
    t1->SetTextSize(0.3);
    t1->SetTextColor(4);
    sprintf(line,"ATLAS SCT: Successive L1As");
    if(time)  sprintf(line,"%s  -  %s",line,time->GetName());
    if(place) sprintf(line,"%s  -  %s",line,place->GetName());
    t1->AddText(line);
    sprintf(line,"Double L1A apart by #Deltat, only reading data from 2nd L1A", 
	    runs[0],scans[0]);
    t1->AddText(line);
    sprintf(line,"Ratio in Noise Occupancy at Qthr=%.1ffC - Projection per module",qth);
    t1->AddText(line);
    t1->Draw();
    can->Update();
    
    for(h=0;h<nmod;h++){
      pad_plot->cd(h+1);
      gPad->SetLogy(1);
      // draw graph
      vst[h]->SetMaximum(diff);
      vst[h]->SetMinimum(0.1);
      vst[h]->Draw("APL");
      vst[h]->GetXaxis()->SetTitle("#Delta t (#mus)");
      vst[h]->GetYaxis()->SetTitle("Occupancy, ratio to single L1A");
      TLine *lin = new TLine(gPad->GetFrame()->GetX1(),0,gPad->GetFrame()->GetX2(),0);
      lin->Draw();
    }

    ps->Close();
    ps->Delete();
  }

  return;
}

void nL1APlot(Int_t run, Int_t fscan, Int_t lscan, Int_t setcut, Int_t output=1){

  Int_t runs[MAX_SCANS],scans[MAX_SCANS],i;
  for(i=fscan;i<=lscan;i++){
    runs[i-fscan] = run;
    scans[i-fscan] = i;
  }

  nL1APlot(runs,scans,lscan-fscan+1,setcut,output);
}
