void nL1ATriggerBurst(Int_t tottrg, Float_t thr_chg, Int_t startdt=0, Int_t stopdt=240, Int_t incrdt=10){
  
  #ifdef DATA_DIRECTORY
    // do not reload parameters.h
  #else
    // must load parameters.h
  #include <parameters.h>
  #endif

  #define TRG_BST   25000

  Int_t n_bursts = (Int_t) tottrg/TRG_BST;
  Int_t firstscan;
  Char_t s[512];

  gROOT->ProcessLine(".L AccumBursts.cpp");
  gROOT->ProcessLine(".L nL1APlot.cpp");
  gROOT->ProcessLine(".L DCSSafety.cpp");

  //Store settings before the scan
  e->SaveState();

  // Configure the scan

  e->ascii_nt=0;
  e->do_cal_loop=0;       // cal loop OFF
  e->do_fits=0;           // integrated fitting OFF
  e->do_vary_ntrigs=0;    // don't increase number of triggers
  e->do_cal_loop=0;       // cal loop OFF
  e->do_fits=0;           // integrated fitting OFF
  e->do_vary_ntrigs=0;    // don't increase number of triggers
  e->do_autostop=0;        // autostop OFF
  e->do_mask_test=0;       // alternating mask patterns OFF 
  //e->config_frequency = 0; // do NOT send full configuration before each event

  e->burst_ntrigs = TRG_BST;
  e->burst_maxtrigs = 3000000;
  e->ConfigureVariable(11,1);  // X1X compression 
  e->ConfigureVariable(13,0);  // EDGE DETECT OFF
  e->ConfigureVariable(14,0);  // Route Mask OFF
  e->ConfigureVariable(41,thr_chg); // set Qthr
  e->ExecuteConfigs();
  e->Sleep(300);

  if(DCSSafety(kTRUE)) return;

  // normal trigger
  e->burst_type = 100;
  e->burst_trtype = 0;
  e->L1A_per_loop = 1;
  e->throw_away = 0;
  e->abort = 0;
  st_start_stopper();
  if(st_do_stop()) return;
  // clear all histograms
  for(Int_t moi=0;moi<e->GetMaxModules();moi++){
    e->m[moi]->ResetData();
  }
  AccumBursts(n_bursts);
  sprintf(s,"%sstrun%d_%d.root",DATA_DIRECTORY,e->runnum,e->scannum);
  TFile *out = new TFile(s,"UPDATE");
  sprintf(s,"%f",thr_chg);
  TObjString *qts = new TObjString(s);
  qts->Write("Qthrq");
  out->Close();
  st_stop_stopper();
  firstscan = e->scannum;
  
  if(DCSSafety(kTRUE)) return;

  // double trigger
  e->burst_trtype = 53;  
  e->L1A_per_loop = 2;
  e->ConfigureVariable(45,e->L1A_per_loop);
  e->throw_away = 1;
  // reset abort flag
  e->abort = 0;
  st_start_stopper();
  for(Int_t del=startdt;del<stopdt+1;del+=incrdt){
    if(st_do_stop()) break;
    e->ConfigureVariable(46,del);
    // clear all histograms
    for(Int_t moi=0;moi<e->GetMaxModules();moi++){
      e->m[moi]->ResetData();
    }
    AccumBursts(n_bursts);
    if(e->abort || DCSSafety(kTRUE)) break;
    sprintf(s,"%sstrun%d_%d.root",DATA_DIRECTORY,e->runnum,e->scannum);
    TFile *out = new TFile(s,"UPDATE");
    sprintf(s,"%f",thr_chg);
    TObjString *qts = new TObjString(s);
    qts->Write("Qthrq");
    sprintf(s,"%f",e->RetrieveVariable(0,46));
    TObjString *da = new TObjString(s);
    da->Write("TrDel");
    out->Close();
    if(st_do_stop()) break;
    nL1APlot(e->runnum,firstscan,e->scannum,50,0);
  }
  st_stop_stopper();

  e->RestoreState();
  if(DCSSafety(kTRUE)) return;

  if(!e->abort) nL1APlot(e->runnum,firstscan,e->scannum,50,1);

  DCSSafety(kFALSE);

  return;
}
