/* optif_local.h
 * Useful internal structure for OPTIF info
 * Version 1.0. J.C.Hill. 28 July 2000.
 * JCH 24 June 2002. Support for ATLAS RCC library (for SBCs).
 */

/* Defines for address offsets */

#define OPTIF_INVERT_CLOCK    0x0
#define OPTIF_INVERT_COMMAND  0x2
#define OPTIF_RESET_BPM       0x4
#define OPTIF_INVERT_RECEIVER 0x8
#define OPTIF_INFO            0x10
#define OPTIF_THRESHOLD       0x80
#define OPTIF_LASER           0x180
#define OPTIF_BPM_BASE        0x100

/* Define BPM register offsets */

#define OPTIF_BPM_SYNCRESET   0x0
#define OPTIF_BPM_PULSEWIDTH  0x2
#define OPTIF_BPM_COARSEDELAY 0x4
#define OPTIF_BPM_MKSPNEG     0x6
#define OPTIF_BPM_MKSPPOS     0x8
#define OPTIF_BPM_FINEDELAY   0xA
#define OPTIF_BPM_CHANNELMASK 0xC
#define OPTIF_BPM_TESTDATA    0xE

/* Maximum number of receive/transmit streams */

#define MAX_TRANSMIT_STREAM 5
#define MAX_RECEIVE_STREAM 11

/* End of defines */

/* structure definition follows */

struct optif_info_structure
{
#if defined(NIVXI) || defined(RCC)
  int window;
#endif
  unsigned long address;
  short bpm_syncreset[MAX_TRANSMIT_STREAM+1];
  short bpm_pulsewidth[MAX_TRANSMIT_STREAM+1];
  short bpm_coarsedelay[MAX_TRANSMIT_STREAM+1];
  short bpm_mkspneg[MAX_TRANSMIT_STREAM+1];
  short bpm_mksppos[MAX_TRANSMIT_STREAM+1];
  short bpm_finedelay[MAX_TRANSMIT_STREAM+1];
  short bpm_channelmask[MAX_TRANSMIT_STREAM+1];
  short bpm_testdata[MAX_TRANSMIT_STREAM+1];
  short laser[MAX_TRANSMIT_STREAM+1];
  short threshold[MAX_RECEIVE_STREAM+1];
  short invert_clock;
  short invert_command;
/* The following is wasteful, as the invert is in blocks of 4,
 * but I think it is clearer for the user.
 */
  short invert_receiver[MAX_RECEIVE_STREAM+1]; 
};

#define optif_info_size sizeof(struct optif_info_structure)

/* End of OPTIF local include file */
