
//
// TModuleDCS.cpp
//
// PWP 10.12.00
//

#include <iostream.h>

extern "C" {

  #include "../stlib/stlib.h"
  #include "../stlib/sct_scans.h"

}

#include "TModuleDCS.h"
#include "dcs_message.h"

ClassImp(TModuleDCS)


TModuleDCS::TModuleDCS(){

  id=-1;
  hv_source=DCS_LOCAL;
  lv_source=DCS_LOCAL;
  t_source =DCS_LOCAL;

  date = 0;
  time = 0;

  t_HVupdate=0;
  t_LVupdate=0;
  t_MTupdate=0;
  t_ETupdate=0;

  HVStopwatch = new TStopwatch();
  HVStopwatch->Stop();
  LVStopwatch = new TStopwatch();
  LVStopwatch->Stop();

  HVStopwatchTime=0;
  LVStopwatchTime=0;
  HVStopwatchRunning=kFALSE;
  LVStopwatchRunning=kFALSE;

  suspended=0;

  // Set defaults
  vdet=0;
  vcc=0;
  vi1=0;
  vdd=0;
  vpin=0;
  vld0=0;
  vld1=0;

  idet=0;
  icc=0;
  ivi1=0;
  idd=0;
  ipin=0;
  ild0=0;
  ild1=0;

  T1=0;
  T2=0;

  HVstatus=0;
  LVstatus=0;
  LVtemp=0;

  ET1=0; 
  ET2=0; 
  hum1=0;
  hum2=0;

  // Set trip defaults
  HVtrip=0;
  LVtrip=0;
  MTtrip=0;
  ETtrip=0;

  nHVtrips=0;
  nLVtrips=0;
  nMTtrips=0;
  nETtrips=0;

  // From here on in, the default values
  // are entirely arbirary...

  maxHVtrips=  1;
  maxLVtrips= 10;
  maxMTtrips= -1;
  maxETtrips= -1;

  T1_maximum=60.0;
  T2_maximum=60.0;
  T1_minimum=-20.0;
  T2_minimum=-20.0;

  ET1_maximum=60.0;
  ET2_maximum=60.0;
  ET1_minimum=-20.0;
  ET2_minimum=-20.0;

  hum1_maximum=50.0;
  hum2_maximum=50.0;
  hum1_minimum=0.0;
  hum2_minimum=0.0;
}


void TModuleDCS::SetID(Short_t new_id){

  id = new_id;
}


TModuleDCS::~TModuleDCS(){

}

//
// Simple DCS actions
//

void TModuleDCS::LVoff(){

  LVStopwatch->Stop();
  LVStopwatchRunning = kFALSE;

  switch(lv_source){
    case DCS_LOCAL:
      st_lv_1off(id);
      break;
    case DCS_TCP:
      break;
    default:
      break;
  }
}

void TModuleDCS::LVon(){

  LVStopwatch->Continue();
  LVStopwatchRunning = kTRUE;

  switch(lv_source){
    case DCS_LOCAL:
      st_lv_1on(id);
      break;
    case DCS_TCP:
      break;
    default:
      break;
  }
}

void TModuleDCS::HVoff(){

  HVStopwatch->Stop();
  HVStopwatchRunning = kFALSE;

  switch(hv_source){
    case DCS_LOCAL:
//      st_hv_1off(id);
      break;
    case DCS_TCP:
      break;
    default:
      break;
  }
}

void TModuleDCS::HVon(){

  HVStopwatch->Continue();
  HVStopwatchRunning = kTRUE;

  switch(hv_source){
    case DCS_LOCAL:
//      st_hv_1on(id);
      break;
    case DCS_TCP:
      break;
    default:
      break;
  }
}





//
// Major routines
//

void TModuleDCS::Action(){

  Action(1);
}

void TModuleDCS::Action(Short_t do_query){
  //
  // Here the information from DCS is checked for error
  // conditions such as over temperature or a power trip.
  // Subject to the limits set for the maximum number of
  // trips to be allowed, recovery will be attempted.
  //

  Int_t ntries,rcode;
  Int_t do_suspend=0;


  if(do_query) Query();


  // TODO: temperature 


  if(HVtrip>0){
    if((nHVtrips<maxHVtrips)&&(suspended==0)){
      // TODO: RECOVER HV

      // if HV cannot be recovered, then suspend
    } else {
      do_suspend=1;
    }
  }

  if(LVtrip>0){
    if((nLVtrips<maxLVtrips)&&(suspended==0)){

      // ATTEMPT RECOVERY

      ntries = 0;
      rcode = 0;
      while(ntries<10){
        cout << "Attempting to recover module "<< id;
        cout << " LV Trip number " << nLVtrips;
        cout << " attempt number " << ntries << endl;

        if(lv_source == DCS_LOCAL){
          rcode = st_lv_recover(id);
          if(rcode==0){ // trip recovered
            st_execute_configs(0,1);
            rcode = st_lv_recover(id);
          }
        }

        ntries++;
        if(rcode==0){ // did not trip again
          cout << " LV Trip has been recovered" << endl;
          ntries=10;
        }
      }

      if((ntries>=10)&&(rcode!=0)){
        cout << " LV Trip cannot be recovered - module will be suspended" << endl;
        do_suspend = 1;
      }

    } else {

      // GIVE UP
      do_suspend=1;
    }
  }

  if(do_suspend){

    // TODO: switch the module off and remove it from the DAQ

    suspended=1;
  }

}


void TModuleDCS::Query(){
  //
  // Query DCS for the latest information for this module.
  // For the time being, we query stlib to find out the 
  // state of the appropriate SCTLV and SCTHV channels.
  // Module temperature information may come from TEMPTRESS,
  // an external DCS system or, later, SCTLV3
  //

  Char_t buf[32];

  Float_t l_HVtrip = 0;
  Float_t l_LVtrip = 0;
  Float_t l_MTtrip = 0;
  Float_t var;

  Int_t nBad;

  // Time and date of this query

  TDatime* dt = new TDatime(); // MEMORY LEAK???
  date= dt->GetDate();
  time= dt->GetTime();

  // Update stopwatches which track how long the module has been powered

  HVStopwatchTime = HVStopwatch->RealTime();
  if(HVStopwatchRunning==kTRUE) HVStopwatch->Continue();

  LVStopwatchTime = LVStopwatch->RealTime();
  if(LVStopwatchRunning==kTRUE) LVStopwatch->Continue();


  // Get HV info

  switch(hv_source){
    case DCS_LOCAL:
      // The scthv library now imposes 1 second intervals
      // between queries.  Here we impose an additional
      // 2 second interval
      dt->Set();
      if( ((dt->GetTime()-t_HVupdate)>2)
        ||((dt->GetTime()-t_HVupdate)<0)  ){ // past midnight!!

        t_HVupdate = dt->GetTime();
        st_hv_mon(id);

        st_retrieve_variable(id,0,0,ST_VDET,&vdet_set);
        st_retrieve_variable(id,0,0,ST_IDET,&idet_lim);
        st_retrieve_variable(id,0,0,ST_RAMP,&var);
        ramp = (Short_t)var;

        st_retrieve_variable(id,0,0,ST_MON_VDET,&vdet);
        st_retrieve_variable(id,0,0,ST_MON_IDET,&idet);
        st_retrieve_variable(id,0,0,ST_HV_STATUS,&var);
        HVstatus = (Short_t)var;

        st_retrieve_variable(id,0,0,ST_HVTRIP,&l_HVtrip);
      }
      break;

    case DCS_TCP:
      // only do this if it has been more than 30
      // seconds since the last request.  TCP is slow.
      dt->Set();
      if( ((dt->GetTime()-t_HVupdate)>30)
        ||((dt->GetTime()-t_HVupdate)<0)  ){ // past midnight!!

        t_HVupdate = dt->GetTime();

        sprintf(buf,"MODULE %d VDET",id);
        vdet = dcs_message(buf);
        sprintf(buf,"MODULE %d IDET",id);
        idet = dcs_message(buf);
        sprintf(buf,"MODULE %d HVS",id);
        HVstatus = (Short_t) dcs_message(buf);
        sprintf(buf,"MODULE %d HVT",id);
        l_HVtrip = dcs_message(buf);
      }
      break;

    default:
      break;
  }

  switch(lv_source){
    case DCS_LOCAL:

      dt->Set();
      t_LVupdate = dt->GetTime();
      st_lv_mon(id);

      st_retrieve_variable(id,0,0,ST_VCC,&vcc_set);
      st_retrieve_variable(id,0,0,ST_VI1,&vi1_set);
      st_retrieve_variable(id,0,0,ST_VDD,&vdd_set);
      st_retrieve_variable(id,0,0,ST_VPIN,&vpin_set);
      st_retrieve_variable(id,0,0,ST_VLED0,&vld0_set);
      st_retrieve_variable(id,0,0,ST_VLED1,&vld1_set);

      st_retrieve_variable(id,0,0,ST_ICC,&icc_lim);
      st_retrieve_variable(id,0,0,ST_IVI1,&ivi1_lim);
      st_retrieve_variable(id,0,0,ST_IDD,&idd_lim);
      st_retrieve_variable(id,0,0,ST_IPIN,&ipin_lim);
      st_retrieve_variable(id,0,0,ST_ILED0,&ild0_lim);
      st_retrieve_variable(id,0,0,ST_ILED1,&ild1_lim);

      st_retrieve_variable(id,0,0,ST_MON_VCC,&vcc);
      st_retrieve_variable(id,0,0,ST_MON_VI1,&vi1);
      st_retrieve_variable(id,0,0,ST_MON_VDD,&vdd);
      st_retrieve_variable(id,0,0,ST_MON_VPIN,&vpin);
      st_retrieve_variable(id,0,0,ST_MON_VLED0,&vld0);
      st_retrieve_variable(id,0,0,ST_MON_VLED1,&vld1);

      st_retrieve_variable(id,0,0,ST_MON_ICC,&icc);
      st_retrieve_variable(id,0,0,ST_MON_IVI1,&ivi1);
      st_retrieve_variable(id,0,0,ST_MON_IDD,&idd);
      st_retrieve_variable(id,0,0,ST_MON_IPIN,&ipin);
      st_retrieve_variable(id,0,0,ST_MON_ILED0,&ild0);
      st_retrieve_variable(id,0,0,ST_MON_ILED1,&ild1);

      st_retrieve_variable(id,0,0,ST_LV_STATUS,&var);
      LVstatus=(Short_t) var;
      st_retrieve_variable(id,0,0,ST_LV_TEMP,&var);
      LVtemp=(Short_t) var;

      st_retrieve_variable(id,0,0,ST_LVTRIP,&l_LVtrip);
      break;

    case DCS_TCP:
      // read lv information over TCP
      break;

    default:
      break;
  }

  switch(t_source){
    case DCS_LOCAL:
      dt->Set();
      t_MTupdate = dt->GetTime();
      st_t_mon(id);

      st_retrieve_variable(id,0,0,ST_MON_T1,&T1);
      st_retrieve_variable(id,0,0,ST_MON_T2,&T2);
      st_retrieve_variable(id,0,0,ST_MTTRIP,&l_MTtrip);
      break;

    case DCS_TCP:
      // only do this if it has been more than 30
      // seconds since the last request.  TCP is slow.
      dt->Set();
      if( ((dt->GetTime()-t_MTupdate)>30)
        ||((dt->GetTime()-t_MTupdate)<0)  ){ // past midnight!!

        t_MTupdate = dt->GetTime();

        sprintf(buf,"MODULE %d T1",id);
        T1 = dcs_message(buf);
        sprintf(buf,"MODULE %d T2",id);
        T2 = dcs_message(buf);
      }
      break;
  }


  //
  // Check that the module parameters remain
  // within the desired  operating range
  //

  // 1. process module temperature information

  nBad = 0;
  if((T1<T1_minimum)||(T1>T1_maximum)) nBad++;
  if((T2<T2_minimum)||(T2>T2_maximum)) nBad++;

  if(nBad>0){
   if(MTtrip==0){        // Set Trip
     MTtrip=1;
     nMTtrips++;
   }
  } else if(MTtrip==1) { // Clear Trip
    MTtrip=0;
  }

  // 2. process environmental information

  nBad = 0;
  if((ET1<ET1_minimum)||(ET1>ET1_maximum)) nBad++;
  if((ET2<ET2_minimum)||(ET2>ET2_maximum)) nBad++;
  if((hum1<hum1_minimum)||(hum1>hum1_maximum)) nBad++;
  if((hum2<hum2_minimum)||(hum2>hum2_maximum)) nBad++;

  if(nBad>0){
   if(ETtrip==0){        // Set Trip
     ETtrip=1;
     nETtrips++;
   }
  } else if(ETtrip==1) { // Clear Trip
    ETtrip=0;
  }
     
  // 3. process HV information

  if(l_HVtrip>0){
   if(HVtrip==0){        // Set Trip
     HVtrip=1;
     nHVtrips++;
   }
  } else if(HVtrip==1) { // Clear Trip
    HVtrip=0;
  }


  // 4. process LV information

  if(l_LVtrip>0){
   if(LVtrip==0){        // Set Trip
     LVtrip=1;
     nLVtrips++;
   }
  } else if(LVtrip==1) { // Clear Trip
    LVtrip=0;
  }

  delete dt; // pwp 20.12.01
}


//
// Display data
//

void TModuleDCS::Print(){
  //
  // Query DCS and print the results
  //

  Print(1);
}

void TModuleDCS::Print(Short_t query){
  //
  // Display the results of the latest DCS query
  //

  if(query) Query();

  cout << "DCS information for module " << id << endl;
  cout << "  date\t " << date << "\ttime\t" << time << endl;

  cout << "\tSET\tMON\t\tLIM\tMON" << endl;
  cout << "  vdet\t" << vdet_set << "\t" << vdet;
  cout << "\tidet\t" << idet_lim << "\t" << idet << endl;
  cout << "  vcc\t" << vcc_set << "\t" << vcc;
  cout << "\ticc\t" << icc_lim << "\t" << icc << endl;
  cout << "  vdd\t" << vdd_set << "\t" << vdd;
  cout << "\tidd\t" << idd_lim << "\t" << idd << endl;
  cout << "  vpin\t" << vpin_set << "\t" << vpin << endl;
//  cout << "\tipin\t" << ipin_lim << "\t" << ipin << endl;
  cout << "  vld0\t" << vld0_set << "\t" << vld0;
  cout << "\tild0\t" << ild0_lim << "\t" << ild0 << endl;
  cout << "  vld1\t" << vld1_set << "\t" << vld1;
  cout << "\tild1\t" << ild1_lim << "\t" << ild1 << endl;
  cout << endl;

  cout << "\tMON\tlow_lim\thi_lim"   << endl;
  cout << "  t1\t"   << T1   << "\t" << T1_minimum   << "\t" << T1_maximum   << endl;
  cout << "  t2\t"   << T2   << "\t" << T2_minimum   << "\t" << T2_maximum   << endl;
  cout << "  et1\t"  << ET1  << "\t" << ET1_minimum  << "\t" << ET1_maximum  << endl;
  cout << "  et2\t"  << ET2  << "\t" << ET2_minimum  << "\t" << ET2_maximum  << endl;
  cout << "  hum1\t" << hum1 << "\t" << hum1_minimum << "\t" << hum1_maximum << endl;
  cout << "  hum2\t" << hum2 << "\t" << hum2_minimum << "\t" << hum2_maximum << endl;
  cout << endl;

  cout << "\t\tState\tNumber\tMaximum" << endl;
  cout << "  HVtrip\t" << HVtrip << "\t" << nHVtrips << "\t" << maxHVtrips << endl;
  cout << "  LVtrip\t" << LVtrip << "\t" << nLVtrips << "\t" << maxLVtrips << endl;
  cout << "  MTtrip\t" << MTtrip << "\t" << nMTtrips << "\t" << maxMTtrips << endl;
  cout << "  ETtrip\t" << ETtrip << "\t" << nETtrips << "\t" << maxETtrips << endl;

  cout << endl;
  cout << "  HVstatus  0x" << hex << HVstatus << dec;
  cout << "  LVstatus  0x" << hex << LVstatus << dec << endl;
  cout << "  LV_temp  "   << LVtemp << " C" << endl;
  cout << "  HV Stopwatch " << HVStopwatchTime << endl;
  cout << "  LV Stopwatch " << LVStopwatchTime << endl;
  cout << "  suspended  "  << suspended << endl;

}
