// # defines for "standard row"

#define SNO  0;  // scan number
#define MID  1;  // module ID
#define CHP  2;  // chip ID
#define MOD  3;  // cal_mode
#define VAR4 4;  // value set to variable TYP4
#define VAR3 3;  // value set to variable TYP3 
#define VAR2 2;  // value set to variable TYP2 
#define VAR1 7;  // value set to variable TYP1 
#define NEVT 8;  // case sob: expected number of events
                 // case eob: number of triggers sent
#define NERR 9; 
#define INDX 10;

// #defines for sob record

#define SOB_PNT4 11; // scan point counters 
#define SOB_PNT3 12;
#define SOB_PNT2 13;
#define SOB_PNT1 14;
#define SOB_TYP4 15; // scan variable types
#define SOB_TYP3 16;
#define SOB_TYP2 17;
#define SOB_TYP1 18;
#define SOB_BNO  19; // burst_number
#define SOB_BTP  20; // burst_type;
#define SOB_LOOP; //   loop_cals;
#define SOB_; //   num_trigs;
#define SOB_; //   trig_type;
#define SOB_; //   cfg_freq;
#define SOB_; //   sr_freq;
#define SOB_; //   bc_freq;
#define SOB_START1  27; //   scan_start1;
#define SOB_STOP1   28; //   scan_stop1;
#define SOB_STEP1   29; //   scan_step1;
#define SOB_POINTS1 30; //   scan_points1;
#define SOB_START2  31; //   scan_start2;
#define SOB_STOP2   32; //   scan_stop2;
#define SOB_STEP2   33; //   scan_step2;
#define SOB_POINTS3 34; //   scan_points2;
#define SOB_START3  35; //   scan_start3;
#define SOB_STOP3   36; //   scan_stop3;
#define SOB_STEP3   37; //   scan_step3;
#define SOB_POINTS3 38; //   scan_points3;
#define SOB_START4  39; //   scan_start4;
#define SOB_STOP4   40; //   scan_stop4;
#define SOB_STEP4   41; //   scan_step4;
#define SOB_POINTS4 42; //   scan_points4;


 
int get_val(char* line, int index, double* val){
  char substring[20];
 
  sscanf(line+=index,"%s ",substring);
  sscanf(substring,"%lf",val);
  return 1 + strlen(substring);
}

st_readnt(short moi, short loi, short run, short scan){

  FILE* in_file = NULL;

  char fname[64];
  char line[1024];
  char htitle[128];
  int chan, i, lo_chip, hi_chip;
  int index, row;
  int first_cal, first_chp, first_sob;
  double data[43];

  double data[64][768];

  if(loi){
    lo_chip = 6;
    hi_chip = 11;
  }else{
    lo_chip = 0;
    hi_chip = 5;
  }
  row=0;
  first_cal = -1;
  first_chp = -1;
  first_sob = 1;

  sprintf(fname,"C:\\sctdaq\\data\\strun%d.dat",run);
  in_file=fopen(fname,"r");
  if (in_file == NULL){ 
    printf("st_readnt: Failed to open st data file %s\n",fname);
    return;
  }
  printf("st_readnt: File %s opened for writing\n",fname);

  while(fgets(line,1024,in_file) != NULL){

    index = 0;
    // read SNO, MID, CHP and MOD
    for (i=0; i<4; i++) index += get_val(line, index, &data[i]);

    // check we have desired SNO
    if (data[SNO]==(double)scan){ 

      if (data[MOD]==-5){  /* module sysmap record */
        printf("sysmap record for module %d - ignored!!\n", (int)data[MID]);
      }

      if (data[MOD]==-4){  /* module DCS record */
        printf("DCS record for module %d - ignored!!\n", (int)data[MID]);
      }
 
      if (data[MOD]==-3){  /* trimdac record */
        printf("trimdac record for module %d chip %d - ignored!!\n",
       (int)data[MID], (int)data[CHP]);
      }

      if (data[MOD]==-2){  /* chip configuration record */
        printf("trimdac record for module %d chip %d - ignored!!\n",
       (int)data[MID], (int)data[CHP]);
      }

      if (data[MOD]==-1){  /* sob record */

        // read rest of data row
        for (i=4; i<43; i++) index += get_val(line, index, &data[i]);

        printf("Found scan %d\n",head.seq);
        printf("  tmax %lf\n",scandef.tmax);
        printf("  scan_start %lf\n",scandef.scan_start1);
        printf("  scan_stop %lf\n",scandef.scan_stop1);
        printf("  scan_step %lf\n",scandef.scan_step1);
        printf("  num_points1 %lf\n",scandef.num_points1);

        // Create histogram if this is first SOB record for this scan
        if(!first_sob){
          printf("Creating histogram for scan %d\n",scan);
          sprintf(htitle,"ATLAS SCT Module test at H8\nScan of ");
          switch((int)data[TYP1]){
            case 1:
             sprintf(htitle,"THRESHOLD (mV) ");
             break;
            case 2:
             sprintf(htitle,"VCAL (mV) ");
             break;
            case 3:
             sprintf(htitle,"STROBE DELAY ");
             break;
            case 4:
             sprintf(htitle,"PREAMP (uA) ");
             break;
            case 5:
             sprintf(htitle,"SHAPER (uA) ");
             break;
            case 20:
             sprintf(htitle,"STREAM DELAY (nS) ");
             break;
            case 23:
             sprintf(htitle,"CLOCK DUTY ");
             break;
            default:
             sprintf(htitle,"UNKNOWN");
             break;
           } 
         sprintf(htitle,"%s from %3.2f to %3.2f in %3.2f steps", htitle, 
                      data[SOB_START1],data[SOB_STOP1],data[SOB_STEP1]);
         printf("%s\n",htitle);
  
          double half_step = data[SOB_STEP1] / 2;
          if (half_step<0) half_step = half_step*-1;
          if(data[SOB_STOP1]>data[SOB_STEP1]){
            printf("scan_stop>scan_start\n");
            h_scan = new TH2F("h_scan",htitle, 768,0.0,768.0, 
                              scandef.num_points1, scandef.scan_start1-half_step, scandef.scan_stop1+half_step);
          }else{
            printf("scan_start>scan_stop\n");
            h_scan = new TH2F("h_scan",htitle, 768,0.0,768.0, 
                              scandef.num_points1, scandef.scan_stop1-half_step, scandef.scan_start1+half_step);
         }
         TAxis * axis = h_scan->GetYaxis();
         float scan_max_points = axis->GetNbins();
         float scan_start = axis->GetBinCenter(1);
         float scan_stop  = axis->GetBinCenter(scan_max_points);
         printf("scan_start %f, stop %f, half_step %f, npoints %f",scan_start, scan_stop, half_step, scan_max_points);
        }
      }

      if((data[MOD]>=0)&&(data[MID]==moi)&&(data[SNO]==scan)
        &&(data[CHP]>=lo_chip)&&(data[CHP]<=hi_chip)){

        // read rest of data row
        for (i=4; i<43; i++) index += get_val(line, index, &data[i]);

        if(data[INDX]==0){
          if(first_chp==-1){
            first_chp=head.chp;
            first_cal=head.mod;
          }else if ((head.chp==first_chp)&&(head.mod==first_cal)){
            row++;
          }
        } //endif indx
  
        if((!(int)scandef.loop_cals)||(head.mod==head.indx)){        
          chan = ((((int)data[CHP])%6)*128)+data[INDX];
          for (i=11; i<43; i++){
            h_scan->Fill((float)chan, (float)scan_var1, (float)data[i]);
            chan += 4;
          }
        } //endif !loop_cals
      }
    }
  }
  printf("\n%d scanpoints found\n",row);
  h_scan->Print();
  fclose(in_file);
}

