/*
 * sct.h:   SCT Test system map structures
 *
 * GFM Nov98
 * GFM May99 Split from sct.h
 * GFM 990721 Redefine link parameters to indicate number readout redundancy status
 * PWP 10.10.00 Revisions and additions for OptIF
 */

typedef struct {
  short present;    /* detector present for cnc and readout */
  short active;     /* detector is participating in scans */

  short link0;      /* link0: 0=> not used; 1=> used for chips 0-5; 2=> used for all chips (redundancy) */
  short link1;      /* link1: 0=> not used; 1=> used for chips 8-13; 2=> used for all chips (redundancy) */

  short suspend0;   /* suspend0 / suspend 1: temporary removal of links 0/1 from DAQ */
  short suspend1;   /*                       due to errors. PWP 11.04.01 */

  module_cfg det;
  char detname[32];  /* detname is not inside module_cfg since it sets the filename
                      from which to load module configuration data */

  char devtype[32];  /* device type - deliberately not part of module_cfg struct
                        to facilitate the use of default detector configuration 
                        file during production module testing PWP 16.01.02 */

  short lv_id;      /* refers to crate number */
  short lv_ch;      /* channel within crate, 0:15 */

  short hv_id;      /* refers to crate number */
  short hv_ch;      /* channel within crate, 0:15 */

  short dcs_id;     /* DCS identifier for this module */

  short slog_id;
  short slog_ch;
  short slog_pg;    /* refers to page within slog ram */

  short r_slog_id;   /* SLOG board */
  short r_slog_ch;   /* and channel for redundant CLK and COM */
  short r_slog_pg; 

  short opto_tx_type;   /* 0 - BiLED, 1 - OptIF */
  short opto_tx_id;
  short opto_tx_ch;
  short opto_tx_iset;   /* 8 bit DAC to set VcSEL drive current */
  short r_opto_tx_type; /* 0 - BiLED, 1 - OptIF */
  short r_opto_tx_id;
  short r_opto_tx_ch;

  short opto_rx_id;
  short opto_rx_ch0;
  short opto_rx_ch1;
  short opto_rx_thr0;
  short opto_rx_thr1;
  
  short mus_id;
  short mus_enable0;
  short mus_enable1;
  short mus_ch0;
  short mus_ch1;
  short mus_delay0;
  short mus_delay1;
} sysmap;

