/*

  sct_hardware.h

  Interim hardware database for systemtest

  ... to be replaced by a runtime configuration system

  GFM 990421

  GFM 990826 Moved variable files outside source tree to ~/sctvar

  BJG/PWP 24 May 2002 - add paths for linux

*/

/* LOCALITY IS NO LONGER DEFINED HERE - PWP 29.05.02 */

#define SCT_MAX_MODULES  6
/* Maximum number of allowed modules */

/* VME modules */

#define SLOG_ADDRESS  0x01800000
#define SLOG_SPACING  0x00020000
#define SLOG_NUMBER   1

#define MUS_ADDRESS   0x01900000
#define MUS_SPACING   0x00100000
#define MUS_NUMBER    1

#define SCTHV_ADDRESS 0x00f00000
#define SCTHV_SPACING 0x00010000
#define SCTHV_NUMBER  2
#define SCTHV_MAX_LOOPS  200

#ifdef SCTLV_A16 /* use A16 for SCTLV */

#define SCTLV_CRATE_ADDRESS 0x0000
#define SCTLV_SPACING 0x1000
#define SCTLV_NUMBER  3

#else            /* use A24 for SCTLV */

#define SCTLV_CRATE_ADDRESS 0x00ff0000
#define SCTLV_SPACING 0x00001000
#define SCTLV_NUMBER  3

#endif

#define CLOAC_ADDRESS 0x00880000
#define CLOAC_NUMBER  1
#define CLOAC_CAL_OFFSET  4    /* request (TRIG_DELAY - CLOAC_CAL_OFFSET) to
                                * achieve TRIG_DELAY BCOs between cal and L1 */

/* NB. PULSE_DELAY, TRIG_DELAY are now in terms of REAL numbers of BCOs 
 * - the offset correction is now calculated in stlib. PWP 01.10.99
 * - default values revised PWP 15.12.99 */
 
#define COM_DELAY 200
#define PULSE_DELAY 129
/* #define TRIG_DELAY 129  */ /* For ABC */
/* #define TRIG_DELAY 136  */ /* For ABCD1.1 and ABCD1.2 */

#if CLOAC_NUMBER==0
/* #define TRIG_DELAY 131  */ /* For ABCD2T, ABCD2NT */
#define TRIG_DELAY 130  /* For ABCD3T */
#else
/* #define TRIG_DELAY 130  */ /* For ABCD2T, ABCD2NT */ 
#define TRIG_DELAY 129  /* For ABCD3T */

#endif

#define SR_FREQUENCY 0  /* => 60 Hz   */
#define TR_FREQUENCY 6  /* => 100 kHz periodic */
#define NL1A 2
#define L1A_SPACING 128

#define BILED_ADDRESS 0x00810000
#define BILED_SPACING 0x00001000  /* BiLED n is at BILED_ADDRESS 
                                      + n* BILEd_SPACING   */

#define BILED_NUMBER 0

#define OPTIF_ADDRESS 0x00900000
#define OPTIF_SPACING 0x00010000 /* ???? */
#define OPTIF_NUMBER 0

#define TEMPTRESS_ADDRESS 0x00980000
#define TEMPTRESS_SPACING 0x00010000 /* ???? */
#define TEMPTRESS_NUMBER 0

#define BLUE_HANDBAG 1   /* conservative power management */

/* Note regarding addresses for PC version of stlib using
   the National Instruments PCI-MXI-VME interface:

   Although there are a maximum of four windows available
   through NI-VXI, a workaround has been implemented such
   that this is enough.  All A32 devices now sit in the
   same 8MB window starting at address SLOG_ADDRESS.
   Hence SLOG must now be mapped before other A32 devices. 
   The NI-VXI user window must also be set suitably large,
   eg, 16MB, using T&M Explorer 
                     -> PCI-MXI-2 Properties
                       -> Hardware Configuration
                         -> PCI
*/

/* PATHS for file io */
/* NB. slash characters now added to end of each path PWP 24.05.02 */

#if defined (VXILINUX) || defined (QB_linux)
#define ST_PATH        "/home/sct/sctdaq/"
#define ST_BIN         "/home/sct/sctdaq/bin/"
#define ST_VAR         "/home/sct/sctvar/"
#define ST_ETC         "/home/sct/sctvar/etc/"
#define ST_DATA_PATH   "/home/sct/sctvar/data/"
#define ST_CONFIG_PATH "/home/sct/sctvar/config/"
#define ST_RESULT_PATH "/home/sct/sctvar/results/"
#define ST_SYSMAP_FILE "/home/sct/sctvar/config/st_system_config.dat"
#endif

#ifdef RCC
#define ST_PATH        "/usera/hill/sctdaq/"
#define ST_BIN         "/usera/hill/sctdaq/bin/"
#define ST_VAR         "/usera/hill/sctvar/"
#define ST_ETC         "/usera/hill/sctvar/etc/"
#define ST_DATA_PATH   "/usera/hill/sctvar/data/"
#define ST_CONFIG_PATH "/usera/hill/sctvar/config/"
#define ST_RESULT_PATH "/usera/hill/sctvar/results/"
#define ST_SYSMAP_FILE "/usera/hill/sctvar/config/st_system_config.dat"
#endif

#ifdef WIN32
#define ST_PATH        "C:\\sctdaq\\"
#define ST_BIN         "C:\\sctdaq\\bin\\"
#define ST_VAR         "D:\\sctvar\\"
#define ST_ETC         "D:\\sctvar\\etc\\"
#define ST_DATA_PATH   "D:\\sctvar\\data\\"
#define ST_CONFIG_PATH "D:\\sctvar\\config\\"
#define ST_RESULT_PATH "D:\\sctvar\\results\\"
#define ST_SYSMAP_FILE "D:\\sctvar\\config\\st_system_config.dat"
#endif

#ifdef RAID
#define ST_PATH        "/usr/people/sctdaq/"
#define ST_BIN         "/usr/people/sctdaq/bin/"
#define ST_VAR         "/usr/people/sctdaq/sctvar/"
#define ST_ETC         "/usr/people/sctdaq/sctvar/etc/"
#define ST_DATA_PATH   "/usr/people/sctdaq/sctvar/data/"
#define ST_CONFIG_PATH "/usr/people/sctdaq/sctvar/config/"
#define ST_RESULT_PATH "/usr/people/sctdaq/sctvar/results/"
#define ST_SYSMAP_FILE "/usr/people/sctdaq/sctvar/config/st_system_config.dat"
#endif

