/*
 *  sct_scans.h 
 *
 * version 0.1 GFM Jun99
 * version 0.2 PWP 12.12.00
 * version 0.3 PWP 13.12.00
 * version 0.4 PWP 23.05.01
 */

// #define ST_MAX_SCANPOINTS 128
#define ST_MAX_SCANPOINTS 160


/* Elapsed == elapsed time; equivalent of a NOP to permit general monitoring */
#define ST_ELAPSED       0

/* ABC(D)(DT)(DNT) DAC Settings */
#define ST_VTHR          1
#define ST_VCAL          2
#define ST_STROBE_DELAY  3
#define ST_PREAMP        4
#define ST_SHAPER        5
#define ST_TRIM          6
#define ST_MASK          7
#define ST_ROLE          8
#define ST_NMASK         9

/* ABC(D) Config Register Settings */
#define ST_CAL_MODE      10
#define ST_COMPRESSION   11
/* #define ST_TEST_MODE     12 */
#define ST_TRIM_RANGE    12
#define ST_EDGE_DETECT   13
#define ST_SEND_MASK     14
#define ST_ACCUMULATE    15   
#define ST_DTM           16 
#define ST_BYPASS        17   /* predefined redundancy configurations
                                 to be used during production testing
                                 PWP 01.11.00 */

#define ST_ACTIVE        18   /* set whether a chip is ACTIVE or not
                                 PWP 08.11.01 */

#define ST_TOKEN         19   /* a bit patterns that defines the bypass 
				 scheme allowing all combinations. A '1'
				 means that the chip is read out, a '0' 
			         means that the chip is bypassed/dead */

/* Derived ABC(D)(DT)(DNT) DAC Settings */
#define ST_QTHR          41
#define ST_QCAL          42

/*
 * Off Detector tuning:
 * MuSTARD, SLOG and CLOAC
 */

#define ST_STREAM_DELAY  20   /* Both uplink streams */
#define ST_STREAM_DELAY0 21   /* Only stream 0 */
#define ST_STREAM_DELAY1 22   /* Only stream 1 */
#define ST_CLOCK_DUTY    23   /* Switch adjustment for SLOG1,
                                 will be automated in SLOG2 */
#define ST_CLOCK_PHASE   24   /* Cable adjustment for SLOG1,
                                 will be automated in SLOG2 */

#define ST_COM_DELAY     25   /* SLOG only - number of BC0s between soft reset and calstrobe*/
#define ST_PULSE_DELAY   26   /* CLOAC and SLOG - nBC between PULSE and L1A */
#define ST_TRIG_DELAY    27   /* CLOAC and SLOG - nBC between CAL and L1A   */
#define ST_SR_FREQUENCY  28   /* CLOAC - bits written to sr frequency register */
#define ST_TR_FREQUENCY  29   /* CLOAC - bits written to tr frequency register */

#define ST_NL1A          45   /* SLOG only, number of multiple L1A triggers */
#define ST_L1A_SPACING   46   /* SLOG only, spacing of multiple L1A triggers */

/*
 * Off Detector tuning:
 * OPTO (BiLED or  OPTIF)
 */

#define ST_OPTO_RX_THR     50
#define ST_OPTO_RX_THR0    51
#define ST_OPTO_RX_THR1    52
#define ST_OPTO_TX_DAC     53
#define ST_OPTO_TX_MKSPNEG 54
#define ST_OPTO_TX_MKSPPOS 55
#define ST_OPTO_TX_CDELAY  56
#define ST_OPTO_TX_FDELAY  57

/*
 * DCS System:
 * 1. Requested Settings
 */

#define ST_VDET          30
#define ST_VCC           31
#define ST_VI1           32
#define ST_VDD           33
#define ST_VPIN          34
#define ST_VLED          35
#define ST_VLED0         36
#define ST_VLED1         37
#define ST_SELECT        38
#define ST_RAMP          39

/*
 * DCS System: 
 * 2. Requested Limits
 */

#define ST_IDET          60
#define ST_ICC           61
#define ST_IVI1          62
#define ST_IDD           63
#define ST_IPIN          64
#define ST_ILED          65
#define ST_ILED0         66
#define ST_ILED1         67

#define ST_T1_LIMIT      68
#define ST_T2_LIMIT      69

/*
 * DCS System: 
 * 3. Control information
 */

#define ST_SUSPENDED     70

#define ST_HVTRIP        71
#define ST_LVTRIP        72
#define ST_MTTRIP        73

#define ST_HV_STATUS     74
#define ST_LV_STATUS     75
#define ST_LV_TEMP       76

/*
 * DCS System: 
 * 4. Monitored Values
 */

#define ST_MON_VDET      80
#define ST_MON_VCC       81
#define ST_MON_VI1       82
#define ST_MON_VDD       83
#define ST_MON_VPIN      84
#define ST_MON_VLED      85
#define ST_MON_VLED0     86
#define ST_MON_VLED1     87

#define ST_MON_IDET      88
#define ST_MON_ICC       89
#define ST_MON_IVI1      90
#define ST_MON_IDD       91
#define ST_MON_IPIN      92
#define ST_MON_ILED      93
#define ST_MON_ILED0     94
#define ST_MON_ILED1     95

#define ST_MON_T1        96
#define ST_MON_T2        97

/*
 * Variables used to control Liverpool forward SC
 */

#define ST_FWD_SC_MODE  100   /* operating mode */
#define ST_BPM_DRIVE    101   /* bpm drive current */
#define ST_VDC_DAC      102   /* vdc_dac setting */
#define ST_VDC_DAC0     103 
#define ST_VDC_DAC1     104 

/* Different types of burst to be used during the scans;
   bursts imply establishment of readout conditions (Mustard modes
   etc.), issuing of triggers and reads (which may or may not be
   synchronised), and finalisation of occupancy results. */

/* Burst types implemented mostly in stlib */

#define ST_TBURST         1   /* default */
#define ST_VBURST         2
#define ST_SBURST         3
#define ST_IBURST         4
#define ST_RAWBURST       5
#define ST_SENDIDBURST    6
#define ST_NSEBURST       7

/* Eventual replacement bursts implemented in TST.cpp */

#define ST_TBURST2       11 
#define ST_VBURST2       12
#define ST_EBURST        20   /* External Trigger */ 
#define ST_DEBURST       21   /* External Trigger with software histogram */ 
#define ST_DSBURST       30   /* Periodic triggers from SLOG */

/* New burst types implemented in TST.cpp */

#define ST_DBURST       100   /* Events passed up to calling routine for decoding */
/* #define ST_DUMPBURST    101  */  /* Dump event to file - integrated with DecodedBurst 26.04.01 */
#define ST_DVBURST      102   /* periodic triggers but with software histogramming */
#define ST_TESTBURST    111
#define ST_TIMEEVBURST  112
#define ST_SEUBURST     120   /* Calls SEUBurst() which calls SEUTest() */

/* Fit functions */

#define ST_ERFC  1
#define ST_ERF   2
#define ST_GAUS  3

